/**********************************************************
 *
 *  Copyright (c) 1996  SeikoEpson Inc
 *  All Rights Reserved
 *
 *  File name : sim33.h
 *  Function  :
 *  This is 33 simulator debugger.
 *  sim33.h has definitions.
 *  Revision  :
 *   8/17/1995  H.Watanabe (ESD)  start
 *   2/ 7/1996  H.Watanabe (ESD)  1st release
 *   6/17/1998  M.Igarashi (ESD)  change break max for hard pc break2
 *   3/30/2000  C.Nakayama        Modify API spec               
 *   3/30/2000  C.Nakayama        Modify API spec               
 *   2/26/2003  T.Tazaki          add Trace
 *   9/ 1/2003  M.Takeishi        add Adv mode
 *  05/17/2005  T.Tazaki          Modify
 *
 **********************************************************/

#ifdef UNIX
#define huge
#endif  /* UNIX */

/*@ data size */
#define CODE        2   /* code = half word data */
#define BYTE_DATA   1   /* byte data */
#define HALF_DATA   2   /* half word data */
#define WORD_DATA   4   /* word data */

/*@ memory block type */
#define MEM_ROM     0xff
#define MEM_RAM     0xaa
#define MEM_IO      0x00
#define MAX_MEM_BLOCK       32

/****************/
/* memory block */
/****************/
struct MemBlock {
    unsigned char *pbMem;   /* pointer to memory block */
    unsigned long ulStart;  /* start address */
    unsigned long ulEnd;    /* end address */
    unsigned long ulSize;   /* size */
    int iType;      /* memory block type (MEM_ROM/MEM_RAM/MEM_IO) */
};

/**************************/
/* 33 core register model */
/**************************/
#define R0      0   /* define register number */
#define R1      1
#define R2      2
#define R3      3
#define R4      4
#define R5      5
#define R6      6
#define R7      7
#define R8      8
#define R9      9
#define R10     10
#define R11     11
#define R12     12
#define R13     13
#define R14     14
#define R15     15
#define PSR     16  /* S0 */
#define SP      17  /* S1 */
#define ALR     18  /* S2 */
#define AHR     19  /* S3 */
/* register No. modified */

#define LCO     20  /* S4 */
#define LSA     21  /* S5 */
#define LEA     22  /* S6 */
#define SOR     23  /* S7 */
#define TTBR    24  /* S8 */
#define DP      25  /* S9 */
#define IDIR    26  /* S10 */   /* add T.Tazaki 2003/11/28 */
#define DBBR    27  /* S11 */   /* add T.Tazaki 2003/11/28 */
#define USP     28  /* S13 */
#define SSP     29  /* S14 */
#define PC      30  /* S15 */ 

// #define POPS_PC     30  /* pops %pc for fnPop(regno) */      /* del T.Tazaki 2003/11/28 */

/***************/
/* ulFlag(PSR) */
/***************/
#define N_FLAG      0x00000001  /* negative flag */
#define Z_FLAG      0x00000002  /* zero flag */
#define V_FLAG      0x00000004  /* overflow flag */
#define C_FLAG      0x00000008  /* carry flag */
#define IE_FLAG     0x00000010  /* interupt enable flag */
#define OE_BIT      0x00000020  /* = 0 */
#define DS_FLAG     0x00000040  /* dividend sign flag */
#define MO_FLAG     0x00000080  /* MAC overflow flag */
#define IL_BITS     0x00000f00  /* interupt level */

#define SV_FLAG     0x00001000  /* supervisor mode flag */
#define ME_FLAG     0x00002000  /* MMU exception flag */
#define DE_FLAG     0x00004000  /* Debug exception flag */
#define S_FLAG      0x00008000  /* saturation flag */
#define HC_FLAG     0x00010000  /* AHR change enable flag */
#define LC_FLAG     0x00020000  /* ALR change enable flag */
#define SE_FLAG     0x00100000  /* shift carry enable flag */
#define OC_FLAG     0x00200000  /* overflow clear enable flag */
#define SW_FLAG     0x00400000  /* 32-bit scan enable flag */
#define RC_BITS     0x0f000000  /* register counter */
#define PM_FLAG     0x10000000  /* push/pop mode flag */
#define LM_FLAG     0x20000000  /* loop mode enable flag */
#define RM_FLAG     0x40000000  /* repeat mode enable flag*/
#define HE_FLAG     0x80000000UL  /* halt sleep enable flag*/

/****************/
/* vector table */
/****************/
#define VECTOR_ADDR_MCU     0x00080000  /* MCU0/MCU1 mode */
#define VECTOR_ADDR_MPU     0x00c00000  /* MPU mode */

/* vector table offset */
#define V_RESET     0   /* reset */
#define V_ZERO_EX   16  /* zero divide exception  */
#define V_OVERFLOW_EX       20  /* overflow exception  */
#define V_ADDR_EX   24  /* address exception  */
#define V_NMI       28  /* NMI */
#define V_SOFT_EX0  48  /* software exception 0 */
#define V_SOFT_EX1  52  /* software exception 1 */
#define V_SOFT_EX2  56  /* software exception 2 */
#define V_SOFT_EX3  60  /* software exception 3 */
#define V_INTR      64  /* interupt */

/***********/
/* iStatus */
/***********/
#define PC_BREAK        0x0001  /* break by PC address */
#define DATA_BREAK      0x0002  /* break by data addr, data, r/w */
#define NOMAP_MEM_BREAK     0x0004  /* no mapped memory */
#define ROM_DATA_BREAK      0x0008  /* break by writing ROM data */
#define ILL_INST_BREAK      0x0010  /* break by illegal instruction */
#define KEY_BREAK       0x0020  /* break by keyboard */
#define SLP_HALT_BREAK      0x0040  /* break by slp/halt instruction */
#define ILL_DELAY_BREAK     0x0080  /* break by illegal delayed instruction */
#define BREAK_STATUS        0x00ff  /* break status */

#define S_ADDR_EX   0x0100  /* address exception */
#define S_OVERFLOW_EX   0x0200  /* overflow exception */
#define S_ZERO_EX   0x0400  /* zero divide exception */
#define S_SOFT_EX0  0x0800  /* software exception 0 */
#define S_SOFT_EX1  0x1000  /* software exception 1 */
#define S_SOFT_EX2  0x2000  /* software exception 2 */
#define S_SOFT_EX3  0x4000  /* software exception 3 */
#define S_EXCEPTION 0x7f00  /* exception status */

#define DELAY_FLAG  0x8000  /* delay flag */

#define STACK_OVERFLOW  0x10000 /* stack overflow by push or pop */
#define PUSHPOP_ILLEGAL 0x20000 /* RC flag error when push/pop */

/**************/
/* Delay Slot */
/**************/
struct DelaySlot {
    int blUsed;     /* NO: not use DelaySlot */
    int blRunDelayed;   /* if running DelaySlot instruction */
    unsigned long ulAddr;   /* address of DelaySlot */
    unsigned long ulNewPC;  /* save new PC */
};

/*************/
/* extension */
/*************/
struct Extension {
    int iType;      /* extension type */
    unsigned long ulData;   /* data */

    /*        2002/01/31  C.Nakayama Add special registers.(GNU33cygwin) */
    /* >>>>> change iruma m.takeishi '03.09.01 */
    /*
    // ext %rs,op,imm2 , ext cond, ext op,imm2 //
    */
    /* ext %rs,op,imm2 , ext op,imm2 */
    /* <<<<< change iruma m.takeishi '03.09.01 */
    int iTypeExt;
    unsigned long ulDataExtRs;   /* data */
    unsigned short uwDataExtOp;   /* data */

    /* >>>>> add iruma m.takeishi '03.09.01 */
    /* ext cond flag */
    int           iTypeExtCond;                 // to be set none,skip,exec
    unsigned long ulPushCycle;                  // to be set cycle if skipped
    /* <<< add iruma m.takeishi '03.09.01 */
};

/* Extension.iType */
#define EXT_HL      2   /* ext:ext */
#define EXT_L       1   /* ext */
#define EXT_NONE    0   /* no ext */

/* Extension.iTypeExt */
#define EXT_RS      3   /* ext cond */
#define EXT_COND    4   /* ext cond */
#define EXT_OP_IMM  5   /* ext op,imm2 */
#define EXT_RS_OP_IMM   6   /* ext %rs,op,imm2 */


/* Extension.iTypeExtCond */
#define EXTCOND_NONE    7   /* ext cond is not done */
#define EXTCOND_GO      8   /* ext cond is started(for trace outputing) */
#define EXTCOND_SKIP    9   /* ext cond is done,and jumped */
#define EXTCOND_EXEC    10  /* ext cond is done,but not jumped */


/********************/
/* Data Break Point */
/********************/
struct BreakData
{
    int iType;      /* Data Access Type for break */
    unsigned long ulAddr;   /* address */
};

#define BREAK_CODE_MAX      21 /* 20 add hard PC break2 98/05/28 M.Igarashi*/

/********/
/* INTR */
/********/
struct StartINTR
{
    int iType;      /* type */
    int iLevel;     /* level */
};

/*******************/
/* Interlock cycle */
/*******************/
#define C33_NOT_INTERLOCK 0
#define C33_NORMAL  1
#define C33_DIV_MAC 2
#define C33_MLT_MAC1    3
#define C33_INDIRECT    4
#define C33_BIT     5
#define C33_PUSHN   6
#define C33_POP     7
#define C33_POPN    8
#define C33_POPS    9
#define C33_MAC_MLT_DIV 10

struct c33_interlock
{
    int iInterlockMode;     /* =0:Non interlock mode, =1:interlock mode */
    int iCycle1;        /* interlock cycle count 1 */
    int iCycle2;        /* interlock cycle count 2 */
    int iNextInst;      /* C33_MAC_MLT_DIV or C33_INDIRECT */
    unsigned short uwRegno; /* Register number ( %rd ) */
    unsigned short uwRegno2;/* Register number ( %rd ) */
};


#define NO_INTR     -1  /* set no interrupt */

/**********************************************/
/* double word (for mlt/div/mac instructions) */
/**********************************************/
struct DoubleWord {
  unsigned long   ulHigh;
  unsigned long   ulLow;
};

/* fnWriteMem()/fnGoStep() */
#define FORCE       1
#define NOT_FORCE   0  

#define SUCCESS     0
#define FAILURE     -1

#define MSB_BIT     0x80000000
#define LSB_BIT     1

#define WITH_CARRY  0
#define WITHOUT_CARRY       1

/* sstBreakPoint[].blUsed, iStatus:DELAY_FLAG */
#define YES     1
#define NO      0

/* trace level */
#define TRACE_NONE  0
#define TRACE_NO_REG        1
#define TRACE_REG   2

/* steps to run */
#define UNLIMITED   -1

#define LINE_LEN    100 /* max length of line in map_file */
#define KEYWORD_LEN 5   /* length of keyword (ROM/RAM/IO) */
#define MAX_MEMORY_SIZE     32*1024*1024    /* max size of memory block */

#define NO_ACCESS   0   /* no access to memory */
#define READ_ACCESS 1   /* read access to memory */
#define WRITE_ACCESS        2   /* write access to memory */
#define BOTH_ACCESS 3   /* read & write access to memory */

#define READ_DATA   0
#define WRITE_DATA  1

#define STATUS_BUF_SIZE     256 /* size of buffer to explain trace status */

#define SET     0   /* set slp/halt mode */
#define CHECK_AND_RESET     1   /* check slp/halt mode & reset */
#define RESET       2   /* reset slp/halt mode */

#define TRUE        1   /* in register dump mode */
#define FALSE       0   /* in instruction dump mode */

/**************/
/* TRACE MODE */
/**************/

#define TM_ON           1   /* tm on */
#define TM_OFF          0   /* tm off */

#define TRACE_WINDOW        0   /* trace data output to window */
#define TRACE_FILE      1   /* tarce data output to file   */

#define TRACE_NORMAL        1   /* trace normal format */
#define TRACE_REGISTER      2   /* trace normal + register format */
#define TRACE_SOURCE        3   /* trace normal + source format */
#define TRACE_REGISTER_SOURCE   4   /* trace normal + register + source format */

#define C33_ERR_COMMAND_TRACE_NUM  "C33 command error, invalid trace number(1-4)."


/* >>>>> add iruma m.takeishi '03.09.01 */
/* instruction combination check definition. */
#define CMB_LOOP_NG          0x0001
#define CMB_LOOP_OK          0x0002
#define CMB_LOOP_OK_A1       0x0003
#define CMB_LOOP_OK_A2       0x0004
#define CMB_LOOP_OK_A3       0x0005
#define CMB_REPEAT_NG        0x0010
#define CMB_REPEAT_OK        0x0020

#define CMB_EXT_MASK         0xff00
#define CMB_EXT_NG           0x0100
#define CMB_EXT_COND_OK      0x0200
#define CMB_EXT_RS_OK        0x0400
#define CMB_EXT_RS_OP_IMM_OK 0x0800
#define CMB_EXT_OP_IMM_OK    0x1000
#define CMB_EXT_IMM_OK       0x2000
/* <<<<< add iruma m.takeishi '03.09.01 */


/**********************************************/
/* Prototypes Global */
/**********************************************/

int  c33_SimSendMessage (int iID, int iParam1, int iParam2, int iParam3, int iParam4, void *pvParam5);
int  c33_SimRecvMessage (int iID, int iParam1, int iParam2, int iParam3, int iParam4, void *pvParam5);
void c33_SimTraceMode(int iParamNum, char* pMode, int iOutput, char* pFilename);
void c33_SimCall(unsigned long ulStartAddr, unsigned long ulArg1,unsigned long ulArg2, unsigned long ulArg3);
void c33_SimInt(int iNmi, int iType, int iLevel);
void c33_SimClock(char *szClock);
void c33_SimSoftPcBreakSet(unsigned long ulBreakAddr);
void c33_SimSoftPcBreakClear(unsigned long ulBreakAddr);
int  c33_SimStdinSet( unsigned long ulBreakAddr );
void c33_SimStdinClear( void );
int  c33_SimStdoutSet( unsigned long ulBreakAddr );
void c33_SimStdoutClear( void );
void sim_Gdbtk_Update (void);
void gdbsim_reset (int iMode);
void gdbsim_reset_reg (int iMode);
int  fnDoSlpHalt (int iCommand);
void c33_SimFinishCommand(unsigned long ulStartPC);
int c33_SimCheckManyCall(unsigned long ulTempAddr,unsigned long ulEndAddr );
int c33_SimCheckCurrentInst(void);
void c33_SimNextCommand(void);
void c33_SimUntilCommand( unsigned long ulTempBreakAddr);
void gdbsim_WindowUpdate ( void );

