/* ICD interface for the EPSON C33 for GDB, the GNU debugger.
   Copyright 1996, 1997, 1998, 1999, 2000, 2001, 2002
   Free Software Foundation, Inc.

   This file is part of GDB.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.	*/

#ifndef __C33COMMON__
#define __C33COMMON__

#define EPSON						1

#define C33_BIT0					(0x0001)
#define C33_BIT1					(0x0002)
#define C33_BIT2					(0x0004)
#define C33_BIT3					(0x0008)
#define C33_BIT4					(0x0010)
#define C33_BIT5					(0x0020)
#define C33_BIT6					(0x0040)
#define C33_BIT7					(0x0080)
#define C33_BIT8					(0x0100)
#define C33_BIT9					(0x0200)
#define C33_BIT10					(0x0400)
#define C33_BIT11					(0x0800)
#define C33_BIT12					(0x1000)
#define C33_BIT13					(0x2000)
#define C33_BIT14					(0x4000)
#define C33_BIT15					(0x8000)
#define C33_BIT13_15				(C33_BIT13 | C33_BIT14 | C33_BIT15)

#define C33_NO_ERR					0
#define C33_ERR 					-1
#define C33_ERR_PARAMETER_NOT_SET	-2
#define C33_ERR_NO_MAP_AREA 		-3
#define C33_ERR_ROM_WRITE			-4

#define C33_BYTE					1
#define C33_HALF_WORD				2
#define C33_WORD					4

#define C33_LITTLE					1
#define C33_BIG 					2

#define C33_FALSE					0
#define C33_TRUE					1

#define C33_READ					0
#define C33_WRITE					1

#define C33_ROM 					1
#define C33_RAM 					2
#define C33_IO						3
#define C33_STACK					4

#define C33_NOT_OPEN				0
#define C33_COM_OPEN				1
#define C33_USB_OPEN				2

#define C33_DUMPFILE_BIN			1
#define C33_DUMPFILE_TEXT			2
#define C33_DUMPFILE_S3 			3
#define C33_DUMPFILE_UNIT			0x8000

#define C33_MOTOROLA_FOOTER 		"S70500000000FA"

#define C33_COMMAND_MAX 			256
#define C33_PARA_FILE_LINE_MAX		256
#define C33_TRACE_DATA_LINE_MAX 	4096

#define C33_COMMAND_ARGUMENT_MAX	20		/* change 10->20 2003/07/07 T.Tazaki */
#define C33_COMMAND_ARGUMENT_END	0
#define C33_COMMAND_ARGUMENT_SYM	1
#define C33_COMMAND_ARGUMENT_STR	2

#define C33_INVALID_ADDR			0xffffffff

/* icd v3 default debug unit address definition */
#define C33_DEBUG_BASE_ROM_ADDR 	0x60000
#define C33_DEBUG_BASE_RAM_ADDR 	0x71000

#define	C33_TIMER_US				(double)0.083333333  /* 1/(MCLK/4) us	48MHz/4 */

/* c33 cpu definition */
#define C33_ADDR_PRODUCT			0x78000
#define C33_ADDR_BRK_FUNC			0x7807c
#define C33_ADDR_TTBR				0x48134
#define C33_PRODUCT_PE				C33_BIT4
#define C33_PRODUCT_ADV 			C33_BIT7
#define C33_PRODUCT_STD 			0
#define C33_BRK_FUNC_ABF			C33_BIT0  /* bus trace */
#define C33_BRK_FUNC_BBF			C33_BIT1  /* bus break */
#define C33_BRK_FUNC_BTF			C33_BIT2  /* area break */
#define C33_PC_MASK 				(~C33_BIT0)
#define C33_SP_MASK 				(~(C33_BIT0 | C33_BIT1))
#define C33_LSA_MASK				(~C33_BIT0)
#define C33_LEA_MASK				(~C33_BIT0)
#define C33_TTBR_MASK				0xfffffc00UL
#define C33_DP_MASK 				(~(C33_BIT0 | C33_BIT1))

/* cpu type definition */
#define C33_STD 					0
#define C33_PE						1
#define C33_ADV 					3
#define C33_AUTO					4	/* add T.Tazaki 2005/04/01 */

/* processir ID : %idir bit31-24 */
#define C33_PROCESSOR_STD			0x2
#define C33_PROCESSOR_MINI			0x3
#define C33_PROCESSOR_ADV			0x4
#define C33_PROCESSOR_STD_NEW		0x5
#define C33_PROCESSOR_PE			0x6
#define C33_PROCESSOR_PE_BIG		0x7


/* c33 instrucsion */
#define C33_DELAYED_BIT 			0x0100			/* delayed instruction bit */
#define C33_RET_INSTRUCTION 		0x0640			/* ret instruction code */
#define C33_RETI_INSTRUCTION		0x04c0			/* reti instruction code */
#define C33_RET_DELAY_INSTRUCTION	(C33_DELAYED_BIT | C33_RET_INSTRUCTION)  /* ret.d instruction code */

/* break status */
#define C33_BRK_STS_BRK_TEMP		C33_BIT2
#define C33_BRK_STS_BRK_HARDPC		C33_BIT3
#define C33_BRK_STS_BRK_DB			C33_BIT4
#define C33_BRK_STS_BRK_MRDB		C33_BIT5
#define C33_BRK_STS_BRK_MWRB		C33_BIT6
#define C33_BRK_STS_BRK_INST		C33_BIT7
#define C33_BRK_STS_BRK_AB			C33_BIT8
#define C33_BRK_STS_BRK_BB			C33_BIT9
#define C33_BRK_STS_ESC 			C33_BIT11
#define C33_BRK_STS_EXT_TRG 		C33_BIT12

/* bus break counter mode */
#define	C33_COUNT_SAVE				0
#define	C33_COUNT_CLEAR				1

#define C33_SOFT_PC_BREAK			1
#define C33_HARD_PC_BREAK			2
#define C33_TRACE_TRGGER_BREAK		3

#define C33_HARD_PC_BREAK_MAX		2
#define C33_SOFT_PC_BREAK_MAX		200			/* change 16 ->200 T.Tazaki 2005/05/06 */
#define C33_STDIN_BREAK_NUM 		201			/* change 17 ->201 T.Tazaki 2005/05/06 */
#define C33_STDOUT_BREAK_NUM		202			/* change 18 ->202 T.Tazaki 2005/05/06 */
#define C33_BREAK_DISABLE			0xffffffff

/* flash writer */
#define C33_DATA_LOAD				0
#define C33_PROGRAM_LOAD			1

/* update condition */
#define C33_UPDATE					C33_BIT0
#define C33_UPDATE_REG				C33_BIT1

/* BBCU bus */
#define C33_BBCU_USR				C33_BIT3	/* USER  */
#define C33_BBCU_DMA				C33_BIT2	/* DMA	 */
#define C33_BBCU_CCU				C33_BIT1	/* CACHE */
#define C33_BBCU_HBCU				C33_BIT0	/* CPU	 */

/* Parallel port */
#define TIMEOUT_SEND_CONSTANT_PARL 7000
#define TIMEOUT_SEND_CONSTANT_OK 1000
#define TIMEOUT_SEND_CONSTANT 20000
#define TIMEOUT_RECV_CONSTANT 6000
#define TIMEOUT_RECV_CONSTANT_OK 1000
#define TIMEOUT_RECV_CONSTANT_SET_MAP 150000
#define TIMEOUT_RECV_CONSTANT_CP_RD_FLSH 150000
#define TIMEOUT_RECV_CONSTANT_CP_WT_FLSH_PROTECT 150000
#define TIMEOUT_RECV_CONSTANT_CP_WT_FLSH 150000
#define TIMEOUT_RECV_CONSTANT_CP_CLR_FLSH 150000
#define TIMEOUT_RECV_CONSTANT_ESC_BRK 2500
#define TIMEOUT_RECV_MOVE 150000
#define TIMEOUT_RECV_CONSTANT_ICE_COMMAND 150000
#define SAND_TIME 500
#define TIMEOUT_RECV_LOOP 100
#define TIMEOUT_SEND_LOOP 100

/* Advanced macro debug unit GDB status */
#define C33_GDB_STS_BUS_TRACE		0x04
#define C33_GDB_STS_BUS_BREAK		0x02
#define C33_GDB_STS_AREA_BREAK		0x01

/* ICDV3 DIPSW1 status */
#define C33_DIPSW1_PC_TRACE 		0x40	/* 0: PC trace enabled, 1:PC trace disabled */
#define C33_DIPSW1_BUS_TRACE		0x04	/* 0: Bus trace enabled, 1:Bus trace disabled */

#define C33_TIMBRK_MAX				300000	/* c33 timebrk max counter	add 2007/07/03 T.Tazaki */

/* ******* c33 command definition ******** */ 
/* command ID */
#define C33_ID_DATA_WRITE			0x01
#define C33_ID_DATA_PARALLEL_WRITE	0x02
#define C33_ID_DATA_READ			0x06
#define C33_ID_DATA_FILL			0x0e
#define C33_ID_DATA_MOVE			0x10
#define C33_ID_REG_READ 			0x30
#define C33_ID_REG_WRITE			0x31
#define C33_ID_SOFT_PC_BREAK_CLR	0x40
#define C33_ID_SOFT_PC_BREAK_SET	0x41
#define C33_ID_HARD_PC_BREAK_CLR	0x42
#define C33_ID_HARD_PC_BREAK_SET	0x43
#define C33_ID_DATA_BREAK_CLR		0x48
#define C33_ID_DATA_BREAK_SET		0x49
#define C33_ID_CAN					0x50
#define C33_ID_GO					0x60
#define C33_ID_STEP 				0x62
#define C33_ID_RUN_MON				0x64
#define C33_ID_RESET				0x68
#define C33_ID_VER_READ 			0x70
#define C33_ID_OK					0x7f
#define C33_ID_INITIALIZE			0x80
#define C33_ID_HARD_PC_BREAK2		0x90
#define C33_ID_TRACE_READ_DATA_SIZE 0x91
#define C33_ID_TRACE_READ_DATA		0x92
#define C33_ID_TRACE_RESUME 		0x93
#define C33_ID_TRACE_TRIGGER		0x94
#define C33_ID_MEASURE_UNIT 		0x98
#define C33_ID_AREA_BREAK_CLR		0xb0
#define C33_ID_AREA_BREAK_SET		0xb1
#define C33_ID_BUS_BREAK_CLR		0xb2
#define C33_ID_BUS_BREAK_SET		0xb3
#define C33_ID_BUS_TRACE_SET		0xb4
#define C33_ID_BUS_TRACE_READ_DATA	0xb5
#define C33_ID_SP_REG_READ			0xb6
#define C33_ID_SP_REG_WRITE 		0xb7
#define C33_ID_DATA_WRITE_FAST		0xb8
#define C33_ID_DATA_READ_EXACT		0xb9
#define C33_ID_CALL 				0xba
#define C33_ID_FLASH_SET			0xa0
#define C33_ID_FLASH_ERASE			0xa1
#define C33_ID_TARGET_RESET 		0xa4
#define C33_ID_TIME_BREAK			0xa5
#define C33_ID_FIRM_UPDATE			0xc0
#define C33_ID_FRASH_ERASE			0xc1
#define C33_ID_FRASH_LOAD			0xc2
#define C33_ID_FRASH_COPY			0xc3
#define C33_ID_FRASH_ADDR_INFO_SET	0xc4
#define C33_ID_FRASH_ADDR_INFO_GET	0xc5
#define C33_ID_FRASH_ENTRY_INFO_SET 0xc6
#define C33_ID_FRASH_ENTRY_INFO_GET 0xc7
#define C33_ID_ICD_MEM_WRITE		0xc8
#define C33_ID_ICD_MEM_READ			0xc9
#define C33_ID_FRASH_ENTRY_INFO_GET 0xc7
#define C33_ID_SET_DEBUG_BASE		0xd0
#define C33_ID_OSCWAIT_COUNT_SET	0xd1
#define C33_ID_ICE_COMMAND			0x100
#define C33_ID_DEF_TIMEOUT			0x101
#define C33_ID_SP_REG_RD_0			0x102
#define C33_ID_SP_REG_WT_0			0x103
#define C33_ID_GO_ACK				0x104

/* command error code */
#define C33_ID_ERR_CODE_TARGET		0x2
#define C33_ID_ERR_CODE_RUN 		0x3
#define C33_ID_ERR_CODE_NOT_RUN 	0x8
#define C33_ID_ERR_CODE_ID			0x9
#define C33_ID_ERR_CODE_SOFT_PC 	0xb
#define C33_ID_ERR_CODE_FLS_ERS 	0x20
#define C33_ID_ERR_CODE_TRC_DIS 	0x21
#define C33_ID_ERR_CODE_FLS_WT		0x80

/* ICD33 constant value */
#define C33_ICD_HOT_RESET			0
#define C33_ICD_COLD_RESET			1
#define C33_ICD_ACK 				0x30
#define C33_ICD_ERR 				0x40
#define C33_ICD_STS 				0x70
#define C33_ICD_NAK 				0x80
#define C33_ICD_RECV_MAX_SIZE		0x2010
#define C33_ICD_ERR_TARGET_DOWN 	0x02
#define C33_ICD_ERR_TARGET_RUN		0x03
#define C33_ICD_ERR_RST_TIMEOUT 	0x07
#define C33_ICD_ERR_TARGET_NOT_RUN	0x08
#define C33_ICD_ERR_INVALID_ID		0x09
#define C33_ICD_ERR_PC_BREAK		0x0b
#define C33_ICD_ERR_ESC_BREAK		0x11
#define C33_ICD_ERR_TMP_BREAK		0x16
#define C33_ICD_ERR_FLASH_SET		0x20
#define C33_ICD_ERR_TRACE			0x21
#define C33_ID_ERR_CODE_NOT_BREAK_MODE	 0x30	/* add T.Tazaki 2005/08/22 */
#define C33_ICD_ERR_FLASH_WRITE 	0x80
#define C33_ICD_VER2				C33_BIT13
#define C33_ICD_VER3				(C33_BIT13 | C33_BIT15)
#define C33_ICD_MON 				(C33_BIT13 | C33_BIT14)

/* C33 error code definition */
#define C33_ERROR_RS_SEND			100
#define C33_ERROR_RS_MAX_SIZE_OVER	101
#define C33_ERROR_RS_ACK			102   /* A text was not answered. */
#define C33_ERROR_RS_RECV_STS		103   /* Unknown STS was received. */
#define C33_ERROR_RS_STS_BUSY		104
#define C33_ERROR_RS_STS_RUN		105
#define C33_ERROR_RS_NAK_FLEMMING	106
#define C33_ERROR_RS_NAK_BCC		107
#define C33_ERROR_RS_NAK_OVERRUN	108
#define C33_ERROR_RS_NAK_TIMEOUT	109
#define C33_ERROR_RS_ERR			110
#define C33_ERROR_RS_COMMUNICATION	111
#define C33_ERROR_RS_TIMEOUT		112
#define C33_ERROR_RS_CLOSE			113   /* Remote connection closed. */
#define C33_ERROR_RS_KEY_BREAK		114   /* Key break request. */
#define C33_ERROR_RS_BCC_CODE		115   /* BCC disagreement. */
#define C33_ERROR_USB_SEND			200
#define C33_ERROR_USB_RECV			201

#define SOFT_BREAK_INST 0x0400		/* add T.Tazaki 2005/05/06 */


#define C33_COMMAND_NAME				 "c33"
#define C33_COMMAND_HELP				 "Send command to ICD33"

#define C33_ERR_MSG_MISMATCH_MACHINE	 "It is not c33 architecture ELF file."
#define C33_ERR_MSG_TARGET_CONNECT		 "It has not connected with a target."
#define C33_ERR_MSG_DEVICE_NAME 		 "Specification is required in the device for connecting.\n(e.g. /dev/ttyS0, /dev/ttya, COM1, etc.)"
#define C33_ERR_MSG_SERIAL_SETUP		 "A setup of a serial port was not completed."
#define C33_ERR_MSG_ICD_INIT			 "Initialization error of ICD33."
#define C33_ERR_MSG_DATA_SEND			 "Data transmitting failure."
#define C33_ERR_MSG_ICD_CONNECT 		 "It was not connectable with ICD33."
#define C33_ERR_MSG_ICD_DETACH			 "There is no argument given to this command."
#define C33_ERR_MSG_USB_DRV 			 "Cannot open ICD33 usb driver."

#define C33_ERR_MSG_COM_NAK 			 "Transmitting failure(%u)."
#define C33_ERR_MSG_COM_STS 			 "ICD33 is busy(%u)."
#define C33_ERR_MSG_COM_RCV_DATA		 "Receiving message is inaccurate."
#define C33_ERR_MSG_COM_FLEMING 		 "Fleming error."
#define C33_ERR_MSG_COM_SYSTEM			 "Communication system error(%u)."
#define C33_ERR_MSG_COM_RCV_ACK 		 "Communication intarnal error(%u)."
#define C33_ERR_MSG_COM_TIMEOUT 		 "Timeout error %u(ICD33 -> host)."
#define C33_ERR_MSG_COM_BCC_CODE		 "Communication error(bcc)."
#define C33_ERR_MSG_USB_SEND			 "USB communication error(host->ICD33)."
#define C33_ERR_MSG_USB_RECV			 "USB communication error(ICD33->host)."
#define C33_ERR_MSG_COM_INVALID_ID		 "Invalid ID error(0x%x)."
#define C33_ERR_INTERNAL_ERR			 "C33 internal error(%s-%s)."

#define C33_ERR_MSG_ICD_TARGET			 "Target down."
#define C33_ERR_MSG_ICD_RUN 			 "CPU is running."
#define C33_ERR_MSG_ICD_NOT_RUN 		 "CPU is not running."
#define C33_ERR_MSG_ICD_ID				 "Not defined ID, %u."
#define C33_ERR_MSG_ICD_SOFT_PC 		 "Cannot set soft pc break."
#define C33_ERR_MSG_ICD_HARD_PC 		 "Cannot set hard pc break."
#define C33_ERR_MSG_ICD_TRG_ADDR		 "Cannot set trigger address."
#define C33_ERR_MSG_ICD_FLS_ERS 		 "Erase routine is not set."
#define C33_ERR_MSG_ICD_FLS_WT			 "Cannot write flash memory."
#define C33_ERR_MSG_ICD_TRC_DIS 		 "ICD trace function is the disabled mode."
#define C33_ERR_MSG_TARGET_AREA_BRK_DIS  "TARGET area break function is disable."
#define C33_ERR_MSG_TARGET_BUS_BRK_DIS	 "TARGET bus break function is disable."
#define C33_ERR_MSG_TARGET_BUS_TRACE_DIS "TARGET bus trace function is disable."
#define C33_ERR_MSG_ICD_NOT_BREAK_MODE	 "CPU is not break mode."  /* add T.Tazaki 2005/08/22 */

#define C33_ERR_MSG_BRK_SOFT_MAX		 "Cannot set soft pc break any more."
#define C33_ERR_MSG_BRK_HARD_MAX		 "Cannot set hard pc break any more."
#define C33_ERR_MSG_BRK_DATA_MAX		 "Cannot set data break any more."
#define C33_ERR_MSG_BRK_SOFT_ROM		 "Cannot set soft pc break at ROM area."
#define C33_ERR_MSG_BRK_SOFT_CLR		 "Cannot clear soft pc break(0x%x)."
#define C33_ERR_MSG_BRK_HARD_CLR		 "Cannot clear hard pc break(0x%x)."
#define C33_ERR_MSG_BRK_DATA_CLR		 "Cannot clear data break(0x%x)."
#define C33_ERR_MSG_BRK_EXT 			 "Address(0x%x) is ext or delayed instruction."
#define C33_ERR_MSG_BRK_BRK_INST		 "Address(0x%x) is break instruction."
#define C33_ERR_MSG_BRK_ROM 			 "Address(0x%x) is no map or ROM area."
#define C33_ERR_MSG_BRK_NO_MAP			 "Address(0x%x) is no map area."
#define C33_ERR_MSG_BRK_OVER_WT 		 "Address(0x%x) is over write."
#define C33_ERR_MSG_BRK_NO_RAM			 "Address(0x%x) is no ram area."

#define C33_ERR_MSG_SET_PC_EXT			 "Address(0x%x) is ext or delayed instruction."

#define C33_ERR_MSG_ICE_COMMAND_SEND	 "Cannot send command."
#define C33_ERR_MSG_ICE_COMMAND_RECV	 "Cannot receive command."

#define C33_ERR_MSG_TARGET_DATA 		 "Target connection causes data error at %u times."
#define C33_ERR_MSG_TARGET_SIO			 "Target connection causes communication error at %u times."
#define C33_ERR_MSG_TARGET_TIME 		 "Target connection causes time out error at %u times."

#define C33_ERR_MSG_PAR_FMT 			 "Invalid parameter file(%u : %s)."
#define C33_ERR_MSG_PAR_ADDR			 "Invalid parameter file, start address > end address(%u : %s)."
#define C33_ERR_MSG_MEM_ERR 			 "Cannot allocate memory."
#define C33_ERR_MSG_NO_MAP				 "Cannot load to no map memory. (0x%x-0x%x)\n"

#define C33_ERR_COMMAND_INVALID 		 "C33 command error, invalid command."
#define C33_ERR_COMMAND_MAX_LEN 		 "C33 command error, command is too long."
#define C33_ERR_COMMAND_PARAM			 "C33 command error, number of parameter."
#define C33_ERR_COMMAND_ADDR			 "C33 command error, start address > end address."
#define C33_ERR_COMMAND_CYCLE			 "C33 command error, start cycle < end cycle."
#define C33_ERR_COMMAND_TRC_SIZE		 "C33 command error, max cycle is 131071(ICD33V2) or 1048575(ICD33V3)."
#define C33_ERR_COMMAND_TRC_LINE		 "C33 command error, pre line and post line is 255 max."
#define C33_ERR_COMMAND_ALREADY_EXEC	 "C33 command error, command is already execution ending."
#define C33_ERR_COMMAND_NO_SUPPORT_MES	 "C33 command error, command is not supported at present mode."
#define C33_ERR_COMMAND_INVALID_PARAM	 "C33 command error, invalid parameter."
#define C33_ERR_COMMAND_OUTPUT_MODE 	 "C33 command error, trace mode is 0 or 1."
#define C33_ERR_TARGET_NOT_STD			 "Target CPU is not standard macro."
#define C33_ERR_COMMAND_NO_PC_TRACE_MES  "C33 command error, ICD pc trace function is the disabled mode."
#define C33_ERR_COMMAND_NO_BUS_TRACE_MES "C33 command error, ICD bus trace function is the disabled mode."
#define C33_ERR_COMMAND_OSCWAIT_NO_SUPPORT_MES	 "C33 command error, command is not supported at present mode. \nPlease update the firmware of ICD to 4.1 or more versions."
#define C33_ERR_CLOCK_CANNOT_DISPLAY	 "Cannot display clock counter.\nTime measurement should use continue or until command."

#define C33_WARN_NOT_ADV				 "Warnning: target file is not advanced macro program.\n"
#define C33_WARN_NOT_STD				 "Warnning: target file is not standard macro program.\n"
#define C33_WARN_NOT_PE 				 "Warnning: target file is not PE program.\n"

#define C33_ERR_FILE_OPEN				 "Cannot open file(%s)."
#define C33_ERR_FILE_WRITE				 "Cannot write file."
#define C33_ERR_NO_PARAMETER_FILE		 "No memory map information."

#define C33_ERR_TIMER_OVERFLOW			 "Clock timer overflow."
#define C33_ERR_CANNOT_MEASIRE			 "Cannot measure clock timer."


#define C33_ERR_FILE_NOT_MOT			 "Load file is not motorola format.(%s)"
#define C33_ERR_LOAD_ADDR_OVER			 "Load max address(0x%x) overflow."
#define C33_ERR_FILE_FORMAT 			 "Load motorola file format error.(%s)"
#define C33_ERR_MAX_ADDR_OVER			 "Max address(0x%x) overflow."
#define C33_ERR_SIZE_LIMIT_OVER 		 "Size limit(0x%x-0x%x) over."
#define C33_ERR_SIZE_LIMIT_OVER2		 "Size limit(0x%x) over."

#define C33_ERR_TIMER_OVER				 "Cannot set timer. Timer Conditions: 1<=Timer<=300000"
#define C33_WARN_DEFAULT_TRACE_MODE_SET	 "Cannot set default trace mode."

#define C33_MSG_LOG_ON					 "log on\n"
#define C33_MSG_LOG_OFF 				 "log off\n"
#define C33_MSG_KEY_BREAK				 "\nBreak by key break."				/* It is cautious of a new-line position. */
#define C33_MSG_EXTERNAL_BREAK			 "\nBreak by external break."			/* It is cautious of a new-line position. */
/* >>>>> add iruma m.takeishi '03.12.03 */
#define C33_MSG_MON_KEYBREAK_ERR		 "key break is done,but target is still running."
#define C33_MSG_MON_TARGETBAD			 "Now can't access target."
#define C33_MSG_MON_KEYBREAK_RECOVER	 "target stopped.please step or continue again."
/* <<<<< add iruma m.takeishi '03.12.03 */

#define C33_MSG_TARGET_ICD_NAME 		 "C33 ICD33 debugging\n"
#define C33_MSG_TARGET_ICD_FILE_INFO	 "Debugging a target via the EPSON C33 ICD33.\n"
#define C33_MSG_TARGET_CONNECT_INFO 	 "Connecting with target ... "
#define C33_MSG_TARGET_SET_DEBUG_BASE	 "CPU type and debug unit address setting ... "
#define C33_MSG_TARGET_INITIALIZE_INFO	 "Initializing ............. "
#define C33_MSG_TARGET_COLD_RESET_INFO	 "CPU cold resetting ....... "
#define C33_MSG_TARGET_HOT_RESET_INFO	 "CPU hot resetting ........ "
#define C33_MSG_TARGET_BOOT_ADDR_INFO	 "Boot address ............. 0x%lX\n"
#define C33_MSG_TARGET_DONE 			 "done \n"
#define C33_MSG_TARGET_FAILURE			 "failure \n"
#define C33_MSG_TARGET_CANCEL			 "\nCancel. \n"
#define C33_MSG_TARGET_END_DEBUG		 "Ending c33 debugging.\n"
#define C33_MSG_TARGET_DIAG_OK			 "Target connection test ... done \n"
#define C33_MSG_TARGET_DIAG_OMITTED 	 "Target connection test ... omitted \n"
// #define C33_MSG_TARGET_ICD_HARD_VER		"ICD hardware version   ... %02x \n"
#define C33_MSG_TARGET_ICD_HARD_VER 	 "ICD hardware version   ... %u.%u \n"
#define C33_MSG_TARGET_ICD_SOFT_VER 	 "ICD software version   ... %u.%u \n"
#define C33_MSG_TARGET_ICD_FLS			 "Flash memory setup\n"
#define C33_MSG_TARGET_RESET_TARGET 	 "TARGET resetting ................... "

#define C33_MSG_MAP_INFO_HEADER 		 "Memory map information Type  Wait(r/w) Size  Endian \n"
#define C33_MSG_MAP_INFO_NO_DATA		 "No data.\n"
#define C33_MSG_MAP_INFO_STACK			 "    %08x-%08x  STACK\n"
#define C33_MSG_MAP_INFO_ROM			 "    %08x-%08x  ROM     %u/%u     %2uBit "
#define C33_MSG_MAP_INFO_RAM			 "    %08x-%08x  RAM     %u/%u     %2uBit "
#define C33_MSG_MAP_INFO_IO 			 "    %08x-%08x  IO      %u/%u     %2uBit "
#define C33_MSG_MAP_INFO_STD			 "CPU : Standard\n"
#define C33_MSG_MAP_INFO_ADV			 "CPU : Advanced\n"
#define C33_MSG_MAP_INFO_PE 			 "CPU : PE\n"
#define C33_MSG_MAP_BIG_ENDIAN			 "Big\n"
#define C33_MSG_MAP_LITTLE_ENDIAN		 "Little\n"
#define C33_MSG_FILL					 "Start address = 0x%x, End address = 0x%x, Fill data = 0x%x ....."
#define C33_MSG_MOVE					 "Start address = 0x%x, End address = 0x%x, Destination address = 0x%x ....."
#define C33_MSG_FLS 					 "Flash start address = 0x%x, Flash end address = 0x%x \nFlash erase routine address = 0x%x, Flash write routine address = 0x%x ....."
#define C33_MSG_FLE 					 "Control register = 0x%x, Start block = 0x%x , End block = 0x%x ....."
#define C33_MSG_FLE_END 				 "Finish with 0x%x\n"
#define C33_MSG_TRC_HEADER		 "   Cycle   Address  Code Unassemble                Clk   Method File             Line   SourceCode"
#define C33_MSG_TRC_READ				 "Reading trace data........."
#define C33_MSG_TRC_ANALYZING			 "Analyzing trace data......."
#define C33_MSG_TRC_MAKING_FILE 		 "Making file................"
#define C33_MSG_TRC_SEARCH_PROCESSING	 "Search processing..."
#define C33_MSG_TRC_FINISHED			 "Finished.\n"
#define C33_MSG_TRC_OVER_WRITE_YES		 "Trace mode       : over write enable mode\n"
#define C33_MSG_TRC_OVER_WRITE_NO		 "Trace mode       : over write disable mode\n"
#define C33_MSG_TRC_AREA				 "Trace mode       : area trace mode\n"
#define C33_MSG_TRC_INST				 "Clock display    : instruction\n"
#define C33_MSG_TRC_ACCUMULATE			 "Clock display    : accumulate\n"
#define C33_MSG_TRC_TRIGGER1_ADDR		 "Trigger address1 : 0x%08x\n"
#define C33_MSG_TRC_TRIGGER2_ADDR		 "Trigger address2 : 0x%08x\n"
#define C33_MSG_TRC_BREAK_NO			 "Trigger2 break   : disable\n"
#define C33_MSG_TRC_BREAK_YES			 "Trigger2 break   : enable\n"
#define C33_MSG_TRC_MEASURE_ALL 		 "Measure mode     : all\n"
#define C33_MSG_TRC_MEASURE_AREA		 "Measure mode     : area\n"
#define C33_MSG_TRC_HARD_PC_BREAK_OFF	 "Warning: Hard PC break does not stop at present mode.\n"
#define C33_MSG_DUMP_FILE_HEADER		 "  addr   +0 +1 +2 +3 +4 +5 +6 +7 +8 +9 +A +B +C +D +E +F\n"
#define C33_MSG_DUMP_BIN				 "Start address = 0x%x, End address = 0x%x, File type = Binary\n"
#define C33_MSG_DUMP_TEXT				 "Start address = 0x%x, End address = 0x%x, File type = Text\n"
#define C33_MSG_DUMP_S3 				 "Start address = 0x%x, End address = 0x%x, File type = Motorola-S3\n"
#define C33_MSG_DUMP_PROCESSING 		 "Processing %08lX-%08lX address.\n"
#define C33_MSG_READ_MEMORY 			 "Read memory ... "

#define C33_MSG_BUS_TRC_READ			 "Reading bus trace data....."
#define C33_MSG_BUS_TRC_ANALYZING		 "Analyzing bus trace data..."
//#define C33_MSG_PC_BUS_NORMAL_TRC_HEADER"   Cycle   Address  Code Unassemble                Clk   Method File             Line   SourceCode                BusMaster Address  Data      R/W  Size USERIN  "
#define C33_MSG_PC_BUS_NORMAL_TRC_HEADER  "   Cycle   Address  Code Unassemble                Clk     BM         Address  Data      I/D RW SZ UIN  Method File             Line   SourceCode  "
#define C33_MSG_PC_BUS_USER_TRC_HEADER	  "   Cycle   Address  Code Unassemble                Clk     DTD        DTS    UIN  Method File             Line   SourceCode  "
//#define C33_MSG_BUS_NORMAL_TRC_HEADER   "   Cycle   Clk     BusMaster Address  Data      R/W  Size USERIN  "
#define C33_MSG_BUS_NORMAL_TRC_HEADER	  "   Cycle   Clk     BM        Address  Data      I/D RW SZ UIN Unassemble"
#define C33_MSG_BUS_USER_TRC_HEADER 	  "   Cycle   Clk     DTD       DTS    UIN  "
#define C33_MSG_BUS_TRC_MODE_DISABLE	  "Bus trace mode is disabled.\n"

#define C33_MSG_FLS3_PROG_LOAD			 "Load flash erase/write program ..."
#define C33_MSG_FLS3_DATA_LOAD			 "Load flash data ..."
#define C33_MSG_FLS3_COPY				 "Flash copy destination address=0x%x,source address=0x%x,size=0x%x ..."
#define C33_MSG_FLS3_ERASE_DATA 		 "Erase flash data ..."
#define C33_MSG_FLS3_ERASE_PROGRAM		 "Erase flash erase/write program ..."
#define C33_MSG_FLS3_DATA_INFO1 		 "ICD data offset address     : %08x\n"
#define C33_MSG_FLS3_DATA_INFO2 		 "CPU data address            : %08x\n"
#define C33_MSG_FLS3_DATA_INFO3 		 "Data size                   : %08x\n"
#define C33_MSG_FLS3_DATA_INFO4 		 "Erase start block           : %08x\n"
#define C33_MSG_FLS3_DATA_INFO5 		 "Erase end block             : %08x\n"
#define C33_MSG_FLS3_DATA_INFO6 		 "Erase parameter             : %08x\n"
#define C33_MSG_FLS3_DATA_INFO7 		 "Comment : %s\n"
#define C33_MSG_FLS3_ENTRY_INFO1		 "ICD program offset address  : %08x\n"
#define C33_MSG_FLS3_ENTRY_INFO2		 "CPU program address         : %08x\n"
#define C33_MSG_FLS3_ENTRY_INFO3		 "Program size                : %08x\n"
#define C33_MSG_FLS3_ENTRY_INFO4		 "Erase routine entry address : %08x\n"
#define C33_MSG_FLS3_ENTRY_INFO5		 "Write routine entry address : %08x\n"
#define C33_MSG_FLS3_ENTRY_INFO6		 "Comment : %s\n"

#define C33_MSG_FIRM_UPDATE 			 "ICD firmware update ..."
#define C33_MSG_FIRM_UPDATE_END 		 "Please quit gdb, and power off ICD when OK LED is on.\n"

#define C33_MSG_GO_BREAK_OVER_WRITE 	 "Warning: Software break address 0x%x was over written by user program.\n"

#define C33_MSG_AREA_BREAK_ON			 "Area break on\n"
#define C33_MSG_AREA_BREAK_OFF			 "Area break off\n"
#define C33_MSG_AREA_BREAK_BBCU_SELECT	 "BBCU bus master select   : %s\n"
#define C33_MSG_AREA_BREAK_RW			 "Read/Write               : %s\n"
#define C33_MSG_AREA_BREAK_AREA 		 "Area                     : %s\n"

#define C33_MSG_BUS_BREAK_ON			 "Bus break on\n"
#define C33_MSG_BUS_BREAK_OFF			 "Bus break off\n"
#define C33_MSG_BUS_BREAK_BUS_SELECT	 "Bus select               : %s\n"
#define C33_MSG_BUS_BREAK_BBCU_SELECT	 "BBCU bus master select   : %s\n"
#define C33_MSG_BUS_BREAK_TRIG_BREAK	 "Trigger/Break            : %s\n"
#define C33_MSG_BUS_BREAK_RW			 "Read/Write               : %s\n"
#define C33_MSG_BUS_BREAK_INST_DATA 	 "Instruction/Data         : %s\n"
#define C33_MSG_BUS_BREAK_ACCESS_SIZE	 "Access size              : %s\n"
#define C33_MSG_BUS_BREAK_INT			 "Break by interrupt level : %s\n"
#define C33_MSG_BUS_BREAK_INT_LEVEL 	 "Interrupt level          : %d\n"
#define C33_MSG_BUS_BREAK_DATA_COMP 	 "Data compare             : %s\n"
#define C33_MSG_BUS_BREAK_ADDRESS		 "Address                  : %08x\n"
#define C33_MSG_BUS_BREAK_ADDRESS_MASK	 "Address mask             ; %08x\n"
#define C33_MSG_BUS_BREAK_DATA			 "Data                     : %08x\n"
#define C33_MSG_BUS_BREAK_DATA_MASK 	 "Data mask                : %08x\n"
#define C33_MSG_BUS_BREAK_COUNT 		 "Break counter            : %08x\n"
#define C33_MSG_BUS_BREAK_COUNT_CLR_SAVE "Break counter Clear/Save : %s\n"

#define C33_MSG_SET_BUS_TRACE			 "Set bus trace\n"
#define C33_MSG_BUS_TRACE_BUS_SELECT	 "Bus select               : %s\n"
#define C33_MSG_BUS_TRACE_BBCU_SELECT	 "BBCU bus master select   : %s\n"
#define C33_MSG_BUS_TRACE_TRACE_MODE	 "Trace mode               : %s\n"
#define C33_MSG_BUS_TRACE_RW			 "Read/Write               : %s\n"
#define C33_MSG_BUS_TRACE_INST_DATA 	 "Instruction/Data         : %s\n"
#define C33_MSG_BUS_TRACE_ACCESS_SIZE	 "Access size              : %s\n"
#define C33_MSG_BUS_TRACE_BUS_OUTPUT	 "Bus output               : %s\n"
#define C33_MSG_BUS_TRACE_BUS_SIZE		 "Address bus size         : %d byte\n"
#define C33_MSG_BUS_TRACE_COMPRESS		 "Compression              : %s\n"
#define C33_MSG_BUS_TRACE_USER_INPUT	 "Bus trace trigger select : %s\n"

// add 3 mssage 2006/06/09 M.Kai >>>
#define C33_MSG_USER_DEF_FILE_NOT_FOUND 		  "'userdefine.gdb' is not found. \n"
#define C33_MSG_USER_RSTC_FILE_NOT_FOUND		  "'resetcold.gdb' is not found. \n"
#define C33_MSG_USER_RSTH_FILE_NOT_FOUND		  "'resethot.gdb' is not found. \n"

#define C33_USER_DEFINE			1	/* add T.Tazaki 2005/06/12 */
#define C33_USER_COLD_RESET		2	/* add T.Tazaki 2005/06/12 */
#define C33_USER_HOT_RESET		3	/* add T.Tazaki 2005/06/12 */
// add 3 mssage 2006/06/09 M.Kai <<<


/* trace mode structure */
struct c33_icd_trace_mode
{
	int iTraceMode; 		   /* trace mode(all-OverWrite Yes, all-OverWrite No, area-OverWrite No) */
	int iCounterMode;		   /* counter mode(accumulate, instruction) */
	unsigned long ulTrg1;
	unsigned long ulTrg2;
	int iBreak;
	int iMeasure;
};

struct c33_map_info
{
	int iMapAttr;					  /* ROM or RAM or I/O or STACK*/
	int iEndian;					  /* big or little */
	unsigned long ulReadCycle;		  /* read cycle */
	unsigned long ulWriteCycle; 	  /* write cycle */
	unsigned long ulDeviceSize; 	  /* C33_BYTE or C33_HALF_WORD or C33_WORD */
	unsigned long ulStartAddr;		  /* map start address */
	unsigned long ulEndAddr;		  /* map end address */
	struct c33_map_info *pMapNext;	  /* next map */
	unsigned char* pMemBlock;		  /* pointer to memory block */
};

struct c33_target_info
{
	/* CPU condition */
	unsigned char ucAdvanced;
	unsigned char ucBrkFunc;
	unsigned char ucCpuType;			  /* ICD V3 target cpu type */
	unsigned long ulDebugBase_rom_addr;   /* ICD V3 debug unit ROMbase address */
	unsigned long ulDebugBase_ram_addr;   /* ICD V3 debug unit RAMbase address */

	/* target condition */
	int iIcd_open;
	int iSim_open;
	int iIcdExeStatus;
	unsigned long ulBootAddr;
	int iErrorCode;
	int iIcdErrorCode;
	int iKeyBreak;
	unsigned char ucIcdErrorInfo[9];

	/* map information */
	struct c33_map_info *pMapInfo;

	int iVerRdSioErr;
	int iVerRdSioTryCount;
	int iVerRdIcdStatus;
	int iVerRdIcdHardVer;
	int iVerRdIcdFirmVer;
	int iVerRdEmuMemNum;
	int iVerRdDIAG;
	int iVerRdTrgtDown;
	int iVerRdMonStatus;

	int iRunMonStatus;
	int iRunBreakStatus;
	int iRunSoftPcBreakStatus;
	int iRunSoftPcClear;
	int iRunSec1;
	int iRunSec2;
	int iRunCycle;
	int iParallelDump1;
	int iParallelDump2;
	int iParallelDump3;
	int iParallelDump4;
	int iRunCounter;		/* Generan Couinter(Cycle,50ns,1us) for ICDV6 add T.Tazaki 2007/07/03 */

	/* call */
	int iFlashSet;
	unsigned long ulFlashStartAddr;
	unsigned long ulFlashEndAddr;
	unsigned long ulEraseRoutineAddr;
	unsigned long ulWriteRoutineAddr;
	int iCallMode;
	char cCallFileName[256];

	/* break */
	unsigned long ulSoftPcBreak[C33_STDOUT_BREAK_NUM];
	unsigned short usSoftPcBreakOrgCode[C33_STDOUT_BREAK_NUM+10];	/* Original code save : add T.Tazaki 2005/05/06 */
	unsigned long ulHardPcBreak[C33_HARD_PC_BREAK_MAX];
	unsigned long ulDataBreak;
	unsigned long ulSoftPcBreakTemp[C33_STDOUT_BREAK_NUM];/* icd only*/

	/* stdio */
	unsigned long ulReadBufAddr;	/* stdin */
	unsigned long ulWriteBufAddr;	/* stdout */
	char cStdinFileName[256];
	char cStdoutFileName[256];

	/* mode */
	int iOnTheFly;

	/* clock mode */
	int iClockMode;
	int iClockFunction;

	/* icd program load flag */
	int iIcdLoad; /* load command executing */

	/* icd program rm flag */
	int iIcdRm;  /* rm command executing */

	/* icd trace mode */
	struct c33_icd_trace_mode c33TraceMode;    /* icd trace mode structure */

	/* Area Break condition */
	int  iArea_Mode;			/* 0:clear 1:set									*/
	char cArea_BBCUBusSelect;	/* BBCU bus select									*/
	int iArea_ReadWrite;		/*read/write set  0:read 1:write 2:read/write		*/
	unsigned char  ucArea1; 	/* CE area 7-0										*/	
	unsigned char  ucArea2; 	/* CE area 15-8 									*/	
	unsigned char  ucArea3; 	/* CE area 22-16									*/	

	/* Bus Break condition */
	int  iBus_Mode; 			/* 0:clear 1:set									*/
	int  iBus_BusSelect;		/* Bus select										*/
	char cBus_BBCUBusSelect;	/* BBCU bus select									*/
	int  iBus_TrigBreak;		/* 0:Trig out only 1:break							*/
	int  iBus_ReadWrite;		/* read/write set  0:read 1:write 2:read/write		*/
	int  iBus_InstData; 		/* 0:Inst 1:data 2:both 							*/
	char cBus_Access;			/* bit0:BYTE bit1:HALF bit2:WORD					*/
	int  iBus_Int;				/* 0:Break by int level disable 1:enable			*/
	int  iBus_IntLevel; 		/* 0:Trig out only 1:break							*/
	int  iBus_DataComp; 		/* 0:32bit compare 1:high16bit/low 16bit compare	*/
	unsigned long ulBus_Address;	/* break address								*/
	unsigned long ulBus_AddressMask;/* break address mask bit						*/
	unsigned long ulBus_Data;		/* break data									*/
	unsigned long ulBus_DataMask;	/* break data mask bit							*/
	unsigned long ulBus_Count;		/* break counter								*/
	int  iBus_iCntClrSave;		/* 0:Save 1:Clear when Go							*/

	/* Bus Trace condition */
	int  iBust_Mode;			/* 0:not set 1:set									*/
/*	  int  iBust_BusSelect;   */	/* Bus select							   del T.Tazaki 2004/06/25		   */
	char cBust_BBCUBusSelect;	/* BBCU bus select									*/
	int  iBust_TraceMode;		/* 0:Normal 1:User									*/
	int  iBust_ReadWrite;		/* read/write set  0:read 1:write 2:read/write		*/
	int  iBust_InstData;		/* 0:Inst 1:data 2:both 							*/
	char cBust_Access;			/* bit0:BYTE bit1:HALF bit2:WORD					*/
	int  iBust_BusOutput;		/* 0:Address bus,1:Data bus 2:both					*/
	int  iBust_AddrBusSize; 	/* address bus size 1-4 							*/
	int  iBust_Compress;		/* 0:NO data/address compress 1:compress			*/
	int  iBust_UserInput;		/* 0:TRC IN signal 1:DBT signal 					*/

	/* Logic analyzer mode */
	int  iLogiana_Mode;			/* mode 0:off 1:on									*/
	int  iLogiana_UserIn;		/* User trace signal 0:external trace input 		*/
								/*					 1:target CPU trigger input 	*/

};

/* icd flash writer command structure*/

struct c33_flash_writer_info
{
	/* address information */
	unsigned long ulIcdDataAddress;   /* ICD destination address(ICD flash)   */
	unsigned long ulCpuDataAddress;   /* Write/Verify address	(CPU flash)   */
	unsigned long ulDataSize;		  /* Write/Verify byte size 			  */
	unsigned long ulEraseStartBlock;  /* Erase start block					  */
	unsigned long ulEraseEndBlock;	  /* Erase end block					  */
	unsigned long ulEraseParam; 	  /* Erase parameter					  */
	char szDataComment[130];		  /* Data Comment string				  */

	/* write/erase program entry information */
	unsigned long ulIcdProgAddress;    /* ICD destination address(ICD flash)  */
	unsigned long ulCpuProgAddress;    /* CPU destination address(CPU flash)  */
	unsigned long ulProgSize;		   /* Program byte size 				  */
	unsigned long ulEraseEntryAddress; /* CPU Erase entry address(CPU RAM)	  */
	unsigned long ulWriteEntryAddress; /* CPU Write entry address(CPU RAM)	  */
	char szProgComment[130];		   /* Program Comment string			  */

};
/* c33 command structure */
struct c33_cmd_structure
{
	int iParamNum;
	char iStr[C33_COMMAND_ARGUMENT_MAX][C33_COMMAND_MAX];
	int iParam[C33_COMMAND_ARGUMENT_MAX];
	int iType[C33_COMMAND_ARGUMENT_MAX];
};

int c33_Cmd_Initialize (void);
int c33_Cmd_OK (void);
int c33_Cmd_DebugAddr_Set (void); /* add proto type definition T.Tazaki 2007/07/11 */
int c33_Cmd_Ver_Read (void);
int c33_Cmd_Reset (int iType);
int c33_Cmd_Reset_no_reg_write (int iType);
int c33_Cmd_Reset2 (int iType);	 /* add T.Tazaki 2008/06/17 for set $pc=boot */
int c33_Cmd_SoftPcBreakSet (unsigned long ulAddr);
int c33_Cmd_SoftPcBreakClear (unsigned long ulAddr);
int c33_Cmd_HardPcBreakSet (unsigned long ulAddr);
int c33_Cmd_HardPcBreakClear (unsigned long ulAddr);
int c33_Cmd_ReadWrite(int iReadWrite, unsigned long ulAddr, unsigned long ulLen, int iUnit, unsigned char* pcuMyAddr);

int c33_SendMessage (int iID, int iParam1, int iParam2, int iParam3, int iParam4, void *pvParam5, int iParam6, int iParam7);
int c33_RecvMessage (int iID, int iParam1, int iParam2, int iParam3, int iParam4, void *pvParam5);
void c33_command (char *arg, int from_tty);
int c33_Initialize (void);
void c33_ErrorMsg (void);
void c33_Gdbtk_Update (int iCondition);
void c33_Gdbtk_Idle (void);
void c33_wait_sec (int iWait);
int c33_BreakAddrCheck (unsigned long ulAddr, int iKind);
int c33_BreakSetROMCheck (unsigned long ulAddr);		/* add T.Tazaki 2004/09/27 */
int c33_insert_hw_breakpoint (CORE_ADDR ulAddr, void* shadow);
int c33_remove_hw_breakpoint (CORE_ADDR ulAddr, void* shadow);
int c33_insert_watchpoint (CORE_ADDR addr,int len,int type);
int c33_remove_watchpoint (CORE_ADDR addr,int len,int type);
void c33_ClearMap(void);
void c33_ClearCallMode(void);
void c33_print_source_lines(unsigned long ulAddr, char* pOut);
int c33_FindFilename(unsigned long ulAddr, char* pOut, unsigned long* ulLineNo);
int c33_ReadWriteMapMem(int iReadWrite, unsigned long ulAddr, unsigned long ulLen, int iUnit, unsigned char* pMyAddr);
int c33_CheckMapArea( CORE_ADDR ulAddr);
int c33_CheckMapArea2( CORE_ADDR ulAddr);
void c33_CreateINI(void);
void c33_gdbtkini_update(void);
void c33_CheckWindowOpen( char *pszwindow, int *iWinFlag);
void c33_Console_Output(char *string);
void c33_InitCancel(void);
int c33_CheckCancel( int iCount );
void c33_EndCancel(void);
void c33_Get_bp_status(void);							/* add T.Tazaki 2003/07/15 */
void c33_Check_bp_addr(char *pFileName, int iLine); 	/* add T.Tazaki 2003/08/29 */
void c33_error(const char *string,...);
void c33_Help(int iParamNum,char *pCmdName1,char *pCmdName2);
void c33_LogOut_String(void);							/* add T.Tazaki 2004/05/10 */
void c33_memory_window_update( int iFlag );				/* add T.Tazaki 2008/05/26 */

extern void c33_Disasm (unsigned long ulCode, char *szDisasm);

#endif
