/* ICD interface for the EPSON C33 for GDB, the GNU debugger.
   Copyright 1996, 1997, 1998, 1999, 2000, 2001, 2002
   Free Software Foundation, Inc.

   This file is part of GDB.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.	*/

#ifndef __C33ICD__
#define __C33ICD__


/* ******* ICD kind definition ******** */ 
#define C33_ICDV4 						0
#define C33_ICDV6 						1

/* ******* ICD mode definition ******** */ 
/* Execution status */
#define C33_EXE_STATUS_STOP 		  0x0
#define C33_EXE_STATUS_RUN			  0x1
#define C33_EXE_STATUS_STEP 		  0x2

#define C33_MD_ON_THE_FLY			  1
#define C33_BIT_ON_TEMP 			  C33_BIT0
#define C33_BIT_ON_THE_FLY			  C33_BIT1

#define C33_PROCESS_ID				  42000
#define C33_COMMAND_TEXT_SIZE		  1024								   /* text buffer size */
#define C33_COMMAND_BUS_TEXT_SIZE	   512								   /* bus trace text buffer size */
#define C33_COMMAND_BUFFER_SIZE 	  (2 + C33_COMMAND_TEXT_SIZE + 1 + 13) /* header + text + BCC + reserve */

#define C33_COMMAND_PARL_TEXT_SIZE	  0x1000				   /* parallel load text buffer size */

#define C33_COMMAND_START_REG_NUM	  1

#define C33_ICD_TRACE_SIZE			  (0x100000)			   /* icd33 trace buffer size			   */
#define C33_ICD_TRACE_INVALID		  0xffffffff			   /* icd33 trace data invalid			   */
#define C33_ICD_TRACE_PC_INVALID	  0xffffffff			   /* icd33 pc value invalid			   */
#define C33_ICD_TRACE_MODE_ALL_OVER_WRITE_NO   1			   /* icd33 trace over write NG 		   */
#define C33_ICD_TRACE_MODE_ALL_OVER_WRITE_YES  2			   /* icd33 trace over write OK 		   */
#define C33_ICD_TRACE_MODE_AREA_OVER_WRITE_NO  3			   /* icd33 trace mode is area			   */
#define C33_ICD_TRACE_COUNTER_ACCUMULATE  1 				   /* icd33 trace counter accumulate mode  */
#define C33_ICD_TRACE_COUNTER_INST		  2 				   /* icd33 trace counter instruction mode */
#define C33_ICD_TRACE_BREAK_OFF 		  1 				   /* icd33 trace trigger2 break off	   */
#define C33_ICD_TRACE_BREAK_ON			  2 				   /* icd33 trace trigger2 break on 	   */
#define C33_ICD_TRACE_MEASURE_ALL		  1 				   /* icd33 measure mode all			   */
#define C33_ICD_TRACE_MEASURE_AREA		  2 				   /* icd33 measure mode area			   */

#define C33_ICD_TRACE_DST_JP_ALWAYS   11					   /* execute relative jump only instruction (call xjp)*/
#define C33_ICD_TRACE_DST_NORM		  0 					   /* execute normal instruction */
#define C33_ICD_TRACE_DST_JP		  C33_BIT0				   /* execute relative jump instruction */
#define C33_ICD_TRACE_DST_ABS_JP	  C33_BIT1				   /* execute absolute jump instruction */
#define C33_ICD_TRACE_DST_IDLE		  (C33_BIT0 | C33_BIT1)    /* idle */
#define C33_ICD_TRACE_DST0_1		  (C33_BIT0 | C33_BIT1)    /* DST signel bit */
#define C33_ICD_TRACE_DST2			  C33_BIT2				   /* DST signel bit */
#define C33_ICD_TRACE_DPCO_BIT		  C33_BIT3				   /* DPCO signal bit */
#define C33_ICD_TRACE_TRG1_HIT		  C33_BIT4				   /* trigget1 hit */
#define C33_ICD_TRACE_TRG2_HIT		  C33_BIT5				   /* trigger2 hit */
#define C33_ICD_TRACE_SEARCH_HIT	  C33_BIT6				   /* search hit flag */
#define C33_ICD_TRACE_SEARCH_PRE_POST C33_BIT7				   /* pre line or post line flag */
#define C33_ICD_TRACE_HIT_PRE_POST	  (C33_ICD_TRACE_SEARCH_HIT | C33_ICD_TRACE_SEARCH_PRE_POST) /* DPCO signal bit */

#define C33_ICD_TRACE_FIX_PC_BIT	  C33_BIT0				   /* PC fix flag */
#define C33_ICD_TRACE_FIX_DPCO		  C33_BIT1				   /* analyze to dpco */
#define C33_ICD_TRACE_FIX_RET		  C33_BIT2				   /* analyze to ret instruction */
#define C33_ICD_TRACE_FIX_RETI		  C33_BIT3				   /* analyze to reti instruction */
#define C33_ICD_TRACE_FIX_MAP		  C33_BIT4				   /* analyze to map information */
#define C33_ICD_TRACE_FIX_TIRGGER	  C33_BIT5				   /* analyze to trigger information */
#define C33_ICD_TRACE_FIX_START_PC	  C33_BIT6				   /* analyze to trigger information */
#define C33_ICD_TRACE_INTERRUPT 	  C33_BIT7				   /* interrupt point flag */

#define C33_ICD_DPCO_FIX29			  29					   /* Address all output 29:dummy2 + 28 - 1 */
#define C33_ICD_DPCO_25 			  25					   /* abs jump address */
#define C33_DPCO_TIMING 			  2 					   /* dpco output timing */

#define C33_ICD_DPCO_FIX32			  32					   /* Address all output 32:dummy1 + 32 - 1 */

#define C33_PRE_POST_LINE_MAX		  255					   /* pre and post line max */

#define C33_ICD_TRACE_FULL_BIT		  C33_BIT8				   /* trace full bit */

/* motorola file */
#define C33_REC_NAME				  0
#define C33_REC_DOWNLOAD			  1
#define C33_REC_IGNORE				  2
#define C33_REC_ERROR				  3

/* area break status */
#define C33_AREA_BREAK_STATUS0		0x78038 				  /* area break status register 0 (B) */
															  /* D7-6 : ABU busmaster unit 11:USER,10:DMA,01:CCU,00:HBCU */
															  /* D5   : R/W   1:read,0:write					*/
															  /* D4-0 : Area									*/
#define C33_AREA_BREAK_STATUS1		0x78039 				  /* area break status register 1 (B) */
															  /* D2   : data type  1:inst,0:data  */
															  /* D1-0 : data size  00:byte,01:half,10:word */
/* bus break */
#define C33_BBCNT					0x78058 				  /* bus break counter register address (W) */

/* bus trace definitions */
/* DTS[4:0] */

#define C33_IDLE			0x00

#define RDDATA_START_B_RD	0x01
#define RDDATA_START_H_RD	0x03
#define RDDATA_COMPR_0		0x04
#define RDDATA_CONT 		0x05
#define RDDATA_COMPR_1		0x06
#define RDDATA_START_W_RD	0x07

#define DMAUSR_START_B_WR	0x08
#define DMAUSR_START_B_RD	0x09
#define DMAUSR_START_H_WR	0x0a
#define DMAUSR_START_H_RD	0x0b
#define DMAUSR_COMPR_MATCH	0x0c
#define DMAUSR_CONT 		0x0d
#define DMAUSR_START_W_WR	0x0e
#define DMAUSR_START_W_RD	0x0f

#define DATA_START_B_WR 	0x10
#define DATA_START_B_RD 	0x11
#define DATA_START_H_WR 	0x12
#define DATA_START_H_RD 	0x13
#define DATA_COMPR_MATCH	0x14
#define DATA_CONT			0x15
#define DATA_START_W_WR 	0x16
#define DATA_START_W_RD 	0x17

#define DMAUSR_COMPR_0		0x18
#define DMAUSR_COMPR_1		0x19

#define DATA_COMPR_0		0x1a
#define DATA_COMPR_1		0x1b

#define INST_COMPR_MATCH	0x1c
#define INST_CONT			0x1d

#define INST_START			0x1f


#define C33_DATA	0
#define C33_DMAUSR	1
#define C33_INST	2

#define C33_ADDRESS_ONLY	0
#define C33_DATA_ONLY		1
#define C33_ADDRESS_DATA	2

#define C33_ADDRESS_NOT_COMPLETE 0
#define C33_ADDRESS_COMPLETE	 1
#define C33_ADDRESS_COMPRESS	 2

/* icd33 trace data structure */ 
struct c33_icd_trace_data
{
	int iOutputMode;				/* trace data output mode:0=PC Trace Only, 1=PC Trace + Bus Trace */

	unsigned char ucDst[C33_ICD_TRACE_SIZE];/* dst signal */
		/* bit0,1 DST			   */
		/* bit3   DPCO			   */
		/* bit4 hit trigger1	   */
		/* bit5 hit trigger2	   */
		/* bit6 search hit		   */
		/* bit7 pre line,post line */
	unsigned char ucMethod[C33_ICD_TRACE_SIZE];/* analyze method and status */
		/* bit0 fix PC flag 	   */
		/* bit1 fix DPCO		   */
		/* bit2 ret 			   */
		/* bit3 reti			   */
		/* bit4 map 			   */
		/* bit5 trigger 		   */
		/* bit6 				   */
		/* bit7 no map			   */
	unsigned char ucDpcoNum[C33_ICD_TRACE_SIZE];/* DPCO signal output number */
	unsigned long ulPC[C33_ICD_TRACE_SIZE]; 	/* PC	*/
	unsigned short usCode[C33_ICD_TRACE_SIZE];	/* code */
	unsigned long ulValidStart; 		  /* valid data point				*/
	unsigned long ulValidEnd;			  /* valid data point				*/
	unsigned long ulDspStart;			  /* display start position 		*/
	unsigned long ulDspEnd; 			  /* display end position			*/

	unsigned long ulStartPcAddr;		  /* exection start pc				*/
	unsigned char ucSearchFlag; 		  /* search data on/off 			*/
	unsigned char ucAnalyzeFlag;		  /* analyze end flag				*/
	unsigned char ucDebugIdleDsp;		  /* Debug function,idle cycle display */
	unsigned long ulCycleOffset;		  /* accumulation of offset 		*/

	/* trace mode */
	struct c33_icd_trace_mode stTraceMode;

	unsigned long ulDataMaxNumber;		  /* trace data max numbar			*/

	unsigned long ulIcdTraceBufferSize;   /* icd trace buffer size	V2=0x2000 V3=0x100000 */
	unsigned long ulSearchMaxNumber;	  /* search data max numbar 		*/

	unsigned char ucDpcoAddrFix;		  /* DPCO address size */

	/* ICD V3 bus trace data */
	unsigned long ulValidBusStart;		  /* valid data point				*/
	unsigned short usBusTraceData[C33_ICD_TRACE_SIZE];

	/* ICD3 bus trace output data */
	unsigned long	ulBusTraceCycle;					/* Bus Trace counter after analysis */
	char			cBusDataExist[C33_ICD_TRACE_SIZE];	/* 0:data none	1:data exist */
	char			cAddrData[C33_ICD_TRACE_SIZE];		/* 0:address only,1:data only,2:address and data */
	unsigned long	ulAddress[C33_ICD_TRACE_SIZE];
	unsigned long	ulData[C33_ICD_TRACE_SIZE];
	char			cSize[C33_ICD_TRACE_SIZE];			/* bus size 1:byte 2:half 4:word */
	char			cAccess[C33_ICD_TRACE_SIZE];		/* 0:Read 1:Write  */
	char			cBus[C33_ICD_TRACE_SIZE];			/* 0:DATA 1:DMA/USR 2:INST */
	char			cUserIn[C33_ICD_TRACE_SIZE];		/* USERIN  0:low 1:high */
	char			cAddressLength[C33_ICD_TRACE_SIZE]; /* get address length */
	char			cDataLength[C33_ICD_TRACE_SIZE];	/* get data length */
	char			cAddrComplete[C33_ICD_TRACE_SIZE];	/* 0:address not complete,1:complete */
	char			cAddrSame[C33_ICD_TRACE_SIZE];		/* 0:address not same,1:same */

};

/************************************************/
/* Bus trace DTS(5bit) status					*/
/************************************************/

#define 	BUS_TRACE_IDLE					0x00
#define 	BUS_TRACE_READ_BYTE_START		0x01
#define 	BUS_TRACE_READ_HALF_START		0x03
#define 	BUS_TRACE_READ_HALF_COMPR0		0x04
#define 	BUS_TRACE_READ_HALF_CONT		0x05
#define 	BUS_TRACE_READ_HALF_COMPR1		0x06
#define 	BUS_TRACE_READ_WORD_START		0x07

#define 	BUS_TRACE_DMA_BYTE_START_WR 	0x08
#define 	BUS_TRACE_DMA_BYTE_START_RD 	0x09
#define 	BUS_TRACE_DMA_HALF_START_WR 	0x0a
#define 	BUS_TRACE_DMA_HALF_START_RD 	0x0b
#define 	BUS_TRACE_DMA_COMPR_MATCH		0x0c
#define 	BUS_TRACE_DMA_CONT				0x0d
#define 	BUS_TRACE_DMA_WORD_START_WR 	0x0e
#define 	BUS_TRACE_DMA_WORD_START_RD 	0x0f

#define 	BUS_TRACE_DATA_BYTE_START_WR	0x10
#define 	BUS_TRACE_DATA_BYTE_START_RD	0x11
#define 	BUS_TRACE_DATA_HALF_START_WR	0x12
#define 	BUS_TRACE_DATA_HALF_START_RD	0x13
#define 	BUS_TRACE_DATA_COMPR_MATCH		0x14
#define 	BUS_TRACE_DATA_CONT 			0x15
#define 	BUS_TRACE_DATA_WORD_START_WR	0x16
#define 	BUS_TRACE_DATA_WORD_START_RD	0x17

#define 	BUS_TRACE_DMA_COMPR0			0x18
#define 	BUS_TRACE_DMA_COMPR1			0x19
#define 	BUS_TRACE_DATA_COMPR0			0x1a
#define 	BUS_TRACE_DATA_COMPR1			0x1b

#define 	BUS_TRACE_INST_COMPR_MATCH		0x1c
#define 	BUS_TRACE_INST_CONT 			0x1d
#define 	BUS_TRACE_INST_WORD_START		0x1f

#define 	INSN_CALL_RB	 				0x0600
#define 	INSN_CALL_RB_D	 				0x0700
#define 	INSN_CALL_IMM	 				0x1c00
#define 	INSN_CALL_IMM_D  				0x1d00
#define 	INSN_EXT		 				0xc000
	
/* >>>>> add iruma m.takeishi '03.12.04 */
/************************************************/
/* error status code for MON33 KeyBreak 	*/
/************************************************/
#define 	C33_MONSTATUS_OK				0x0
#define 	C33_MONSTATUS_NG				0x1
/* <<<<< add iruma m.takeishi '03.12.04 */

/* ******* function definition ******** */ 
int c33_IcdSendMessage (int iID, int iParam1, int iParam2, int iParam3, int iParam4, void *pvParam5, int iParam6, int iParam7);
int c33_IcdRecvMessage (int iID, int iParam1, int iParam2, int iParam3, int iParam4, void *pvParam5);
void c33_IcdFlashSet(int iParamNum, unsigned long ulFlashStartAddr, unsigned long ulFlashEndAddr, unsigned long ulEraseRoutineAddr, unsigned long ulWriteRoutineAddr);
/* void c33_IcdFlashErase(int iParamNum, int iCtrlReg, unsigned long ulStartBlock, unsigned long ulEndBlock);	*/	  /* change T.Tazaki 2006/05/16 */
void c33_IcdFlashErase(int iParamNum, int iCtrlReg, unsigned long ulStartBlock, unsigned long ulEndBlock,int iTimer); /* change T.Tazaki 2006/05/16 */
void c33_IcdCall(unsigned long ulStartAddr, unsigned long ulArg1,unsigned long ulArg2, unsigned long ulArg3);
void c33_IcdClockMode(int iClockFunction);

void c33_IcdTraceFile(int iParamNum, char* pFilename, unsigned long ulStartCycle, unsigned long ulEndCycle);
void c33_IcdTraceSearch(int iParamNum, unsigned long ulSearchAddr, unsigned long ulPreLine, unsigned long ulPostLine);
void c33_IcdTraceDisplay(int iParamNum, unsigned long ulStartCycle, unsigned long ulEndCycle); /* add T.Tazaki 2003/07/08 */
void c33_IcdIceCommand(int iParamNum, char* pFilename);
void c33_IcdReadMemory(int iParamNum,unsigned long ulStartAddr,unsigned long ulEndAddr);

void c33_IcdDebug(int iParam1, int iParam2, int iParam3, int iParam4);
void c33_UsbLog(int iParam1, int iParam2, char *pLogFileName);

void c33_IcdTraceMode(int iParamNum, int iTrcMode, int iCounterMode, int iTrg1, int iTrg2, int iBrk, int iMeasure);
void c33_IcdVer3FlashErase(int iParamNum, int iEraseObject);
void c33_IcdVer3FlashProgLoad(int iParamNum, char* pFilename, unsigned long ulEraseEntryAddress,
						 unsigned long ulWriteEntryAddress, char *pComment );
void c33_IcdVer3FlashDataLoad(int iParamNum, char* pFilename, unsigned long ulEraseStartBlock,
						 unsigned long ulEraseEndBlock, unsigned long ulEraseParam, char *pComment );
void c33_IcdVer3FlashDataCopy(int iParamNum, unsigned long ulSrcAddress, unsigned long ulSize,
						 unsigned long ulEraseStartBlock, unsigned long ulEraseEndBlock,unsigned long ulEraseParam,char *pComment);
void c33_IcdVer3FlashInfoDisplay(int iParamNum);

void c33_IcdVer3DebugBaseSet(int iParamNum,unsigned long ulRomAddr,unsigned long ulRamAddr,unsigned char ucCpuType); /* add T.Tazaki 2003/06/19 */

void c33_IcdVer3AreaBreak(int iParamNum, char *pMode, char cBBCUBusSelect, char *pReadWrite, unsigned long ulArea );		 /* add T.Tazaki 2003/07/02 */
void c33_IcdVer3AreaBreak_on(void); 	/* add T.Tazaki 2004/04/19 */
void c33_IcdVer3AreaBreak_off(void);	/* add T.Tazaki 2004/04/19 */
void c33_IcdVer3AreaBreak_usage(void);	/* add T.Tazaki 2003/06/25 */
void c33_IcdVer3BusBreak(int iParamNum, char *pMode, int iBusSelect ,char cBBCUBusSelect, int iTrigBreak,
	 char *pReadWrite, int iInstData, char cAccess, int iInt, int iIntLevel, int iDataComp,
	 unsigned long ulAddress, unsigned long ulAddressMask, unsigned long ulData, unsigned long ulDataMask,
	 unsigned long ulCount,int iCntClrSave );
void c33_IcdVer3BusBreak_on(void);		/* add T.Tazaki 2004/04/19 */
void c33_IcdVer3BusBreak_off(void); 	/* add T.Tazaki 2004/04/19 */
void c33_IcdVer3BusBreak_usage(void);	/* add T.Tazaki 2003/06/25 */
void c33_IcdVer3BusTrace(int iParamNum, int iTraceMode,int iBusSelect ,char cBBCUBusSelect, char *pReadWrite,		 /* add T.Tazaki 2003/07/02 */
	 int iInstData, char cAccess, int iBusOutput, int iAddrBusSize, int iCompress, int iUserInput );
void c33_IcdVer3BusTrace_on(void);		/* add T.Tazaki 2004/04/19 */
void c33_IcdVer3BusTrace_off(void); 	/* add T.Tazaki 2004/09/29 */
void c33_IcdVer3BusTrace_usage(void);	/* add T.Tazaki 2003/06/25 */
void c33_IcdVer3TraceFile(int iParamNum, int iMode, char* pFilename, unsigned long ulStartCycle, unsigned long ulEndCycle);    /* add T.Tazaki 2003/06/25 */
void c33_IcdVer3TraceDisplay(int iParamNum, int iMode, unsigned long ulStartCycle, unsigned long ulEndCycle); /* add T.Tazaki 2003/06/25 */


void c33_AddCRLF(char* cTraceStr);
unsigned long c33_ttbr_reg_get(void);
void c33_Default_reg_set(void);
void c33_Default_special_reg_set(int iType);
void c33_pc_reg_set(unsigned long ulPC);
unsigned long c33_pc_reg_get(void);
int  c33_IcdCheckCurrentInst(void);
int  c33_IcdCheckC_asm_call(void);		/* add T.Tazaki 2004/08/25 */

int c33icd_ParlOpen(void);
int c33icd_ParlClose(void);
int c33icd_ParallelSendData( unsigned char* pMyAddr, unsigned long ulLen,int iUnit );
void c33icd_ParlClrBuff(void);
void c33icd_WindowUpdate(void);

void c33_Dclock(int iParamNum, int iDclock);		 /* add T.Tazaki 2005/05/24 */
void c33_IcdOscWait(int iParamNum, int iWaitCount);  /* add T.Tazaki 2005/06/08 */
void c33_IcdVer3CacheHit( void );					 /* add T.Tazaki 2005/06/23 */
void c33_LogianaMode(int iParamNum, char *cMode, int iUser_in); /* add T.Tazaki 2005/09/22 */

void c33_Cmd_TargetReset(void);						/* for ICDV6 add T.Tazaki 2007/07/03 */
void c33_IcdVer6TimerBreak(int iParamNum, unsigned long ulTimer);/* for ICDV6 add T.Tazaki 2007/07/03 */

#endif
