/* Tcl/Tk command definitions for Insight - Registers
   Copyright 2001 Free Software Foundation, Inc.

   This file is part of GDB.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include "defs.h"
#include "frame.h"
#include "value.h"
#include "regcache.h"

#include <tcl.h>
#include "gdbtk.h"
#include "gdbtk-cmds.h"

/* add T.Tazaki 2004/04/27 >>> */
#include "c33common.h"
#include "Sim33.h"
extern struct c33_target_info c33TargetInfo; /* target information */
/* add T.Tazaki 2004/04/27 <<< */

 extern int g_reg_update_now;		/* add T.Tazaki 2004/06/03 */

/* This contains the previous values of the registers, since the last call to
   gdb_changed_register_list.  */

static char *old_regs;

static int gdb_changed_register_list (ClientData, Tcl_Interp *, int,
				      Tcl_Obj * CONST[]);
static int gdb_fetch_registers (ClientData, Tcl_Interp *, int,
				Tcl_Obj * CONST[]);
static int gdb_regnames (ClientData, Tcl_Interp *, int, Tcl_Obj * CONST[]);
static int get_pc_register (ClientData, Tcl_Interp *, int, Tcl_Obj * CONST[]);
static int gdb_check_sv_flag (ClientData, Tcl_Interp *, int, Tcl_Obj * CONST[]);/* add T.Tazaki 2004/04/27 for ADV */
static void get_register (int, void *);
static void get_register_name (int, void *);
static void get_register_name_adv (int, void *);		/* add T.Tazaki 2004/04/27 for ADV */
static int map_arg_registers (int, Tcl_Obj * CONST[],
			      void (*)(int, void *), void *);
static void register_changed_p (int, void *);
static void setup_architecture_data (void);

int
Gdbtk_Register_Init (Tcl_Interp *interp)
{
  Tcl_CreateObjCommand (interp, "gdb_changed_register_list", gdbtk_call_wrapper,
			gdb_changed_register_list, NULL);
  Tcl_CreateObjCommand (interp, "gdb_fetch_registers", gdbtk_call_wrapper,
			gdb_fetch_registers, NULL);
  Tcl_CreateObjCommand (interp, "gdb_regnames", gdbtk_call_wrapper, gdb_regnames,
			NULL);
  Tcl_CreateObjCommand (interp, "gdb_pc_reg", gdbtk_call_wrapper, get_pc_register,
			NULL);
#ifdef EPSON        /* add T.Tazaki 2004/04/27  check %psr : SV flag chenge */
  Tcl_CreateObjCommand (interp, "gdb_check_sv_flag", gdbtk_call_wrapper, gdb_check_sv_flag,
			NULL);
#endif

  /* Register/initialize any architecture specific data */
  setup_architecture_data ();
  register_gdbarch_swap (&old_regs, sizeof (old_regs), NULL);
  register_gdbarch_swap (NULL, 0, setup_architecture_data);

  return TCL_OK;
}

/* This implements the tcl command "gdb_changed_register_list"
 * It takes a list of registers, and returns a list of
 * the registers on that list that have changed since the last
 * time the proc was called.
 *
 * Tcl Arguments:
 *    A list of registers.
 * Tcl Result:
 *    A list of changed registers.
 */
static int
gdb_changed_register_list (clientData, interp, objc, objv)
     ClientData clientData;
     Tcl_Interp *interp;
     int objc;
     Tcl_Obj *CONST objv[];
{
  objc--;
  objv++;

  return map_arg_registers (objc, objv, register_changed_p, NULL);
}

/* This implements the tcl command gdb_fetch_registers
 * Pass it a list of register names, and it will
 * return their values as a list.
 *
 * Tcl Arguments:
 *    format: The format string for printing the values
 *    args: the registers to look for
 * Tcl Result:
 *    A list of their values.
 */
static int
gdb_fetch_registers (clientData, interp, objc, objv)
     ClientData clientData;
     Tcl_Interp *interp;
     int objc;
     Tcl_Obj *CONST objv[];
{
  int format, result;
	/* Initialize Reg window %psr bit display   add T.Tazaki 2005/05/19 >>> */
	unsigned long ulPSR;
	int iChange = 0;
    int iResult = C33_NO_ERR;
	char szPSR1[30];
	char szPSR2[30];
	char szPSR3[30];
	char szPSR4[30];
	char szPSR5[30];
	char szPSR6[30];
	char szPSR7[30];
	char szPSR8[30];
	int iHE, iRM, iLM, iPM, iRC, iSW, iOC, iSE, iLC, iHC, iS, iDE, iME, iSV, iIL;
	int iMO, iDS, iIE, iC, iV, iZ, iN;
	/* Initialize Reg window %psr bit display   add T.Tazaki 2005/05/19 <<< */

  if (objc < 2)
    {
      Tcl_WrongNumArgs (interp, 1, objv, "format ?register1 register2 ...?");
      return TCL_ERROR;
    }
  objc -= 2;
  objv++;
  format = *(Tcl_GetStringFromObj (objv[0], NULL));
  objv++;

  if (objc != 1)
    result_ptr->flags |= GDBTK_MAKES_LIST;    /* Output the results as a list */
  result = map_arg_registers (objc, objv, get_register, (void *) format);
  if (objc != 1)
    result_ptr->flags &= ~GDBTK_MAKES_LIST;

	/* Initialize Reg window %psr bit display   add T.Tazaki 2005/05/19 >>> */
	
    if ( c33TargetInfo.iIcd_open != 0 )
    {
		ulPSR = read_register (PSR_REGNUM);
	}
	else if ( c33TargetInfo.iSim_open != 0 )
	{
		ulPSR = gdbsim_fetch_register( PSR );
    }

	if( ulPSR & 0x80000000 ) { iHE = 1; } else { iHE = 0; }
	if( ulPSR & 0x40000000 ) { iRM = 1; } else { iRM = 0; }
	if( ulPSR & 0x20000000 ) { iLM = 1; } else { iLM = 0; }
	if( ulPSR & 0x10000000 ) { iPM = 1; } else { iPM = 0; }
	iRC = ( ulPSR & 0x0f000000 ) >> 24;
	if( ulPSR & 0x00400000 ) { iSW = 1; } else { iSW = 0; }
	if( ulPSR & 0x00200000 ) { iOC = 1; } else { iOC = 0; }
	if( ulPSR & 0x00100000 ) { iSE = 1; } else { iSE = 0; }
	if( ulPSR & 0x00020000 ) { iLC = 1; } else { iLC = 0; }
	if( ulPSR & 0x00010000 ) { iHC = 1; } else { iHC = 0; }
	if( ulPSR & 0x00008000 ) { iS  = 1; } else { iS  = 0; }
	if( ulPSR & 0x00004000 ) { iDE = 1; } else { iDE = 0; }
	if( ulPSR & 0x00002000 ) { iME = 1; } else { iME = 0; }
	if( ulPSR & 0x00001000 ) { iSV = 1; } else { iSV = 0; }
	iIL = ( ulPSR & 0x00000f00 ) >> 8;
	if( ulPSR & 0x00000080 ) { iMO = 1; } else { iMO = 0; }
	if( ulPSR & 0x00000040 ) { iDS = 1; } else { iDS = 0; }
	if( ulPSR & 0x00000010 ) { iIE = 1; } else { iIE = 0; }
	if( ulPSR & 0x00000008 ) { iC  = 1; } else { iC  = 0; }
	if( ulPSR & 0x00000004 ) { iV  = 1; } else { iV  = 0; }
	if( ulPSR & 0x00000002 ) { iZ  = 1; } else { iZ  = 0; }
	if( ulPSR & 0x00000001 ) { iN  = 1; } else { iN  = 0; }
	
	if( c33TargetInfo.ucAdvanced == C33_PRODUCT_PE )
	{
		sprintf( szPSR1,"            IE:%d", iIE );	/* PE */
	}
	else
	{
		sprintf( szPSR1,"  MO:%d DS:%d IE:%d", iMO,iDS,iIE );	/* STD,ADV */
	}
	sprintf( szPSR2,"C:%d V:%d Z:%d N:%d", iC,iV,iZ,iN );
	sprintf( szPSR3,"   IL:%2d         ", iIL );
	sprintf( szPSR4,"  HE:%d RM:%d LM:%d", iHE,iRM,iLM );
	sprintf( szPSR5,"PM:%d RC:%2d SW:%d ", iPM,iRC,iSW );
	sprintf( szPSR6,"  OC:%d SE:%d LC:%d", iOC,iSE,iLC );
	sprintf( szPSR7,"HC:%d S:%d DE:%d   ", iHC,iS, iDE );
	sprintf( szPSR8,"  ME:%d SV:%d     ",  iME,iSV );

	/* global c33_XXXX is used by regwin.itb : RegWin::update {}. */
	Tcl_SetVar( interp,"c33_MO_DS_IE",szPSR1,TCL_GLOBAL_ONLY );		
	Tcl_SetVar( interp,"c33_C_V_Z_N" ,szPSR2,TCL_GLOBAL_ONLY );		
	Tcl_SetVar( interp,"c33_IL",      szPSR3,TCL_GLOBAL_ONLY );		
	Tcl_SetVar( interp,"c33_HE_RM_LM",szPSR4,TCL_GLOBAL_ONLY );		
	Tcl_SetVar( interp,"c33_PM_RC_SW",szPSR5,TCL_GLOBAL_ONLY );		
	Tcl_SetVar( interp,"c33_OC_SE_LC",szPSR6,TCL_GLOBAL_ONLY );		
	Tcl_SetVar( interp,"c33_HC_S_DE", szPSR7,TCL_GLOBAL_ONLY );		
	Tcl_SetVar( interp,"c33_ME_SV",   szPSR8,TCL_GLOBAL_ONLY );		

	/* Initialize Reg window %psr bit display   add T.Tazaki 2005/05/19 <<< */

  return result;
}

/* This implements the TCL command `gdb_regnames'.  Its syntax is:

   gdb_regnames [-numbers] [REGNUM ...]

   Return a list containing the names of the registers whose numbers
   are given by REGNUM ... .  If no register numbers are given, return
   all the registers' names.

   Note that some processors have gaps in the register numberings:
   even if there is no register numbered N, there may still be a
   register numbered N+1.  So if you call gdb_regnames with no
   arguments, you can't assume that the N'th element of the result is
   register number N.

   Given the -numbers option, gdb_regnames returns, not a list of names,
   but a list of pairs {NAME NUMBER}, where NAME is the register name,
   and NUMBER is its number.  */
static int
gdb_regnames (clientData, interp, objc, objv)
     ClientData clientData;
     Tcl_Interp *interp;
     int objc;
     Tcl_Obj *CONST objv[];
{
  int numbers = 0;

  objc--;
  objv++;

  if (objc >= 1)
    {
      char *s = Tcl_GetStringFromObj (objv[0], NULL);
      if (STREQ (s, "-numbers"))
	{
	  numbers = 1;
	  objc--;
	  objv++;
	}
    }

/* change T.Tazaki 2004/04/27 >>> */
  if( c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV ) /* Parameter file is advanced mode ? */
  {
	  return map_arg_registers (objc, objv, get_register_name_adv, &numbers);
  }
  else
  {
	  return map_arg_registers (objc, objv, get_register_name, &numbers);
  }
/* change T.Tazaki 2004/04/27 <<< */
}

/* This implements the tcl command get_pc_reg
 * It returns the value of the PC register
 *
 * Tcl Arguments:
 *    None
 * Tcl Result:
 *    The value of the pc register.
 */
static int
get_pc_register (clientData, interp, objc, objv)
     ClientData clientData;
     Tcl_Interp *interp;
     int objc;
     Tcl_Obj *CONST objv[];
{
  char *buff;

  xasprintf (&buff, "0x%llx", (long long) read_register (PC_REGNUM));
  Tcl_SetStringObj (result_ptr->obj_ptr, buff, -1);
  free(buff);
  return TCL_OK;
}

#ifdef EPSON        /* add T.Tazaki 2004/04/27  check %psr : sv flag change */
/* This implements the tcl command get_pc_reg
 * It returns the value of the status of changing of %psr : sv flag
 *
 * Tcl Arguments:
 *    None
 * Tcl Result:
 *    status of changing of %psr : sv flag
 *		1 : changed
 *		0 : not changed
 */

/* ICD */
int g_ulOldIcdSVflag = 0;
extern int g_iIcdSVflag;	/* =%psr:SV flag c33icd.c : c33_IcdRecvRegisterReadŐݒ */
extern int g_iNow_OnUSBRead; /* = 1 : Now USB Read 		add T.Tazaki 2004/05/13 */

/* SIM */
int g_iReg_first = 1;
unsigned long g_ulOldSV = 0;

static int
gdb_check_sv_flag (clientData, interp, objc, objv)
     ClientData clientData;
     Tcl_Interp *interp;
     int objc;
     Tcl_Obj *CONST objv[];
{
	unsigned long ulPSR;
	unsigned long ulSV;
	int iChange = 0;
    int iResult = C33_NO_ERR;
	
	if( c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV )
	{
	    if ( c33TargetInfo.iIcd_open != 0 )
	    {
			/* check if Now USB Reading.  add T.Tazaki 2004/05/13 >>> */
			if( g_iNow_OnUSBRead == 1 )
			{
				iChange = 0;	/* same */
			}
			else
			{
			/* check if Now USB Reading.  add T.Tazaki 2004/05/13 <<< */
	            iResult = c33_SendMessage(C33_ID_REG_READ, 0, 0, 0, 0, 0, 0, 0);
	  	        /* Reception of a message. */
	  	        if (iResult == C33_NO_ERR){
	  	            iResult = c33_RecvMessage(C33_ID_REG_READ, 0, 0, 0, 0, 0);
	  	        }
	  	        if (iResult == C33_NO_ERR)  /* add T.Tazaki 2003/08/05 */
	  	        {
					if( g_iReg_first == 1 )
					{
						g_iReg_first = 0;
						g_ulOldIcdSVflag  = g_iIcdSVflag;
					}
					else
					{
						if( g_ulOldIcdSVflag == g_iIcdSVflag )
						{
							iChange = 0;	/* same */
						}
						else
						{
							iChange = 1;	/* diff */
							g_ulOldIcdSVflag  = g_iIcdSVflag;
						}
					}
	  			}
	  		}
		}
		else if ( c33TargetInfo.iSim_open != 0 )
		{
			ulPSR = gdbsim_fetch_register( PSR );
			if( g_iReg_first == 1 )
			{
				g_iReg_first = 0;
				g_ulOldSV = ulPSR & SV_FLAG;
			}
			else
			{
				ulSV = ulPSR & SV_FLAG;
				if( ulSV == g_ulOldSV )
				{
					iChange = 0;	/* same */
				}
				else
				{
					iChange = 1;	/* diff */
					g_ulOldSV = ulSV;
				}
			}
		}
	}

	if( iChange == 1 )
	{
	  Tcl_SetBooleanObj (result_ptr->obj_ptr, 1);
	}
	else
	{
	  Tcl_SetBooleanObj (result_ptr->obj_ptr, 0);
	}
	
  return TCL_OK;
}
#endif

static void
get_register (regnum, fp)
     int regnum;
     void *fp;
{
  struct type *reg_vtype;
  char raw_buffer[MAX_REGISTER_RAW_SIZE];
  char virtual_buffer[MAX_REGISTER_VIRTUAL_SIZE];
  int format = (int) fp;
  int optim;

  if (format == 'N')
    format = 0;

  /* read_relative_register_raw_bytes returns a virtual frame pointer
     (FRAME_FP (selected_frame)) if regnum == FP_REGNUM instead
     of the real contents of the register. To get around this,
     use get_saved_register instead. */
  get_saved_register (raw_buffer, &optim, (CORE_ADDR *) NULL, selected_frame,
		      regnum, (enum lval_type *) NULL);
  if (optim)
    {
      Tcl_ListObjAppendElement (NULL, result_ptr->obj_ptr,
				Tcl_NewStringObj ("Optimized out", -1));
      return;
    }

  /* Convert raw data to virtual format if necessary.  */

  reg_vtype = REGISTER_VIRTUAL_TYPE (regnum);
  if (REGISTER_CONVERTIBLE (regnum))
    {
      REGISTER_CONVERT_TO_VIRTUAL (regnum, reg_vtype,
      				   raw_buffer, virtual_buffer);
    }
  else
    memcpy (virtual_buffer, raw_buffer, REGISTER_VIRTUAL_SIZE (regnum));

  if (format == 'r')
    {
      int j;
      char *ptr, buf[1024];

      strcpy (buf, "0x");
      ptr = buf + 2;
      for (j = 0; j < REGISTER_RAW_SIZE (regnum); j++)
	{
	  register int idx = TARGET_BYTE_ORDER == BIG_ENDIAN ? j
	  : REGISTER_RAW_SIZE (regnum) - 1 - j;
	  sprintf (ptr, "%02x", (unsigned char) raw_buffer[idx]);
	  ptr += 2;
	}
      fputs_filtered (buf, gdb_stdout);
    }
  else
    if ((TYPE_CODE (reg_vtype) == TYPE_CODE_UNION)
        && (strcmp (FIELD_NAME (TYPE_FIELD (reg_vtype, 0)), REGISTER_NAME (regnum)) == 0))
      {
        val_print (FIELD_TYPE (TYPE_FIELD (reg_vtype, 0)), virtual_buffer, 0, 0,
	           gdb_stdout, format, 1, 0, Val_pretty_default);
      }
    else
      val_print (REGISTER_VIRTUAL_TYPE (regnum), virtual_buffer, 0, 0,
	         gdb_stdout, format, 1, 0, Val_pretty_default);

}

static void
get_register_name (regnum, argp)
     int regnum;
     void *argp;
{
  /* Non-zero if the caller wants the register numbers, too.  */
  int numbers = * (int *) argp;
  Tcl_Obj *name = Tcl_NewStringObj (REGISTER_NAME (regnum), -1);
  Tcl_Obj *elt;

  if (numbers)
    {
      /* Build a tuple of the form "{REGNAME NUMBER}", and append it to
	 our result.  */
      Tcl_Obj *array[2];

      array[0] = name;
      array[1] = Tcl_NewIntObj (regnum);
      elt = Tcl_NewListObj (2, array);
    }
  else
    elt = name;

  Tcl_ListObjAppendElement (NULL, result_ptr->obj_ptr, elt);
}

/* add T.Tazaki 2004/04/27 >>> */

static void
get_register_name_adv (regnum, argp)
     int regnum;
     void *argp;
{
  /* Non-zero if the caller wants the register numbers, too.  */
  int numbers = * (int *) argp;
  Tcl_Obj *name;	
  Tcl_Obj *elt;
  unsigned long ulPSR;
  int iResult = C33_NO_ERR;
  /* add T.Tazaki 2004/06/03 >>> */
    char    szbuf[20];
    char    *pszbuf;
    int     iPsrSV_change = 0;
    extern Tcl_Interp *gdbtk_interp;
    
    pszbuf = szbuf;
  /* add T.Tazaki 2004/06/03 <<< */


	if( c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV )
	{
	    if ( c33TargetInfo.iIcd_open != 0 )
	    {
			/* check if Now USB Reading.  add T.Tazaki 2004/05/13 >>> */
			if( g_iNow_OnUSBRead == 1 ) 
			{
			  	if( g_iIcdSVflag == 1 )	/* SV = 1 ? */
			  	{
					/* User Mode */
				  	name = Tcl_NewStringObj (REGISTER_NAME_USP (regnum), -1);
				}
				else
				{
					/* Supervisor Mode */
					name = Tcl_NewStringObj (REGISTER_NAME_SSP (regnum), -1);
				}
			}
			/* check if Now USB Reading.  add T.Tazaki 2004/05/13 <<< */
			else
			{
				/* add T.Tazaki 2004/06/03 >>> */
			    pszbuf = Tcl_GetVar(gdbtk_interp, "c33_change_sv", TCL_GLOBAL_ONLY);   /* get Sv change flage */
			    iPsrSV_change = atoi( pszbuf );
				/* add T.Tazaki 2004/06/03 <<< */

				if( g_reg_update_now == 0 && iPsrSV_change == 0)	/* check now reg window update ?   T.Tazaki 2004/06/03 */
				{
		            iResult = c33_SendMessage(C33_ID_REG_READ, 0, 0, 0, 0, 0, 0, 0);
		  	        /* Reception of a message. */
		  	        if (iResult == C33_NO_ERR)
		  	        {
		  	            iResult = c33_RecvMessage(C33_ID_REG_READ, 0, 0, 0, 0, 0);
		  	        }
				}
	  	        if (iResult == C33_NO_ERR)  
	  	        {
				  	if( g_iIcdSVflag == 1 )	/* SV = 1 ? */
				  	{
						/* User Mode */
					  	name = Tcl_NewStringObj (REGISTER_NAME_USP (regnum), -1);
					}
					else
					{
						/* Supervisor Mode */
						name = Tcl_NewStringObj (REGISTER_NAME_SSP (regnum), -1);
					}
				}
			}
		}
		else if ( c33TargetInfo.iSim_open != 0 )
		{
		  	ulPSR = gdbsim_fetch_register( PSR );
		  	if( ulPSR & SV_FLAG )
		  	{
				/* User Mode */
			  	name = Tcl_NewStringObj (REGISTER_NAME_USP (regnum), -1);
			}
			else
			{
				/* Supervisor Mode */
				name = Tcl_NewStringObj (REGISTER_NAME_SSP (regnum), -1);
			}
		}
	}

  if (numbers)
    {
      /* Build a tuple of the form "{REGNAME NUMBER}", and append it to
	 our result.  */
      Tcl_Obj *array[2];

      array[0] = name;
      array[1] = Tcl_NewIntObj (regnum);
      elt = Tcl_NewListObj (2, array);
    }
  else
    elt = name;

  Tcl_ListObjAppendElement (NULL, result_ptr->obj_ptr, elt);
}
/* add T.Tazaki 2004/04/27 <<< */

/* This is a sort of mapcar function for operations on registers */

static int
map_arg_registers (objc, objv, func, argp)
     int objc;
     Tcl_Obj *CONST objv[];
     void (*func) (int regnum, void *argp);
     void *argp;
{
  int regnum, numregs;

  /* Note that the test for a valid register must include checking the
     REGISTER_NAME because NUM_REGS may be allocated for the union of
     the register sets within a family of related processors.  In this
     case, some entries of REGISTER_NAME will change depending upon
     the particular processor being debugged.  */

  numregs = NUM_REGS + NUM_PSEUDO_REGS;
  
  if (objc == 0)		/* No args, just do all the regs */
    {
      for (regnum = 0;
	   regnum < numregs;
	   regnum++)
	{
	  if (REGISTER_NAME (regnum) == NULL
	      || *(REGISTER_NAME (regnum)) == '\0')
	    continue;
	  
	  func (regnum, argp);
	}
      
      return TCL_OK;
    }

  /* Else, list of register #s, just do listed regs */
  for (; objc > 0; objc--, objv++)
    {
      if (Tcl_GetIntFromObj (NULL, *objv, &regnum) != TCL_OK)
	{
	  result_ptr->flags |= GDBTK_IN_TCL_RESULT;
	  return TCL_ERROR;
	}

      if (regnum >= 0
	  && regnum < numregs
	  && REGISTER_NAME (regnum) != NULL
	  && *REGISTER_NAME (regnum) != '\000')
	func (regnum, argp);
      else
	{
	  Tcl_SetStringObj (result_ptr->obj_ptr, "bad register number", -1);
	  return TCL_ERROR;
	}
    }

  return TCL_OK;
}

static void
register_changed_p (regnum, argp)
     int regnum;
     void *argp;		/* Ignored */
{
  char raw_buffer[MAX_REGISTER_RAW_SIZE];

  if (read_relative_register_raw_bytes (regnum, raw_buffer))
    return;

  if (memcmp (&old_regs[REGISTER_BYTE (regnum)], raw_buffer,
	      REGISTER_RAW_SIZE (regnum)) == 0)
    return;

  /* Found a changed register.  Save new value and return its number. */

  memcpy (&old_regs[REGISTER_BYTE (regnum)], raw_buffer,
	  REGISTER_RAW_SIZE (regnum));

  Tcl_ListObjAppendElement (NULL, result_ptr->obj_ptr, Tcl_NewIntObj (regnum));
}

static void
setup_architecture_data ()
{
  /* don't trust REGISTER_BYTES to be zero. */
  old_regs = xmalloc (REGISTER_BYTES + 1);
  memset (old_regs, 0, REGISTER_BYTES + 1);
}

