/*
 * Created on 2004/09/02
 * Modified on 2005/07/19 by D.Fujimoto modified build() to accept the visitor and
 * 										added class ResourceDeltaVisitor.
 * Modified on 2005/11/11 by D.Fujimoto modified generateXXXFile() to close stream.
 * Modified on 2005/12/02 by D.Fujimoto created generateFile() and refactored abstract  
 * 											invocations of generateXXXFile() methods.
 * Modified on 2006/01/25 by D.Fujimoto modified build() changed wait logic not to
 * 											use Thread.slepp() because of file out of sync
 * 											problems. Also, return value changed to null.
 * Modified on 2006/01/25 by D.Fujimoto modified ResourceDeltaVisitor.visit() not to visit 
 * 											children unless resource is not File.
 * Modified on 2006/03/14 by D.Fujimoto modified generateFile() to match changes to
 * 											AbstractFileGenerator.
 * Modified on 2006/05/16 by D.Fujimoto modified generateCmdFile() for generating command file output.
 * Modified on 2006/07/21 by D.Fujimoto modified generateCmdFile() to create file only if it 
 * 											does not exist. 
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 * Modified on 2007/03/26 by D.Fujimoto modified generateFile() to refresh generated file.
 * Modified on 2007/08/29 by D.Fujimoto added createBuildProblemMarker() and 
 * 											deleteBuildProblemMarkers() to report build errors.
 */
package jp.co.epson.ide.eclipse.gnu33;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.Map;

import jp.co.epson.ide.eclipse.gnu33.build.BuildSetting;
import jp.co.epson.ide.eclipse.gnu33.debug.DebuggerSetting;
import jp.co.epson.ide.eclipse.gnu33.file.AbstractFileGenerator;
import jp.co.epson.ide.eclipse.gnu33.file.CommandFileGenerator;
import jp.co.epson.ide.eclipse.gnu33.file.LinkerScriptFileGenerator;
import jp.co.epson.ide.eclipse.gnu33.file.MakeFileGenerator;
import jp.co.epson.ide.eclipse.gnu33.file.MakeFileGenerator2;
import jp.co.epson.ide.eclipse.gnu33.file.ParameterFileGenerator;
import jp.co.epson.ide.eclipse.gnu33.linkerscript.LinkerScriptSetting;
import jp.co.epson.ide.eclipse.gnu33.others.ProjectAccessor;
import jp.co.epson.ide.eclipse.gnu33.parameter.ParameterSetting;
import jp.co.epson.ide.eclipse.gnu33.ui.RefreshFileInEditorJob;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.resources.IncrementalProjectBuilder;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.ui.actions.WorkspaceModifyOperation;

/**
 * This Builder creates .mak/.lds/.par/.cmd files for a project. 
 * @author D.Fujimoto
 *
 */
public class Gnu33FilesBuilder extends IncrementalProjectBuilder {

	public static final String BUILDER_ID = "gnu33filebuilder";
	
	// wait interval msec 
	private static final int INTERVAL_AFTER_BUILD =
		Gnu33CorePlugin.getDefault().getPreferenceStore().getInt("interval.build");
	
	private static final boolean TWO_PASS = Gnu33CorePlugin.getDefault().getPreferenceStore().getBoolean("perform2pass");
	
	private static final String MSG_FAILED_GEN = ResourceProvider.getResourceString("build.generateFile.failed");
	
	
	/**
	 * Access to the descriptor file.
	 */
	private Gnu33ProjectDescriptorManager descManager;
	
	/**
	 * The non-argument constructor.
	 */
	public Gnu33FilesBuilder() {
		super();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.internal.events.InternalBuilder#build(int, java.util.Map, org.eclipse.core.runtime.IProgressMonitor)
	 */
	protected IProject[] build(int kind, Map args, IProgressMonitor monitor)
			throws CoreException {
	
		
		if (Gnu33CorePlugin.isDebug) {
			System.out.println("GNU33 File Builder------------" + "for "+ getProject().getName());
		}
	
		// make a ProgressMonitor if necessary
		if (monitor == null) {
			monitor = new NullProgressMonitor();
		}
		
		// refresh in case some files may have changed
		getProject().refreshLocal(IResource.DEPTH_INFINITE, null);

		
		// retrieve the descriptor
		descManager = Gnu33ProjectDescriptorManager.getDescriptorManager(getProject());
		
		switch (kind) {
			case IncrementalProjectBuilder.AUTO_BUILD :
				// every build when Build Automatically is on
				// but not when clean
			case IncrementalProjectBuilder.INCREMENTAL_BUILD :
				// Build All, Build Project
				if (Gnu33CorePlugin.isDebug) {
					System.out.println("AUTO or INCREMENTAL");
				}
				IResourceDelta delta = getDelta(getProject());
				if (delta == null) {
					// ǂ̃t@CɂύXȂꍇAnull̂Ƃ
					if (Gnu33CorePlugin.isDebug) {
						System.out.println("NO DELTA");
					}
					performFullBuild(descManager);
				} else {
					// deltaɂrhH
					if (Gnu33CorePlugin.isDebug) {
						System.out.println("DELTA FOUND");
					}

					// rebuild files if project file changes
					// only rebuild when it is needed
					// create new .mak/.lds/.par/.cmd files if the following files have changed,
					// description file(.gnu33project)
					// makefile(projectname.mak)
					// ldsfile(projectname.lds)
					// parfile(projectname.par)
					// cmdfile(projectname.cmd)

					ResourceDeltaVisitor visitor = new ResourceDeltaVisitor(); 
					delta.accept(visitor);
					if (visitor.needsRebuild()) {
						performFullBuild(descManager);
					}

				}

				break;
			case IncrementalProjectBuilder.FULL_BUILD :
				// Clean all, or clean project
				if (Gnu33CorePlugin.isDebug) {
					System.out.println("FULL BUILD");
				}
				performFullBuild(descManager);
				break;


		}

		checkCancel(monitor);

		// dummy wait to prevent clock skew
		long start = System.currentTimeMillis();
		while (System.currentTimeMillis() - start < INTERVAL_AFTER_BUILD) {}
		
		if (Gnu33CorePlugin.isDebug) {
			System.out.println("GNU33 File Builder------------end");
		}

		return null;

	}

	/**
	 * Clean does not delete the files.
	 * Just forgets the build state.
	 */
	protected void clean(IProgressMonitor monitor) throws CoreException {
		super.clean(monitor);
		deleteBuildProblemMarkers();
	}
		
	/**
	 * The Vistor for checking changes made to the project file or
	 * mak/lds/par/cmd files.
	 * 
	 * If any of these files get modified, then we should rebuild them, 
	 * otherwise we would not. 
	 * 
	 * @author D.Fujimoto
	 * @since 1.1.1
	 *
	 */
	private class ResourceDeltaVisitor implements IResourceDeltaVisitor {

		private String suffix = ResourceProvider.getResourceString("common.suffix");
		private String[] genFiles = {
				suffix + ResourceProvider.getResourceString("makefile.extension"),
				suffix + ResourceProvider.getResourceString("linkerscriptfile.extension"),
				suffix + ResourceProvider.getResourceString("commandfile.extension"),
				suffix + ResourceProvider.getResourceString("parameterfile.extension")
		};
		
		// Flag indicating we should rebuild
		private boolean shouldRebuild = false;
		
		public boolean needsRebuild() {
			return shouldRebuild;
		}
		
		/* ( Javadoc)
		 * @see org.eclipse.core.resources.IResourceDeltaVisitor#visit(org.eclipse.core.resources.IResourceDelta)
		 */
		public boolean visit(IResourceDelta delta) throws CoreException {
			IResource resource = delta.getResource();
						
			if (resource != null && resource.getProject().equals(getProject())) {
				switch (delta.getKind()) {
					case IResourceDelta.ADDED :
					case IResourceDelta.REMOVED :
					case IResourceDelta.CHANGED :
						if (isProjectFile(resource) || isGenFile(resource)) {
							shouldRebuild = true;
							if (Gnu33CorePlugin.isDebug) {
								System.out.println(">>" + resource.getName() + " MODIFIED");
							}
						}
						break;
				}
				if (resource.getType() != IResource.FILE) {
					return true;
				}
			
			}
			return false;
		}
		
		/**
		 * Returns true if the resource is the project file
		 * @param res
		 * @return true if the resource is the project file, otherwise false
		 */
		public boolean isProjectFile(IResource res) {
			String name = res.getName();
			
			if (name.equals(Gnu33ProjectDescriptorManager.getDescriptorFileName())) {
				return true;
			}
			return false;
		}
		
		/**
		 * Returns true if the resource is a generated file depending on 
		 * the project file. 
		 * @param res
		 * @return true if the resource is a generated file, otherwise false
		 */
		public boolean isGenFile(IResource res) {
			String name = res.getName();
			
			for (int i = 0; i < genFiles.length; i++) {
				if (name.endsWith(genFiles[i])) {
					return true;
				}
			}
			return false;
		}
	
	}
	
	
	
	/**
	 * Performs a full build for this builder.
	 * Generates .mak/.lds/.par/.cmd files completly from scratch.
	 * @param manager the descriptor manager which has access to the descriptor file
	 * @throws CoreException
	 */
	private void performFullBuild(Gnu33ProjectDescriptorManager manager) throws CoreException {
		deleteBuildProblemMarkers();
		
		// make the .mak file
		generateMakeFile(manager);
		
		// make the .lds file
		generateLdsFile(manager);
		
		// make the .par file
		generateParFile(manager);

		// make the .cmd file (if not present)
		generateCmdFile(manager);

	}
	
	/**
	 * Part where the generator makes the contents, and
	 * writes out the file.
	 * @param generator	handles making contents of the file
	 * @since 1.2.0
	 * 
	 */
	private void generateFile(final AbstractFileGenerator generator) {
		
		try {
			
			generator.setContentsToStream(generator.makeContents());

			WorkspaceModifyOperation op = new WorkspaceModifyOperation() {
				protected void execute(IProgressMonitor monitor) throws CoreException, InvocationTargetException, InterruptedException {
					generator.createFile(monitor);
				}
			};
			op.run(new NullProgressMonitor());
			
			generator.releaseStream();

/* >>>>> ADDED by D.Fujimoto 2007/03/26 refresh file if open in an editor */			
			// refresh the editor
			String fileToOpen = generator.getFileName();
			Job job = new RefreshFileInEditorJob(getProject().getFile(fileToOpen));
			job.setUser(true);
			job.schedule();
/* <<<<< ADDED by D.Fujimoto 2007/03/26 refresh file if open in an editor */
			
		} catch (InvocationTargetException e) {
			Gnu33CorePlugin.log(e);
/* >>>>> ADDED by D.Fujimoto 2007/08/29 display error as marker */
			try {
				createBuildProblemMarker(getProject().getFile(generator.getFileName()), 
						MSG_FAILED_GEN + " " + generator.getFileName());
			} catch (CoreException e1) {
				e1.printStackTrace();
			}
/* <<<<< ADDED by D.Fujimoto 2007/08/29 display error as marker */
		} catch (InterruptedException e) {
			Gnu33CorePlugin.log(e);
		} catch (IOException e) {
			Gnu33CorePlugin.log(e);
		}
	}
	
	
	/**
	 * Creates a problem marker attached to the specified resource
	 * @param resource		the resource that caused the error
	 * @param message		the message for the problem
	 * @throws CoreException
	 * @since 1.4.0 
	 */
	protected void createBuildProblemMarker(final IResource resource, final String message) throws CoreException {
		
		IWorkspaceRunnable runnable = new IWorkspaceRunnable() {

			public void run(IProgressMonitor monitor) throws CoreException {
				IMarker marker = resource.createMarker(IMarker.PROBLEM);
				marker.setAttribute("markerID", BUILDER_ID);
				marker.setAttribute(IMarker.TRANSIENT, true);
				marker.setAttribute(IMarker.MESSAGE, message);
				marker.setAttribute(IMarker.LOCATION, resource.getName());
			}
		};
		
		getProject().getWorkspace().run(runnable, null);
		
	}
	
	/**
	 * Deletes all the problem markers created by createBuildProblemMarker().
	 * @throws CoreException
	 * @since 1.4.0
	 */
	protected void deleteBuildProblemMarkers() throws CoreException {
		
		IWorkspaceRunnable runnable = new IWorkspaceRunnable() {
			public void run(IProgressMonitor monitor) throws CoreException {
				IMarker[] markers = getProject().findMarkers(IMarker.PROBLEM, true, IResource.DEPTH_INFINITE);
				
				for (int i = 0; i < markers.length; i++) {
					String markerID = markers[i].getAttribute("markerID", "");
					if (markerID.equals(BUILDER_ID)) {
						markers[i].delete();
					}
				}
			} 
		};
		getProject().getWorkspace().run(runnable, null);
	}
	
	
	/**
	 * Generates a .mak file into the project from the descriptor file.
	 * @see MakeFileGenerator
	 * @param manager the descriptor manager
	 */
	private void generateMakeFile(Gnu33ProjectDescriptorManager manager) throws CoreException {
		BuildSetting buildSetting = (BuildSetting) manager.getBuildList().get(0);

		MakeFileGenerator generator = null;
		if (TWO_PASS) {
			// 2 pass make (switchable 2pass/1pass)
			generator = new MakeFileGenerator2(getProject(), manager, buildSetting.getMakefileName());
		} else {
			// this is only for old versions (do not use)
			generator = new MakeFileGenerator(getProject(), manager, buildSetting.getMakefileName());
		}
			
		
		generateFile(generator);
		
	}
	
	/**
	 * Generates a .lds file into the project from the descriptor file.
	 * @see LinkerScriptFileGenerator
	 * @param manager the descriptor manager
	 */
	private void generateLdsFile(Gnu33ProjectDescriptorManager manager) throws CoreException {
		LinkerScriptSetting ldsSetting = manager.getLdsSetting();
		
		LinkerScriptFileGenerator generator = new LinkerScriptFileGenerator(getProject(), 
				manager, ldsSetting.getSettingName());
		
		generateFile(generator);
		
	}
	
	/**
	 * Generates a .par file into the project from the descriptor file.
	 * @see ParameterFileGenerator
	 * @param manager the descriptor manager
	 */
	private void generateParFile(Gnu33ProjectDescriptorManager manager) throws CoreException {
		ParameterSetting parSetting = (ParameterSetting) manager.getParameterList().get(0);
		
		ParameterFileGenerator generator = new ParameterFileGenerator(getProject(), 
				manager, parSetting.getName());
		
		generateFile(generator);
		
	}

	/**
	 * Generates a .cmd file into the project from the descriptor file.
	 * @see CommandFileGenerator
	 * @param manager the descriptor manager
	 */
	private void generateCmdFile(Gnu33ProjectDescriptorManager manager) throws CoreException {

		DebuggerSetting gdbSetting = manager.getGDBSetting();

		CommandFileGenerator generator = new CommandFileGenerator(getProject(), 
				manager, gdbSetting.getName());

		// make the .cmd file only if it does not exist
		ProjectAccessor accessor = new ProjectAccessor(getProject());
		if (accessor.fileExistsInProject(
				gdbSetting.getName() + ResourceProvider.getResourceString("commandfile.extension"))) {
			// update baud rate only
			generator.setBaudRateToFile();

			return;
		}
		
		generateFile(generator);
		
		// update baud rate
		generator.setBaudRateToFile();
		
	}

	
	/**
	 * Checks whether the build has been canceled.
	 */
	private void checkCancel(IProgressMonitor monitor) {
		if (monitor != null && monitor.isCanceled())
			throw new OperationCanceledException();
	}
	
}
