/*
 * Created on 2004/07/30
 * Modified on 2005/06/16 by D.Fujimoto modified generateLinkerscriptSettings() for adding
 * 											new attribute ATTR_LDS_AUTO_UPD.
 * 										modified readLinkerscriptSettings() for adding
 * 											new attribute ATTR_LDS_AUTO_UPD.
 * Modified on 2005/08/05 by D.Fujimoto refactored readFile().
 * Modified on 2005/08/08 by D.Fujimoto deleted reading/writing flag useHighPrecisionLib from 
 * 											readGeneralInfo()/generateGeneralInfo()
 * Modified on 2005/11/09 by D.Fujimoto deleted data pointer selection check in 
 * 											generateLinkerscriptSettings(), readLinkerscriptSettings()
 * Modified on 2005/11/18 by D.Fujimoto overloaded prepareDefaults() for 
 * 											initial vector section settings.
 * Modified on 2006/03/13 by D.Fujimoto added additional commands for DebuggerSetting in
 * 											readGDBSettings() and generateGDBSettings().
 * Modified on 2006/03/17 by D.Fujimoto modified readGDBSettings() use vector flag always true
 * Modified on 2006/03/22 by D.Fujimoto modified readLinkerScriptSettings() special treatments for 
 * 											default sections.
 * Modified on 2006/03/28 by D.Fujimoto added environment ops to generateBuildSettings() and
 * 											readBuildSettings().
 * Modified on 2006/05/16 by D.Fujimoto added generateDebuggerCommand() for project creation.
 * Modified on 2006/07/21 by D.Fujimoto deleted generateDebuggerCommand().
 * Modified on 2006/07/25 by D.Fujimoto modified readGDBSettings() and generateGDBSettings()
 * 											removed raw commands read/write.
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 */
package jp.co.epson.ide.eclipse.gnu33;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import jp.co.epson.ide.eclipse.gnu33.build.BuildSetting;
import jp.co.epson.ide.eclipse.gnu33.build.BuildSettingFactory;
import jp.co.epson.ide.eclipse.gnu33.build.EnvVar;
import jp.co.epson.ide.eclipse.gnu33.build.IBuildCategory;
import jp.co.epson.ide.eclipse.gnu33.build.IBuildOption;
import jp.co.epson.ide.eclipse.gnu33.build.IBuildTool;
import jp.co.epson.ide.eclipse.gnu33.build.IEnvVar;
import jp.co.epson.ide.eclipse.gnu33.cpuinfo.CpuSettingsManager;
import jp.co.epson.ide.eclipse.gnu33.debug.DebuggerSetting;
import jp.co.epson.ide.eclipse.gnu33.general.GeneralInfo;
import jp.co.epson.ide.eclipse.gnu33.linkerscript.DataPointer;
import jp.co.epson.ide.eclipse.gnu33.linkerscript.LinkerScriptSetting;
import jp.co.epson.ide.eclipse.gnu33.linkerscript.SectionData;
import jp.co.epson.ide.eclipse.gnu33.parameter.ParameterArea;
import jp.co.epson.ide.eclipse.gnu33.parameter.ParameterSetting;

import org.eclipse.core.filesystem.URIUtil;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.ProcessingInstruction;
import org.w3c.dom.Text;
import org.xml.sax.SAXException;

/**
 * This class has access to the .gnu33project file.
 * Plus it has every information contained in the file .
 * @author D.Fujimoto
 *	
 */
public class Gnu33ProjectDescriptor implements IAdaptable {
	
	public static final String DESCRIPTION_FILE_NAME = ".gnu33project"; //$NON-NLS-1$
	public static final String ENCODING = "UTF-8";
	
	public static final String TAG_ROOT = "gnu33project"; //$NON-NLS-1$
	public static final String TAG_GENERAL = "general"; //$NON-NLS-1$
	public static final String TAG_MAKE = "make"; //$NON-NLS-1$
	public static final String TAG_MAKE_SET = "makeSetting"; //$NON-NLS-1$
	public static final String TAG_MAKE_COMPILER = "compiler"; //$NON-NLS-1$
	public static final String TAG_MAKE_ASSEMBLER = "assembler"; //$NON-NLS-1$
	public static final String TAG_MAKE_LINKER = "linker"; //$NON-NLS-1$
	public static final String TAG_MAKE_OPTION = "option"; //$NON-NLS-1$
	public static final String TAG_ENV_SET = "environments"; //$NON-NLS-1$
	public static final String TAG_ENV_VAR = "envvar"; //$NON-NLS-1$
	public static final String TAG_PAR = "parameterMap"; //$NON-NLS-1$
	public static final String TAG_PAR_SET = "parSetting"; //$NON-NLS-1$
	public static final String TAG_PAR_SET_COMMENT = "parComment"; //$NON-NLS-1$
	public static final String TAG_PAR_AREA = "parArea"; //$NON-NLS-1$
	public static final String TAG_PAR_COMMENT = "comment"; //$NON-NLS-1$
	public static final String TAG_LDS = "linkerscript"; //$NON-NLS-1$
	public static final String TAG_LDS_SEC_LIST = "sections"; //$NON-NLS-1$
	public static final String TAG_LDS_SEC = "section"; //$NON-NLS-1$
	public static final String TAG_LDS_OBJ = "object"; //$NON-NLS-1$
	public static final String TAG_LDS_DPS = "datapointer"; //$NON-NLS-1$
	public static final String TAG_RUN = "runGDB"; //$NON-NLS-1$
	public static final String TAG_RUN_DEBUGGER = "debugger"; //$NON-NLS-1$
	public static final String TAG_RUN_CONNECT = "connection"; //$NON-NLS-1$
	public static final String TAG_RUN_RAW_CMD = "rawCommands"; //$NON-NLS-1$
	
	public static final String ATTR_GEN_CPU = "cpuType"; //$NON-NLS-1$
	public static final String ATTR_GEN_CPU_NAME = "cpuName";	//$NON-NLS-1$
	public static final String ATTR_GEN_MEM = "memoryModel";	 //$NON-NLS-1$
	public static final String ATTR_GEN_CONF = "config";	//$NON-NLS-1$
	public static final String ATTR_GEN_USE_HIGH = "usePrecisionLib"; //$NON-NLS-1$
	public static final String ATTR_MAKE_SET_NAME = "name"; //$NON-NLS-1$
	public static final String ATTR_MAKE_FILE_NAME = "makefile"; //$NON-NLS-1$
	public static final String ATTR_MAKE_2PASS = "is2passBuild";	//$NON-NLS-1$
	public static final String ATTR_MAKE_GOAL = "buildGoal";	//$NON-NLS-1$
	public static final String ATTR_MAKE_TOOL_CMD = "command"; //$NON-NLS-1$
	public static final String ATTR_ENV_NAME = "name"; //$NON-NLS-1$
	public static final String ATTR_ENV_VALUE = "value"; //$NON-NLS-1$
	public static final String ATTR_PAR_SET_NAME = "name"; //$NON-NLS-1$
	public static final String ATTR_PAR_AREA_TYPE = "areaType";  //$NON-NLS-1$
	public static final String ATTR_PAR_START_ADDR = "startAddr"; //$NON-NLS-1$
	public static final String ATTR_PAR_END_ADDR = "endAddr"; //$NON-NLS-1$
	public static final String ATTR_PAR_WAIT_READ = "waitRead"; //$NON-NLS-1$
	public static final String ATTR_PAR_WAIT_WRITE = "waitWrite"; //$NON-NLS-1$
	public static final String ATTR_PAR_ACCESS = "access"; //$NON-NLS-1$
	public static final String ATTR_PAR_USE_BE = "useBE"; //$NON-NLS-1$
	public static final String ATTR_LDS_NAME = "name"; //$NON-NLS-1$
	public static final String ATTR_LDS_SEC_NAME = "sectionName"; //$NON-NLS-1$
	public static final String ATTR_LDS_VMA = "vma"; //$NON-NLS-1$
	public static final String ATTR_LDS_VMA_SEC = "vmaSectionName"; //$NON-NLS-1$
	public static final String ATTR_LDS_LMA_EQ_VMA = "lmaEqualsVma"; //$NON-NLS-1$
	public static final String ATTR_LDS_LMA = "lma"; //$NON-NLS-1$
	public static final String ATTR_LDS_LMA_SEC = "lmaSectionName"; //$NON-NLS-1$
	public static final String ATTR_LDS_SEC_ATTR = "fileAttribute"; //$NON-NLS-1$
	public static final String ATTR_LDS_AUTO_UPD = "autoUpdate"; //$NON-NLS-1$
	public static final String ATTR_LDS_OBJ_FILE = "file"; //$NON-NLS-1$
	public static final String ATTR_LDS_DP_TYPE = "type"; //$NON-NLS-1$
	public static final String ATTR_LDS_DP_ADDR = "address"; //$NON-NLS-1$
	public static final String ATTR_LDS_DP_SELECT = "selected"; //$NON-NLS-1$
	
	public static final String ATTR_RUN_NAME = "name"; //$NON-NLS-1$
	public static final String ATTR_RUN_DEBUGGER_TYPE = "type"; //$NON-NLS-1$
	public static final String ATTR_RUN_VEC_ADDR = "vectorAddr"; //$NON-NLS-1$
	public static final String ATTR_RUN_USE_VEC_ADDR = "useVectorAddr"; //$NON-NLS-1$
	public static final String ATTR_RUN_PORT = "port"; //$NON-NLS-1$
	public static final String ATTR_RUN_PORT_NO = "portNo"; //$NON-NLS-1$
	public static final String ATTR_RUN_PARALLEL = "parallelPort"; //$NON-NLS-1$
	public static final String ATTR_RUN_WAIT = "wait"; //$NON-NLS-1$
	public static final String ATTR_RUN_BAUD_RATE = "baudRate"; //$NON-NLS-1$
	public static final String ATTR_RUN_DBG_RAM_ADDR = "debugRamAddr"; //$NON-NLS-1$


	
	/**
	 * The project associated with the Description file
	 * This is only for temporary use 
	 */
	private IProject project;
	
	/**
	 * Flag to create a new descriptor file or update it.
	 */
	private boolean isCreate;
	
	/**
	 * Contains GeneralInfo from description file.
	 */
	private GeneralInfo generalInfo;
	
	/**
	 * Contains BuildSetting from description file.
	 */
	private List buildSettingsList;
	
	/**
	 * Contains LinkerScriptSettings from description file.
	 */
	private LinkerScriptSetting ldsSetting;
	
	/**
	 * Contains ParameterSettings from description file.
	 */
	private List parSettingsList;
	
	/**
	 * Contains DebuggerSetting from description file.
	 */
	private DebuggerSetting gdbSetting;
	
	/**
	 * The current version stated in the project file 
	 */
	private String version;
	
	/**
	 * Constructor only for internal use.
	 */
	private Gnu33ProjectDescriptor() {
	}
	
	/**
	 * Creates a new Gnu33ProjectDescriptor.
	 * 
	 * @param project - the associated project
	 * @param create - true to create a Description file
	 * @throws CoreException
	 */
	protected Gnu33ProjectDescriptor(IProject project, boolean create) throws CoreException {
		this.project = project;
		isCreate = create;
		
		// See if it's already there
		IProjectDescription desc = project.getDescription();
		IPath descLocation = null;
		URI descURI = desc.getLocationURI();
		if (descURI == null) {
			descLocation = Platform.getLocation().append(project.getFullPath());
		} else {
			descLocation = URIUtil.toPath(descURI);
		}
		
		IStatus status = null;
		
		IPath descPath = descLocation.append(DESCRIPTION_FILE_NAME);
		if (descPath.toFile().exists()) {
			if (create) {
				// going to overwrite the project file
				return;
			}
			
			// read the descriptor
			readFile();
			
		} else if (!create) {
			status = new Status(IStatus.ERROR, Gnu33CorePlugin.getUniqueIdentifier(),
					Gnu33CorePlugin.STATUS_CODE_DEFAULT,
					DESCRIPTION_FILE_NAME + " "
					+ ResourceProvider.getResourceString("descriptor.unexist"), null);
			throw new CoreException(status);
		}
	}
	
	/**
	 * Create a new descriptor to the project.
	 * @throws CoreException
	 */
	public void createNewFile() throws CoreException {
		if (!isCreate) {
			return;
		}
		
		// if it's not there, check and create
		if (isCreate && checkHasNature(project) == true) {
			// create an xml file
			createFile();
		}
	}
	
	/**
	 * Prepare the default values for the description file
	 * based on GeneralInfo.
	 * The files .mak/.lds/.par/.cmd will have the same suffix.
	 * 
	 * @param generalInfo 	the provided general information
	 */
	public void prepareDefaults(GeneralInfo generalInfo) {
		prepareDefaults(generalInfo, null);
	}
	
	/**
	 * Prepare the default values for the description file
	 * based on GeneralInfo.
	 * The files .mak/.lds/.par/.cmd will have the same suffix.
	 * 
	 * @param generalInfo 	the provided general information
	 * @param vector		the vector section information(may be null) 
	 * @since 1.2.0
	 * 
	 * Heavily modified in 1.4.0
	 * The CpuSettingsManager creates the defaults
	 */
	public void prepareDefaults(GeneralInfo generalInfo, SectionData vector) {
		
		this.generalInfo = generalInfo;

		String projName = project.getName();	
		
		// default compiler options
		BuildSetting buildSetting = BuildSetting.createDefaultSetting(projName, generalInfo);
		buildSettingsList = new ArrayList();
		buildSettingsList.add(buildSetting);
		
		// default linker script settings
		ldsSetting = CpuSettingsManager.createDefaultLinkerScriptSetting(projName, vector);
		
		// default parameter settings
		ParameterSetting parSetting = CpuSettingsManager.createDefaultParameterSetting(projName);
		parSettingsList = new ArrayList();
		parSettingsList.add(parSetting);
		
		// default debugger settings(don't need cpu type?)
		gdbSetting = CpuSettingsManager.createDefaultDebuggerSetting(projName);
		
	}
	
	/**
	 * Checks the ProjectDescription 
	 * to make sure it has the Gnu33 project nature
	 * 
	 * @return boolean
	 * @throws CoreException
	 */
	public boolean checkHasNature(IProject project) throws CoreException {
		
		IProjectDescription desc = project.getDescription();
		// add aditional check here
		if (desc.hasNature(Gnu33ProjectNature.NATURE_ID)) {
			return true;
		}
		return false;
		
	}
	
	/**
	 * Create a new Gnu33 Project Descriptor file to the project
	 * If one already exists, modify the contents and overwrites the file  
	 * This will lock the targetted project.
	 */
	public void createFile() throws CoreException {
		project.getWorkspace().run(new IWorkspaceRunnable() {
			
			// internal XML file create operation
			public void run(IProgressMonitor monitor) throws CoreException {
				
				try {
					String xml = createFileContents();
					InputStream s = new ByteArrayInputStream(xml.getBytes(ENCODING));
					IFile newDescFile = getFile();
					
//					ResourceAttributes attr = newDescFile.getResourceAttributes();
//					if (attr != null && attr.isHidden()) {
//						attr.setHidden(false);
//						newDescFile.setResourceAttributes(attr);
//					}
					
					if (newDescFile.exists()) {
						// update the file
						newDescFile.setContents(s, IResource.FORCE, monitor);
					} else {
						// create a new file
						newDescFile.create(s, IResource.FORCE, monitor);
					}
				} catch (UnsupportedEncodingException e) {
					e.printStackTrace();// should never happen
				} catch (CoreException e) {
					Gnu33CorePlugin.log(e);
/* >>>>> ADDED by D.Fujimoto 2007/08/03 throw when create fails */
					throw e;
/* <<<<< ADDED by D.Fujimoto 2007/08/03 throw when create fails */

				}
				
			}
			
		}, project, IWorkspace.AVOID_UPDATE, null);
	}
	
	/**
	 * Create the content of the file as XML
	 * @return The XML contents as a String
	 */
	private String createFileContents() throws UnsupportedEncodingException {
		
		ByteArrayOutputStream s = new ByteArrayOutputStream();
		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory
			.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			Document doc = builder.newDocument();
			
			// file version
			ProcessingInstruction version = doc.createProcessingInstruction(
					"gnu33-eclipse", 
					"version=\"" + Gnu33CorePlugin.getDefault().getVersion() + "\"");
			doc.appendChild(version);
			
			// root element
			Element root = doc.createElement(TAG_ROOT);
			doc.appendChild(root);
			
			// general info
			generateGeneralInfo(doc);
			
			// make settings
			generateBuildSettings(doc);
			
			// parameter map settings
			generateParSettings(doc);
			
			// linkerscipt settings
			generateLinkerscriptSettings(doc);
			
			// run settings
			generateGDBSettings(doc);
			
			// ŌɃt@CƂďo
			TransformerFactory tfactory = TransformerFactory.newInstance();
			Transformer transformer = tfactory.newTransformer();
			transformer.setOutputProperty(OutputKeys.METHOD, "xml"); // xml̏o
			transformer.setOutputProperty(OutputKeys.ENCODING, ENCODING); // EncodingUTF-8
			transformer.setOutputProperty(OutputKeys.INDENT, "yes"); // Cfg
			
			// DOMɕϊhLgw
			DOMSource source = new DOMSource(doc);
			StreamResult outputTarget = new StreamResult(s);
			// XMLc[Resultɏo
			transformer.transform(source, outputTarget);
			
//			  writeToFile(s, "c:\\gnu.xml");
			
		} catch (FactoryConfigurationError e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} catch (TransformerException e) {
			e.printStackTrace();
		}
		
		return s.toString(ENCODING);
		
	}
	
	/**
	 * Generate XML contents for General Info.
	 * 
	 * @param doc XML document
	 */
	protected void generateGeneralInfo(Document doc) {
		
		Element general = doc.createElement(TAG_GENERAL);
		general.setAttribute(ATTR_GEN_CPU, GeneralInfo.parseCPUString(generalInfo.getCpuType()));
		general.setAttribute(ATTR_GEN_CPU_NAME, generalInfo.getCpuName());
		general.setAttribute(ATTR_GEN_MEM, GeneralInfo.parseMemModelString(generalInfo.getMemoryModel()));
		general.setAttribute(ATTR_GEN_CONF, generalInfo.getCurrentConfName());
        general.setAttribute(ATTR_GEN_USE_HIGH, Boolean.toString(generalInfo.isUseHighPrecisionLib()));
		
		doc.getDocumentElement().appendChild(general);
		
	}
	
	/**
	 * Generate XML contents for Compiler settings.
	 * 
	 * @param doc XML document
	 */
	protected void generateBuildSettings(Document doc) {
		

		Element makeTag = doc.createElement(TAG_MAKE);
		
		for (Iterator iter = buildSettingsList.iterator(); iter.hasNext();) {
			BuildSetting setting = (BuildSetting) iter.next();
			
			Element makeSettingTag = doc.createElement(TAG_MAKE_SET);
			
			makeSettingTag.setAttribute(ATTR_MAKE_SET_NAME, setting.getSettingName());
			makeSettingTag.setAttribute(ATTR_MAKE_FILE_NAME, setting.getMakefileName());
			makeSettingTag.setAttribute(ATTR_MAKE_2PASS, Boolean.toString(setting.is2passBuild()));
			makeSettingTag.setAttribute(ATTR_MAKE_GOAL, String.valueOf(setting.getBuildGoal()));
			
			// write compiler settings
			Element compilerTag = doc.createElement(TAG_MAKE_COMPILER);
			IBuildTool toolComp = setting.getCompiler();
			compilerTag.setAttribute(ATTR_MAKE_TOOL_CMD, toolComp.getCommand());
			
			writeToolsOptions(compilerTag, toolComp);

			// write assembler settings
			Element assemblerTag = doc.createElement(TAG_MAKE_ASSEMBLER);
			IBuildTool toolAsm = setting.getAssembler();
			assemblerTag.setAttribute(ATTR_MAKE_TOOL_CMD, toolAsm.getCommand());
			
			writeToolsOptions(assemblerTag, toolAsm);
			
			// write linker settings
			Element linkerTag = doc.createElement(TAG_MAKE_LINKER);
			IBuildTool toolLink = setting.getLinker();
			linkerTag.setAttribute(ATTR_MAKE_TOOL_CMD, toolLink.getCommand());
			
			writeToolsOptions(linkerTag, toolLink);
			
			// environment setting		 
			Element envTag = doc.createElement(TAG_ENV_SET);
			List envVars = setting.getEnvVars();
			for (Iterator it = envVars.iterator(); it.hasNext(); ) {
				Element envElement = doc.createElement(TAG_ENV_VAR);
				
				IEnvVar env = (IEnvVar) it.next();
				envElement.setAttribute(ATTR_ENV_NAME, env.getName());
				envElement.setAttribute(ATTR_ENV_VALUE, env.getValue());

				envTag.appendChild(envElement);
			}
			makeSettingTag.appendChild(envTag);


			makeSettingTag.appendChild(compilerTag);
			makeSettingTag.appendChild(assemblerTag);
			makeSettingTag.appendChild(linkerTag);
			makeTag.appendChild(makeSettingTag);
		}
		
	  
		
		doc.getDocumentElement().appendChild(makeTag);
	}
	
	/**
	 * Writes the option setting for the specified IBuildTool.
	 * @param parent
	 * @param tool
	 */
	private void writeToolsOptions(Element parent, IBuildTool tool) {
		Document doc = parent.getOwnerDocument();
		
		List catList = tool.getOptionCategories();
		for (Iterator iterator = catList.iterator(); iterator.hasNext();) {
			IBuildCategory cat = (IBuildCategory) iterator.next();
			List optList = cat.getOptions();
			for (Iterator it = optList.iterator(); it.hasNext();) {
				IBuildOption option = (IBuildOption) it.next();
				
				Element optTag = doc.createElement(TAG_MAKE_OPTION);
				Attr optName = doc.createAttribute("name");
				Attr optValue = doc.createAttribute("value");
				optName.setNodeValue(option.getId());
				switch (option.getType()) {
					case IBuildOption.TYPE_BOOLEAN :
						optValue.setNodeValue(Boolean.toString(option.getBooleanValue()));
						break;
					case IBuildOption.TYPE_ENUM :
						optValue.setNodeValue(String.valueOf(option.getEnumValue()));
						break;
					case IBuildOption.TYPE_LIST :
						String[] values = option.getListValue();
						String work = new String();
						for (int i = 0; i < values.length; i++) {
							work += values[i] + IBuildOption.LIST_DELIMITER;
						}
						optValue.setNodeValue(work);
						break;
					case IBuildOption.TYPE_STRING :
					case IBuildOption.TYPE_STRING_MULTI :
						optValue.setNodeValue(option.getStringValue());
						break;
				}					 
				
				// add <option> tag
				optTag.setAttributeNode(optName);
				optTag.setAttributeNode(optValue);
				parent.appendChild(optTag);

			}
			
		}
	}
	
	/**
	 * Generate XML contents for GDB settings.
	 * @param doc XML document
	 */
	protected void generateGDBSettings(Document doc) {
		
		Element runGDB = doc.createElement(TAG_RUN);
		runGDB.setAttribute(ATTR_RUN_NAME, gdbSetting.getName());
		
		Element debugger = doc.createElement(TAG_RUN_DEBUGGER);
		debugger.setAttribute(ATTR_RUN_DEBUGGER_TYPE, DebuggerSetting.parseDebuggerTypeString(gdbSetting.getDebuggerType()));
		debugger.setAttribute(ATTR_RUN_VEC_ADDR, Long.toHexString(gdbSetting.getVectorAddress()));
		debugger.setAttribute(ATTR_RUN_USE_VEC_ADDR, Boolean.toString(gdbSetting.isUseVectorAddress()));
		
		Element connect = doc.createElement(TAG_RUN_CONNECT);
		int port = gdbSetting.getConnectPort();
		connect.setAttribute(ATTR_RUN_PORT, DebuggerSetting.parseConnectPortString(port));
		if (port == DebuggerSetting.PORT_COM) {
			// other attributes are only written when port is COM
			connect.setAttribute(ATTR_RUN_PORT_NO, Integer.toString(gdbSetting.getComPortNo()));
			connect.setAttribute(ATTR_RUN_PARALLEL, DebuggerSetting.parseParallelPortString(gdbSetting.getParallelPort()));
			connect.setAttribute(ATTR_RUN_WAIT, Integer.toString(gdbSetting.getPortWait()));
			connect.setAttribute(ATTR_RUN_BAUD_RATE, Integer.toString(gdbSetting.getBaudRate()));
		}

/* >>>>> MODIFIED by D.Fujimoto always write debug RAM address when PE */		
//		if (gdbSetting.getDebugRamAddr() != DebuggerSetting.RAM_ADDR_DEFAULT) {
		if (generalInfo.getCpuType() == GeneralInfo.CPU_PE) {
/* <<<<< MODIFIED by D.Fujimoto write debug RAM address when PE */
			// write out 
			connect.setAttribute(ATTR_RUN_DBG_RAM_ADDR, Long.toHexString(gdbSetting.getDebugRamAddr()));
		}


		debugger.appendChild(connect);
		runGDB.appendChild(debugger);
		
		
		doc.getDocumentElement().appendChild(runGDB);
		
	}
	
	/**
	 * Generate XML contents for Parameter Settings.
	 * @param doc XML Document
	 */
	protected void generateParSettings(Document doc) {
		
		ParameterSetting[] parSettings = (ParameterSetting[]) parSettingsList.toArray(new ParameterSetting[0]);
		
		Element parameterMap = doc.createElement(TAG_PAR);
		for (int j = 0; j < parSettings.length; j++) {
			Element setting = doc.createElement(TAG_PAR_SET);
			setting.setAttribute(ATTR_PAR_SET_NAME, parSettings[j].getName());
			Element settingComment = doc.createElement(TAG_PAR_SET_COMMENT);
			Text textSetComment = doc.createTextNode(parSettings[j].getComment()); 
			settingComment.appendChild(textSetComment);
			setting.appendChild(settingComment);
			
			List areaList = parSettings[j].getAreaList();
			for (int k = 0; k < areaList.size(); k++) {
				ParameterArea parArea = (ParameterArea) areaList.get(k);

				int areaType = parArea.getAreaType();
				Element area = doc.createElement(TAG_PAR_AREA);
				area.setAttribute(ATTR_PAR_AREA_TYPE, parArea.getAreaTypeString());
				area.setAttribute(ATTR_PAR_START_ADDR, Long.toHexString(parArea
						.getStartAddress()));
				area.setAttribute(ATTR_PAR_END_ADDR, Long.toHexString(parArea
						.getEndAddress()));
				
				if (areaType != ParameterArea.AREA_STACK) {
					// stack area attributes are not written to file
					area.setAttribute(ATTR_PAR_WAIT_READ, Integer.toHexString(parArea
							.getWaitCycleRead()));
					area.setAttribute(ATTR_PAR_WAIT_WRITE, Integer.toHexString(parArea
							.getWaitCycleWrite()));
					area.setAttribute(ATTR_PAR_ACCESS, parArea.getAccessTypeString());
					area.setAttribute(ATTR_PAR_USE_BE, Boolean.toString(parArea
							.isUseBigEndian()));
				}
				if (parArea.getComment() != null && !parArea.getComment().equals("")) {
					Element comment = doc.createElement(TAG_PAR_COMMENT);
					Text textComment = doc.createTextNode(parArea.getComment());
					comment.appendChild(textComment);
					area.appendChild(comment);
				}
				setting.appendChild(area);
			}

			parameterMap.appendChild(setting);
		}
		
		doc.getDocumentElement().appendChild(parameterMap);
	   
	}
	
	/**
	 * Generate XML contents for Linkerscript settings.
	 * @param doc XML document
	 */
	protected void generateLinkerscriptSettings(Document doc) {
		
		Element linkerscript = doc.createElement(TAG_LDS);
		linkerscript.setAttribute(ATTR_LDS_NAME, ldsSetting.getSettingName());
		Element sectionList = doc.createElement(TAG_LDS_SEC_LIST);

		SectionData[] sections = (SectionData[]) ldsSetting.getSections().toArray(new SectionData[0]);
		for (int i = 0; i < sections.length; i++) {
			Element section = doc.createElement(TAG_LDS_SEC);
			
			// section name
			section.setAttribute(ATTR_LDS_SEC_NAME, sections[i].getSectionName());
			
			// vma address
			if (sections[i].isVmaIsAddress()) {
				section.setAttribute(ATTR_LDS_VMA, Long.toHexString(sections[i].getVma()));
			} else {
				section.setAttribute(ATTR_LDS_VMA_SEC, sections[i].getVmaSectionName());
			}
			
			// lma equals vma
			section.setAttribute(ATTR_LDS_LMA_EQ_VMA, Boolean.toString(sections[i].isLMAequalsVMA()));
			
			if (sections[i].isLMAequalsVMA() == false) {
				// lma address or section name
				if (sections[i].isLmaIsAddress()) {
					section.setAttribute(ATTR_LDS_LMA, Long.toHexString(sections[i].getLma()));
				} else {
					section.setAttribute(ATTR_LDS_LMA_SEC, sections[i].getLmaSectionName());
				}	 
			}
			
			// attribute
			section.setAttribute(ATTR_LDS_SEC_ATTR, sections[i].getAttribute());
			
			// autoUpdate
			section.setAttribute(ATTR_LDS_AUTO_UPD, Boolean.toString(sections[i].isAutoUpdate()));
			
			// object file names
			List objList = sections[i].getObjectList();
			for (int j = 0; j < objList.size(); j++) {
				String objFile = (String) objList.get(j);
				Element obj = doc.createElement(TAG_LDS_OBJ);
				obj.setAttribute(ATTR_LDS_OBJ_FILE, objFile);
				
				section.appendChild(obj);
			}
			
			sectionList.appendChild(section);
		}
	   
		
		// data pointers
		Map map = ldsSetting.getDataPointers();
		DataPointer[] dps = (DataPointer[]) map.values().toArray(new DataPointer[0]);
		for (int i = 0; i < dps.length; i++) {
			Element pointer = doc.createElement(TAG_LDS_DPS);
			pointer.setAttribute(ATTR_LDS_DP_TYPE, Integer.toString(dps[i].getType()));
			pointer.setAttribute(ATTR_LDS_DP_ADDR, Long.toHexString(dps[i].getAddress()));
			linkerscript.appendChild(pointer);
		}
		
		linkerscript.appendChild(sectionList);
		doc.getDocumentElement().appendChild(linkerscript);
	}
	
	/**
	 * Reads a Project Descriptor file.
	 * This will lock the targetted file.
	 * 
	 * @since 1.1.1
	 */
	public void readFile() throws CoreException {
		
		// check if there are undone modifying jobs left
		boolean remaining = true;
		int loops = 1000;
		while (remaining) {
			Job[] jobs = Platform.getJobManager().find(project.getName());
			if (jobs.length == 0 || loops-- == 0) {
				remaining = false;
			}
		}

		// read file by obtaining a lock
		project.getWorkspace().run(new IWorkspaceRunnable() {
			public void run(IProgressMonitor monitor) throws CoreException {
				readFileContents();
			}
						
		}, getFile(), IWorkspace.AVOID_UPDATE, null);
	
	}
	
	/**
	 * Reads the xml contents from the Project Descriptor file.
	 * Refactored.
	 * @since 1.1.1
	 */
	private void readFileContents() {
		// get the path where the file must be located
		IPath descPath = project.getLocation().append(DESCRIPTION_FILE_NAME);

		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			Document doc = builder.parse(descPath.toFile());
			// Document doc = builder.parse("c:\\gnu.xml");

			// version
			readVersion(doc);
			
			// general info
			readGeneralInfo(doc);

			// make settings
			readBuildSettings(doc);

			// linkerscript settings
			readLinkerscriptSettings(doc);

			// parameter settings
			readParSettings(doc);

			// run settings
			readGDBSettings(doc);

		} catch (FactoryConfigurationError e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} catch (SAXException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
			Gnu33CorePlugin.log(ResourceProvider.getResourceString("descriptor.ioexception") + DESCRIPTION_FILE_NAME);
		} catch (NumberFormatException e) {
			e.printStackTrace();// should never happen(the project file is invalid)
			Gnu33CorePlugin.log("File " + DESCRIPTION_FILE_NAME + " is invalid!");
		}
	}
	

	/**
	 * Reads the version instruction from the project file
	 * @param	doc	XML document
	 * @since 1.4.0
	 */
	protected void readVersion(Document doc) {
		
		Node node = doc.getFirstChild();
		if (node.getNodeType() == Node.PROCESSING_INSTRUCTION_NODE) {
			String name = node.getNodeName();
			if (name.equals("gnu33-eclipse")) {
				String value = node.getNodeValue();
				version = value.substring("version=\"".length(), value.lastIndexOf("\""));
			}
			
		}

		if (version == null) {
			version = Gnu33CorePlugin.getDefault().getVersion();
		}
	}
	
	
	/**
	 * Read General Info from XML file.
	 * 
	 * @param doc
	 *            XML document
	 */
	protected void readGeneralInfo(Document doc) {
		GeneralInfo generalInfo = new GeneralInfo();
		
		NodeList generalNodes = doc.getElementsByTagName(TAG_GENERAL);
		Element general = (Element) generalNodes.item(0);
		
		String cpuString = general.getAttribute(ATTR_GEN_CPU);
		generalInfo.setCpuType(GeneralInfo.parseCPUInt(cpuString));

		// give a CPU name from the CPU type (for compatibility)
		String cpuName = general.getAttribute(ATTR_GEN_CPU_NAME);
		// create one for compatibility (projects before 1.3.0)
		if (cpuName == null || cpuName.length() == 0) {
			cpuName = CpuSettingsManager.newCpuNameFromType(GeneralInfo.parseCPUInt(cpuString));
		}
		generalInfo.setCpuName(cpuName);

		String memString = general.getAttribute(ATTR_GEN_MEM);
		// set one for compatibility (projects before 1.3.0)
		if (memString == null || memString.length() == 0) {
			memString = GeneralInfo.MEM_32_STR;
		}
		generalInfo.setMemoryModel(GeneralInfo.parseMemModelInt(memString));
		
		String confString = general.getAttribute(ATTR_GEN_CONF);
		if (confString == null || confString.length() == 0) {
			confString = "default";
		}
		generalInfo.setCurrentConfName(confString);
		
        Boolean usePrecision = Boolean.valueOf(general.getAttribute(ATTR_GEN_USE_HIGH));
        generalInfo.setUseHighPrecisionLib(usePrecision.booleanValue());
		
		this.generalInfo = generalInfo;
		
	}
	
	
	/**
	 * Read Compiler settings from XML file.
	 * @param doc XML document
	 */
	protected void readBuildSettings(Document doc) {
		
		List buildSettingsList = new ArrayList();
		
		NodeList makeSettingNodes = doc.getElementsByTagName(TAG_MAKE_SET);
		
		for (int i = 0; i < makeSettingNodes.getLength(); i++) {

			BuildSetting setting = BuildSetting.createDefaultSetting(project.getName(), generalInfo);
			
			Element makeSetting = (Element) makeSettingNodes.item(i);
			
			setting.setSettingName(makeSetting.getAttribute(ATTR_MAKE_SET_NAME));
			setting.setMakefileName(makeSetting.getAttribute(ATTR_MAKE_FILE_NAME));

			// 2pass switch since 1.4.1
			String is2PassStr = makeSetting.getAttribute(ATTR_MAKE_2PASS);		
			if (is2PassStr == null || is2PassStr.equals("")) {
				// for compatiblity (projects that do not have the attribute)
				setting.setIs2passBuild(BuildSettingFactory.is2passDefault());
			} else {
				setting.setIs2passBuild(Boolean.valueOf(is2PassStr).booleanValue());	
			}
			
			// build goal since 1.4.1
			String buildGoal = makeSetting.getAttribute(ATTR_MAKE_GOAL);
			if (buildGoal == null || buildGoal.equals("")) {
				setting.setBuildGoal(BuildSettingFactory.getDefaultGoal());
			} else {
				setting.setBuildGoal(Integer.parseInt(buildGoal));
			}
			
			// read compiler settings
			NodeList compNodes = makeSetting.getElementsByTagName(TAG_MAKE_COMPILER);
			Element compiler = (Element) compNodes.item(0);
			IBuildTool toolComp = setting.getCompiler();
			toolComp.setCommand(compiler.getAttribute(ATTR_MAKE_TOOL_CMD));
			
			readToolsOptions(compiler, toolComp);
			
			// read assembler settings
			NodeList asmNodes = makeSetting.getElementsByTagName(TAG_MAKE_ASSEMBLER);
			Element assembler = (Element) asmNodes.item(0);
			IBuildTool toolAsm = setting.getAssembler();
			toolAsm.setCommand(assembler.getAttribute(ATTR_MAKE_TOOL_CMD));
			
			readToolsOptions(assembler, toolAsm);
			
			// read linker settings
			NodeList linkNodes = makeSetting.getElementsByTagName(TAG_MAKE_LINKER);
			Element linker = (Element) linkNodes.item(0);
			IBuildTool toolLink = setting.getLinker();
			toolLink.setCommand(linker.getAttribute(ATTR_MAKE_TOOL_CMD));
			
			readToolsOptions(linker, toolLink);
			
			// read environment setting
			List envVars = new ArrayList();
			
			NodeList envNodes = makeSetting.getElementsByTagName(TAG_ENV_SET);
			Element envElement = (Element) envNodes.item(0);
			// old projects may not have this element
			if (envElement != null) {
				NodeList envVarNodes = envElement.getElementsByTagName(TAG_ENV_VAR);
				
				for (int j = 0; j < envVarNodes.getLength(); j++) {
					Element envVarElement = (Element) envVarNodes.item(j);
					String name = envVarElement.getAttribute(ATTR_ENV_NAME);
					String value = envVarElement.getAttribute(ATTR_ENV_VALUE);
					
					IEnvVar newEnv = new EnvVar(name, value);
					envVars.add(newEnv);
					
				}
			}
			setting.setEnvVars(envVars);
			
			buildSettingsList.add(setting);
			
		}
		
		
		this.buildSettingsList = buildSettingsList;
		
	}
	
	/**
	 * Reads the options of the specified IBuildTool.
	 * @param toolElement
	 * @param tool
	 */
	private void readToolsOptions(Element toolElement, IBuildTool tool) {
		
		NodeList optNodes = toolElement.getElementsByTagName(TAG_MAKE_OPTION);
		for (int j = 0; j < optNodes.getLength(); j++) {
			Element option = (Element) optNodes.item(j);
			String id = option.getAttribute("name");
			IBuildOption buildOpt = tool.getOption(id);
			if (buildOpt != null) {
				String value = option.getAttribute("value");
				switch (buildOpt.getType()) {
					case IBuildOption.TYPE_BOOLEAN :
						buildOpt.setBooleanValue(Boolean.valueOf(value).booleanValue());
						break;
					case IBuildOption.TYPE_STRING :
					case IBuildOption.TYPE_STRING_MULTI :
						buildOpt.setStringValue(value);
						break;
					case IBuildOption.TYPE_LIST :
						String[] listValue = null;
						if (value == null || value.equals("")) {
							listValue = new String[0];
						} else {
							listValue = value.split(IBuildOption.LIST_DELIMITER);
						}
						buildOpt.setListValue(listValue);
						break;
					case IBuildOption.TYPE_ENUM :
						buildOpt.setEnumValue(Integer.parseInt(value));
						break;
				}
			}
		}
	}

	/**
	 * Read GDB run settings from XML file.
	 * @param doc XML document
	 */
	protected void readGDBSettings(Document doc) {
		DebuggerSetting gdbSetting = new DebuggerSetting();
		
		NodeList runNodes = doc.getElementsByTagName(TAG_RUN);
		Element runGDB = (Element) runNodes.item(0);
		gdbSetting.setName(runGDB.getAttribute(ATTR_RUN_NAME));
		
		Element debugger = (Element) runGDB.getElementsByTagName(TAG_RUN_DEBUGGER).item(0);
		gdbSetting.setDebuggerType(DebuggerSetting.parseDebuggerTypeInt(debugger.getAttribute(ATTR_RUN_DEBUGGER_TYPE))); 
		gdbSetting.setVectorAddress(Long.parseLong(debugger.getAttribute(ATTR_RUN_VEC_ADDR), 16));
		// use vector always true		 
		gdbSetting.setUseVectorAddress(true);
		
		Element connect = (Element) debugger.getElementsByTagName(TAG_RUN_CONNECT).item(0);
		gdbSetting.setConnectPort(DebuggerSetting.parseConnectPortInt(connect.getAttribute(ATTR_RUN_PORT)));
		if (gdbSetting.getConnectPort() == DebuggerSetting.PORT_COM) {
			gdbSetting.setComPortNo(Integer.parseInt(connect.getAttribute(ATTR_RUN_PORT_NO)));
			gdbSetting.setParallelPort(DebuggerSetting.parseParallelPortInt(connect.getAttribute(ATTR_RUN_PARALLEL)));
			gdbSetting.setPortWait(Integer.parseInt(connect.getAttribute(ATTR_RUN_WAIT)));
			gdbSetting.setBaudRate(Integer.parseInt(connect.getAttribute(ATTR_RUN_BAUD_RATE)));

		}
		String ramAddr = connect.getAttribute(ATTR_RUN_DBG_RAM_ADDR);
		if (ramAddr != null && !ramAddr.equals("")) {
			gdbSetting.setDebugRamAddr(Long.parseLong(ramAddr, 16));
		}

		
		this.gdbSetting = gdbSetting;
	}
	
	/**
	 * Read Parameter Settings from XML file.
	 * @param doc XML Document
	 */
	protected void readParSettings(Document doc) {
		List parSettingsList = new ArrayList();
		
		NodeList parSettingNodes = doc.getElementsByTagName(TAG_PAR_SET);
		
		for (int i = 0; i < parSettingNodes.getLength(); i++) {
			ParameterSetting setting = new ParameterSetting();
			
			Element parSetting = (Element) parSettingNodes.item(i);
			setting.setName(parSetting.getAttribute(ATTR_PAR_SET_NAME));
			NodeList parSettingCommentNodes = doc.getElementsByTagName(TAG_PAR_SET_COMMENT);
			Element parSettingComment = (Element) parSettingCommentNodes.item(0);
			if (parSettingComment != null) {
				Node commentNode = parSettingComment.getFirstChild();
				setting.setComment(commentNode == null ? "" : commentNode.getNodeValue());
			}
			
			NodeList parAreaNodes = parSetting.getElementsByTagName(TAG_PAR_AREA);
			for (int j = 0; j < parAreaNodes.getLength(); j++) {
				ParameterArea area = new ParameterArea();
				
				Element parArea = (Element) parAreaNodes.item(j);
				area.setAreaType(ParameterArea.parseAreaTypeInt(parArea.getAttribute(ATTR_PAR_AREA_TYPE)));
				area.setStartAddress(Long.parseLong(parArea.getAttribute(ATTR_PAR_START_ADDR), 16));
				area.setEndAddress(Long.parseLong(parArea.getAttribute(ATTR_PAR_END_ADDR), 16));
				if (!area.isStackArea()) {
					area.setWaitCycleRead(Integer.parseInt(parArea.getAttribute(ATTR_PAR_WAIT_READ), 16));
					area.setWaitCycleWrite(Integer.parseInt(parArea.getAttribute(ATTR_PAR_WAIT_WRITE), 16));
					area.setAccessType(ParameterArea.parseAccessTypeInt(parArea.getAttribute(ATTR_PAR_ACCESS)));
					area.setUseBigEndian(Boolean.valueOf(parArea.getAttribute(ATTR_PAR_USE_BE)).booleanValue());
				}
				NodeList commentNodes = parArea.getElementsByTagName(TAG_PAR_COMMENT);
				Element parComment = (Element) commentNodes.item(0);
				if (parComment != null) {
					Node commentNode = parComment.getFirstChild();
					area.setComment(commentNode == null ? "" : commentNode.getNodeValue());
				}
				
				
				setting.addArea(area);
			}
			parSettingsList.add(setting);
			
		}
		
		this.parSettingsList = parSettingsList;
		
	}
	
	/**
	 * Read Linker script settings from XML file.
	 * @param doc XML document
	 */
	protected void readLinkerscriptSettings(Document doc) {
		
		LinkerScriptSetting ldsSetting = new LinkerScriptSetting();
		
		NodeList rootNode = doc.getElementsByTagName(TAG_LDS); 
		Element ldsRoot = (Element) rootNode.item(0);
		ldsSetting.setSettingName(ldsRoot.getAttribute(ATTR_LDS_NAME));
		
		NodeList sectionNodes = ldsRoot.getElementsByTagName(TAG_LDS_SEC);
		
		List sectionList = ldsSetting.getSections();
		for (int i = 0; i < sectionNodes.getLength(); i++) {
			Element element = (Element) sectionNodes.item(i);
			SectionData section = new SectionData();
			
			// section name
			section.setSectionName(element.getAttribute(ATTR_LDS_SEC_NAME));

			// vma address
			String vma = element.getAttribute(ATTR_LDS_VMA);
			if (vma.equals("")) {
				String vmaSectionName = element.getAttribute(ATTR_LDS_VMA_SEC);
				section.setVmaSectionName(vmaSectionName);
				section.setVmaIsAddress(false);
			} else {
				section.setVma(Long.parseLong(vma, 16));
				section.setVmaIsAddress(true);
			}
			
			// lma equals vma
			section.setLMAequalsVMA(Boolean.valueOf(element.getAttribute(ATTR_LDS_LMA_EQ_VMA)).booleanValue());
			if (section.isLMAequalsVMA() == false) {
				// lma address or section name
				String lma = element.getAttribute(ATTR_LDS_LMA); 
				if (lma.equals("")) {
					section.setLmaSectionName(element.getAttribute(ATTR_LDS_LMA_SEC));
					section.setLmaIsAddress(false);
				} else {
					section.setLma(Long.parseLong(lma, 16));
					section.setLmaIsAddress(true);
				}
			}
			section.createLabels();

			// attribute
			section.setAttribute(element.getAttribute(ATTR_LDS_SEC_ATTR));
			// attribute forced to section name
			if (section.isDefaultSection()) {
				section.setAttribute(element.getAttribute(ATTR_LDS_SEC_NAME));
			}
			
			// autoUpdate flag
			// auto update flag forced
		   	section.setAutoUpdate(section.isDefaultSection());
			
			// object file names
			NodeList objNodes = element.getElementsByTagName(TAG_LDS_OBJ);
			for (int j = 0; j < objNodes.getLength(); j++) {
				Element obj = (Element) objNodes.item(j);
				section.getObjectList().add(obj.getAttribute(ATTR_LDS_OBJ_FILE));
			}
			
			sectionList.add(section);

		}
		
		// data pointers
		Map pointers = ldsSetting.getDataPointers();
		NodeList datapointerNodes = ldsRoot.getElementsByTagName(TAG_LDS_DPS);
		for (int i = 0; i < datapointerNodes.getLength(); i++) {
			Element element = (Element) datapointerNodes.item(i);
			int type = Integer.parseInt(element.getAttribute(ATTR_LDS_DP_TYPE));
			if (type > 0 && type <= pointers.size()) {
				DataPointer dp = (DataPointer) pointers.get(new DataPointer(type).getSymbol());
				dp.setAddress(Long.parseLong(element.getAttribute(ATTR_LDS_DP_ADDR), 16));
			}
			
		}
		
		this.ldsSetting = ldsSetting;
		
	}
	
	
	/**
	 * Returns the GeneralInfo.
	 * @return The GeneralInfo object
	 */
	public GeneralInfo getGeneralInfo() {
		return generalInfo;
	}
	
	/**
	 * Sets the GeneralInfo.
	 * @param generalInfo
	 */
	public void setGeneralInfo(GeneralInfo generalInfo) {
		this.generalInfo = generalInfo;
	}
	
	/**
	 * Returns the CPU type.
	 * @return CPU type
	 */
	public int getCPUType() {
		return generalInfo.getCpuType();
	}
	

	/**
	 * Returns the BuildSetting.
	 * @return List of BuildSettings
	 */
	public List getBuildSettingsList() {
		return buildSettingsList;
	}
	/**
	 * Sets the BuildSetting
	 * @param buildSettingList
	 */
	public void setBuildSettingsList(List buildSettingList) {
		this.buildSettingsList = buildSettingList;
	}
	/**
	 * Returns the ParameterSettings as List.
	 * @return List of ParameterSettings
	 */
	public List getParSettingsList() {
		return parSettingsList;
	}
	/**
	 * Sets the ParameterSetting.
	 * @param parSettingsList
	 */
	public void setParSettingsList(List parSettingsList) {
		this.parSettingsList = parSettingsList;
	}
	
	/**
	 * Returns the DebuggerSetting.
	 * @return The DebuggerSetting object
	 */
	public DebuggerSetting getGDBSetting() {
		return gdbSetting;
	}
	/**
	 * Sets the DebuggerSetting.
	 * @param gdbSetting
	 */
	public void setGDBSetting(DebuggerSetting gdbSetting) {
		this.gdbSetting = gdbSetting;
	}
	/**
	 * Returns the LinkerScriptSetting.
	 * @return The LinkerScriptSetting object
	 */
	public LinkerScriptSetting getLdsSetting() {
		return ldsSetting;
	}
	/**
	 * Sets the LinkerScript.
	 * @param ldsSetting
	 */
	public void setLdsSetting(LinkerScriptSetting ldsSetting) {
		this.ldsSetting = ldsSetting;
	}
	
	/**
	 * Returns the project handle.
	 * @return Project handle of a project
	 */
	public IProject getProject() {
		return project;
	}
	
	public IFile getFile() {
		return project.getFile(DESCRIPTION_FILE_NAME);
	}
	
	public String getVersion() {
		return version;
	}
	
	/**
	 * for debugging
	 * writes out the descriptor file
	 */ 
	private void writeToFile(ByteArrayOutputStream s, String file) {
		try {
			// ڃt@Cɏꍇ͂
			s.writeTo(new FileOutputStream(file));
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	
	
	public static void main(String[] args) {
		//Gnu33ProjectDescriptor desc = new Gnu33ProjectDescriptor();
		//		  desc.prepareDefaults();
		//		  desc.createFileContents(); 
		
		// new Gnu33ProjectDescriptor().readFile();
		
	}
	
	/**
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 * @since 1.1.1
	 */
	public Object getAdapter(Class klass) {
		if (klass.equals(IResource.class)) {
			return getFile();
		}
		return null;
	}
	
}
