/*
 * 쐬: 2005/10/11
 * Modified on 2006/07/26 by D.Fujimoto modified removeOldFiles() not to delete cmd files.
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 */
package jp.co.epson.ide.eclipse.gnu33;

import jp.co.epson.ide.eclipse.gnu33.others.Logger;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.resources.WorkspaceJob;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;

/**
 * Util Job class that is triggered when a project is renamed.
 * Operations are defined in runInWorkspace().
 * 
 * @author D.Fujimoto
 * @since 1.2.0
 */
public class ProjectRenameJob extends WorkspaceJob {

	public static final String JOB_NAME = "project rename";
	
	/**
	 * The project subject to update
	 */
	private IProject project;
	
	/**
	 * The previous project name.
	 */
	private String previousName;
	
	/**
	 * The family name of this job
	 */
	protected String family;
	
	/**
	 * @param projectName	name of the project for this job
	 */
	public ProjectRenameJob(String projectName) {
		super(JOB_NAME);
		this.family = projectName;

	}
	
	public ProjectRenameJob(IProject project, String previousName) {
		
		this(project.getName());
		this.project = project;
		this.previousName = previousName;
		
		setPriority(Job.INTERACTIVE);
		setSystem(false);
		setRule(ResourcesPlugin.getWorkspace().getRoot());
	
	}
	

	/* ( Javadoc)
	 * @see org.eclipse.core.internal.resources.InternalWorkspaceJob#runInWorkspace(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public IStatus runInWorkspace(IProgressMonitor monitor)
			throws CoreException {

		if (monitor == null) {
			monitor = new NullProgressMonitor();
		}
		
		try {
			// do the update
			removeOldFiles(project, previousName);
			monitor.worked(1);
			
			Gnu33ProjectFactory.renameProject(project, previousName);
			monitor.worked(1);
		} catch (CoreException e) {
			e.printStackTrace();
			Logger.log("Failed operations while renaming to " + project.getName());
			return e.getStatus();
		}
		
		return Status.OK_STATUS;
	}

	/**
	 * Removes the mak/lds/par/map files with the old name
	 * if they exist.
	 * 
	 * @param project		the new project
	 * @param previousName	the old project name
	 * @throws CoreException
	 */
	private void removeOldFiles(IProject project, String previousName) throws CoreException {
		
		String suffix = ResourceProvider.getResourceString("common.suffix");
		
		// command files will not be deleted
		String[] delFiles = {
			previousName + suffix + ResourceProvider.getResourceString("makefile.extension"),
			previousName + suffix + ResourceProvider.getResourceString("parameterfile.extension"),
			previousName + suffix + ResourceProvider.getResourceString("linkerscriptfile.extension"),
			previousName + ResourceProvider.getResourceString("mapfile.extension")
		};
		
		
		for (int i = 0; i < delFiles.length; i++) {
			IFile file = project.getFile(delFiles[i]);
			if (file.exists()) {
				file.delete(true, null);
			}
		}
	
	}
	
	/**
	 * 
	 * @see org.eclipse.core.runtime.jobs.Job#belongsTo(java.lang.Object)
	 */
	public boolean belongsTo(Object family) {
		return this.family.equals(family);
	}
	
	/**
	 * The job will be scheduled if the target project exists and is open.
	 * @see org.eclipse.core.runtime.jobs.Job#shouldSchedule()
	 */
	public boolean shouldSchedule() {
		if (!project.exists()) {
			return false;
		} else if (!project.isOpen()) {
			return false;
		}
		return true;
	}
	
	/**
	 * The job will be run if the target project exists.
	 * @see org.eclipse.core.runtime.jobs.Job#shouldRun()
	 */
	public boolean shouldRun() {
		if (!project.exists() || !project.getFile(IProjectDescription.DESCRIPTION_FILE_NAME).exists()) {
			return false;
		} else if (!project.isOpen()) {
			return false;
		}
		return true;
	}
}
