/*
 * Created on 2004/08/17
 *
 */
package jp.co.epson.ide.eclipse.gnu33.build;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * An implementation of IBuildCategory.
 * @author D.Fujimoto
 *
 */
public class BuildCategory implements IBuildCategory {

	/**
	 * The name of this category.
	 */
	private String categoryName;
	
	/**
	 * List of IBuildOptions contained in this category.
	 */
	private List options;
	
	/**
	 * The tool this category belongs to.
	 */
	private IBuildTool tool;
	
	/**
	 * Constructor.
	 */
	public BuildCategory(String categoryName, IBuildTool tool) {
		super();
		options = new ArrayList();
		
		this.categoryName = categoryName;
		this.tool = tool;
		tool.addOptionCategory(this);
		
	}
	

	public String getCategoryName() {
		return categoryName;
	}
	
	public String getCategoryID() {
		return tool.getToolID() + "." + getCategoryName(); //$NON-NLS-1$
	}
	
	public void addOption(IBuildOption option) {
		options.add(option);
	}
	
	public List getOptions() {
		return options;
	}
	
	public Map getOptionMap() {
		Map map = new HashMap();
		
		List list = getOptions();
		for (Iterator iter = list.iterator(); iter.hasNext();) {
			IBuildOption option = (IBuildOption) iter.next();
			map.put(option.getId(), option);
			
		}
		
		return new HashMap(map);
		
	}

	public IBuildTool getTool() {
		return tool;
	}
	 
	public String toString() {
		return categoryName;
	}
}
