/*
 * Created on 2004/08/17
 *
 */
package jp.co.epson.ide.eclipse.gnu33.build;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * The implementation of an IBuildTool.
 * A tool has several categories in itself.
 * @author D.Fujimoto
 *
 */
public class BuildTool implements IBuildTool {

	/**
	 * ID of the tool.
	 */
	private String id;
	
	/**
	 * Command name of the tool.
	 */
	private String command;
	/**
	 * Label name for the tool shown in Tree.
	 */
	private String name;
	
	/**
	 * Tool is disabled when true
	 */
	private boolean isDisabled;
	
	/**
	 * The categories contained in this tool
	 */
	List optionCategories;
	
	private String[] defaultOptions = new String[0];
	
	/**
	 * Constructor.
	 */
	public BuildTool(String id, String command) {
		super();
		this.id = id;
		setCommand(command);
		optionCategories = new ArrayList();
	}
   

	public void addOptionCategory(IBuildCategory category) {
		optionCategories.add(category);
	}
	public List getOptionCategories() {
		return optionCategories;
	}
  
	public void setCommand(String command) {
		this.command = command;
	}
	public String getCommand() {
		return command;
	}
	
	public void setName(String name) {
		this.name = name;
	}
	public String getName() {
		return name;
	}
	
	public String getToolID() {
		return id;
	}
	
	public void setDisabled(boolean isDisabled) {
		this.isDisabled = isDisabled;
	}
	public boolean isDisabled() {
		return isDisabled;
	}
	public String[] getDefaultOptions() {
		return defaultOptions;
	}
	public void setDefaultOptions(String[] defaultOptions) {
		this.defaultOptions = defaultOptions;
	}
	
	
	public IBuildOption getOption(String id) {
		
		for (Iterator iter = optionCategories.iterator(); iter.hasNext();) {
			IBuildCategory cat = (IBuildCategory) iter.next();
			List options = cat.getOptions();
			for (Iterator it = options.iterator(); it.hasNext();) {
				IBuildOption option = (IBuildOption) it.next();
				if (option.getId().equals(id)) {
					return option;
				}
			}
			
		}
		// return null if not found
		return null;
	}
	
	public IBuildOption[] getOptions() {
		
		List allOptions = new ArrayList();
		
		for (Iterator iter = optionCategories.iterator(); iter.hasNext();) {
			IBuildCategory cat = (IBuildCategory) iter.next();
			List options = cat.getOptions();
			for (Iterator it = options.iterator(); it.hasNext();) {
				IBuildOption option = (IBuildOption) it.next();
				allOptions.add(option);
			}
			
		}
		
		if (allOptions.size() != 0) {
			return (IBuildOption[]) allOptions.toArray(new IBuildOption[0]);
		}
		return null;
	
	}
	

	public String toString() {
		return id + "=" + command; //$NON-NLS-1$
	}


}
