/*
 * 쐬: 2006/01/23
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 *
 */
package jp.co.epson.ide.eclipse.gnu33.build;

import jp.co.epson.ide.eclipse.gnu33.file.AbstractFileGenerator;
import jp.co.epson.ide.eclipse.gnu33.file.MakeFileGenerator2;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.WorkspaceJob;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;

/**
 * Job that deletes the object file and dependency file that
 * goes with the given source file.
 * 
 * @author D.Fujimoto
 * @since 1.2.0
 */
public class DeleteDependencyFilesJob extends WorkspaceJob {

	public static final String JOB_NAME = "delete dependencies";
	
	private IFile removed;
	
	/**
	 * The family name of this job
	 */
	protected String family;
	
	/**
	 * @param removed	the removed source file
	 */
	public DeleteDependencyFilesJob(IFile removed) {
		super(JOB_NAME);
		this.family = removed.getProject().getName();
		
		this.removed = removed;
		
		setPriority(Job.INTERACTIVE);
		setSystem(false);
		setRule(removed.getProject());
		
	}

	/* ( Javadoc)
	 * @see org.eclipse.core.internal.resources.InternalWorkspaceJob#runInWorkspace(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public IStatus runInWorkspace(IProgressMonitor monitor)
			throws CoreException {
		
		if (monitor == null) {
			monitor = new NullProgressMonitor();
		}

		deleteDepFiles();
		monitor.worked(1);
	
		return Status.OK_STATUS;
	}

	/**
	 * Delete the dependency files.(.o and .d and .ext0)
	 */
	public synchronized void deleteDepFiles() {
		
		// file extensions for candidate files
		String[] delFileExt = {
				AbstractFileGenerator.EXT_OBJ.replaceFirst(".", ""),	// object files
				"d",													// dep files
				MakeFileGenerator2.EXT_2PASS							// ext files
		};
		
		// files to delete
		IFile[] deletes = new IFile[delFileExt.length];
		
		// get the root
		IWorkspaceRoot root = removed.getWorkspace().getRoot();
		
		// get the path and remove the extension
		IPath path = removed.getFullPath();
		
		
		// delete them
		for (int i = 0; i < deletes.length; i++) {
			path = path.removeFileExtension().addFileExtension(delFileExt[i]);
			deletes[i] = root.getFile(path);

			if (deletes[i].exists()) {
				try {
					// not interested if deleting these files fail... 
					deletes[i].delete(true, null);
				} catch (CoreException e) {
					e.printStackTrace();
				}
			}
		}

		
	}
	
	
	/**
	 * 
	 * @see org.eclipse.core.runtime.jobs.Job#belongsTo(java.lang.Object)
	 */
	public boolean belongsTo(Object family) {
		return this.family.equals(family);
	}
	
	/* ( Javadoc)
	 * @see org.eclipse.core.runtime.jobs.Job#shouldRun()
	 */
	public boolean shouldRun() {
		boolean should = true;
		if (!removed.getProject().exists()) {
			should = false;
		} else if (!removed.getProject().isOpen()) {
			should = false;
		}
		return should;
	}
	
}
