/*
 * 쐬: 2006/03/24
 *
 */
package jp.co.epson.ide.eclipse.gnu33.build;

/**
 * Represents an environment variable which will be
 * substituted to a value during a build.
 * 
 * @author D.Fujimoto
 * @since 1.3.0
 *
 */
public class EnvVar implements IEnvVar {

	/**
	 * The name of the variable
	 */
	private String name;
	
	/**
	 * The value of the variable
	 */
	private String value;
	
	/**
	 * Flag indicating this variable is in use.
	 * As of 1.3.0, this field is ignored.
	 */
	private boolean inUse;
	
	/**
	 * Flag indicating this variable is a reserved one.
	 */
	private boolean reserved;
	
	/**
	 * Constructor.
	 * Defaults to inUse = true
	 */
	public EnvVar(String name, String value) {
		this(name, value, true, false);
	}
	
	/**
	 * Constructor.
	 * @param name		name
	 * @param value	value
	 * @param inUse	true if on, otherwise false
	 * @param reserved	true if reserved value
	 */
	public EnvVar(String name, String value, boolean inUse, boolean reserved) {
		this.name = name;
		this.value = value;
		this.inUse = inUse;
		this.reserved = reserved;
	}
	
	/* ( Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 * Returns true if the reference is the same
	 * or the name is the same.
	 */
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		
		if (obj instanceof IEnvVar) {
			IEnvVar other = (IEnvVar) obj;
			return this.name.equals(other.getName());
		}
		
		return false;
	}
	
	/* ( Javadoc)
	 * Compares each other with their names
	 * 
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 * @throws ClassCastException if the argument is not an IEnvVar
	 */
	public int compareTo(Object o) {
		
		IEnvVar other = (IEnvVar) o;
		
		return this.name.compareToIgnoreCase(other.getName());
	}
	
	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.build.ex.IEnvVar#getName()
	 */
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}
	
	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.build.ex.IEnvVar#getValue()
	 */
	public String getValue() {
		return value;
	}
	
	public void setValue(String value) {
		this.value = value;
	}
	
	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.build.ex.IEnvVar#isInUse()
	 */
	public boolean isInUse() {
		return inUse;
	}
	
	public void setInUse(boolean inUse) {
		this.inUse = inUse;
	}

	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.build.IEnvVar#isReserved()
	 */
	public boolean isReserved() {
		return reserved;
	}

}
