/*
 * Created on 2004/10/12
 * Modified on 2005/03/30 by D.Fujimoto deleted unnecessary comments.
 * Modified on 2005/07/08 by D.Fujimoto added field alignment and 
 * 										added method setDefaultAlignment()
 * 										modified method makeContents() for adding alignments.
 * Modified on 2005/07/25 by D.Fujimoto added string replacements in makeContents().
 * Modified on 2005/08/01 by D.Fujimoto modified method makeContents() for attribute output.
 * Modified on 2005/08/05 by D.Fujimoto modified method makeContents() for removing data pointer outputs.
 * Modified on 2005/11/09 by D.Fujimoto modified writeDps() deleted data pointer selection checks.
 * Modified on 2006/02/20 by D.Fujimoto modified makeContents() added lma end label.
 * Modified on 2006/03/14 by .D.Fujimoto modified(refactored) makeContents().
 * Modified on 2006/03/30 by D.Fujimoto modified makeContents() to replace user envs.
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 */
package jp.co.epson.ide.eclipse.gnu33.file;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import jp.co.epson.ide.eclipse.gnu33.Gnu33CorePlugin;
import jp.co.epson.ide.eclipse.gnu33.Gnu33ProjectDescriptorManager;
import jp.co.epson.ide.eclipse.gnu33.ResourceProvider;
import jp.co.epson.ide.eclipse.gnu33.build.BuildSetting;
import jp.co.epson.ide.eclipse.gnu33.general.GeneralInfo;
import jp.co.epson.ide.eclipse.gnu33.linkerscript.DataPointer;
import jp.co.epson.ide.eclipse.gnu33.linkerscript.LinkerScriptSetting;
import jp.co.epson.ide.eclipse.gnu33.linkerscript.SectionData;
import jp.co.epson.ide.eclipse.gnu33.others.UIUtil;
import jp.co.epson.ide.eclipse.gnu33.variables.ToolPathVarResolver;

import org.eclipse.core.resources.IProject;
import org.eclipse.jface.preference.IPreferenceStore;

/**
 * Creates a linker script file(.lds) during a build.
 * @author D.Fujimoto
 *
 */
public class LinkerScriptFileGenerator extends AbstractFileGenerator {

	private GeneralInfo generalInfo;
	private LinkerScriptSetting ldsSetting;
	private BuildSetting buildSetting;
	
	private int alignment; 
	
	private IPreferenceStore store;
	
	// preference to put the label
	private boolean writeEndOfSectionLabelOutside;
	
	// List to save the lma sections for later 
	List loadSections = new ArrayList(); 
	
	/**
	 * Constructor. Get the LinkerScriptSetting from the descriptor file.
	 * @see AbstractFileGenerator
	 */
	public LinkerScriptFileGenerator(IProject project,
			Gnu33ProjectDescriptorManager manager, String fileName) {
		super(project, manager, fileName + EXT_LNK); 
		
		ldsSetting = manager.getLdsSetting();
		generalInfo = manager.getGeneralInfo();
		buildSetting = (BuildSetting) manager.getBuildList().get(0);
		
		setDefaultAlignment();
		
		store = Gnu33CorePlugin.getDefault().getPreferenceStore();
		
		// should we put the label outside? 
		writeEndOfSectionLabelOutside = store.getBoolean("writeEndOfSectionLabelOutside");
	}

	/**
	 * Get the alignment bytes from the prop file.
	 * @since 1.1.1
	 */
	private void setDefaultAlignment() {
		String alignString = ResourceProvider.getResourceString("linkerscript.section.align.default");
		alignment = Integer.parseInt(alignString);
	}

	/** 
	 * Makes the contents of the file using the descriptor file.
	 * @see jp.co.epson.ide.eclipse.gnu33.file.AbstractFileGenerator#makeContents()
	 */
	public String makeContents() {
		
		StringBuffer buf = new StringBuffer();
		
		// file header
		buf.append("/* Linker Script file generated by Gnu33 Plug-in for Eclipse */").append(NEW_LINE);
		
		
		// default headers(format, arch, search dir)
		buf.append(
				"OUTPUT_FORMAT(\"elf32-c33\", \"elf32-c33\", \"elf32-c33\")").append(NEW_LINE);
		buf.append("OUTPUT_ARCH(c33)").append(NEW_LINE);
		buf.append("SEARCH_DIR(.);").append(NEW_LINE);
		buf.append(NEW_LINE);

		buf.append("SECTIONS").append(NEW_LINE);
		buf.append("{").append(NEW_LINE);
		
		// data pointer addresses
		Map dpMap = ldsSetting.getDataPointers();
		buf.append(TAB).append("/* data pointer symbols */").append(NEW_LINE);
		
		DataPointer dp = (DataPointer) dpMap.get(DataPointer.TYPE_D_STR);
		writeDps(buf, dp);
/* >>>>> DELETED by D.Fujimoto 2005/08/05 for data pointer removal */		 
//		  dp = (DataPointer) dpMap.get(DataPointer.TYPE_G_STR);
//		  writeDps(buf, dp);
//		  dp = (DataPointer) dpMap.get(DataPointer.TYPE_Z_STR);
//		  writeDps(buf, dp);
//		  dp = (DataPointer) dpMap.get(DataPointer.TYPE_T_STR);
//		  writeDps(buf, dp);
//		  dp = (DataPointer) dpMap.get(DataPointer.TYPE_S_STR);
//		  writeDps(buf, dp);
/* <<<<< DELETED by D.Fujimoto 2005/08/05 for data pointer removal */		 
		
		buf.append(NEW_LINE);
		
		// location counter
		buf.append(TAB).append("/* location counter */").append(NEW_LINE);
		buf.append(TAB).append(". = 0x0").append(";").append(NEW_LINE);
		buf.append(NEW_LINE);
		
		// sections
		buf.append(TAB).append("/* section information */").append(NEW_LINE);
		List sections = ldsSetting.getSections();
		for (int i = 0; i < sections.size(); i++) {
			SectionData section = (SectionData) sections.get(i);
			
			// section name and vma
			buf.append(TAB).append(section.getSectionName()).append(SPACE);
			if (section.isVmaIsAddress()) {
				buf.append("0x").append(UIUtil.getAddressString(section.getVma()));
			} else {
				String symbol = ldsSetting.getEndSymbol(section.getVmaSectionName());
				buf.append(symbol);
				
			}
			buf.append(" : ");
			
			// lma
			if (!section.isLMAequalsVMA()) {
				buf.append("AT( ");
				if (section.isLmaIsAddress()) {
					buf.append("0x").append(UIUtil.getAddressString(section.getLma()));
				} else {
					String lmaSectionName = section.getLmaSectionName();
					SectionData referencedSection = ldsSetting.getSection(lmaSectionName);
					String symbol = null;
					if (referencedSection.isLMAequalsVMA()) {
						// section will be placed after the referencedSection's vma
						symbol = ldsSetting.getEndSymbol(lmaSectionName);
						buf.append(symbol);
					} else {
						// section will be placed after the referencedSection's lma
						symbol = ldsSetting.getLmaSymbol(lmaSectionName);
						buf.append(symbol);
						buf.append(" + SIZEOF( ").append(lmaSectionName).append(" )");
					}
				}
				buf.append(" )");
			}
			buf.append(NEW_LINE);
			
			// files contained in the section
			buf.append(TAB).append("  {").append(NEW_LINE);
			
			// add alignments for specific sections
			if (section.needsAlign()) {
				buf.append(TAB).append(SPACE).append(SPACE).append(SPACE);
				buf.append(". = ALIGN(" + alignment + ");").append(NEW_LINE);
			}
			
			buf.append(TAB).append(SPACE).append(SPACE).append(SPACE);
			buf.append(section.createLabels()[0]).append(" = . ;").append(NEW_LINE);
			
			List objList = section.getObjectList();
			for (int j = 0; j < objList.size(); j++) {
				String objFile = (String) objList.get(j);
				
				buf.append(TAB).append(SPACE).append(SPACE).append(SPACE);
/* >>>>> ADDED by D.Fujimoto 2005/07/25 tool dir replacements */
				// replace $(TOOL_DIR) string to the real path
				objFile = objFile.replaceAll("\\$\\(" + MakeFileGenerator.MACRO_TOOL_DIR + "\\)", ToolPathVarResolver.getGnuToolsPath().toString());
/* <<<<< ADDED by D.Fujimoto 2005/07/25 tool dir replacements */
/* >>>>> MODIFIED by D.Fujimoto 2005/08/01 attribute output */
//				  buf.append(objFile).append("(").append(section.getAttribute()).append(")");
				buf.append(objFile).append("(").append(section.getAttributeOutputString()).append(")");
				buf.append(NEW_LINE);
/* <<<<< MODIFIED by D.Fujimoto 2005/08/01 attribute output */
			}

/* >>>>> MODIFIED by D.Fujimoto 2007/11/13 write label outside of section range */
//			buf.append(TAB).append(SPACE).append(SPACE).append(SPACE);
//			buf.append(section.createLabels()[1]).append(" = . ;").append(NEW_LINE);
			if (!writeEndOfSectionLabelOutside) {
				buf.append(TAB).append(SPACE).append(SPACE).append(SPACE);
				buf.append(section.createLabels()[1]).append(" = . ;").append(NEW_LINE);
				buf.append(TAB).append("  }").append(NEW_LINE);
			} else {
				buf.append(TAB).append("  }").append(NEW_LINE);
				buf.append(TAB).append(SPACE).append(SPACE);
				buf.append(section.createLabels()[1]).append(" = . ;").append(NEW_LINE);
			}
/* <<<<< MODIFIED by D.Fujimoto 2007/11/13 label may be outside of section range */
			
			// lma symbols will be written later
			if (!section.isLMAequalsVMA()) {
				loadSections.add(section);
			}
			
			buf.append(NEW_LINE);
		}
		
		
		// LMA symbols
		buf.append(TAB).append("/* load address symbols */").append(NEW_LINE);
		
		// get the order of the lma
		Collections.sort(loadSections, new LmaComparator());
		for (Iterator iter = loadSections.iterator(); iter.hasNext();) {
			SectionData sec = (SectionData) iter.next();
			buf.append(TAB).append(sec.createLabels()[2]);
			buf.append(" = LOADADDR( ");
			buf.append(sec.getSectionName());
			buf.append(" )").append(";").append(NEW_LINE);
/* >>>>> ADDED by D.Fujimoto 2006/02/20 lma end label */
			buf.append(TAB).append(sec.createLabels()[3]);
			buf.append(" = " + sec.createLabels()[2] + " + SIZEOF( " + sec.getSectionName() + " )")
				.append(";").append(NEW_LINE);
/* <<<<< ADDED by D.Fujimoto 2006/02/20 lma end label */
		}		 
		
		
		buf.append("}").append(NEW_LINE);
		
/* >>>>> MODIFIED by D.Fujimoto 2006/03/30 for replacing user envs */
//		return new String(buf);
		return replaceEnvVars(new String(buf), buildSetting.getEnvVars());
/* <<<<< MODIFIED by D.Fujimoto 2006/03/30 for replacing user envs */

	}

	/**
	 * Writes the DataPointer address to the given buffer.
	 * Writes it out only when the specified DataPointer is enabled.
	 * @param buf the StringBuffer to write to
	 * @param dp the target DataPointer object
	 */
	private void writeDps(StringBuffer buf, DataPointer dp) {
/* >>>>> MODIFIED by D.Fujimoto 2005/11/09 selected flag deleted */ 	   
//	  	if (dp.isSelected()) {
//			  buf.append(TAB);
//			  buf.append(dp.getSymbol()).append(" = 0x").append(dp.getAddressString()).append(";");
//			  buf.append(NEW_LINE);
//		  }
		buf.append(TAB);
		buf.append(dp.getSymbol()).append(" = 0x").append(dp.getAddressString()).append(";");
		buf.append(NEW_LINE);
	
/* <<<<< MODIFIED by D.Fujimoto 2005/11/09 selected flag deleted */
	}
	
	/**
	 * Comparator for sorting a List containing SectionData 
	 * that are LMAVMA.
	 * The purpose is to sort the list in lma order.
	 * The list should only contain SectionData that are LMAVMA.
	 * 
	 * @author D.Fujimoto
	 *
	 */
	private class LmaComparator implements Comparator {

		static final String SEC1 = "sec1";
		static final String SEC2 = "sec2";

		// List of all sections
		List sections;
		
		// counter for how many times getNearestLmaAddress and getNearestVmaAddress
		// was invoked
		Map counterMap = new HashMap();
		
		/**
		 * Constructor.
		 * Get the all sections List
		 */
		public LmaComparator() {
			sections = LinkerScriptFileGenerator.this.ldsSetting.getSections();
		}
		
		
		/**
		 * Compares SectionData by their address or, the referenced section.
		 * @return -1 when o1 should come before o2 in the list, 
		 * 		1 when o2 should come before o1 in the list, 
		 * 		0 when o1 and o2 are considered equal.
		 * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
		 */
		public int compare(Object o1, Object o2) {
			int result = -1;
			
			SectionData sec1 = (SectionData) o1;
			SectionData sec2 = (SectionData) o2;
			
			// reset counter
			counterMap.put(SEC1, new Integer(0));
			counterMap.put(SEC2, new Integer(0));
			
			long addr1 = -1;
			long addr2 = -1;
			
			if (sec1.isLmaIsAddress()) {
				addr1 = sec1.getLma();
				if (sec2.isLmaIsAddress()) {
					addr2 = sec2.getLma();
					// sec1 and sec2 are both addresses
					result = new Long(addr1).compareTo(new Long(addr2));
				} else {
					// find the nearest lma recursively
					addr2 = getNearestLmaAddress(sec2, SEC2);
					// sec1 is an address and sec2 is a reference
					result = new Long(addr1).compareTo(new Long(addr2));
					if (result == 0) {
						// the one with the real address comes first
						result = -1;
					}
				}
			} else {
				addr1 = getNearestLmaAddress(sec1, SEC1);
				if (sec2.isLmaIsAddress()) {
					addr2 = sec2.getLma();
					// sec1 is a reference and sec2 is an address
					result = new Long(addr1).compareTo(new Long(addr2));
					if (result == 0) {
						// the one with the real address comes first
						result = 1;
					}
				} else {
					addr2 = getNearestLmaAddress(sec2, SEC2);
					// sec1 and 2 are both references
					// so decide the order by how far the address is
					result = new Long(addr1).compareTo(new Long(addr2));
					if (result == 0) {
						Integer count1 = (Integer) counterMap.get(SEC1);
						Integer count2 = (Integer) counterMap.get(SEC2);
						result = count1.compareTo(count2);
					}
				}
			}
			
			return result;
		}
		
		/**
		 * Returns the given section's nearest real lma or vma address.
		 * If this section's lma is not a real address, this method
		 * recursively searches until it finds the real lma or vma address.
		 * <p>
		 * Counts up how many times this method was invoked and stores it in a Map by the
		 * given key
		 * @param section	the SectionData in question
		 * @param key	the key to the counterMap
		 * @return	the address or -1 if it cannot find one
		 */
		private long getNearestLmaAddress(SectionData section, String key) {
			
			// count up
			int count = ((Integer) counterMap.get(key)).intValue();
			count++;
			counterMap.put(key, new Integer(count));
			
			// the section must be in the lma section list
			if (!sections.contains(section)) {
				return -1;
			}
			
			if (section.isLMAequalsVMA()) {
				if (section.isVmaIsAddress()) {
					return section.getVma();
				} else {
					return getNearestVmaAddress(section, key);
				}
			} else if (section.isLmaIsAddress()) {
				// return back its lma address
				return section.getLma();
			} else {
				// if it references another section, find recursively
				String aSectionName = section.getLmaSectionName();
				SectionData aSection = ldsSetting.getSection(aSectionName);
				
				if (aSection != null) {
					return getNearestLmaAddress(aSection, key);
				}
			}
				
			// shouldn't come this far
			return -1;
			
		}
		
		/**
		 * Returns the given section's nearest real vma address.
		 * If this section's vma is not a real address, this method 
		 * recursively searches until it finds the real address.
		 * <p>
		 * Counts up how many times this method was invoked and stores it in a Map by the 
		 * given key.
		 * @param section the SectionData in question
		 * @param key the key to the counterMap
		 * @return the address or -1 if it cannot find
		 */
		private long getNearestVmaAddress(SectionData section, String key) {

			// count up
			int count = ((Integer) counterMap.get(key)).intValue();
			count++;
			counterMap.put(key, new Integer(count));
			
			// the section must be in the list
			if (!sections.contains(section)) {
				return -1;
			}
			 
			if (section.isVmaIsAddress()) {
				// return back its vma address
				return section.getVma();
			} else {
				// if it references another section, find recursiviely 
				String aSectionName = section.getVmaSectionName();
				SectionData aSection = ldsSetting.getSection(aSectionName);

				if (aSection != null) {
					// avoid endless
					if (!section.getSectionName().equals(aSection.getVmaSectionName())) {
						return getNearestVmaAddress(aSection, key);    
					}
				}
			}
			
			// shouldn't come this far
			return -1;

		}
		
	}

}
