/*
 * Created on 2004/08/03
 * Modified on 2005/11/09 by D.Fujimoto selected flag is deprecated.
 *
 */
package jp.co.epson.ide.eclipse.gnu33.linkerscript;


/**
 * This class represents a Data Pointer for a Linker Script.
 * This class is contained in LinkerScriptSetting.
 * 
 * As of version 1.1.1, the data pointers other than D will never be used.
 * As of version 1.2.0 the selected flag is no longer used.
 * 
 * @author D.Fujimoto
 *
 */
public class DataPointer {

	public static final int TYPE_D = 1;
	public static final int TYPE_G = 5;
	public static final int TYPE_Z = 2;
	public static final int TYPE_T = 3;
	public static final int TYPE_S = 4;
	public static final String TYPE_D_STR = "__dp";
	public static final String TYPE_G_STR = "__gdp";
	public static final String TYPE_Z_STR = "__zdp";
	public static final String TYPE_T_STR = "__tdp";
	public static final String TYPE_S_STR = "__sdp";
	
	/**
	 * true if this DataPointer is to use
	 * @deprecated As of 1.2.0, this flag is irrelevant(always true).
	 */
	private boolean selected = true;
	
	/**
	 * The type of this DataPointer
	 */
	private int type;
	
	/**
	 * The address for this DataPointer
	 */
	private long address;
	
	/**
	 * Constructs a DataPointer by type.
	 * @param type One of TYPE_D | TYPE_G | TYPE_Z | TYPE_T | TYPE_S 
	 */
	public DataPointer(int type) {
		super();
		this.type = type;
	}

   
	/**
	 * 
	 * @return the hex representation of the given address
	 */
	public String getAddressString() {
		String retStr = Long.toHexString(address).toUpperCase();
		while (retStr.length() < 8) {
			retStr = "0" + retStr;
		}
		return retStr;
	}
	
	// getters and setters 
	
	public long getAddress() {
		return address;
	}
	public void setAddress(long address) {
		this.address = address;
	}
	
/* >>>>> DELETED by D.Fujimoto 2005/11/09 selected flag deprecated */
//	  public boolean isSelected() {
//		  return selected;
//	  }
//	  public void setSelected(boolean enabled) {
//		  this.selected = enabled;
//	  }
/* <<<<< DELETED by D.Fujimoto 2005/11/09 selected flag deprecated */
	
	/**
	 * Returns the symbol to be written in the linker script file.
	 * @return the symbol String
	 */
	public String getSymbol() {
		String ret = "";
		switch (type) {
			case TYPE_D :
				ret = TYPE_D_STR;
				break;
			case TYPE_G :
				ret = TYPE_G_STR;
				break;
			case TYPE_Z :
				ret = TYPE_Z_STR;
				break;
			case TYPE_T :
				ret = TYPE_T_STR;
				break;
			case TYPE_S :
				ret = TYPE_S_STR;
				break;
				
				
		}
		return ret;
	}
	public int getType() {
		return type;
	}
	public void setType(int type) {
		this.type = type;
	}
	
	

}
