/*
 * Created on 2005/06/16
 *
 */
package jp.co.epson.ide.eclipse.gnu33.linkerscript;


import java.util.List;

import jp.co.epson.ide.eclipse.gnu33.Gnu33ProjectDescriptorManager;
import jp.co.epson.ide.eclipse.gnu33.others.Logger;
import jp.co.epson.ide.eclipse.gnu33.others.ProjectAccessor;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.resources.WorkspaceJob;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;

/**
 * Util class that updates the project file as a Job.
 * 
 * @author D.Fujimoto
 * @since 1.1.1
 *	
 */
public class LinkerScriptUpdater extends WorkspaceJob {

	public static final String JOB_NAME = "linker script updater";
	
	/**
	 * The project subject to update
	 */
	private IProject project;

	/**
	 * The family name of this job
	 */
	protected String family;
	
	/**
	 * @param projectName
	 *			  the project which linkerscript setting needs to be changed
	 */
	public LinkerScriptUpdater(String projectName) {
		super(JOB_NAME);
		this.family = projectName;
		
		project = ResourcesPlugin.getWorkspace().getRoot().getProject(
				projectName);

		setPriority(Job.INTERACTIVE);
		setSystem(true);
		setRule(project);
	}

	/*
	 * ( Javadoc)
	 * 
	 * @see org.eclipse.core.resources.WorkspaceJob#runInWorkspace(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public IStatus runInWorkspace(IProgressMonitor monitor) {

		if (monitor == null) {
			monitor = new NullProgressMonitor();
		}
		try {
			updateLinkerScript(project);
			monitor.worked(1);
		} catch (CoreException e) {
			e.printStackTrace();
			Logger.log("Failed to update " + Gnu33ProjectDescriptorManager.getDescriptorFileName() + " for " + project.getName());
			return e.getStatus();
		}

		return Status.OK_STATUS;
	}

	/**
	 * The main operation on changing the setting. The object files will be
	 * placed before any of the libraries
	 * 
	 * @throws CoreException
	 */
	public synchronized void updateLinkerScript(IProject project)
			throws CoreException {

		// get the assumed .o files in this project
		ProjectAccessor util = new ProjectAccessor(project);
		List objFiles = util.getObjectListForSection();

		// get the linkerscript setting of this project
		Gnu33ProjectDescriptorManager desc = Gnu33ProjectDescriptorManager.getDescriptorManager(project);
		LinkerScriptSetting setting = desc.getLdsSetting();
		
		// update all of the sections that are auto update
		setting.updateAllObjList(objFiles);
		
		// update it!
		desc.updateLinkerScriptSetting(setting);

	}
	
	/**
	 * Light method for external use.
	 * @throws CoreException
	 */
	public synchronized void updateLinkerScript() throws CoreException {
		updateLinkerScript(project);
	}

	/**
	 * 
	 * @see org.eclipse.core.runtime.jobs.Job#belongsTo(java.lang.Object)
	 */
	public boolean belongsTo(Object family) {
		return this.family.equals(family);
	}
	
	/**
	 * The job will be scheduled if the target project exists.
	 * 
	 * @see org.eclipse.core.runtime.jobs.Job#shouldSchedule()
	 */
	public boolean shouldSchedule() {
		if (!project.exists()) {
			return false;
		} else if (!project.isOpen()) {
			return false;
		}
		return true;

	}

	/** 
	 * The job will be run if the target project exists.
	 * @see org.eclipse.core.runtime.jobs.Job#shouldRun()
	 */
	public boolean shouldRun() {
		if (!project.exists() || !project.getFile(Gnu33ProjectDescriptorManager.getDescriptorFileName()).exists()) {
			return false;
		} else if (!project.isOpen()) {
			return false;
		}
		return true;

	}

}
