/*
 * Created : 2007/03/22
 * 
 * This class should be common to GNU33 and GNU17.
 */
package jp.co.epson.ide.eclipse.gnu33.others;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.resources.WorkspaceJob;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;

/**
 * Util Job class that silently deletes invalid projects
 * that were once created.
 * 
 * @author D.Fujimoto
 * @since 1.4.0
 */
public class DeleteInvalidProjectJob extends WorkspaceJob {

	public static final String JOB_NAME = "invalid projects delete";
	
	
	/**
	 * The resource to delete
	 */
	private String resource;

	/**
	 * Constructor.
	 * Gets the resource name to be deleted
	 * 
	 * @param resource
	 */
	public DeleteInvalidProjectJob(String resource) {
		super(JOB_NAME);
		this.resource = resource;
		
		setPriority(Job.BUILD);
		setSystem(false);
		setRule(ResourcesPlugin.getWorkspace().getRoot());
	}

	
	/*
	 * (non-Javadoc)
	 * Handler that delets the project.
	 * 
	 * @see org.eclipse.core.resources.WorkspaceJob#runInWorkspace(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public IStatus runInWorkspace(IProgressMonitor monitor) throws CoreException {
		if (monitor == null) {
			monitor = new NullProgressMonitor();
		}
		
		// do not care if the delete() throws an exception since the project is
		// already invalid
		IProject project = ResourcesPlugin.getWorkspace().getRoot().getProject(resource);
		project.delete(IResource.ALWAYS_DELETE_PROJECT_CONTENT | IResource.FORCE, monitor);
		
		return Status.OK_STATUS;
	}
	
	/**
	 * The job will be run if the target project exists.
	 * @see org.eclipse.core.runtime.jobs.Job#shouldRun()
	 */
	public boolean shouldRun() {
		IProject project = ResourcesPlugin.getWorkspace().getRoot().getProject(resource);
		
		if (!project.exists()) {
			return false;
		}
		return true;
	}
	
}
