/*
 * 쐬: 2006/04/05
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 */
package jp.co.epson.ide.eclipse.gnu33.ui.actions;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

import jp.co.epson.ide.eclipse.gnu33.ResourceProvider;

import org.eclipse.cdt.core.model.IBinary;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;

/**
 * Abstract super class for object file conversion actions.
 * Provides general operations for these actions.
 * 
 * @author D.Fujimoto
 *
 */
public abstract class AbstractObjectFileConversionAction implements
		IObjectActionDelegate {

	// file extensions
	public static final String EXT_SREC = ResourceProvider.getResourceString("srecfile.extension");
	public static final String EXT_BIN = ResourceProvider.getResourceString("objcopy.binfile.extension");
	
	
	// the file to convert
	protected IFile inFile;
	
	// shell for dialog display
	protected Shell shell;
	
	
	// extensions
	String objExt;
	String execExt;
	
	/**
	 * 
	 */
	public AbstractObjectFileConversionAction() {
		super();
		
		objExt = ResourceProvider.getResourceString("objectfile.extension").replaceAll("\\.", "");
		execExt = ResourceProvider.getResourceString("executable.extension").replaceAll("\\.", "");
	}
	
	/* ( Javadoc)
	 * @see org.eclipse.ui.IObjectActionDelegate#setActivePart(org.eclipse.jface.action.IAction, org.eclipse.ui.IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
		shell = targetPart.getSite().getShell();
	}

	/* ( Javadoc)
	 * Generat operation when funning this action.
	 * See generateCommandLine() for details
	 *  
	 * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
	 */
	public void run(IAction action) {
		// create a specific command line
		String cmdLine = generateCommandLine(inFile);
			
			
//		System.out.println(cmdLine);
		int exitValue = -1;
		try {
			Process p = Runtime.getRuntime().exec(cmdLine);
			p.waitFor();
			
			// show errors in a dialog
			exitValue = p.exitValue();
			if (exitValue != 0) {

				BufferedReader br = new BufferedReader(new InputStreamReader(p.getErrorStream()));
				String line = null;
				StringBuffer sb = new StringBuffer();
				while ((line = br.readLine()) != null) {
					sb.append(line);
				}
				br.close();	
				
				MessageDialog.openError(shell, getErrorDialogTitle(), new String(sb));
//				showInConsole(new String(sb));
			}
			
		} catch (IOException e) {
			e.printStackTrace();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
		// refresh the project
		try {
			inFile.getProject().refreshLocal(IResource.DEPTH_INFINITE, null);
		
//			// force focus to Navigator
//			if (exitValue == 0) {
//				IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
//				IResourceNavigator view = 
//					(IResourceNavigator) page.findView(IPageLayout.ID_RES_NAV);
//				page.showView(IPageLayout.ID_RES_NAV);
//			}

		} catch (CoreException e1) {
			e1.printStackTrace();
		}

	}

	
	/**
	 * Abstract method for obtaining the command line.
	 * @return	the command line for a particular action.
	 */
	protected abstract String generateCommandLine(IFile infile);
	
	/**
	 * Returns the title for the error dialog.
	 * Subclasses should extend.
	 * @return	the title. This impl will return null.
	 */
	public String getErrorDialogTitle() {
		return null;
	}
	

	/* ( Javadoc)
	 * Only files with extension o and elf is permitted
	 * 
	 * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction, org.eclipse.jface.viewers.ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		action.setEnabled(false);
		
		IFile targetFile = null;
		
		IStructuredSelection sel = (IStructuredSelection) selection;
		if (sel != null && !sel.isEmpty()) {
			Object o = sel.getFirstElement();
			
			if  (o instanceof IBinary) {
				// for CView
				IBinary bin = (IBinary) o;
				targetFile = (IFile) bin.getResource();
			} else 	if (o instanceof IFile) {
				// for Navigator View
				targetFile = (IFile) o; 
			
			} 
			
			if (targetFile != null) {
/* >>>>> MODIFIED by D.Fujimoto 2007/04/02 check file extension not null */
//				if (targetFile.getFileExtension().equals(objExt) || targetFile.getFileExtension().equals(execExt)) {
/* <<<<< MODIFIED by D.Fujimoto 2007/04/02 check file extension not null */
				String ext = targetFile.getFileExtension();
				if (ext != null && (ext.equals(objExt) || ext.equals(execExt))) {
					action.setEnabled(true);
					this.inFile = targetFile;
				}	
			}
		}

	}
	
	
//	/**
//	 * Show the message in a console.
//	 * @param message
//	 */
//	public void showInConsole(String message) {
//	
//		MessageConsole console = null;
//		
//		IConsoleManager man = ConsolePlugin.getDefault().getConsoleManager();
//		IConsole[] consoles = man.getConsoles();
//		
//		// get the console
//		boolean found = false;
//		for (int i = 0; i < consoles.length; i++) {
//			if (consoles[i].getName().equals(getConsoleName())) {
//				found = true;
//				console = (MessageConsole) consoles[i];
//				break;
//			}
//		}
//		
//		// create a new console if not registered
//		if (!found) {
//			console = new MessageConsole(getConsoleName(), null);
//			man.addConsoles(new IConsole[] {console});
//		}
//		
//		// print to console
//		MessageConsoleStream stream = console.newMessageStream();
//		stream.print(message);
//	
//	}
//	
//	/**
//	 * Returns the name of the console.
//	 * Subclass should override.
//	 * @return console name
//	 */
//	protected String getConsoleName() {
//		return "Conversion";
//	}

}
