/*
 * 쐬: 2006/03/30
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 */
package jp.co.epson.ide.eclipse.gnu33.ui.actions;

import jp.co.epson.ide.eclipse.gnu33.ResourceProvider;
import jp.co.epson.ide.eclipse.gnu33.variables.ToolPathVarResolver;

import org.eclipse.core.resources.IFile;

/**
 * Action for creating an srec file for .o and .elf files.
 * @author D.Fujimoto
 *
 */
public class MakeSrecAction extends AbstractObjectFileConversionAction {

	// program options
	public static final String OPTIONS = "-O srec --srec-forceS3";
	
	// extension for output file
	String srecExt;
	

	/**
	 * Constructor.
	 */
	public MakeSrecAction() {
		super();
		
		srecExt = EXT_SREC.replaceAll("\\.", "");
	}


	/**
	 * Generates a command line to invoke from this action.
	 * @param	infile	input file as IFile
	 * @return	the command line for this action
	 */
	protected String generateCommandLine(IFile infile) {
		
		String toolPath = ToolPathVarResolver.getGnuToolsPath()
			.append(ResourceProvider.getResourceString("gnu33tools.objcopy")).toString();
		

		// create the command line
		String cmdLine = 
			toolPath + " " 
			+ OPTIONS + " "
			+ infile.getLocation().toString() + " " 
			+ infile.getLocation().removeFileExtension().addFileExtension(srecExt).toString();
		
		return cmdLine;
	}
	
	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.ui.actions.AbstractObjectFileConversionAction#getErrorDialogTitle()
	 */
	public String getErrorDialogTitle() {
		return ResourceProvider.getResourceString("srec.conversion.failed");
	}

}
