/*
 * Created on 2007/07/31
 * This class is common to GNU33 and GNU17
 */
package jp.co.epson.ide.eclipse.gnu33.ui.actions;

import org.eclipse.core.runtime.IProduct;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.ide.AboutInfo;
import org.eclipse.ui.internal.ide.dialogs.WelcomeEditorInput;

/**
 * Opens the welcome page in old format.
 * This may show warnings  
 */
public class OpenWelcomeAction implements IWorkbenchWindowActionDelegate {
	
	public static final String EDITOR_ID = "org.eclipse.ui.internal.ide.dialogs.WelcomeEditor";
	
	private IWorkbenchWindow window;
	/**
	 * The constructor.
	 */
	public OpenWelcomeAction() {
	}

	/**
	 * The action has been activated. The argument of the
	 * method represents the 'real' action sitting
	 * in the workbench UI.
	 * @see IWorkbenchWindowActionDelegate#run
	 */
	public void run(IAction action) {
		IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
		openWelcomeEditor(page);
	}
	
	/**
	 * Open the welcome.
	 * Ignore access restriction warnings
	 * @param page
	 */
	private void openWelcomeEditor(IWorkbenchPage page) {
		// open the Welcome editor
      IProduct product = Platform.getProduct();
      AboutInfo productInfo = new AboutInfo(product);
      try {
			page.openEditor(new WelcomeEditorInput(productInfo), EDITOR_ID);
		} catch (PartInitException e) {
			e.printStackTrace();
		}
		
	}

	/**
	 * Selection in the workbench has been changed. We 
	 * can change the state of the 'real' action here
	 * if we want, but this can only happen after 
	 * the delegate has been created.
	 * @see IWorkbenchWindowActionDelegate#selectionChanged
	 */
	public void selectionChanged(IAction action, ISelection selection) {
	}

	/**
	 * We can use this method to dispose of any system
	 * resources we previously allocated.
	 * @see IWorkbenchWindowActionDelegate#dispose
	 */
	public void dispose() {
	}

	/**
	 * We will cache window object in order to
	 * be able to provide parent shell for the message dialog.
	 * @see IWorkbenchWindowActionDelegate#init
	 */
	public void init(IWorkbenchWindow window) {
		this.window = window;
	}
}
