/*
 * Created on 2004/08/18
 * Modified on 2005/07/21 by D.Fujimoto added method constrainFpicState() for medda32 and fPIC
 * 											option constraints.
 *
 */
package jp.co.epson.ide.eclipse.gnu33.ui.dialogs.build;

import java.util.ArrayList;
import java.util.List;

import jp.co.epson.ide.eclipse.gnu33.build.BuildSettingFactory;
import jp.co.epson.ide.eclipse.gnu33.general.GeneralInfo;

import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.IPreferenceStore;

/**
 * Parent class of a BuildSettings page shown 
 * when the tree is selected. 
 * @author D.Fujimoto
 *
 */
public class BuildSettingsInputPage extends FieldEditorPreferencePage {


	/**
	 * The List of FieldEditors contained in this page.
	 */
	protected List fieldList = new ArrayList();
	
	/**
	 * The CPU Type of this project.
	 */
	protected int cpuType;
	
	/**
	 * Constructor.
	 */
	public BuildSettingsInputPage() {
		super(GRID);
		noDefaultAndApplyButton();
	}
	
	public BuildSettingsInputPage(int cpuType) {
		this();
		this.cpuType = cpuType;
	}

	/**
	 * This implementation sets the PreferenceStore from the Container 
	 * this page is contained in.
	 * @see org.eclipse.jface.preference.FieldEditorPreferencePage#createFieldEditors()
	 */
	protected void createFieldEditors() {
		// Get the preference store for the build settings
		
		IPreferenceStore settings = getPreferenceStore();
		setPreferenceStore(settings);

	}
	
	/**
	 * Store every FieldEditor's values to the PreferenceStore.
	 * @see FieldEditorPreferencePage#performOk()
	 */
	public void storeSettings() {
		super.performOk();

	}

	/**
	 * Update fields from the PreferenceStore.
	 */
	public void updateFields() {
		for (int i = 0; i < fieldList.size(); i++) {
			FieldEditor editor = (FieldEditor) fieldList.get(i);
			editor.loadDefault();
			editor.load();
		}
	}


	/**
	 * Special constraints for medda32 and fPIC options.
	 *
	 * @since 1.1.1
	 */
	public void constrainFpicState() {
		
		// get the current value from the store
		IPreferenceStore store = getPreferenceStore();
		boolean medda = store.getBoolean(BuildSettingFactory.C_MEDDA_OPT_ID);
		boolean fPIC = store.getBoolean(BuildSettingFactory.FPIC_OPT_ID);
		
		// get the field editor
		FieldEditor meddaField = null;
		FieldEditor fPICField = null;
		for (int i = 0; i < fieldList.size(); i++) {
			FieldEditor editor = (FieldEditor) fieldList.get(i);
			String prefName = editor.getPreferenceName();
			if (prefName.equals(BuildSettingFactory.C_MEDDA_OPT_ID)) {
				meddaField = editor;
			} else if (prefName.equals(BuildSettingFactory.FPIC_OPT_ID)) {
				fPICField = editor;
			}
		}
		
		// set the value and enablement of the FieldEditors
		if (medda) {
			// whenever medda is ON->medda check ON/fPIC check OFF]
			if (meddaField != null) {
				meddaField.setEnabled(true, getFieldEditorParent());
			}
			if (fPICField != null) {
				fPICField.setEnabled(false, getFieldEditorParent());
			}
			// reset the fPIC value
			store.setValue(BuildSettingFactory.FPIC_OPT_ID, false);
		} else if (fPIC) {
			// the medda check OFF/fPIC check OFF
			if (meddaField != null) {
				meddaField.setEnabled(false, getFieldEditorParent());
			}
			if (fPICField != null) {
				fPICField.setEnabled(true, getFieldEditorParent());
			}
		} else {
			// when both are OFF
			if (meddaField != null) {
				// for ADV, field should be disabled
				meddaField.setEnabled(!(cpuType == GeneralInfo.CPU_ADV), getFieldEditorParent());
			}
			if (fPICField != null) {
				fPICField.setEnabled(true, getFieldEditorParent());
			}
		}
	
	}

}
