/*
 * Created on 2004/08/16
 * Modified on 2005/07/21 by D.Fujimoto modified displayOptionsForCategory() for option constraints.
 * Modified on 2005/07/25 by D.Fujimoto modified performOk() for lds setting auto updating.
 * Modified on 2005/08/05 by D.Fujimoto modified displayOptionsForCategory() and
 * 											performOk() for data pointer removal.
 * Modified on 2006/03/28 by D.Fujimoto modified getProjectSettings(), refreshPage(), and various parts
 * 											of createContents() for new environment UI. 
 * Modified on 2006/06/15 by D.Fujimoto modified performOk() for env settings.
 * Modified on 2006/06/29 by D.Fujimoto modified refreshPage() to show target file.
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 * Modified on 2007/08/03 by D.Fujimoto modified performOk() to show error when pj file is read only.
 */
package jp.co.epson.ide.eclipse.gnu33.ui.dialogs.build;


import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import jp.co.epson.ide.eclipse.gnu33.Gnu33CorePlugin;
import jp.co.epson.ide.eclipse.gnu33.Gnu33ProjectDescriptorManager;
import jp.co.epson.ide.eclipse.gnu33.ResourceProvider;
import jp.co.epson.ide.eclipse.gnu33.build.BuildCategory;
import jp.co.epson.ide.eclipse.gnu33.build.BuildSetting;
import jp.co.epson.ide.eclipse.gnu33.build.BuildTool;
import jp.co.epson.ide.eclipse.gnu33.build.IBuildCategory;
import jp.co.epson.ide.eclipse.gnu33.build.IBuildOption;
import jp.co.epson.ide.eclipse.gnu33.build.IBuildTool;
import jp.co.epson.ide.eclipse.gnu33.linkerscript.LinkerScriptUpdater;
import jp.co.epson.ide.eclipse.gnu33.others.CleanJob;
import jp.co.epson.ide.eclipse.gnu33.others.Logger;
import jp.co.epson.ide.eclipse.gnu33.ui.dialogs.AbstractGnu33PropertyPage;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.IPreferencePageContainer;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceStore;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Layout;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;

/**
 * The Property Page for a Build Setting in a Gnu33 Project.
 * @author D.Fujimoto
 *
 */
public class BuildSettingsPropertyPage extends AbstractGnu33PropertyPage 
	implements IPreferencePageContainer { 


	public static final String GROUP_LABEL = BuildSettingDialogResources.getString("build.option.label");
	public static final String ENV_LABEL = BuildSettingDialogResources.getString("env.label");
	public static final String PASS_GROUP_LABEL = BuildSettingDialogResources.getString("build.pass.group.label"); 
	public static final String TWOPASS_LABEL = BuildSettingDialogResources.getString("build.2pass.label");
	public static final String ONEPASS_LABEL = BuildSettingDialogResources.getString("build.1pass.label");
	public static final String GOAL_GROUP_LABEL = BuildSettingDialogResources.getString("goal.group.label");
	public static final String GOAL_MASK_LABEL = BuildSettingDialogResources.getString("goal.mask.label");
	public static final String GOAL_EXE_LABEL = BuildSettingDialogResources.getString("goal.exe.label");
	
	// preference names
	public static final String P_TWOPASS = "is2passBuild";
	public static final String P_GOAL = "buildGoal";

	
	// name for the temporary PreferenceStore
	public static final String STORE_NAME = "options";
	
	/**
	 * List of BuildSetting from the Project Description.
	 */
	private List buildSettingsList;
	private BuildSetting setting;
	/**
	 * List of IBuildTool to show in the Tree.
	 */
	private List toolList;
	
	/**
	 * environment setting
	 * @since 1.3.0
	 */
	private List envVars;
	
	/**
	 * currently selected tool in the Tree .
	 */
	private IBuildTool selectedTool;
	/**
	 * currently selected category in the Tree.
	 */
	private IBuildCategory selectedCategory;
	/**
	 * currently selected page.
	 */
	private BuildSettingsInputPage currentSettingsPage; 
	/**
	 * Map of BuildSettingsInputPage.
	 * Pages that are create are stored in this map. 
	 */
	private Map pageMap = new HashMap();
	
	/**
	 * PreferenceStore that stores the settings of each option.
	 */
	private IPreferenceStore settingsStore;
	
	private boolean storeChanged;
	
	/**
	 * Page size for the option input page.
	 */
	private Point minimumPageSize = new Point(350, 200);
	/**
	 * Weights between the Tree and the ScrolledComposite.
	 */
	private static final int[] SASH_WEIGHTS = new int[] {32, 68};
	
	
	// the controls
	private TreeViewer optionTree;				// Tree
	private ScrolledComposite containerSC;		// right side of Tree
	private Composite inputContainer;			// Composite to show the inputs
	private Label targetFileLabel;				// the target file label
	private Button radio2pass;					// 2passbuild switch 
	private Button radio1pass;					// 1passbuild switch 
//	private Button radioPsa;					// goal switch
//	private Button radioElf;					// goal switch
	
	/**
	 * The Environment tab
	 * @since 1.3.0
	 */
	private EnvironmentsBlock envBlock;

	/**
	 * Contructor. Called by the framework.
	 */
	public BuildSettingsPropertyPage() {
		super();
		
		// create the environment block instance
		envBlock = new EnvironmentsBlock();
	}

	/**
	 * Create the UI here.
	 * @return the UI Controls 
	 * @see org.eclipse.jface.preference.PreferencePage#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent) {
	
		// make the UI here
		// label showing target file
		targetFileLabel = new Label(parent, SWT.WRAP);
		GridData gdTargetLabel = new GridData();
		gdTargetLabel.grabExcessHorizontalSpace = true;		// for big fonts
		targetFileLabel.setLayoutData(gdTargetLabel);
	
		Composite container = new Composite(parent, SWT.NONE);
	
		container.setLayout(new FillLayout());
		
	
		// TabFolder
		TabFolder folder = new TabFolder(container, SWT.NONE);
		folder.setLayout(new GridLayout());
		
		TabItem options = new TabItem(folder, SWT.NONE);
		options.setText(GROUP_LABEL);
	
		// Create build options
		Composite sashGroup = new Composite(folder, SWT.NONE);
		GridLayout sashLayout = new GridLayout();
		sashLayout.verticalSpacing = 2;		// 10 to 2
		sashLayout.marginHeight = 0;
		sashGroup.setLayout(sashLayout);
		sashGroup.setLayoutData(new GridData(GridData.FILL_BOTH));
	
/* >>>>> ADDED by D.Fujimoto 2008/06/04 2pass/goal switches */
		Composite generalComp = new Composite(sashGroup, SWT.NONE);
		GridData gdGeneralComp = new GridData(GridData.FILL_HORIZONTAL);
		gdGeneralComp.verticalAlignment = GridData.VERTICAL_ALIGN_BEGINNING;
		generalComp.setLayoutData(gdGeneralComp);
		GridLayout layoutGeneral = new GridLayout();
		layoutGeneral.makeColumnsEqualWidth = true;
		layoutGeneral.numColumns = 1;
		layoutGeneral.verticalSpacing = 2;
		generalComp.setLayout(layoutGeneral);

		// N pass build switch
		Group buildPassGroup = new Group(generalComp, SWT.NONE);
		GridData gdPassGroup = new GridData(GridData.FILL_BOTH);
		gdPassGroup.grabExcessHorizontalSpace = true;
		gdPassGroup.grabExcessVerticalSpace = true;
		buildPassGroup.setLayoutData(gdPassGroup);
		buildPassGroup.setText(PASS_GROUP_LABEL);

		GridLayout gridPass = new GridLayout();
		gridPass.makeColumnsEqualWidth = true;
		gridPass.numColumns = 2;
		gridPass.horizontalSpacing = 20;
		buildPassGroup.setLayout(gridPass);

		// radio buttons
		radio2pass = new Button(buildPassGroup, SWT.RADIO);
		GridData gd2pass = new GridData();
		gd2pass.horizontalAlignment = SWT.BEGINNING;
		gd2pass.widthHint = 250;
		radio2pass.setLayoutData(gd2pass);
		radio2pass.setText(TWOPASS_LABEL);
		radio2pass.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {
			}

			public void widgetSelected(SelectionEvent e) {
				boolean selected = ((Button) e.widget).getSelection();
				settingsStore.setValue(P_TWOPASS, selected);	// ON
			}
		});
		
		radio1pass = new Button(buildPassGroup, SWT.RADIO);
		GridData gd1pass = new GridData();
		gd1pass.horizontalAlignment = SWT.BEGINNING;
		radio1pass.setLayoutData(gd1pass);
		radio1pass.setText(ONEPASS_LABEL);
		radio1pass.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {
			}

			public void widgetSelected(SelectionEvent e) {
				boolean selected = ((Button) e.widget).getSelection();
				settingsStore.setValue(P_TWOPASS, !selected);	// OFF
			}
		});
		
		// not yet supported
//		// goal switch
//		Group goalGroup = new Group(generalComp, SWT.NONE);
//		GridData gdGoalGroup = new GridData(GridData.FILL_BOTH);
//		gdGoalGroup.grabExcessHorizontalSpace = true;
//		gdGoalGroup.grabExcessVerticalSpace = true;
//		goalGroup.setLayoutData(gdGoalGroup);
//		goalGroup.setText(GOAL_GROUP_LABEL);
//		
//		GridLayout gridGoal = new GridLayout();
//		gridGoal.makeColumnsEqualWidth = true;
//		gridGoal.numColumns = 2;
//		gridGoal.horizontalSpacing = 20;
//		goalGroup.setLayout(gridGoal);
//
//		// radio buttons
//		radioPsa = new Button(goalGroup, SWT.RADIO);
//		GridData gdPsa = new GridData();
//		gdPsa.horizontalAlignment = SWT.BEGINNING;
//		gdPsa.widthHint = 150;
//		radioPsa.setLayoutData(gdPsa);
//		radioPsa.setText(GOAL_MASK_LABEL);
//		radioPsa.addSelectionListener(new SelectionListener() {
//			public void widgetDefaultSelected(SelectionEvent e) {
//			}
//
//			public void widgetSelected(SelectionEvent e) {
//				boolean selected = ((Button) e.widget).getSelection();
//				settingsStore.setValue(P_GOAL, selected ? BuildSetting.GOAL_PSA : BuildSetting.GOAL_ELF);
//			}
//			
//		});
//		
//		radioElf = new Button(goalGroup, SWT.RADIO);
//		GridData gdElf = new GridData();
//		gdElf.horizontalAlignment = SWT.BEGINNING;
//		radioElf.setLayoutData(gdElf);
//		radioElf.setText(GOAL_EXE_LABEL);
//		radioElf.addSelectionListener(new SelectionListener() {
//			public void widgetDefaultSelected(SelectionEvent e) {
//			}
//
//			public void widgetSelected(SelectionEvent e) {
//				boolean selected = ((Button) e.widget).getSelection();
//				settingsStore.setValue(P_GOAL, selected ? BuildSetting.GOAL_ELF : BuildSetting.GOAL_PSA);
//			}
//			
//		});
//	
/* <<<<< ADDED by D.Fujimoto 2008/06/04 2pass/goal switches */
		
		SashForm sashForm = new SashForm(sashGroup, SWT.HORIZONTAL);
		sashForm.setOrientation(SWT.HORIZONTAL);
		GridData gridData = new GridData(GridData.FILL_BOTH);
		gridData.grabExcessVerticalSpace = true;	// add
		sashForm.setLayoutData(gridData);
		GridLayout sashFormLayout = new GridLayout();
		sashFormLayout.numColumns = 2;
		sashForm.setLayout(sashFormLayout);
		
		
		// The option tree
		Composite treeBlock = new Composite(sashForm, SWT.NONE);
		treeBlock.setLayout(new GridLayout());
		treeBlock.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		optionTree = new TreeViewer(treeBlock, SWT.SINGLE | SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER);
		optionTree.setContentProvider(new BuildSettingContentProvider());
		optionTree.setLabelProvider(new BuildSettingLabelProvider());
		optionTree.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent e) {
				handleOptionSelection();
			}
		});
		optionTree.getControl().setLayoutData(new GridData(GridData.FILL_BOTH));
	
	
		// The option inputs
		containerSC = new ScrolledComposite(sashForm, SWT.H_SCROLL | SWT.V_SCROLL);	
		GridData gridSC = new GridData(GridData.FILL_HORIZONTAL);
		containerSC.setLayoutData(gridSC);
		containerSC.setExpandVertical(true);
		containerSC.setExpandHorizontal(true);
		
		inputContainer = new Composite(containerSC, SWT.NULL);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		inputContainer.setLayout(new PageLayout());
		
		containerSC.setContent(inputContainer);
	
		containerSC.setMinSize(inputContainer.computeSize(SWT.DEFAULT, SWT.DEFAULT));
		containerSC.layout();
		
		sashForm.setWeights(SASH_WEIGHTS);
		
		// set the build options
		options.setControl(sashGroup);
	
		
		// Create environment tab
		TabItem env = new TabItem(folder, SWT.NONE);
		env.setText(ENV_LABEL);
		env.setControl(envBlock.createContents(folder));
		
		
		// Do not call performDefaults()
		getProjectSettings();
		refreshPage();
		
		
		return container;
	}
	
	/**
	 * Get the BuildSettings from the Project Description file. 
	 */
	public void getProjectSettings() {
		
		super.getProjectSettings();
		   
		// get settings from descriptor
		buildSettingsList = descManager.getBuildList();
		setting = (BuildSetting) buildSettingsList.get(0);
		
		// prepare a List for the TableViewer
		toolList = new ArrayList();
		toolList.add(setting.getCompiler());
		toolList.add(setting.getAssembler());
		toolList.add(setting.getLinker());
		
		// set the settings to a PreferenceStore which is not for a Plugin 
		// the PreferenceStore we use should be expendable so create it in
		// the plugin state area(the file name is irrelevant) 
		IPath storePath = Gnu33CorePlugin.getDefault().getStateLocation().append(STORE_NAME);
		settingsStore = new PreferenceStore(storePath.toOSString());
		for (Iterator iter = toolList.iterator(); iter.hasNext();) {
			IBuildTool tool = (IBuildTool) iter.next();
			settingsStore.setValue(tool.getToolID(), tool.getCommand());
			
			IBuildOption[] options = tool.getOptions();
			for (int i = 0; i < options.length; i++) {
				switch (options[i].getType()) {
					case IBuildOption.TYPE_BOOLEAN :
						settingsStore.setValue(options[i].getId(), options[i].getBooleanValue());
						break;
					case IBuildOption.TYPE_STRING :
					case IBuildOption.TYPE_STRING_MULTI :
						settingsStore.setValue(options[i].getId(), options[i].getStringValue());
						break;
					case IBuildOption.TYPE_ENUM :
						String[] enumOps = options[i].getEnumOptions();
						if (enumOps.length - 1 >= options[i].getEnumValue()) {
							settingsStore.setValue(options[i].getId(), enumOps[options[i].getEnumValue()]);    
						}
						
						break;
					case IBuildOption.TYPE_LIST :
						// make a ; separated String
						String[] values = options[i].getListValue();
						String str = "";
						for (int j = 0; j < values.length; j++) {
							str += values[j] + IBuildOption.LIST_DELIMITER;
						}
						settingsStore.setValue(options[i].getId(), str);
						break;
				
				}
			}
		}
		
		// get the env settings and give it to the UI
		envVars = setting.getEnvVars();
		envBlock.setVarElements(envVars);
		
/* >>>>> ADDED by D.Fujimoto 2008/06/04 2pass and goal switch */
		// set the 2pass switch settings
		radio2pass.setSelection(setting.is2passBuild());
		radio1pass.setSelection(!setting.is2passBuild());
		settingsStore.setDefault(P_TWOPASS, setting.is2passBuild());

		// not yet supported
//		// set the goal settings 
//		boolean isGoalPsa = setting.getBuildGoal() == BuildSetting.GOAL_PSA;
//		radioPsa.setSelection(isGoalPsa);	
//		radioElf.setSelection(!isGoalPsa);
//		settingsStore.setDefault(P_GOAL, setting.getBuildGoal());
/* <<<<< ADDED by D.Fujimoto 2008/06/04 2pass and goal switch */
		
	}
	
	/**
	 * Refreshes the whole displays of this page.
	 * This method should be called only at creation of this page, 
	 * or when the cpu type has changed in GeneralPropertyPage. 
	 */
	public void refreshPage() {
	
		// label showing target file
		String targetFile = MessageFormat.format(BuildSettingDialogResources.getString("build.targetFile.label"), 
				new Object[] {setting.getMakefileName() + ResourceProvider.getResourceString("makefile.extension")});
		targetFileLabel.setText(targetFile);
		
		// clear the page mapping and the current page
		pageMap.clear();
		selectedTool = null;
		selectedCategory = null;
		currentSettingsPage = null;

		optionTree.setInput(toolList);
		optionTree.expandAll();
		
		// select the top level tool of Tree
		optionTree.setSelection(new StructuredSelection(toolList.get(0)), true);

		// environment settings
		envBlock.refresh();

/* >>>>> ADDED by D.Fujimoto 2007/03/12 detect option changed */				
		// set an option change listener here(not in getProjectSettings())
		storeChanged = false;
		settingsStore.addPropertyChangeListener(new IPropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent event) {
/* >>>>> MODIFIED by D.Fujimoto 2007/04/13 compare with the orignal value with store */			
				// check if the store and the project file has some differences
				String name = event.getProperty();
				String newValue = settingsStore.getString(name);
				String oldValue = getOriginalOptionValue(name, toolList);
				if (oldValue != null && !oldValue.equals(newValue)) {
					storeChanged = true;					
				}
/* >>>>> MODIFIED by D.Fujimoto 2007/04/13 compare with the orignal value with store */
			}
		});
/* <<<<< ADDED by D.Fujimoto 2007/03/12 detect option changed */
		

/* >>>>> ADDED by D.Fujimoto 2008/06/04 listener for switches */		
		// listener for switches 
		settingsStore.addPropertyChangeListener(new IPropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent event) {
				String name = event.getProperty();
				if (name.equals(P_TWOPASS)) {
					storeChanged |= !settingsStore.isDefault(P_TWOPASS);
				} else if (name.equals(P_GOAL)) {
					storeChanged |= !settingsStore.isDefault(P_GOAL);
				}
			}
		});
/* <<<<< ADDED by D.Fujimoto 2008/06/04 listener for switches */
		
	}
		
	/**
	 * Reads the original option value with the given <code>name</code>
	 * from the project file. 
	 * @param name			property name of the option
	 * @param toolList		the tool List<IBuildTool> to search from
	 * @return	the original value (may be empty String) for the option as String
	 * 			or null if not found.
	 * 
	 * Note that some property names are only used for UI storing (like the all options field) 
	 * and may not be found, hence will return null.
	 */
	private String getOriginalOptionValue(String name, List toolList) {
		String orgValue = "";
		boolean nameFound = false;
		
		for (Iterator iter = toolList.iterator(); iter.hasNext();) {
			IBuildTool tool = (IBuildTool) iter.next();
			
			IBuildOption[] options = tool.getOptions();
			for (int i = 0; i < options.length; i++) {
				
				if (options[i].getId().equals(name)) {
					nameFound = true;
					
					switch (options[i].getType()) {
						case IBuildOption.TYPE_BOOLEAN :
							orgValue = Boolean.toString(options[i].getBooleanValue());
							break;
						case IBuildOption.TYPE_STRING :
						case IBuildOption.TYPE_STRING_MULTI :
							orgValue = options[i].getStringValue();
							break;
						case IBuildOption.TYPE_ENUM :
							String[] enumOps = options[i].getEnumOptions();
							if (enumOps.length - 1 >= options[i].getEnumValue()) {
								orgValue = enumOps[options[i].getEnumValue()];    
							}
							break;
						case IBuildOption.TYPE_LIST :
							// make a ; separated String
							String[] values = options[i].getListValue();
							String str = "";
							for (int j = 0; j < values.length; j++) {
								str += values[j] + IBuildOption.LIST_DELIMITER;
							}
							// delete last delimiter
							if (str.trim().endsWith(IBuildOption.LIST_DELIMITER)) {
								str = str.substring(0, str.lastIndexOf(IBuildOption.LIST_DELIMITER));
							}
							orgValue = str;
							break;
					
					}
				}
			}
		}
		
		if (!nameFound) {
			orgValue = null;
		}
		return orgValue;
	}
	

	
	/**
	 * Event handler for tree selection.
	 */
	private void handleOptionSelection() {

		IStructuredSelection sel = (IStructuredSelection) optionTree.getSelection();
		Object element = sel.getFirstElement();
		if (element instanceof BuildTool) {
			IBuildTool tool = (IBuildTool) element;
			displayOptionsForTool(tool);
		} else if (element instanceof BuildCategory) {
			IBuildCategory cat = (IBuildCategory) element;
			displayOptionsForCategory(cat);
		}
		
		containerSC.setContent(inputContainer);
		containerSC.setMinSize(inputContainer.computeSize(SWT.DEFAULT, SWT.DEFAULT));
		containerSC.layout();
		
	}
	

	/**
	 * Shows the input controls to the right of the tree when a Tool is selected.
	 * @param tool 	the selected tool
	 */
	private void displayOptionsForTool(IBuildTool tool) {
	   
		// do nothing if the tool is unchanged
		if (tool == selectedTool) {
			return;
		}
		
		selectedCategory = null;
		selectedTool = tool;
		
		// Cache the current page
		BuildSettingsInputPage cachedPage = currentSettingsPage;
		
		// has the page already been created?
		currentSettingsPage = (BuildSettingsInputPage) pageMap.get(tool.getToolID());
		if (currentSettingsPage == null) {
			// create a page
			currentSettingsPage = new BuildSettingsToolPage(tool, setting.getSettingName());
			pageMap.put(tool.getToolID(), currentSettingsPage);
			currentSettingsPage.setContainer(this);
			if (currentSettingsPage.getControl() == null) {
				currentSettingsPage.createControl(inputContainer);
			}
		}
		
		
		// Make all the other pages invisible
		Control[] children = inputContainer.getChildren();
		Control currentControl = currentSettingsPage.getControl();
		for (int i = 0; i < children.length; i++) {
			if (children[i] != currentControl)
				children[i].setVisible(false);
		}
		currentSettingsPage.setVisible(true);

		// save the last pages options
		if (cachedPage != null) {
			cachedPage.storeSettings();
		}
		
		if (currentSettingsPage instanceof BuildSettingsToolPage) {
			((BuildSettingsToolPage) currentSettingsPage).displayAllOptions();
		}		
		
		containerSC.setMinSize(inputContainer.computeSize(SWT.DEFAULT, SWT.DEFAULT, true));
		containerSC.layout();
	}
	
	/**
	 * Shows the input controls to the right of the tree when a category is selected
	 * @param category the selected category
	 */
	private void displayOptionsForCategory(IBuildCategory category) {
		// do nothing if the category is unchanged
		if (category == selectedCategory) {
			return;
		}
		
		selectedTool = null;
		selectedCategory = category;
		
		// Cache the current page
		BuildSettingsInputPage cachedPage = currentSettingsPage;
		
		// has the page already been created?
		currentSettingsPage = (BuildSettingsInputPage) pageMap.get(category.getCategoryID());
		if (currentSettingsPage == null) {
			// create a page if necessary
			currentSettingsPage = new BuildSettingsOptionPage(category, descManager.getCPUType());
			pageMap.put(category.getCategoryID(), currentSettingsPage);
			currentSettingsPage.setContainer(this);
			if (currentSettingsPage.getControl() == null) {
				currentSettingsPage.createControl(inputContainer);
			}
		}
		
		
		// Make all the other pages invisible
		Control[] children = inputContainer.getChildren();
		Control currentControl = currentSettingsPage.getControl();
		for (int i = 0; i < children.length; i++) {
			if (children[i] != currentControl)
				children[i].setVisible(false);
		}
		currentSettingsPage.setVisible(true);
		
		
		// save the last pages options
		if (cachedPage != null) {
			cachedPage.storeSettings();
		}
		
		// force medda32 and fPIC options states 
		currentSettingsPage.constrainFpicState();
		
		// refresh the page
		((BuildSettingsOptionPage) currentSettingsPage).updateFields();
		
		containerSC.setMinSize(inputContainer.computeSize(SWT.DEFAULT, SWT.DEFAULT, true));
		containerSC.layout();
	}
	
		
	/**
	 * Handles default button pressed.
	 * Restores every option to its default setting in the descriptor file
	 * and the view remains.
	 */
	protected void performDefaults() {

		
		boolean doit = MessageDialog.openConfirm(getShell(), 
				DEFAULT_DLG_TITLE, DEFAULT_DLG_MSG);
	   
		if (doit) {
			
			// clear the page mapping
			pageMap.clear();
			selectedCategory = null;
			selectedTool = null;
			currentSettingsPage = null;
	
			// get the settings from the descriptor file
			getProjectSettings();
	
			// reset the whole page
			refreshPage();

		}
	}
	
	/**
	 * Handles ok button events.
	 * Gathers the input values, and updates the description file.
	 * @see BuildSettingsToolPage#performOk()
	 * @see BuildSettingsOptionPage#performOk() 
	 */
	public boolean performOk() {
	   

		// nothing to do if the map is empty
		if (pageMap == null) {
			return true;
		}
		
		for (Iterator iter = pageMap.values().iterator(); iter.hasNext();) {
			BuildSettingsInputPage page = (BuildSettingsInputPage) iter.next();
			if (page instanceof BuildSettingsToolPage) {
				((BuildSettingsToolPage) page).performOk();
			} else if (page instanceof BuildSettingsOptionPage) {
				((BuildSettingsOptionPage) page).performOk();
			}
		}

		// get the env settings
		envBlock.performOk();
		setting.setEnvVars(envBlock.getVarElements()); 
		
		// refresh the env block because this might be an apply button op
		envBlock.setVarElements(envBlock.getVarElements());
		envBlock.refresh();
		
/* >>>>> ADDED by D.Fujimoto 2008/06/04 2pass goal switch */		
		// set the 2pass switch 
		setting.setIs2passBuild(radio2pass.getSelection());
		
		// not yet supported
//		// set the goal switch
//		boolean isGoalPsa = radioPsa.getSelection();
//		if (isGoalPsa) {
//			setting.setBuildGoal(BuildSetting.GOAL_PSA);	
//		} else {
//			setting.setBuildGoal(BuildSetting.GOAL_ELF);
//		}
/* <<<<< ADDED by D.Fujimoto 2008/06/04 2pass goal switch */
		
		// write it out
		try {
			descManager.updateBuildSetting(buildSettingsList);
			// also update the auto sections in the linkerscript setting
			new LinkerScriptUpdater(project.getName()).updateLinkerScript();
			
/* >>>>> ADDED by D.Fujimoto 2007/04/13 clean build when option changed */
			// perform a clean build here(when something changes)
			if (storeChanged) {
				boolean doClean = false;
				String title = BuildSettingDialogResources.getString("needsbuild_title");
				String msg = BuildSettingDialogResources.getString("needsbuild_message");
				MessageDialog dialog = new MessageDialog(getShell(), title, null, msg, MessageDialog.QUESTION, new String[] { IDialogConstants.YES_LABEL, IDialogConstants.NO_LABEL }, 1);
				int res = dialog.open();
				if (res == Dialog.OK) {
					CleanJob cleanJob = new CleanJob(project);
					cleanJob.schedule();
				} else {
					; // No pressed, no clean
				}

				// forget the changed state
				storeChanged = false;
				
			}
/* <<<<< ADDED by D.Fujimoto 2007/04/13 clean build when option changed */
			
		} catch (CoreException e) {
/* >>>>> MODIFIED by D.Fujimoto 2007/08/03 display error */
			String title = MSG_UPDATE_FAILED + Gnu33ProjectDescriptorManager.getDescriptorFileName();
			String message = e.getLocalizedMessage();
			Logger.log(message);
			displayErrorDialog(title, message);
/* <<<<< MODIFIED by D.Fujimoto 2007/08/03 display error */

			return false;
		}
		
		return super.performOk();
	}
	
	/**
	 * Returns the PreferenceStore for this page.
	 * @see org.eclipse.jface.preference.IPreferencePageContainer#getPreferenceStore()
	 */
	public IPreferenceStore getPreferenceStore() {
		return settingsStore;
	}
	
	/**
	 * This method does nothing.
	 * @see org.eclipse.jface.preference.IPreferencePageContainer#updateButtons()
	 */
	public void updateButtons() {
	}

	/**
	 * This method does nothing.
	 * @see org.eclipse.jface.preference.IPreferencePageContainer#updateMessage()
	 */
	public void updateMessage() {
	}

	/**
	 * This method does nothing.
	 * @see org.eclipse.jface.preference.IPreferencePageContainer#updateTitle()
	 */
	public void updateTitle() {}

	/**
	 * Provides access to the Apply Button.
	 */
	protected Button getApplyButton() {
		return super.getApplyButton();
	}
	
	/**
	 * ContentProvider for the BuildSettings TreeViewer.
	 * @author D.Fujimoto
	 *
	 */
	private class BuildSettingContentProvider implements ITreeContentProvider {

	   
		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
		 */
		public Object[] getChildren(Object parentElement) {
			if (parentElement instanceof List) {
				List list = (List) parentElement;
				IBuildTool[] tools = (IBuildTool[]) list.toArray(new IBuildTool[0]);
				return tools;
			} else if (parentElement instanceof IBuildTool) {
				IBuildTool tool = (IBuildTool) parentElement;
				List list = tool.getOptionCategories();
				IBuildCategory[] cats = (IBuildCategory[]) list.toArray(new IBuildCategory[0]);
				return cats;
			} else {
				// return an empty Object
				return new Object[0];
		   }
		   
		}

		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
		 */
		public Object getParent(Object element) {
			if (element instanceof IBuildCategory) {
				IBuildCategory cat = (IBuildCategory) element;
				return cat.getTool();
			}
			return null;
		}

		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
		 */
		public boolean hasChildren(Object element) {
			return getChildren(element).length > 0;
		}

		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
		 */
		public Object[] getElements(Object inputElement) {
			return getChildren(inputElement);
		}

		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
		 */
		public void dispose() {
		}

		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
		 */
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}
		
	}

	/**
	 * LabelProvider for the BuildSettings TreeViewer.
	 * @author D.Fujimoto
	 *
	 */
	private class BuildSettingLabelProvider extends LabelProvider implements ILabelProvider {

		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.ILabelProvider#getText(java.lang.Object)
		 */
		public String getText(Object element) {
			if (element instanceof IBuildTool) {
				IBuildTool tool = (IBuildTool) element;
				return tool.getName();
			} else if (element instanceof IBuildCategory) {
				IBuildCategory cat = (IBuildCategory) element;
				return cat.getCategoryName();
			}
			return null;
		}

	}
	
	/**
	 * Special Layout for the option input page.
	 *
	 */
	private class PageLayout extends Layout {
		public void layout(Composite composite, boolean force) {
			Rectangle rect = composite.getClientArea();
			Control[] children = composite.getChildren();
			for (int i = 0; i < children.length; i++) {
				children[i].setSize(rect.width, rect.height);
			}
		}
		public Point computeSize(Composite composite, int wHint, int hHint,	boolean force) {
			if (wHint != SWT.DEFAULT && hHint != SWT.DEFAULT) {
				return new Point(wHint, hHint);
			}
			int x = minimumPageSize.x;
			int y = minimumPageSize.y;

			Control[] children = composite.getChildren();
			for (int i = 0; i < children.length; i++) {
				Point size = children[i].computeSize(SWT.DEFAULT, SWT.DEFAULT, force);
				x = Math.max(x, size.x);
				y = Math.max(y, size.y);
			}
			if (wHint != SWT.DEFAULT) {
				x = wHint;
			}
			if (hHint != SWT.DEFAULT) {
				y = hHint;
			}
			return new Point(x, y);
		}
	}

}
