/*
 * 쐬: 2006/03/27
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 */
package jp.co.epson.ide.eclipse.gnu33.ui.dialogs.build;

import java.io.File;
import java.util.Iterator;
import java.util.List;

import jp.co.epson.ide.eclipse.gnu33.build.EnvVar;
import jp.co.epson.ide.eclipse.gnu33.build.IEnvVar;
import jp.co.epson.ide.eclipse.gnu33.file.MakeFileGenerator;
import jp.co.epson.ide.eclipse.gnu33.others.UIUtil;

import org.eclipse.core.runtime.Path;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

/**
 * Dialog for editing the environment name/value pair
 * @author D.Fujimoto
 * @since 1.3.0
 *
 */
public class EnvVarEditDialog extends Dialog {

	// controls
	private Text nameText;
	private Text pathText;
	private Label messageLabel;
	
	/**
	 * The current IEnvVar being edited
	 */
	private IEnvVar currentVar;
	
	/**
	 * The whole setting.
	 * The added/edited EnvVar object will be stored in here.
	 */
	private List envVars;
	
	/**
	 * Constructor.
	 * @param parent	parent shell
	 * @param	env		the setting to be edited, may be null when add mode
	 * @param	envVars	the whole setting 
	 */
	public EnvVarEditDialog(Shell parent, IEnvVar env, List envVars) {
		super(parent);
		currentVar = env;
		this.envVars = envVars;
	}

	/* ( Javadoc)
	 * Sets the title for this dialog
	 * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
	 */
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		
		if (currentVar == null) {
			newShell.setText(BuildSettingDialogResources.getString("envDialog.title.new"));	
		} else {
			newShell.setText(BuildSettingDialogResources.getString("envDialog.title.edit"));
		}
		
	}	
	
	/* ( Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent) {
		Control control = super.createContents(parent);
		// do initial input check after creating the buttons
		verifyInputs();
		return control;
	}

	
	/* ( Javadoc)
	 * Creates the UI part
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent) {
		Composite composite = (Composite) super.createDialogArea(parent);
		
		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		layout.marginHeight = 10;
		composite.setLayout(layout);
		
		//
		Label nameLabel = new Label(composite, SWT.NONE);
		nameLabel.setText(BuildSettingDialogResources.getString("envDialog.field.name"));
		GridData gdLabel = new GridData();
		gdLabel.horizontalAlignment = GridData.FILL;
		nameLabel.setLayoutData(gdLabel);
		
		// name field
		nameText = new Text(composite, SWT.SINGLE | SWT.BORDER);
		GridData gdText = new GridData();
		gdText.horizontalAlignment = GridData.FILL;
		gdText.grabExcessHorizontalSpace = false;
		gdText.widthHint = convertWidthInCharsToPixels(30);
		nameText.setLayoutData(gdText);
		nameText.setTextLimit(200);
		nameText.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				// never allow $ ( ) { } chars
				UIUtil.enableAlphNumInput(e);
			}
		});
		nameText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				verifyInputs();
			}
		});
		
		Label dummy = new Label(composite, SWT.NONE);
		
		Label pathLabel = new Label(composite, SWT.NONE);
		pathLabel.setText(BuildSettingDialogResources.getString("envDialog.field.value"));
		GridData gdLabel2 = new GridData();
		gdLabel2.horizontalAlignment = GridData.FILL;
		pathLabel.setLayoutData(gdLabel2);
		
		// path field
		pathText = new Text(composite, SWT.SINGLE | SWT.BORDER);
		GridData gdText2 = new GridData();
		gdText2.horizontalAlignment = GridData.FILL;
		gdText2.grabExcessHorizontalSpace = false;
		gdText2.widthHint = convertWidthInCharsToPixels(50);
		pathText.setLayoutData(gdText2);
		pathText.setTextLimit(200);
		pathText.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				UIUtil.enablePathInput(e);
			}
		});
		pathText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				verifyInputs();
			}
		});
		
		// browse button
		Button fileButton = new Button(composite, SWT.PUSH);
		fileButton.setText(BuildSettingDialogResources.getString("envDialog.button.folder"));
		fileButton.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent e) {
				DirectoryDialog dialog = new DirectoryDialog(getShell());
				dialog.setFilterPath(pathText.getText());
				String res = dialog.open();
				if (res != null) {
					// replace backslashes to slashes
					pathText.setText(res.replaceAll("\\\\", "/"));
				}
			}

			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});
		
		Label dummy2 = new Label(composite, SWT.NONE);
		
		// error message label
		messageLabel = new Label(composite, SWT.NONE);
		GridData gdMsg = new GridData();
		gdMsg.horizontalSpan = 2;
		gdMsg.horizontalAlignment = GridData.FILL;
		gdMsg.heightHint = convertHeightInCharsToPixels(3);
		messageLabel.setLayoutData(gdMsg);
		
		// set the UI
		populateSetting();
		verifyInputs();
		
		applyDialogFont(composite);
		return composite; 
	}
	
	
	/**
	 * Populates the UI.
	 */
	private void populateSetting() {
		
		if (currentVar != null) {
			// edit mode
			nameText.setText(currentVar.getName());
			pathText.setText(currentVar.getValue());	
		} else {
			// new mode
			nameText.setText("");
			pathText.setText("");
		}
		

	}

	/**
	 * Perform error checks
	 */
	private void verifyInputs() {
		String errorMessage = null;
		

		// check name
		String name = nameText.getText().trim();
		if (name.length() == 0) {
			errorMessage = BuildSettingDialogResources.getString("envDialog.error.nameLength");
		} else {
			// check invalid chars
			if (!UIUtil.isAlphNumString(name)) {
				errorMessage = BuildSettingDialogResources.getString("envDialog.error.nameInvalidChar");
			}
		}
		
		// check duplicate names
		for (Iterator iter = envVars.iterator(); iter.hasNext();) {
			IEnvVar others = (IEnvVar) iter.next();
			if (!others.equals(currentVar) && name.equals(others.getName())) {
				errorMessage = name + " " + BuildSettingDialogResources.getString("envDialog.error.nameExists");
				break;
			}
		}
		
		// check for reserved names
		String[] reserved = MakeFileGenerator.RESERVED_MACROS;
		for (int i = 0; i < reserved.length; i++) {
			if (name.equals(reserved[i])) {
				errorMessage = name + " " + BuildSettingDialogResources.getString("envDialog.error.nameReserved");
				break;
			}
		}
		
		if (errorMessage == null) {
			// check path (if name wasn't an error)
			String path = pathText.getText().trim();
			if (path.length() == 0) {
				errorMessage = BuildSettingDialogResources.getString("envDialog.error.valueLength");
			} else if (!UIUtil.isPathString(path)) {
				// check invalid chars
				errorMessage = BuildSettingDialogResources.getString("envDialog.error.valueInvalidChar");
			} else {
				// check invalid paths
				if (!Path.EMPTY.isValidPath(path)) {
					errorMessage = BuildSettingDialogResources.getString("envDialog.error.valueInvalid");
				} else if (!new File(path).exists() || !new File(path).isDirectory()) {
					errorMessage = BuildSettingDialogResources.getString("envDialog.error.valueNotExist");
				}
			}
		}
		
		// set the message and update button
		messageLabel.setText(errorMessage == null ? "" : errorMessage);
		Button ok = getButton(IDialogConstants.OK_ID);
		if (ok != null) {
			ok.setEnabled(errorMessage == null);
		}
		
	}
	
	
	/* ( Javadoc)
	 * Stores the result when OK button is pressed
	 * @see org.eclipse.jface.dialogs.Dialog#okPressed()
	 */
	protected void okPressed() {
	
		// verify them once more(in case of ok when New dialog is showing)
		verifyInputs();
		if (messageLabel.getText() == null || messageLabel.getText().equals("")) {

			// replace the backslashes to slashes
			String path = pathText.getText();
			IEnvVar newEnv = new EnvVar(nameText.getText(), path.replaceAll("\\\\", "/"));
			
			// store the setting
			if (currentVar == null) {
				// new mode
				envVars.add(newEnv);	
			} else {

				// edit mode(remove old and add new)
				IEnvVar element = null;
				for (Iterator iter = envVars.iterator(); iter.hasNext();) {
					element = (IEnvVar) iter.next();
					if (element.getName().equals(currentVar.getName())) {
						break;
					}
				}
				if (element != null) {
					envVars.remove(element);
				}
				envVars.add(newEnv);
			}
			
			// set current for selection
			currentVar = newEnv;
				
		}
		
		super.okPressed();
	}
	
	/**
	 * Returns the currently edited object
	 * for restoring the selection in the viewer
	 * @return	the current EnvVar 
	 */
	public IEnvVar getCurrent() {
		return currentVar;
	}
	
}
