/*
 * Created  on 2006/03/28 by D.Fujimoto
 * Modified on 2006/06/15 by D.Fujimoto modified various parts for displaying
 * 											reserved IEnvVar objects.
 *
 */
package jp.co.epson.ide.eclipse.gnu33.ui.dialogs.build;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import jp.co.epson.ide.eclipse.gnu33.build.BuildSettingFactory;
import jp.co.epson.ide.eclipse.gnu33.build.IEnvVar;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IColorProvider;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;

/**
 * UI for the environment settings.
 * 
 * @author D.Fujimoto
 * @since 1.3.0
 *
 */
public class EnvironmentsBlock extends PreferencePage {

	/**
	 * The data to be shown in the viewer 
	 */ 
	private List varElements;
	
	private List reservedEnvVars = BuildSettingFactory.getReservedEnvVars();
	
	// the controls
	private Control control;
	private TableViewer envViewer;
	private Button buttonAdd;
	private Button buttonEdit;
	private Button buttonRemove;
	
	/**
	 * Default Constructor.
	 */
	public EnvironmentsBlock() {
		super();
	}
	
	/**
	 * Sets the main setting
	 * @param envVars	List of IEnv
	 */
	public void setVarElements(List envVars) {
		
		// add the vars from the project and the reserved ones 
		varElements = new ArrayList();
		varElements.addAll(envVars);
		varElements.addAll(reservedEnvVars);
	}
	
	/**
	 * Returns the modified elements for saving to the project file
	 * 
	 */
	public List getVarElements() {
		return varElements;
	}
	
	/* ( Javadoc)
	 * @see org.eclipse.jface.preference.PreferencePage#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent) {
		return createEnvControls(parent);
	}

	
	/**
	 * Creates the UI for the environment setting.
	 * @param parent	the parent Composite
	 * @return	the created Control
	 */
	private Control createEnvControls(Composite parent) {
		
		Composite envComp = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		envComp.setLayout(layout);
	
		// description label
		Label descLabel = new Label(envComp, SWT.WRAP | SWT.LEFT);
		GridData gdDesc = new GridData();
		gdDesc.horizontalSpan = 2;
		descLabel.setLayoutData(gdDesc);
		descLabel.setText(BuildSettingDialogResources.getString("env.description"));
		
		// table
		Table envTable = new Table(envComp, SWT.FULL_SELECTION | SWT.SINGLE | SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
		envTable.setHeaderVisible(true);
		envTable.setLinesVisible(true);
		GridData gdTable = new GridData(GridData.FILL_BOTH);
		gdTable.heightHint = 100;
		envTable.setLayoutData(gdTable);
		
		TableColumn c1 = new TableColumn(envTable, SWT.NONE);
		c1.setText(BuildSettingDialogResources.getString("env.tableCol.name"));
		c1.setWidth(200);
		
		TableColumn c2 = new TableColumn(envTable, SWT.NONE);
		c2.setText(BuildSettingDialogResources.getString("env.tableCol.value"));
		c2.setWidth(200);
		
		// Table Viewer
		envViewer = new TableViewer(envTable);
		envViewer.setContentProvider(new ArrayContentProvider());
		envViewer.setLabelProvider(new EnvVarLabelProvider());
		envViewer.setSorter(new EnvVarViewerSorter());
		envViewer.addPostSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				handleEnvSelection();
			}
		});
		envViewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				IEnvVar selected = (IEnvVar) ((IStructuredSelection) event.getSelection()).getFirstElement();
				// no action for reserve vars
				if (!selected.isReserved()) {
					handleButtonPressed(2);
				}
				
			}
		});
		
		// button area
		Composite buttonComp = new Composite(envComp, SWT.NONE);
		GridData gdButtons = new GridData();
		gdButtons.verticalAlignment = GridData.BEGINNING;
		buttonComp.setLayoutData(gdButtons);
		buttonComp.setLayout(new GridLayout());	
		
		buttonAdd = new Button(buttonComp, SWT.PUSH);
		GridData gdAdd = new GridData(GridData.FILL_HORIZONTAL);
		buttonAdd.setLayoutData(gdAdd);
		buttonAdd.setText(BuildSettingDialogResources.getString("env.button.New.label"));
		buttonAdd.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent e) {
				handleButtonPressed(1);
			}
	
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});
		
		
		buttonEdit = new Button(buttonComp, SWT.PUSH);
		GridData gdEdit = new GridData(GridData.FILL_HORIZONTAL);
		buttonEdit.setLayoutData(gdEdit);
		buttonEdit.setText(BuildSettingDialogResources.getString("env.button.Edit.label"));
		buttonEdit.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent e) {
				handleButtonPressed(2);
			}
	
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});
		
		buttonRemove = new Button(buttonComp, SWT.PUSH);
		GridData gdRemove = new GridData(GridData.FILL_HORIZONTAL);
		buttonRemove.setLayoutData(gdRemove);
		buttonRemove.setText(BuildSettingDialogResources.getString("env.button.Remove.label"));
		buttonRemove.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent e) {
				handleButtonPressed(3);
			}
	
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});
	
		Dialog.applyDialogFont(envComp);
		control = envComp;
		return envComp;
	}

	/**
	 * Provide access to the TableViewer 
	 * @return TableViewer
	 */
	public TableViewer getViewer() {
		return envViewer;
	}
	
	/**
	 * Refreshes the (viewer's) contents of the UI
	 * Must be invoked after the controls have been created.
	 */
	public void refresh() {
		
		// sets the input and selects it
		Collections.sort(varElements);
		envViewer.setInput(varElements);
        if (varElements.size() > 0) {
        	envViewer.setSelection(new StructuredSelection(varElements.get(0)), true);
        } else {
        	envViewer.setSelection(StructuredSelection.EMPTY);
        }
		
	}
	
	/**
	 * Event handler when buttons for environment settings is pressed
	 * @param kind 1:buttonAdd 2:buttonEdit 3:buttonRemove
	 */
	private void handleButtonPressed(int kind) {
		
		// get the currently selected element
		IEnvVar selected = (IEnvVar) ((IStructuredSelection) envViewer.getSelection()).getFirstElement();
		
		// one that should be selected after this method finishes
		IStructuredSelection newSelection = null;
		newSelection = selected == null ? StructuredSelection.EMPTY : new StructuredSelection(selected);
		
		if (kind == 1) {
			// Add
			EnvVarEditDialog dialog = new EnvVarEditDialog(control.getShell(), null, varElements);
			dialog.open();
			
			// set the new selection
			IEnvVar edited = dialog.getCurrent();
			if (edited != null) {
				newSelection = new StructuredSelection(edited);
			}
			
		} else if (kind == 2) {
			// Edit
			EnvVarEditDialog dialog = new EnvVarEditDialog(control.getShell(), selected, varElements);
			dialog.open();
			
			// set the new selection
			IEnvVar edited = dialog.getCurrent();
			if (edited != null) {
				newSelection = new StructuredSelection(edited);
			}
			
		} else {
			// Remove
			varElements.remove(selected);
			newSelection = StructuredSelection.EMPTY;
		}
		
		envViewer.refresh();
		envViewer.setSelection(newSelection, true);
	}

	/**
	 * Changes the button state when the viewer item gets
	 * selected/unselected
	 */
	private void handleEnvSelection() {
		
		buttonAdd.setEnabled(true);
		IStructuredSelection selection = (IStructuredSelection) envViewer.getSelection();
		if (selection.isEmpty()) {
			buttonEdit.setEnabled(false);
			buttonRemove.setEnabled(false);
		} else {
			IEnvVar selectedVar = (IEnvVar) selection.getFirstElement();

			// reserved ones are not editable
			boolean isReserved = selectedVar.isReserved();
			buttonEdit.setEnabled(!isReserved);
			buttonRemove.setEnabled(!isReserved);
		}
	
	}
	
	/* ( Javadoc)
	 * @see org.eclipse.jface.preference.PreferencePage#performOk()
	 */
	public boolean performOk() {
		
		// remove the reserved ones
		if (varElements != null) {
			varElements.removeAll(reservedEnvVars);
		}
		return super.performOk();
	}

	/**
	 * LabelProvider for the Environment table
	 * @author D.Fujimoto
	 */
	private class EnvVarLabelProvider extends LabelProvider implements ITableLabelProvider, IColorProvider {
	
		/* ( Javadoc)
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object, int)
		 */
		public Image getColumnImage(Object element, int columnIndex) {
			return null;
		}
	
		/* ( Javadoc)
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object, int)
		 */
		public String getColumnText(Object element, int columnIndex) {
			if (element instanceof IEnvVar) {
				IEnvVar env = (IEnvVar) element;
				switch (columnIndex) {
					case 0 :	// name
						String ret = env.getName();
						
						if (env.isReserved()) {
							ret += " (" + BuildSettingDialogResources.getString("env.reserved.string")+ ")";
						}
						return ret;
						
					case 1 :	// value
						return env.getValue();
				}
			}
				
			return super.getText(element);
		}

		/* ( Javadoc)
		 * @see org.eclipse.jface.viewers.IColorProvider#getForeground(java.lang.Object)
		 */
		public Color getForeground(Object element) {
			return null;
		}

		/* ( Javadoc)
		 * @see org.eclipse.jface.viewers.IColorProvider#getBackground(java.lang.Object)
		 */
		public Color getBackground(Object element) {

			if (element instanceof IEnvVar) {
				IEnvVar env = (IEnvVar) element;
				// bg color for reserved vars
				if (env.isReserved()) {
					return Display.getCurrent().getSystemColor(SWT.COLOR_INFO_BACKGROUND);
				}
			}
			return null;
		}
		
	}

	/**
	 * ViewerSorter for the Environment table
	 * @author D.Fujimoto
	 */
	private class EnvVarViewerSorter extends ViewerSorter {
		
		/* ( Javadoc)
		 * @see org.eclipse.jface.viewers.ViewerSorter#compare(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
		 */
		public int compare(Viewer viewer, Object e1, Object e2) {
			if (e1 instanceof IEnvVar && e2 instanceof IEnvVar) {
				// firstly, try to sort by category
				int cat1 = category(e1);
				int cat2 = category(e2);
				if (cat1 != cat2) {
					return cat1 - cat2;
				}

				// if the categories are the same, compare by name
				return ((IEnvVar) e1).compareTo(e2);
			}
			
			return super.compare(viewer, e1, e2);
		}
		
		/* ( Javadoc)
		 * Reserved vars are in the first category
		 * Normal vars in the second category
		 * 
		 * @see org.eclipse.jface.viewers.ViewerSorter#category(java.lang.Object)
		 */
		public int category(Object element) {
			
			if (element instanceof IEnvVar) {
				IEnvVar env = (IEnvVar) element;
				if (env.isReserved()) {
					// reserved one's come first
					return 0;
				}
			}
			
			// default category
			return 1;
		}
	}


}
