/*
 * Created on 2004/09/07
 * Modified on 2005/06/16 by D.Fujimoto added auto update check box(checkAutoUpdate)
 * 											and other UI adjustments,
 * 											including modifications to populateSetting()
 * Modified on 2005/06/29 by D.Fujimoto added method checkDupObjFiles() and modified verifyInputs()
 * 											to invoke this.
 * 										refactored method gatherInputValues() that was in okPressed().
 * 										modified methods handleAddFile() and handleDelFile() to invoke
 * 											verifyInputs().
 * Modified on 2005/11/10 by D.Fujimoto added checks for section name and attribute in
 * 											verifyInputs().
 * Modified on 2005/11/14 by D.Fujimoto added editedSectionData and its getter.
 * Modified on 2006/03/22 by D.Fujimoto modified populateSetting(), createDialogArea() for
 * 											special treatements to default standard sections.
 * Modified on 2006/06/23 by D.Fujimoto added special treatements to okPressed() for standard sections.
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0) for Eclipse3.2
 * Modified on 2007/04/03 by D.Fujimoto changed populateObjInProjList() for allowing
 * 											duplicated library names in both lists. 
 * Modified on 2007/04/13 by D.Fujimoto changed textSecName to textSecName
 */
package jp.co.epson.ide.eclipse.gnu33.ui.dialogs.linkerscript;

import java.util.ArrayList;
import java.util.Arrays;

import jp.co.epson.ide.eclipse.gnu33.linkerscript.LinkerScriptSetting;
import jp.co.epson.ide.eclipse.gnu33.linkerscript.SectionData;
import jp.co.epson.ide.eclipse.gnu33.others.ProjectAccessor;
import jp.co.epson.ide.eclipse.gnu33.others.UIUtil;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

/**
 * This is the input dialog for editing a new Section.
 * @author D.Fujimoto
 *
 */
public class LinkerScriptEditDialog extends TitleAreaDialog {

	// title and message of this dialog
	public static final String DLG_DESC = LinkerScriptSettingDialogResources.getString("linkerscript.dlg.desc");
	public static final String DLG_MSG = LinkerScriptSettingDialogResources.getString("linkerscript.dlg.msg");

	public static final String MSG_SEC_NAME_BLANK = LinkerScriptSettingDialogResources.getString("linkerscript.dlg.msg.secNameBlank");
	public static final String MSG_SEC_NAME_INVALID = LinkerScriptSettingDialogResources.getString("linkerscript.dlg.msg.secNameInvalid");
	public static final String MSG_INVALID_CHAR_SEC_NAME = LinkerScriptSettingDialogResources.getString("linkerscript.dlg.msg.secNameInvalidChar");
	public static final String MSG_SEC_NAME_NUM_START = LinkerScriptSettingDialogResources.getString("linkerscript.dlg.msg.secNameNumStart");
	public static final String MSG_SEC_NAME_NO_DOT = LinkerScriptSettingDialogResources.getString("linkerscript.dlg.msg.secNameNoDot");
	public static final String MSG_VMA_BLANK = LinkerScriptSettingDialogResources.getString("linkerscript.dlg.msg.vmaBlank");
	public static final String MSG_INVALID_CHAR_VMA = LinkerScriptSettingDialogResources.getString("linkerscript.dlg.msg.vmaInvalidChar");
	public static final String MSG_LMA_BLANK = LinkerScriptSettingDialogResources.getString("linkerscript.dlg.msg.lmaBlank");
	public static final String MSG_INVALID_CHAR_LMA = LinkerScriptSettingDialogResources.getString("linkerscript.dlg.msg.lmaInvalidChar");
	public static final String MSG_FILE_ATTR_BLANK = LinkerScriptSettingDialogResources.getString("linkerscript.dlg.msg.fileAttrBlank");
	public static final String MSG_FILE_ATTR_NUM_START = LinkerScriptSettingDialogResources.getString("linkerscript.dlg.msg.fileAttrNumStart");
	public static final String MSG_INVALID_CHAR_FILE_ATTR = LinkerScriptSettingDialogResources.getString("linkerscript.dlg.msg.fileAttrInvalidChar");


	/**
	 * Add mode constant for setMode().
	 */
	public static final int MODE_ADD = 1;

	/**
	 * Edit mode constant for setMode().
	 */
	public static final int MODE_EDIT = 2;

	/**
	 * Mode for the dialog.
	 */
	private int mode = MODE_ADD;

	/**
	 * The LinkerScriptSetting.
	 */
	private LinkerScriptSetting setting;

	/**
	 * Contains the edited data for this dialog.
	 */
	private SectionData sectionData = new SectionData();

	/**
	 * The List for SectionData contained in setting.
	 */
	private java.util.List sections;

	/**
	 * User libraries set in the Build Setting.
	 */
	private String[] userLibs;

	/**
	 * Utility class to gather information of the project.
	 */
	private ProjectAccessor accessor;

	/**
	 * The section which should be selected in the 
	 * TableViewer after this dialog closes.
	 */
	private SectionData editedSectionData; 
	

	/**
	 * The error message string.
	 */
	private String errorMessage;

	// the controls
	private Text textSecName;
	private Combo comboVMA;
	private Button checkLMA;
	private Combo comboLMA;
	private Combo comboAttr;
	private Button checkAutoUpdate;
	private List fromList;
	private List toList;
	private Button buttonAddFile;
	private Button buttonDelFile;
	private Button buttonUp;
	private Button buttonDown;
	private Button buttonSelAllLeft;
	private Button buttonSelAllRight;
	private SelectionListener toListListener;
	private Label labelObjLocate;

	// the original backgroud color for the 2 lists
	Color fromListBgColor;
	Color toListBgColor;

	// Items for the section name combo.
	String[] sectionItems = SectionData.ATTR_NAMES;

	/**
	 * Initialize the dialog.
	 * @param parentShell parent window(dialog)
	 * @param setting LinkerScriptSetting object this dialog will add a section to
	 * @param userLibs	the user libraries set in the build setting
	 * @param project	the targetted project
	 */
	public LinkerScriptEditDialog(Shell parentShell, LinkerScriptSetting setting, String[] userLibs, IProject project) {
		super(parentShell);
		this.setting = setting;
		sections = setting.getSections();
		this.userLibs = userLibs;

		accessor = new ProjectAccessor(project);

	}


	/**
	 * Sets an initial size of this dialog.
	 */
	protected Point getInitialSize() {
		return super.getInitialSize();
	}

	/**
	 * Sets the title for the dialog.
	 */
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);

		// set the title here
		String title = null;
		if (mode == MODE_ADD) {
			title = LinkerScriptSettingDialogResources.getString("linkerscript.dlg.title.add");
		} else if (mode == MODE_EDIT) {
			title = LinkerScriptSettingDialogResources.getString("linkerscript.dlg.title.edit");
		}
		newShell.setText(title + LinkerScriptSettingDialogResources.getString("linkerscript.dlg.title.section"));
	}

	/**
	 * Creates the controls for this dialog.
	 *
	 * @param parent
	 *			  The parent Composite
	 */
	protected Control createDialogArea(Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		container.setLayout(new GridLayout());

		Composite composite = new Composite(container, SWT.NONE);
		GridData gridData = new GridData(GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL | GridData.VERTICAL_ALIGN_BEGINNING);
		gridData.horizontalAlignment = GridData.FILL;
		composite.setLayoutData(gridData);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		composite.setLayout(gridLayout);

		int textLimit = 255;

		// section name
		Label labelSecName = new Label(composite, SWT.NONE);
		labelSecName.setText(LinkerScriptSettingDialogResources.getString("linkerscript.dlg.secName.label"));
		textSecName = new Text(composite, SWT.SINGLE | SWT.BORDER);
		GridData gridSecName = new GridData();
		gridSecName.widthHint = 90;
		textSecName.setTextLimit(textLimit);
		textSecName.setLayoutData(gridSecName);
		textSecName.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				UIUtil.enableSectionInput(e);
			}
		});
		textSecName.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				verifyInputs();
			}
		});


		// VMA
		Label labelVMA = new Label(composite, SWT.NONE);
		labelVMA.setText(LinkerScriptSettingDialogResources.getString("linkerscript.dlg.vma.label"));
		comboVMA = new Combo(composite, SWT.DROP_DOWN);
		GridData gridVMA = new GridData();
		gridVMA.widthHint = 90;
		comboVMA.setTextLimit(textLimit);
		comboVMA.setLayoutData(gridVMA);
		comboVMA.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				UIUtil.enableSectionInput(e);
			}
		});
		comboVMA.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				verifyInputs();
			}
		});


		// LMA  VMA checkbox
		checkLMA = new Button(composite, SWT.CHECK);
		GridData gridCheckLMA = new GridData();
		gridCheckLMA.horizontalSpan = 2;
		checkLMA.setLayoutData(gridCheckLMA);
		checkLMA.setText(LinkerScriptSettingDialogResources.getString("linkerscript.dlg.lmaCheck.label"));
		checkLMA.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleLMACheck();
			}
		});

		// LMA
		Label labelLMA = new Label(composite, SWT.NONE);
		labelLMA.setText(LinkerScriptSettingDialogResources.getString("linkerscript.dlg.lma.label"));
		comboLMA = new Combo(composite, SWT.DROP_DOWN);
		GridData gridLMA = new GridData();
		gridLMA.widthHint = 90;
		comboLMA.setTextLimit(textLimit);
		comboLMA.setLayoutData(gridLMA);
		comboLMA.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				UIUtil.enableSectionInput(e);
			}
		});
		comboLMA.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				verifyInputs();
			}
		});


		// separator
		Label separator = new Label(composite, SWT.HORIZONTAL | SWT.SEPARATOR);
		GridData gridSep = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridSep.horizontalSpan = 2;
		separator.setLayoutData(gridSep);
		separator.setText("label");

		// attribute
		Label labelAttr = new Label(composite, SWT.NONE);
		labelAttr.setText(LinkerScriptSettingDialogResources.getString("linkerscript.dlg.attr.label"));
		comboAttr = new Combo(composite, SWT.DROP_DOWN | SWT.READ_ONLY);	// select only
		GridData gridAttr = new GridData();
		gridAttr.widthHint = 90;
		comboAttr.setTextLimit(textLimit);
		comboAttr.setLayoutData(gridAttr);
		comboAttr.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				UIUtil.enableSectionInput(e);
			}
		});
		comboAttr.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				verifyInputs();
			}
		});

		
		// auto update check box(as for 1.3.0, made invisible )
		checkAutoUpdate = new Button(composite, SWT.CHECK);
		GridData gridAutoCheck = new GridData();
		gridAutoCheck.horizontalSpan = 2;
		gridAutoCheck.widthHint = 0;
		gridAutoCheck.heightHint = 0;
		checkAutoUpdate.setLayoutData(gridAutoCheck);
		checkAutoUpdate.setVisible(false);
		checkAutoUpdate.setText(LinkerScriptSettingDialogResources.getString("linkerscript.dlg.autoCheck.label"));
		checkAutoUpdate.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleAutoUpdateCheck();
			}
		});

		
		// file selection
		Composite compFileSel = new Composite(composite, SWT.NONE);
		GridData gridCompFile = new GridData(GridData.FILL_HORIZONTAL | GridData.FILL_VERTICAL);
		gridCompFile.horizontalSpan = 2;
		compFileSel.setLayoutData(gridCompFile);
		GridLayout layoutCompFile = new GridLayout();
		layoutCompFile.numColumns = 4;
		compFileSel.setLayout(layoutCompFile);

		// object file locate state
		labelObjLocate = new Label(compFileSel, SWT.NONE);
		GridData gdLabel = new GridData();
		gdLabel.horizontalSpan = 3;
		gdLabel.horizontalAlignment = GridData.CENTER;
		labelObjLocate.setLayoutData(gdLabel);
		Label dummy = new Label(compFileSel, SWT.NONE);
		
		int listHeight = 155;
		int listWidth = 120;
		int listIndent = 10;

		// list labels
		Label fromLabel = new Label(compFileSel, SWT.NONE);
		fromLabel.setText(LinkerScriptSettingDialogResources.getString("linkerscript.dlg.fromlist.label"));
		GridData gdFromLabel = new GridData();
		gdFromLabel.horizontalSpan = 2;
		gdFromLabel.widthHint = listWidth;
		gdFromLabel.horizontalIndent = listIndent;
		fromLabel.setLayoutData(gdFromLabel);
		
		Label toLabel = new Label(compFileSel, SWT.NONE);
		toLabel.setText(LinkerScriptSettingDialogResources.getString("linkerscript.dlg.tolist.label"));
		GridData gdToLabel = new GridData();
		gdToLabel.horizontalSpan = 2;
		gdToLabel.widthHint = listWidth;
		toLabel.setLayoutData(gdToLabel);
		
		// from list
		fromList = new List(compFileSel, SWT.BORDER | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		GridData gridFromList = new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.VERTICAL_ALIGN_FILL);
		gridFromList.heightHint = listHeight;
		gridFromList.widthHint = listWidth;
		gridFromList.horizontalIndent = listIndent;
		gridFromList.grabExcessVerticalSpace = true;
		fromList.setLayoutData(gridFromList);
		fromList.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleLeftListSelection();
			}
		});
		fromListBgColor = fromList.getBackground();


		Composite compListButtons = new Composite(compFileSel, SWT.NONE);
		GridData gridCompButton = new GridData(GridData.HORIZONTAL_ALIGN_CENTER);
		gridCompButton.widthHint = 50;
		compListButtons.setLayoutData(gridCompButton);
		compListButtons.setLayout(new GridLayout());

		// add button
		buttonAddFile = new Button(compListButtons, SWT.ARROW | SWT.RIGHT);
		buttonAddFile.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		buttonAddFile.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleAddFile();
			}
		});

		// del button
		buttonDelFile = new Button(compListButtons, SWT.ARROW | SWT.LEFT);
		buttonDelFile.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		buttonDelFile.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleDelFile();
			}
		});

		// to list
		toList = new List(compFileSel, SWT.BORDER | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		GridData gridToList = new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.VERTICAL_ALIGN_FILL);
		gridToList.heightHint = listWidth;
		gridToList.widthHint = listWidth;
		toList.setLayoutData(gridToList);
		// create a selection listener but add it later
		toListListener = new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleRightListSelection();
			}
		};
		toListBgColor = toList.getBackground();

		Composite compUpDownButtons = new Composite(compFileSel, SWT.NONE);
		GridData gridCompButton2 = new GridData(GridData.HORIZONTAL_ALIGN_CENTER);
		gridCompButton2.widthHint = 60;
		compUpDownButtons.setLayoutData(gridCompButton2);
		compUpDownButtons.setLayout(new GridLayout());

		// up button
		buttonUp = new Button(compUpDownButtons, SWT.NONE);
		buttonUp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		buttonUp.setText(LinkerScriptSettingDialogResources.getString("linkerscript.dlg.upButton.label"));
		buttonUp.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleUpPressed();
			}
		});

		// down button
		buttonDown = new Button(compUpDownButtons, SWT.NONE);
		buttonDown.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		buttonDown.setText(LinkerScriptSettingDialogResources.getString("linkerscript.dlg.downButton.label"));
		buttonDown.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleDownPressed();
			}
		});

		// select all button (left)
		buttonSelAllLeft = new Button(compFileSel, SWT.NONE);
		GridData gdSelLeft = new GridData();
		gdSelLeft.horizontalSpan = 2;
		gdSelLeft.horizontalIndent = listIndent;
		buttonSelAllLeft.setLayoutData(gdSelLeft);
		buttonSelAllLeft.setText(LinkerScriptSettingDialogResources.getString("linkerscript.dlg.selectAllButton.label"));
		buttonSelAllLeft.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				selectAll(0);
			}
		});



		// select all button (right)
		buttonSelAllRight = new Button(compFileSel, SWT.NONE);
		GridData gdSelRight = new GridData();
		gdSelRight.horizontalSpan = 2;
		buttonSelAllRight.setLayoutData(gdSelRight);
		buttonSelAllRight.setText(LinkerScriptSettingDialogResources.getString("linkerscript.dlg.selectAllButton.label"));
		buttonSelAllRight.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				selectAll(1);
			}
		});

		// the title and the message
		setTitle(DLG_DESC);
		setMessage(DLG_MSG, IMessageProvider.NONE);

		// default work to do
		populateSetting();

		//
		return container;
	}

	/**
	 * Fills in the controls with the set SectionData object.
	 */
	private void populateSetting() {

/* >>>>> MODIFIED by D.Fujimoto 2007/04/13 not combo */	
		// set the section name 
		textSecName.setText(sectionData.getSectionName());
/* <<<<< MODIFIED by D.Fujimoto 2007/04/13 not combo */

		// set the vma combo
		comboVMA.setItems(makeComboItems());	// set some default items
		if (sectionData.isVmaIsAddress()) {
			comboVMA.setText(UIUtil.getAddressString(sectionData.getVma()));
		} else {
			// this section is placed after another section
			comboVMA.setText(sectionData.getVmaSectionName());
		}


		// set the LMA  VMA check
		comboLMA.setItems(makeComboItems());	// set some default items
		if (sectionData.isLMAequalsVMA()) {
			checkLMA.setSelection(false);
		} else {
			checkLMA.setSelection(true);
			if (sectionData.isLmaIsAddress()) {
				comboLMA.setText(UIUtil.getAddressString(sectionData.getLma()));
			} else {
				comboLMA.setText(sectionData.getLmaSectionName());
			}
		}

		// set the file attributes combo
		comboAttr.setItems(sectionItems);	// set some default items
		comboAttr.setText(sectionData.getAttribute());

		// set the auto update check
		checkAutoUpdate.setSelection(sectionData.isAutoUpdate());
		handleAutoUpdateCheck();

		handleLMACheck();

		// disabled controls for default standard sections
		// and label for object allocation
		if (sectionData.isDefaultSection()) {
			textSecName.setEnabled(false);
			comboAttr.setEnabled(false);
			labelObjLocate.setText(LinkerScriptSettingDialogResources.getString("linkerscript.dlg.allocate.label.auto"));
			
			// force auto update true
			checkAutoUpdate.setSelection(true);
			handleAutoUpdateCheck();
		} else {
			labelObjLocate.setText(LinkerScriptSettingDialogResources.getString("linkerscript.dlg.allocate.label.man"));
		}
		
	}

	/**
	 * Creates items for the left list.
	 * The items are assumed object files that are contained in the project
	 * excluded from the right list.
	 * Also contains libraries that are set in the Build Settings.
	 * Should be called after the right list items are set.
	 */
	private void populateObjInProjList() {
		// get the right list items
		java.util.List objList = Arrays.asList(toList.getItems());

		// set the left list items
		// the items of the list are built upon all object files that
		// are not contained in the right list
		fromList.removeAll();
		java.util.List objInProjList = accessor.getAssumedObjectFiles();
		for (int i = 0; i < objInProjList.size(); i++) {
			String buf = (String) objInProjList.get(i);
			if (!objList.contains(buf)) {
				fromList.add(buf);
			}
		}

		// then add the libraries
		for (int i = 0; i < userLibs.length; i++) {
			String buf = userLibs[i];
			buf = UIUtil.removeDoubleQuotes(buf);
			buf = new Path(buf).toString();
			fromList.add(buf);
		}
		// remove from the left list
		for (int i = 0; i < objList.size(); i++) {
			for (int j = 0; j < fromList.getItemCount(); j++) {
				String toItem = (String) objList.get(i);
				String fromItem = fromList.getItem(j);
				
				if (toItem.equals(fromItem)) {
					// remove only once(allow duplicated lib names) 
					fromList.remove(j);
					break;
				}
				
			} 
			
		}

	}

	/**
	 * Make default combo items.
	 * The items are section names that are already contained in the section list.
	 * For MODE_EDIT, the section name of the section which is to be edited, is excluded from the items.
	 * @return the items as String[]
	 */
	private String[] makeComboItems() {
		String[] secNames = setting.getSectionNames();
		if (mode == MODE_EDIT) {
			String excludedName = sectionData.getSectionName();
			ArrayList items = new ArrayList();
			for (int i = 0; i < secNames.length; i++) {
				if (!secNames[i].equals(excludedName)) {
					items.add(secNames[i]);
				}
			}
			secNames = (String[]) items.toArray(new String[items.size()]);

		}
		return secNames;
	}


	/**
	 * Handles LMA  VMA checkbox events.
	 */
	private void handleLMACheck() {
		comboLMA.setEnabled(checkLMA.getSelection());

		// check if the LMA combo is empty
		verifyInputs();
	}

	/**
	 * Handles Auto Update checkbox events.
	 * When the check is ON,
	 * 		the controls except the right list will be disabled.
	 * 		the 2 lists' bg will turn shadowed.
	 * 		the right list will populate itself.
	 *
	 * @since 1.1.1
	 *
	 */
	private void handleAutoUpdateCheck() {
		boolean selected = checkAutoUpdate.getSelection();

		java.util.List objList = null;	// the object list to be refreshed

		fromList.deselectAll();
		toList.deselectAll();

		// disable/enable the left list
		fromList.setEnabled(!selected);
		// disable/enable the attribute combo
		comboAttr.setEnabled(!selected);


		if (selected) {

			// set the right list contents
			objList = accessor.getObjectListForSection();
			java.util.List subList = setting.subtractObjectList(objList, sectionData, comboAttr.getText());
			String[] objFiles = (String[]) subList.toArray(new String[0]);
			toList.setItems(objFiles);
			// set the left list
			populateObjInProjList();

			// disable all buttons
			buttonSelAllLeft.setEnabled(!selected);
			buttonAddFile.setEnabled(!selected);
			buttonDelFile.setEnabled(!selected);
			buttonUp.setEnabled(!selected);
			buttonDown.setEnabled(!selected);
			buttonSelAllRight.setEnabled(!selected);

			// do not want to enable del/up/down buttons
			toList.removeSelectionListener(toListListener);

			// change the color of the list
			fromList.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WIDGET_LIGHT_SHADOW));
			toList.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WIDGET_LIGHT_SHADOW));

		} else {

			// restore the listener for del/up/down buttons
			toList.addSelectionListener(toListListener);

			// set the right list to what it is stored in the project file
			objList = sectionData.getObjectList();
			String[] objFiles = (String[]) objList.toArray(new String[0]);
			toList.setItems(objFiles);
			// set the left list
			populateObjInProjList();

			// restore the buttons
			handleLeftListSelection();
			handleRightListSelection();
			enableSelectAllButtons();

			// restore the color of the list
			fromList.setBackground(fromListBgColor);
			toList.setBackground(toListBgColor);
		}

		// check the duplicate obj
		verifyInputs();

	}


	/**
	 * Handles list selection events.
	 * Enables/disables the add button.
	 */
	private void handleLeftListSelection() {
		if (fromList.getSelectionCount() <= 0) {
			buttonAddFile.setEnabled(false);
		} else {
			buttonAddFile.setEnabled(true);
		}
	}

	/**
	 * Handles list selection events.
	 * Enables/disables delete, up, down buttons.
	 */
	private void handleRightListSelection() {
		buttonDelFile.setEnabled(false);
		buttonUp.setEnabled(false);
		buttonDown.setEnabled(false);

		int selCount = toList.getSelectionCount();

		if (selCount > 0) {
			buttonDelFile.setEnabled(true);
			if (selCount == 1) {
				int selIndex = toList.getSelectionIndex();
				if (selIndex >= 1) {
					buttonUp.setEnabled(true);
				}
				if (selIndex < toList.getItemCount() -1) {
					buttonDown.setEnabled(true);
				}
			}
		}
	}

	/**
	 * Handles add button pressed.
	 * Adds the file from the left list to right list.
	 */
	private void handleAddFile() {
		String[] selection = fromList.getSelection();
		for (int i = 0; i < selection.length; i++) {
			toList.add(selection[i]);
		}

		// recreate the left list
		populateObjInProjList();

		enableSelectAllButtons();
		handleLeftListSelection();
		handleRightListSelection();

		// check the duplicate obj
		verifyInputs();


	}

	/**
	 * Handles delete button pressed.
	 * Deletes the file from the right list to left list.
	 */
	private void handleDelFile() {
		toList.remove(toList.getSelectionIndices());

		// recreate the left list
		populateObjInProjList();

		enableSelectAllButtons();
		handleLeftListSelection();
		handleRightListSelection();

		// check the duplicate obj
		verifyInputs();

	}

	/**
	 * Handles up button pressed.
	 * Ascends the item in the right list.
	 */
	private void handleUpPressed() {

		int selIndex = toList.getSelectionIndex();
		if (selIndex <= 0 ) {
			return;
		}
		String item = toList.getItem(selIndex);
		toList.remove(selIndex);
		toList.add(item, selIndex - 1);

		// set the moved item selected
		toList.select(selIndex - 1);
		handleRightListSelection();

	}

	/**
	 * Handles down button pressed.
	 * Descends the item in the right list.
	 */
	private void handleDownPressed() {
		int selIndex = toList.getSelectionIndex();
		if (selIndex >= toList.getItemCount() - 1) {
			return;
		}
		String item = toList.getItem(selIndex);
		toList.remove(selIndex);
		toList.add(item, selIndex + 1);

		// set the moved item selected
		toList.select(selIndex + 1);
		handleRightListSelection();

	}

	/**
	 * Handls select all button pressed.
	 * Selects all items in the list.
	 * @param button -- 0 for the left, 1 for the right button.
	 */
	private void selectAll(int button) {
		if (button == 0) {
			fromList.selectAll();
			handleLeftListSelection();
		} else if (button == 1) {
			toList.selectAll();
			handleRightListSelection();
		}
	}

	/**
	 * Enables the Select All buttons if the associated list
	 * has items.
	 */
	private void enableSelectAllButtons() {
		buttonSelAllLeft.setEnabled(false);
		buttonSelAllRight.setEnabled(false);
		if (fromList.getItemCount() != 0) {
		   buttonSelAllLeft.setEnabled(true);
		}
		if (toList.getItemCount() != 0) {
			buttonSelAllRight.setEnabled(true);
		}
	}



	/**
	 * Input checks.
	 * This method only checks if the values are input correctly.
	 * This method does not check if it is a valid section.
	 * Triggered by any combo modifying event.
	 * This method may even be called by setText() or setItems().
	 */
	private void verifyInputs() {
		errorMessage = null;

		// section name check
		String secName = textSecName.getText().trim();
		if (secName.equals("")) {
			errorMessage = MSG_SEC_NAME_BLANK;
		} else if (UIUtil.isPeriodAndUnderbar(secName)) {
			errorMessage = MSG_SEC_NAME_INVALID;
		// should start with dot
		} else if (secName.charAt(0) != '.') {
			errorMessage = MSG_SEC_NAME_NO_DOT;
		} else if (!UIUtil.isSectionString(secName)) {
			errorMessage = MSG_INVALID_CHAR_SEC_NAME;
		} else if (UIUtil.startsWithDecimalNum(secName)) {
			errorMessage = MSG_SEC_NAME_NUM_START;
		}


		// virtual map address check
		String vma = comboVMA.getText().trim();
		if (vma.equals("")) {
			errorMessage = MSG_VMA_BLANK;
		} else if (!UIUtil.isSectionString(vma)) {
			errorMessage = MSG_INVALID_CHAR_VMA;
		}

		// load map address check
		if (checkLMA.getSelection()) {
			String lma = comboLMA.getText().trim();
			if (lma.equals("")) {
				errorMessage = MSG_LMA_BLANK;
			} else if (!UIUtil.isSectionString(lma)) {
				errorMessage = MSG_INVALID_CHAR_LMA;
			}
		}

		// file attributes check
		String attr = comboAttr.getText().trim();
		if (attr.equals("")) {
			errorMessage = MSG_FILE_ATTR_BLANK;
		} else if (!UIUtil.isSectionString(attr)) {
			errorMessage = MSG_INVALID_CHAR_FILE_ATTR;
		} else if (UIUtil.startsWithDecimalNum(attr)) {
			errorMessage = MSG_FILE_ATTR_NUM_START;
		}

		// duplicated object file/attribute pair check
	   	checkDupObjFiles();


		// disable the OK button on errors
		if (errorMessage == null) {
			setMessage(DLG_MSG, IMessageProvider.NONE);	
		} else {
			setMessage(errorMessage, IMessageProvider.ERROR);
		}
		Button ok = getButton(IDialogConstants.OK_ID);
		if (ok != null) {
			ok.setEnabled(errorMessage == null);
		}

	}

	/**
	 * Checks if the object files in the right list with the same attribute is
	 * not contained in another section.
	 * This method sets an error message to the dialog if it is contained in another.
	 *
	 * @since 1.1.1
	 */
	private void checkDupObjFiles() {

		// check if the inputs are filled
		if (errorMessage != null) {
			// quit
			return;
		}

		// gather the input values
		SectionData newSection = gatherInputValues();

		String originalName = "";
		if (mode == MODE_EDIT) {
			originalName = sectionData.getSectionName();
		}
		// set an error message if duplicated
		boolean duplicated = setting.checkDuplicateObjFiles(newSection, originalName);
		if (duplicated) {
			errorMessage = setting.getErrorMessage();
			setMessage(errorMessage, IMessageProvider.ERROR);
		}

	}


	/**
	 * Triggered when OK button is pressed.
	 */
	protected void okPressed() {

		// check if required controls are entered
		verifyInputs();
		if (errorMessage != null) {
			// don't close the dialog
			return;
		}

		SectionData newSection = gatherInputValues();
		
		// add the section
		int result = 0;
		if (mode == MODE_ADD) {
			// default section adds for compatibility
			if (newSection.isDefaultSection()) {
				newSection.setAutoUpdate(true);
				newSection.setAttribute(newSection.getSectionName());
			}
			
			// add the new area to the list
			result = setting.addSection(newSection);

		} else if (mode == MODE_EDIT) {
			int index = setting.getSectionIndex(sectionData.getSectionName());
			// add the edited area to the list
			result = setting.editSection(index, newSection);
			
		}

		// show an error message if failed
		if (result != LinkerScriptSetting.OK) {
			errorMessage = setting.getErrorMessage();
			setMessage(errorMessage, IMessageProvider.ERROR);

			// for cancelling after the editing failed(restore selection in the viewer)
			if (mode == MODE_EDIT) {
				editedSectionData = setting.getSection(sectionData.getSectionName());
			}
			
			return;		// don't close the window
		}

		// for selection in the viewer
		editedSectionData = newSection;

		// Return OK code
		super.okPressed();
	}

	/**
	 * Retreives all input values in this dialog.
	 *
	 * @return	the SectionData with its values filled
	 * @since 1.1.1
	 */
	private SectionData gatherInputValues() {
		// gather the input values -- begin
		SectionData newSection = new SectionData();

		// section name
		newSection.setSectionName(textSecName.getText().trim());

		// vma is assumed as an address when it consists of
		// 0-F and the length is within 8.
		// Others are considered as a section name.
		String vmaInput = comboVMA.getText().trim();
		if (UIUtil.isHexString(vmaInput) && vmaInput.length() <= UIUtil.getAddressMaxChars()) {
			// vma is hex
			newSection.setVma(Long.parseLong(vmaInput, 16));
			newSection.setVmaIsAddress(true);
		} else {
			// vma is section
			newSection.setVmaSectionName(vmaInput);
			newSection.setVmaIsAddress(false);
		}

		// lma equals vma
		if (checkLMA.getSelection()) {
			newSection.setLMAequalsVMA(false);
			// lma is assumed as an address when it consists of
			// 0-F and the length is within 8.
			// Others are considered as as section name.
			String lmaInput = comboLMA.getText().trim();
			if (UIUtil.isHexString(lmaInput) && lmaInput.length() <= UIUtil.getAddressMaxChars()) {
				// lma is hex
				newSection.setLma(Long.parseLong(lmaInput, 16));
				newSection.setLmaIsAddress(true);
			} else {
				// lma is section
				newSection.setLmaSectionName(lmaInput);
				newSection.setLmaIsAddress(false);
			}
		} else {
			newSection.setLMAequalsVMA(true);
		}

		// file attributes
		newSection.setAttribute(comboAttr.getText().trim());

		// auto update
		newSection.setAutoUpdate(checkAutoUpdate.getSelection());

		// objectList
		String[] objects = toList.getItems();
		newSection.setObjectList(Arrays.asList(objects));
		// gather the input values -- over

		return newSection;
	}


	/**
	 * @see org.eclipse.jface.dialogs.Dialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
	 */
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL,
				true);
		createButton(parent, IDialogConstants.CANCEL_ID,
				IDialogConstants.CANCEL_LABEL, false);

		// another input check
		verifyInputs();
	}

	/**
	 * Sets the open mode of this dialog.
	 * Modes are MODE_ADD | MODE_EDIT
	 * @param mode
	 */
	public void setMode(int mode) {
		this.mode = mode;
	}

	/**
	 * Sets the SectionData to edit.
	 * This is used only in EDIT mode.
	 * @param sectionData the SectionData to be edited
	 */
	public void setSectionToEdit(SectionData sectionData) {
		this.sectionData = sectionData;
	}
	
	/**
	 * Returns the SectionData that was edited in this dialog.
	 * @return the edited SectionData
	 * @since 1.2.0
	 */
	public SectionData getEditedSectionData() {
		return editedSectionData;
	}
}

