/*
 * Created on 2004/08/23
 * Modified on 2005/06/21 by D.Fujimoto modified populateTable() because of the refresh of
 * 											auto update sections.
 * Modified on 2005/08/05 by D.Fujimoto deleted controls for data pointers except dp,
 * 											modified createControls() populateDps() handleDpCheck()
 * 												checkDpInputs() gatherDpInputs().
 * Modified on 2005/11/09 by D.Fujimoto deleted checkbox for data pointer dp
 * 											modified createContents(), populateDps(), refreshPage(), gatherDpInputs()
 * 											deleted handleDpCheck()
 * Modified on 2005/11/14 by D.Fujimoto modified populateTable(), handleButtonPressed(), refreshPage() to specify
 * 											the selection after performing Add/Edit/Delete.
 * Modified on 2005/11/14 by D.Fujimoto modified createContents() added double click listener to TableViewer.
 * Modified on 2006/02/20 by D.Fujimoto modified LinkerScriptSettingLabelProvider to show lma end label.
 * Modified on 2006/03/20 by D.Fujimoto added vector address UI
 * 											added gdbSetting, modified createContents(), getProjectSettings(),
 * 											refreshPage(), performOk(), handleButtonPressed(), verifyInputs()
 * 											added okToLeave().
 * Modified on 2006/03/22 by D.Fujimoto modified enableButtons() and LinkerScriptSettingLabelProvider for
 * 											standard sections.
 * Modified on 2006/05/24 by D.Fujimoto deleted again controls and associates for vector address input,
 * 											also gdbSetting.
 * Modified on 2006/06/29 by D.Fujimoto modified refreshPage() to show target file.
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 * Modified on 2007/08/03 by D.Fujimoto modified performOk() to show error when pj file is read only.
 * Modified on 2007/08/13 by D.Fujimoto modified performOk() to not to refresh other pages.
 */
package jp.co.epson.ide.eclipse.gnu33.ui.dialogs.linkerscript;

import java.text.MessageFormat;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import jp.co.epson.ide.eclipse.gnu33.Gnu33ProjectDescriptorManager;
import jp.co.epson.ide.eclipse.gnu33.build.BuildSetting;
import jp.co.epson.ide.eclipse.gnu33.build.BuildSettingFactory;
import jp.co.epson.ide.eclipse.gnu33.build.IBuildOption;
import jp.co.epson.ide.eclipse.gnu33.file.LinkerScriptFileGenerator;
import jp.co.epson.ide.eclipse.gnu33.linkerscript.DataPointer;
import jp.co.epson.ide.eclipse.gnu33.linkerscript.LinkerScriptSetting;
import jp.co.epson.ide.eclipse.gnu33.linkerscript.SectionData;
import jp.co.epson.ide.eclipse.gnu33.others.Logger;
import jp.co.epson.ide.eclipse.gnu33.others.ProjectAccessor;
import jp.co.epson.ide.eclipse.gnu33.others.UIUtil;
import jp.co.epson.ide.eclipse.gnu33.ui.dialogs.AbstractGnu33PropertyPage;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IColorProvider;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;

/**
 * The Property Page for a Linker Script Setting in a Gnu33 Project.
 *
 * @author D.Fujimoto
 */
public class LinkerScriptPropertyPage extends AbstractGnu33PropertyPage {

	private static final int BTN_ADD = 1;
	private static final int BTN_EDIT = 2;
	private static final int BTN_DEL = 3;

	
	
	/**
	 * LinkerScriptSetting from the Project Descriptor.
	 */
	private LinkerScriptSetting ldsSetting;
	
	
	/**
	 * Constructor. Called by the framework.
	 */
	public LinkerScriptPropertyPage() {
		super();
	}

	// the controls
	private Label targetFileLabel;
	private TableViewer sectionViewer;
	private Table tableSections;
	private Button buttonAdd;
	private Button buttonEdit;
	private Button buttonDelete;
	private Text textDP;

	// Selection after Add/Edit/Delete has been performed
	private ISelection refreshedSelection = StructuredSelection.EMPTY; 

	
	/**
	 * Create the UI here.
	 * @return Control the UI controls
	 * @see org.eclipse.jface.preference.PreferencePage#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent) {

		// make the UI here
		Composite container = new Composite(parent, SWT.NONE);
		container.setLayout(new GridLayout());

		// label showing target file
		targetFileLabel = new Label(container, SWT.WRAP);
		GridData gdTargetLabel = new GridData();
		gdTargetLabel.grabExcessHorizontalSpace = true;		// for big fonts
		targetFileLabel.setLayoutData(gdTargetLabel);
		
		// copied
		Composite composite = new Composite(container, SWT.NONE);
		GridData gridComp = new GridData(GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL | GridData.VERTICAL_ALIGN_BEGINNING);
		composite.setLayoutData(gridComp);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		composite.setLayout(gridLayout);


		//***********************
		// 	Section Table Area
		//***********************
		tableSections = new Table(composite, SWT.SINGLE | SWT.BORDER | SWT.FULL_SELECTION);
		GridData gridSecTable = new GridData();
		gridSecTable.heightHint = 210;
		gridSecTable.widthHint = 470;
		tableSections.setLayoutData(gridSecTable);
		tableSections.setLinesVisible(true);
		tableSections.setHeaderVisible(true);

		// VMA column
		TableColumn colVMA = new TableColumn(tableSections, SWT.NONE);
		colVMA.setWidth(100);
		colVMA.setText(LinkerScriptSettingDialogResources.getString("table.column.vma"));	//$NON-NLS-1$
		colVMA.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				sectionViewer.setSorter(new LinkerScriptSectionsSorter(LinkerScriptSectionsSorter.VMA));
			}
		});
		
		
		// LMA column
		TableColumn columnLMA = new TableColumn(tableSections, SWT.NONE);
		columnLMA.setWidth(100);
		columnLMA.setText(LinkerScriptSettingDialogResources.getString("table.column.lma"));	//$NON-NLS-1$
		columnLMA.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				sectionViewer.setSorter(new LinkerScriptSectionsSorter(LinkerScriptSectionsSorter.LMA));
			}
		});

		// section name column
		TableColumn colSection = new TableColumn(tableSections, SWT.NONE);
		colSection.setWidth(100);
		colSection.setText(LinkerScriptSettingDialogResources.getString("table.column.sectionName"));	//$NON-NLS-1$

		// label column
		TableColumn colLabels = new TableColumn(tableSections, SWT.NONE);
		colLabels.setWidth(175);
		colLabels.setText(LinkerScriptSettingDialogResources.getString("table.column.labels"));	//$NON-NLS-1$

		sectionViewer = new TableViewer(tableSections);
		sectionViewer.setContentProvider(new ArrayContentProvider());
		sectionViewer.setLabelProvider(new LinkerScriptSettingLabelProvider());
		sectionViewer.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				// do not get the selection here otherwise the refreshedSelection
				// routine in populateTable() will have some problems

				enableButtons(true);
			}
		});
		sectionViewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				// open the edit dialog
				handleButtonPressed(BTN_EDIT);
			}
		});
		sectionViewer.setSorter(new LinkerScriptSectionsSorter(LinkerScriptSectionsSorter.VMA));
		
		
		//***********************
		// 	Button Area
		//***********************
		Composite compButtons = new Composite(composite, SWT.NONE);
		compButtons.setLayoutData(new GridData(GridData.VERTICAL_ALIGN_BEGINNING | GridData.GRAB_HORIZONTAL));
		compButtons.setLayout(new GridLayout());

		// add button
		buttonAdd = new Button(compButtons, SWT.NONE);
		buttonAdd.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
		buttonAdd.setText(LinkerScriptSettingDialogResources.getString("button.addSection.label"));	//$NON-NLS-1$
		buttonAdd.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleButtonPressed(BTN_ADD);
			}
		});

		// edit button
		buttonEdit = new Button(compButtons, SWT.NONE);
		buttonEdit.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
		buttonEdit.setText(LinkerScriptSettingDialogResources.getString("button.editSection.label"));	//$NON-NLS-1$
		buttonEdit.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleButtonPressed(BTN_EDIT);
			}
		});

		// delete button
		buttonDelete = new Button(compButtons, SWT.NONE);
		buttonDelete.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
		buttonDelete.setText(LinkerScriptSettingDialogResources.getString("button.deleteSection.label"));	//$NON-NLS-1$
		buttonDelete.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleButtonPressed(BTN_DEL);
			}
		});


		//***********************
		// 	Data Pointer Area
		//***********************
		int textWidth = 75;
		Composite compDps = new Composite(composite, SWT.NONE);
		GridData gridCompDps = new GridData();
		gridCompDps.heightHint = 50;
		gridCompDps.widthHint = 400;
		compDps.setLayoutData(gridCompDps);
		GridLayout layoutCompDps = new GridLayout();
		layoutCompDps.numColumns = 2;
		compDps.setLayout(layoutCompDps);

		Label dpLabel = new Label(compDps, SWT.NONE);
		dpLabel.setText(LinkerScriptSettingDialogResources.getString("dp.address.label"));	//$NON-NLS-1$

		textDP = new Text(compDps, SWT.BORDER);
		GridData gdDP = new GridData();
		gdDP.widthHint = textWidth;
		textDP.setLayoutData(gdDP);
		textDP.setTextLimit(UIUtil.getAddressMaxChars());
		textDP.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				UIUtil.enableHexInput(e);
			}
		});
		textDP.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				verifyInputs();
			}
		});
		// copied


	  	// default things to do
		resetToDefaults();

		return container;
	}


	/**
	 * Get the LinkerScriptSettings from the Project Description file.
	 */
	public void getProjectSettings() {
		super.getProjectSettings();
		ldsSetting = descManager.getLdsSetting();
		
	}

	/**
	 * Refreshes the whole displays of this page.
	 * @see jp.co.epson.ide.eclipse.gnu33.ui.dialogs.AbstractGnu33PropertyPage#refreshPage()
	 */
	public void refreshPage() {

		// reset the selection
		refreshedSelection = StructuredSelection.EMPTY;

		// fill the section table
		populateTable();
		
		// fill data pointers
		populateDps();

		verifyInputs();

		// label showing target file
		String targetFile = MessageFormat.format(LinkerScriptSettingDialogResources.getString("linkerscript.targetFile.label"), 
				new Object[] {ldsSetting.getSettingName() + LinkerScriptFileGenerator.EXT_LNK});
    	targetFileLabel.setText(targetFile);

	}

	/**
	 * Populates the settings Table with the LinkerScriptSetting object.
	 *
	 */
	private void populateTable() {

		// for LinkerScriptSettingContentProvider use(eats LinkerScriptSetting and
		// returns SectionData)
		// sectionViewer.setInput(ldsSetting);

		// auto update the sections
		// get the assumed .o files in this project
		ProjectAccessor util = new ProjectAccessor(project);
		List objFiles = util.getObjectListForSection();
		ldsSetting.updateAllObjList(objFiles);

	
		// for ArrayContentProvider use(eats List of SectionData and returns SectionData)
		sectionViewer.setInput(ldsSetting.getSections());
	
	  
		if (tableSections.getItemCount() > 0) {
			// check the selection specified after Add/Edit/Delete
			if (refreshedSelection.isEmpty()) {
				tableSections.select(0);
			} else {
				sectionViewer.setSelection(refreshedSelection, true);
			} 
			
			enableButtons(true);
		} else {
			buttonEdit.setEnabled(false);
			buttonDelete.setEnabled(false);
		}
	}


	/**
	 * Fills in the data pointer addresses.
	 */
	private void populateDps() {
		Map dpMap = ldsSetting.getDataPointers();

		for (Iterator it = dpMap.values().iterator(); it.hasNext(); ) {
			DataPointer dp = (DataPointer) it.next();
			Text text = null;
			switch (dp.getType()) {
				case DataPointer.TYPE_D:
					text = textDP;
					break;

			}
			text.setText(dp.getAddressString());
		}

	}


	/**
	 * Enables/disables all the buttons related to the table.
	 * @param enable
	 */
	private void enableButtons(boolean enable) {
		buttonAdd.setEnabled(enable);
		buttonEdit.setEnabled(enable);
		buttonDelete.setEnabled(enable);
		
		// undisposable standard sections
		IStructuredSelection sel = (IStructuredSelection) sectionViewer.getSelection();
		SectionData sec = (SectionData) sel.getFirstElement();
		if (sec != null && sec.isDefaultSection()) {
			buttonDelete.setEnabled(false);
		}
		
	}

	/**
	 * Handles Add, Edit, Delete button events.
	 * @param buttonID indicates which button was pressed
	 */
	private void handleButtonPressed(int buttonID) {

		try {
			descManager.fetchProjectDescriptor();
			BuildSetting buildSetting = (BuildSetting) descManager.getBuildList().get(0);
			IBuildOption option = buildSetting.getLinker().getOption(BuildSettingFactory.LD_LIB_OPT_ID);
			String[] userLibs = option.getListValue();

			LinkerScriptEditDialog dialog = new LinkerScriptEditDialog(getShell(), ldsSetting, userLibs, project);

			if (buttonID == BTN_ADD) {
				// add
				dialog.setMode(LinkerScriptEditDialog.MODE_ADD);
				dialog.open();
				
				// when successfully added, select the added section in the viewer
				SectionData addedSection = dialog.getEditedSectionData();
				if (addedSection != null) {
					refreshedSelection = new StructuredSelection(addedSection);	
				} else {
					refreshedSelection = sectionViewer.getSelection();
				}
				
			} else if (buttonID == BTN_EDIT) {
				// edit
				dialog.setMode(LinkerScriptEditDialog.MODE_EDIT);
				IStructuredSelection sel = (IStructuredSelection) sectionViewer.getSelection();
				dialog.setSectionToEdit((SectionData) sel.getFirstElement());
				dialog.open();
				
				// when successfully edited, the select the edited section in the viewer
				SectionData editedSection = dialog.getEditedSectionData();
				if (editedSection != null) {
					refreshedSelection = new StructuredSelection(editedSection);	
				} else {
					refreshedSelection = sel;
				}				
				
			} else if (buttonID == BTN_DEL) {
				// delete
				boolean doIt = MessageDialog.openConfirm(getShell(),
						LinkerScriptSettingDialogResources.getString("deleteSection.dlg.title"),	//$NON-NLS-1$
						LinkerScriptSettingDialogResources.getString("deleteSection.dlg.msg"));	//$NON-NLS-1$
				
				IStructuredSelection sel = (IStructuredSelection) sectionViewer.getSelection();
				
				if (doIt) {
					ldsSetting.removeSection((SectionData) sel.getFirstElement());
					// reset the next selection
					refreshedSelection = StructuredSelection.EMPTY;
				} else {
					// set the next selection
					refreshedSelection = sel;
				}
			}

		} catch (Exception e) {
			Logger.log(e);
			MessageDialog.openError(getShell(), "Error", "Failed to read the descriptor file.");	//$NON-NLS-1$

		}


		// refresh the viewer and its buttons
		populateTable();
		verifyInputs();

	}

	/**
	 * Checks if the data pointer inputs are entered correctly.
	 * Sets an error message if any have errors.
	 * 
	 * Refactored in 1.3.0
	 */
	private void verifyInputs() {
		errorMessage = null;

		// data pointer address checks
		if (textDP.isEnabled()) {
			if (!UIUtil.isHexString(textDP.getText())) {
				errorMessage = LinkerScriptSettingDialogResources.getString("dp.input.invalid");	//$NON-NLS-1$
			}
		}


		setErrorMessage(errorMessage);
		// this will update Apply and OK button state
		setValid(errorMessage == null);

	}


	/**
	 * Gathers the data pointer input values and stores them into the
	 * LinkerScriptSetting object.
	 */
	private void gatherDpInputs() {

		Map dpMap = ldsSetting.getDataPointers();
		DataPointer dp = null;

		dp = (DataPointer) dpMap.get(DataPointer.TYPE_D_STR);

		dp.setAddress(Long.parseLong(textDP.getText(), 16));

	}

	/**
	 * Checks whether it is alright to leave this page.
	 * @return false to abort page flipping, true to permit
	 * @see org.eclipse.jface.preference.IPreferencePage#okToLeave()
	 * @since 1.3.0
	 */
	public boolean okToLeave() {

		boolean canFlip = true;

		// check the vector address
		if (isValid()) {
		} else {
			canFlip = false;
		}
		
		return canFlip;
	}

	/**
	 * Handles ok button events.
	 */
	public boolean performOk() {

		// write the settings to the description file
		// (settings only from this page)

		verifyInputs();
		if (!isValid()) {
			return false;
		}

		// collect the data pointer inputs
		gatherDpInputs();

		try {
			descManager.updateLinkerScriptSetting(ldsSetting);

/* >>>>> DELETED by D.Fujimoto 2007/08/13 do not refresh other pages */	
			// refreshing will discard modification made other pages.  
//			// refresh other Property Pages(that are related to Gnu33 project)
//			refreshAllPages();
/* <<<<< DELETED by D.Fujimoto 2007/08/13 do not refresh other pages */
			
		} catch (CoreException e) {
/* >>>>> MODIFIED by D.Fujimoto 2007/08/03 display error */
			String title = MSG_UPDATE_FAILED + Gnu33ProjectDescriptorManager.getDescriptorFileName();
			String message = e.getLocalizedMessage();
			Logger.log(message);
			displayErrorDialog(title, message);
/* <<<<< MODIFIED by D.Fujimoto 2007/08/03 display error */
			return false;
		}

		return super.performOk();
	}


	/**
	 * The LabelProvider for the LinkerScriptSetting TableViewer.
	 * @author D.Fujimoto
	 * @since 1.3.0
	 *
	 */
	private class LinkerScriptSettingLabelProvider extends LabelProvider implements ITableLabelProvider, IColorProvider {

		// default color for default sections
		private static final int DEFAULT_SECTION_COLOR = SWT.COLOR_BLUE;
		
		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object, int)
		 */
		public Image getColumnImage(Object element, int columnIndex) {
			return null;
		}

		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object, int)
		 */
		public String getColumnText(Object element, int columnIndex) {
			String ret = "";

			SectionData section = (SectionData) element;

			switch (columnIndex) {
				case 0 : // VMA
					if (section.isVmaIsAddress()) {
						ret = UIUtil.getAddressString(section.getVma());
					} else {
						// OɂZNV(@ˑs)
						ret = LinkerScriptSettingDialogResources.getString("table.cell.vma.equal");
					}
					break;
				case 1 : // LMA
					if (section.isLMAequalsVMA()) {
						// no LMA
						ret = LinkerScriptSettingDialogResources.getString("table.cell.noLma");
					} else {
						if (section.isLmaIsAddress()) {
							ret = UIUtil.getAddressString(section.getLma());
						} else {
							ret = LinkerScriptSettingDialogResources.getString("table.cell.lmaRef") + "(" +  section.getLmaSectionName() + ")";
						}
					}
					break;
				case 2 : // Section name
					ret = section.getSectionName();
					break;
				case 3 : // Labels
					String[] labels = section.createLabels();
					ret = labels[0] + ":" + labels[1];
					if (labels[2] != null) {
						ret += ":" + labels[2] + ":" + labels[3];
					}
					break;
			}

			return ret;
		}

		/* ( Javadoc)
		 * Sets the color for default(standard) section color  
		 * 
		 * @see org.eclipse.jface.viewers.IColorProvider#getForeground(java.lang.Object)
		 */
		public Color getForeground(Object element) {
			SectionData section = (SectionData) element;
			
			if (section.isDefaultSection()) {
				return Display.getCurrent().getSystemColor(DEFAULT_SECTION_COLOR);	
			}
			return null;
		}

		/* ( Javadoc)
		 * @see org.eclipse.jface.viewers.IColorProvider#getBackground(java.lang.Object)
		 */
		public Color getBackground(Object element) {

			return null;
		}

	}

	/**
	 * The Sorter for the LinkerScriptSetting TableViewer.
	 * <Under Construction>
	 * @author D.Fujimoto
	 *
	 */
	private class LinkerScriptSectionsSorter extends ViewerSorter {
		
		// sort types
		public static final int VMA = 1;
		public static final int LMA = 2;
		
		private int sortType;	
		
		public LinkerScriptSectionsSorter(int sortType) {
			this.sortType = sortType;
		}
	
		/*
		 *	( Javadoc)
		 * @see org.eclipse.jface.viewers.ViewerSorter#compare(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
		 */
		public int compare(Viewer viewer, Object e1, Object e2) {
			
			List secList = (List) viewer.getInput();
			
			// sort depends on type and Comparator
			switch (sortType) {
				case VMA :	// sort by VMA
//					return new VmaSectionComparator(secList).compare(e1, e2);
				case LMA : // sort by LMA
//					return new LmaSectionComparator(secList).compare(e1, e2);
			}
			
			return 0;
			
		}
	}
   
}
