/*
 * Created on 2004/08/04
 * Modified on 2005/07/28 by D.Fujimoto in createContents() the par name field made uneditable.
 * Modified on 2005/11/14 by D.Fujimoto added TableViewer
 * 										modified createContents(), refreshPage(), populateTable(), handleButtonPressed()
 * 										added ParameterSettingsLabelProvider.
 * Modified on 2006/06/29 by D.Fujimoto modified refreshPage() to show target file and
 * 											deleted textParName field.
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 * Modified on 2007/08/03 by D.Fujimoto modified performOk() to show error when pj file is read only.
 */
package jp.co.epson.ide.eclipse.gnu33.ui.dialogs.parameter;

import java.text.MessageFormat;
import java.util.List;
import java.util.StringTokenizer;

import jp.co.epson.ide.eclipse.gnu33.Gnu33ProjectDescriptorManager;
import jp.co.epson.ide.eclipse.gnu33.file.ParameterFileGenerator;
import jp.co.epson.ide.eclipse.gnu33.others.Logger;
import jp.co.epson.ide.eclipse.gnu33.others.UIUtil;
import jp.co.epson.ide.eclipse.gnu33.parameter.ParameterArea;
import jp.co.epson.ide.eclipse.gnu33.parameter.ParameterSetting;
import jp.co.epson.ide.eclipse.gnu33.ui.dialogs.AbstractGnu33PropertyPage;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;

/**
 * The Property Page for a Parameter Setting in a Gnu33 Project.
 *
 * @author D.Fujimoto
 */
public class ParameterSettingsPropertyPage extends AbstractGnu33PropertyPage {

	public static final String MSG_INVALID_COMMENT = ParameterSettingDialogResources.getString("comment.input.invalid");
	
	private static final int BTN_ADD = 1;
	private static final int BTN_EDIT = 2;
	private static final int BTN_DEL = 3;

	/**
	 * List of ParameterSetting from the Project Descriptor.
	 */
	private List parSettingList;

	/**
	 * The selected index of the Table.
	 */
	private int selectedIndex = -1;

	private ISelection refreshedSelection = StructuredSelection.EMPTY;

	// the controls
	private Label targetFileLabel;
	private Text textParComment;
	private TableViewer parViewer;
	private Table tableParMap;
	private Button buttonAdd;
	private Button buttonEdit;
	private Button buttonDelete;

	/**
	 * Constructor. Called by the framework.
	 */
	public ParameterSettingsPropertyPage() {
		super();
	}

	/**
	 * Create the UI here.
	 * @return Control - the UI controls
	 * @see org.eclipse.jface.preference.PreferencePage#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent) {

		Composite container = new Composite(parent, SWT.NONE);

		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		container.setLayout(layout);
		GridData data = new GridData();
		data.verticalAlignment = GridData.FILL;
		data.horizontalAlignment = GridData.FILL;
		container.setLayoutData(data);

		// label showing target file
		targetFileLabel = new Label(container, SWT.WRAP);
		GridData gdTargetLabel = new GridData();
		gdTargetLabel.horizontalSpan = 2;
		gdTargetLabel.grabExcessHorizontalSpace = true;		// for big fonts
		targetFileLabel.setLayoutData(gdTargetLabel);

		//************************
		//	Parameter Header Area
		//************************
		{
			Composite parHeadBlock = new Composite(container, SWT.NONE);
			GridData gridData = new GridData();
			gridData.heightHint = 86;
			gridData.widthHint = 360;
			gridData.grabExcessHorizontalSpace = true;
			parHeadBlock.setLayoutData(gridData);
			GridLayout gridLayout = new GridLayout();
			gridLayout.numColumns = 3;
			parHeadBlock.setLayout(gridLayout);

			// Comment
			Label labelParComment = new Label(parHeadBlock, SWT.NONE);
			labelParComment.setText(ParameterSettingDialogResources.getString("parPage.comment.label"));

			textParComment = new Text(parHeadBlock, SWT.BORDER | SWT.MULTI
					| SWT.V_SCROLL | SWT.WRAP);
			textParComment.setTextLimit(255);
			GridData gridTextParComment = new GridData(
					GridData.HORIZONTAL_ALIGN_FILL);
			gridTextParComment.horizontalSpan = 2;
			gridTextParComment.heightHint = 50;
			gridTextParComment.widthHint = 250;
			gridTextParComment.grabExcessHorizontalSpace = true;
			textParComment.setLayoutData(gridTextParComment);
			textParComment.addKeyListener(new KeyAdapter() {
				public void keyPressed(KeyEvent e) {
					UIUtil.enableAsciiInput(e);
				}
			});
			textParComment.addModifyListener(new ModifyListener() {
				public void modifyText(ModifyEvent e) {
					verifyInputs();
				}
			});
			
		}

		//************************
		//	Button Area
		//************************
		{
			Composite parButtonBlock = new Composite(container, SWT.NONE);
			GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.VERTICAL_ALIGN_FILL);
			gridData.heightHint = 279;
			gridData.verticalSpan = 2;
			gridData.widthHint = 97;
			parButtonBlock.setLayoutData(gridData);
			parButtonBlock.setLayout(new GridLayout());

			buttonAdd = new Button(parButtonBlock, SWT.PUSH);
			GridData gridData_1 = new GridData(GridData.FILL_HORIZONTAL);
			gridData_1.widthHint = 42;
			buttonAdd.setLayoutData(gridData_1);
			buttonAdd.setText(ParameterSettingDialogResources.getString("parPage.addButton.label"));
			buttonAdd.setEnabled(true);
			buttonAdd.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					handleButtonPressed(BTN_ADD);
				}
			});

			buttonEdit = new Button(parButtonBlock, SWT.PUSH);
			buttonEdit.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			buttonEdit.setText(ParameterSettingDialogResources.getString("parPage.editButton.label"));
			buttonEdit.setEnabled(false);
			buttonEdit.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					handleButtonPressed(BTN_EDIT);
				}
			});

			buttonDelete = new Button(parButtonBlock, SWT.PUSH);
			buttonDelete.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			buttonDelete.setText(ParameterSettingDialogResources.getString("parPage.deleteButton.label"));
			buttonDelete.setEnabled(false);
			buttonDelete.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					handleButtonPressed(BTN_DEL);
				}
			});

		}

		//************************
		//	Parameter Map Area
		//************************
		{
			Composite parMapBlock = new Composite(container, SWT.NONE);
			parMapBlock.setLayout(new GridLayout());
			GridData gridData_1 = new GridData(
					GridData.HORIZONTAL_ALIGN_FILL
							| GridData.VERTICAL_ALIGN_FILL);
			parMapBlock.setLayoutData(gridData_1);

			tableParMap = new Table(parMapBlock, SWT.BORDER | SWT.FULL_SELECTION);
			GridData gridData = new GridData();
			gridData.widthHint = 360;
			gridData.heightHint = 210;
			tableParMap.setLayoutData(gridData);
			tableParMap.setLinesVisible(true);
			tableParMap.setHeaderVisible(true);
			tableParMap.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					selectedIndex = tableParMap.getSelectionIndex();
					enableButtons(true);
				}

			});


			TableColumn columnType = new TableColumn(tableParMap, SWT.DEFAULT);
			columnType.setWidth(50);
			columnType.setText(ParameterSettingDialogResources.getString("parPage.table.column.type"));

			TableColumn columnAddress = new TableColumn(tableParMap, SWT.DEFAULT);
			columnAddress.setWidth(125);
			columnAddress.setText(ParameterSettingDialogResources.getString("parPage.table.column.address"));

			TableColumn columnWait = new TableColumn(tableParMap, SWT.DEFAULT);
			columnWait.setWidth(100);
			columnWait.setText(ParameterSettingDialogResources.getString("parPage.table.column.waitRW"));

			TableColumn columnComment = new TableColumn(tableParMap, SWT.DEFAULT);
			columnComment.setWidth(90);
			columnComment.setText(ParameterSettingDialogResources.getString("parPage.table.column.comments"));

			
			// create the TableViewer
			parViewer = new TableViewer(tableParMap);
			parViewer.setContentProvider(new ArrayContentProvider());
			parViewer.setLabelProvider(new ParameterSettingsLabelProvider());
			parViewer.addDoubleClickListener(new IDoubleClickListener() {
				public void doubleClick(DoubleClickEvent event) {
					// open the edit dialog
					handleButtonPressed(BTN_EDIT);
				}
			});
			
		}
		// ܂

		resetToDefaults();

		return container;
	}

	/**
	 * Get the ParameterSettings from the Project Description file
	 * and set it to parSettingList.
	 */
	public void getProjectSettings() {
		super.getProjectSettings();
		parSettingList = descManager.getParameterList();

	}

	/**
	 * Refreshes the whole displays of this page.
	 */
	public void refreshPage() {
		
		// reset the selection
		refreshedSelection = StructuredSelection.EMPTY;
		
		ParameterSetting setting = (ParameterSetting) parSettingList.get(0);
		textParComment.setText(setting.getComment());
		populateTable(setting);
		
		
		// label showing target file
		String targetFile = MessageFormat.format(ParameterSettingDialogResources.getString("parameter.targetFile.label"), 
				new Object[] {setting.getName() + ParameterFileGenerator.EXT_PAR});
		targetFileLabel.setText(targetFile);

	}

	/**
	 * Populates the settings Table with the given ParameterSetting object.
	 */
	private void populateTable(ParameterSetting setting) {

		List areaList = setting.getAreaList();

		parViewer.setInput(areaList);
	
		if (tableParMap.getItemCount() > 0) {
			// check the selection specified after Add/Edit/Delete
			if (refreshedSelection.isEmpty()) {
				tableParMap.select(0);	
			} else {
				parViewer.setSelection(refreshedSelection, true);
			}		 	
			
			enableButtons(true);
		} else {
			// disable buttons when no items are in the table
			buttonAdd.setEnabled(true);
			buttonEdit.setEnabled(false);
			buttonDelete.setEnabled(false);
		}

		selectedIndex = tableParMap.getSelectionIndex();

	}
	
	/**
	 * Enables/disables all the buttons related to the table.
	 * @param enable
	 */
	private void enableButtons(boolean enable) {
		buttonAdd.setEnabled(enable);
		buttonEdit.setEnabled(enable);
		buttonDelete.setEnabled(enable);

	}

	/**
	 * Checks if the inputs in this composite are entered correctly.
	 */
	private void verifyInputs() {
		errorMessage = null;
		
		// comment
		if (!UIUtil.isAscii(textParComment.getText())) {
			errorMessage = MSG_INVALID_COMMENT;
		}
		
		setErrorMessage(errorMessage);
		// this will update Apply and OK button state
		setValid(errorMessage == null);
	}
	
	/**
	 * Triggered by pressing Add/Edit/Delete buttons.
	 * @param flag indicates which button was pressed
	 */
	private void handleButtonPressed(int flag) {

		ParameterSetting setting = (ParameterSetting) parSettingList.get(0);
		ParameterEditDialog dialog = new ParameterEditDialog(getShell(), setting);

		if (flag == 1) {
			// add
			dialog.setMode(ParameterEditDialog.MODE_ADD);
			dialog.open();

			// when successfully added, select the added area in the viewer
			ParameterArea edited = dialog.getEditedArea();
			if (edited != null) {
				refreshedSelection = new StructuredSelection(edited);
			} else {
				refreshedSelection = parViewer.getSelection();
			}
			
			
		} else if (flag == 2) {
			// edit
			if (selectedIndex < 0) {
				return;
			}

			dialog.setMode(ParameterEditDialog.MODE_EDIT);
			dialog.setEditIndex(selectedIndex);
			dialog.open();
			
			// when successfully added, select the added area in the viewer
			ParameterArea edited = dialog.getEditedArea();
			if (edited != null) {
				refreshedSelection = new StructuredSelection(edited);
			} else {
				refreshedSelection = parViewer.getSelection();
			}
			
		} else if (flag == 3) {
			// delete
			if (selectedIndex < 0) {
				return;
			}
			
			IStructuredSelection sel = (IStructuredSelection) parViewer.getSelection();

			boolean doIt = MessageDialog.openConfirm(getShell(),
					ParameterSettingDialogResources.getString("deleteArea.dlg.title"),
					ParameterSettingDialogResources.getString("deleteArea.dlg.msg"));
			if (doIt) {
				setting.removeArea(selectedIndex);
				refreshedSelection = StructuredSelection.EMPTY;
			} else {
				// set the next selection
				refreshedSelection = sel;
			}
		}

		// refresh the table
		populateTable(setting);

	}

	/**
	 * Handles ok button events.
	 */
	public boolean performOk() {

		// write the settings to the description file
		// (the settings only from this page)
		if (!isValid()) {
			return false;
		}

		ParameterSetting setting = (ParameterSetting) parSettingList.get(0);
		setting.setComment(textParComment.getText());
		try {
			descManager.updateParameterSetting(parSettingList);
		} catch (CoreException e) {
/* >>>>> MODIFIED by D.Fujimoto 2007/08/03 display error */
			String title = MSG_UPDATE_FAILED + Gnu33ProjectDescriptorManager.getDescriptorFileName();
			String message = e.getLocalizedMessage();
			Logger.log(message);
			displayErrorDialog(title, message);
/* <<<<< MODIFIED by D.Fujimoto 2007/08/03 display error */
			return false;
		}

		return super.performOk();
	}

	
	/**
	 * The LabelProvider for the ParameterSettings TableViewer
	 * @author D.Fujimoto
	 * @since 1.2.0
	 *
	 */
	private class ParameterSettingsLabelProvider extends LabelProvider implements ITableLabelProvider {

		/* ( Javadoc)
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object, int)
		 */
		public Image getColumnImage(Object element, int columnIndex) {
			return null;
		}

		/* ( Javadoc)
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object, int)
		 */
		public String getColumnText(Object element, int columnIndex) {
			String ret = "";
			
			ParameterArea area = (ParameterArea) element;
			
			switch (columnIndex) {
				case 0 : //type
					ret = area.getAreaTypeString();
					break;

				case 1 : // addresses
					ret = UIUtil.getAddressString(area.getStartAddress()) +  "-" + UIUtil.getAddressString(area.getEndAddress());
					break;
					
				case 2 : // wait cycle and endian
					if (!area.isStackArea()) {
						ret = area.getWaitCycleString(area.getWaitCycleRead()) + 
							area.getWaitCycleString(area.getWaitCycleWrite()) + " " +
							area.getAccessTypeString() + " " + 
							area.getUseBigEndianString();
					}
					break;
					
				case 3 : // comments
					// replace \t\n\r\f to white spaces
					String comment = area.getComment();
					StringBuffer buf = new StringBuffer();
					StringTokenizer tokenizer = new StringTokenizer(comment, "\t\n\r\f");
					while (tokenizer.hasMoreTokens()) {
						buf.append(tokenizer.nextToken());
						buf.append(" ");
					}
					ret = buf.toString();
					
					break;
			
			}
			return ret;
		}
		
	}

}
