/*
 * Created on 2004/07/07
 * Modified on 2005/03/31 by D.Fujimoto modified doRun()
 * 										addGnu33Descriptor() and
 * 										modifyMakeSettings() and
 * 										configureExternalTool() have been
 * 										refactored and moved to Gnu33ProjectFactory
 * Modified on 2005/08/12 by D.Fujimoto modified addPages() changed first page of wizard.
 * Modified on 2005/11/18 by D.Fujimoto overrided canFinish() to make the Finish button disabled 
 * 											when the first page is open.
 * 										modified doRun() to get the vector section settings.											
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 * Modified on 2007/03/22 by D.Fujimoto delete invalid project in doRun().
 * Modified on 2007/03/22 by D.Fujimoto Finish supressed when CPU is invalid in canFinish().
 */
package jp.co.epson.ide.eclipse.gnu33.ui.wizards;

import jp.co.epson.ide.eclipse.gnu33.Gnu33ProjectFactory;
import jp.co.epson.ide.eclipse.gnu33.cpuinfo.CpuSettingsManager;
import jp.co.epson.ide.eclipse.gnu33.general.GeneralInfo;
import jp.co.epson.ide.eclipse.gnu33.linkerscript.SectionData;
import jp.co.epson.ide.eclipse.gnu33.others.DeleteInvalidProjectJob;

import org.eclipse.cdt.make.internal.ui.MakeUIPlugin;
import org.eclipse.cdt.make.ui.IMakeHelpContextIds;
import org.eclipse.cdt.make.ui.wizards.MakeProjectWizardOptionPage;
import org.eclipse.cdt.make.ui.wizards.NewMakeProjectWizard;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbench;

/**
 * This wizard extends the C/Make Project Wizard. It has an additional Page
 * 
 * @author D.Fujimoto
 *	
 */
public class NewGnu33CDTProjectWizard extends NewMakeProjectWizard {
	
	/*
	 * Wizard window title taken from ResourceBundle
	 */
	private static final String WIZARD_WINDOW_TITLE = "NewWizard.windowTitle";
	
	/*
	 * Wizard title taken from ResourceBundle
	 */
	private static final String WIZARD_TITLE = "NewWizard.title";

	/*
	 * Wizard description taken from ResourceBundle
	 */
	private static final String WIZARD_DESCRIPTION = "NewWizard.description";
	
	private static final String WIZARD_PROJ_ERROR = "NewProject.errorMessage";

	private static final String WZ_SETTINGS_TITLE = "MakeCWizardSettings.title"; //$NON-NLS-1$
	private static final String WZ_SETTINGS_DESC = "MakeCWizardSettings.description"; //$NON-NLS-1$'
	
	/*
	 * the name of the first page
	 */
	private static final String FIRST_PAGE_NAME = "NewGnu33ProjectWizard";
	
	/**
	 * The 2nd page of the wizard.
	 */
	private NewGnu33CDTProjectWizardPage wizPage; 
	
	/**
	 * Constructor
	 * @see org.eclipse.cdt.make.ui.wizards.NewMakeCProjectWizard
	 */
	public NewGnu33CDTProjectWizard() {
		super(NewWizardResources.getString(WIZARD_TITLE), 
				NewWizardResources.getString(WIZARD_DESCRIPTION));
	}

	/**
	 * Sets a new window title.
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench, org.eclipse.jface.viewers.IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection currentSelection) {
		super.init(workbench, currentSelection);
		setWindowTitle(NewWizardResources.getString(WIZARD_WINDOW_TITLE));
	   
	}
	
	
	/**
	 * Sets the title and description to the main page.
	 * This wizard has 3 pages.
	 * The project name page, the GNU33 page, and the CDT Make settings page.
	 * @see org.eclipse.jface.wizard.IWizard#addPages()
	 */
	public void addPages() {

  
		// Add page No.1
		fMainPage = new NewGnu33ProjectWizardFirstPage(FIRST_PAGE_NAME);
		addPage(fMainPage);

		// Set the main page title & description
		fMainPage.setTitle(NewWizardResources.getString(WIZARD_TITLE));
		fMainPage.setDescription(NewWizardResources.getString(WIZARD_DESCRIPTION));
		
		// Add our new page here
		wizPage = new NewGnu33CDTProjectWizardPage();
		addPage(wizPage);
		
		// Lastly add the CDT page with tabs
		addPage(
				fOptionPage =
					new MakeProjectWizardOptionPage(
						MakeUIPlugin.getResourceString(WZ_SETTINGS_TITLE),
						MakeUIPlugin.getResourceString(WZ_SETTINGS_DESC)));
	
		// Set the third page description
		fOptionPage.setDescription(fOptionPage.getDescription() + "\n" + 
				NewWizardResources.getString("NewWizard.optionPageDescription"));
		

	}
	  
	  
	/**
	 * Creates the pages in this Wizard.
	 * Also sets some help contexts to the pages.
	 */
	public void createPageControls(Composite pageContainer) {
 
		IWizardPage [] pages = getPages();
		for (int i = 0; i < pages.length; i++) {
			pages[i].createControl(pageContainer);
		}
		
		if( pages != null && pages.length == 3 ){
			getWorkbench().getHelpSystem().setHelp(pages[0].getControl(), IMakeHelpContextIds.MAKE_PROJ_WIZ_NAME_PAGE);
	
			MakeProjectWizardOptionPage optionPage = (MakeProjectWizardOptionPage) pages[2];
			optionPage.setupHelpContextIds();

		}
	}
	
	/**
	 * Creates the project when OK is pressed.
	 * @see org.eclipse.cdt.ui.wizards.NewCProjectWizard#doRun(org.eclipse.core.runtime.IProgressMonitor)
	 */
	protected void doRun(IProgressMonitor monitor) throws CoreException {
		if (monitor == null) {
			// Create a new ProgressMonitor
			monitor = new NullProgressMonitor();
		}
		
		monitor.beginTask(NewWizardResources.getString("NewWizard.monitorMessage"), 100);
		
		try {
		
			try {
				// Create the C Project
				// super.doRun() does:
				// creates an IProject resource and .project and .cdtproject file
				// then adds a cnature, make nature, scannerconfig nature
				super.doRun(new SubProgressMonitor(monitor, 50));
				monitor.worked(50);
			} catch (CoreException e) {
				
				// project creation message (from createNewProject() of BasicNewProjectResourceWizard)
				ErrorDialog.openError(getShell(), NewWizardResources.getString(WIZARD_PROJ_ERROR),
                        null, // no special message
                        e.getStatus());
				
				throw e;
			}
			
			
			// get the wizard result as GeneralInfo object
			GeneralInfo general = wizPage.getGeneralInfo();
/* >>>>> DELETED by D.Fujimoto 2007/03/15 wizard inputs will be received as GeneralInfo object */
//			int cpuType = wizPage.getSelectedCPU();
//			boolean use = wizPage.isUseHighPrecision();
/* <<<<< DELETED by D.Fujimoto 2007/03/15 wizard inputs will be received as GeneralInfo object */
			
			// get the vector section setting
			SectionData vector = wizPage.getVectorSectionData();

/* >>>>> MODIFIED by D.Fujimoto 2007/03/22 delete invalid projects */
//			// create project with GeneralInfo and vector section object
//			Gnu33ProjectFactory.create(getProjectHandle(), general, vector, monitor);
//			monitor.worked(50);
			
			if (general != null && vector != null) {
				Gnu33ProjectFactory.create(getProjectHandle(), general, vector, monitor);
				monitor.worked(50);
			} else {
				// dispatch a delete job(project has already been created in super.doRun())
				new DeleteInvalidProjectJob(getProjectHandle().getName()).schedule();
			}		
/* <<<<< MODIFIED by D.Fujimoto 2007/03/22 delete invalid projects */
			


		} catch (CoreException e) {
			e.printStackTrace();
		} finally {
			monitor.done();	
		}
		
	}
	
	
	/**
	 * This implementation makes the Finish button disabled when
	 * the wizard is showing the first page(project name and location).
	 * 
	 * @see org.eclipse.jface.wizard.IWizard#canFinish()
	 * @since 1.2.0
	 */
	public boolean canFinish() {
		IWizardPage currentPage = getContainer().getCurrentPage();
		if (currentPage.getName().equals(FIRST_PAGE_NAME)) {
			return false;
		}

/* >>>>> ADDED by D.Fujimoto 2007/03/22 suppress Finish when CPU settings failed */		
		if (!CpuSettingsManager.getManager().isInitialized()) {
			return false;
		}
/* <<<<< ADDED by D.Fujimoto 2007/03/22 suppress Finish when CPU settings failed */
		
		return super.canFinish();
	}
}