/*
 * Created on 2005/08/12
 * Modified on 2005/10/11 by D.Fujimoto added checks for invalid characters to
 * 											validatePage()
 * Modified on 2007/08/13 by D.Fujimoto added checks for spaces in location to
 * 											validatePage()
 */
package jp.co.epson.ide.eclipse.gnu33.ui.wizards;

import jp.co.epson.ide.eclipse.gnu33.ResourceProvider;
import jp.co.epson.ide.eclipse.gnu33.others.UIUtil;

import org.eclipse.cdt.ui.wizards.NewCProjectWizardPage;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;



/**
 * The first page of the wizard.
 * Extends the CProjectWizardPage and adds additional validations for
 * the project name and path.
 * 
 * @author D.Fujimoto
 * @since 1.1.1
 */
public class NewGnu33ProjectWizardFirstPage extends NewCProjectWizardPage {

	// max length of project name
	public static final int MAX_PROJECT_NAME_LENGTH = 100;
	// max length of location path
	public static final int MAX_LOCATION_PATH_LENGTH = 200;
	// max length of state location path
	public static final int MAX_STATE_LOCATION_PATH_LENGTH = 200;
	
	
	/**
	 * @param pageName
	 */
	public NewGnu33ProjectWizardFirstPage(String pageName) {
		super(pageName);

	}

	/**
	 * Do additional checks for the project name and path inputs.
	 * This method should be invoked when the project name of location
	 * field gets modified.
	 * 
	 * @see org.eclipse.ui.dialogs.WizardNewProjectCreationPage#validatePage()
	 */
	protected boolean validatePage() {
		if (super.validatePage() == true) {
			
			String projectName = getProjectName();
			String locationPath = getLocationPath().toOSString();

			// Check the project name length 
			if (projectName.length() > MAX_PROJECT_NAME_LENGTH) {
				setErrorMessage(NewWizardResources.getString("NewWizard.msg.longProjectName"));
				return false;
			}
			
			// Check the project path length
			if (locationPath.length() > MAX_LOCATION_PATH_LENGTH) {
				setErrorMessage(NewWizardResources.getString("NewWizard.msg.longProjectLocation"));
				return false;
			}
			
			// Check the assumed metadata path length
			IPath statePath = ResourcesPlugin.getPlugin().getStateLocation();
			statePath = statePath.append(".projects").append(projectName);
						
			if (statePath.toOSString().length() > MAX_STATE_LOCATION_PATH_LENGTH) {
				setErrorMessage(NewWizardResources.getString("NewWizard.msg.longMetadataLocation"));
				return false;
			}
			
			// the project name should not contain non-aphabets
			if (!UIUtil.isAlphNumString(projectName)) {
				setErrorMessage(ResourceProvider.getResourceString("msg.invalidProjectName"));
				return false;
			}
			
/* >>>>> ADDED by D.Fujimoto 2007/08/13 no spaces in location */ 			
			// the location should not contain spaces
			if (locationPath.indexOf(" ") != -1) {
				setErrorMessage(NewWizardResources.getString("NewWizard.msg.locationContainsBlank"));
				return false;
			}
/* <<<<< ADDED by D.Fujimoto 2007/08/13 no spaces in location */
			
			return true;
		}
		return false;
	}
}
