/*
 * 쐬: 2005/09/21
 *
 */
package jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os;

import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.OSMonitorPlugin;
import gnu33dll.OSMonitorConnection;
import gnu33dll.OSMonitorConnectionException;

/**
 * @author D.Fujimoto
 *
 */
public class OSInfoManager {


	/**
	 * Address symbol where the info is stored.
	 * This symbol should not be over gnu33dll.OSMonitorConnection#ADDRESS_SYMBOL_MAX
	 */
	private static final String ADDRESS_SYMBOL = OSMonitorPlugin.getResourceString("osinfo.addressSymbol");	//$NON-NLS-1$
	
	/**
	 * Number of bytes to read.
	 */
	private static final int READ_BYTES = Integer.parseInt(
			OSMonitorPlugin.getResourceString("osinfo.readBytes"));	//$NON-NLS-1$
	
	
	/**
	 * The OS info model.
	 */
	private IOSInfo osInfo;
	
	/**
	 * Flag indicating that OS has been retreived.
	 */
	private boolean isOsRead;
	
	
	/**
	 * Reads the OS Info from the debugger.
	 * If reading succeeds, the IOSInfo object will be LOADED.
	 * If the the target is not connected, the object will be marked as UNLOADED.
	 * 
	 * @return 0 if success, other if failed(depending on OSMonitorConnection)
	 */
	public int readOSInfo() {
		
		int result = 0;
		OSMonitorConnection conn = null;
		boolean loaded = false;
		char[] readData = new char[READ_BYTES];
		
		try {
			// connect with debugger
			conn = OSMonitorConnection.getConnection();
			
			// do the reading
			result = conn.read(ADDRESS_SYMBOL, 0, readData);
			
			if (result == OSMonitorConnection.RET_OK) {

				// create the info instance
				osInfo = new RTOSInfo(readData);
				loaded = true;
				
			} else if (result == OSMonitorConnection.RET_NO_TARGET_ERR) {
				// not connected with debugger
			} else if (result == OSMonitorConnection.RET_TARGET_RUNNING) {
				// target still running
				// use the last state of OS
				loaded = isOsRead;
			}
			
		} catch (OSMonitorConnectionException e) {
			e.printStackTrace();
			result = e.getCauseFlag();
		} finally {
			conn = null;
		}
		
		isOsRead = loaded;
		
		return result;
	
	}
	
	/**
	 * Returns if the OS info is read.
	 * @return	true if info is read, false otherwise.
	 */
	public boolean isOsRead() {
		return isOsRead;
	}
	
	/**
	 * Access to the IOSInfo.
	 *  
	 * @return OS info object. Returns null if OS cannot be read.
	 */
	public IOSInfo getOSInfo() {
		return osInfo;
	} 

}
