/*
 * 쐬: 2005/09/21
 *
 */
package jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps;

import java.nio.CharBuffer;

import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.DataWithParser;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.OSMonitorPlugin;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.util.DataParser;


/**
 * This class holds the addresses and sizes 
 * of how the kernelObjInfo is stored for Toppers/JSP OS. 
 * 
 * @author D.Fujimoto
 *
 */
public class ToppersObjectTable extends DataWithParser {

	/**
	 * The address where the kernelObjInfo is stored in.
	 * The address should not be over gnu33dll.OSMonitorConnection#ADDRESS_SYMBOL_MAX
	 */
	public static final String ADDRESS_SYMBOL = OSMonitorPlugin.getResourceString("tps.objInfo.addressSymbol"); //$NON-NLS-1$
	
	/**
	 * The kernelObjInfo size to read.
	 * The size should not be over 1024
	 */
	public static final int READ_SIZE = Integer.parseInt(OSMonitorPlugin.getResourceString("tps.objInfo.readSize")); //$NON-NLS-1$
	
	// types of KernelObjectInfo
	// must be identical with the index of kernelObjInfo
	public static final int TYPE_INTERRUPT_NEST 	= 0;
	public static final int TYPE_DISPATCH_FLAG 	= 1;
	public static final int TYPE_RUN_TASK_ADDR 	= 2;
	public static final int TYPE_RDY_TASK_ADDR 	= 3;
	public static final int TYPE_SYSTIME 		= 4;
	public static final int TYPE_READY_QUEUE 	= 5;
	public static final int TYPE_TASK_CTRL 		= 6;
	public static final int TYPE_TASK_INIT 		= 7;
	public static final int TYPE_CYC_CTRL		= 8;
	public static final int TYPE_CYC_INIT 		= 9;
	public static final int TYPE_INTERRUPT_INIT 	= 10;
	public static final int TYPE_EXCEPTION_INIT 	= 11;
	public static final int TYPE_MAILBOX_CTRL 	= 12;
	public static final int TYPE_MAILBOX_INIT 	= 13;
	public static final int TYPE_SEM_CTRL 		= 14;
	public static final int TYPE_SEM_INIT 		= 15;
	public static final int TYPE_EVENT_CTRL 		= 16;
	public static final int TYPE_EVENT_INIT 		= 17;
	public static final int TYPE_DATA_QUEUE_CTRL	= 18;
	public static final int TYPE_DATA_QUEUE_INIT	= 19;
	public static final int TYPE_FIX_MEM_CTRL 	= 20;
	public static final int TYPE_FIX_MEM_INIT 	= 21;
	
	
	/**
	 * The actual table which contains the address and size.
	 */
	private KernelObjectInfo[] kernelObjInfo;
	
	
	/**
	 * Constructor.
	 *
	 */
	public ToppersObjectTable() {
		super();
		
		// create kernelObjInfo table
		kernelObjInfo = new KernelObjectInfo[22];
		for (int i = 0; i < kernelObjInfo.length; i++) {
			kernelObjInfo[i] = new KernelObjectInfo();

			// index and type should be identical to the TYPE_XXX constants 
			kernelObjInfo[i].type = i;
		}
		
	}

	/**
	 * Sets the KernelObjectInfo array's fields from the 
	 * given array.
	 *  
	 * The operations in this method should be ordered properly.
	 * 
	 * @param parseType	the parse type switch(not used)
	 * @param readData		the data expressed as an array
	 */
	public void parseData(int parseType, char[] readData) {
		
		CharBuffer buf = CharBuffer.wrap(readData);
		
		// buffer to get from the CharBuffer
		char[] data = null;
	
		for (int i = 0; i < kernelObjInfo.length; i++) {
		
			// parse the size
			data = new char[KernelObjectInfo.SIZE_LENGTH];
			buf.get(data);
			kernelObjInfo[i].dataSize = (int) DataParser.parse(data);
			
			// parse the count
			data = new char[KernelObjectInfo.COUNT_LENGTH];
			buf.get(data);
			kernelObjInfo[i].dataCount = (int) DataParser.parse(data);
			
			// parse the address
			data = new char[KernelObjectInfo.ADDR_LENGTH];
			buf.get(data);
			kernelObjInfo[i].dataAddress = DataParser.parse(data);
			
		}

	}
	
	/**
	 * Access to the object info table
	 * @return	KernelObjectInfo[]
	 */
	public KernelObjectInfo[] getObjectInfoTable() {
		return kernelObjInfo;
	}
	

}
