/*
 * 쐬: 2005/09/22
 *
 */
package jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps;

import java.nio.CharBuffer;

import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.DataWithParser;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.OSMonitorPlugin;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.util.DataParser;


/**
 * This class holds the data for the system info
 * for Toppers/JSP OS.
 * 
 * @author D.Fujimoto
 *
 */
public class ToppersSystemInfo extends DataWithParser {
	
	/**
	 * The address where the system info is stored in.
	 * The symbol should not be over gnu33dll.OSMonitorConnection#ADDRESS_SYMBOL_MAX
	 */
	public static final String ADDRESS_SYMBOL = OSMonitorPlugin.getResourceString("tps.sysInfo.addressSymbol"); //$NON-NLS-1$
	
	/**
	 * The system info size to read.
	 * The size should not be over 1024
	 */
	public static final int READ_SIZE = Integer.parseInt(OSMonitorPlugin.getResourceString("tps.sysInfo.readSize")); //$NON-NLS-1$
	
	// sizes
	private static final int DATA_LENGTH = 2;
	
	private int minPriority;
	
	private int maxPriority;
	
	private long timeTick1;
	
	private long timeTick2;
	
	private long sysTimeUpdateCycle;
	
	private int cpuLockLevel;
	

	/**
	 * Constructor.
	 *
	 */
	public ToppersSystemInfo() {
		super();
	}
	
	/**
	 * Sets the SystemInfo's fields. 
	 * 
	 * The operations in this method should be ordered properly.
	 * @param parseType	the parse type switch(not used)
	 * @param readData		the kernelObjInfo array
	 */
	public void parseData(int parseType, char[] readData) {
		
		CharBuffer buf = CharBuffer.wrap(readData);
		
		// buffer to get from the CharBuffer
		char[] data = null;
		
		
		// parse the minPri
		data = new char[DATA_LENGTH];
		buf.get(data);
		minPriority = (int) DataParser.parse(data);
		
		// parse the maxPri
		buf.get(data);
		maxPriority = (int) DataParser.parse(data);
		
		// parse the timeTick1
		buf.get(data);
		timeTick1 = DataParser.parse(data);
		
		// parse the timeTick2
		buf.get(data);
		timeTick2 = DataParser.parse(data);
		
		// parse the updateCycle
		buf.get(data);
		sysTimeUpdateCycle = DataParser.parse(data);
		
		// parse the lockLevel
		buf.get(data);
		cpuLockLevel = (int) DataParser.parse(data);
		
	}
	
	
	public int getCpuLockLevel() {
		return cpuLockLevel;
	}
	public int getMaxPriority() {
		return maxPriority;
	}
	public int getMinPriority() {
		return minPriority;
	}
	public long getSysTimeUpdateCycle() {
		return sysTimeUpdateCycle;
	}
	public long getTimeTick1() {
		return timeTick1;
	}
	public long getTimeTick2() {
		return timeTick2;
	}
}
