/*
 * 쐬: 2005/06/14
 *
 */
package jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input;

import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.AbstractDataModel;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.KernelObjectInfo;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.ToppersObjectTable;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.CyclicHandlerData;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.DataQueueData;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.EventFlagData;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.ExceptionHandlerData;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.FixedMemoryPoolData;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.InterruptHandlerData;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.MailBoxData;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.ReadyQueueData;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.SemaphoreData;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.TaskData;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.TaskManagementData;

/**
 * Contains all the data(in pieces) for the Monitor.
 * 
 * @author D.Fujimoto
 *
 */
public class ToppersMonitorInput implements IToppersMonitorInput {
	
	// mng(only one)
	private TaskManagementData taskMng;
	
	// rdy
	private ReadyQueueData[] readyQueue;
	
	// tsk
	private TaskData[] taskCntl;
	
	// cyc
	private CyclicHandlerData[] cycHandler;
	
	// mbx
	private MailBoxData[] mailBox;
	
	// sem
	private SemaphoreData[] semaphore;
	
	// flg
	private EventFlagData[] eventFlag;
	
	// dtq
	private DataQueueData[] dataQueue;
	
	// mpf
	private FixedMemoryPoolData[] fixedMemPool;
	
	// interrupt handler
	private InterruptHandlerData[] intHandler;
	
	// exception handler
	private ExceptionHandlerData[] excHandler;
	
	/**
	 * minimum priority value of system
	 * origin=1
	 */
	private int minPriority;
	
	/**
	 * Empty constructor only for internal use.
	 */
	public ToppersMonitorInput() {
		super();
	}
	
	/**
	 * Create the ingredients of this input using the 
	 * KernelObjectInfo[] object.<br>
	 * The array size of each data model objects created herein will not
	 * change for the duration of the debugging session.
	 *  
	 * @param objInfo		the info that contains the size for each data model
	 */
	public ToppersMonitorInput(KernelObjectInfo[] objInfo) {
		
		int dataCount = 0;
		
		// task
		taskMng = new TaskManagementData();
		
		// ready queue
		dataCount = objInfo[ToppersObjectTable.TYPE_READY_QUEUE].dataCount;
		readyQueue = new ReadyQueueData[dataCount];
		for (int i = 0; i < readyQueue.length; i++) {
			readyQueue[i] = new ReadyQueueData(i + 1);
		}
		
		// task
		dataCount = objInfo[ToppersObjectTable.TYPE_TASK_CTRL].dataCount;
		taskCntl = new TaskData[dataCount];
		for (int i = 0; i < taskCntl.length; i++) {
			taskCntl[i] = new TaskData(i + 1); 
		}
		
		
		// cyclic handler
		dataCount = objInfo[ToppersObjectTable.TYPE_CYC_CTRL].dataCount;
		cycHandler = new CyclicHandlerData[dataCount];
		for (int i = 0; i < cycHandler.length; i++) {
			cycHandler[i] = new CyclicHandlerData(i + 1);
		}
		
		// interrupt handler
		dataCount = objInfo[ToppersObjectTable.TYPE_INTERRUPT_INIT].dataCount;
		intHandler = new InterruptHandlerData[dataCount];
		for (int i = 0; i < intHandler.length; i++) {
			intHandler[i] = new InterruptHandlerData(i + 1); 
		}
		
		// exception handler
		dataCount = objInfo[ToppersObjectTable.TYPE_EXCEPTION_INIT].dataCount;
		excHandler = new ExceptionHandlerData[dataCount];
		for (int i = 0; i < excHandler.length; i++) {
			excHandler[i] = new ExceptionHandlerData(i + 1);
		}
		
		// mail box
		dataCount = objInfo[ToppersObjectTable.TYPE_MAILBOX_CTRL].dataCount;
		mailBox = new MailBoxData[dataCount];
		for (int i = 0; i < mailBox.length; i++) {
			mailBox[i] = new MailBoxData(i + 1);
		}
		
		// semaphore
		dataCount = objInfo[ToppersObjectTable.TYPE_SEM_CTRL].dataCount;
		semaphore = new SemaphoreData[dataCount];
		for (int i = 0; i < semaphore.length; i++) {
			semaphore[i] = new SemaphoreData(i + 1);
		}
		
		// event flag
		dataCount = objInfo[ToppersObjectTable.TYPE_EVENT_CTRL].dataCount;
		eventFlag = new EventFlagData[dataCount];
		for (int i = 0; i < eventFlag.length; i++) {
			eventFlag[i] = new EventFlagData( i + 1);
		}
		
		// data queue
		dataCount = objInfo[ToppersObjectTable.TYPE_DATA_QUEUE_CTRL].dataCount;
		dataQueue = new DataQueueData[dataCount];
		for (int i = 0; i < dataQueue.length; i++) {
			dataQueue[i] = new DataQueueData(i + 1);
		}
		
		// fixed memory pool
		dataCount = objInfo[ToppersObjectTable.TYPE_FIX_MEM_CTRL].dataCount;
		fixedMemPool = new FixedMemoryPoolData[dataCount];
		for (int i = 0; i < fixedMemPool.length; i++) {
			fixedMemPool[i] = new FixedMemoryPoolData(i + 1);
		}
		
		// add others here
		
	}
	
	/**
	 * Sets the prioritiy values to each ReadyQueueData.
	 * @param minPriority	the base minimum priority value.
	 */
	public void setReadyQueuePriority(int minPriority) {
		this.minPriority = minPriority;
		
		for (int i = 0; i < readyQueue.length; i++) {
			readyQueue[i].setPriority(minPriority + i);
		}
		
	}
	
	
	/**
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.IToppersMonitorInput#getTaskManagementData()
	 */
	public TaskManagementData getTaskManagementData() {
		return taskMng;
	}
	public void setTaskManagementData(AbstractDataModel[] models) {
		taskMng = (TaskManagementData) models[0];
	}
	
	
	public ReadyQueueData[] getReadyQueueData() {
		return readyQueue;
	}
	public void setReadyQueueData(AbstractDataModel[] models) {
		readyQueue = (ReadyQueueData[]) models;
	}
	
	public TaskData[] getTaskData() {
		return taskCntl;
	}
	public void setTaskData(AbstractDataModel[] models) {
		taskCntl = (TaskData[]) models;
	}
	
	public CyclicHandlerData[] getCyclicHandlerData() {
		return cycHandler;
	}
	public void setCyclicHandlerData(AbstractDataModel[] models) {
		cycHandler = (CyclicHandlerData[]) models;
	}
	
	public MailBoxData[] getMailBoxData() {
		return mailBox;
	}
	public void setMailBoxData(AbstractDataModel[] models) {
		mailBox = (MailBoxData[]) models;
	}
	
	public SemaphoreData[] getSemaphoreData() {
		return semaphore;
	}
	public void setSemaphoreData(AbstractDataModel[] models) {
		semaphore = (SemaphoreData[]) models; 
	}
	
	public EventFlagData[] getEventFlagData() {
		return eventFlag;
	}
	public void setEventFlagData(AbstractDataModel[] models) {
		eventFlag = (EventFlagData[]) models;
	}
	
	public DataQueueData[] getDataQueueData() {
		return dataQueue;
	}
	public void setDataQueueData(AbstractDataModel[] models) {
		dataQueue = (DataQueueData[]) models;
	}
	
	public FixedMemoryPoolData[] getFixedMemoryPoolData() {
		return fixedMemPool;	
	}
	public void setFixedMemoryPoolData(AbstractDataModel[] models) {
		fixedMemPool = (FixedMemoryPoolData[]) models;
	}
	
	public InterruptHandlerData[] getInterruptHandlerData() {
		return intHandler;
	}
	public void setInterruptHandlerData(AbstractDataModel[] models) {
		intHandler = (InterruptHandlerData[]) models;
	}
	
	public ExceptionHandlerData[] getExceptionHandlerData() {
		return excHandler;
	}
	public void setExceptionHandlerData(AbstractDataModel[] models) {
		excHandler = (ExceptionHandlerData[]) models;
	}
	
	/*
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.IToppersMonitorInput#getMinPriority()
	 */
	public int getMinPriority() {
		return minPriority;
	}
	
	/*
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.IToppersMonitorInput#getRunTaskAddress()
	 */
	public long getRunTaskAddress() {
		return taskMng.getRunningTaskAddress();
	}
	
	/*
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.IToppersMonitorInput#getTaskNames()
	 */
	public String[] getTaskNames() {
		
		String[] ret = new String[taskCntl.length];
		
		for (int i = 0; i < taskCntl.length; i++) {
			ret[i] = taskCntl[i].getProcessName(); 
		}
		
		return ret;
		
	}
	
}
