/*
 * 쐬: 2005/06/14
 *
 */
package jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data;

import java.nio.CharBuffer;

import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.AbstractDataModel;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.ToppersObjectTable;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.util.DataParser;


/**
 * Data model for cyclic handler info.
 * 
 * @author D.Fujimoto
 *
 */
public class CyclicHandlerData extends AbstractDataModel implements INamedProcess {

	
	/**
	 * The name of this handler.
	 */
	private String handlerName;
	
	/**
	 * Indicates whether this handler is activated or not.
	 */
	private boolean activated;
	
	/**
	 * Time when this handler will be next activated. 
	 */
	private long entryTime;
	
	/**
	 * Address where additional info is stored in.
	 */
	private long expandInfoAddress;
	
	/**
	 * Start address(Program counter).
	 */
	private long pcAddress;
	
	/**
	 * Activation cycle time.
	 */
	private long cycleTime;
	
	/**
	 * Activation phase.
	 */
	private long phaseTime;
	
	/**
	 * Constructor.
	 * @param id
	 */
	public CyclicHandlerData(int id) {
		super(id);
	}

	
	public boolean getActivated() {
		return activated;
	}
	public long getCycleTime() {
		return cycleTime;
	}
	public long getEntryTime() {
		return entryTime;
	}
	public long getExpandInfoAddress() {
		return expandInfoAddress;
	}
	public long getPcAddress() {
		return pcAddress;
	}
	public long getPhaseTime() {
		return phaseTime;
	}
	
	
	/** 
	 * Parses the given data by the aforeset parse type.
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.IParser#parseData(int, char[])
	 */
	public void parseData(int parseType, char[] readData) {

		CharBuffer buf = CharBuffer.wrap(readData);
		
		// buffer to get from the CharBuffer
		char[] data = new char[FOUR_BYTES_LENGTH];
		
		switch (parseType) {
			
			// cyclic handler control block
			case ToppersObjectTable.TYPE_CYC_CTRL :
				
				// init block address
				buf.get(data);
				initBlockAddress = DataParser.parse(data);
				
				// activated flag
				buf.get(data);
				activated = (DataParser.parse(data) != 0) ? true : false; 
			
				// entry time
				buf.get(data);
				entryTime = DataParser.parse(data);
				
				// time event block	(ignore)
				
				break;
			
			// cyclic handler init block
			case ToppersObjectTable.TYPE_CYC_INIT :
				
				// attribute	(ignore)
				buf.get(data);
				
				// expand info
				buf.get(data);
				expandInfoAddress = DataParser.parse(data);
				
				// pc address
				buf.get(data);
				pcAddress = DataParser.parse(data);
				
				// cycle time
				buf.get(data);
				cycleTime = DataParser.parse(data);
				
				// phase time
				buf.get(data);
				phaseTime = DataParser.parse(data);
				
				break;
		}
		
	}


	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.INamedProcess#getAddress()
	 */
	public long getAddress() {
		return pcAddress;
	}


	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.INamedProcess#getProcessName()
	 */
	public String getProcessName() {
		return handlerName;
	}


	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.INamedProcess#setProcessName(java.lang.String)
	 */
	public void setProcessName(String name) {
		handlerName = name;		
	}
}
