/*
 * 쐬: 2005/06/14
 *
 */
package jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data;

import java.nio.CharBuffer;

import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.AbstractDataModel;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.ToppersObjectTable;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.util.DataParser;


/**
 * Data model for data queue info.
 * 
 * @author D.Fujimoto
 *
 */
public class DataQueueData extends AbstractDataModel implements IQueue {

	// queue types for set/getQueueIndices()
	public static final int SEND_WAIT_QUEUE_TYPE = 0;
	public static final int RECV_WAIT_QUEUE_TYPE = 1;
	

	/**
	 * Address for previous queue(send).
	 */
	private long previousQueueSend;
	
	/**
	 * Address for next queue(send).
	 */
	private long nextQueueSend;
	
	/**
	 * Address for previous queue(receive).
	 */
	private long previousQueueRecv;
	
	/**
	 * Address for next queue(eceive).
	 */
	private long nextQueueRecv;
	
	/**
	 * Data count in the queue. 
	 */
	private int queuingDataCount;
	
	/**
	 * Attribute for this queue.
	 */
	private int dataQueueAttribute;
	
	/**
	 * Address to the first data. 
	 */
	private long headDataAddress;
	
	/**
	 * Initial top address of queue area. 
	 */
	private long dataQTopAddress;
	
	/**
	 * Indices for send queue.
	 */
	private int[] sendQueue;
	
	/**
	 * Indices for receive queue.
	 */
	private int[] recvQueue;
	
	
	/**
	 * Constructor. 
	 * @param id
	 */
	public DataQueueData(int id) {
		super(id);
	}
	
	
	public long getDataQTopAddress() {
		return dataQTopAddress;
	}
	public int getDataQueueAttribute() {
		return dataQueueAttribute;
	}
	public int getQueuingDataCount() {
		return queuingDataCount;
	}
	public long getHeadDataAddress() {
		return headDataAddress;
	}
	
	
	
	/**
	 * Parses the given data by the aforeset parse type.
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.IParser#parseData(int, char[])
	 */
	public void parseData(int parseType, char[] readData) {
		
		CharBuffer buf = CharBuffer.wrap(readData);
		
		// buffer to get from the CharBuffer
		char[] data = new char[FOUR_BYTES_LENGTH];
		
		switch (parseType) {
			
			// data queue control block
			case ToppersObjectTable.TYPE_DATA_QUEUE_CTRL :
				
				// QUEUE send
				// next
				buf.get(data);
				nextQueueSend = DataParser.parse(data);
				
				// previous
				buf.get(data);
				previousQueueSend = DataParser.parse(data);
				
				// init block address
				buf.get(data);
				initBlockAddress = DataParser.parse(data);
				
				// QUEUE read
				// next
				buf.get(data);
				nextQueueRecv = DataParser.parse(data);
				
				// previous
				buf.get(data);
				previousQueueRecv = DataParser.parse(data);
				
				// count
				buf.get(data);
				queuingDataCount = (int) DataParser.parse(data);
				
				// top offset
				buf.get(data);
				headDataAddress = DataParser.parse(data);
				
				// tail offset	(ignore)
				buf.get(data);
				
				break;
			
			// data queue init block
			case ToppersObjectTable.TYPE_DATA_QUEUE_INIT :
				
				// attribute
				buf.get(data);
				dataQueueAttribute = (int) DataParser.parse(data);
			
				// SKIP
				// dtqcnt	(4)
				buf.get(data);
				
				// data queue top address
				buf.get(data);
				dataQTopAddress = DataParser.parse(data);
				
				break;
		}
	}


	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue#getNextQueue(int)
	 */
	public long getNextQueue(int type) {
		switch (type) {
			case SEND_WAIT_QUEUE_TYPE :
				return nextQueueSend;
				
			case RECV_WAIT_QUEUE_TYPE :
				return nextQueueRecv;

		}
		
		return 0;
	}


	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue#getPrevQueue(int)
	 */
	public long getPrevQueue(int type) {
		switch (type) {
			case SEND_WAIT_QUEUE_TYPE :
				return previousQueueSend;
				
			case RECV_WAIT_QUEUE_TYPE :
				return previousQueueRecv;
		}
		return 0;
	}


	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue#setQueueIndices(int[], int)
	 */
	public void setQueueIndices(int[] indices, int type) {
		switch (type) {
			case SEND_WAIT_QUEUE_TYPE :
				sendQueue = indices;
				break;
			case RECV_WAIT_QUEUE_TYPE :
				recvQueue = indices;
				break;
		}
	}


	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue#getQueueIndices(int)
	 */
	public int[] getQueueIndices(int type) {
		switch (type) {
			case SEND_WAIT_QUEUE_TYPE :
				return sendQueue;
			case RECV_WAIT_QUEUE_TYPE :
				return recvQueue;
		}
		
		return null;
	}
}
