/*
 * 쐬: 2005/06/14
 *
 */
package jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data;

import java.nio.CharBuffer;

import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.AbstractDataModel;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.ToppersObjectTable;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.util.DataParser;


/**
 * Data model for semaphore info.
 * 
 * @author D.Fujimoto
 *
 */
public class SemaphoreData extends AbstractDataModel implements IQueue {

	/**
	 * Address for previous queue.
	 */
	private long previousQueue;
	
	/**
	 * Address for next queue.
	 */
	private long nextQueue;
	
	/**
	 * The semaphore count.
	 */
	private int semaphoreCount;
	
	/**
	 * The attribute for this semaphore.
	 */
	private int semAttribute;
	
	/**
	 * Max semaphore counts.
	 */
	private int maxSemaphoreCount;
	
	/**
	 * Indices for wait queue.
	 */
	private int[] waitQueue;
	
	/**
	 * Constructor. 
	 * @param id
	 */
	public SemaphoreData(int id) {
		super(id);
	}
	
	
	public int getMaxSemaphoreCount() {
		return maxSemaphoreCount;
	}
	public int getSemaphoreCount() {
		return semaphoreCount;
	}
	public int getSemAttribute() {
		return semAttribute;
	}
	
	
	/**
	 * Parses the given data by the aforeset parse type.
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.IParser#parseData(int, char[])
	 */
	public void parseData(int parseType, char[] readData) {

		CharBuffer buf = CharBuffer.wrap(readData);
		
		// buffer to get from the CharBuffer
		char[] data = new char[FOUR_BYTES_LENGTH];
		
		switch (parseType) {
			
			// semaphore control block
			case ToppersObjectTable.TYPE_SEM_CTRL :
				
				// QUEUE next
				buf.get(data);
				nextQueue = DataParser.parse(data);
				
				// QUEUE previous
				buf.get(data);
				previousQueue = DataParser.parse(data);
				
				// init block address
				buf.get(data);
				initBlockAddress = DataParser.parse(data);
			
				// count
				buf.get(data);
				semaphoreCount = (int) DataParser.parse(data);
			
				break;
			
			// semaphore init block
			case ToppersObjectTable.TYPE_SEM_INIT :

				// attribute
				buf.get(data);
				semAttribute = (int) DataParser.parse(data);
				
				// SKIP
				// initial sem count	(ignore)
				buf.get(data);
				
				// max sem count
				buf.get(data);
				maxSemaphoreCount = (int) DataParser.parse(data);
				
				break;
	
		}
		
	}


	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue#getNextQueue(int)
	 */
	public long getNextQueue(int type) {
		return nextQueue;
	}


	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue#getPrevQueue(int)
	 */
	public long getPrevQueue(int type) {
		return previousQueue;
	}


	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue#setQueueIndices(int[], int)
	 */
	public void setQueueIndices(int[] indices, int type) {
		waitQueue = indices;
	}


	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue#getQueueIndices(int)
	 */
	public int[] getQueueIndices(int type) {
		return waitQueue;
	}
}
