/*
 * 쐬: 2005/06/15
 *
 */
package jp.co.epson.ide.eclipse.gnu33.osmonitor.ui.views.content;

import java.util.StringTokenizer;

import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.OSMonitorPlugin;

import org.eclipse.jface.viewers.IContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;



/**
 * Parent class for the monitor's ViewContent 
 * @author D.Fujimoto
 *
 */
public abstract class AbstractMonitorViewContent implements IMonitorViewContent {

	/**
	 * Display message when no kernel object is created. 
	 */
	public static final String NO_OBJ_MSG = OSMonitorPlugin.getResourceString("tps.value.noObject"); //$NON-NLS-1$
	
	/**
	 * The table name.
	 * Used as the tab text.
	 */
	protected String tableName;
	
	/**
	 * The labels for the columns.
	 */
	protected String[] columnLabels;
	
	/**
	 * Widths for the columns. 
	 */
	protected int[] columnWidth;
	
	/**
	 * A Single TableViewer in a TabItem.
	 */
	protected TableViewer viewer;
	
	/**
	 * stock for task name display 
	 * (set by the ContentViewer and used by the LabelProvider)
	 *
	 */
	protected String[] taskNames;
	
	// default values
	protected int defaultTableStyle = SWT.SINGLE | SWT.BORDER | SWT.FULL_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL | SWT.VIRTUAL;
	protected int defaultColumnStyle = SWT.LEFT;
	protected boolean headerVisible = true;
	protected boolean linesVisible = false;
	
	/**
	 * Constructor.
	 * @param id			key to pick Strings from the property file
	 * @param columnCount	number of columns
	 */
	public AbstractMonitorViewContent(String id, int columnCount) {
		this.tableName = OSMonitorPlugin.getResourceString(id + ".name"); //$NON-NLS-1$
		columnLabels = new String[columnCount];
		columnWidth = new int[columnCount];
		
		getColumnLabelsFromPropFile(id + ".labels"); //$NON-NLS-1$
		getColumnWidthFromPropFile(id + ".widths"); //$NON-NLS-1$
	}
	
	/**
	 * Get the labels for the columns with the given key 
	 * @param key
	 */
	public void getColumnLabelsFromPropFile(String key) {
		String labels = OSMonitorPlugin.getResourceString(key);
		StringTokenizer tok = new StringTokenizer(labels, ",");
		int count = 0;
		while (tok.hasMoreTokens()) {
			String label = tok.nextToken();
			label = label.trim();
			columnLabels[count] = label;
			
			count++;
		}

	}
	
	/**
	 * Get column widths from the properties file.
	 * As of 1.0.0, this is not in use.
	 * 
	 * @param key
	 * 
	 */
	public void getColumnWidthFromPropFile(String key) {
		String width = OSMonitorPlugin.getResourceString(key);
		StringTokenizer tok = new StringTokenizer(width, ",");
		
		String[] vals = new String[tok.countTokens()];
		for (int i = 0; i < vals.length; i++) {
			vals[i] = tok.nextToken().trim();
		}
		
		// force to 90
		for (int i = 0; i < columnWidth.length; i++) {
			columnWidth[i] = 90;
		}
	}
	
	
	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.views.IMonitorViewContent#getTableName()
	 */
	public String getTableName() {
		return tableName;
	}

	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.views.IMonitorViewContent#getColumnLabel()
	 */
	public String[] getColumnLabels() {
		return columnLabels;
	}

	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.views.IMonitorViewContent#getColumnWidth()
	 */
	public int[] getColumnWidth() {
		return columnWidth;
	}

	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.views.IMonitorViewContent#getColumnCount()
	 */
	public int getColumnCount() {
		return columnLabels.length;
	}


	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.views.IMonitorViewContent#getTableViewer()
	 */
	public TableViewer getTableViewer() {
		return viewer;
	}

	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.views.IMonitorViewContent#getContentProvider()
	 */
	public IContentProvider getContentProvider() {
		return viewer.getContentProvider();
	}

	
	/**
	 * Class used when no object is supplied and to show 
	 * messages in the viewer.
	 * @author D.Fujimoto
	 *
	 */
	class NoObject {
	}
	
	/**
	 * Returns a length 1 NoObject array for ContentProvider purposes.
	 */
	public Object[] getNoObjectElement() {
		return new NoObject[] { new NoObject() };
	}
	

}
