/*
 * 쐬: 2005/10/24
 *
 */
package jp.co.epson.ide.eclipse.gnu33.osmonitor.ui.views.content;

import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.OSMonitorPlugin;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.IModelAttributeConstants;

import org.eclipse.jface.viewers.LabelProvider;


/**
 * Common ops for a LabelProvider.
 * @author D.Fujimoto
 *
 */
public abstract class AbstractToppersMonitorLabelProvider extends LabelProvider {

	public static final String ON_VAL = OSMonitorPlugin.getResourceString("tps.value.on"); //$NON-NLS-1$
	public static final String OFF_VAL = OSMonitorPlugin.getResourceString("tps.value.off"); //$NON-NLS-1$
	public static final String REQ_VAL = OSMonitorPlugin.getResourceString("tps.value.requested"); //$NON-NLS-1$
	public static final String NONE_VAL = OSMonitorPlugin.getResourceString("tps.value.none"); //$NON-NLS-1$
	public static final String NOTSPEC_VAL = OSMonitorPlugin.getResourceString("tps.value.notspecified"); //$NON-NLS-1$
	public static final String UNKNOWN_VAL = OSMonitorPlugin.getResourceString("tps.value.unknown"); //$NON-NLS-1$
	public static final String EXIST_VAL = OSMonitorPlugin.getResourceString("tps.value.exist"); //$NON-NLS-1$
	public static final String EMPTY_VAL = OSMonitorPlugin.getResourceString("tps.value.empty"); //$NON-NLS-1$

	public static final String ATTR_TFIFO = OSMonitorPlugin.getResourceString("tps.attr.tfifo"); //$NON-NLS-1$
	public static final String ATTR_TPRI = OSMonitorPlugin.getResourceString("tps.attr.tpri"); //$NON-NLS-1$
	public static final String ATTR_MFIFO = OSMonitorPlugin.getResourceString("tps.attr.mfifo"); //$NON-NLS-1$
	public static final String ATTR_MPRI = OSMonitorPlugin.getResourceString("tps.attr.mpri"); //$NON-NLS-1$

	public static final String ATTR_WSGL = OSMonitorPlugin.getResourceString("tps.attr.wait.single"); //$NON-NLS-1$
	public static final String ATTR_WMUL = OSMonitorPlugin.getResourceString("tps.attr.wait.multiple"); //$NON-NLS-1$
	public static final String ATTR_CLR = OSMonitorPlugin.getResourceString("tps.attr.wait.clear"); //$NON-NLS-1$
	
	public static final String ATTR_ANDW = OSMonitorPlugin.getResourceString("tps.attr.wait.mode.and"); //$NON-NLS-1$
	public static final String ATTR_ORW = OSMonitorPlugin.getResourceString("tps.attr.wait.mode.or"); //$NON-NLS-1$

	static final String ARROW = "->";
	
	
	/**
	 * Creates the display string for the queue list.
	 * Ex, task_name(task_id)->task_name(task_id)
	 * @param indices	the queue indices
	 * @param taskNames
	 * @return	the display string
	 */
	protected String createTaskQueueString(int[] indices, String[] taskNames) {
		String ret = "";
		
		
		// when there is no queue
		if (indices == null || indices.length == 0) {
			ret = NOTSPEC_VAL;
		} else {
			for (int i = 0; i < indices.length; i++) {
				ret += createProcessNameString(taskNames[indices[i]]) + "(" + (indices[i] + 1) + ")";
				
				if (i != indices.length - 1) {
					ret += ARROW;
				}
			}
			
		}
		
		return ret;
	
	}
	
	
	/**
	 * Creates the display string for task queue attribute.
	 * @param attr	the attribute value
	 * @return	the display string
	 */
	protected String createTaskQueueAttributeString(int attr) {
		String ret = "";
		
		if ((attr & IModelAttributeConstants.ATTR_TPRI) != 0) {
			ret = ATTR_TPRI;
		} else {
			ret = ATTR_TFIFO;
		}
		
		return ret;
	}
	
	/**
	 * Creates the display string for the process name.
	 * If the given name is null, returns a string as "unknown".
	 * @param	processName the function name of the task/handler
	 * @return	returns the process name itself. If null, an string 
	 * indicating as "unknown".
	 */
	protected String createProcessNameString(String processName) {
		return processName == null ? UNKNOWN_VAL : processName;
	}
	
}
