/*
 * 쐬: 2005/06/15
 *
 */
package jp.co.epson.ide.eclipse.gnu33.osmonitor.ui.views.content;

import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.IToppersMonitorInput;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.CyclicHandlerData;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.util.HexFormatter;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;



/**
 * @author D.Fujimoto
 *  
 */
public class CyclicHandlerMonitorContent extends AbstractMonitorViewContent {

	/**
	 * @param id	prefix to the properties 
	 * @param columnCount
	 */
	public CyclicHandlerMonitorContent(String id, int columnCount) {
		super(id, columnCount);
	}

	/*
	 * ( Javadoc)
	 * 
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.views.IMonitorViewContent#createTableViewer(org.eclipse.swt.widgets.Composite)
	 */
	public TableViewer createTableViewer(Composite parent) {

		Table table = new Table(parent, defaultTableStyle);
		table.setHeaderVisible(headerVisible);
		table.setLinesVisible(linesVisible);

		TableColumn[] cols = new TableColumn[getColumnCount()];
		for (int i = 0; i < cols.length; i++) {
			cols[i] = new TableColumn(table, defaultColumnStyle, i);
			cols[i].setText(columnLabels[i]);
			cols[i].setWidth(columnWidth[i]);
		}

		viewer = new TableViewer(table);

		viewer.setContentProvider(new CyclicHandlerContentProvider());
		viewer.setLabelProvider(new CyclicHandlerLabelProvider());

		return viewer;
	}

	public class CyclicHandlerContentProvider implements
			IStructuredContentProvider {

		/*
		 * ( Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
		 */
		public Object[] getElements(Object inputElement) {
			if (inputElement instanceof IToppersMonitorInput) {
				IToppersMonitorInput element = (IToppersMonitorInput) inputElement;
				
				Object[] elements = element.getCyclicHandlerData();
				// when there is no data 
				if (elements.length == 0) {
					return getNoObjectElement();
				}
				
				return elements;

			}
			return null;
		}

		/*
		 * ( Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
		 */
		public void dispose() {}

		/*
		 * ( Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
		 *      java.lang.Object, java.lang.Object)
		 */
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {}

	}

	public class CyclicHandlerLabelProvider extends AbstractToppersMonitorLabelProvider implements
			ITableLabelProvider {

		/*
		 * ( Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object,
		 *      int)
		 */
		public Image getColumnImage(Object element, int columnIndex) {
			return null;
		}

		/*
		 * ( Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object,
		 *      int)
		 */
		public String getColumnText(Object element, int columnIndex) {
			String ret = "";

			if (element instanceof CyclicHandlerData) {
			
				CyclicHandlerData data = (CyclicHandlerData) element;
				switch (columnIndex) {
					case 0:
						ret = Integer.toString(data.getId());
						break;
					case 1:
						ret = createProcessNameString(data.getProcessName());
						break;
					case 2:
						ret = (data.getActivated()) ? ON_VAL : OFF_VAL ;
						break;
					case 3:
						ret = HexFormatter.format(data.getExpandInfoAddress());
						break;
					case 4:
						ret = HexFormatter.format(data.getPcAddress());					
						break;
					case 5:
						ret = Long.toString(data.getPhaseTime());
						break;
					case 6:
						ret = Long.toString(data.getCycleTime());
						break;
					case 7 :
						ret = Long.toString(data.getEntryTime());
						break;
					default:
						break;

				}
		
			} else if (element instanceof NoObject) {
				// No object specified
				if (columnIndex == 0) {
					ret = NO_OBJ_MSG;	
				}
			}
			
			return ret;

		}

	}

}