/*
 * 쐬: 2005/06/15
 *
 */
package jp.co.epson.ide.eclipse.gnu33.osmonitor.ui.views.content;

import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.IModelAttributeConstants;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.IToppersMonitorInput;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.EventFlagData;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.util.HexFormatter;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;



/**
 * @author D.Fujimoto
 *  
 */
public class EventFlagMonitorContent extends AbstractMonitorViewContent {

	/**
	 * @param id	prefix to the properties 
	 * @param columnCount
	 */
	public EventFlagMonitorContent(String id, int columnCount) {
		super(id, columnCount);
	}

	/*
	 * ( Javadoc)
	 * 
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.views.IMonitorViewContent#createTableViewer(org.eclipse.swt.widgets.Composite)
	 */
	public TableViewer createTableViewer(Composite parent) {

		Table table = new Table(parent, defaultTableStyle);
		table.setHeaderVisible(headerVisible);
		table.setLinesVisible(linesVisible);

		TableColumn[] cols = new TableColumn[getColumnCount()];
		for (int i = 0; i < cols.length; i++) {
			cols[i] = new TableColumn(table, defaultColumnStyle, i);
			cols[i].setText(columnLabels[i]);
			cols[i].setWidth(columnWidth[i]);
		}

		viewer = new TableViewer(table);

		viewer.setContentProvider(new EventFlagContentProvider());
		viewer.setLabelProvider(new EventFlagLabelProvider());

		return viewer;
	}

	public class EventFlagContentProvider implements IStructuredContentProvider {

		/*
		 * ( Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
		 */
		public Object[] getElements(Object inputElement) {
			if (inputElement instanceof IToppersMonitorInput) {
				IToppersMonitorInput element = (IToppersMonitorInput) inputElement;
				
				// get task names for queue display
				taskNames = element.getTaskNames();
				
				Object[] elements = element.getEventFlagData();
				// when there is no data
				if (elements.length == 0) {
					return getNoObjectElement();
				}
				
				return elements;

			}
			return null;
		}

		/*
		 * ( Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
		 */
		public void dispose() {}

		/*
		 * ( Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
		 *      java.lang.Object, java.lang.Object)
		 */
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {}

	}

	public class EventFlagLabelProvider extends AbstractToppersMonitorLabelProvider implements
			ITableLabelProvider {

		/*
		 * ( Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object,
		 *      int)
		 */
		public Image getColumnImage(Object element, int columnIndex) {
			return null;
		}

		/*
		 * ( Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object,
		 *      int)
		 */
		public String getColumnText(Object element, int columnIndex) {
			String ret = "";
			
			if (element instanceof EventFlagData) {
				EventFlagData data = (EventFlagData) element;
				
				int[] q = data.getQueueIndices(IQueue.DEFAULT_TYPE);
				
				switch (columnIndex) {
					case 0:
						ret = Integer.toString(data.getId());
						break;
						
					case 1:
						int attr = data.getEventFlagAttribute();
						
						// the task queue attribute
						ret = createTaskQueueAttributeString(attr);
						
						// the max waiting attribute
						if ((attr & IModelAttributeConstants.ATTR_WMUL) != 0) {
							ret += " | " + ATTR_WMUL;
						} else {
							ret += " | " + ATTR_WSGL;
						}
						
						// the clear attribute
						if ((attr & IModelAttributeConstants.ATTR_CLR) != 0) {
							ret += " | " + ATTR_CLR;
						}
						break;
						
					case 2:
						ret = HexFormatter.format(data.getFlagPattern());
						break;
						
					case 3:
						if (q == null || q.length == 0) {
							ret = NOTSPEC_VAL;
						} else {
							ret = HexFormatter.format(data.getWaitPattern());
						}
						break;
						
					case 4:
						if (q == null || q.length == 0) {
							ret = NOTSPEC_VAL;
						} else {
							int mode = data.getWaitMode();
							if (mode == IModelAttributeConstants.ATTR_ORW) {
								ret = ATTR_ORW;
							} else {
								ret = ATTR_ANDW;
							}
						}
						break;
						
					case 5:
						ret = createTaskQueueString(q, taskNames);
						break;
						
					default:
						break;
	
				}
				
			} else if (element instanceof NoObject) {
				
				// No object specified
				if (columnIndex == 0) {
					ret = NO_OBJ_MSG;	
				}
			}
			return ret;

		}

	}

}