/*
 * 쐬: 2005/06/15
 *
 */
package jp.co.epson.ide.eclipse.gnu33.osmonitor.ui.views.content;

import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.IModelAttributeConstants;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.IToppersMonitorInput;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.MailBoxData;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;



/**
 * @author D.Fujimoto
 *  
 */
public class MailBoxMonitorContent extends AbstractMonitorViewContent {

	/**
	 * @param id	prefix to the properties 
	 * @param columnCount
	 */
	public MailBoxMonitorContent(String id, int columnCount) {
		super(id, columnCount);
	}

	/*
	 * ( Javadoc)
	 * 
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.views.IMonitorViewContent#createTableViewer(org.eclipse.swt.widgets.Composite)
	 */
	public TableViewer createTableViewer(Composite parent) {

		Table table = new Table(parent, defaultTableStyle);
		table.setHeaderVisible(headerVisible);
		table.setLinesVisible(linesVisible);

		TableColumn[] cols = new TableColumn[getColumnCount()];
		for (int i = 0; i < cols.length; i++) {
			cols[i] = new TableColumn(table, defaultColumnStyle, i);
			cols[i].setText(columnLabels[i]);
			cols[i].setWidth(columnWidth[i]);
		}

		viewer = new TableViewer(table);

		viewer.setContentProvider(new MailBoxContentProvider());
		viewer.setLabelProvider(new MailBoxLabelProvider());

		return viewer;
	}

	public class MailBoxContentProvider implements IStructuredContentProvider {

		/*
		 * ( Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
		 */
		public Object[] getElements(Object inputElement) {
			if (inputElement instanceof IToppersMonitorInput) {
				IToppersMonitorInput element = (IToppersMonitorInput) inputElement;
				
				// get task names for queue display
				taskNames = element.getTaskNames();
				
				Object[] elements = element.getMailBoxData();
				// when there is no data
				if (elements.length == 0) {
					return getNoObjectElement();
				}
				
				return elements;

			}
			return null;
		}

		/*
		 * ( Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
		 */
		public void dispose() {}

		/*
		 * ( Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
		 *      java.lang.Object, java.lang.Object)
		 */
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {}

	}

	public class MailBoxLabelProvider extends AbstractToppersMonitorLabelProvider implements
			ITableLabelProvider {

		
		/*
		 * ( Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object,
		 *      int)
		 */
		public Image getColumnImage(Object element, int columnIndex) {
			return null;
		}

		/*
		 * ( Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object,
		 *      int)
		 */
		public String getColumnText(Object element, int columnIndex) {
			String ret = "";
			
			if (element instanceof MailBoxData) {
			
				MailBoxData data = (MailBoxData) element;
				switch (columnIndex) {
					case 0:
						ret = Integer.toString(data.getId());
						break;
					case 1:
						int attr = data.getMbxAttribute();
						
						// the task queue
						ret = createTaskQueueAttributeString(attr);
						
						// the msg queue
						if ((attr & IModelAttributeConstants.ATTR_MPRI) != 0) {
							ret += " | " + ATTR_MPRI;
						} else {
							ret += " | " + ATTR_MFIFO;
						}
					
						break;
					case 2:
						ret = Integer.toString(data.getMaxPriority());
						break;
					case 3:
						ret = data.msgExists() ? EXIST_VAL : EMPTY_VAL ;
						break;
					case 4:
						int[] q = data.getQueueIndices(IQueue.DEFAULT_TYPE);
						ret = createTaskQueueString(q, taskNames);
						break;
					default:
						break;
	
				}
				
			} else if (element instanceof NoObject) {
				
				// No object specified
				if (columnIndex == 0) {
					ret = NO_OBJ_MSG;	
				}
			}
			return ret;

		}

	}

}