/*
 * 쐬: 2005/06/15
 *
 */
package jp.co.epson.ide.eclipse.gnu33.osmonitor.ui.views.content;

import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.OSMonitorPlugin;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.IToppersMonitorInput;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.TaskData;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.TaskManagementData;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;



/**
 * @author D.Fujimoto
 *  
 */
public class TaskManagementMonitorContent extends AbstractMonitorViewContent {

	/**
	 * @param id	prefix to the properties 
	 * @param columnCount
	 */
	public TaskManagementMonitorContent(String id, int columnCount) {
		super(id, columnCount);
	}

	/*
	 * ( Javadoc)
	 * 
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.views.IMonitorViewContent#createTableViewer(org.eclipse.swt.widgets.Composite)
	 */
	public TableViewer createTableViewer(Composite parent) {

		Table table = new Table(parent, defaultTableStyle);
		table.setHeaderVisible(headerVisible);
		table.setLinesVisible(linesVisible);

		TableColumn[] cols = new TableColumn[getColumnCount()];
		for (int i = 0; i < cols.length; i++) {
			cols[i] = new TableColumn(table, defaultColumnStyle, i);
			cols[i].setText(columnLabels[i]);
			cols[i].setWidth(columnWidth[i]);
		}

		viewer = new TableViewer(table);

		viewer.setContentProvider(new TaskManagementContentProvider());
		viewer.setLabelProvider(new TaskManagementLabelProvider());

		return viewer;
	}

	public class TaskManagementContentProvider implements
			IStructuredContentProvider {

		final String STATUS_LBL = OSMonitorPlugin.getResourceString("tps.tskmng.status");
		final String CURTASK_LBL = OSMonitorPlugin.getResourceString("tps.tskmng.currentTask");
		final String NEXTASK_LBL = OSMonitorPlugin.getResourceString("tps.tskmng.nextTask");
		final String SYSYIME_LBL = OSMonitorPlugin.getResourceString("tps.tskmng.systime");
		final String IL_NEST_LBL = OSMonitorPlugin.getResourceString("tps.tskmng.interruptCount");
		
		final String TSK_CTXT = OSMonitorPlugin.getResourceString("tps.tskmng.taskContext");
		final String TSK_NCTXT = OSMonitorPlugin.getResourceString("tps.tskmng.nonTaskContext");
		final String DDSP = OSMonitorPlugin.getResourceString("tps.tskmng.dispatchDisabled");
		final String LOCK = OSMonitorPlugin.getResourceString("tps.tskmng.cpuLocked");
				
		final String NOTSPEC_VAL = OSMonitorPlugin.getResourceString("tps.value.notspecified"); //$NON-NLS-1$
		final String UNKNOWN_VAL = OSMonitorPlugin.getResourceString("tps.value.unknown"); //$NON-NLS-1$
		
		// rows to show
		Object[] obj = new Object[5];

		/*
		 * ( Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
		 */
		public Object[] getElements(Object inputElement) {
			if (inputElement instanceof IToppersMonitorInput) {
				IToppersMonitorInput element = (IToppersMonitorInput) inputElement;

				// get task names for task name display
				taskNames = element.getTaskNames();
				
				TaskManagementData data = element.getTaskManagementData();
				TaskData[] tasks = element.getTaskData();
				
				// the system status 
				String statusString = "";
				statusString = data.getInterruptNestCount() == 0 ? TSK_CTXT : TSK_NCTXT;
				
				if (!data.isDispatchAllowed()) {
					statusString += " " + DDSP; 
				}
				
				if (data.isLocked()) {
					statusString += " " + LOCK; 
				}
				
				
				// get the current task id
				int taskIndex = data.getRunTaskIndex(); 
				String runTask = null;
				if (taskIndex >= 0 && taskIndex < tasks.length) {
					runTask = taskNames[taskIndex] == null ? UNKNOWN_VAL : taskNames[taskIndex];
					runTask += "(" + Integer.toString(tasks[taskIndex].getId()) + ")";
				} else if (taskIndex == -1) {
					runTask = NOTSPEC_VAL;
				} else {
					runTask = UNKNOWN_VAL;
				}
				
				// get the next task id
				taskIndex = data.getReadyTaskIndex();
				String rdyTask = null;
				if (taskIndex >= 0 && taskIndex < tasks.length) {
					rdyTask = taskNames[taskIndex] == null ? UNKNOWN_VAL : taskNames[taskIndex];
					rdyTask += "(" + Integer.toString(tasks[taskIndex].getId()) + ")";
				} else if (taskIndex == -1) {
					rdyTask = NOTSPEC_VAL;
				} else {
					rdyTask = UNKNOWN_VAL;
				}
				
				
				obj[0] = new String[] {STATUS_LBL, statusString};
				obj[1] = new String[] {CURTASK_LBL, runTask};
				obj[2] = new String[] {NEXTASK_LBL, rdyTask};
				obj[3] = new String[] {SYSYIME_LBL, Long.toString(data.getSystemTime()) + " msec"};
				obj[4] = new String[] {IL_NEST_LBL, Integer.toString(data.getInterruptNestCount())};			

				return obj;

			}
			
			return new Object[0];
		}

		/*
		 * ( Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
		 */
		public void dispose() {}

		/*
		 * ( Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
		 *      java.lang.Object, java.lang.Object)
		 */
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {}

	}

	public class TaskManagementLabelProvider extends AbstractToppersMonitorLabelProvider
		implements ITableLabelProvider {

		/*
		 * ( Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object,
		 *      int)
		 */
		public Image getColumnImage(Object element, int columnIndex) {
			return null;
		}

		/*
		 * ( Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object,
		 *      int)
		 */
		public String getColumnText(Object element, int columnIndex) {
			String ret = "";
			String[] rowElement = (String[]) element ;
			switch (columnIndex) {
				case 0: // no break
				case 1:
					ret = rowElement[columnIndex];
					break;
				default:
					break;

			}

			return ret;
		}

	}

}