/* ICD interface for the EPSON C33 for GDB, the GNU debugger.
   Copyright 1996, 1997, 1998, 1999, 2000, 2001, 2002
   Free Software Foundation, Inc.

   This file is part of GDB.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */
                                  
//#define C33_DEBUG 1
                                                
#include "defs.h"
#include "gdb_string.h"
#include "frame.h"
#include "symtab.h"
#include "inferior.h"
#include "breakpoint.h"
#include "symfile.h"
#include "target.h"
#include "objfiles.h"
#include "gdbcore.h"
#include "value.h"
#include "command.h"
#include "serial.h"
#include <string.h>
#include "gdb_assert.h"
#include <time.h>
#include "source.h"

#include <tcl.h>
#include "c33common.h"
#include "c33icd.h"
#include "Sim33.h"
#include <stdio.h>

// 2006/06/05 M.Kai 
#include<stdlib.h>
#ifdef __CYGWIN__				/* add T.Tazaki 2006/06/15 */
#include<windows.h>
#endif          				/* add T.Tazaki 2006/06/15 */

/* Prototypes for local functions */
static void c33_MemoryFill(int iParamNum, int iType, unsigned long ulStartAddr, unsigned long ulEndAddr, unsigned long ulData);
static void c33_MemoryMove(int iParamNum, int iType, unsigned long ulStartAddr, unsigned long ulEndAddr, unsigned long ulDstAddr);
static void c33_ReadParameterFile(int iParamNum, char *pFilename);
static int c33_StrToCycleData(char *cParam, struct c33_map_info *pstMapInfo);
static int c33_AddMap(struct c33_map_info stMapInfo);
static void c33_ShowMapInfo(int iParamNum);
static void c33_DumpFile(int iParamNum, unsigned long ulStartAddr, unsigned long ulEndAddr, int iFileFormat, char* pFilename, char* pAppend);
static int c33_ReadWriteMapMemExe(int iReadWrite, unsigned long ulAddr, int iUnit, unsigned char* pMyAddr);
static int c33_SearchMem (int iReadWrite, unsigned long ulAddr, unsigned long ulLen, unsigned long* ulExeLen, unsigned char** ppcMyAddr, int* piEndian);
static void c33_CallMode(int iParamNum, int iCallMode , char *cFileName );
static void c33_Call(int iParamNum, int iStartAddr, int iArg1, int iArg2, int iArg3);
static void c33_Int(int iParamNum, int iType , int iLevel );
static void c33_ClockMode(int iParamNum, int iClockMode , int iClockFunction );
static void c33_Clock(int iParamNum);
static void c33_Stdin(int iParamNum, int iSet, unsigned long ulBreakAddr, unsigned long ulReadBufAddr, char *cFileName );
static void c33_Stdout(int iParamNum, int iSet, unsigned long ulBreakAddr, unsigned long ulWriteBufAddr, char *cFileName );
static void c33_command_analysis (char *arg, struct c33_cmd_structure *pCmd);
static void c33_DspErrBrkStatus (void);
static void c33_GetNormalString( char *pszbuf, char *pszbuf_normal ); /* add 2003/02/27 T.Tazaki */
static void c33_DeleteDataBreak( int iParamNum, char* pSymbol );	  /* add 2003/12/08 T.Tazaki */

/* global variable */
struct c33_target_info c33TargetInfo; /* target information */

unsigned long g_ulClockCount[3];      /* clock timer/counter */
extern unsigned long gulCycle;        /* SIM mode clock/cycle counter */

unsigned long long g_ullClockCount;	/* add T.Tazaki 2007/07/03 for ICDV6 */

FILE *fpStdinfile  = NULL;
FILE *fpStdoutfile = NULL;
FILE *fpLogfile    = NULL;
FILE *fpINI        = NULL;            /* add 2003/02/27 T.Tazaki : gdbtk.ini output */
int g_Gdbtk_Update = 0;               /* Do not log file output flag 1=ON */
int g_iReadParameterFile = 0;         /* add 2003/02/28 T.Tazaki : c33 rpf flag(2'nd=>error) */
void c33_LogConvert( char *pSrc );    /* add 2003/01/16 T.Tazaki : log command */
char szLogbuf[3100];                  /* 512 --> 3100 Modify 2004/01/21 T.Tazaki : log command */
int  iLogCR = 0;                      /* add 2003/01/16 T.Tazaki : log command */

int g_c33_std_in = 0;   /* 1:already key input */
int g_c33_Index = 0;
char g_c33_stdin_string[1000];

int g_iGdbtk_Update_cnt = 0;         /* add 2003/03/18 T.Tazaki : Command Cancel Window Update counter */
int g_c33_iSourceBreak = 0;          /* add 2003/03/26 T.Tazaki */


int g_C33_ERR_MSG_TARGET_CONNECT = 0; /* add T.Tazaki 2006/07/21 : bug fix */
#if 0 /* del 2004/01/08 T.Tazaki */
int g_iSet_PE_reg_mode = 0;			 /* add 2003/11/18 T.Tazaki */
#endif
extern int g_use_windows;            /* add 2003/03/12 T.Tazaki */
extern int g_no_version;             /* add 2003/03/26 T.Tazaki : --c33_no_ver option */
extern int g_error_func;             /* add 2003/03/26 T.Tazaki : called by error() */
extern int g_command_loop;           /* add 2003/03/26 T.Tazaki : log command ,1=command loop running */
extern int giFrameClear;             /* add 2003/04/28 M.Takeishi : if giFrameClear = 1, and c33_Gdbtk_Update  */
                                     /*                then, flush_cashed_frames() is called. */
extern int g_c33icd_das;             /* =1:"c33 das" command executed add T.Tazaki 2003/06/11 */

extern void c33_GetBreakPoint(void);
extern int g_SoftPcBreakTable[ 510 ];                                   /* add 2003/05/08 T.Tazaki : Soft PC break address table */
extern int g_SoftPcBreakTable_Index;                                    /* add 2003/05/08 T.Tazaki : Soft PC break count */
extern int c33_GetBreakPointStatus(int iNum);                           /* add 2003/07/15 T.Tazaki */
extern int c33_GetHardBreakPointStatus(int iNum);                       /* add 2003/10/15 T.Tazaki */
extern void c33_SimSoftPcBreakSet(unsigned long ulBreakAddr);           /* add 2003/09/26 T.Tazaki */
extern void c33_SimSoftPcBreakClear(unsigned long ulBreakAddr);         /* add 2003/09/26 T.Tazaki */
extern int  c33_SimHardPcBreakSet(unsigned long ulBreakAddr,int iID);   /* add 2003/09/19 T.Tazaki */
extern void c33_SimHardPcBreakClear(int iID);                           /* add 2003/09/19 T.Tazaki */
extern void c33icd_WindowUpdate(void);                                  /* add 2003/10/16 T.Tazaki */
extern void gdbsim_WindowUpdate(void);                                  /* add 2003/10/20 T.Tazaki */

extern int g_iIcdLoadFirst; /* 1=High Speed program load external process routine write request add T.Tazaki 2003/07/29 */

extern int  g_iSamePCMode;  /*  add T.Tazaki 2003/09/19*/

extern int  g_iParallel_mode;        /* 1:icdv2 fast write by parallel port mode         */
extern int  g_iParallel_LPTNo;       /* lptX (1 or 2)                                    */

extern int g_iSim_open;			/* add T.Tazaki 2004/06/11 */

extern int g_iMemReadMode;		/* add T.Tazaki 2004/11/01 */
extern int g_iDasCpu;			/* = 1:set c33 das cpu type , =0:No set */	/* add T.Tazaki 2005/04/01 */
extern int g_iCalledByContinueCmd;	/* = 1 : called by contine_command add T.Tazaki 2005/05/11 */

extern int g_iClockResetCmdErr;	/* add T.Tazaki 2005/06/24 */

extern int g_iTargetResetDown;	/* add T.Tazaki 2005/07/28 */
extern int g_com_file_error;	/* utils.c add T.Tazaki 2007/06/25 */
extern int g_iICD_kind;			/* for ICDV6 add T.Tazaki 2007/07/03 */
extern int g_iTimeBreak;		/* time break mode  add T.Tazaki 2007/07/03 */

#ifdef __CYGWIN__				/* add T.Tazaki 2005/09/08 */
extern void c33_OSMon(void);
#endif

/****************************************************************************/
/* Format        : int c33_Initialize (void)                                */
/* Arguments     : void                                                     */
/* Return value                                                             */
/*  C33_NO_ERR   : Success return                                           */
/*  C33_ERR      : Error return                                             */
/* Explanation   : A connection check with a target is carried out.         */
/*                 Version information is read.                             */
/*                 A target is initialized.                                 */
/*                 A target is reset.                                       */
/****************************************************************************/
int
c33_Initialize (void)
{
    int i;
    int i_das_temp = 0;			/* add T.Tazaki 2004/01/16 */
	unsigned char ucAdvanced;	/* add T.Tazaki 2004/01/16 */

    /* A connection check with a target. */
    if (c33_Cmd_OK() != C33_NO_ERR)
    {
        return C33_ERR;
    }

    if( c33TargetInfo.iIcd_open == C33_USB_OPEN )   /* ICD V3 ? */
    {

        /* CPU type & Debug Unit Base addres set */
        if (c33_Cmd_DebugAddr_Set() != C33_NO_ERR)
        {
            return C33_ERR;
        }
    }
    

#if 0	/* del T.Tazaki 2004/07/16 */
    /* Version information is read. */
    if (c33_Cmd_Ver_Read() != C33_NO_ERR)
    {
        return C33_ERR;
    }
#endif
    /* A target is initialized. */
    if (c33_Cmd_Initialize() != C33_NO_ERR)
    {
        return C33_ERR;
    }

	/* Virtual CPU type set  #2    add T.Tazaki 2004/01/19 >>> */
    if( c33TargetInfo.iIcd_open == C33_USB_OPEN )   /* ICD V3 ? */
    {
		if( g_c33icd_das == 0 )	/* "c33 das" not execute? 	add T.Tazaki 2004/12/27 */
		{
			/* no "c33 das" */
		    c33_Cmd_ReadWrite(C33_READ, C33_ADDR_PRODUCT, C33_BYTE, C33_BYTE, &ucAdvanced);

		    ucAdvanced &= C33_PRODUCT_ADV;
		    if (ucAdvanced == C33_PRODUCT_ADV)
			{
		        c33TargetInfo.ucCpuType = C33_ADV;    /* ADV (Virtual) set */

		        i_das_temp = g_c33icd_das;
		        g_c33icd_das = 1;


		        /* CPU type & Debug Unit Base addres set */
		        if (c33_Cmd_DebugAddr_Set() != C33_NO_ERR)
		        {
		            g_c33icd_das = i_das_temp;
		            return C33_ERR;
		        }
		        g_c33icd_das = i_das_temp;

			}
		}
		else
		{	
			/* add T.Tazaki 2005/04/01 >>> */
			if( g_iDasCpu == C33_AUTO )	
			{
		    	c33_Cmd_ReadWrite(C33_READ, C33_ADDR_PRODUCT, C33_BYTE, C33_BYTE, &ucAdvanced);
			    ucAdvanced &= C33_PRODUCT_ADV;
			    if (ucAdvanced == C33_PRODUCT_ADV)
				{
			        c33TargetInfo.ucCpuType = C33_ADV;    /* ADV (Virtual) set */

			        /* CPU type & Debug Unit Base addres set */
			        if (c33_Cmd_DebugAddr_Set() != C33_NO_ERR)
			        {
			            return C33_ERR;
			        }
				}
			}
			else
			{
			/* add T.Tazaki 2005/04/01 <<< */
		        /* add T.Tazaki 2004/12/27 >>> */
		        if( c33TargetInfo.ucCpuType != C33_STD )
		        {
			        /* CPU type & Debug Unit Base addres set */
			        if (c33_Cmd_DebugAddr_Set() != C33_NO_ERR)
			        {
			            return C33_ERR;
			        }
			    }
		        /* add T.Tazaki 2004/12/27 <<< */
			}
	    }
		
	    /* A target is reset.  No Write Regsters & Special Resiters */
	    if (c33_Cmd_Reset_no_reg_write(C33_ICD_COLD_RESET) != C33_NO_ERR)
	    {
	        return C33_ERR;
	    }
	}
	/* add T.Tazaki 2004/01/19 <<< */

	/* add T.Tazaki 2004/07/16 >>> */
    /* Version information is read. */
    if (c33_Cmd_Ver_Read() != C33_NO_ERR)
    {
        return C33_ERR;
    }
	/* add T.Tazaki 2004/07/16 <<< */

    /* Initialization of all break information. */
    /* Clear soft pc break */
    for (i = 0 ; i < C33_STDOUT_BREAK_NUM ; i++)
    {
        c33TargetInfo.ulSoftPcBreak[i] = C33_BREAK_DISABLE;
    }
    /* Clear hard pc break */
    for (i = 0 ; i < C33_HARD_PC_BREAK_MAX ; i++)
    {
        c33TargetInfo.ulHardPcBreak[i] = C33_BREAK_DISABLE;
    }
    /* Clear data break */
    c33TargetInfo.ulDataBreak = C33_BREAK_DISABLE;

    /* Reset counters */
    c33TargetInfo.iClockMode = 2;
    c33TargetInfo.iClockFunction = 1;

    g_ulClockCount[0] = 0;  /* cycle count total */
    g_ulClockCount[1] = 0;  /* 50ns time total */
    g_ulClockCount[2] = 0;  /* 1us time  total */

	g_ullClockCount = 0;  /* cycle count total */	/* ICDV6 add T.Tazaki 2007/07/03 */

    /* Success return */
    return C33_NO_ERR;
}

/****************************************************************************/
/* Format        : int c33_Cmd_OK (void)                                    */
/* Arguments     : void                                                     */
/* Return value                                                             */
/*  C33_NO_ERR   : Success return                                           */
/*  C33_ERR      : Error return                                             */
/* Explanation   : Connection with a target is checked.                     */
/****************************************************************************/
int
c33_Cmd_OK (void)
{
    int iResult;
    puts_filtered (C33_MSG_TARGET_CONNECT_INFO);

    /* Transmission of a message. */
    iResult = c33_SendMessage(C33_ID_OK, 0, 0, 0, 0, 0, 0, 0);

    /* Reception of a message. */
    if (iResult == C33_NO_ERR){
        iResult = c33_RecvMessage(C33_ID_OK, 0, 0, 0, 0, 0);
    }

    if (iResult == C33_NO_ERR)
    {
        puts_filtered (C33_MSG_TARGET_DONE);
    }
    else
    {
        c33TargetInfo.iIcd_open = C33_NOT_OPEN;
        puts_filtered (C33_MSG_TARGET_FAILURE);
/*      c33_ErrorMsg();								*/ /* del T.Tazaki 2004/12/20 for back to prompt */
    }

    return iResult;
}
/****************************************************************************/
/* Format        : int c33_Cmd_DebugAddr_Set (void)                         */
/* Arguments     : void                                                     */
/* Return value                                                             */
/*  C33_NO_ERR   : Success return                                           */
/*  C33_ERR      : Error return                                             */
/* Explanation   : Set CPU type and debug unit address.                     */
/****************************************************************************/
int
c33_Cmd_DebugAddr_Set (void)
{
    int iResult;
    puts_filtered (C33_MSG_TARGET_SET_DEBUG_BASE);

    /* check if "c33 das" command do not execute. */
    if( g_c33icd_das == 0 )
    {
        /* defualt setting */
        c33TargetInfo.ucCpuType = C33_STD;    /* Standard macro */
        c33TargetInfo.ulDebugBase_rom_addr = C33_DEBUG_BASE_ROM_ADDR;
        c33TargetInfo.ulDebugBase_ram_addr = C33_DEBUG_BASE_RAM_ADDR;
    }
    /* Transmission of a message. */
    iResult = c33_SendMessage(C33_ID_SET_DEBUG_BASE, 0, 0, 0, 0, 0, 0, 0);

    /* Reception of a message. */
    if (iResult == C33_NO_ERR){
        iResult = c33_RecvMessage(C33_ID_SET_DEBUG_BASE, 0, 0, 0, 0, 0);
    }

    if (iResult == C33_NO_ERR)
    {
        puts_filtered (C33_MSG_TARGET_DONE);
    }
    else
    {
        puts_filtered (C33_MSG_TARGET_FAILURE);
/*      c33_ErrorMsg();								*/ /* del T.Tazaki 2004/12/20 for back to prompt */
    }

    return iResult;
}

/****************************************************************************/
/* Format        : int c33_Cmd_Initialize (void)                            */
/* Arguments     : void                                                     */
/* Return value                                                             */
/*  C33_NO_ERR   : Success return                                           */
/*  C33_ERR      : Error return                                             */
/* Explanation   : A target is initialized.                                 */
/****************************************************************************/
int
c33_Cmd_Initialize (void)
{
    int iResult;

    puts_filtered (C33_MSG_TARGET_INITIALIZE_INFO);

    /* Transmission of a message. */
    iResult = c33_SendMessage(C33_ID_INITIALIZE, 0, 0, 0, 0, 0, 0, 0);

    /* Reception of a message. */
    if (iResult == C33_NO_ERR){
        iResult = c33_RecvMessage(C33_ID_INITIALIZE, 0, 0, 0, 0, 0);
    }

    if (iResult == C33_NO_ERR)
    {
        puts_filtered (C33_MSG_TARGET_DONE);
    }
    else
    {
        puts_filtered (C33_MSG_TARGET_FAILURE);
/*      c33_ErrorMsg();								*/ /* del T.Tazaki 2004/12/20 for back to prompt */
    }

    return iResult;
}

/****************************************************************************/
/* Format        : int c33_Cmd_Ver_Read (void)                              */
/* Arguments     : void                                                     */
/* Return value                                                             */
/*  C33_NO_ERR   : Success return                                           */
/*  C33_ERR      : Error return                                             */
/* Explanation   : The version information on a target is read.             */
/****************************************************************************/
int
c33_Cmd_Ver_Read (void)
{
    int iResult;

    /* Transmission of a message. */
    iResult = c33_SendMessage(C33_ID_VER_READ, 0, 0, 0, 0, 0, 0, 0);

    /* Reception of a message. */
    if (iResult == C33_NO_ERR){
        iResult = c33_RecvMessage(C33_ID_VER_READ, 0, 0, 0, 0, 0);
    }

    if (iResult != C33_NO_ERR)
    {
        c33_ErrorMsg();
    }
    else
    {
        /* Display version read information */
        if (c33TargetInfo.iVerRdDIAG == 0)
        {
            /* DIAG test ok */
            puts_filtered (C33_MSG_TARGET_DIAG_OK);
        }
        else if (c33TargetInfo.iVerRdDIAG == 1)
        {
            /* DIAG test omitted */
            puts_filtered (C33_MSG_TARGET_DIAG_OMITTED);
        }
        else
        {
            /* DIAG test error */
            if (c33TargetInfo.iVerRdSioErr == 1)
            {
                c33_error (C33_ERR_MSG_TARGET_DATA, c33TargetInfo.iVerRdSioTryCount);
                return C33_ERR;
            }
            else if (c33TargetInfo.iVerRdSioErr == 2)
            {
                c33_error (C33_ERR_MSG_TARGET_SIO, c33TargetInfo.iVerRdSioTryCount);
                return C33_ERR;
            }
            else
            {
                c33_error (C33_ERR_MSG_TARGET_TIME, c33TargetInfo.iVerRdSioTryCount);
                return C33_ERR;
            }
        }
    }
    return iResult;
}
/* 2006/06/05  M.Kai */
/****************************************************************************/
/* Format        : int get_gdb_startpath (void)                             */
/* Arguments                                                                */
/* int iType    : none                                                      */
/* Return value  path length                                                */
/* Explanation   : get path to "xxx.gdb" file.                              */
/****************************************************************************/
int
get_gdb_startpath(char* gdbpath)
{
	int pathlen;

#ifdef __CYGWIN__
	GetModuleFileName( NULL, gdbpath, 256);
	pathlen = strlen(gdbpath);
	gdbpath[pathlen-8 + 1] = '\0'; 	//8 means "/gdb.exe"
#else
	gdbpath[0] = 0;		/* add T.Tazaki 2006/07/13 */
#endif

	return pathlen - 8 + 1 ;

}
/* 2006/06/05  M.Kai */
/****************************************************************************/
/* Format        : int c33_Cmd_UserDef (void)                               */
/* Arguments                                                                */
/*  int iType    : none                                                     */
/* Return value                                                             */
/*  C33_NO_ERR   : Success return                                           */
/*  C33_ERR      : Error return                                             */
/* Explanation   : Excecute userdefine.gdb file.                            */
/****************************************************************************/
int
c33_Cmd_UserDef (int command)
{

    FILE *stream;
    char path[256];
    char *file;
	
	get_gdb_startpath(path);
	
	switch (command){
		
		case C33_USER_DEFINE:
			strcat(path, "userdefine.gdb");
			break;
				
		case C33_USER_COLD_RESET:
			strcat(path, "resetcold.gdb");
			break;
		
		case C33_USER_HOT_RESET:
			strcat(path, "resethot.gdb");
			break;
			
		default:								/* add T.Tazai 2007/07/12 */
			break;
	}

    file = tilde_expand (path);

    stream = fopen (file, FOPEN_RT);
    if (!stream) //File open is failed
    {
      return C33_ERR;
    }
    
    script_from_file (stream, file);

    return C33_NO_ERR;
}


/****************************************************************************/
/* Format        : int c33_Cmd_Reset (int iType)                            */
/* Arguments                                                                */
/*  int iType    : 0 - Hot reset, 1 - Cold reset							*/
/* Return value                                                             */
/*  C33_NO_ERR   : Success return                                           */
/*  C33_ERR      : Error return                                             */
/* Explanation   : Cold reset is performed.                                 */
/****************************************************************************/
int
c33_Cmd_Reset (int iType)
{
    int iResult;
    unsigned long ulPC;
    unsigned long ulTTBR = 0;

    if (iType == C33_ICD_COLD_RESET)
    {
        puts_filtered (C33_MSG_TARGET_COLD_RESET_INFO);
    }
    else if(iType == C33_ICD_HOT_RESET)
    {
        puts_filtered (C33_MSG_TARGET_HOT_RESET_INFO);
        ulTTBR = c33_ttbr_reg_get();
    }
	else
    {
        c33_error (C33_ERR_INTERNAL_ERR, "c33common.c", "c33_Cmd_Reset()");
        return C33_ERR;
    }

    /* Transmission of a message. */
    iResult = c33_SendMessage(C33_ID_RESET, iType, 0, 0, 0, 0, 0, 0);

    /* Reception of a message. */
    if (iResult == C33_NO_ERR){
        iResult = c33_RecvMessage(C33_ID_RESET, 0, 0, 0, 0, 0);
    }
    if (iResult == C33_NO_ERR)
    {
        /* check if ICD V3 and Target is Advanced macro. */ /* add T.Tazaki 2003/11/21 >>> */
        if ( c33TargetInfo.iIcd_open == C33_USB_OPEN && 
             (c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV || c33TargetInfo.ucAdvanced == C33_PRODUCT_PE)
              && (c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) == C33_ICD_VER3)
        {

            /* w[TTBR] --> PC */
            if (iType == C33_ICD_COLD_RESET)
            {
				/* add T.Tazaki 2005/01/19 >>>  */
                iResult = c33_SendMessage(C33_ID_SP_REG_READ, 0, 0, 0, 0, 0, 0, 0);
	            /* Reception of a message. */
	            if (iResult == C33_NO_ERR){
	                iResult = c33_RecvMessage(C33_ID_SP_REG_READ, 0, 0, 0, 0, 0);
	            }
		        ulTTBR = c33_ttbr_reg_get();
				/* add T.Tazaki 2005/01/19 <<< */
				
#if 0			/* del T.Tazaki 2005/01/19 >>> */
				if( c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV )
				{
	                iResult = c33_Cmd_ReadWrite(C33_READ, 0x20000000UL, 4, C33_WORD, &ulPC);
	            }
	            else
	            {
	                iResult = c33_Cmd_ReadWrite(C33_READ, 0xc00000UL, 4, C33_WORD, &ulPC);	/*add PE T.Tazaki 2004/08/09 */
				}
#endif			/* del T.Tazaki 2005/01/19 <<< */
                iResult = c33_Cmd_ReadWrite(C33_READ, ulTTBR, 4, C33_WORD, &ulPC);
            }
            else
            {
                iResult = c33_Cmd_ReadWrite(C33_READ, ulTTBR, 4, C33_WORD, &ulPC);
            }

            if (iResult == C33_NO_ERR)
            {
                /* register default value write to buffers. */
                c33_Default_reg_set();

                /* PC register write to buffers. */
                c33_pc_reg_set(ulPC);

                iResult = c33_SendMessage(C33_ID_REG_WRITE, 0, 0, 0, 0, 0, 0, 0);
                if (iResult == C33_NO_ERR)
                {
                    iResult = c33_RecvMessage(C33_ID_REG_WRITE, 0, 0, 0, 0, 0);
                    if (iResult == C33_NO_ERR)
                    {
                        /* Special register default value write to buffers. 
                           if HOT RESET then TTBR not update. */
                        c33_Default_special_reg_set(iType);

                        iResult = c33_SendMessage(C33_ID_SP_REG_WRITE, 0, 0, 0, 0, 0, 0, 0);

                        if (iResult == C33_NO_ERR)
                        {
                            iResult = c33_RecvMessage(C33_ID_SP_REG_WRITE, 0, 0, 0, 0, 0);
                        }
                    }
                }
            }
        }                                                   /* add T.Tazaki 2003/07/28 <<< */
	
	    puts_filtered (C33_MSG_TARGET_DONE);

        /* add T.Tazaki 2003/10/16 >>> */
        if ( c33TargetInfo.iIcd_open != 0 )
        {
            c33icd_WindowUpdate();
        }
        else
        {
            gdbsim_WindowUpdate();  /* add T.Tazaki 2003/10/20 */
        }
        /* add T.Tazaki 2003/10/16 <<< */


        /* >>>>> add iruma m.takeishi '03.04.28 */
        giFrameClear = 1;
        /* >>>>> add iruma m.takeishi '03.04.28 */
        c33_Gdbtk_Update (C33_UPDATE_REG);

    }


    if (iResult != C33_NO_ERR)
    {
        puts_filtered (C33_MSG_TARGET_FAILURE);
        c33_ErrorMsg();
    }

    /* Reset counters */
    g_ulClockCount[0] = 0;  /* cycle count total */
    g_ulClockCount[1] = 0;  /* 50ns time total */
    g_ulClockCount[2] = 0;  /* 1us time  total */
	g_ullClockCount = 0;  /* cycle count total */	/* ICDV6 add T.Tazaki 2007/07/03 */

    /* Clock Counter Reset : SIM */
    gulCycle = 0;

	/* c33 rsth -> next -> c33 rsth -> next bug fix add T.Tazaki 2008/06/17 : DDTS gnu33v32-00025 >>> */ 
	init_wait_for_inferior ();
	clear_proceed_status ();
	/* c33 rsth -> next -> c33 rsth -> next bug fix add T.Tazaki 2008/06/17 : DDTS gnu33v32-00025 <<< */ 

    return iResult;
}

/* set $pc=xxxx + next bug fix  add T.Tazaki 2008/06/17 >>> */
/****************************************************************************/
/* Format        : int c33_Cmd_Reset2(int iType)                            */
/* Arguments                                                                */
/*  int iType    : 0 - Hot reset, 1 - Cold reset							*/
/* Return value                                                             */
/*  C33_NO_ERR   : Success return                                           */
/*  C33_ERR      : Error return                                             */
/* Explanation   : Cold reset is performed.                                 */
/****************************************************************************/
int
c33_Cmd_Reset2 (int iType)
{
    int iResult;
    unsigned long ulPC;
    unsigned long ulTTBR = 0;

    if(iType == C33_ICD_HOT_RESET)
    {
        ulTTBR = c33_ttbr_reg_get();
    }
	else
    {
        c33_error (C33_ERR_INTERNAL_ERR, "c33common.c", "c33_Cmd_Reset()");
        return C33_ERR;
    }

    /* Transmission of a message. */
    iResult = c33_SendMessage(C33_ID_RESET, iType, 0, 0, 0, 0, 0, 0);

    /* Reception of a message. */
    if (iResult == C33_NO_ERR){
        iResult = c33_RecvMessage(C33_ID_RESET, 0, 0, 0, 0, 0);
    }
    if (iResult == C33_NO_ERR)
    {
        /* check if ICD V3 and Target is Advanced macro. */
        if ( c33TargetInfo.iIcd_open == C33_USB_OPEN && 
             (c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV || c33TargetInfo.ucAdvanced == C33_PRODUCT_PE)
              && (c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) == C33_ICD_VER3)
        {

            /* w[TTBR] --> PC */
            if (iType == C33_ICD_COLD_RESET)
            {
                iResult = c33_SendMessage(C33_ID_SP_REG_READ, 0, 0, 0, 0, 0, 0, 0);
	            /* Reception of a message. */
	            if (iResult == C33_NO_ERR){
	                iResult = c33_RecvMessage(C33_ID_SP_REG_READ, 0, 0, 0, 0, 0);
	            }
		        ulTTBR = c33_ttbr_reg_get();
				
                iResult = c33_Cmd_ReadWrite(C33_READ, ulTTBR, 4, C33_WORD, &ulPC);
            }
            else
            {
                iResult = c33_Cmd_ReadWrite(C33_READ, ulTTBR, 4, C33_WORD, &ulPC);
            }

            if (iResult == C33_NO_ERR)
            {
                /* register default value write to buffers. */
                c33_Default_reg_set();

                /* PC register write to buffers. */
                c33_pc_reg_set(ulPC);

                iResult = c33_SendMessage(C33_ID_REG_WRITE, 0, 0, 0, 0, 0, 0, 0);
                if (iResult == C33_NO_ERR)
                {
                    iResult = c33_RecvMessage(C33_ID_REG_WRITE, 0, 0, 0, 0, 0);
                    if (iResult == C33_NO_ERR)
                    {
                        /* Special register default value write to buffers. 
                           if HOT RESET then TTBR not update. */
                        c33_Default_special_reg_set(iType);

                        iResult = c33_SendMessage(C33_ID_SP_REG_WRITE, 0, 0, 0, 0, 0, 0, 0);

                        if (iResult == C33_NO_ERR)
                        {
                            iResult = c33_RecvMessage(C33_ID_SP_REG_WRITE, 0, 0, 0, 0, 0);
                        }
                    }
                }
            }
        }

        if ( c33TargetInfo.iIcd_open != 0 )
        {
            c33icd_WindowUpdate();
        }
        else
        {
            gdbsim_WindowUpdate();
        }


        giFrameClear = 1;
        c33_Gdbtk_Update (C33_UPDATE_REG);

    }

	init_wait_for_inferior ();
	clear_proceed_status ();

    return iResult;
}
/* set $pc=xxxx + next bug fix  add T.Tazaki 2008/06/17 <<<< */


/****************************************************************************/ /* add T.Tazaki 2004/01/16 */
/* Format        : int c33_Cmd_Reset_no_reg_write (int iType)               */
/* Arguments                                                                */
/*  int iType    : 0 - Hot reset, 1 - Cold reset                            */
/* Return value                                                             */
/*  C33_NO_ERR   : Success return                                           */
/*  C33_ERR      : Error return                                             */
/* Explanation   : Cold reset is performed (called by c33_Initialize)       */
/*				   Do not write default register to ICD.					*/
/****************************************************************************/
int
c33_Cmd_Reset_no_reg_write (int iType)
{
    int iResult;
    unsigned long ulTTBR = 0;

    if (iType == C33_ICD_COLD_RESET)
    {
        puts_filtered (C33_MSG_TARGET_COLD_RESET_INFO);
    }
    else if(iType == C33_ICD_HOT_RESET)
    {
        puts_filtered (C33_MSG_TARGET_HOT_RESET_INFO);
        ulTTBR = c33_ttbr_reg_get();
    }
    else
    {
        c33_error (C33_ERR_INTERNAL_ERR, "c33common.c", "c33_Cmd_Reset_no_reg_write()");
        return C33_ERR;
    }

    /* Transmission of a message. */
    iResult = c33_SendMessage(C33_ID_RESET, iType, 0, 0, 0, 0, 0, 0);

    /* Reception of a message. */
    if (iResult == C33_NO_ERR){
        iResult = c33_RecvMessage(C33_ID_RESET, 0, 0, 0, 0, 0);
    }

    if (iResult == C33_NO_ERR)
    {
        puts_filtered (C33_MSG_TARGET_DONE);

#if 0	/* del T.Tazaki 2004/10/15 >>> */
        /* add T.Tazaki 2003/10/16 >>> */
        if ( c33TargetInfo.iIcd_open != 0 )
        {
            c33icd_WindowUpdate();
        }
        else
        {
            gdbsim_WindowUpdate();  /* add T.Tazaki 2003/10/20 */
        }
        /* add T.Tazaki 2003/10/16 <<< */


        /* >>>>> add iruma m.takeishi '03.04.28 */
        giFrameClear = 1;
        /* >>>>> add iruma m.takeishi '03.04.28 */
        c33_Gdbtk_Update (C33_UPDATE_REG);
#endif
    }
    

    if (iResult != C33_NO_ERR){
        puts_filtered (C33_MSG_TARGET_FAILURE);
/*      c33_ErrorMsg();								*/ /* del T.Tazaki 2004/12/20 for back to prompt */
    }

    /* Reset counters */
    g_ulClockCount[0] = 0;  /* cycle count total */
    g_ulClockCount[1] = 0;  /* 50ns time total */
    g_ulClockCount[2] = 0;  /* 1us time  total */

    /* Clock Counter Reset : SIM */
    gulCycle = 0;

	/* c33 rsth -> next -> c33 rsth -> next bug fix add T.Tazaki 2008/06/17 : DDTS gnu33v32-00025 >>> */ 
	init_wait_for_inferior ();
	clear_proceed_status ();
	/* c33 rsth -> next -> c33 rsth -> next bug fix add T.Tazaki 2008/06/17 : DDTS gnu33v32-00025 <<< */ 

    c33TargetInfo.iKeyBreak = 0;	/* add T.Tazaki 2004/11/09  */

    return iResult;
}

/****************************************************************************/
/* Format        : int c33_Cmd_ReadWrite(int iReadWrite, unsigned long ulAddr, unsigned long ulLen, int iUnit, unsigned char* pcuMyAddr) */
/* Arguments                                                                */
/*  int iReadWrite           : 0 - Read, 1 - Write                          */
/*  unsigned long ulAddr     : Target address                               */
/*  unsigned long ulLen      : Data Length                                  */
/*  int iUnit                : C33_BYTE,C33_HALF_WORD,C33_WORD              */
/*  unsigned char* pcuMyAddr : Debugger address                             */
/* Return value                                                             */
/*  C33_NO_ERR   : Success return                                           */
/*  C33_ERR      : Error return                                             */
/* Explanation   : The memory read or write is performed.                   */
/****************************************************************************/
int
c33_Cmd_ReadWrite(int iReadWrite, unsigned long ulAddr, unsigned long ulLen, int iUnit, unsigned char* pcuMyAddr)
{
    int iResult = C33_NO_ERR;
    int iID;
	int iSizeOddFlag = 0;
	unsigned long ulLoadAddr;
	unsigned long ulLoadLen;

	/* check if Now USB Reading.  add T.Tazaki 2004/05/10 >>> */
	extern int g_iNow_OnUSBRead;
	
	if( g_iNow_OnUSBRead == 1 ) 
	{
    	return iResult;
	}
	/* check if Now USB Reading.  add T.Tazaki 2004/05/10 <<< */

	ulLoadAddr = ulAddr;
	ulLoadLen  = ulLen;

    if (iUnit == C33_WORD)
    {
        ulAddr &= 0xfffffffc;
        ulLen  &= 0xfffffffc;
    }
    else if (iUnit == C33_HALF_WORD)
    {
        ulAddr &= 0xfffffffe;
        ulLen  &= 0xfffffffe;
    }

    if (iReadWrite)
    {

        /* modify 2003/07/29 T.Tazaki >>> */
        if ( c33TargetInfo.iIcdLoad == 1 && (c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) == C33_ICD_VER3 )
        {
			if( ( ulLoadAddr % 2 ) == 1 )	/* odd address ? --> 1byte data write   add T.Tazaki 2004/01/09 */
			{
	            /* Memory write */
	            iResult = c33_SendMessage(C33_ID_DATA_WRITE, ulLoadAddr, 1, C33_BYTE, 0, pcuMyAddr, 0, 0);
	            if (iResult == C33_NO_ERR)
	            {
	                iResult = c33_RecvMessage(C33_ID_DATA_WRITE, ulLoadAddr, 1, C33_BYTE, 0, pcuMyAddr);
	            }
	            if (iResult == C33_NO_ERR)  /* add T.Tazaki 2003/08/05 */
	            {
					++ulLoadAddr;
					++pcuMyAddr;
					--ulLoadLen;
	            }
			}

			if( ( ulLoadLen % 2 ) == 1 )	/* odd data size ? --> 1byte data write   add T.Tazaki 2004/01/09 */
			{
				--ulLoadLen;
				iSizeOddFlag = 1;
			}

            if (iResult == C33_NO_ERR)
            {
				if( ulLoadLen > 0 )
				{
		            /* Memory write */
		            iResult = c33_SendMessage(C33_ID_DATA_WRITE_FAST, ulLoadAddr, ulLoadLen, iUnit, 0, pcuMyAddr, 0, 0);
		            if (iResult == C33_NO_ERR)
		            {
		                iResult = c33_RecvMessage(C33_ID_DATA_WRITE_FAST, ulLoadAddr, ulLoadLen, iUnit, 0, pcuMyAddr);
		                if (iResult == C33_NO_ERR)
		                {
		                    if( g_iIcdLoadFirst == 1 )  /* check if this ID is first.  */
		                    {
		                        g_iIcdLoadFirst = 0;    /* off */
		                    }
		                }
		            }
				}
			}

            if (iResult == C33_NO_ERR)
            {
				if( iSizeOddFlag == 1 )
				{
					ulLoadAddr += ulLoadLen;
					pcuMyAddr += ulLoadLen;

		            /* Memory write */
		            iResult = c33_SendMessage(C33_ID_DATA_WRITE, ulLoadAddr, 1, C33_BYTE, 0, pcuMyAddr, 0, 0);
		            if (iResult == C33_NO_ERR)
		            {
		                iResult = c33_RecvMessage(C33_ID_DATA_WRITE, ulLoadAddr, 1, C33_BYTE, 0, pcuMyAddr);
		            }
				}
			}
		}
        /* check if parallel load mode  ICDV2 only    add T.Tazaki 2003/10/01 */
        else if ( c33TargetInfo.iIcdLoad == 1 && g_iParallel_mode == 1 && ulLen >= 2)	/* size check more than 2 byte ?   add T.Tazaki 2004/02/09 */
        {
            iResult = c33_SendMessage(C33_ID_DATA_PARALLEL_WRITE, ulAddr, ulLen, 0, 0, 0, 0, 0);
            if (iResult == C33_NO_ERR)
            {
                /* Parallel Memory write */
                iResult = c33icd_ParallelSendData( pcuMyAddr, ulLen, iUnit );
                if (iResult == C33_NO_ERR)
                {
                    iResult = c33_RecvMessage(C33_ID_DATA_PARALLEL_WRITE, ulAddr, ulLen + 1, 0, 0, 0);
                }
            }
        }
        else
        {
            /* Memory write */
            iResult = c33_SendMessage(C33_ID_DATA_WRITE, ulAddr, ulLen, iUnit, 0, pcuMyAddr, 0, 0);
            if (iResult == C33_NO_ERR)
            {
                iResult = c33_RecvMessage(C33_ID_DATA_WRITE, ulAddr, ulLen, iUnit, 0, pcuMyAddr);
            }
            if (iResult == C33_NO_ERR)  /* add T.Tazaki 2003/08/05 */
            {
                g_iIcdLoadFirst = 1;    /* on */
            }
        }
        /* modify 2003/07/29 T.Tazaki <<< */
    }
    else
    {
#if 0 	/* this is slowB del 2004/10/07 T.Tazaki */
        if ( (c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) == C33_ICD_VER3 )
        {
            iID = C33_ID_DATA_READ_EXACT;
        }
        else
        {
           iID = C33_ID_DATA_READ;
        }
#endif
		/* add T.Tazaki 2004/11/01 >>> */
		if( g_iMemReadMode == 0 )	/* Normal read ? */
		{
	        iID = C33_ID_DATA_READ;
	    }
	    else
	    {
            iID = C33_ID_DATA_READ_EXACT;
        }
		/* add T.Tazaki 2004/11/01 <<< */
        
        /* Memory read */
        iResult = c33_SendMessage(iID, ulAddr, ulLen, iUnit, 0, pcuMyAddr, 0, 0);
        if (iResult == C33_NO_ERR)
        {
            iResult = c33_RecvMessage(iID, ulAddr, ulLen, iUnit, 0, pcuMyAddr);
        }
        if (iResult == C33_NO_ERR)  /* add T.Tazaki 2003/08/05 */
        {
            g_iIcdLoadFirst = 1;    /* on */
        }
    }

    if (iResult != C33_NO_ERR)
    {
        c33_ErrorMsg();
    }

//  if ( c33TargetInfo.iIcdLoad == 1 && iReadWrite == C33_WRITE)	/* del check load command 2004/06/02 T.Tazaki */
    if ( iReadWrite == C33_WRITE)									/* del check load command 2004/06/02 T.Tazaki */
    {
        c33_ReadWriteMapMem(C33_WRITE, ulAddr, ulLen, iUnit, pcuMyAddr);
    }
    return iResult;
}
/****************************************************************************/
/* Format        : int c33_Cmd_IcdV3ReadWrite(int iReadWrite, unsigned long ulAddr, unsigned long ulLen, int iUnit, unsigned char* pcuMyAddr) */
/* Arguments                                                                */
/*  int iReadWrite           : 0 - Read, 1 - Write                          */
/*  unsigned long ulAddr     : Target address                               */
/*  unsigned long ulLen      : Data Length                                  */
/*  int iUnit                : C33_BYTE,C33_HALF_WORD,C33_WORD              */
/*  unsigned char* pcuMyAddr : Debugger address                             */
/* Return value                                                             */
/*  C33_NO_ERR   : Success return                                           */
/*  C33_ERR      : Error return                                             */
/* Explanation   : The icdv3 memory read or write is performed.             */
/****************************************************************************/
int
c33_Cmd_IcdV3ReadWrite(int iReadWrite, unsigned long ulAddr, unsigned long ulLen, int iUnit, unsigned char* pcuMyAddr)
{
    int iResult = C33_NO_ERR;
	unsigned long ulLoadAddr;
	unsigned long ulLoadLen;


    if ( (c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) != C33_ICD_VER3 )
    {
		return C33_ERR;	/* not ICDV3 */
	}

	ulLoadAddr = ulAddr;
	ulLoadLen  = ulLen;

    if (iUnit == C33_WORD)
    {
        ulAddr &= 0xfffffffc;
        ulLen  &= 0xfffffffc;
    }
    else if (iUnit == C33_HALF_WORD)
    {
        ulAddr &= 0xfffffffe;
        ulLen  &= 0xfffffffe;
    }

    if (iReadWrite)
    {
        /* ICDV3 Memory write */
        iResult = c33_SendMessage(C33_ID_ICD_MEM_WRITE, ulAddr, ulLen, iUnit, 0, pcuMyAddr, 0, 0);
        if (iResult == C33_NO_ERR)
        {
	        iResult = c33_RecvMessage(C33_ID_ICD_MEM_WRITE, ulAddr, ulLen, iUnit, 0, pcuMyAddr);
        }
    }
    else
    {
        /* ICDV3 Memory read */
        iResult = c33_SendMessage(C33_ID_ICD_MEM_READ, ulAddr, ulLen, iUnit, 0, pcuMyAddr, 0, 0);
        if (iResult == C33_NO_ERR)
        {
            iResult = c33_RecvMessage(C33_ID_ICD_MEM_READ, ulAddr, ulLen, iUnit, 0, pcuMyAddr);
        }
    }

    if (iResult != C33_NO_ERR)
    {
        c33_ErrorMsg();
    }

    return iResult;
}

/****************************************************************************/
/* Format        : void c33_Gdbtk_Update (int iCondition)                   */
/* Arguments                                                                */
/*  int iCondition : Condition flag                                         */
/* Return value  : void                                                     */
/* Explanation   : The window is re-displayed.                              */
/****************************************************************************/
void
c33_Gdbtk_Update (int iCondition)
{
    extern Tcl_Interp *gdbtk_interp;

    g_Gdbtk_Update = 1;     /* Do not log file output */

/* Change Update process 2003/03/24 T.Tazaki : for CUI not change %pc >>> */
/*
    if (gdbtk_interp != NULL)
    {
        if( (iCondition & C33_UPDATE_REG) == C33_UPDATE_REG )
        {
            registers_changed ();
        }
        flush_cached_frames ();
        Tcl_Eval (gdbtk_interp, "gdbtk_update");
    }
*/
    if( (iCondition & C33_UPDATE_REG) == C33_UPDATE_REG )
    {
        registers_changed ();
    }
    /* >>>>> delete iruma m.takeishi '03.04.28 */
    /*
    flush_cached_frames ();
    */
    /* <<<<< delete iruma m.takeishi '03.04.28 */

    if (gdbtk_interp != NULL)
    {
        /* >>>>> change iruma m.takeishi '03.04.28 */
        if ( giFrameClear == 1 ) {
          flush_cached_frames ();
          giFrameClear = 0;
        }
        /* <<<<< change iruma m.takeishi '03.04.28 */
        Tcl_Eval (gdbtk_interp, "gdbtk_update");
    }
/* Change Update process 2003/03/24 T.Tazaki : for CUI not change %pc <<< */

    g_Gdbtk_Update = 0;

}

/****************************************************************************/
/* Format        : void c33_Gdbtk_Idle (void)                               */
/* Arguments     : void                                                     */
/* Return value  : void                                                     */
/* Explanation   : The window is only re-displayed.                         */
/****************************************************************************/
void
c33_Gdbtk_Idle (void)
{
    extern int (*ui_loop_hook) (int);
    if (ui_loop_hook)
    {
        ui_loop_hook(0);
    }
}

/****************************************************************************/
/* Format        : void c33_ErrorMsg (struct c33_target_info* pc33TargetInfo) */
/* Arguments     : void                                                     */
/* Return value  : void                                                     */
/* Explanation   : The message of an error code is displayed.               */
/****************************************************************************/
void
c33_ErrorMsg (void)
{
    unsigned long ulErrorAddr;
    unsigned long ulErrorCode;

    int iErrorCode = c33TargetInfo.iErrorCode;
    c33TargetInfo.iErrorCode = C33_NO_ERR;

    if (iErrorCode != C33_NO_ERR)
    {
        puts_filtered ("\n");
    }

    switch (iErrorCode)
    {
        case C33_ERROR_RS_NAK_FLEMMING:
        case C33_ERROR_RS_NAK_BCC:
        case C33_ERROR_RS_NAK_OVERRUN:
        case C33_ERROR_RS_NAK_TIMEOUT:
        case C33_ERROR_RS_SEND:
            c33_error (C33_ERR_MSG_COM_NAK, iErrorCode);
            break;

        case C33_ERROR_RS_STS_BUSY:
        case C33_ERROR_RS_STS_RUN:
            c33_error (C33_ERR_MSG_COM_STS, iErrorCode);
            break;

        case C33_ERROR_RS_MAX_SIZE_OVER:
            c33_error (C33_ERR_MSG_COM_RCV_DATA);
            break;

        case C33_ERROR_RS_COMMUNICATION:
            c33_error (C33_ERR_MSG_COM_FLEMING);
            break;

        case C33_ERROR_RS_CLOSE:
            c33_error (C33_ERR_MSG_COM_SYSTEM, iErrorCode);
            break;

        case C33_ERROR_RS_RECV_STS:
        case C33_ERROR_RS_ACK:
            c33_error (C33_ERR_MSG_COM_RCV_ACK, iErrorCode);
            break;

        case C33_ERROR_RS_TIMEOUT:
            c33_error (C33_ERR_MSG_COM_TIMEOUT, c33TargetInfo.iIcdErrorCode);
            break;

        case C33_ERROR_RS_BCC_CODE:
            c33_error (C33_ERR_MSG_COM_BCC_CODE);
            break;

        case C33_ERROR_RS_ERR:
            switch (c33TargetInfo.iIcdErrorCode)
            {
                case C33_ID_ERR_CODE_TARGET:
                    c33_error (C33_ERR_MSG_ICD_TARGET);
                    break;
                case C33_ID_ERR_CODE_RUN:
                    c33_error (C33_ERR_MSG_ICD_RUN);
                    break;
                case C33_ID_ERR_CODE_NOT_RUN:
                    c33_error (C33_ERR_MSG_ICD_NOT_RUN);
                    break;
/*add T.Tazaki 2005/08/22 : when target cancel, break mode time out >>> */
                case C33_ID_ERR_CODE_NOT_BREAK_MODE:
					g_iTargetResetDown = 1;
                    c33_error (C33_ERR_MSG_ICD_NOT_BREAK_MODE);
                    break;
/*add T.Tazaki 2005/08/22 : when target cancel, break mode time out <<< */
                case C33_ID_ERR_CODE_ID:
                    c33_error (C33_ERR_MSG_ICD_ID, c33TargetInfo.ucIcdErrorInfo[0]);
                    break;
                case C33_ID_ERR_CODE_SOFT_PC:
                    c33_DspErrBrkStatus();
                    break;
                case C33_ID_ERR_CODE_FLS_ERS:
                    c33_error (C33_ERR_MSG_ICD_FLS_ERS);
                    break;
                case C33_ID_ERR_CODE_FLS_WT:
                    ulErrorAddr = (c33TargetInfo.ucIcdErrorInfo[0] << 24) + (c33TargetInfo.ucIcdErrorInfo[1] << 16) + (c33TargetInfo.ucIcdErrorInfo[2] << 8) + (c33TargetInfo.ucIcdErrorInfo[3] << 0);
                    ulErrorCode = (c33TargetInfo.ucIcdErrorInfo[4] << 24) + (c33TargetInfo.ucIcdErrorInfo[5] << 16) + (c33TargetInfo.ucIcdErrorInfo[6] << 8) + (c33TargetInfo.ucIcdErrorInfo[7] << 0);
                    c33_error (C33_ERR_MSG_ICD_FLS_WT, ulErrorAddr, ulErrorCode);
                    break;
                case C33_ID_ERR_CODE_TRC_DIS:
                    c33_error (C33_ERR_MSG_ICD_TRC_DIS);
                    break;
				default:								/* add T.Tazai 2007/07/12 */
					break;
            }
            break;

        case C33_ERROR_USB_SEND:
            c33_error (C33_ERR_MSG_USB_SEND);
            break;
        case C33_ERROR_USB_RECV:
            c33_error (C33_ERR_MSG_USB_RECV);
            break;

        defualt:
            c33_error (C33_ERR_INTERNAL_ERR, "c33common.c", "c33_ErrorMsg()");
            break;
    }
}

/****************************************************************************/
/* Format        : void c33_wait_sec (int iWait)                            */
/* Arguments                                                                */
/*  int iWait    : Waiting time                                             */
/* Return value  : void                                                     */
/* Explanation   : It waits the specified seconds.                          */
/****************************************************************************/
void
c33_wait_sec (int iWait)
{
#if 0
    int iCnt = 0;
#endif
    time_t tmStart;
    time_t tmEnd;

    time(&tmStart);
    do
    {
        time(&tmEnd);
#if 0   /* del T.Tazaki 2003/06/19 */
        if( g_use_windows == 1 )  /* GUI Mode ? */
        {
            ++iCnt;
            if ( iCnt == 30 )
            {
                c33_Gdbtk_Update (C33_UPDATE_REG);
                iCnt = 0;
            }

            if( c33TargetInfo.iKeyBreak == 1 )
            {
                break;
            }
        }
#endif
    }
    while(difftime (tmEnd, tmStart) < iWait);

}

/****************************************************************************/
/* Format        : int c33_Cmd_SoftPcBreakSet (unsigned long ulAddr)        */
/* Arguments                                                                */
/*  unsigned long ulAddr : Break address                                    */
/* Return value                                                             */
/*  C33_NO_ERR   : Success return                                           */
/*  C33_ERR      : Error return                                             */
/* Explanation   : A soft PC break is set up.                               */
/****************************************************************************/
int
c33_Cmd_SoftPcBreakSet (unsigned long ulAddr)
{
    int i;
    int iResult = C33_NO_ERR;
	/* add T.Tazaki 2005/05/06 >>> */
	unsigned short usCode;
	/* add T.Tazaki 2005/05/06 <<< */
    ulAddr &= 0xfffffffe;


#ifdef C33_DEBUG
	printf_filtered( "1:c33_Cmd_SoftPcBreakSet ulAdd=[%x]\n",ulAddr );
#endif

    /* Search a free break */
    for (i = 0 ; i < C33_SOFT_PC_BREAK_MAX ; i++)
    {
        if ( c33TargetInfo.ulSoftPcBreak[i] == ulAddr )
        {
            break;
        }
        if ( c33TargetInfo.ulSoftPcBreak[i] == C33_BREAK_DISABLE )
        {
            break;
        }
    }

    /* Check break address*/
    if ( c33_BreakAddrCheck(ulAddr, C33_SOFT_PC_BREAK) == C33_ERR )
    {
        iResult = C33_ERR;
    }
    else
    {
        if ( i == C33_SOFT_PC_BREAK_MAX )
        {
            c33_error (C33_ERR_MSG_BRK_SOFT_MAX);
            iResult = C33_ERR;
            return iResult;
        }
        else
        {
            c33TargetInfo.ulSoftPcBreakTemp[i] = ulAddr;

            if(c33TargetInfo.iIcd_open != 0)
            {
				/* change T.Tazaki 2005/05/06 >>> */

			    /* read original code */
			    
#if 0			/* bug fix 2007/08/13 T.Tazaki */
//			    iResult = c33_Cmd_ReadWrite(C33_READ, ulAddr, 2, C33_HALF_WORD, (unsigned char*)&usCode );
			 	iResult = c33_ReadWriteMapMem(C33_READ, ulAddr, C33_HALF_WORD, C33_HALF_WORD, (unsigned char*)&usCode);			/* speed up T.Tazaki 2005/05/12 */
#else		
				/* MAP AREA ? */
			 	iResult = c33_ReadWriteMapMem(C33_READ, ulAddr, C33_HALF_WORD, C33_HALF_WORD, (unsigned char*)&usCode);
			    if( iResult == C33_NO_ERR )
				{
					/* YES! MAP AREA */
				    iResult = c33_Cmd_ReadWrite(C33_READ, ulAddr, 2, C33_HALF_WORD, (unsigned char*)&usCode );
				}
#endif
			    if( iResult == C33_NO_ERR )
			    {
			        /* save code */
			        c33TargetInfo.usSoftPcBreakOrgCode[i] = usCode;
			        c33TargetInfo.ulSoftPcBreak[i]        = ulAddr;
			        /* write "BRK" */
			        usCode = SOFT_BREAK_INST;
			        iResult = c33_Cmd_ReadWrite(C33_WRITE, ulAddr, 2, C33_HALF_WORD, (unsigned char*)&usCode);

			    }

#if 0
                iResult = c33_SendMessage(C33_ID_SOFT_PC_BREAK_SET, ulAddr, i + 1, 0, 0, 0, 0, 0);
                if (iResult == C33_NO_ERR)
                {
                    iResult = c33_RecvMessage(C33_ID_SOFT_PC_BREAK_SET, 0, 0, 0, 0, 0);
                }
#endif
				/* change T.Tazaki 2005/05/06 <<< */
            }
            else if(c33TargetInfo.iSim_open != 0)
            {
                c33_SimSoftPcBreakSet(ulAddr);
            }
        }
    }

    /* Save address */
    if (iResult == C33_NO_ERR)
    {
        c33TargetInfo.ulSoftPcBreak[i] = ulAddr;
    }
    else
    {

#ifdef C33_DEBUG
		printf_filtered( "c33_Cmd_SoftPcBreakSet ERROR\n" );
#endif
		/* all soft break clear               add T.Tazaki 2004/01/07   for ROM area soft break set >>> */
        if(c33TargetInfo.iIcd_open != 0)
        {
		    for (i = 0 ; i < C33_SOFT_PC_BREAK_MAX ; i++)
		    {
		        if ( c33TargetInfo.ulSoftPcBreak[i] != C33_BREAK_DISABLE )
		        {
					/* change T.Tazaki 2005/05/06 >>> */
					
			        /* write Original code */
			        
			        usCode = c33TargetInfo.usSoftPcBreakOrgCode[i];
			        iResult = c33_Cmd_ReadWrite(C33_WRITE, ulAddr, 2, C33_HALF_WORD, (unsigned char*)&usCode);
					c33TargetInfo.ulSoftPcBreak[i] = C33_BREAK_DISABLE;

#if 0
		            iResult = c33_SendMessage(C33_ID_SOFT_PC_BREAK_CLR, i + 1, 0, 0, 0, 0, 0, 0);
		            if (iResult == C33_NO_ERR)
		            {
		                iResult = c33_RecvMessage(C33_ID_SOFT_PC_BREAK_CLR, 0, 0, 0, 0, 0);
		            }
#endif
					/* change T.Tazaki 2005/05/06 <<< */
		        }
	        }
	    }
		/* add T.Tazaki 2004/01/07  <<< */
	    
        c33_ErrorMsg();
    }

    return iResult;
}

/****************************************************************************/
/* Format        : int c33_Cmd_SoftPcBreakClear (unsigned long ulAddr)      */
/* Arguments                                                                */
/*  unsigned long ulAddr : Break address                                    */
/* Return value                                                             */
/*  C33_NO_ERR   : Success return                                           */
/*  C33_ERR      : Error return                                             */
/* Explanation   : A soft PC break is clear.                                */
/****************************************************************************/
int
c33_Cmd_SoftPcBreakClear (unsigned long ulAddr)
{
    int i;
    int iResult = C33_NO_ERR;
	/* add T.Tazaki 2005/05/06 >>> */
	unsigned short usCode;
	/* add T.Tazaki 2005/05/06 <<< */
    ulAddr &= 0xfffffffe;


    /* Search same address */
    for (i = 0 ; i < C33_SOFT_PC_BREAK_MAX ; i++)
    {
        if ( c33TargetInfo.ulSoftPcBreak[i] == ulAddr )
        {
            break;
        }
    }

    if ( i == C33_SOFT_PC_BREAK_MAX )
    {
		/* when this function is called by continue_command, not error   add T.Tazaki 2005/05/11 */
		if( g_iCalledByContinueCmd == 0 )
		{
	        c33_error (C33_ERR_MSG_BRK_SOFT_CLR, (int)ulAddr);
	        iResult = C33_ERR;
	    }
        return iResult;
    }
    else
    {
        if(c33TargetInfo.iIcd_open != 0)
        {
			/* change T.Tazaki 2005/05/06 >>> */
			
	        /* write Original code */
	        
	        usCode = c33TargetInfo.usSoftPcBreakOrgCode[i];

	        iResult = c33_Cmd_ReadWrite(C33_WRITE, ulAddr, 2, C33_HALF_WORD, (unsigned char*)&usCode);
			c33TargetInfo.ulSoftPcBreak[i] = C33_BREAK_DISABLE;
#if 0
            iResult = c33_SendMessage(C33_ID_SOFT_PC_BREAK_CLR, i + 1, 0, 0, 0, 0, 0, 0);
            if (iResult == C33_NO_ERR)
            {
                iResult = c33_RecvMessage(C33_ID_SOFT_PC_BREAK_CLR, 0, 0, 0, 0, 0);
            }
#endif
			/* change T.Tazaki 2005/05/06 <<< */

        }
        else if(c33TargetInfo.iSim_open != 0)
        {
            c33_SimSoftPcBreakClear( ulAddr );
        }
    }

    /* Save address */
    if (iResult == C33_NO_ERR)
    {
        c33TargetInfo.ulSoftPcBreak[i] = C33_BREAK_DISABLE;
    }
    else
    {
        c33_ErrorMsg();
    }

    return iResult;
}


/****************************************************************************/
/* Format        : int c33_Cmd_HardPcBreakSet (unsigned long ulAddr)        */
/* Arguments                                                                */
/*  unsigned long ulAddr : Break address                                    */
/* Return value                                                             */
/*  C33_NO_ERR   : Success return                                           */
/*  C33_ERR      : Error return                                             */
/* Explanation   : A hard PC break is set up.                               */
/****************************************************************************/
int
c33_Cmd_HardPcBreakSet (unsigned long ulAddr)
{
    int i;
    int iResult = C33_NO_ERR;
    int iID;
    ulAddr &= 0xfffffffe;

#ifdef C33_DEBUG
	printf_filtered( "1:c33_Cmd_HardPcBreakSet ulAdd=[%x]\n",ulAddr );

	printf_filtered( "c33TargetInfo.ulHardPcBreak[0]=[%x]\n",c33TargetInfo.ulHardPcBreak[0] );
	printf_filtered( "c33TargetInfo.ulHardPcBreak[1]=[%x]\n",c33TargetInfo.ulHardPcBreak[1] );
#endif

    /* Search a free break */
    for (i = 0 ; i < C33_HARD_PC_BREAK_MAX ; i++)
    {
        if ( c33TargetInfo.ulHardPcBreak[i] == C33_BREAK_DISABLE )
        {
            break;
        }
    }

    /* Check break address*/
    if ( c33_BreakAddrCheck(ulAddr, C33_HARD_PC_BREAK) == C33_ERR )
    {
        iResult = C33_ERR;
    }
    else
    {
        if ( i == C33_HARD_PC_BREAK_MAX )
        {
            c33_error (C33_ERR_MSG_BRK_HARD_MAX);
            iResult = C33_ERR;
            return iResult;
        }
        else
        {
            if ( i == 0 )
            {
                /* Set hard PC break 1 */
                iID = C33_ID_HARD_PC_BREAK_SET;
            }
            else
            {
                /* Set hard PC break 2 */
                iID = C33_ID_HARD_PC_BREAK2;
            }
            if(c33TargetInfo.iIcd_open != 0)
            {
                iResult = c33_SendMessage(iID, ulAddr, 0, 0, 0, 0, 0, 0);
                if (iResult == C33_NO_ERR)
                {
                    iResult = c33_RecvMessage(iID, 0, 0, 0, 0, 0);
                }
            }
        }
    }

    /* Save address */
    if (iResult == C33_NO_ERR)
    {
        c33TargetInfo.ulHardPcBreak[i] = ulAddr;
    }
    else
    {
        c33_ErrorMsg();
    }

    if ( iResult != C33_NO_ERR )
    {
        return 1;
    }
    return 0;
}
/****************************************************************************/
/* Format        : int c33_Cmd_HardPcBreakClear (unsigned long ulAddr)      */
/* Arguments                                                                */
/*  unsigned long ulAddr : Break address                                    */
/* Return value                                                             */
/*  C33_NO_ERR   : Success return                                           */
/*  C33_ERR      : Error return                                             */
/* Explanation   : A hard PC break is clear.                                */
/****************************************************************************/
int
c33_Cmd_HardPcBreakClear (unsigned long ulAddr)
{
    int i;
    int iResult = C33_NO_ERR;
    int iID;
    ulAddr &= 0xfffffffe;

#ifdef C33_DEBUG
	printf_filtered( "1:c33_Cmd_HardPcBreakClear ulAdd=[%x]\n",ulAddr );
#endif

    /* Search same address */
    for (i = 0 ; i < C33_HARD_PC_BREAK_MAX ; i++)
    {
        if ( c33TargetInfo.ulHardPcBreak[i] == ulAddr )
        {
            break;
        }
    }

    if ( i == C33_HARD_PC_BREAK_MAX )
    {
        c33_error (C33_ERR_MSG_BRK_HARD_CLR, (int)ulAddr);
        iResult = C33_ERR;
        return iResult;
    }
    else
    {
        if ( i == 0 )
        {
            /* Clear hard PC break 1 */
            iID = C33_ID_HARD_PC_BREAK_CLR;
        }
        else
        {
            /* Clear hard PC break 2 */
            iID = C33_ID_HARD_PC_BREAK2;
        }
        if(c33TargetInfo.iIcd_open != 0)
        {
            iResult = c33_SendMessage(iID, C33_BREAK_DISABLE, 0, 0, 0, 0, 0, 0);
            if (iResult == C33_NO_ERR)
            {
                iResult = c33_RecvMessage(iID, 0, 0, 0, 0, 0);
            }
        }
    }

    /* Save address */
    if (iResult == C33_NO_ERR)
    {
        c33TargetInfo.ulHardPcBreak[i] = C33_BREAK_DISABLE;
    }
    else
    {
        c33_ErrorMsg();
    }

    if ( iResult != C33_NO_ERR )
    {
        return 1;
    }
    return 0;
}

/****************************************************************************/
/* Format        : int c33_insert_hw_breakpoint (CORE_ADDR ulAddr, void* shadow) */
/* Arguments                                                                */
/*  CORE_ADDR ulAddr : Break address                                        */
/*  void* shadow     : Does not use                                         */
/* Return value  : 0 Success return, 1 Error return                         */
/* Explanation   : A hard PC break is set up.                               */
/****************************************************************************/
int
c33_insert_hw_breakpoint (CORE_ADDR ulAddr, void* shadow)
{
    int i;
    int iResult;
    int iID;
    ulAddr &= 0xfffffffe;

#ifdef C33_DEBUG
	printf_filtered( "1:c33_insert_hw_breakpoint ulAddr=[%x]\n",ulAddr );
#endif

    if( g_iSamePCMode == 1 )  
    {
#ifdef C33_DEBUG
	printf_filtered( "SKIP\n" );
#endif
        return 0;
    }

    /* Search a free break */
    for (i = 0 ; i < C33_HARD_PC_BREAK_MAX ; i++)
    {
        if ( c33TargetInfo.ulHardPcBreak[i] == C33_BREAK_DISABLE )
        {
            break;
        }
    }

    /* Check break address*/
    if ( c33_BreakAddrCheck(ulAddr, C33_HARD_PC_BREAK) == C33_ERR )
    {
        iResult = C33_ERR;
    }
    else
    {
        if ( i == C33_HARD_PC_BREAK_MAX )
        {
            c33_error (C33_ERR_MSG_BRK_HARD_MAX);
            iResult = C33_ERR;
            return iResult;
        }
        else
        {
            if ( i == 0 )
            {
                /* Set hard PC break 1 */
                iID = C33_ID_HARD_PC_BREAK_SET;
            }
            else
            {
                /* Set hard PC break 2 */
                iID = C33_ID_HARD_PC_BREAK2;
            }
            iResult = c33_SendMessage(iID, ulAddr, 0, 0, 0, 0, 0, 0);
            if (iResult == C33_NO_ERR)
            {
                iResult = c33_RecvMessage(iID, 0, 0, 0, 0, 0);
            }
        }
    }

    /* Save address */
    if (iResult == C33_NO_ERR)
    {
        c33TargetInfo.ulHardPcBreak[i] = ulAddr;
    }
    else
    {
        c33_ErrorMsg();
    }

    if ( iResult != C33_NO_ERR )
    {
        return 1;
    }
    return 0;
}

/****************************************************************************/
/* Format        : int c33_remove_hw_breakpoint (CORE_ADDR ulAddr, void* shadow) */
/* Arguments                                                                */
/*  CORE_ADDR ulAddr : Break address                                        */
/*  void* shadow     : Does not use                                         */
/* Return value  : 0 Success return, 1 Error return                         */
/* Explanation   : A hard PC break is clear.                                */
/****************************************************************************/
int
c33_remove_hw_breakpoint (CORE_ADDR ulAddr, void* shadow)
{
    int i;
    int iResult;
    int iID;
    ulAddr &= 0xfffffffe;

#ifdef C33_DEBUG
	printf_filtered( "1:c33_remove_hw_breakpoint ulAddr=[%x]\n",ulAddr );
#endif
    if( g_iSamePCMode == 1 ) 
    {
#ifdef C33_DEBUG
	printf_filtered( "SKIP\n" );
#endif
        return 0;
    }

    /* Search same address */
    for (i = 0 ; i < C33_HARD_PC_BREAK_MAX ; i++)
    {
        if ( c33TargetInfo.ulHardPcBreak[i] == ulAddr )
        {
            break;
        }
    }

    if ( i == C33_HARD_PC_BREAK_MAX )
    {
        c33_error (C33_ERR_MSG_BRK_HARD_CLR, (int)ulAddr);
        iResult = C33_ERR;
        return iResult;
    }
    else
    {
        if ( i == 0 )
        {
            /* Clear hard PC break 1 */
            iID = C33_ID_HARD_PC_BREAK_CLR;
        }
        else
        {
            /* Clear hard PC break 2 */
            iID = C33_ID_HARD_PC_BREAK2;
        }
        iResult = c33_SendMessage(iID, C33_BREAK_DISABLE, 0, 0, 0, 0, 0, 0);
        if (iResult == C33_NO_ERR)
        {
            iResult = c33_RecvMessage(iID, 0, 0, 0, 0, 0);
        }
        
    }

    /* Save address */
    if (iResult == C33_NO_ERR)
    {
        c33TargetInfo.ulHardPcBreak[i] = C33_BREAK_DISABLE;
    }
    else
    {
        c33_ErrorMsg();
    }

    if ( iResult != C33_NO_ERR )
    {
        return 1;
    }
    return 0;
}

/****************************************************************************/
/* Format        : static void c33_Log(int iParamNum, char *cFileName )     */
/* Arguments                                                                */
/*  int iParamNum             : Parameter number                            */
/*  char *cFileName           : Log output file name                        */
/* Return value  : void                                                     */
/* Explanation   : Gdb log out start / end                                  */
/****************************************************************************/
static void
c33_Log(int iParamNum, char *cFileName )
{
    FILE    *fpLogfileTemp = NULL;
    

    /* Check number of parameter */
    if ( iParamNum != 1 && iParamNum != 2 )
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }

    if ( iParamNum == 2 )
    {
        /* Log start */
        if( cFileName == NULL )
        {
            c33_error (C33_ERR_COMMAND_PARAM);
            return;
        }
        if( strlen( cFileName ) > 256 )
        {
            c33_error (C33_ERR_COMMAND_PARAM);
            return;
        }
        
        /* already Log on */
        if (fpLogfile != NULL)
        {
            /* file open error */
            c33_error (C33_ERR_FILE_OPEN, cFileName);
            return;
        }
        fpLogfileTemp = fopen(cFileName, "wb");
        if (fpLogfileTemp == NULL)
        {
            /* file open error */
            c33_error (C33_ERR_FILE_OPEN, cFileName);
            return;
        }
        printf_filtered( C33_MSG_LOG_ON );
        fpLogfile = fpLogfileTemp;

    }
    else
    {
        /* Log end */
        if (fpLogfile != NULL)
        {
            fclose( fpLogfile );
            fpLogfile = NULL;
        }
        printf_filtered( C33_MSG_LOG_OFF );

    }

}

/****************************************************************************/
/* Format        : void c33_LogOut_String( void ) 							*/
/* Arguments                                                                */
/* Return value  : void                                                     */
/* Explanation   : Log file output. called by TCL(GUI mode)gdbtk-cmds.c     */
/****************************************************************************/
void
c33_LogOut_String()
{
    extern Tcl_Interp *gdbtk_interp;
    char    szbuf[3200];
    char    *pszbuf;

	pszbuf = szbuf;
	
	/* get log string in console.itb( Console::insert{} ) */
    pszbuf = Tcl_GetVar(gdbtk_interp, "c33_logout_string", TCL_GLOBAL_ONLY);
	
	if( pszbuf && strlen(pszbuf) <= 3000 && fpLogfile )		/* Change T.Tazaki 2007/06/25 */
	{
		if ( g_com_file_error == 0 ) /* add T.Tazaki 2007/06/25 
									 "Error in sourced command file"? */
		{
			c33_LogConvert( pszbuf );
			fprintf( fpLogfile,"%s", szLogbuf );
		}
		g_com_file_error = 0;		/* tONA      add T.Tazaki 2007/06/25  */
	}
}

/****************************************************************************/
/* Format        : static void c33_MemoryFill(int iParamNum, int iType, unsigned long ulStartAddr, unsigned long ulEndAddr, unsigned long ulData) */
/* Arguments                                                                */
/*  int iParamNum             : Parameter number                            */
/*  int iType                 : Byte,half word,word                         */
/*  unsigned long ulStartAddr : Start address                               */
/*  unsigned long ulEndAddr   : End address                                 */
/*  unsigned long ulData      : Fill data                                   */
/* Return value  : void                                                     */
/* Explanation   : Memory fill execution.                                   */
/****************************************************************************/
static void
c33_MemoryFill(int iParamNum, int iType, unsigned long ulStartAddr, unsigned long ulEndAddr, unsigned long ulData)
{
    int iResult;
    unsigned long ulSize;
    unsigned long ulSize2;

    /* Check number of parameter */
    if ( iParamNum != 4 )
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }

    /* Check address */
    if ( ulStartAddr > ulEndAddr )
    {
        c33_error (C33_ERR_COMMAND_ADDR);
        return;
    }

    /* Check boundary */
    if ( iType == C33_BYTE)
    {
        ulData &= 0x000000ff;
    }
    else if ( iType == C33_HALF_WORD)
    {
        ulData &= 0x0000ffff;
        ulStartAddr &= 0xfffffffe;
        ulEndAddr &= 0xfffffffe;
    }
    else if ( iType == C33_WORD)
    {
        ulData &= 0xffffffff;
        ulStartAddr &= 0xfffffffc;
        ulEndAddr &= 0xfffffffc;
    }

    printf_filtered (C33_MSG_FILL, (int)ulStartAddr, (int)ulEndAddr, (int)ulData);

    /* Get fill size */
    ulSize = ulEndAddr - ulStartAddr + iType;

    /* check if ICE is not ICDV3.   */
    if ( (c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) != C33_ICD_VER3 )       /* add T.Tazaki 2003/10/15 : ICDV3 USB STOP */
    {
        /*======================*/
        /* SIM, ICD2 , MON      */
        /*======================*/
        
        iResult = c33_SendMessage(C33_ID_DATA_FILL, iType, ulStartAddr, ulSize, ulData, 0, 0, 0);
        if (iResult == C33_NO_ERR)
        {
            iResult = c33_RecvMessage(C33_ID_DATA_FILL, 0, 0, 0, 0, 0);
        }

        if (iResult == C33_NO_ERR)
        {
            puts_filtered (C33_MSG_TARGET_DONE);
        }
        else
        {
            /* add T.Tazaki 2003/03/18  >>> */
            if( c33TargetInfo.iErrorCode == C33_ERROR_RS_KEY_BREAK )    
            {
                iResult = c33_SendMessage(C33_ID_CAN, 0, 0, 0, 0, 0, 0, 0);
                if (iResult == C33_NO_ERR)
                {
                    iResult = c33_RecvMessage(C33_ID_CAN, 0, 0, 0, 0, 0);
                }
                puts_filtered( C33_MSG_TARGET_CANCEL);
            }
            /* add T.Tazaki 2003/03/18  <<< */
            else
            {
                puts_filtered (C33_MSG_TARGET_FAILURE);
                c33_ErrorMsg();
            }
        }
    }
    else
    {
        /*======================*/
        /* ICD3                 */
        /*======================*/

        c33_InitCancel();   /* Cancel Check Init */

        do
        {
            if( ulSize >= 0x100000 )
            {
                ulSize2 = 0x100000;
                ulSize -= 0x100000;
            }
            else
            {
                ulSize2 = ulSize;
                ulSize = 0;
            }
            
            iResult = c33_SendMessage(C33_ID_DATA_FILL, iType, ulStartAddr, ulSize2, ulData, 0, 0, 0);
            if (iResult == C33_NO_ERR)
            {
                iResult = c33_RecvMessage(C33_ID_DATA_FILL, 0, 0, 0, 0, 0);
            }
            if (iResult != C33_NO_ERR)
            {
                break;
            }
            
            ulStartAddr += ulSize2;
            
            /* check STOP button */
            if (c33TargetInfo.iKeyBreak == 1 || c33_CheckCancel( 0 ) == 1 )
            {
                /* Cancel */
                c33TargetInfo.iErrorCode = C33_ERROR_RS_KEY_BREAK;
                break;
            }
        }while( ulSize > 0 );
        
        if( c33TargetInfo.iErrorCode == C33_ERROR_RS_KEY_BREAK )    
        {
            puts_filtered( C33_MSG_TARGET_CANCEL);
        }
        else
        {
            if (iResult == C33_NO_ERR)
            {
                puts_filtered (C33_MSG_TARGET_DONE);
            }
            else
            {
                puts_filtered (C33_MSG_TARGET_FAILURE);
                c33_ErrorMsg();
            }
        }

        c33_EndCancel();    /* Cancel Check End */

    }
    
    /* Update windows */
    if( g_use_windows == 1 )  /* GUI Mode ? */
    {
        if(c33TargetInfo.iIcd_open != C33_NOT_OPEN)
        {
            c33_Gdbtk_Update (C33_UPDATE_REG);
        }
        else if (c33TargetInfo.iSim_open != 0)
        {
            sim_Gdbtk_Update ();
        }
    }
}

/****************************************************************************/
/* Format        : static void c33_MemoryMove(int iParamNum, int iType, unsigned long ulStartAddr, unsigned long ulEndAddr, unsigned long ulDstAddr) */
/* Arguments                                                                */
/*  int iParamNum             : Parameter number                            */
/*  int iType                 : Byte,half word,word                         */
/*  unsigned long ulStartAddr : Start address                               */
/*  unsigned long ulEndAddr   : End address                                 */
/*  unsigned long ulDstAddr   : Destination address                         */
/* Return value  : void                                                     */
/* Explanation   : Memory move execution.                                   */
/****************************************************************************/
static void
c33_MemoryMove(int iParamNum, int iType, unsigned long ulStartAddr, unsigned long ulEndAddr, unsigned long ulDstAddr)
{
    int iResult;
    unsigned long ulSize;
    unsigned long ulSize2;

    /* Check number of parameter */
    if ( iParamNum != 4 )
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }

    /* Check address */
    if ( ulStartAddr > ulEndAddr )
    {
        c33_error (C33_ERR_COMMAND_ADDR);
        return;
    }

    /* Check boundary */
    if ( iType == C33_BYTE)
    {
    }
    else if ( iType == C33_HALF_WORD)
    {
        ulStartAddr &= 0xfffffffe;
        ulEndAddr &= 0xfffffffe;
        ulDstAddr &= 0xfffffffe;
    }
    else if ( iType == C33_WORD)
    {
        ulStartAddr &= 0xfffffffc;
        ulEndAddr &= 0xfffffffc;
        ulDstAddr &= 0xfffffffc;
    }

    printf_filtered (C33_MSG_MOVE, (int)ulStartAddr, (int)ulEndAddr, (int)ulDstAddr);

    /* Get move size */
    ulSize = ulEndAddr - ulStartAddr + iType;

    /* check if ICE is not ICDV3.   */
    if ( (c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) != C33_ICD_VER3 )       /* add T.Tazaki 2003/10/15 : ICDV3 USB STOP */
    {
        /*======================*/
        /* SIM, ICD2 , MON      */
        /*======================*/
        
        iResult = c33_SendMessage(C33_ID_DATA_MOVE, iType, ulStartAddr, ulDstAddr, ulSize, 0, 0, 0);
        if (iResult == C33_NO_ERR)
        {
            iResult = c33_RecvMessage(C33_ID_DATA_MOVE, 0, 0, 0, 0, 0);
        }

        if (iResult == C33_NO_ERR)
        {
            puts_filtered (C33_MSG_TARGET_DONE);
        }
        else
        {
            /* add T.Tazaki 2003/03/18  >>> */
            if( c33TargetInfo.iErrorCode == C33_ERROR_RS_KEY_BREAK )    
            {
                iResult = c33_SendMessage(C33_ID_CAN, 0, 0, 0, 0, 0, 0, 0);
                if (iResult == C33_NO_ERR)
                {
                    iResult = c33_RecvMessage(C33_ID_CAN, 0, 0, 0, 0, 0);
                }
                puts_filtered( C33_MSG_TARGET_CANCEL);
            }
            /* add T.Tazaki 2003/03/18  <<< */
            else
            {
                puts_filtered (C33_MSG_TARGET_FAILURE);
                c33_ErrorMsg();
            }
        }
    }
    else
    {
        /*======================*/
        /* ICD3                 */
        /*======================*/

        c33_InitCancel();   /* Cancel Check Init */

        do
        {
            if( ulSize >= 0x100000 )
            {
                ulSize2 = 0x100000;
                ulSize -= 0x100000;
            }
            else
            {
                ulSize2 = ulSize;
                ulSize = 0;
            }
            
            iResult = c33_SendMessage(C33_ID_DATA_MOVE, iType, ulStartAddr, ulDstAddr, ulSize2, 0, 0, 0);
            if (iResult == C33_NO_ERR)
            {
                iResult = c33_RecvMessage(C33_ID_DATA_MOVE, 0, 0, 0, 0, 0);
            }
            if (iResult != C33_NO_ERR)
            {
                break;
            }

            ulStartAddr += ulSize2;
            ulDstAddr += ulSize2;

            /* check STOP button */
            if (c33TargetInfo.iKeyBreak == 1 || c33_CheckCancel( 0 ) == 1 )
            {
                /* Cancel */
                c33TargetInfo.iErrorCode = C33_ERROR_RS_KEY_BREAK;
                break;
            }
        }while( ulSize > 0 );

        
        if( c33TargetInfo.iErrorCode == C33_ERROR_RS_KEY_BREAK )    
        {
            puts_filtered( C33_MSG_TARGET_CANCEL);
        }
        else
        {
            if (iResult == C33_NO_ERR)
            {
                puts_filtered (C33_MSG_TARGET_DONE);
            }
            else
            {
                puts_filtered (C33_MSG_TARGET_FAILURE);
                c33_ErrorMsg();
            }
        }

        c33_EndCancel();    /* Cancel Check End */

    }

    /* Update windows */
    if( g_use_windows == 1 )  /* GUI Mode ? */
    {
        if(c33TargetInfo.iIcd_open != C33_NOT_OPEN)
        {
            c33_Gdbtk_Update (C33_UPDATE_REG);
        }
        else if (c33TargetInfo.iSim_open != 0)
        {
            sim_Gdbtk_Update ();
        }
    }
}

/****************************************************************************/
/* Format        : int c33_insert_watchpoint (CORE_ADDR addr,int len,int type) */
/* Arguments                                                                */
/*  CORE_ADDR addr : Break address                                          */
/*  int len        : Watch length                                           */
/*  int type       : Access type                                            */
/* Return value  : 0 Success return, 1 Error return                         */
/* Explanation   : A data break is set up.                                  */
/****************************************************************************/
int c33_insert_watchpoint (CORE_ADDR addr,int len,int type)
{
    int iResult;



    if( g_iSamePCMode == 1 )   
    {
        return 0;
    }

    /* Check a data break */
    if ( c33TargetInfo.ulDataBreak == C33_BREAK_DISABLE )
    {
        iResult = c33_SendMessage(C33_ID_DATA_BREAK_SET, addr, type, 0, 0, 0, 0, 0);
        if (iResult == C33_NO_ERR)
        {
            iResult = c33_RecvMessage(C33_ID_DATA_BREAK_SET, 0, 0, 0, 0, 0);
        }
    }
    else
    {
        c33_error (C33_ERR_MSG_BRK_DATA_MAX);
        iResult = C33_ERR;
        return 1;
    }

    /* Save address */
    if (iResult == C33_NO_ERR)
    {
        c33TargetInfo.ulDataBreak = addr;
    }
    else
    {
        c33_ErrorMsg();
    }

    if ( iResult != C33_NO_ERR )
    {
        return 1;
    }
    return 0;
}

/****************************************************************************/
/* Format        : int c33_remove_watchpoint (CORE_ADDR addr,int len,int type) */
/* Arguments                                                                */
/*  int len      : Watch length                                             */
/*  int type     : Watch type                                               */
/* Return value  : 0 Success return, 1 Error return                         */
/* Explanation   : A data break is clear.                                   */
/****************************************************************************/
int c33_remove_watchpoint (CORE_ADDR addr,int len,int type)
{
    int iResult;


    if( g_iSamePCMode == 1 )   
    {
        return 0;
    }

    /* Cheak break address */
    if ( c33TargetInfo.ulDataBreak == addr )
    {
        iResult = c33_SendMessage(C33_ID_DATA_BREAK_CLR, 0, 0, 0, 0, 0, 0, 0);
        if (iResult == C33_NO_ERR)
        {
            iResult = c33_RecvMessage(C33_ID_DATA_BREAK_CLR, 0, 0, 0, 0, 0);
        }
    }
    else
    {
        if(c33TargetInfo.ulDataBreak != C33_BREAK_DISABLE )  /* add T.Tazaki 2003/06/25 bug fix */
        {
            c33_error (C33_ERR_MSG_BRK_DATA_CLR, (int)addr);
            iResult = C33_ERR;
            return 1;
        }
        else
        {
            return 0;
        }
    }

    /* Save address */
    if (iResult == C33_NO_ERR)
    {
        c33TargetInfo.ulDataBreak = C33_BREAK_DISABLE;
    }
    else
    {
        c33_ErrorMsg();
    }

    if ( iResult != C33_NO_ERR )
    {
        return 1;
    }
    return 0;
}

/****************************************************************************/
/* Format        : static void c33_ReadParameterFile(int iParamNum, char *pFilename) */
/* Arguments                                                                */
/*  int iParamNum   : Parameter number                                      */
/*  char *pFilename : parameter filename                                    */
/* Return value  : void                                                     */
/* Explanation   : Read parameter file.                                     */
/****************************************************************************/
static void
c33_ReadParameterFile(int iParamNum, char *pFilename)
{
    FILE *fpParaFile; /* pointer to parameter file */
    char cLine[C33_PARA_FILE_LINE_MAX + 1];
    char cParam1[C33_PARA_FILE_LINE_MAX + 1];
    char cParam2[C33_PARA_FILE_LINE_MAX + 1];
    char cParam3[C33_PARA_FILE_LINE_MAX + 1];
    char cParam4[C33_PARA_FILE_LINE_MAX + 1];
    char *pComment;
    int iStartAddr;
    int iEndAddr;
    struct c33_map_info stMapInfo;
    int iLineNum = 0;
    char cErrMsg[C33_PARA_FILE_LINE_MAX + 1];

    
    if (c33TargetInfo.iSim_open != 0)
    {
        /* Check c33 rpf command count */
        if ( g_iReadParameterFile == 1 )
        {
            c33_error (C33_ERR_COMMAND_ALREADY_EXEC);
            return;
        }
    }
    /* Check number of parameter */
    if ( iParamNum != 2 )
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }

    /* Open parameter file */
    if ( pFilename[0] == 0 )
    {
        c33_error (C33_ERR_FILE_OPEN, pFilename);
        return;
    }
    fpParaFile = fopen(pFilename, "r");
    if (fpParaFile == NULL)
    {
        /* file open error */
        c33_error (C33_ERR_FILE_OPEN, pFilename);
        return;
    }

    c33_ClearMap();

    /* Clear error message */
    cErrMsg[0] = 0;

    /* Read parameter file */
    while( fgets(cLine, C33_PARA_FILE_LINE_MAX, fpParaFile) != 0 )
    {
        /* Cut of comment */
        pComment = strstr(cLine,"#");
        if(pComment != 0)
        {
            *pComment = 0;
        }
        /* Cut of CR code */
        pComment = strstr(cLine,"\r");
        if(pComment != 0)
        {
            *pComment = 0;
        }
        pComment = strstr(cLine,"\n");
        if(pComment != 0)
        {
            *pComment = 0;
        }
        iLineNum ++;

        /* Set map default value */
        stMapInfo.iEndian = C33_LITTLE;
        stMapInfo.ulReadCycle = 7;
        stMapInfo.ulWriteCycle = 7;
        stMapInfo.ulDeviceSize = C33_HALF_WORD;
        cParam2[0] = 0;
        cParam3[0] = 0;

        iParamNum = sscanf(cLine, "%s %x %x %s %s %s", &cParam1[0], &iStartAddr, &iEndAddr, &cParam2[0], &cParam3[0], &cParam4[0]);
        if (iParamNum == EOF || iParamNum == 0)
        {
            continue;
        }
        else
        {
            /* Analyze parameter file */
            if (strcmp(cParam1,"C33_ADVANCED") == 0 && iParamNum == 1)
            {
                c33TargetInfo.ucAdvanced = C33_PRODUCT_ADV;
            }
            else if (strcmp(cParam1,"C33_PE") == 0 && iParamNum == 1)	/* add PE mode 2004/07/07 T.Tazaki */
            {
                c33TargetInfo.ucAdvanced = C33_PRODUCT_PE;
            }
            else if ( strcmp(cParam1,"ROM") == 0 || strcmp(cParam1,"RAM") == 0 || strcmp(cParam1,"IO") == 0 || strcmp(cParam1,"STACK") == 0 )
            {
                if (iParamNum > 2 && iParamNum < 6)
                {
                    /* Set attribute */
                    if (strcmp(cParam1,"ROM") == 0)
                    {
                        stMapInfo.iMapAttr = C33_ROM;
                    }
                    else if (strcmp(cParam1,"RAM") == 0)
                    {
                        stMapInfo.iMapAttr = C33_RAM;
                    }
                    else if (strcmp(cParam1,"IO") == 0)
                    {
                        stMapInfo.iMapAttr = C33_IO;
                    }
                    else if (strcmp(cParam1,"STACK") == 0 && iParamNum == 3)
                    {
                        stMapInfo.iMapAttr = C33_STACK;
                    }
                    else{
                        sprintf( cErrMsg, C33_ERR_MSG_PAR_FMT, iLineNum, cLine );
                        break;
                    }
                    /* Set start and end address */
                    stMapInfo.ulStartAddr = (unsigned long)iStartAddr;
                    stMapInfo.ulEndAddr = (unsigned long)iEndAddr;
                    if (stMapInfo.ulStartAddr > stMapInfo.ulEndAddr)
                    {
                        sprintf( cErrMsg, C33_ERR_MSG_PAR_ADDR, iLineNum, cLine );
                        break;
                    }
                    if (c33_StrToCycleData(cParam2,&stMapInfo) != C33_NO_ERR)
                    {
                        sprintf( cErrMsg, C33_ERR_MSG_PAR_FMT, iLineNum, cLine );
                        break;
                    }
                    if (c33_StrToCycleData(cParam3,&stMapInfo) != C33_NO_ERR)
                    {
                        sprintf( cErrMsg, C33_ERR_MSG_PAR_FMT, iLineNum, cLine );
                        break;
                    }
                    if (c33_AddMap(stMapInfo) != C33_NO_ERR)
                    {
                        sprintf( cErrMsg, C33_ERR_MSG_MEM_ERR );
                        break;
                    }
                }
                else
                {
                    sprintf( cErrMsg, C33_ERR_MSG_PAR_FMT, iLineNum, cLine );
                    break;
                }
            }
            else
            {
                sprintf( cErrMsg, C33_ERR_MSG_PAR_FMT, iLineNum, cLine );
                break;
            }
        }
    }

    /* file close */
    if (fpParaFile != NULL)
    {
        fclose(fpParaFile);
    }

    /* Print error message */
    if ( cErrMsg[0] != 0 )
    {
        c33_error (cErrMsg);
        return;
    }

    g_iReadParameterFile = 1;

}

/****************************************************************************/
/* Format        : static int c33_AddMap(struct c33_map_info stMapInfo)     */
/* Arguments                                                                */
/*  struct c33_map_info stMapInfo : Map structure                           */
/* Return value                                                             */
/*  C33_NO_ERR   : Success return                                           */
/*  C33_ERR      : Error return                                             */
/* Explanation   : Set map information to c33 target information structure. */
/****************************************************************************/
static int
c33_AddMap(struct c33_map_info stMapInfo)
{
    struct c33_map_info **ppstMapInfo;
    unsigned char* pMem = 0;

    ppstMapInfo = &c33TargetInfo.pMapInfo;

    /* NULL pointer is looked for. */
    while(*ppstMapInfo != NULL)
    {
        ppstMapInfo = &((*ppstMapInfo)->pMapNext);
    }

    /* malloc memory */
    if (stMapInfo.iMapAttr == C33_ROM || stMapInfo.iMapAttr == C33_RAM || stMapInfo.iMapAttr == C33_IO)
    {
        pMem = malloc(stMapInfo.ulEndAddr - stMapInfo.ulStartAddr + 1);
        if (pMem == 0)
        {
            return C33_ERR;
        }
        if(stMapInfo.iMapAttr == C33_ROM)
        {
            memset(pMem, 0xff, stMapInfo.ulEndAddr - stMapInfo.ulStartAddr + 1);
        }
        else if(stMapInfo.iMapAttr == C33_RAM)
        {
            memset(pMem, 0xaa, stMapInfo.ulEndAddr - stMapInfo.ulStartAddr + 1);
        }
        else if(stMapInfo.iMapAttr == C33_IO)
        {
            memset(pMem, 0x00, stMapInfo.ulEndAddr - stMapInfo.ulStartAddr + 1);
        }
    }

    /* Registere map information. */
    *ppstMapInfo = malloc(sizeof(struct c33_map_info));
    if (*ppstMapInfo == NULL)
    {
        if(pMem != NULL)
        {
            free(pMem);
        }
        return C33_ERR;
    }
    else
    {
        memcpy(*ppstMapInfo, &stMapInfo, sizeof(struct c33_map_info) );
        (*ppstMapInfo)->pMapNext = NULL;/* This is new last pointer */
        (*ppstMapInfo)->pMemBlock = pMem;
    }
    return C33_NO_ERR;
}

/****************************************************************************/
/* Format        : static int c33_StrToCycleData(char *cParam, struct c33_map_info *pstMapInfo) */
/* Arguments                                                                */
/*  char *cParam                    : Cycle string                          */
/*  struct c33_map_info *pstMapInfo : Map structure                         */
/* Return value                                                             */
/*  C33_NO_ERR   : Success return                                           */
/*  C33_ERR      : Error return                                             */
/* Explanation   : Set map structure from cycle string.                     */
/****************************************************************************/
static int
c33_StrToCycleData(char *cParam, struct c33_map_info *pstMapInfo)
{
    char cCycleStr[C33_PARA_FILE_LINE_MAX + 1];
    char cDevSize[C33_PARA_FILE_LINE_MAX + 1];
    int iReadCycle;
    int iWriteCycle;
    int iValidNum;

    if(strcmp(cParam,"B") == 0){
        pstMapInfo->iEndian = C33_BIG;
    }
    else{
        /* Strings case were xxB and xB */
        if(strlen(cParam) == 2){
            cCycleStr[5] = 0;
            cCycleStr[4] = cParam[1];
            cCycleStr[3] = ' ';
            cCycleStr[2] = cParam[0];
            cCycleStr[1] = ' ';
            cCycleStr[0] = cParam[0];
        }
        else if(strlen(cParam) == 3){
            cCycleStr[5] = 0;
            cCycleStr[4] = cParam[2];
            cCycleStr[3] = ' ';
            cCycleStr[2] = cParam[1];
            cCycleStr[1] = ' ';
            cCycleStr[0] = cParam[0];
        }
        else if(strlen(cParam) == 0){
            return(C33_NO_ERR);
        }
        else{
            return(C33_ERR);
        }

        iValidNum = sscanf(cCycleStr, "%x %x %s", &iReadCycle, &iWriteCycle, cDevSize);
        if(iValidNum != 3){
            return(C33_ERR);
        }
        else{
            if(strlen(cParam) == 2){
                iWriteCycle++;
            }
            if(iReadCycle > 16 || iWriteCycle > 16){
                return(C33_ERR);
            }
            pstMapInfo->ulReadCycle = (unsigned long)iReadCycle;
            pstMapInfo->ulWriteCycle = (unsigned long)iWriteCycle;

            if(strcmp(cDevSize,"B") == 0){
                pstMapInfo->ulDeviceSize = C33_BYTE;
            }
            else if(strcmp(cDevSize,"H") == 0){
                pstMapInfo->ulDeviceSize = C33_HALF_WORD;
            }
            else if(strcmp(cDevSize,"W") == 0){
                pstMapInfo->ulDeviceSize = C33_WORD;
            }
            else{
                return(C33_ERR);
            }
        }
    }
    return C33_NO_ERR;
}

/****************************************************************************/
/* Format        : static void c33_DspMapInfo(int iParamNum)                */
/* Arguments                                                                */
/*  int iParamNum   : Parameter number                                      */
/* Return value  : void                                                     */
/* Explanation   : Show map information.                                    */
/****************************************************************************/
static void
c33_ShowMapInfo(int iParamNum)
{
    int iDeviceSize = 0;
    struct c33_map_info *pstMapInfo;
    pstMapInfo   = c33TargetInfo.pMapInfo;

    /* Check number of parameter */
    if ( iParamNum != 1 )
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }

    if ( c33TargetInfo.ucAdvanced == C33_PRODUCT_STD )
    {
        puts_filtered(C33_MSG_MAP_INFO_STD);
    
    }
    else if ( c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV )
    {
        puts_filtered(C33_MSG_MAP_INFO_ADV);
    }
    else if ( c33TargetInfo.ucAdvanced == C33_PRODUCT_PE )	/* add PE  T.Tazaki 2003/11/18 */
    {
        puts_filtered(C33_MSG_MAP_INFO_PE);
    }

    puts_filtered(C33_MSG_MAP_INFO_HEADER);
    if (pstMapInfo == NULL)
    {
        puts_filtered(C33_MSG_MAP_INFO_NO_DATA);
    }

    while(pstMapInfo != NULL)
    {
        switch(pstMapInfo->ulDeviceSize)
        {
            case C33_BYTE:
                iDeviceSize = 8;
                break;
            case C33_HALF_WORD:
                iDeviceSize = 16;
                break;
            case C33_WORD:
                iDeviceSize = 32;
                break;
			default:								/* add T.Tazai 2007/07/12 */
				break;
        }
        switch(pstMapInfo->iMapAttr)
        {
            case C33_ROM:
                printf_filtered (C33_MSG_MAP_INFO_ROM, (int)pstMapInfo->ulStartAddr, (int)pstMapInfo->ulEndAddr, (int)pstMapInfo->ulReadCycle, (int)pstMapInfo->ulWriteCycle, iDeviceSize);
                break;
            case C33_RAM:
                printf_filtered (C33_MSG_MAP_INFO_RAM, (int)pstMapInfo->ulStartAddr, (int)pstMapInfo->ulEndAddr, (int)pstMapInfo->ulReadCycle, (int)pstMapInfo->ulWriteCycle, iDeviceSize);
                break;
            case C33_IO:
                printf_filtered (C33_MSG_MAP_INFO_IO, (int)pstMapInfo->ulStartAddr, (int)pstMapInfo->ulEndAddr, (int)pstMapInfo->ulReadCycle, (int)pstMapInfo->ulWriteCycle, iDeviceSize);
                break;
            case C33_STACK:
                printf_filtered (C33_MSG_MAP_INFO_STACK, (int)pstMapInfo->ulStartAddr, (int)pstMapInfo->ulEndAddr);
                break;
			default:								/* add T.Tazai 2007/07/12 */
				break;
        }
        if (pstMapInfo->iMapAttr != C33_STACK)
        {
            if (pstMapInfo->iEndian == C33_BIG)
            {
                puts_filtered(C33_MSG_MAP_BIG_ENDIAN);
            }
            else
            {
                puts_filtered(C33_MSG_MAP_LITTLE_ENDIAN);
            }
        }
        /* Get next map */
        pstMapInfo = pstMapInfo->pMapNext;
    }
}

/****************************************************************************/
/* Format        : void c33_ClearMap(void)                                  */
/* Arguments     : void                                                     */
/* Return value  : void                                                     */
/* Explanation   : Clear map information.                                   */
/****************************************************************************/
void
c33_ClearMap(void)
{
    struct c33_map_info *pstMapInfo;
    struct c33_map_info *pNext;

    pstMapInfo = c33TargetInfo.pMapInfo;
    c33TargetInfo.pMapInfo = NULL;

    if (c33TargetInfo.iSim_open != 0)
    {
        c33TargetInfo.ucAdvanced = C33_PRODUCT_STD;
    }

    while(pstMapInfo != NULL)
    {
        /* Get next map */
        pNext = pstMapInfo->pMapNext;

        /* free memory block */
        if(pstMapInfo->pMemBlock != NULL)
        {
            free(pstMapInfo->pMemBlock);
        }

        /* free memory */
        free(pstMapInfo);
        pstMapInfo = pNext;
    }
}

/****************************************************************************/
/* Format        : int c33_SendMessage (int iID, int iParam1, int iParam2, int iParam3, int iParam4, void *pvParam5, int iParam6, int iParam7) */
/* Arguments                                                                */
/*  int iID      : Command index                                            */
/*  int iParam1  : Command parameter1                                       */
/*  int iParam2  : Command parameter2                                       */
/*  int iParam3  : Command parameter3                                       */
/*  int iParam4  : Command parameter4                                       */
/*  void *pvPraram5: Command parameter5                                     */
/*  int iParam6  : Command parameter6                                       */
/*  int iParam7  : Command parameter7                                       */
/* Return value                                                             */
/*  C33_NO_ERR   : Success return                                           */
/*  C33_ERR      : Error return                                             */
/* Explanation   : The function for targets is called.                      */
/****************************************************************************/
int
c33_SendMessage (int iID, int iParam1, int iParam2, int iParam3, int iParam4, void *pvParam5, int iParam6, int iParam7)
{
    int iResult = C33_NO_ERR;

  	if( g_iTargetResetDown == 1 )/* add T.Tazaki 2005/07/28 */
 	{
	    return iResult;
	}
	
    if(c33TargetInfo.iIcd_open != C33_NOT_OPEN)
    {
        iResult = c33_IcdSendMessage (iID, iParam1, iParam2, iParam3, iParam4, pvParam5, iParam6, iParam7);
    }
    else if (c33TargetInfo.iSim_open != 0)
    {
        iResult = c33_SimSendMessage (iID, iParam1, iParam2, iParam3, iParam4, pvParam5);
    }
    else
    {
/* del T.Tazkai 2006/07/21 >>> */
//      c33_error (C33_ERR_MSG_TARGET_CONNECT);
/* del T.Tazkai 2006/07/21 <<< */
    }
    return iResult;
}

/****************************************************************************/
/* Format        : int c33_RecvMessage (int iID, int iParam1, int iParam2, int iParam3, int iParam4, void *pvParam5) */
/* Arguments                                                                */
/*  int iID      : Command index                                            */
/*  int iParam1  : Command parameter1                                       */
/*  int iParam2  : Command parameter2                                       */
/*  int iParam3  : Command parameter3                                       */
/*  int iParam4  : Command parameter4                                       */
/*  void *pvPraram5: Command parameter5                                     */
/* Return value                                                             */
/*  C33_NO_ERR   : Success return                                           */
/*  C33_ERR      : Error return                                             */
/* Explanation   : The function for targets is called.                      */
/****************************************************************************/
int
c33_RecvMessage (int iID, int iParam1, int iParam2, int iParam3, int iParam4, void *pvParam5)
{
    int iResult = C33_NO_ERR;

  	if( g_iTargetResetDown == 1 )/* add T.Tazaki 2005/07/28 */
 	{
	    return iResult;
	}

    if(c33TargetInfo.iIcd_open != C33_NOT_OPEN)
    {
        iResult = c33_IcdRecvMessage (iID, iParam1, iParam2, iParam3, iParam4, pvParam5);
    }
    else if (c33TargetInfo.iSim_open != 0)
    {
        iResult = c33_SimRecvMessage (iID, iParam1, iParam2, iParam3, iParam4, pvParam5);
    }
    return iResult;
}

/****************************************************************************/
/* Format        : void c33_print_source_lines (unsigned long ulAddr, char* pOut) */
/* Arguments                                                                */
/*  unsigned long ulAddr : address                                          */
/*  char* pOut           : source code buffer                               */
/* Return value  : void                                                     */
/* Explanation   : Get source code of specification address.                */
/****************************************************************************/
void
c33_print_source_lines(unsigned long ulAddr, char* pOut)
{
    struct symtab_and_line sal;
    int desc;
    unsigned long l;
    FILE *stream;
    char* pTmp;
    /* add T.Tazaki 2003/07/09 >>> */
    char szTemp[C33_TRACE_DATA_LINE_MAX + 1];
    int iTabSize;
    int iNoTabCnt;
    int iLen;
    int iCnt;
    int i,j,k;
    char    szbuf[20];
    char    *pszbuf;
    extern Tcl_Interp *gdbtk_interp;
    
    pszbuf = szbuf;
    /* add T.Tazaki 2003/07/09 <<< */

    pOut[0] = 0;

    sal = find_pc_line (ulAddr, 0);
    if (sal.symtab)
    {
        if(sal.pc == ulAddr)
        {
            desc = open_source_file (sal.symtab);
            if (desc < 0)
            {
            }
            else
            {
                stream = fdopen (desc, FOPEN_RT);
                clearerr (stream);


                /* >>>>> change iruma m.takeishi '03.12.22 */
                /*
                for (l = 0 ; l < sal.line ; l ++)
                */
                for (l = 0 ; l < sal.line ; )
                /* <<<<< change iruma m.takeishi '03.12.22 */
                {
                    if (fgets (pOut, C33_TRACE_DATA_LINE_MAX, stream) != NULL)
                    {
                    }
                    else
                    {
                        pOut[0] = 0;
                        break;
                    }
                    /* >>>>> add iruma m.takeishi '03.12.22 */
                    if ( pOut[ 0 ] != '\0' ) {
                        if ( pOut[ strlen( pOut )-1 ] == '\n' ) {
                            l++;
                        } else {
                            if ( l == sal.line -1 ) {
                                break;
                            }
                        }
                    }
                    /* <<<<< add iruma m.takeishi '03.12.22 */
                }
                fclose (stream);
            }
        }
    }
    pTmp = strstr(pOut, "\n");
    if ( pTmp != NULL )
    {
        *pTmp = 0;
    }
    pTmp = strstr(pOut, "\r");
    if ( pTmp != NULL )
    {
        *pTmp = 0;
    }

    /* Change tab to space */       /* add T.Tazaki 2003/07/09 >>> */
    if( g_use_windows == 1 )
    {
        Tcl_Eval( gdbtk_interp, "c33_get_tab_size" ); /* gdbtcl/srcbar.itcl */
        pszbuf = Tcl_GetVar(gdbtk_interp, "c33_tab_size", TCL_GLOBAL_ONLY);   /* get current tab size */
        iTabSize = atoi( pszbuf );
    }
    else
    {
        iTabSize = 4;   /* Default TAB size */
    }

    iLen = strlen( pOut );
    j = 0;
    iNoTabCnt = 0;
    for( i = 0; i < iLen; ++i )
    {
        if( *(pOut+i) == 0x09 )    /* TAB ? */
        {
            if( iNoTabCnt == 0 )
            {
                iCnt = iTabSize;
            }
            else
            {
                iCnt = iTabSize - (iNoTabCnt % iTabSize);
                if( iCnt == 0 )
                {
                    iCnt = iTabSize;
                }
            }
            for( k = 0; k < iCnt; ++k )
            {
                if( j < C33_TRACE_DATA_LINE_MAX )
                {
                    szTemp[j] = ' ';
                    ++j;
                }
            }
            iNoTabCnt = 0;
        }
        else
        {
            if( j < C33_TRACE_DATA_LINE_MAX )
            {
                szTemp[j] = *(pOut+i);
                ++j;
                ++iNoTabCnt;
            }
        }
    }
    szTemp[j] = 0;

    for( i = 0; i < j; ++i )
    {
        *(pOut + i ) = szTemp[i];
    }
    *(pOut + i ) = 0;
    /* add T.Tazaki 2003/07/09 <<< */
}

/****************************************************************************/
/* Format        : int c33_FindFilename(unsigned long ulAddr, char* pOut, unsigned long* ulLineNo) */
/* Arguments                                                                */
/*  unsigned long ulAddr    : address                                       */
/*  char* pOut              : file name buffer                              */
/*  unsigned long* ulLineNo : line number pointer                           */
/* Return value                                                             */
/*  C33_NO_ERR   : Success return                                           */
/*  C33_ERR      : Error return                                             */
/* Explanation   : Get file name and line number of specification address.  */
/****************************************************************************/
int
c33_FindFilename(unsigned long ulAddr, char* pOut, unsigned long* ulLineNo)
{
    struct symtab_and_line sal;

    pOut[0] = 0;

    sal = find_pc_line (ulAddr, 0);
    if (sal.symtab)
    {
        if(sal.pc == ulAddr)
        {
            strcpy(pOut, sal.symtab->filename);
            *ulLineNo = sal.line;
            return C33_NO_ERR;
        }
    }
    return C33_ERR;
}

/****************************************************************************/
/* Format        : static void c33_DumpFile(int iParamNum, unsigned long ulStartAddr, unsigned long ulEndAddr, int iFileFormat, char* pFilename, char* pAppend) */
/* Arguments                                                                */
/*  int iParamNum             : Parameter number                            */
/*  unsigned long ulStartAddr : Start address                               */
/*  unsigned long ulEndAddr   : End address                                 */
/*  int iFileFormat           : File format                                 */
/*  char* pFilename           : Filename                                    */
/*  char* pAppend             : Append Mode "a"                             */
/* Return value  : void                                                     */
/* Explanation   : Memory dump to file.                                     */
/****************************************************************************/
static void
c33_DumpFile(int iParamNum, unsigned long ulStartAddr, unsigned long ulEndAddr, int iFileFormat, char* pFilename, char* pAppend)
{
    unsigned long ulSize;
    unsigned long l,m;
    FILE* fpDumpfile = NULL;
    char cErrMsg[C33_COMMAND_MAX + 1];
    char cReadData[C33_COMMAND_TEXT_SIZE + 1];
    char cText[sizeof(C33_MSG_DUMP_FILE_HEADER) + C33_COMMAND_MAX];
    char cTmp[sizeof(C33_MSG_DUMP_FILE_HEADER) + C33_COMMAND_MAX];
    unsigned short usSum;
    unsigned long ulDspStartAddr,ulDspEndAddr;

    cErrMsg[0] = 0;

    /* Check number of parameter */
    if ( iParamNum != 5 && iParamNum != 6)
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }

    /* Check address */
    if ( ulStartAddr > ulEndAddr )
    {
        c33_error (C33_ERR_COMMAND_ADDR);
        return;
    }

	if( iParamNum == 6 )		/* Append mode : add T.Tazaki 2004/05/26 */
    {
		if( *pAppend != 'a' && *pAppend != 'f')
		{
	        c33_error (C33_ERR_COMMAND_INVALID_PARAM);
	        return;
	    }
    }

    /* Check iFileFormat */
    if (iFileFormat != C33_DUMPFILE_BIN && iFileFormat != C33_DUMPFILE_TEXT && iFileFormat != C33_DUMPFILE_S3)
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }


    switch(iFileFormat)
    {
        case C33_DUMPFILE_BIN:
            printf_filtered (C33_MSG_DUMP_BIN, (int)ulStartAddr, (int)ulEndAddr);
            break;
        case C33_DUMPFILE_TEXT:
            ulStartAddr &= 0xfffffff0;
            ulEndAddr   |= 0x0000000f;
            printf_filtered (C33_MSG_DUMP_TEXT, (int)ulStartAddr, (int)ulEndAddr);
            break;
        case C33_DUMPFILE_S3:
            ulStartAddr &= 0xffffffe0;
            ulEndAddr   |= 0x0000001f;
            printf_filtered (C33_MSG_DUMP_S3, (int)ulStartAddr, (int)ulEndAddr);
            break;
		default:								/* add T.Tazai 2007/07/12 */
			break;
    }

    /* Open file */
	if( iParamNum == 5 )
    {
	    if( iFileFormat == C33_DUMPFILE_TEXT)
	    {
	        fpDumpfile = fopen(pFilename, "w");
	    }
	    else
	    {
	        fpDumpfile = fopen(pFilename, "wb");
	    }
	}
	else		/* add T.Tazaki 2004/05/26 : append mode */
	{
	    if( iFileFormat == C33_DUMPFILE_TEXT)
	    {
	        fpDumpfile = fopen(pFilename, "a+");
	    }
	    else
	    {
	        fpDumpfile = fopen(pFilename, "a+b");
	    }
	}
		
    if (fpDumpfile == NULL)
    {
        /* file open error */
        c33_error (C33_ERR_FILE_OPEN, pFilename);
        return;
    }

    /* Header output */
    if ( iFileFormat == C33_DUMPFILE_TEXT )
    {
		if( iParamNum == 5 )		/* No Append mode ?  : add T.Tazaki 2004/05/26 */
	    {
	        if (fputs(C33_MSG_DUMP_FILE_HEADER, fpDumpfile) == EOF)
	        {
	            sprintf(cErrMsg, C33_ERR_FILE_WRITE);
	        }
	        cTmp[0] = 0;
/*	        c33_AddCRLF(cTmp);		*/ /* del T.Tazaki 2004/09/28 */
	        if (fputs(cTmp, fpDumpfile) == EOF)
	        {
	            sprintf(cErrMsg, C33_ERR_FILE_WRITE);
	        }
		}
    }

    ulDspStartAddr = ulStartAddr;

    c33_InitCancel();   /* Cancel Check Preparation */

    while (ulStartAddr <= ulEndAddr)
    {
        /* Get transaction size */
        ulSize = ulEndAddr - ulStartAddr + 1;
        if (ulSize > C33_COMMAND_TEXT_SIZE)
        {
            ulSize = C33_COMMAND_TEXT_SIZE;
        }

        if (ulStartAddr == ulDspStartAddr)
        {
            ulDspEndAddr = ulStartAddr + C33_DUMPFILE_UNIT - C33_BYTE;
            if( ulDspEndAddr > ulEndAddr )
            {
                ulDspEndAddr = ulEndAddr;
            }
            printf_filtered(C33_MSG_DUMP_PROCESSING, ulDspStartAddr, ulDspEndAddr);
            ulDspStartAddr += C33_DUMPFILE_UNIT;
        }

        /* Read data from target */
        if (c33_Cmd_ReadWrite(C33_READ, ulStartAddr, ulSize, C33_BYTE, cReadData) != C33_NO_ERR)
        {
            break;
        }

        if ( iFileFormat == C33_DUMPFILE_BIN )
        {
            /* Output to file */
            for (l = 0 ; l < ulSize ; l++)
            {
                if (fputc(cReadData[l], fpDumpfile) == EOF)
                {
                    sprintf(cErrMsg, C33_ERR_FILE_WRITE);
                    break;
                }
            }
        }
        else if ( iFileFormat == C33_DUMPFILE_TEXT )
        {
            /* Output to file */
            for (l = 0 ; l < ulSize ; l += 16)
            {
                sprintf(cText, "%08X ", (int)(ulStartAddr + l));
                for (m = 0 ; m < 16 ; m++)
                {
				    if(c33TargetInfo.iSim_open != 0)							/* add T.Tazaki 2006/6/30 */
				    {															/* add T.Tazaki 2006/6/30 */
	                    if( c33_CheckMapArea(ulStartAddr + l + m) == C33_NO_ERR )
	                    {
	                        sprintf(cTmp, "%02X ", (cReadData[l + m] & 0xff) );
	                    }
	                    else
	                    {
	                        sprintf(cTmp, "** ");
	                    }
	                }
	                else														/* add T.Tazaki 2006/6/30 */
	                {															/* add T.Tazaki 2006/6/30 */
                        sprintf(cTmp, "%02X ", (cReadData[l + m] & 0xff) );		/* add T.Tazaki 2006/6/30 */
                    }															/* add T.Tazaki 2006/6/30 */
                    strcat(cText, cTmp);
                }
/*		        c33_AddCRLF(cText);		*/ /* del T.Tazaki 2004/09/28 */
				strcat( cText,"\n" );	/* add T.Tazaki 2004/09/28 */
                if (fputs( cText, fpDumpfile) == EOF)
                {
                    sprintf(cErrMsg, C33_ERR_FILE_WRITE);
                    break;
                }
            }
        }
        else if ( iFileFormat == C33_DUMPFILE_S3 )
        {
            for (l = 0 ; l < ulSize ; l += 32)
            {
                usSum = 0x25;
                usSum += (unsigned short)( ( (ulStartAddr + l) & 0xff000000 ) >> 24 );
                usSum += (unsigned short)( ( (ulStartAddr + l) & 0x00ff0000 ) >> 16 );
                usSum += (unsigned short)( ( (ulStartAddr + l) & 0x0000ff00 ) >>  8 );
                usSum += (unsigned short)( ( (ulStartAddr + l) & 0x000000ff ) >>  0 );

                /* length & address output */
                if ( fprintf( fpDumpfile,"S325%08lX", (ulStartAddr + l) ) < 0)
                {
                    sprintf(cErrMsg, C33_ERR_FILE_WRITE);
                }
                /* data output */
                for( m = 0 ; m < 32 ; m++ )
                {
                    usSum += cReadData[l + m]; /* add sum */
                    fprintf(fpDumpfile, "%02X", cReadData[l + m] & 0xff );
                }
                /* sum output */
                if ( fprintf(fpDumpfile, "%02X", ((unsigned char)usSum) ^= 0xff ) < 0) /* sum write */
                {
                    sprintf(cErrMsg, C33_ERR_FILE_WRITE);
                }
                cTmp[0] = 0;
                c33_AddCRLF(cTmp);
                if (fputs(cTmp, fpDumpfile) == EOF)
                {
                    sprintf(cErrMsg, C33_ERR_FILE_WRITE);
                }
            }
        }
        ulStartAddr += ulSize;


        /* add 2003/03/24 T.Tazaki : Check STOP button or ESC key >>>  */
        if( c33TargetInfo.iKeyBreak == 1 )
        {
            /* Stop dump */
            printf_filtered( C33_MSG_TARGET_CANCEL);
            break;
        }
        if( c33_CheckCancel(0) == 1 )
        {
            /* Stop dump */
            printf_filtered( C33_MSG_TARGET_CANCEL);
            break;
        }
        /* add 2003/03/24 T.Tazaki : Check STOP button or ESC key <<<  */

    }

    if ( iFileFormat == C33_DUMPFILE_S3 )
    {
		if( iParamNum == 6 && *pAppend == 'a' )		/* No outout Footer : add T.Tazaki 2004/05/26 */
	    {
		}
		else
		{
	        strcpy(cText, C33_MOTOROLA_FOOTER);
	        c33_AddCRLF(cText);
	        if( fprintf( fpDumpfile, cText ) != strlen(cText) )
	        {
	            sprintf(cErrMsg, C33_ERR_FILE_WRITE);
	        }
	    }
	}

    /* file close */
    if (fpDumpfile != NULL)
    {
        fclose(fpDumpfile);
    }

    c33_EndCancel();    /* Cancel Check End */
    
    /* Print error message */
    if ( cErrMsg[0] != 0 )
    {
        c33_error (cErrMsg);
    }

}


/****************************************************************************/
/* Format        : static int c33_ReadWriteMapMemExe(int iReadWrite, unsigned long ulAddr, int iUnit, unsigned char* pMyAddr) */
/* Arguments                                                                */
/*  int iReadWrite         : C33_READ or C33_WRITE                          */
/*  unsigned long ulAddr   : Address                                        */
/*  int iUnit              : Size(C33_BYTE,C33_HALF_WORD,C33_WORD)          */
/*  unsigned char* pMyAddr : Memory address                                 */
/* Return value                                                             */
/*  C33_NO_ERR   : Success return                                           */
/*  Other        : Error code                                               */
/* Explanation   : read/write to mapping memory.                            */
/****************************************************************************/
static int
c33_ReadWriteMapMemExe(int iReadWrite, unsigned long ulAddr, int iUnit, unsigned char* pMyAddr)
{
    unsigned char* pMapAddr;
    unsigned char* pSrc;
    unsigned char* pDst;
    unsigned long ulExeLen;
    int iResult;
    int iEndian;
    int i;

    /* Convert address to pointer */
    iResult = c33_SearchMem(iReadWrite, ulAddr, (unsigned long)iUnit, &ulExeLen, &pMapAddr, &iEndian);

    if ( iResult == C33_NO_ERR)
    {
        for ( i = 0 ; i < iUnit ; i++ )
        {
            if (iReadWrite == C33_READ)
            {
                pSrc = pMyAddr;
                pDst = pMapAddr;
            }
            else
            {
                pSrc = pMapAddr;
                pDst = pMyAddr;
            }
            if (iEndian == C33_LITTLE)
            {
                *(pSrc + i) = *(pDst + i);
            }
            else
            {
                *(pSrc + i)= *(pDst + iUnit - i - 1);
            }
        }
    }
    return iResult;
}

/****************************************************************************/
/* Format        : int c33_ReadWriteMapMem(int iReadWrite, unsigned long ulAddr, unsigned long ulLen, int iUnit, unsigned char* pMyAddr) */
/* Arguments                                                                */
/*  int iReadWrite         : C33_READ or C33_WRITE                          */
/*  unsigned long ulAddr   : Address                                        */
/*  unsigned long ulLen    : Length                                         */
/*  int iUnit              : Size(C33_BYTE,C33_HALF_WORD,C33_WORD)          */
/*  unsigned char* pMyAddr : Memory address                                 */
/* Return value                                                             */
/*  C33_NO_ERR   : Success return                                           */
/*  Other        : Error code                                               */
/* Explanation   : Top function of read/write to mapping memory.            */
/****************************************************************************/
int
c33_ReadWriteMapMem(int iReadWrite, unsigned long ulAddr, unsigned long ulLen, int iUnit, unsigned char* pMyAddr)
{
    unsigned long l;
    int iResult = C33_NO_ERR;

    if (ulLen % iUnit)
    {
        c33_error (C33_ERR_INTERNAL_ERR, "c33common.c", "c33_ReadWriteMapMem()");
        return C33_ERR;
    }

    for ( l = 0 ; l < ulLen ; l += iUnit)
    {
        iResult = c33_ReadWriteMapMemExe(iReadWrite, ulAddr + l, iUnit, pMyAddr + l);
        if ( iResult != C33_NO_ERR )
        {
            break;
        }
    }


    return iResult;
}

/****************************************************************************/
/* Format        : static int c33_SearchMem(int iReadWrite, unsigned long ulAddr, unsigned long ulLen, unsigned long* ulExeLen, unsigned char** ppcMyAddr, int* piEndian) */
/* Arguments                                                                */
/*  int iReadWrite            : C33_READ or C33_WRITE                       */
/*  unsigned long ulAddr      : Address                                     */
/*  unsigned long ulLen       : Length                                      */
/*  unsigned long* ulExeLen   : Enable length                               */
/*  unsigned char** ppcMyAddr : pointer to map memory                       */
/*  int* piEndian             : Endian information                          */
/* Return value                                                             */
/*  C33_NO_ERR   : Success return                                           */
/*  Other        : Error code                                               */
/* Explanation   : Top function of read/write to mapping memory.            */
/****************************************************************************/
static int
c33_SearchMem (int iReadWrite, unsigned long ulAddr, unsigned long ulLen, unsigned long* ulExeLen, unsigned char** ppcMyAddr, int* piEndian)
{
    struct c33_map_info *pstMapInfo;
    struct c33_map_info *pNext;

    pstMapInfo = c33TargetInfo.pMapInfo;

    if (c33TargetInfo.pMapInfo == NULL)
    {
        return C33_ERR_PARAMETER_NOT_SET;
    }

    if (ulLen == 0)
    {
        return C33_NO_ERR;
    }

    while(pstMapInfo != NULL)
    {
        /* Get next map */
        pNext = pstMapInfo->pMapNext;

        if(ulAddr >= pstMapInfo->ulStartAddr && ulAddr <= pstMapInfo->ulEndAddr)
        {
            if(pstMapInfo->iMapAttr == C33_ROM || pstMapInfo->iMapAttr == C33_RAM || pstMapInfo->iMapAttr == C33_IO)
            {
//              if(iReadWrite == C33_WRITE && pstMapInfo->iMapAttr == C33_ROM && c33TargetInfo.iIcdLoad != 1)
                if(iReadWrite == C33_WRITE && pstMapInfo->iMapAttr == C33_ROM && c33TargetInfo.iIcdLoad == 0 && c33TargetInfo.iIcdRm == 0) /* bug fix T.Tazaki 2004/06/15 */
                {
                    return C33_ERR_ROM_WRITE;
                }
                *ppcMyAddr = pstMapInfo->pMemBlock + ulAddr - pstMapInfo->ulStartAddr;
                *piEndian = pstMapInfo->iEndian;
                if( ulAddr + ulLen - 1 <= pstMapInfo->ulEndAddr )
                {
                    *ulExeLen = ulLen;
                }
                else
                {
                    *ulExeLen = pstMapInfo->ulEndAddr - ulAddr + 1;
                }
                return C33_NO_ERR;
            }
        }

        pstMapInfo = pNext;
    }
    return C33_ERR_NO_MAP_AREA;
}

/****************************************************************************/
/* Format        : static void c33_CallMode(int iParamNum, int iCallMode , char *cFileName ) */
/* Arguments                                                                */
/*  int iParamNum             : Parameter number                            */
/*  int iCallMode             : call mode number 1:window,2:file,3:window & file  */
/*  char *cFileName           : call result output file name                */
/* Return value  : void                                                     */
/* Explanation   : Set call mode.                                           */
/****************************************************************************/
static void
c33_CallMode(int iParamNum, int iCallMode , char *cFileName )
{
    /* Check number of parameter */
    if ( iParamNum != 2 && iParamNum != 3 )
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }

    /* Check mode number */
    if( iCallMode < 1 || iCallMode > 3 )
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }

    if( strlen( cFileName ) > 256 )
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }
    
    if( iCallMode == 2 || iCallMode == 3 )
    {
        if( iParamNum != 3 )
        {
            c33_error (C33_ERR_COMMAND_PARAM);
            return;
        }
        /* Check file name length */
        if( strlen( cFileName ) > 255 )
        {
            c33_error (C33_ERR_COMMAND_PARAM);
            return;
        }
        /* Set output file name */
        strcpy( c33TargetInfo.cCallFileName,cFileName );
    }
    
    /* Set call mode number */
    c33TargetInfo.iCallMode = iCallMode;
}

/****************************************************************************/
/* Format        : static void c33_Call(int iParamNum, int iStartAddr, int iArg1, int iArg2, int iArg3) */
/* Arguments                                                                */
/*  int iParamNum             : Parameter number                            */
/*  char *pFunction           : Function name                               */
/*  int iArg1                 : Argument1                                   */
/*  int iArg2                 : Argument2                                   */
/*  int iArg3                 : Argument3                                   */
/* Return value  : void                                                     */
/* Explanation   : Function call execution.                                 */
/****************************************************************************/
static void
c33_Call(int iParamNum, int iStartAddr, int iArg1, int iArg2, int iArg3)
{
    /* Check number of parameter */
    if ( iParamNum < 2 || iParamNum > 5 )
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }

    if(c33TargetInfo.iIcd_open != 0)
    {
        c33_IcdCall( (unsigned long)iStartAddr, (unsigned long)iArg1, (unsigned long)iArg2, (unsigned long)iArg3 );
    }
    else if( c33TargetInfo.iSim_open != 0)
    {
        c33_SimCall( (unsigned long)iStartAddr, (unsigned long)iArg1, (unsigned long)iArg2, (unsigned long)iArg3 );
    }
}
/****************************************************************************/
/* Format        : static void c33_Int(int iParamNum, int iType, int iLevel)*/
/* Arguments                                                                */
/*  int iParamNum             : Parameter number                            */
/*  int iType                 : Interrupt type  0-0x7fff                    */
/*  int iLevel                : Interrupt level 0-15                        */
/* Return value  : void                                                     */
/* Explanation   : Function Int execute                                     */
/****************************************************************************/
static void
c33_Int(int iParamNum, int iType, int iLevel )
{
    if (c33TargetInfo.iSim_open == 0)
    {
        c33_error (C33_ERR_COMMAND_INVALID);
        return;
    }

    /* Check number of parameter */
    if ( iParamNum != 1 && iParamNum != 3 )
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }

    if ( iParamNum == 3 )
    {
        /* Check type number */
        if( iType > 0x7fff )
        {
            c33_error (C33_ERR_COMMAND_PARAM);
            return;
        }
        /* Check level number */
        if( iLevel > 15 )
        {
            c33_error (C33_ERR_COMMAND_PARAM);
            return;
        }

        c33_SimInt( 0, iType, iLevel );
    }
    else
    {
        c33_SimInt( 1, 0, 0 );      /* NMI */
    }
    
}
/****************************************************************************/
/* Format        : static void c33_ClocklMode(int iParamNum, int iClockMode , int iClockFuntion ) */
/* Arguments                                                                */
/*  int iParamNum             : Parameter number                            */
/*  int iClockMode            : clock mode number 1:reset,2:hold            */
/*  int iClockFunction        : clock funtion numbrt 1:cycle,2:time usec,3:time sec */
/* Return value  : void                                                     */
/* Explanation   : Set clock mode.                                          */
/****************************************************************************/
static void
c33_ClockMode(int iParamNum, int iClockMode , int iClockFunction )
{

    /* Check number of parameter */
    if ( iParamNum != 2 && iParamNum != 3 )
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }

    /* Check mode number */
    if( iClockMode < 1 || iClockMode > 2 )
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }
    if(c33TargetInfo.iIcd_open != 0)
    {
        if( iClockFunction < 1 || iClockFunction > 3 )
        {
            c33_error (C33_ERR_COMMAND_PARAM);
            return;
        }
        /* Set Clock function command */
        c33_IcdClockMode( iClockFunction );

    }
    else if( c33TargetInfo.iSim_open != 0)
    {
        if( iClockFunction != 1 )
        {
            c33_error (C33_ERR_COMMAND_PARAM);
            return;
        }
    }
    
    /* check if counter mode change */
    if( c33TargetInfo.iClockMode == 1 ) /* counter reset mode ? */
    {
        if( iClockMode == 2 )   /* hold mode ? */
        {
            /* Reset counters : ICD */
            g_ulClockCount[0] = 0;  /* cycle count total */
            g_ulClockCount[1] = 0;  /* 50ns time total */
            g_ulClockCount[2] = 0;  /* 1us time  total */
			g_ullClockCount = 0;  /* cycle count total */	/* ICDV6 add T.Tazaki 2007/07/03 */

            /* Clock Counter Reset : SIM */
            gulCycle = 0;
        }
    }
    else
    {
        if( iClockMode == 1 )   /* reset mode ? */
        {
            /* Reset counters : ICD */
            g_ulClockCount[0] = 0;  /* cycle count total */
            g_ulClockCount[1] = 0;  /* 50ns time total */
            g_ulClockCount[2] = 0;  /* 1us time  total */
			g_ullClockCount = 0;  /* cycle count total */	/* ICDV6 add T.Tazaki 2007/07/03 */

            /* Clock Counter Reset : SIM */
            gulCycle = 0;
        }
    }
    
    /* Set clock mode number and function */
    c33TargetInfo.iClockMode = iClockMode;
    c33TargetInfo.iClockFunction = iClockFunction;
    
}
/****************************************************************************/
/* Format        : static void c33_Clock(int iParamNum )                    */
/* Arguments                                                                */
/*  int iParamNum  : Parameter number                                       */
/* Return value  : void                                                     */
/* Explanation   : Function clock execution.                                */
/****************************************************************************/
static void
c33_Clock(int iParamNum )
{
    char szClock[20];
    char szbuf[30];
    double  dTmp;
    
    szClock[0]=0;
    
    /* Check number of parameter */
    if ( iParamNum > 1 )
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }

	/* check if now after Not "cont or until".   add T.Tazaki 2005/06/24 >>> */
	if( g_iClockResetCmdErr == 1 )
	{
		c33_error(C33_ERR_CLOCK_CANNOT_DISPLAY);
        return;
	}
	/* check if now after Not "cont or until".   add T.Tazaki 2005/06/24 <<< */

    if(c33TargetInfo.iIcd_open != 0)
    {
         /* output to window */
        switch( c33TargetInfo.iClockFunction )
        {
        case 2: /* Time micro sec display */
                if( g_ulClockCount[1] & 0x80000000 )
                {
                    puts_filtered( C33_ERR_TIMER_OVERFLOW );
                }
                else
                {
                    dTmp = g_ulClockCount[1];
                    if( g_ulClockCount[1] > 0 )
                    {
                        dTmp /= 20;
                    }
                    sprintf( szbuf,"   %11.2f us", dTmp );
                    puts_filtered( szbuf );
                }
                break;
        case 3: /* Time Sec display */
                if( g_ulClockCount[2] & 0x80000000 )
                {
                    puts_filtered( C33_ERR_TIMER_OVERFLOW );
                }
                else
                {
                    dTmp = g_ulClockCount[2];
                    if( g_ulClockCount[2] > 0 )
                    {
                        dTmp /= 1000000;
                    }
                    sprintf( szbuf,"   %10.6f s", dTmp );
                    puts_filtered( szbuf );
                }
                break;
        case 1: /* Cycle display */
        default:
                if( g_ulClockCount[0] & 0x80000000 )
                {
                    puts_filtered( C33_ERR_TIMER_OVERFLOW );
                }
                else
                {
                    sprintf( szbuf,"   %10u cycle",g_ulClockCount[0] );
                    puts_filtered( szbuf );
                }
                break;
        }
    }
    else if( c33TargetInfo.iSim_open != 0)
    {
        c33_SimClock( (char *)szClock );
        if( szClock[0] != 0 )
        {
            /* output to window */
            sprintf( szbuf,"   %s cycle",szClock );
            puts_filtered( szbuf );
        }
    }

    puts_filtered( "\n" );


}

/****************************************************************************/
/* Format        : static void c33_Stdin(int iParamNum, int iSet, unsigned long ulBreakAddr, unsigned long ulReadBufAddr, char *cFileName ) */
/* Arguments                                                                */
/*  int iParamNum             : Parameter number                            */
/*  int iSet                  : 1=Set, 2=Clear                              */
/*  unsigned long ulBreakAddr   : break address                             */
/*  unsigned long ulReadBufAddr : read buffer address                       */
/*  char *cFileName           : simulater input file name                   */
/* Return value  : void                                                     */
/* Explanation   : Simulated in.                                            */
/****************************************************************************/
static void
c33_Stdin(int iParamNum, int iSet, unsigned long ulBreakAddr, unsigned long ulReadBufAddr, char *cFileName )
{
    int iResult = C33_NO_ERR;
    int i;
	/* add T.Tazaki 2005/05/06 >>> */
	unsigned short usCode;
	/* add T.Tazaki 2005/05/06 <<< */

    /* Check number of parameter */
    if ( iParamNum != 2 && iParamNum != 4 && iParamNum != 5 )
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }

    /* Check set/clear */
    if( iSet != 1 && iSet != 2 )
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }

    if( iSet == 1 && iParamNum == 2 )
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }

    if ( iParamNum == 5 )
    {
        if( strlen( cFileName ) > 256 )
        {
            c33_error (C33_ERR_COMMAND_PARAM);
            return;
        }
    }

    if ( iParamNum == 4 || iParamNum == 5 )
    {
        ulBreakAddr &= 0xfffffffe;
        ulReadBufAddr &= 0xfffffffe;
    }


    if( iSet == 1 )
    {
        /* stdin SET */

        c33_GetBreakPoint();

        /* Search same address */
        for( i = 0; i < g_SoftPcBreakTable_Index; ++i )
        {
            if( ulBreakAddr == g_SoftPcBreakTable[ i ] )
            {
                break;
            }
        }
        if ( i != g_SoftPcBreakTable_Index )
        {
            c33_error (C33_ERR_MSG_BRK_SOFT_MAX);
            return;
        }

        /* Check break address*/
        if ( c33_BreakAddrCheck(ulBreakAddr, C33_SOFT_PC_BREAK) == C33_ERR )
        {
            return;
        }

        /* check already set stdin soft pc break */
        if( c33TargetInfo.ulSoftPcBreak[C33_STDIN_BREAK_NUM-1] != C33_BREAK_DISABLE )
        {
            c33_error (C33_ERR_MSG_BRK_SOFT_MAX);
            return;
        }
        
        if(c33TargetInfo.iIcd_open != 0)
        {
			/* change T.Tazaki 2005/05/06 >>> */
			
		    /* read original code */
		    iResult = c33_Cmd_ReadWrite(C33_READ, ulBreakAddr, 2, C33_HALF_WORD, (unsigned char*)&usCode );
		    if( iResult == C33_NO_ERR )
		    {
		        /* save code */
		        c33TargetInfo.usSoftPcBreakOrgCode[C33_STDIN_BREAK_NUM-1] = usCode;
			    c33TargetInfo.ulSoftPcBreak[C33_STDIN_BREAK_NUM-1]        = ulBreakAddr;
		        
		        /* write "BRK" */
		        usCode = SOFT_BREAK_INST;
		        iResult = c33_Cmd_ReadWrite(C33_WRITE, ulBreakAddr, 2, C33_HALF_WORD, (unsigned char*)&usCode);
		    }

#if 0
            iResult = c33_SendMessage(C33_ID_SOFT_PC_BREAK_SET, ulBreakAddr, C33_STDIN_BREAK_NUM, 0, 0, 0, 0, 0);
            if (iResult == C33_NO_ERR)
            {
                iResult = c33_RecvMessage(C33_ID_SOFT_PC_BREAK_SET, 0, 0, 0, 0, 0);
            }
#endif            
			/* change T.Tazaki 2005/05/06 <<< */
        }
        else if(c33TargetInfo.iSim_open != 0)
        {
            iResult = c33_SimStdinSet( ulBreakAddr );
        }

        if (iResult == C33_NO_ERR)
        {
            if ( iParamNum == 5 )   /* input from file ? */
            {
                /* input file open */
                fpStdinfile = fopen(cFileName, "r");	/* "r" --> "rb" --> "r" change T.Tazaki 2004/07/12 */
                if (fpStdinfile == NULL)
                {
                    /* file open error */
                    c33_error (C33_ERR_FILE_OPEN, cFileName);
                    return;
                }

                strcpy( c33TargetInfo.cStdinFileName,cFileName );
            }
            else
            {
                strcpy( c33TargetInfo.cStdinFileName,"" );
            }
            c33TargetInfo.ulSoftPcBreak[C33_STDIN_BREAK_NUM-1] = ulBreakAddr;
            c33TargetInfo.ulReadBufAddr = ulReadBufAddr;
        }
        
        /* flag & read buffer index clear */
        g_c33_std_in = 0;
        g_c33_Index = 0;

    }
    else
    {
        /* stdin CLEAR */
        if ( iParamNum != 2 )
        {
            c33_error (C33_ERR_COMMAND_PARAM);
            return;
        }

        if(c33TargetInfo.iIcd_open != 0)
        {
			/* change T.Tazaki 2005/05/06 >>> */
			
	        /* write Original code */
	        
	        usCode = c33TargetInfo.usSoftPcBreakOrgCode[C33_STDIN_BREAK_NUM-1];
	        iResult = c33_Cmd_ReadWrite(C33_WRITE, c33TargetInfo.ulSoftPcBreak[C33_STDIN_BREAK_NUM-1], 2, C33_HALF_WORD, (unsigned char*)&usCode);
			c33TargetInfo.ulSoftPcBreak[C33_STDIN_BREAK_NUM-1] = C33_BREAK_DISABLE;
#if 0
            iResult = c33_SendMessage(C33_ID_SOFT_PC_BREAK_CLR, C33_STDIN_BREAK_NUM, 0, 0, 0, 0, 0, 0);
            if (iResult == C33_NO_ERR)
            {
                iResult = c33_RecvMessage(C33_ID_SOFT_PC_BREAK_CLR, 0, 0, 0, 0, 0);
            }
#endif
			/* change T.Tazaki 2005/05/06 <<< */
        }
        else if(c33TargetInfo.iSim_open != 0)
        {
            c33_SimStdinClear();
        }
        
        c33TargetInfo.ulSoftPcBreak[C33_STDIN_BREAK_NUM-1] = C33_BREAK_DISABLE;

        /* input file close */
        if (fpStdinfile != NULL)
        {
            fclose(fpStdinfile);
        }
    }
}

/****************************************************************************/
/* Format        : static void c33_Stdout(int iParamNum, int iSet, unsigned long ulBreakAddr, unsigned long ulWriteBufAddr, char *cFileName ) */
/* Arguments                                                                */
/*  int iParamNum             : Parameter number                            */
/*  int iSet                  : 1=Set, 2=Clear                              */
/*  unsigned long ulBreakAddr    : break address                            */
/*  unsigned long ulWriteBufAddr : write buffer address                     */
/*  char *cFileName           : simulater output file name                  */
/* Return value  : void                                                     */
/* Explanation   : Simulated out.                                           */
/****************************************************************************/
static void
c33_Stdout(int iParamNum, int iSet, unsigned long ulBreakAddr, unsigned long ulWriteBufAddr, char *cFileName )
{
    int iResult = C33_NO_ERR;
    int i;
	/* add T.Tazaki 2005/05/06 >>> */
	unsigned short usCode;
	/* add T.Tazaki 2005/05/06 <<< */

    /* Check number of parameter */
    if ( iParamNum != 2 && iParamNum != 4 && iParamNum != 5 )
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }

    /* Check set/clear */
    if( iSet != 1 && iSet != 2 )
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }

    if( iSet == 1 && iParamNum == 2 )
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }

    if ( iParamNum == 5 )
    {
        if( strlen( cFileName ) > 256 )
        {
            c33_error (C33_ERR_COMMAND_PARAM);
            return;
        }
    }

    if ( iParamNum == 4 || iParamNum == 5 )
    {
        ulBreakAddr &= 0xfffffffe;
        ulWriteBufAddr &= 0xfffffffe;
    }

    if( iSet == 1 )
    {
        /* stdout SET */

        c33_GetBreakPoint();

        /* Search same address */
        for( i = 0; i < g_SoftPcBreakTable_Index; ++i )
        {
            if( ulBreakAddr == g_SoftPcBreakTable[ i ] )
            {
                break;
            }
        }
        if ( i != g_SoftPcBreakTable_Index )
        {
            c33_error (C33_ERR_MSG_BRK_SOFT_MAX);
            return;
        }

        /* Check break address*/
        if ( c33_BreakAddrCheck(ulBreakAddr, C33_SOFT_PC_BREAK) == C33_ERR )
        {
            return;
        }

        /* check already set stdout soft pc break */
        if( c33TargetInfo.ulSoftPcBreak[C33_STDOUT_BREAK_NUM-1] != C33_BREAK_DISABLE )
        {
            c33_error (C33_ERR_MSG_BRK_SOFT_MAX);
            return;
        }
        
        if(c33TargetInfo.iIcd_open != 0)
        {
			/* change T.Tazaki 2005/05/06 >>> */

		    /* read original code */
		    iResult = c33_Cmd_ReadWrite(C33_READ, ulBreakAddr, 2, C33_HALF_WORD, (unsigned char*)&usCode );
		    if( iResult == C33_NO_ERR )
		    {
		        /* save code */
		        c33TargetInfo.usSoftPcBreakOrgCode[C33_STDOUT_BREAK_NUM-1] = usCode;
		        c33TargetInfo.ulSoftPcBreak[C33_STDOUT_BREAK_NUM-1]        = ulBreakAddr;
		        
		        /* write "BRK" */
		        usCode = SOFT_BREAK_INST;
		        iResult = c33_Cmd_ReadWrite(C33_WRITE, ulBreakAddr, 2, C33_HALF_WORD, (unsigned char*)&usCode);
		    }

#if 0
            iResult = c33_SendMessage(C33_ID_SOFT_PC_BREAK_SET, ulBreakAddr, C33_STDOUT_BREAK_NUM, 0, 0, 0, 0, 0);
            if (iResult == C33_NO_ERR)
            {
                iResult = c33_RecvMessage(C33_ID_SOFT_PC_BREAK_SET, 0, 0, 0, 0, 0);
            }
#endif
			/* change T.Tazaki 2005/05/06 <<< */
        }
        else if(c33TargetInfo.iSim_open != 0)
        {
            iResult = c33_SimStdoutSet( ulBreakAddr );
        }
        
        if (iResult == C33_NO_ERR)
        {
            if ( iParamNum == 5 )   /* output to file ? */
            {
                /* output file open */
                fpStdoutfile = fopen(cFileName, "wb");	/* "w" --> "wb" change T.Tazaki 2004/05/27 */
                if (fpStdoutfile == NULL)
                {
                    /* file open error */
                    c33_error (C33_ERR_FILE_OPEN, cFileName);
                    return;
                }
                strcpy( c33TargetInfo.cStdoutFileName,cFileName );
            }
            else
            {
                strcpy( c33TargetInfo.cStdoutFileName,"" );
            }
            c33TargetInfo.ulSoftPcBreak[C33_STDOUT_BREAK_NUM-1] = ulBreakAddr;
            c33TargetInfo.ulWriteBufAddr = ulWriteBufAddr;
        }
    }
    else
    {
        /* stdout CLEAR */
        if ( iParamNum != 2 )
        {
            c33_error (C33_ERR_COMMAND_PARAM);
            return;
        }

        if(c33TargetInfo.iIcd_open != 0)
        {
			/* change T.Tazaki 2005/05/06 >>> */
			unsigned short usCode2;	/* usCode --> usCode2 CHange T.Tazaki 2007/07/12 */
			/* unsigned long ulAddr; */ /* not used  del T.Tazaki 2007/07/12 */
			
	        /* write Original code */
	        
	        usCode2 = c33TargetInfo.usSoftPcBreakOrgCode[C33_STDOUT_BREAK_NUM-1];
	        iResult = c33_Cmd_ReadWrite(C33_WRITE, c33TargetInfo.ulSoftPcBreak[C33_STDOUT_BREAK_NUM-1], 2, C33_HALF_WORD, (unsigned char*)&usCode2);
			c33TargetInfo.ulSoftPcBreak[C33_STDOUT_BREAK_NUM-1] = C33_BREAK_DISABLE;
#if 0
            iResult = c33_SendMessage(C33_ID_SOFT_PC_BREAK_CLR, C33_STDOUT_BREAK_NUM, 0, 0, 0, 0, 0, 0);
            if (iResult == C33_NO_ERR)
            {
                iResult = c33_RecvMessage(C33_ID_SOFT_PC_BREAK_CLR, 0, 0, 0, 0, 0);
            }
#endif
			/* change T.Tazaki 2005/05/06 <<< */
        }
        else if(c33TargetInfo.iSim_open != 0)
        {
            c33_SimStdoutClear();
        }
        
        c33TargetInfo.ulSoftPcBreak[C33_STDOUT_BREAK_NUM-1] = C33_BREAK_DISABLE;

        /* output file close */
        if (fpStdoutfile != NULL)
        {
            fclose(fpStdoutfile);
        }
    }
}
/****************************************************************************/
/* Format        : static void c33_Lpt(int iParamNum, int iPortNum )        */
/* Arguments                                                                */
/*  int iParamNum             : Parameter number                            */
/*  int iPortNum              : Parallel port number                        */
/* Return value  : void                                                     */
/* Explanation   : icdv2 fast load mode set or reset.                       */
/****************************************************************************/
static void
c33_Lpt(int iParamNum, int iPortNum )
{
    /* Check number of parameter */
    if ( iParamNum != 1 && iParamNum != 2 )
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }
    if( iParamNum == 1 )
    {
        /* parallel mode OFF */
        g_iParallel_mode = 0;
    }
    else
    {
        if ( iPortNum >= 1 && iPortNum <= 2 )
        {
            /* parallel mode ON */
            g_iParallel_mode = 1;
            g_iParallel_LPTNo = iPortNum;
        }
        else
        {
            c33_error (C33_ERR_COMMAND_INVALID_PARAM);
            return;
        }
    }
}

/****************************************************************************/
/* Format        : static void c33_DeleteDataBreak(int iParamNum, char *cSymbol ) */
/* Arguments                                                                */
/*  int iParamNum             : Parameter number                            */
/*  char cSymbol              : Delete symbol name                          */
/* Return value  : void                                                     */
/* Explanation   : delete watch point                                       */
/****************************************************************************/
static void
c33_DeleteDataBreak(int iParamNum, char *cSymbol )
{
    /* Check number of parameter */
    if ( iParamNum != 2 )
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }

	c33_delete_watch_breakpoint( cSymbol );
}


/****************************************************************************/
/* Format        : static void c33_command_analysis (char *arg, struct c33_cmd_structure *pCmd) */
/* Arguments                                                                */
/*  char *arg                      : Command string                         */
/*  struct c33_cmd_structure *pCmd : C33 command structure                  */
/* Return value  : void                                                     */
/* Explanation   : C33 command analysis.                                    */
/****************************************************************************/
static void
c33_command_analysis (char *arg, struct c33_cmd_structure *pCmd)
{
    int i;
    int iResult;

    for (i = 0 ; i < C33_COMMAND_ARGUMENT_MAX ; i++)
    {
        while( *arg == '\t' || *arg == ' ')
        {
            if( *arg == 0)
            {
                return;
            }
            arg++;
        }

        switch (pCmd->iType[i])
        {
            case C33_COMMAND_ARGUMENT_SYM:
                iResult = sscanf(arg, "%s", &(pCmd->iStr[i][0]));
                if (iResult == 1)
                {
                    /* This function include error processing. */
                    pCmd->iParam[i] = parse_and_eval_address (&(pCmd->iStr[i][0]));
                    pCmd->iParamNum++;
                }
                break;

            case C33_COMMAND_ARGUMENT_STR:
                iResult = sscanf(arg, "%s", &(pCmd->iStr[i][0]));
                if (iResult == 1)
                {
                    pCmd->iParamNum++;
                }
                break;

            case C33_COMMAND_ARGUMENT_END:
                iResult = sscanf(arg, "%s", &(pCmd->iStr[i][0]));
                if (iResult == 1)
                {
                    pCmd->iParamNum++;
                }
                return;
            default:
                return;
        }

        while( *arg != '\t' && *arg != ' ')
        {
            if( *arg == 0)
            {
                return;
            }
            arg++;
        }
    }
}

/****************************************************************************/
/* Format        : void c33_command (char *arg, int from_tty)               */
/* Arguments                                                                */
/*  char *arg    : Command string                                           */
/*  int from_tty : Not used                                                 */
/* Return value  : void                                                     */
/* Explanation   : C33 command analysis and execution.                      */
/****************************************************************************/
void
c33_command (char *arg, int from_tty)
{
    extern Tcl_Interp *gdbtk_interp;
    struct c33_cmd_structure c33Command;
    memset(&c33Command, 0, sizeof(c33Command));

    if (arg == NULL)
    {
        c33_error (C33_ERR_COMMAND_INVALID);
        return;
    }

    if (strlen(arg) > C33_COMMAND_MAX)
    {
        c33_error (C33_ERR_COMMAND_MAX_LEN);
        return;
    }

    c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
    c33Command.iType[5] = C33_COMMAND_ARGUMENT_END;
    c33_command_analysis(arg, &c33Command);

    /* Check support command */
    if( (strcmp(c33Command.iStr[0],"ice") == 0 || strcmp(c33Command.iStr[0],"tf")  == 0 ||
         strcmp(c33Command.iStr[0],"ts")  == 0 || strcmp(c33Command.iStr[0],"td")  == 0 ||
         strcmp(c33Command.iStr[0],"fls") == 0 || strcmp(c33Command.iStr[0],"fle") == 0 ||
#ifdef __CYGWIN__
         strcmp(c33Command.iStr[0],"rm")  == 0 || strcmp(c33Command.iStr[0],"lpt") == 0) && c33TargetInfo.iIcd_open == C33_NOT_OPEN )
#else
         strcmp(c33Command.iStr[0],"rm")  == 0 ) && c33TargetInfo.iIcd_open == C33_NOT_OPEN )
#endif
    {
        c33_error (C33_ERR_COMMAND_NO_SUPPORT_MES);
        return;
    }

#ifdef __CYGWIN__
    if( strcmp(c33Command.iStr[0],"lpt") == 0 && c33TargetInfo.iIcd_open != C33_NOT_OPEN &&
        (c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) != C33_ICD_VER2 )
    {
        c33_error (C33_ERR_COMMAND_NO_SUPPORT_MES);
        return;
    }
#endif
    if( (strcmp(c33Command.iStr[0],"cycle") == 0 || strcmp(c33Command.iStr[0],"cycle0") == 0) && c33TargetInfo.iSim_open == 0 )
    {
        c33_error (C33_ERR_COMMAND_NO_SUPPORT_MES);
        return;
    }

	if( c33TargetInfo.iLogiana_Mode == 0 )		/* Not Logiana mode ? add T.Tazaki 2005/09/26 */
	{
	    if( (strcmp(c33Command.iStr[0],"oab") == 0 || strcmp(c33Command.iStr[0],"obb") == 0 ||
	         strcmp(c33Command.iStr[0],"obt") == 0 || strcmp(c33Command.iStr[0],"otd") == 0 || strcmp(c33Command.iStr[0],"cachehit") == 0 || /* add cachehit T.Tazaki 205/06/23 */
	         strcmp(c33Command.iStr[0],"otf") == 0 || strcmp(c33Command.iStr[0],"mmu") == 0) && ( c33TargetInfo.iIcd_open != C33_USB_OPEN || c33TargetInfo.ucAdvanced != C33_PRODUCT_ADV ))
	    {
	        c33_error (C33_ERR_COMMAND_NO_SUPPORT_MES);
	        return;
	    }
	}
	else
	{	/* Not check otd.otf when logiana mode add T.Tazaki 2005/09/26 */
	    if( (strcmp(c33Command.iStr[0],"oab") == 0 || strcmp(c33Command.iStr[0],"obb") == 0 ||
	         strcmp(c33Command.iStr[0],"obt") == 0 || strcmp(c33Command.iStr[0],"cachehit") == 0 || 
	         strcmp(c33Command.iStr[0],"mmu") == 0) && ( c33TargetInfo.iIcd_open != C33_USB_OPEN || c33TargetInfo.ucAdvanced != C33_PRODUCT_ADV ))
	    {
	        c33_error (C33_ERR_COMMAND_NO_SUPPORT_MES);
	        return;
	    }
	}

	/* mode command check@add T.Tazaki 2004/10/07 >>> */
	if( c33TargetInfo.iIcd_open == C33_USB_OPEN )			/* ICDV3 ? */
	{
		if(c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV)  /* ADV ? */
		{
		    if( strcmp(c33Command.iStr[0],"int") == 0 || strcmp(c33Command.iStr[0],"lpt") == 0 ||
		        strcmp(c33Command.iStr[0],"logiana") == 0 )  /* add "logiana" 2005/09/29 T.Tazaki */
		    {
		        c33_error (C33_ERR_COMMAND_NO_SUPPORT_MES);
		        return;
		    }
		}
		else
		{
			if( c33TargetInfo.iLogiana_Mode == 0 )		/* Not Logiana mode ? add T.Tazaki 2005/09/26 */
			{
				/* STD, PE */
			    if(  strcmp(c33Command.iStr[0],"int")  == 0 || strcmp(c33Command.iStr[0],"lpt")  == 0 || 
			         strcmp(c33Command.iStr[0],"oab")  == 0 || strcmp(c33Command.iStr[0],"obb")  == 0 ||
			         strcmp(c33Command.iStr[0],"obt")  == 0 || strcmp(c33Command.iStr[0],"otd")  == 0 ||
			         strcmp(c33Command.iStr[0],"otf")  == 0 || strcmp(c33Command.iStr[0],"mmu")  == 0 )
			    {
			        c33_error (C33_ERR_COMMAND_NO_SUPPORT_MES);
			        return;
			    }
			}
			else
			{
				/* STD, PE */			/* Not check otd.otf when logiana mode add T.Tazaki 2005/09/26 */
			    if(  strcmp(c33Command.iStr[0],"int")  == 0 || strcmp(c33Command.iStr[0],"lpt")  == 0 || 
			         strcmp(c33Command.iStr[0],"oab")  == 0 || strcmp(c33Command.iStr[0],"obb")  == 0 ||
			         strcmp(c33Command.iStr[0],"obt")  == 0 || strcmp(c33Command.iStr[0],"mmu")  == 0 )
			    {
			        c33_error (C33_ERR_COMMAND_NO_SUPPORT_MES);
			        return;
			    }
			}
		}
		/* for ICDV6 add T.Tazaki 2007/07/03 */
		if( g_iICD_kind == C33_ICDV4 )
		{
		    if(  strcmp(c33Command.iStr[0],"rstt")  == 0 || strcmp(c33Command.iStr[0],"timebrk")  == 0 )
		    {
		        c33_error (C33_ERR_COMMAND_NO_SUPPORT_MES);
		        return;
		    }
		}
	}
	if(c33TargetInfo.iSim_open != 0)	/* SIM ? */
	{
	    if(  strcmp(c33Command.iStr[0],"rm")   == 0 ||
	         strcmp(c33Command.iStr[0],"fls")  == 0 || strcmp(c33Command.iStr[0],"fle")  == 0 ||
	         strcmp(c33Command.iStr[0],"td")   == 0 || strcmp(c33Command.iStr[0],"ts")   == 0 ||
	         strcmp(c33Command.iStr[0],"tf")   == 0 || 
	         strcmp(c33Command.iStr[0],"fwe")  == 0 || strcmp(c33Command.iStr[0],"fwlp") == 0 ||
	         strcmp(c33Command.iStr[0],"fwld") == 0 || strcmp(c33Command.iStr[0],"fwdc") == 0 ||
	         strcmp(c33Command.iStr[0],"fwd")  == 0 || strcmp(c33Command.iStr[0],"das")  == 0 ||
	         strcmp(c33Command.iStr[0],"lpt")  == 0 || strcmp(c33Command.iStr[0],"oscwait")  == 0 ||	/* add "oscwait" 2005/06/08 T.Tazaki */									
	         strcmp(c33Command.iStr[0],"oab")  == 0 || strcmp(c33Command.iStr[0],"obb")  == 0 ||
	         strcmp(c33Command.iStr[0],"obt")  == 0 || strcmp(c33Command.iStr[0],"otd")  == 0 ||
	         strcmp(c33Command.iStr[0],"otf")  == 0 || strcmp(c33Command.iStr[0],"mmu")  == 0 ||
	         strcmp(c33Command.iStr[0],"readmd") == 0 || strcmp(c33Command.iStr[0],"dclk")  == 0 || 	/* add "dclk" 2005/05/24 T.Tazaki */
	         strcmp(c33Command.iStr[0],"logiana") == 0 || 												/* add "logiana" 2005/09/22 T.Tazaki */
	         strcmp(c33Command.iStr[0],"rstt")  == 0 || strcmp(c33Command.iStr[0],"timebrk")  == 0 ) 	/* for ICDV6 add T.Tazaki 2007/07/03 */
	    {
	        c33_error (C33_ERR_COMMAND_NO_SUPPORT_MES);
	        return;
	    }
	}
	if( c33TargetInfo.iIcd_open == C33_COM_OPEN )
	{
		if((c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) == C33_ICD_VER2)  /* ICDV2 ? */
		{
		    if(  strcmp(c33Command.iStr[0],"int")  == 0 || strcmp(c33Command.iStr[0],"logiana") == 0 ||		/* add "logiana" 2005/09/29 T.Tazaki */
		         strcmp(c33Command.iStr[0],"fwe")  == 0 || strcmp(c33Command.iStr[0],"fwlp") == 0 ||
		         strcmp(c33Command.iStr[0],"fwld") == 0 || strcmp(c33Command.iStr[0],"fwdc") == 0 ||
		         strcmp(c33Command.iStr[0],"fwd")  == 0 || strcmp(c33Command.iStr[0],"das")  == 0 ||
		         strcmp(c33Command.iStr[0],"oab")  == 0 || strcmp(c33Command.iStr[0],"obb")  == 0 ||
		         strcmp(c33Command.iStr[0],"obt")  == 0 || strcmp(c33Command.iStr[0],"otd")  == 0 ||
		         strcmp(c33Command.iStr[0],"otf")  == 0 || strcmp(c33Command.iStr[0],"mmu")  == 0 ||
		         strcmp(c33Command.iStr[0],"readmd") == 0 || strcmp(c33Command.iStr[0],"oscwait")  == 0 ||	/* add "oscwait" 2005/06/08 T.Tazaki */	
		         strcmp(c33Command.iStr[0],"rstt")  == 0  || strcmp(c33Command.iStr[0],"timebrk")  == 0 ) 	/* for ICDV6 add T.Tazaki 2007/07/03 */
		    {
		        c33_error (C33_ERR_COMMAND_NO_SUPPORT_MES);
		        return;
		    }
		}
		if((c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) == C33_ICD_MON)  /* MON ? */
		{
		    if(  strcmp(c33Command.iStr[0],"rm")    == 0 || strcmp(c33Command.iStr[0],"int")     == 0 ||
		         strcmp(c33Command.iStr[0],"tm")    == 0 || strcmp(c33Command.iStr[0],"td")      == 0 ||
		         strcmp(c33Command.iStr[0],"ts")    == 0 || strcmp(c33Command.iStr[0],"tf")      == 0 ||
		         strcmp(c33Command.iStr[0],"fwe")   == 0 || strcmp(c33Command.iStr[0],"fwlp")    == 0 ||
		         strcmp(c33Command.iStr[0],"fwld")  == 0 || strcmp(c33Command.iStr[0],"fwdc")    == 0 ||
		         strcmp(c33Command.iStr[0],"fwd")   == 0 || strcmp(c33Command.iStr[0],"clockmd") == 0 ||
		         strcmp(c33Command.iStr[0],"clock") == 0 || strcmp(c33Command.iStr[0],"das")     == 0 ||
		         strcmp(c33Command.iStr[0],"lpt")   == 0 || strcmp(c33Command.iStr[0],"oscwait") == 0 ||	/* add "oscwait" 2005/06/08 T.Tazaki */	
		         strcmp(c33Command.iStr[0],"oab")   == 0 || strcmp(c33Command.iStr[0],"obb")     == 0 ||
		         strcmp(c33Command.iStr[0],"obt")   == 0 || strcmp(c33Command.iStr[0],"otd")     == 0 ||
		         strcmp(c33Command.iStr[0],"otf")   == 0 || strcmp(c33Command.iStr[0],"mmu")     == 0 ||
		         strcmp(c33Command.iStr[0],"logiana") == 0 || 												/* add "logiana" 2005/09/29 T.Tazaki */
		         strcmp(c33Command.iStr[0],"rstt")  == 0 || strcmp(c33Command.iStr[0],"timebrk")  == 0 ) 	/* for ICDV6 add T.Tazaki 2007/07/03 */
		    {
		        c33_error (C33_ERR_COMMAND_NO_SUPPORT_MES);
		        return;
		    }
		}
	}
	/* add T.Tazaki 2004/10/07 <<< */
	
	/* bus trace disabled ?(DIPSW1:D4) and "c33 obt" ? add T.Tazaki 2004/05/31 */
	if( c33TargetInfo.iIcd_open == C33_USB_OPEN )
	{
		if( (c33TargetInfo.iVerRdIcdStatus & C33_DIPSW1_BUS_TRACE) != 0 )	/* bus trace is disable ? */
		{
			if( strcmp(c33Command.iStr[0],"obt") == 0 || strcmp(c33Command.iStr[0],"otd") == 0 || strcmp(c33Command.iStr[0],"otf") == 0 )
			{
		        c33_error (C33_ERR_COMMAND_NO_BUS_TRACE_MES);
		        return;
		    }
			/* check if Bus trace mode is disabled.		add T.Tazaki 2004/09/29 >>> */
			/* c33 obt2 is secret command. it is called /share */
		    if( strcmp(c33Command.iStr[0],"obt2") == 0 )
		    {
				if( g_use_windows == 1 )
				{
			        Tcl_SetVar( gdbtk_interp,"c33_obt_flag","NG",TCL_GLOBAL_ONLY );		/* bustrace.itb:BusTrace::_ok{} set variable */
			        return;
			    }
				strcpy( c33Command.iStr[0],"obt" );		/* obt2 --> obt */
			}
			/* check if Bus trace mode is disabled.		add T.Tazaki 2004/09/29 <<< */

		}
		/* check if Bus trace mode is disabled.		add T.Tazaki 2004/09/29 >>> */
	    if( c33TargetInfo.iBust_Mode == 0 )
		{
			if( strcmp(c33Command.iStr[0],"otd") == 0 || strcmp(c33Command.iStr[0],"otf") == 0 )
			{
		        c33_error (C33_ERR_COMMAND_NO_BUS_TRACE_MES);
		        return;
		    }
		}
	    if( (c33TargetInfo.ucBrkFunc & C33_GDB_STS_BUS_TRACE) == 0 )
	    {
		    if( strcmp(c33Command.iStr[0],"obt2") == 0 )
		    {
				if( g_use_windows == 1 )
				{
			        Tcl_SetVar( gdbtk_interp,"c33_obt_flag","NG",TCL_GLOBAL_ONLY );		/* bustrace.itb:BusTrace::_ok{} set variable */
			        return;
			    }
				strcpy( c33Command.iStr[0],"obt" );		/* obt2 --> obt  */
			}
	    }
		/* check if Bus trace mode is disabled.		add T.Tazaki 2004/09/29 <<< */
		
		/* when ICD3 firm version is less than 4.1, oscwait command is invalid 		add T.Tazaki 2005/06/08 >>> */
	    if( strcmp(c33Command.iStr[0],"oscwait") == 0 )
	    {
			if( g_iICD_kind == C33_ICDV4 && c33TargetInfo.iVerRdIcdFirmVer <= 0x40 )	/* Check ICDV4  add T.Tazaki 2007/07/18 */
			{
		        c33_error (C33_ERR_COMMAND_OSCWAIT_NO_SUPPORT_MES);
		        return;
		    }
		}
		/* when ICD3 firm version is less than 4.1, oscwait command is invalid 		add T.Tazaki 2005/06/08 <<< */
		
	}
	/* add T.Tazaki 2004/09/29 >>> */
    if( strcmp(c33Command.iStr[0],"obt2") == 0 )
	{
		strcpy( c33Command.iStr[0],"obt" );		/* obt2 --> obt  */
	}
	/* add T.Tazaki 2004/09/29 <<< */
	
	if( (c33TargetInfo.iVerRdIcdStatus & C33_DIPSW1_PC_TRACE) != 0 )	/* pc trace is disable ? 	add T.Tazaki 2004/06/02 */
	{
		if( strcmp(c33Command.iStr[0],"td") == 0 || strcmp(c33Command.iStr[0],"tf") == 0 || strcmp(c33Command.iStr[0],"ts") == 0 )
		{
	        c33_error (C33_ERR_COMMAND_NO_PC_TRACE_MES);
	        return;
	    }
	}

    if( strcmp(c33Command.iStr[0],"firmupdate") == 0 && c33TargetInfo.iIcd_open != C33_USB_OPEN )	/* add T.Tazaki 2004/05/06 */
    {
        c33_error (C33_ERR_COMMAND_NO_SUPPORT_MES);
        return;
    }
    /* check if target command not done  add T.Tazaki 2007/12/25 */
	if(c33TargetInfo.iSim_open == 0 && c33TargetInfo.iIcd_open == C33_NOT_OPEN)	/* NO SIM & NO ICD  */
	{
	    if ( strcmp(arg,"rstc") == 0 || strcmp(arg,"rsth") == 0 || strcmp(arg,"rstt") == 0 )
	    {
	        c33_error (C33_ERR_COMMAND_NO_SUPPORT_MES);
	        return;
	    }
	}

    if (strcmp(arg,"rstc") == 0)
    {
        /* Cold reset */
        c33_Cmd_Reset(C33_ICD_COLD_RESET);
    }
    else if (strcmp(arg,"rsth") == 0)
    {
        /* Hot reset */
        c33_Cmd_Reset(C33_ICD_HOT_RESET);
    }
    else if (strcmp(arg,"rstt") == 0)	/* for ICDV6 add T.Tazaki 2007/07/03 */
    {
		/* Target Reset */
        c33_Cmd_TargetReset();
    }
    /* add 2006/06/05 M.Kai  add user_def, user_rstc, user_rsth command  >>> */
 	else if (strcmp(arg, "user_def") == 0)
	{
		if( c33_Cmd_UserDef(C33_USER_DEFINE) == C33_ERR){
			 puts_filtered (C33_MSG_USER_DEF_FILE_NOT_FOUND); //file not found
		}
	
	}   
 	else if (strcmp(arg, "user_rstc") == 0)
	{
		if( c33_Cmd_UserDef(C33_USER_COLD_RESET) == C33_ERR){
			 puts_filtered (C33_MSG_USER_RSTC_FILE_NOT_FOUND); //file not found
		}		
		
	
	}   
 	else if (strcmp(arg, "user_rsth") == 0)
	{
		if( c33_Cmd_UserDef(C33_USER_HOT_RESET) == C33_ERR){
			 puts_filtered (C33_MSG_USER_RSTH_FILE_NOT_FOUND); //file not found
		}		
	}   
    /* add 2006/06/05 M.Kai  add user_def, user_rstc, user_rsth command  <<< */
    else if (strcmp(c33Command.iStr[0],"log") == 0)
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_Log(c33Command.iParamNum, c33Command.iStr[1]);
    }
    else if (strcmp(c33Command.iStr[0],"fb") == 0)
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[3] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[4] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[5] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_MemoryFill(c33Command.iParamNum, C33_BYTE, c33Command.iParam[1], c33Command.iParam[2], c33Command.iParam[3]);
    }
    else if (strcmp(c33Command.iStr[0],"fh") == 0)
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[3] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[4] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[5] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_MemoryFill(c33Command.iParamNum, C33_HALF_WORD, c33Command.iParam[1], c33Command.iParam[2], c33Command.iParam[3]);
    }
    else if (strcmp(c33Command.iStr[0],"fw") == 0)
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[3] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[4] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[5] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_MemoryFill(c33Command.iParamNum, C33_WORD, c33Command.iParam[1], c33Command.iParam[2], c33Command.iParam[3]);
    }
    else if (strcmp(c33Command.iStr[0],"mvb") == 0)
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[3] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[4] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[5] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_MemoryMove(c33Command.iParamNum, C33_BYTE, c33Command.iParam[1], c33Command.iParam[2], c33Command.iParam[3]);
    }
    else if (strcmp(c33Command.iStr[0],"mvh") == 0)
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[3] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[4] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[5] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_MemoryMove(c33Command.iParamNum, C33_HALF_WORD, c33Command.iParam[1], c33Command.iParam[2], c33Command.iParam[3]);
    }
    else if (strcmp(c33Command.iStr[0],"mvw") == 0)
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[3] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[4] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[5] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_MemoryMove(c33Command.iParamNum, C33_WORD, c33Command.iParam[1], c33Command.iParam[2], c33Command.iParam[3]);
    }
    else if (strcmp(c33Command.iStr[0],"rpf") == 0)
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_ReadParameterFile(c33Command.iParamNum, c33Command.iStr[1]);
    }
    else if (strcmp(c33Command.iStr[0],"map") == 0)
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_ShowMapInfo(c33Command.iParamNum);
    }
    else if (strcmp(c33Command.iStr[0],"df") == 0)
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[3] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[4] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[5] = C33_COMMAND_ARGUMENT_STR;	/* add Append mode T.Tazaki 2004/05/26 */
        c33Command.iType[6] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_DumpFile(c33Command.iParamNum, c33Command.iParam[1], c33Command.iParam[2], c33Command.iParam[3], c33Command.iStr[4],c33Command.iStr[5]);
    }
    else if (strcmp(c33Command.iStr[0],"tm") == 0)
    {
        if(c33TargetInfo.iIcd_open != C33_NOT_OPEN)
        {
            memset(&c33Command, 0, sizeof(c33Command));
            c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
            c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
            c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
            c33Command.iType[3] = C33_COMMAND_ARGUMENT_SYM;
            c33Command.iType[4] = C33_COMMAND_ARGUMENT_SYM;
            c33Command.iType[5] = C33_COMMAND_ARGUMENT_SYM;
            c33Command.iType[6] = C33_COMMAND_ARGUMENT_SYM;
            c33Command.iType[7] = C33_COMMAND_ARGUMENT_END;
            c33_command_analysis(arg, &c33Command);
            c33_IcdTraceMode(c33Command.iParamNum, c33Command.iParam[1], c33Command.iParam[2], c33Command.iParam[3], c33Command.iParam[4], c33Command.iParam[5], c33Command.iParam[6]);
        }
        else if (c33TargetInfo.iSim_open != 0)
        {
            memset(&c33Command, 0, sizeof(c33Command));
            c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
            c33Command.iType[1] = C33_COMMAND_ARGUMENT_STR;
            c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
            c33Command.iType[3] = C33_COMMAND_ARGUMENT_STR;
            c33Command.iType[4] = C33_COMMAND_ARGUMENT_END;
            c33_command_analysis(arg, &c33Command);
            c33_SimTraceMode(c33Command.iParamNum, c33Command.iStr[1], c33Command.iParam[2], c33Command.iStr[3]);
        }
    }
    else if (strcmp(c33Command.iStr[0],"callmd") == 0)
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[3] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_CallMode(c33Command.iParamNum, c33Command.iParam[1], c33Command.iStr[2]);
    }
    else if (strcmp(c33Command.iStr[0],"call") == 0)
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[3] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[4] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[5] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_Call(c33Command.iParamNum, c33Command.iParam[1], c33Command.iParam[2], c33Command.iParam[3], c33Command.iParam[4]);
    }
    else if (strcmp(c33Command.iStr[0],"int") == 0)
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[3] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_Int(c33Command.iParamNum, c33Command.iParam[1], c33Command.iParam[2]);
    }

    /* Icd function */
    else if (strcmp(c33Command.iStr[0],"fls") == 0)
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[3] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[4] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[5] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_IcdFlashSet(c33Command.iParamNum, c33Command.iParam[1], c33Command.iParam[2], c33Command.iParam[3], c33Command.iParam[4]);
    }
    else if (strcmp(c33Command.iStr[0],"fle") == 0)
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[3] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[4] = C33_COMMAND_ARGUMENT_SYM;	/* add T.Tazaki 2006/05/16 [Timer] */
        c33Command.iType[5] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_IcdFlashErase(c33Command.iParamNum, c33Command.iParam[1], c33Command.iParam[2], c33Command.iParam[3], c33Command.iParam[4]);/* add T.Tazaki 2006/05/16 [Timer] */
    }
    else if (strcmp(c33Command.iStr[0],"tf") == 0)
    {
        if(c33TargetInfo.iIcd_open != C33_NOT_OPEN)
        {
            memset(&c33Command, 0, sizeof(c33Command));
            c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
            c33Command.iType[1] = C33_COMMAND_ARGUMENT_STR;
            c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
            c33Command.iType[3] = C33_COMMAND_ARGUMENT_SYM;
            c33Command.iType[4] = C33_COMMAND_ARGUMENT_END;
            c33_command_analysis(arg, &c33Command);
            c33_IcdTraceFile(c33Command.iParamNum, c33Command.iStr[1], c33Command.iParam[2], c33Command.iParam[3]);
        }
    }
    else if (strcmp(c33Command.iStr[0],"ts") == 0)
    {
        if(c33TargetInfo.iIcd_open != C33_NOT_OPEN)
        {
            memset(&c33Command, 0, sizeof(c33Command));
            c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
            c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
            c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
            c33Command.iType[3] = C33_COMMAND_ARGUMENT_SYM;
            c33Command.iType[4] = C33_COMMAND_ARGUMENT_END;
            c33_command_analysis(arg, &c33Command);
            c33_IcdTraceSearch(c33Command.iParamNum, c33Command.iParam[1], c33Command.iParam[2], c33Command.iParam[3]);
        }
    }
    else if (strcmp(c33Command.iStr[0],"td") == 0)  /* add T.Tazaki 2003/07/08 */
    {
        if(c33TargetInfo.iIcd_open != C33_NOT_OPEN)
        {
            memset(&c33Command, 0, sizeof(c33Command));
            c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
            c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
            c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
            c33Command.iType[3] = C33_COMMAND_ARGUMENT_END;
            c33_command_analysis(arg, &c33Command);
            c33_IcdTraceDisplay(c33Command.iParamNum, c33Command.iParam[1], c33Command.iParam[2]);
        }
    }
    else if (strcmp(c33Command.iStr[0],"ice") == 0)
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_IcdIceCommand(c33Command.iParamNum, c33Command.iStr[1]);
    }
    else if (strcmp(c33Command.iStr[0],"clockmd") == 0)
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[3] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_ClockMode(c33Command.iParamNum, c33Command.iParam[1], c33Command.iParam[2]);
    }
    else if (strcmp(c33Command.iStr[0],"clock") == 0)
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_Clock(c33Command.iParamNum);
    }
    else if (strcmp(c33Command.iStr[0],"stdin") == 0)
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[3] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[4] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[5] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_Stdin(c33Command.iParamNum, c33Command.iParam[1], c33Command.iParam[2], c33Command.iParam[3], c33Command.iStr[4]);
    }
    else if (strcmp(c33Command.iStr[0],"stdout") == 0)
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[3] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[4] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[5] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_Stdout(c33Command.iParamNum, c33Command.iParam[1], c33Command.iParam[2], c33Command.iParam[3], c33Command.iStr[4]);
    }
    else if (strcmp(c33Command.iStr[0],"fwe") == 0)
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_IcdVer3FlashErase(c33Command.iParamNum, c33Command.iParam[1]);
    }
    else if (strcmp(c33Command.iStr[0],"fwlp") == 0)
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[3] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[4] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[5] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_IcdVer3FlashProgLoad(c33Command.iParamNum, c33Command.iStr[1], c33Command.iParam[2], c33Command.iParam[3], c33Command.iStr[4]);
    }
    else if (strcmp(c33Command.iStr[0],"fwld") == 0)
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[3] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[4] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[5] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[6] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_IcdVer3FlashDataLoad(c33Command.iParamNum, c33Command.iStr[1], c33Command.iParam[2], c33Command.iParam[3],c33Command.iParam[4], c33Command.iStr[5]);
    }
    else if (strcmp(c33Command.iStr[0],"fwdc") == 0)
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[3] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[4] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[5] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[6] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[7] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_IcdVer3FlashDataCopy(c33Command.iParamNum, c33Command.iParam[1], c33Command.iParam[2], c33Command.iParam[3],
                                                       c33Command.iParam[4], c33Command.iParam[5], c33Command.iStr[6]);
    }
    else if (strcmp(c33Command.iStr[0],"fwd") == 0)
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_IcdVer3FlashInfoDisplay(c33Command.iParamNum);
    }
    else if (strcmp(c33Command.iStr[0],"das") == 0)     /* add 2003/06/11 T.Tazaki */
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[3] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[4] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_IcdVer3DebugBaseSet(c33Command.iParamNum, c33Command.iParam[1], c33Command.iParam[2], c33Command.iParam[3]);
    }
    else if (strcmp(c33Command.iStr[0],"oab") == 0)     /* add 2003/06/25 T.Tazaki */
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[3] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[4] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[5] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_IcdVer3AreaBreak(c33Command.iParamNum, c33Command.iStr[1],c33Command.iParam[2], c33Command.iStr[3], c33Command.iParam[4]);
    }
    else if (strcmp(c33Command.iStr[0],"obb") == 0)     /* add 2003/06/25 T.Tazaki */
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[3] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[4] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[5] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[6] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[7] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[8] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[9] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[10] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[11] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[12] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[13] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[14] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[15] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[16] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[17] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_IcdVer3BusBreak(c33Command.iParamNum, c33Command.iStr[1],   c33Command.iParam[2], c33Command.iParam[3], c33Command.iParam[4],
                               c33Command.iStr[5],   c33Command.iParam[6], c33Command.iParam[7], c33Command.iParam[8], c33Command.iParam[9],
                                c33Command.iParam[10],c33Command.iParam[11],c33Command.iParam[12],c33Command.iParam[13],c33Command.iParam[14],
                                c33Command.iParam[15],c33Command.iParam[16]);
    }
    else if (strcmp(c33Command.iStr[0],"obt") == 0)     /* add 2003/06/25 T.Tazaki */
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[3] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[4] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[5] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[6] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[7] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[8] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[9] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[10] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[11] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_IcdVer3BusTrace(c33Command.iParamNum, c33Command.iParam[1], c33Command.iParam[2], c33Command.iParam[3], c33Command.iStr[4],
                                c33Command.iParam[5], c33Command.iParam[6], c33Command.iParam[7], c33Command.iParam[8], c33Command.iParam[9],
                                c33Command.iParam[10] );
    }
    else if (strcmp(c33Command.iStr[0],"otf") == 0)     /* add 2003/06/25 T.Tazaki */
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;	/* add Mode 2004/03/31 T.Tazaki */
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[3] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[4] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[5] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_IcdVer3TraceFile(c33Command.iParamNum, c33Command.iParam[1], c33Command.iStr[2], c33Command.iParam[3], c33Command.iParam[4]);
    }
    else if (strcmp(c33Command.iStr[0],"otd") == 0)     /* add 2003/06/25 T.Tazaki */
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;	/* add Mode 2004/03/31 T.Tazaki */
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[3] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[4] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_IcdVer3TraceDisplay(c33Command.iParamNum, c33Command.iParam[1], c33Command.iParam[2], c33Command.iParam[3]);
    }
    else if (strcmp(c33Command.iStr[0],"mmu") == 0)     /* add 2004/08/03 T.Tazaki */
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_IcdVer3MMU(c33Command.iParamNum, c33Command.iParam[1] );
    }
    else if (strcmp(c33Command.iStr[0],"rm") == 0)     /* add 2003/07/29 T.Tazaki */
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[3] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_IcdReadMemory(c33Command.iParamNum, c33Command.iParam[1], c33Command.iParam[2]);
    }
    else if (strcmp(c33Command.iStr[0],"firmupdate") == 0)	/* add 2004/04/26 T.Tazaki */
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_IcdVer3FirmUpdate(c33Command.iParamNum, c33Command.iStr[1]);
    }
    else if (strcmp(c33Command.iStr[0],"readmd") == 0)     /* add 2004/11/01 T.Tazaki */
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_IcdVer3ReadMode(c33Command.iParamNum, c33Command.iParam[1] );
    }
	else if (strcmp(c33Command.iStr[0],"timebrk") == 0)		/* add 2007/07/03 T.Tazaki */
	{
		memset(&c33Command, 0, sizeof(c33Command));
		c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
		c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
		c33Command.iType[2] = C33_COMMAND_ARGUMENT_END;
		c33_command_analysis(arg, &c33Command);
		c33_IcdVer6TimerBreak(c33Command.iParamNum,c33Command.iParam[1]); 
	}
#ifdef __CYGWIN__
    else if (strcmp(c33Command.iStr[0],"lpt") == 0)     /* add 2003/10/01 T.Tazaki */
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_Lpt(c33Command.iParamNum, c33Command.iParam[1]);
    }
#endif
    else if (strcmp(c33Command.iStr[0],"delete") == 0)	/* add 2003/12/08 T.Tazaki */
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_DeleteDataBreak(c33Command.iParamNum, c33Command.iStr[1]);
    }
    else if (strcmp(c33Command.iStr[0],"dclk") == 0)		/* add T.Tazaki 2005/05/24 */
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_Dclock(c33Command.iParamNum, c33Command.iParam[1]);
    }
    else if (strcmp(c33Command.iStr[0],"logiana") == 0)		/* add T.Tazaki 2005/09/22 */
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[3] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_LogianaMode(c33Command.iParamNum, c33Command.iStr[1],c33Command.iParam[2]);
    }
    else if (strcmp(c33Command.iStr[0],"clip_copy") == 0)	/* add 2005/05/06 T.Tazaki */
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_ClipCopy(c33Command.iParamNum, c33Command.iStr[1]);
    }
    else if (strcmp(c33Command.iStr[0],"oscwait") == 0)	/* add 2005/06/08 T.Tazaki */
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_IcdOscWait(c33Command.iParamNum, c33Command.iParam[1]);
    }
    else if (strcmp(c33Command.iStr[0],"cachehit") == 0) /* add 2005/06/23 T.Tazaki */
    {
        c33_IcdVer3CacheHit();
    }
#ifdef __CYGWIN__		/* add T.Tazaki 2005/09/05 */
    else if (strcmp(c33Command.iStr[0],"os_mon") == 0) 		/* add T.Tazaki 2005/09/05 */
    {
        c33_OSMon();
    }
#endif
    else if (strcmp(c33Command.iStr[0],"help") == 0)     /* add 2003/08/05 T.Tazaki */
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[3] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_Help(c33Command.iParamNum, c33Command.iStr[1], c33Command.iStr[2]);
    }
    else if (strcmp(c33Command.iStr[0],"help_dev") == 0)     /* add 2005/02/02 T.Tazaki */
    {
        c33_Help_dev();
    }
    else if (strcmp(arg,"get_bp_enable") == 0)                      /* add 2003/07/15 T.Tazaki */
    {
        /* Get soft pc break status(enable/disable) : This is the internal command for TCL */
        c33_Get_bp_status();
    }
    /* Debug function */
    else if (strcmp(c33Command.iStr[0],"debug_icd") == 0)
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[3] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[4] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[5] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_IcdDebug(c33Command.iParam[1], c33Command.iParam[2], c33Command.iParam[3], c33Command.iParam[4]);
    }
    /* QA function 				add T.Tazaki 2004/10/18 */
    else if (strcmp(c33Command.iStr[0],"usb_log") == 0)
    {
        memset(&c33Command, 0, sizeof(c33Command));
        c33Command.iType[0] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[1] = C33_COMMAND_ARGUMENT_SYM;
        c33Command.iType[2] = C33_COMMAND_ARGUMENT_STR;
        c33Command.iType[3] = C33_COMMAND_ARGUMENT_END;
        c33_command_analysis(arg, &c33Command);
        c33_UsbLog(c33Command.iParam[1], 0, c33Command.iStr[2]);
    }
    else {
        c33_error (C33_ERR_COMMAND_INVALID);
    }
}

/****************************************************************************/
/* Format        : int c33_CheckMapArea( CORE_ADDR ulAddr)                  */
/* Arguments                                                                */
/*  CORE_ADDR ulAddr : Check address                                        */
/* Return value                                                             */
/*  C33_NO_ERR      : Success return                                        */
/*  NOMAP_MEM_BREAK : No map area                                           */
/* Explanation   : Check memory map area from address.                      */
/****************************************************************************/
int
c33_CheckMapArea( CORE_ADDR ulAddr)
{
    struct c33_map_info *pstMapInfo;
    int iStatus = C33_NO_ERR;
    
    if( c33TargetInfo.pMapInfo != NULL ){

        iStatus = NOMAP_MEM_BREAK;

        pstMapInfo = (struct c33_map_info *)c33TargetInfo.pMapInfo;
        
        while( pstMapInfo )
        {
            /* check if address renge */
            if(( ulAddr >= pstMapInfo->ulStartAddr ) && ( ulAddr <= pstMapInfo->ulEndAddr ))
            {
                iStatus = C33_NO_ERR;    /* address is in map area */
                break;
            }
            
            pstMapInfo = (struct c33_map_info *)pstMapInfo->pMapNext;
        }
    }
    
    return iStatus;
}

/* add T.Tazaki 2005/05/12 >>> */
/****************************************************************************/
/* Format        : int c33_CheckMapArea2( CORE_ADDR ulAddr)                 */
/* Arguments                                                                */
/*  CORE_ADDR ulAddr : Check address                                        */
/* Return value                                                             */
/*  C33_NO_ERR      : Success return                                        */
/*  C33_ERR         : No map area                                           */
/* Explanation   : Check memory map area from address.                      */
/****************************************************************************/
int
c33_CheckMapArea2( CORE_ADDR ulAddr)
{
    struct c33_map_info *pstMapInfo;
    int iStatus = C33_ERR;
    
    if( c33TargetInfo.pMapInfo != NULL ){

        iStatus = C33_ERR;

        pstMapInfo = (struct c33_map_info *)c33TargetInfo.pMapInfo;
        
        while( pstMapInfo )
        {
			if( pstMapInfo->iMapAttr == C33_RAM )
			{
	            /* check if address renge */
	            if(( ulAddr >= pstMapInfo->ulStartAddr ) && ( ulAddr <= pstMapInfo->ulEndAddr ))
	            {
	                iStatus = C33_NO_ERR;    /* address is in map area */
	                break;
	            }
            }
            pstMapInfo = (struct c33_map_info *)pstMapInfo->pMapNext;
        }
    }
    
    return iStatus;
}
/* add T.Tazaki 2005/05/12 <<< */

/****************************************************************************/
/* Format        : int c33_BreakAddrCheck(unsigned long ulAddr, int iKind)  */
/* Arguments                                                                */
/*  unsigned long ulAddr : Break address                                    */
/*  int iKind            : C33_SOFT_PC_BREAK or C33_HARD_PC_BREAK or C33_TRACE_TRGGER_BREAK */
/* Return value                                                             */
/*  C33_NO_ERR   : Success return                                           */
/*  C33_ERR      : Error return                                             */
/* Explanation   : It investigates in the address which can be set up.      */
/****************************************************************************/
int
c33_BreakAddrCheck (unsigned long ulAddr, int iKind)
{
    char cErrMsg[1024];
    int iResult = C33_NO_ERR;
    unsigned short usCode[2];


	/* check if CPU is ADV and Hard Break.    add T.Tazaki 2004/06/24 >>> */
    if (c33TargetInfo.ucCpuType == C33_ADV)
    {
		if( iKind == C33_HARD_PC_BREAK || iKind == C33_TRACE_TRGGER_BREAK )
		{
			return C33_NO_ERR;	/* Skip check */
		}
	}
	/* add T.Tazaki 2004/06/24 <<< */

	/* soft pc break need heap area by "c33 rpf" command.  add T.Tazaki 2005/05/13 >>> */
    if(c33TargetInfo.iIcd_open != C33_NOT_OPEN)
    {
		if( iKind == C33_SOFT_PC_BREAK )
		{
			/* check if ulAddr is in map area. */
			if( c33_CheckMapArea2( ulAddr ) == C33_ERR )
			{
		        sprintf(cErrMsg, C33_ERR_MSG_BRK_NO_RAM, (int)ulAddr);
		        strcat(cErrMsg, "\n");
	            strcat(cErrMsg, C33_ERR_MSG_ICD_SOFT_PC);
	    	    c33_error (cErrMsg);
				return C33_ERR;		/* Skip check */
			}
		}
	}
	/* add T.Tazaki 2005/05/13 <<< */

    memset(&usCode, 0, sizeof(usCode));/* 0 is nop*/

    /* read target instruction */
//  c33_Cmd_ReadWrite(C33_READ, ulAddr , 2, 2, (char*)&usCode[1]);									/* speed up T.Tazaki 2005/05/12 */
 	c33_ReadWriteMapMem(C33_READ, ulAddr, C33_HALF_WORD, C33_HALF_WORD, (char*)&usCode[1]);			/* speed up T.Tazaki 2005/05/12 */
    if (ulAddr > 2 )
    {
//      c33_Cmd_ReadWrite(C33_READ, ulAddr - 2, 2, 2, (char*)&usCode[0]);							/* speed up T.Tazaki 2005/05/12 */
 		c33_ReadWriteMapMem(C33_READ, ulAddr - 2, C33_HALF_WORD, C33_HALF_WORD, (char*)&usCode[0]);	/* speed up T.Tazaki 2005/05/12 */
    }

    /* ext check */
    if ((usCode[1] & 0xe000) == 0xc000)
    {
        /* ext instruction */
        if ((usCode[0] & 0xe000) == 0xc000)
        {
            /* ext xx               */ /* usCode[0] */
            /* ext xx  <- can't set */ /* usCode[1] */
            iResult = C33_ERR;
        }
        else
        {
            /* nop xx               */ /* usCode[0] */
            /* ext xx  <- can set   */ /* usCode[1] */
        }
    }
    else
    { 
        /* not ext instruction or delayed, not ld.c add T.Tazaki 2004/07/08 */
        if ((usCode[0] & 0xe000) == 0xc000 || ((usCode[0] & 0xe000) == 0 && (usCode[0] & C33_DELAYED_BIT) != 0) && usCode[0] != 0x01d0 )
        {
            /* ext xx               */ /* usCode[0] */
            /* add xx  <- can't set */ /* usCode[1] */
            /* jp.d xx              */ /* usCode[0] */
            /* add xx  <- can't set */ /* usCode[1] */
            iResult = C33_ERR;
        }
        else
        {
            /* nop xx               */ /* usCode[0] */
            /* add xx  <- can   set */ /* usCode[1] */
        }
    }


    if(iResult != C33_NO_ERR)
    {
        sprintf(cErrMsg, C33_ERR_MSG_BRK_EXT, (int)ulAddr);
        strcat(cErrMsg, "\n");
        if (iKind == C33_SOFT_PC_BREAK)
        {
            strcat(cErrMsg, C33_ERR_MSG_ICD_SOFT_PC);
        }
        else if (iKind == C33_HARD_PC_BREAK)
        {
            strcat(cErrMsg, C33_ERR_MSG_ICD_HARD_PC);
        }
        else if (iKind == C33_TRACE_TRGGER_BREAK)
        {
            strcat(cErrMsg, C33_ERR_MSG_ICD_TRG_ADDR);
        }
        c33_error (cErrMsg);
    }
    return iResult;
}

/****************************************************************************/
/* Format        : int c33_BreakSetROMCheck(unsigned long ulAddr)			*/
/* Arguments                                                                */
/*  unsigned long ulAddr : Set Soft Break Address                           */
/* Return value                                                             */
/*  C33_NO_ERR   : not ROM area                                             */
/*  C33_ERR      : ROM area                                                 */
/* Explanation   : check if address is ROM area.                            */
/****************************************************************************/
int
c33_BreakSetROMCheck (unsigned long ulAddr)
{
    int iResult = C33_NO_ERR;

	struct c33_map_info *pstMapInfo;

	/* SIM mode is No error : add T.Tazaki 2005/03/22 >>> */
    if (c33TargetInfo.iSim_open != 0)   /* SIM mode ? */
    {
	    return iResult;		/* NO ERROR */
	}
	/* SIM mode is No error : add T.Tazaki 2005/03/22 <<< */
	
	/* MAPROMGAɃu[N|Cgݒ肩H */
    if( c33TargetInfo.pMapInfo != NULL ){

        pstMapInfo = (struct c33_map_info *)c33TargetInfo.pMapInfo;
        
        while( pstMapInfo )
        {
			if( pstMapInfo->iMapAttr == C33_ROM )
			{
	            /* check if address range */
	            if(( ulAddr >= pstMapInfo->ulStartAddr ) && ( ulAddr <= pstMapInfo->ulEndAddr ))
	            {
   					iResult = C33_ERR;
   					break;
	            }
            }
            pstMapInfo = (struct c33_map_info *)pstMapInfo->pMapNext;
        }
    }
    return iResult;
}

/****************************************************************************/
/* Format        : static void c33_DspErrBrkStatus(void)                    */
/* Arguments     : void                                                     */
/* Return value  : void                                                     */
/* Explanation   : Display error message when software pc break error.      */
/****************************************************************************/
static void
c33_DspErrBrkStatus(void)
{
    char cErrMsg[1024];
    char cTemp[256];
    int iBreakNum;
    int i;
    unsigned char ucStatus,ucStatus1,ucStatus2;

    cErrMsg[0] = 0;
    for(i = 0 ; i < 9 ; i++)
    {
        ucStatus  = c33TargetInfo.ucIcdErrorInfo[i];
        ucStatus1 = ucStatus & 0x0f; /* odd break number */
        ucStatus2 = ucStatus >> 4;   /* even break number */

        cTemp[0] = 0;
        iBreakNum = i * 2 + 0;
        switch (ucStatus1)
        {
            case C33_BIT0: /* no area */
            case C33_BIT1: /* specify rom area */
                sprintf(cTemp, C33_ERR_MSG_BRK_ROM, (int)c33TargetInfo.ulSoftPcBreakTemp[iBreakNum]);
                break;
            case C33_BIT0 | C33_BIT1: /* ext or delayed instruction */
                sprintf(cTemp, C33_ERR_MSG_BRK_EXT, (int)c33TargetInfo.ulSoftPcBreakTemp[iBreakNum]);
                break;
            case C33_BIT2: /* break instruction */
                sprintf(cTemp, C33_ERR_MSG_BRK_BRK_INST, (int)c33TargetInfo.ulSoftPcBreakTemp[iBreakNum]);
                break;
            case C33_BIT3: /* over write by user when recovered from break instruction */
                sprintf(cTemp, C33_ERR_MSG_BRK_OVER_WT, (int)c33TargetInfo.ulSoftPcBreakTemp[iBreakNum]);
                break;
            case 0:
            default:
                // NULL
                break;
        }

        if (cTemp[0] != 0)
        {
            strcat(cErrMsg, cTemp);
            strcat(cErrMsg, "\n");
        }

        cTemp[0] = 0;
        iBreakNum = i * 2 + 1;
        switch (ucStatus2)
        {
            case C33_BIT0:
            case C33_BIT1:
                sprintf(cTemp, C33_ERR_MSG_BRK_ROM, (int)c33TargetInfo.ulSoftPcBreakTemp[iBreakNum]);
                break;
            case C33_BIT0 | C33_BIT1:
                sprintf(cTemp, C33_ERR_MSG_BRK_EXT, (int)c33TargetInfo.ulSoftPcBreakTemp[iBreakNum]);
                break;
            case C33_BIT2:
                sprintf(cTemp, C33_ERR_MSG_BRK_BRK_INST, (int)c33TargetInfo.ulSoftPcBreakTemp[iBreakNum]);
                break;
            case C33_BIT3:
                sprintf(cTemp, C33_ERR_MSG_BRK_OVER_WT, (int)c33TargetInfo.ulSoftPcBreakTemp[iBreakNum]);
                break;
            case 0:
            default:
                // NULL
                break;
        }

        if (cTemp[0] != 0)
        {
            strcat(cErrMsg, cTemp);
            strcat(cErrMsg, "\n");
        }

    }

    if (cErrMsg[0] != 0)
    {
        strcat(cErrMsg, C33_ERR_MSG_ICD_SOFT_PC);
        c33_error (cErrMsg);
    }
}

/****************************************************************************/
/* Format        : void c33_LogConvert (char *pSrc )                        */
/* Arguments     : void                                                     */
/* Output        ; char szLogbuf[3100]                                      */
/* Explanation   : Only LF --> CRLF, CRLF --> CRLF.                         */
/****************************************************************************/
void
c33_LogConvert (char *pSrc )
{

    
#ifdef __CYGWIN__

    char *pSrcTmp;
    int  icnt = 0;

    pSrcTmp = pSrc;
    
//    while( pSrcTmp != 0 ) /* cui bug fix  2004/01/29 T.Tazaki */
    while( *pSrcTmp != 0 )	/* cui bug fix  2004/01/29 T.Tazaki */
    {
        if( *pSrcTmp == 0x0d )  /* CR ? */
        {
            if( iLogCR == 1 )
            {
                ++pSrcTmp; /* Skip  CR + CR */
            }
            else
            {
                iLogCR = 1;
                szLogbuf[icnt] = *pSrcTmp;
                ++icnt;
                ++pSrcTmp;
            }
        }
        else if( *pSrcTmp == 0x0a ) /* LF ? */
        {
            if( iLogCR == 0 )
            {
                szLogbuf[icnt] = 0x0d;      /* add CR */
                ++icnt;
            }
            szLogbuf[icnt] = *pSrcTmp;
            ++icnt;
            ++pSrcTmp;
            iLogCR = 0;
        }
        else
        {
            iLogCR = 0;
            szLogbuf[icnt] = *pSrcTmp;
            ++icnt;
            ++pSrcTmp;
        }
        if( icnt >= 3000 )	/* Modify 500 -> 3000 T.Tazaki 2004/01/21 */
        {
            break;
        }
    }
    
    szLogbuf[icnt] = 0;

#else

    strcpy( szLogbuf, pSrc );
    
#endif
}

#ifdef __CYGWIN__

#include <windows.h>

#endif

/* add T.Tazaki 2005/05/06 >>> */
/****************************************************************************/
/* Format        : static void c33_ClipCopy(int iParamNum, char *cSymbol )  */
/* Arguments                                                                */
/*  int iParamNum             : Parameter number                            */
/*  char cSymbol              : symbol name                                 */
/* Return value  : void                                                     */
/* Explanation   : copy symbol to clipboard.                                */
/****************************************************************************/
static void
c33_ClipCopy(int iParamNum, char *cSymbol )
{
#ifdef __CYGWIN__
	HGLOBAL hglb;
	LPVOID lptstr;
#endif
	 
    /* Check number of parameter */
    if ( iParamNum != 2 )
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }

#ifdef __CYGWIN__

    hglb = GlobalAlloc(GHND, strlen(cSymbol) + 1);
    if( hglb == 0 )
    {
		return;
	}

    lptstr = GlobalLock(hglb);
    if( lptstr == 0 )
    {
        GlobalFree(hglb);
		return;
	}

    strcpy(lptstr, cSymbol);

    if( GlobalUnlock(hglb) != 0)
    {
        GlobalFree(hglb);
		return;
	}

    if( OpenClipboard(0) == 0 )
    {
        GlobalFree(hglb);
		return;
	}

    if( EmptyClipboard() == 0 )
    {
        CloseClipboard();
        GlobalFree(hglb);
		return;
	}

    if( SetClipboardData(CF_TEXT, hglb) == 0 )
    {
        CloseClipboard();
        GlobalFree(hglb);
		return;
	}

    CloseClipboard();
#endif
}
/* add T.Tazaki 2005/05/06 <<< */

/* default gdbtk.ini file image */   /* add 2003/02/27 T.Tazaki : gdbtk.ini output */
const char DimGdbtkIni[96][120] = 
{
    "# GDBtk Init file",
    "# GDBtkInitVersion: 1",
    "[global]",
    "font/bold=-family windows-caption",
//    "font/default=-family windows-message",
	"font/default=-family {MS UI Gothic} -size 9 -weight normal -slant roman -underline 0 -overstrike 0", /* T.Tazaki 2004/06/30 */
    "font/menu=-family windows-menu",
//    "font/status=-family windows-status",
	"font/status=-family {MS UI Gothic} -size 9 -weight normal -slant roman -underline 0 -overstrike 0", /* T.Tazaki 2004/06/30 */
//    "font/fixed=-family fixedsys",
	"font/fixed=-family FixedSys -size 14 -weight normal -slant roman -underline 0 -overstrike 0",	/* T.Tazaki 2004/06/30 */
    "font italic=-size 9 -weight normal -overstrike 0 -underline 0 -slant italic -family {MS UI Gothic}",	
    "font_cache=",
    "main_names=MAIN___ MAIN__ main cyg_user_start cyg_start",
    "use_icons=1",
    "B1_behavior=1",
//    "recent-projects=c:/gnu33/sample/tst/sample.elf",		/* T.Tazaki 2004/06/30 */
    "recent-projects=",										/* T.Tazaki 2004/06/30 */
    "ImageDir=images2",
    "mode=0",
    "control_target=1",
    "[load]",
    "main=1",
    "verbose=0",
    "baud=115200",
    "exit=1",
    "port=com1",
    "check=0",
    "bp_at_func=0",
    "[console]",
    "wrap=0",
    "prompt=(gdb) ",
    "prompt_fg=DarkGreen",
    "log_fg=green",
    "font=src-font",
    "target_fg=blue",
    "error_fg=red",
    "deleteLeft=1",
    "[src]",
    "tab_size=4",
    "run_attach=0",
    "trace_fg=blue",    /* magenta --> blue T.Tazaki 2003/10/17 trace_fg = H/W Break color */
    "thread_fg=pink",
    "font=src-font",
    "top_control=1",
    "bp_fg=red",
    "run_run=1",
    "run_cont=0",
    "source2_fg=navy",
    "break_fg=black",
    "handlebg=red",
    "disabled_fg=black",
    "run_load=0",
    "temp_bp_fg=orange",
    "PC_TAG=green",
    "variableBalloons=1",
    "STACK_TAG=green",
    "BROWSE_TAG=#9595e2",
    "linenums=1",
    "[reg]",
    "highlight_fg=blue",
    "rows=16",
    "[stack]",
    "font=src-font",
    "[locals]",
    "[watch]",
    "[bp]",
    "show_threads=0",
    "[search]",
    "filter_mode=starts with",
    "[process]",
    "[geometry]",
    ".memwin0.memwin=589x233+424+460", 
    
/*    ".regwin0.regwin=293x400+721+1",	*/	/* change 375-->400(default height) T.Tazaki 2003/11/27 */
/*    ".regwin0.regwin=343x403+721+1",  */  /* change 2005/05/24 T.Tazaki */
    ".regwin0.regwin=343x512+721+1",
    ".srcwin0.srcwin=713x493+0+1",
    ".console0.console=711x172+1+540",
    "[help]",
    "browser=0",
    "[browser]",
    "top_height=0",
    "view_height=-1",
    "width=0",
    "view_is_open=0",
    "hide_h=0",
    "[kod]",
    "show_icon=0",
    "[window]",
    "active={ManagedWin::open MemWin} {ManagedWin::open Console} {ManagedWin::open SrcWin} {ManagedWin::open RegWin}",
    "[session]",
    "[mem]",
    "size=4",
    "ascii_char=.",
    "numbytes=0",
    "color=green",
    "format=x",
    "ascii=1",
    "bytes_per_row=16",
    "END"
};

/****************************************************************************/
/* Format        : void c33_CreateINI (void )                               */
/* Arguments     : void                                                     */
/* Explanation   : if INI file is not exist, create gdbtk.ini.              */
/****************************************************************************/
void
c33_CreateINI()
{
#ifdef __CYGWIN__

    HANDLE hHandle = INVALID_HANDLE_VALUE;
    int iSize;
    int i;
    char szPath[260];

    hHandle = GetModuleHandle(NULL);     
    if( hHandle != 0 )
    {
        GetModuleFileName(hHandle, szPath, 256 );

        iSize = strlen( szPath );
        for( i = (iSize - 1); i >= 0; --i )
        {
            if( szPath[i] == '\\' )
            {
                break;
            }
        }
        szPath[i+1] = (char)NULL;
                                        /*                012345678901234567  */
        strcat( szPath,"gdbtk.ini" );   /* sample szPath="c:\gnu33\gdbtk.ini" */

        /* check if gdbtk.ini is exist */
        if( ( fpINI = fopen( szPath,"r" ) ) == NULL )
        {
            /* Create Defualt gdbtk.ini */
            if( ( fpINI = fopen( szPath,"w" ) ) != NULL )
            {
                i = 0;
                
                do
                {
                    if( strcmp( DimGdbtkIni[ i ],"END") == 0 )
                    {
                        break;
                    }
                    fputs( DimGdbtkIni[ i ], fpINI );
#ifdef __CYGWIN__ /* change Line feed code LF --> CRLF  T.Tazaki 2006/07/20 >>> */
					fputc( 0x0d , fpINI );
					fputc( 0x0a , fpINI );
#else
                    fputs( "\n", fpINI );
#endif
				  /* change Line feed code LF --> CRLF  T.Tazaki 2006/07/20 <<< */
                    ++i;
                }while( 1 );

                fclose( fpINI );
            }
        }
    }
    
#endif
}
/****************************************************************************/
/* Format        : void c33_gdbtkini_update (void )                         */
/* Arguments     : void                                                     */
/* Explanation   : gdbtk.ini geometry section update.                       */
/****************************************************************************/
void c33_gdbtkini_update()
{
#ifdef __CYGWIN__

    extern Tcl_Interp *gdbtk_interp;

    HANDLE hHandle = INVALID_HANDLE_VALUE;
    int iSize;
    int i;
    int iCnt;
    BOOL bBool;
    char szbuf[100];
    char szbuf_normal[100];
    char szPath[260];
    char *pszbuf;
    char *pszwindow;
    char szwindow[300];
    int iWinFlag[20];
	char sz1[10],sz2[10],sz3[10],sz4[10];	/* add T.Tazaki 2003/11/27 */
	int  i1,i2,i3,i4,iHeight;				/* add T.Tazaki 2003/11/27 */

    pszbuf = szbuf;

    pszwindow = szwindow;

    hHandle = GetModuleHandle(NULL);     /* 2003/02/26 T.Tazaki */
    if( hHandle != 0 )
    {
        GetModuleFileName(hHandle, szPath, 256 );

        iSize = strlen( szPath );
        for( i = (iSize - 1); i >= 0; --i )
        {
            if( szPath[i] == '\\' )
            {
                break;
            }
        }
        szPath[i+1] = (char)NULL;
                                        /*                012345678901234567  */
        strcat( szPath,"gdbtk.ini" );   /* sample szPath="c:\gnu33\gdbtk.ini" */

        /* execute proc pref_save2{} : prefs_tcl */
        Tcl_Eval(gdbtk_interp, "pref_save2");

        /*========================================================*/
        /* Get Window [global]section by pref_save2   : prefs.tcl */
        /*========================================================*/
        
//#if 0	// add T.Tazaki 2004/03/09 >>>
        pszbuf = Tcl_GetVar(gdbtk_interp, "g_global_font_bold", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "global","font/bold",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_global_font_default", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "global","font/default",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_global_font_menu", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "global","font/menu",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_global_font_status", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "global","font/status",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_global_font_fixed", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "global","font/fixed",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_global_font_italic", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "global","font/italic",pszbuf,szPath );
//#endif // add T.Tazaki 2004/03/09 <<<
        /*========================================================*/
        /* Get Window [src]section by pref_save2      : prefs.tcl */
        /*========================================================*/
        pszbuf = Tcl_GetVar(gdbtk_interp, "g_src_tab_size", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "src","tab_size",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_src_run_attach", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "src","run_attach",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_src_trace_fg", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "src","trace_fg",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_src_thread_fg", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "src","thread_fg",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_src_font", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "src","font",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_src_top_control", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "src","top_control",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_src_bp_fg", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "src","bp_fg",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_src_run_run", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "src","run_run",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_src_run_cont", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "src","run_cont",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_src_source2_fg", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "src","source2_fg",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_src_break_fg", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "src","break_fg",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_src_handlebg", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "src","handlebg",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_src_disabled_fg", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "src","disabled_fg",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_src_run_load", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "src","run_load",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_src_temp_bp_fg", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "src","temp_bp_fg",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_src_pc_tag", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "src","PC_TAG",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_src_variableBalloons", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "src","variableBalloons",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_src_stack_tag", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "src","STACK_TAG",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_src_browse_tag", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "src","BROWSE_TAG",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_src_linenums", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "src","linenums",pszbuf,szPath );

		/* add T.Tazaiki 2006/07/24 >>> */
		/*========================================================*/
        /* Get Window [mem]section by pref_save2      : prefs.tcl */
        /*========================================================*/
        pszbuf = Tcl_GetVar(gdbtk_interp, "g_mem_size", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "mem","size",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_mem_ascii_char", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "mem","ascii_char",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_mem_numbytes", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "mem","numbytes",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_mem_color", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "mem","color",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_mem_format", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "mem","format",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_mem_ascii", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "mem","ascii",pszbuf,szPath );

        pszbuf = Tcl_GetVar(gdbtk_interp, "g_mem_bytes_per_row", TCL_GLOBAL_ONLY);
        WritePrivateProfileString( "mem","bytes_per_row",pszbuf,szPath );
		/* add T.Tazaiki 2006/07/24 <<< */


        /*========================================================*/
        /* Get Window [geometry]section by pref_save2 : prefs.tcl */
        /*========================================================*/

        /* check all open window */
        pszwindow = Tcl_GetVar(gdbtk_interp, "g_window", TCL_GLOBAL_ONLY);

		/* add Console window     T.Tazaki 2004/02/02 >>> */
        c33_CheckWindowOpen( pszwindow, iWinFlag );

        /* Console window */
        if( iWinFlag[1] == 0 )
        {
			strcat( pszwindow," {ManagedWin::open Console}" );
	    }
		/* add T.Tazaki 2004/02/02 <<< */

		/* add T.Tazaki 2004/06/09 >>> */
        /* Register window */
        if( iWinFlag[0] == 0 )
        {
			strcat( pszwindow," {ManagedWin::open RegWin}" );
	    }
		/* add T.Tazaki 2004/06/09 <<< */

        WritePrivateProfileString( "window","active",pszwindow,szPath );

        c33_CheckWindowOpen( pszwindow, iWinFlag );
        
        /* Registers window */
        if( iWinFlag[0] == 1 )
        {
            pszbuf = Tcl_GetVar(gdbtk_interp, "g_regwin", TCL_GLOBAL_ONLY);
            c33_GetNormalString( pszbuf, szbuf_normal );
            
			/* add T.Tazaki 2003/11/27  Register window height size update >>> */
//			if ( c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV || c33TargetInfo.ucAdvanced == C33_PRODUCT_PE ) /* del T.Tazaki 2004/06/11 */
//			{
#if 0		/* del T.Tazaki 2007/07/19 >>> */
            if(g_iSim_open == 0 || c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV )	/* check No SIM mode ? add T.Tazaki 2004/06/16 */
            {
#endif
	            /* size,locate format "293x382+721+1" */
	            
				i1 = 0;
	            for( i = 0; ; ++i )
	            {
					if( szbuf_normal[i] == 'x' )
					{
						++i;
						sz1[i1] = 0;
						break;
					}
					sz1[i1] = szbuf_normal[i];
					++i1;
				}
				i2 = 0;
	            for( ; ; ++i )
	            {
					if( szbuf_normal[i] == '+' )
					{
						++i;
						sz2[i2] = 0;
						break;
					}
					sz2[i2] = szbuf_normal[i];
					++i2;
				}
				i3 = 0;
	            for( ; ; ++i )
	            {
					if( szbuf_normal[i] == '+' )
					{
						++i;
						sz3[i3] = 0;
						break;
					}
					sz3[i3] = szbuf_normal[i];
					++i3;
				}
				i4 = 0;
	            for( ; ; ++i )
	            {
					if( szbuf_normal[i] == 0 )
					{
						sz4[i4] = 0;
						break;
					}
					sz4[i4] = szbuf_normal[i];
					++i4;
				}
#if 0			/* del T.Tazaki 2007/07/19 >>> */
//				iHeight = atoi(sz2) + 19;
//				sprintf( szbuf_normal,"%sx%d+%s+%s",sz1,iHeight,sz3,sz4);
//			}
#endif
			/* add T.Tazaki 2003/11/27  Register window height size update <<< */

			/* add T.Tazaki 2007/07/19 >>> */
			switch( c33TargetInfo.ucAdvanced )
			{
				case C33_PRODUCT_PE:  iSize = 296;		iHeight = 440;	 break;
				case C33_PRODUCT_ADV: iSize = 349;		iHeight = 529;	 break;
				default : 			  iSize = 296;		iHeight = 423;	 break;
            }
			sprintf( szbuf_normal,"%dx%d+%s+%s",iSize,iHeight,sz3,sz4);
			/* add T.Tazaki 2007/07/19 <<< */

            WritePrivateProfileString( "geometry",".regwin0.regwin",szbuf_normal,szPath );
        }
        /* Console window */
        if( iWinFlag[1] == 1 )
        {
            pszbuf = Tcl_GetVar(gdbtk_interp, "g_console", TCL_GLOBAL_ONLY);
            c33_GetNormalString( pszbuf, szbuf_normal );
            WritePrivateProfileString( "geometry",".console0.console",szbuf_normal,szPath );
        }
	    
        /* Source window */
        if( iWinFlag[2] == 1 )
        {
            pszbuf = Tcl_GetVar(gdbtk_interp, "g_srcwin", TCL_GLOBAL_ONLY);
            c33_GetNormalString( pszbuf, szbuf_normal );
            WritePrivateProfileString( "geometry",".srcwin0.srcwin",szbuf_normal,szPath );
        }
        /* Memory window */
        if( iWinFlag[3] == 1 )
        {
            pszbuf = Tcl_GetVar(gdbtk_interp, "g_memwin", TCL_GLOBAL_ONLY);
            c33_GetNormalString( pszbuf, szbuf_normal );
            WritePrivateProfileString( "geometry",".memwin0.memwin",szbuf_normal,szPath );
        }
        /* Local window */
        if( iWinFlag[4] == 1 )
        {
            pszbuf = Tcl_GetVar(gdbtk_interp, "g_localswin", TCL_GLOBAL_ONLY);
            c33_GetNormalString( pszbuf, szbuf_normal );
            WritePrivateProfileString( "geometry",".localswin0.localswin",szbuf_normal,szPath );
        }
        /* Watch window */
        if( iWinFlag[5] == 1 )
        {
            pszbuf = Tcl_GetVar(gdbtk_interp, "g_watchwin", TCL_GLOBAL_ONLY);
            c33_GetNormalString( pszbuf, szbuf_normal );
            WritePrivateProfileString( "geometry",".watchwin0.watchwin",szbuf_normal,szPath );
        }
        /* Breakpoints window */
        if( iWinFlag[6] == 1 )
        {
            pszbuf = Tcl_GetVar(gdbtk_interp, "g_bpwin", TCL_GLOBAL_ONLY);
            c33_GetNormalString( pszbuf, szbuf_normal );
            WritePrivateProfileString( "geometry",".bpwin0.bpwin",szbuf_normal,szPath );
        }
        /* Simulated I/O window */ /* add 2003/03/11 T.Tazaki */
        if( iWinFlag[7] == 1 )
        {
            pszbuf = Tcl_GetVar(gdbtk_interp, "g_simwin", TCL_GLOBAL_ONLY);
            c33_GetNormalString( pszbuf, szbuf_normal );
            WritePrivateProfileString( "geometry",".simio0.simio",szbuf_normal,szPath );
        }
        /* Trace I/O window */ /* add 2003/07/10 T.Tazaki */
        if( iWinFlag[8] == 1 )
        {
            pszbuf = Tcl_GetVar(gdbtk_interp, "g_tracewin", TCL_GLOBAL_ONLY);
            c33_GetNormalString( pszbuf, szbuf_normal );
            WritePrivateProfileString( "geometry",".tracewin0.tracewin",szbuf_normal,szPath );
        }
    }
    
#endif

}

/****************************************************************************/
/* Format        : static void c33_GetNormalString( char *pszbuf, char *psz_normal)*/
/* Arguments     : char *pszbuf         abnormal window locate and size     */
/*               : char *psz_normal     normal window locate and size       */
/* Explanation   : Abnormal character '-' delete.                           */
/****************************************************************************/
static
void c33_GetNormalString( char *pszbuf, char *pszbuf_normal )
{
    int iSize;
    int i;
    int iCnt;

    iSize = strlen( pszbuf );
    iCnt = 0;
    for( i = 0; i < iSize; ++i )
    {
        if( pszbuf[ i ] != '-' )
        {
            pszbuf_normal[iCnt] = pszbuf[ i ];
            ++iCnt;
        }
    }
    pszbuf_normal[iCnt] = (char)NULL;
}

/****************************************************************************/
/* Format        : void ( char *pszwindow, int *iWinFlag)                   */
/* Arguments     : char *pszwindow   "active=" key string                   */
/*               : int iWinFlag[]     result 1=open window                  */
/* Explanation   : Set active window flag from "active=" key string         */
/* pszwindow format = "{ManagedWin::open MemWin} {ManagedWin::open Console} */
/*          {ManagedWin::open BpWin} {ManagedWin::open WatchWin} {ManagedWin::open SrcWin} */
/*          {ManagedWin::open RegWin} {ManagedWin::open LocalsWin}          */
/*          {ManagedWin::open SimIO}  {ManagedWin::open TraceWin}           */
/****************************************************************************/
void c33_CheckWindowOpen( char *pszwindow, int *iWinFlag)
{
    char *pszwindow2;
    char szwindow2[300];
    int i;
    int iSize;

    pszwindow2 = szwindow2;

    strcpy( (char *)szwindow2, pszwindow );

    for( i = 0; i < 20; ++i ) {
        iWinFlag[i] = 0;
    }
    for( i = 0; i < 300; ++i ) {
        szwindow2[i] = 0;
    }

    iSize = strlen( pszwindow );

    pszwindow2 = pszwindow;
    for( i = 0; i < iSize; ++i ) {
        if( *pszwindow2 == 'R' && *(pszwindow2+1) == 'e' && *(pszwindow2+2) == 'g' ) {
            iWinFlag[0] = 1;
            break;
        }
        ++pszwindow2;
    }

    pszwindow2 = pszwindow;
    for( i = 0; i < iSize; ++i ) {
        if( *pszwindow2 == 'C' && *(pszwindow2+1) == 'o' && *(pszwindow2+2) == 'n' ) {
            iWinFlag[1] = 1;
            break;
        }
        ++pszwindow2;
    }

    pszwindow2 = pszwindow;
    for( i = 0; i < iSize; ++i ) {
        if( *pszwindow2 == 'S' && *(pszwindow2+1) == 'r' && *(pszwindow2+2) == 'c' ) {
            iWinFlag[2] = 1;
            break;
        }
        ++pszwindow2;
    }

    pszwindow2 = pszwindow;
    for( i = 0; i < iSize; ++i ) {
        if( *pszwindow2 == 'M' && *(pszwindow2+1) == 'e' && *(pszwindow2+2) == 'm' ) {
            iWinFlag[3] = 1;
            break;
        }
        ++pszwindow2;
    }

    pszwindow2 = pszwindow;
    for( i = 0; i < iSize; ++i ) {
        if( *pszwindow2 == 'L' && *(pszwindow2+1) == 'o' && *(pszwindow2+2) == 'c' ) {
            iWinFlag[4] = 1;
            break;
        }
        ++pszwindow2;
    }

    pszwindow2 = pszwindow;
    for( i = 0; i < iSize; ++i ) {
        if( *pszwindow2 == 'W' && *(pszwindow2+1) == 'a' && *(pszwindow2+2) == 't' ) {
            iWinFlag[5] = 1;
            break;
        }
        ++pszwindow2;
    }

    pszwindow2 = pszwindow;
    for( i = 0; i < iSize; ++i ) {
        if( *pszwindow2 == 'B' && *(pszwindow2+1) == 'p' && *(pszwindow2+2) == 'W' ) {
            iWinFlag[6] = 1;
            break;
        }
        ++pszwindow2;
    }

    pszwindow2 = pszwindow;
    for( i = 0; i < iSize; ++i ) {
        if( *pszwindow2 == 'S' && *(pszwindow2+1) == 'i' && *(pszwindow2+2) == 'm' ) {
            iWinFlag[7] = 1;
            break;
        }
        ++pszwindow2;
    }

    pszwindow2 = pszwindow;     /* add trace window T.Tazaki 2003/07/10 */
    for( i = 0; i < iSize; ++i ) {
        if( *pszwindow2 == 'T' && *(pszwindow2+1) == 'r' && *(pszwindow2+2) == 'a' ) {
            iWinFlag[8] = 1;
            break;
        }
        ++pszwindow2;
    }

}

/****************************************************************************/
/* Format        : void c33_Console_Output(char *string)                    */
/* Arguments     : char *string                                             */
/* Return value  : void                                                     */
/* Explanation   : Console output called by utils.c                         */
/****************************************************************************/
void
c33_Console_Output(char *string)
{
    extern Tcl_Interp *gdbtk_interp;
    
    Tcl_SetVar( gdbtk_interp,"c33_console_msg",string,TCL_GLOBAL_ONLY );
    Tcl_Eval( gdbtk_interp, "c33_console" ); /* gdbtcl/srcbar.itcl */

}

/****************************************************************************/
/* Format        : void c33_InitCancel(void)                                */
/* Arguments     : void                                                     */
/* Return value  : void                                                     */
/* Explanation   : TCL global  c33_esc_flag <-- "OFF"   : console.itb       */
/*               : TCL global  c33_mem_update <-- "OFF" : memwin.itb        */
/****************************************************************************/
void
c33_InitCancel()
{
    extern Tcl_Interp *gdbtk_interp;

    if( g_use_windows == 1 )  /* GUI Mode ? */
    {
        g_iGdbtk_Update_cnt = 0;    /* global update counter initialize */

        c33TargetInfo.iKeyBreak = 0;
        
        /* Console Window ESC key flag OFF */
        Tcl_SetVar( gdbtk_interp,"c33_esc_flag","OFF",TCL_GLOBAL_ONLY ); 

        if(c33TargetInfo.iIcd_open != C33_NOT_OPEN)	/* add T.Tazaki 2004/05/26 : for [SIM mode] not display data in Memory Window */
        {
	        /* Memory Window update flag OFF */
	        Tcl_SetVar( gdbtk_interp,"c33_mem_update","OFF",TCL_GLOBAL_ONLY ); 
	    }
    }
}

/****************************************************************************/
/* Format        : int c33_CheckCancel(int iCount)                          */
/* Arguments     : int iCount update timming count                          */
/* Return value  : 1 : Escape key ON , 0 : OFF                              */
/* Explanation   : Check if ESC key press                                   */
/*               : when esc key press then,                                 */
/*               : TCL global  c33_esc_flag <-- "OFF"   : console.itb       */
/*               : TCL global  c33_mem_update <-- "ON"  : memwin.itb        */
/****************************************************************************/
int
c33_CheckCancel( int iCount )
{
    int      iRet = 0;
    char     szbuf[100];
    char     *pszbuf;
    extern Tcl_Interp *gdbtk_interp;

    pszbuf = szbuf;

    if( g_use_windows == 1 )  /* GUI Mode ? */
    {
        if( g_iGdbtk_Update_cnt == 0 )
        {
            if(c33TargetInfo.iIcd_open != C33_NOT_OPEN)
            {
                c33_Gdbtk_Update (C33_UPDATE_REG);
            }
            else if (c33TargetInfo.iSim_open != 0)
            {
                sim_Gdbtk_Update ();
            }
            g_iGdbtk_Update_cnt = iCount;
        }
        else
        {
            --g_iGdbtk_Update_cnt;
        }

        pszbuf = Tcl_GetVar(gdbtk_interp, "c33_esc_flag", TCL_GLOBAL_ONLY);
        if( !strcmp( pszbuf,"ON" ) )
        {
            c33TargetInfo.iKeyBreak = 1;

            if( g_command_loop == 1 )   /* now on source command loop ? */
            {
                g_c33_iSourceBreak = 1;
            }
            /* Console Window ESC key flag OFF */
            Tcl_SetVar( gdbtk_interp,"c33_esc_flag","OFF",TCL_GLOBAL_ONLY ); 

            /* Memory Window update flag ON */
            Tcl_SetVar( gdbtk_interp,"c33_mem_update","ON",TCL_GLOBAL_ONLY ); 
            iRet = 1;
        }
    }
    
    return iRet;
}

/****************************************************************************/
/* Format        : void c33_EndCancel(void)                                 */
/* Arguments     : void                                                     */
/* Return value  : void                                                     */
/* Explanation   : TCL global  c33_esc_flag <-- "OFF"   : console.itb       */
/*               : TCL global  c33_mem_update <-- "ON"  : memwin.itb        */
/****************************************************************************/
void
c33_EndCancel()
{
    extern Tcl_Interp *gdbtk_interp;

    if( g_use_windows == 1 )  /* GUI Mode ? */
    {
        g_iGdbtk_Update_cnt = 0;    /* global update counter initialize */
    
        c33TargetInfo.iKeyBreak = 0;
        
        /* Console Window ESC key flag OFF */
        Tcl_SetVar( gdbtk_interp,"c33_esc_flag","OFF",TCL_GLOBAL_ONLY ); 

        /* Memory Window update flag ON */
        Tcl_SetVar( gdbtk_interp,"c33_mem_update","ON",TCL_GLOBAL_ONLY ); 
    }
}
/****************************************************************************/
/* Format        : void c33_memory_window_update(int iFlag)                 */
/* Arguments     :  iFlag = 0 OFF, = 1 ON                                   */
/* Return value  : void                                                     */
/****************************************************************************/
void
c33_memory_window_update( int iFlag )
{
    extern Tcl_Interp *gdbtk_interp;

	if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
	{
		if( iFlag == 0 )
		{
	        /* Memory Window update flag OFF */
	        Tcl_SetVar( gdbtk_interp,"c33_mem_update","OFF",TCL_GLOBAL_ONLY ); 
	    }
	    else
	    {
	        /* Memory Window update flag ON */
	        Tcl_SetVar( gdbtk_interp,"c33_mem_update","ON",TCL_GLOBAL_ONLY ); 
	    }
	}
}
/****************************************************************************/
/* Format        : void c33_Get_bp_status (void)                            */
/* Arguments     : void                                                     */
/* Return value  : void                                                     */
/* Explanation   : get soft break status                                    */
/*                 Break Number : global c33_bp_num(bpwin.itb)              */
/*                 Status       : global c33_bp_enable(bpwin.itb) "on"/"off"*/
/*               : get hard break status                                    */
/*                 Break Number : global c33_bp_num(bpwin.itb)              */
/*                 Status       : global c33_bp_hw_enable(bpwin.itb) "on"/"off"*/
/****************************************************************************/
void
c33_Get_bp_status()
{
    int iNum;
    int iStatus;
    extern Tcl_Interp *gdbtk_interp;
    char     szbuf[1000];
    char     *pszbuf;

    if( g_use_windows == 1 )
    {
        pszbuf = szbuf;

        pszbuf = Tcl_GetVar(gdbtk_interp, "c33_bp_num", TCL_GLOBAL_ONLY);
        iNum = atoi( pszbuf );
        
        iStatus = c33_GetBreakPointStatus( iNum );  /* breakpoint.c */
        
        if( iStatus == 1 )
        {
            Tcl_SetVar( gdbtk_interp,"c33_bp_enable","on",TCL_GLOBAL_ONLY );
        }
        else
        {
            Tcl_SetVar( gdbtk_interp,"c33_bp_enable","off",TCL_GLOBAL_ONLY );
        }

        /* add Hard Break Enable infomation     T.Tazaki 2003/10/15 >>> */
        iStatus = c33_GetHardBreakPointStatus( iNum );      /* breakpoint.c */
        
        if( iStatus == 1 )
        {
            Tcl_SetVar( gdbtk_interp,"c33_bp_hw_enable","on",TCL_GLOBAL_ONLY );
        }
        else
        {
            Tcl_SetVar( gdbtk_interp,"c33_bp_hw_enable","off",TCL_GLOBAL_ONLY );
        }
        /* add Hard Break Enable infomation     T.Tazaki 2003/10/15 >>> */
    }
}
/****************************************************************************/
/* Format        : void c33_Check_bp_addr(char *pFileName, int iLine)       */
/* Arguments                                                                */
/*  char *pFileName : break set source filename                             */
/*  int  iLine      : break set source line number                          */
/* Return value  : void                                                     */
/* Explanation   : Check soft pc break set address(ext or delayed)          */
/*               : This is the internal command for TCL(srctextwin.itb)     */
/****************************************************************************/

char    g_szBuf[sizeof(struct symtab)];

void
c33_Check_bp_addr(char *pFileName, int iLine)
{
    unsigned long ulBreakAddr;
    struct symtab *symtab;
    
    if( g_use_windows == 0 )    /* CUI mode ? */
    {
        return;
    }
    

    symtab = (struct symtab *)&g_szBuf[0];

    symtab->filename = pFileName;

    /* Get address from finename and line number. */
    if( find_line_pc ((struct symtab *)symtab, iLine, &ulBreakAddr) == 0 )   /* symtab.c */
    {
        return;
    }

    /* Check break address*/
    c33_BreakAddrCheck(ulBreakAddr, C33_SOFT_PC_BREAK);

}

/****************************************************************************/
/* Format        : void c33_error (const char *string,...)                  */
/* Arguments     : string  error message                                    */
/* Return value  : void                                                     */
/* Explanation   : Do not use verror()                                      */
/****************************************************************************/
void
c33_error (const char *string,...)
{
	
	
    va_list args;

	/* add 2004/01/06 T.Tazaki >>> */
    extern Tcl_Interp *gdbtk_interp;
	char	szbuf[300];
	int		iToolBarCommand = 0;
    char 	*pszbuf;

    pszbuf = szbuf; 
	/* add 2004/01/06 T.Tazaki <<< */


    g_error_func = 1;

    va_start (args, string);

	/* add 2004/01/06 T.Tazaki >>> */
	if ( g_use_windows == 1 )
	{
		/* check if focus is not console window. */
        pszbuf = Tcl_GetVar(gdbtk_interp, "c33_toolbar_cmd", TCL_GLOBAL_ONLY); 
        iToolBarCommand = atoi( pszbuf );
	}

    if( g_use_windows == 1 && iToolBarCommand == 1 )	/* tool botton  ? */
	{
		sprintf( szbuf,string, args );
  	    va_end (args);
        Tcl_SetVar( gdbtk_interp,"c33_error_message",szbuf,TCL_GLOBAL_ONLY );	
        Tcl_Eval( gdbtk_interp, "c33_error_display" ); /* gdbtcl/srcbar.itcl */
		return_to_top_level (RETURN_ERROR);
	}
	else
	{
	/* add 2004/01/06 T.Tazaki <<< */
	    if( g_no_version == 1 )
	    {
		    printf_filtered(string, args);
		    printf_filtered( "\n");
		}
	    else
	    {
	        verror (string, args);
	    }
	}

    va_end (args);
}

/* add T.Tazaki 2007/06/25 >>> */

NORETURN void
c33_verror (const char *string, va_list args)
{
  char *linebuffer;
  struct cleanup *old_cleanups;

  xvasprintf (&linebuffer, string, args);
  
  old_cleanups = make_cleanup (xfree, linebuffer);

	if ( g_com_file_error == 0 ) /* 		add T.Tazaki 2007/06/19 
								 "Error in sourced command file"? */
	{
      c33_AddCRLF(linebuffer);				/* add T.Tazaki 2007/06/26 LF --> CRLF */
	  c33_LogConvert( linebuffer );
//	  printf_filtered( "%s\n", szLogbuf );	/* Change T.Tazaki 2007/06/26 LF --> CRLF */
	  printf_filtered( "%s", szLogbuf );	/* Change T.Tazaki 2007/06/26 LF --> CRLF */

	}
	g_com_file_error = 0;		/*      add T.Tazaki 2007/06/19  */
	
  do_cleanups (old_cleanups);
}
/* add T.Tazaki 2007/06/25 <<< */

/****************************************************************************/
/* Help Messages                                                            */
/****************************************************************************/
#define C33_COMMAND_NAME_SIZE       30
#define C33_COMMAND_HELP_MSG_SIZE   3000
#define C33_MAX_CORE_COMMAND_NAME   42      /* xxx      */
#define C33_MAX_COMMAND_NAME        48      /* c33 xxx  */
#define C33_MAX_SET_COMMAND_NAME     2      /* set xxx  */
#define C33_MAX_INFO_COMMAND_NAME    4      /* info xxx */


/* "c33 help" */
#define C33_HELP_ALL_HELPMSG   "\
group  0: memory ........... c33 fb,c33 fh,c33 fw,x /b,x /h,x /w,set {char},set {short},set {int}\n\
                             c33 mvb,c33 mvh,c33 mvw,c33 df,c33 rm,c33 readmd \n\
group  1: register ......... info reg,set $ \n\
group  2: execution ........ continue,until,step,stepi,next,nexti,finish,c33 callmd,c33 call \n\
group  3: CPU reset ........ c33 rstc,c33 rsth,c33 rstt \n\
group  4: interrupt ........ c33 int \n\
group  5: break ............ break,tbreak,hbreak,thbreak,watch,rwatch,awatch,delete,clear,enable\n\
                             disable,ignore,info breakpoints,c33 oab,c33 obb,c33 timebrk \n\
group  6: symbol ........... info locals,info var,print \n\
group  7: file ............. file,load,c33 lpt\n\
group  8: map .............. c33 rpf,c33 map \n\
group  9: flash memory ..... c33 fls,c33 fle \n\
group 10: trace ............ c33 tm,c33 td,c33 ts,c33 tf,c33 obt,c33 otd,c33 otf \n\
group 11: simulated I/O .... c33 stdin,c33 stdout \n\
group 12: flash writer ..... c33 fwe,c33 fwlp,c33 fwld,c33 fwdc,c33 fwd \n\
group 13: others ........... c33 dclk,c33 oscwait,c33 log,source,c33 clockmd,c33 clock,c33 firmupdate\n\
                             target,detach,c33 das,pwd,cd,c33 help,quit,c33 cachehit,c33 logiana\n\
 Please type \"c33 help 1\" to show group 1 or type \"c33 help c33 fb\" to get usage of command \"c33 fb\".\n"

/* group 0 */
#define C33_HELP_GROUP0_HELPMSG "\
group 0: memory\n\
 c33 fb             Fill byte\n\
 c33 fh             Fill half\n\
 c33 fw             Fill word\n\
 x                  Dump\n\
 set {}             Data input\n\
 c33 mvb            Move byte\n\
 c33 mvh            Move half\n\
 c33 mvw            Move word\n\
 c33 df             Save memory contents\n\
 c33 rm             Read memory\n\
 c33 readmd         Set read memory mode\n\
 Please type \"c33 help c33 fb\" to get usage of command \"c33 fb\".\
\n"

#define C33_HELP_GROUP1_HELPMSG "\
group 1: register\n\
 info reg           Display register\n\
 set $              Modify register\n\
 Please type \"c33 help info reg\" to get usage of command \"info reg\".\
\n"

#define C33_HELP_GROUP2_HELPMSG "\
group 2: execution\n\
 continue           Execute continously\n\
 until              Execute continously with temporary break\n\
 step               Single-step every line\n\
 stepi              Single-step every mnemonic\n\
 next               Single-step with skip every line\n\
 nexti              Single-step with skip every mnemonic\n\
 finish             Quit function\n\
 c33 callmd         Set user function call mode\n\
 c33 call           Call user function\n\
 Please type \"c33 help continue\" to get usage of command \"continue\".\
\n"

#define C33_HELP_GROUP3_HELPMSG "\
group 3: CPU reset\n\
 c33 rstc           Cold reset\n\
 c33 rsth           Hot reset\n\
 c33 rstt           Target reset\n\
 Please type \"c33 help c33 rstc\" to get usage of command \"c33 rstc\".\
\n"

#define C33_HELP_GROUP4_HELPMSG "\
group 4: interrupt\n\
 c33 int            Interrupt\n\
 Please type \"c33 help c33 int\" to get usage of command \"c33 int\".\
\n"

#define C33_HELP_GROUP5_HELPMSG "\
group 5: break\n\
 break              Set software PC break\n\
 tbreak             Set temporary software PC brea\n\
 hbreak             Set hardware PC break\n\
 thbreak            Set temporary hardware PC break\n\
 watch              Set data write break\n\
 rwatch             Set data read break\n\
 awatch             Set data read/write break\n\
 delete             Clear break by break number\n\
 clear              Clear break by break position\n\
 enable             Enable breakpoint\n\
 disable            Disable breakpoint\n\
 ignore             Disable breakpoint with ignore counts\n\
 info breakpoints   Display breakpoint list\n\
 c33 oab            Set area break\n\
 c33 obb            Set bus break\n\
 c33 timebrk        Set timer break\n\
 Please type \"c33 help break\" to get usage of command \"break\".\
\n"

#define C33_HELP_GROUP6_HELPMSG "\
group 6: symbol\n\
 info locals        Display local symbol\n\
 info var           Display global symbol\n\
 print              Alter symbol value\n\
 Please type \"c33 help c33 info locals\" to get usage of command \"info locals\".\
\n"

#define C33_HELP_GROUP7_HELPMSG "\
group 7: file\n\
 file               Load debugging information\n\
 load               Load program\n\
 c33 lpt            Set Fast Load program mode and parallel port\n\
 Please type \"c33 help file\" to get usage of command \"file\".\
\n"

#define C33_HELP_GROUP8_HELPMSG "\
group 8: map\n\
 c33 rpf            Set map information\n\
 c33 map            Display map information\n\
 Please type \"c33 help c33 rpf\" to get usage of command \"c33 rpf\".\
\n"

#define C33_HELP_GROUP9_HELPMSG "\
group 9: flash memory\n\
 c33 fls            Set flash memory\n\
 c33 fle            Erase flash memory\n\
 Please type \"c33 help c33 fls\" to get usage of command \"c33 fls\".\
\n"

#define C33_HELP_GROUP10_HELPMSG    "\
group 10: trace\n\
 c33 tm             Trace mode\n\
 c33 td             Display trace contents\n\
 c33 ts             Search trace contents\n\
 c33 tf             Save trace contents\n\
 c33 obt            Set bus trace mode\n\
 c33 otd            Display bus trace contents\n\
 c33 otf            Save bus trace contents\n\
 Please type \"c33 help c33 tm\" to get usage of command \"c33 tm\".\
\n"

#define C33_HELP_GROUP11_HELPMSG    "\
group 11: simulated I/O\n\
 c33 stdin          Data input simulation\n\
 c33 stdout         Data output simulation\n\
 Please type \"c33 help c33 stdin\" to get usage of command \"c33 stdin\".\
\n"

#define C33_HELP_GROUP12_HELPMSG    "\
group 12: flash writer\n\
 c33 fwe            Erase program/data\n\
 c33 fwlp           Load program\n\
 c33 fwld           Load data\n\
 c33 fwdc           Copy target memory\n\
 c33 fwd            Display flash writer information\n\
 Please type \"c33 help c33 fwe\" to get usage of command \"c33 fwe\".\
\n"

#define C33_HELP_GROUP13_HELPMSG    "\
group 13: other\n\
 c33 log            Logging\n\
 source             Execute command file\n\
 c33 clockmd        Set execution counter mode\n\
 c33 clock          Display execution counter\n\
 c33 firmupdate     ICDV3 Firmware update\n\
 target             Connect target\n\
 detach             Disconnect target\n\
 c33 das            Debug unit address set\n\
 pwd                Display current directory\n\
 cd                 Change current directory\n\
 c33 cachehit       Display the rate of cache hit\n\
 c33 logiana        Set Logic analyzer mode\n\
 c33 help           Help\n\
 quit               Quit debugger\n\
 Please type \"c33 help c33 log\" to get usage of command \"c33 log\".\
\n"

#define C33_HELP_SET_COMMAND_HELPMSG    "\
 set {}             Data input\n\
 set $              Modify register\n\
 Please type \"c33 help set {}\" to get usage of command \"set {}\".\
\n"

#define C33_HELP_INFO_COMMAND_HELPMSG   "\
 info reg           Display register\n\
 info breakpoints   Display breakpoint list\n\
 info locals        Display local symbol\n\
 info var           Display global symbol\n\
 Please type \"c33 help info reg\" to get usage of command \"info reg\".\
\n"

#define C33_CONTINUE_HELPMSG    "\
continue: Execute continuously                 [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: continue [IgnoreCount]\n\
       cont [IgnoreCount]\n\n\
       IgnoreCount:Specifies the number of breaks(decimal or hexadecimal)\n\
                   The program is run continuously until break conditions \n\
                   are met the specified number of times.\n\n\
example:\n\
(gdb)contiune\n\
(gdb)cont 5\n"
       
#define C33_UNTIL_HELPMSG   "\
until: Execute continuously with temporary break [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: (gdb)until [Breakpoint]\n\
       Breakpoint: Temporary breakpoint\n\
                   Can be specified by one of the following\n\
                   Function name\n\
                   Source file name:line number,or line number only\n\
                   Address(decimal,hexadecimal,or symbol)\n\
       Conditions:0-0xffffffff\n\n\
example:\n\
(gdb)until main\n\
(gdb)until main.c:10\n\
(gdb)until *0xc0001e\n"

#define C33_STEP_HELPMSG    "\
step: Single-step,every line                   [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: step [Count]\n\
       Count: Number of steps to execute (decimal or hexadecimal)\n\
              One step is assumed if ommitted.\n\
       Conditions: 1-0x7fffffff\n\n\
example:\n\
(gdb)step\n\
(gdb)step 10\n"


#define C33_STEPI_HELPMSG   "\
stepi: Single-step,every mnemonic              [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: stepi [Count]\n\
       Count: Number of steps to execute (decimal or hexadecimal)\n\
              One step is assumed if ommitted.\n\
       Conditions: 1-0x7fffffff\n\n\
example:\n\
(gdb)stepi\n\
(gdb)stepi 10\n"

#define C33_NEXT_HELPMSG    "\
next: Single-step with skip,every line         [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: next [Count]\n\
       Count: Number of steps to execute (decimal or hexadecimal)\n\
              One step is assumed if ommitted.\n\
       Conditions: 1-0x7fffffff\n\n\
example:\n\
(gdb)next\n\
(gdb)next 10\n"

#define C33_NEXTI_HELPMSG   "\
nexti: Single-step with skip,every mnemonic    [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: nexti [Count]\n\
       Count: Number of steps to execute (decimal or hexadecimal)\n\
              One step is assumed if ommitted.\n\
       Conditions: 1-0x7fffffff\n\n\
example:\n\
(gdb)nexti\n\
(gdb)nexti 10\n"

#define C33_FINISH_HELPMSG  "\
finish: Finish function                        [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: finish\n\
       The target program is executed from current PC address and halted after a return\n\n\
example:\n\
(gdb)finish\n"

#define C33_BREAK_HELPMSG   "\
break: Set software PC break                   [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: break [Breakpoint]\n\
       Breakpoint: Breakpoint\n\
                   A breakpoint can be specified with one of the following\n\
                   Function name\n\
                   Source file name:line number,or line number only\n\
                   Address(decimal,hexadecimal,or symbol)\n\
                   When omitted,a breakpoint is set at the address display on the PC.\n\
       Conditions:0-0xffffffff\n\n\
example:\n\
(gdb)break main\n\
(gdb)break main.c:10\n\
(gdb)break *0xc0001e\n"

#define C33_TBREAK_HELPMSG  "\
tbreak: Set temporary software PC break        [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: tbreak [Breakpoint]\n\
       Breakpoint: Breakpoint\n\
                   A breakpoint can be specified with one of the following\n\
                   Function name\n\
                   Source file name:line number,or line number only\n\
                   Address(decimal,hexadecimal,or symbol)\n\
                   When omitted,a breakpoint is set at the address display on the PC.\n\
       Conditions:0-0xffffffff\n\n\
example:\n\
(gdb)tbreak main\n\
(gdb)tbreak main.c:10\n\
(gdb)tbreak *0xc0001e\n"

#define C33_HBREAK_HELPMSG  "\
hbreak: Set hardware PC break                  [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: hbreak [Breakpoint]\n\
       Breakpoint: Breakpoint\n\
                   A breakpoint can be specified with one of the following\n\
                   Function name\n\
                   Source file name:line number,or line number only\n\
                   Address(decimal,hexadecimal,or symbol)\n\
                   When omitted,a breakpoint is set at the address display on the PC.\n\
       Conditions:0-0xffffffff\n\n\
example:\n\
(gdb)hbreak main\n\
(gdb)hbreak main.c:10\n\
(gdb)hbreak *0xc0001e\n"

#define C33_THBREAK_HELPMSG "\
thbreak: Set temporary hardware PC break       [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: thbreak [Breakpoint]\n\
       Breakpoint: Breakpoint\n\
                   A breakpoint can be specified with one of the following\n\
                   Function name\n\
                   Source file name:line number,or line number only\n\
                   Address(decimal,hexadecimal,or symbol)\n\
                   When omitted,a breakpoint is set at the address display on the PC.\n\
       Conditions:0-0xffffffff\n\n\
example:\n\
(gdb)thbreak main\n\
(gdb)thbreak main.c:10\n\
(gdb)thbreak *0xc0001e\n"

#define C33_WATCH_HELPMSG   "\
watch: Set data write break                    [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: watch Breakpoint\n\
       Breakpoint: Breakpoint\n\
                   A breakpoint can be specified with one of the following\n\
                   Symbol\n\
                   Address(decimal,hexadecimal,or symbol)\n\
       Conditions:0-0xffffffff\n\n\
example:\n\
(gdb)watch i\n\
(gdb)watch *0x0\n"

#define C33_RWATCH_HELPMSG  "\
rwatch: Set data read break                    [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: rwatch Breakpoint\n\
       Breakpoint: Breakpoint\n\
                   A breakpoint can be specified with one of the following\n\
                   Symbol\n\
                   Address(decimal,hexadecimal,or symbol)\n\
       Conditions:0-0xffffffff\n\n\
example:\n\
(gdb)rwatch i\n\
(gdb)rwatch *0x0\n"

#define C33_AWATCH_HELPMSG  "\
awatch: Set data read/write break              [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: awatch Breakpoint\n\
       Breakpoint: Breakpoint\n\
                   A breakpoint can be specified with one of the following\n\
                   Symbol\n\
                   Address(decimal,hexadecimal,or symbol)\n\
       Conditions:0-0xffffffff\n\n\
example:\n\
(gdb)awatch i\n\
(gdb)awatch *0x0\n"

#define C33_DELETE_HELPMSG  "\
delete: Clear break by break number            [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: delete [BreakNo]\n\
       BreakNo: Break number(decimal or hexadecimal)\n\
               When this entry is omitted,all brealpoints are deleted.\n\n\
example:\n\
(gdb)delete 1\n\
(gdb)delete 1 3\n\
(gdb)delete\n"

#define C33_CLEAR_HELPMSG   "\
clear: Clear break by break position           [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: clear [Breakpoint]\n\
       Breakpoint: Breakpoint\n\
                   Can be specified by one of following.\n\
                   Function name\n\
                   Source file name:line number,or line number only\n\
                   Address(decimal,hexadecimal,or symbol)\n\
                   When this entry is omitted,all breakpoint set in the source\n\
                   that includes the current PC address are deleted.\n\
       Conditions:0-0xffffffff\n\n\
example:\n\
(gdb)clear main\n\
(gdb)clear main.c:22\n\
(gdb)clear *0xc0001e\n\
(gdb)clear\n"

#define C33_ENABLE_HELPMSG  "\
enable: Enable breakpoint                      [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: enable [BreakNo]\n\
       BreakNo: Break number(decimal or hexadecimal)\n\
                When this entry is omitted,all brealpoints are enabled.\n\n\
example:\n\
(gdb)enable 1\n\
(gdb)enable 1 3\n\
(gdb)enable\n"

#define C33_DISABLE_HELPMSG "\
disable: Disable breakpoint                    [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: disable [BreakNo]\n\
       BreakNo: Break number(decimal or hexadecimal)\n\
                When this entry is omitted,all brealpoints are disabled.\n\n\
example:\n\
(gdb)disable 1\n\
(gdb)disable 1 3\n\
(gdb)disable\n"

#define C33_IGNORE_HELPMSG  "\
ignore: Disable breakpoint with ignore counts  [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: ignore BreakNo Count\n\
       BreakNo: Break number(decimal or hexadecimal)\n\
       Count:   Number of break hits to be disabled(decimal or hexadicimal)\n\n\
example\n\
(gdb)ignore 2 2\n"

#define C33_PRINT_HELPMSG   "\
print: Alter symbol value                      [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: print Symbol[=Value]\n\
       Symbol:  Variable name\n\
       Value:   Value used to alter(decimal,hexadecimal,or symbol)\n\
                When this entry omitted,the current symbol vale is displayd.\n\
                Conditions:0<=valu<=valid range of type\n\n\
example:\n\
(gdb)print j\n\
(gdb)print j=5\n"

#define C33_FILE_HELPMSG    "\
file: Load debugging information               [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: file Filename\n\
       Filename: Name of object file in elf format to be debugged(with \n\
       path also specifiable)\n\n\
example:\n\
(gdb)file sample.elf\n"

#define C33_LOAD_HELPMSG    "\
load: Load program                             [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: load [Filename]\n\
       Filename: Name of object file in elf format to be debugged\n\
                 (with path also specifiable)\n\
                 When this entry is omitted,the file specified previously \n\
                 by the file command is loaded.\n\
                 This specication is usaully omitted\n\n\
example:\n\
(gdb)load\n\
(gdb)load sample.elf\n"

#define C33_LPT_HELPMSG    "\
c33 lpt: Set Fast Load program mode and parallel port       [ICD2]\n\n\
usage: c33 lpt [port number]\n\
       port number: parallel port number [1 or 2]\n\
                 File loading is performed from an appointed parallel port.\n\
                 When this entry is omitted,fast load program mode is setted off.\n\
                 When you confirm the mode, please execute before \"load\" command.\n\n\
example:\n\
(gdb)c33 lpt 1\n\
(gdb)c33 lpt\n"

#define C33_OAB_HELPMSG    "\
c33 oab: Set area break                                     [ICD3/ICD4/ICD6]\n\n\
usage: c33 oab  mode BBCU_BusMasterSelect ReadWriteCondition Area\n\
       mode: in chip area break mode\n\
             on   Set on chip area break\n\
             off  Clear on chip area break\n\
       BBCU_BusMasterSelect: BBCU bus master[0x1 - 0xf]\n\
             bit3 =1 memory access by USER\n\
             bit2 =1 memory access by DMA\n\
             bit1 =1 memory access by CACHE\n\
             bit0 =1 memory access by CPU\n\
       ReadWriteCondition: condition of memory read/write\n\
             r   read\n\
             w   write\n\
             rw  read or write\n\
       Area:Set break area[0x1 - 0x7fffff]\n\
             bit0-bit22 are equivalent to the area 0 - the area 22.\n\
             It is invalid, if 1 is set as a bit and validity and 0 will be set up.\n\
\n\
example:\n\
(gdb)c33 oab on 7 w 0x103000\n\
(gdb)c33 oab off\n"

#define C33_OBB_HELPMSG    "\
c33 obb: Set bus break                                      [ICD3/ICD4/ICD6]\n\n\
usage: c33 obb mode BusSelect BBCU_BusMasterSelect TrigBreak ReadWriteCondition InstData\n\
               access int IntLevel DataComp Address AddressMask Data DataMask Counter CounterClear\n\
       mode: in chip area break mode\n\
             on   Set on chip bus break\n\
             off  Clear on chip bus break\n\
       BusSelect: Bus Selection\n\
             0   CPU data bus\n\
             1   CPU Instruction bus\n\
             2   BBCU\n\
       BBCU_BusMasterSelect: BBCU bus master[0x1 - 0xf]\n\
             bit3 =1 memory access by USER\n\
             bit2 =1 memory access by DMA\n\
             bit1 =1 memory access by CACHE\n\
             bit0 =1 memory access by CPU\n\
       TrigBreak: Trigger Break selection\n\
             0   Trigger output only\n\
             1   break\n\
       ReadWriteCondition: condition of memory read/write\n\
             r   read\n\
             w   write\n\
             rw  read or write\n\
       InstData: Condition of instruction and data\n\
             0   Instruction\n\
             1   data\n\
             2   Instriction and data\n\
       access: Condition of access[0 - 0x7]\n\
             bit2 =1 word access\n\
             bit1 =1 half word access\n\
             bit0 =1 byte access\n\
       int: Condition of interruption\n\
             0   The bus break by the interruption level is invalid.\n\
             1   The bus break by the interruption level is effective.\n\
       IntLevel: Interrupt level[0 - 15]\n\
       DataComp: Data value division comparison specification\n\
             0   32 bits compares the method in the case of comparing the data value on a bus.\n\
             1   The method in the case of comparing the data value on a bus is compared individually\n\
                 [ 16 bits of higher ranks, and 16 bits of low ranks ], and coincidence conditions are ORed.\n\
       Address: Break Address\n\
                Conditions:0-0xffffffff\n\
       AddressMask: Break Address mask bits\n\
                Conditions:0-0xffffffff\n\
       Data:    Break Data\n\
                Conditions:0-0xffffffff\n\
       DataMask: Break Data mask bits\n\
                Conditions:0-0xffffffff\n\
       Counter: Number-of-times break counter\n\
                Conditions:1-0x7fffffff\n\
       CounterClear: Counter clear/save\n\
              0  clear break counter by program run\n\
              1  save break counter by program run\n\

\n\
example:\n\
(gdb)c33 obb on 0 1 1 w 1 0x04 0 0 0 0 0 0x55555555 0xffffffff 3 0\n\
(gdb)c33 obb off\n"

#define C33_OBT_HELPMSG    "\
c33 obb: Set bus tracek                                     [ICD3/ICD4/ICD6]\n\n\
usage: c33 obt TraceMode BusSelect BBCU_BusMasterSelect ReadWriteCondition InstData\n\
               access BusOutput AddrBusSize Compress UserInput\n\
       TraceMode: trace mode\n\
             0   normal mode\n\
             1   user mode\n\
             2   off\n\
       BusSelect: Bus Selection\n\
             0   CPU data bus\n\
             1   CPU Instruction bus\n\
             2   BBCU\n\
       BBCU_BusMasterSelect: BBCU bus master[0x1 - 0xf]\n\
             bit3 =1 memory access by USER\n\
             bit2 =1 memory access by DMA\n\
             bit1 =1 memory access by CACHE\n\
             bit0 =1 memory access by CPU\n\
       ReadWriteCondition: condition of memory read/write\n\
             r   read\n\
             w   write\n\
             rw  read or write\n\
       InstData: Condition of instruction and data\n\
             0   Instruction\n\
             1   data\n\
             2   Instriction and data\n\
       access: Condition of access[0 - 0x7]\n\
             bit2 =1 word access\n\
             bit1 =1 half word access\n\
             bit0 =1 byte access\n\
       BusOutput: Condition of bus output\n\
             0   address bus\n\
             1   data bus\n\
             2   address/data bus\n\
       AddrBusSize: size of address bus [1 - 4]\n\
       Compress: Address Compress output\n\
             0   address/data compress is disable\n\
             1   address/data compress is enable\n\
       UserInput: user input signal\n\
             0   external trace signal(TRC IN)\n\
             1   debug trace trigger signal(DBT)\n\
\n\
example:\n\
(gdb)c33 obt 0 2 0x8 rw 2 0x04 0 4 0 0\n"

#define C33_OTD_HELPMSG    "\
c33 otd: Display bus trace contents                         [ICD3/ICD4/ICD6]\n\n\
usage: c33 otd mode [StartClock EndClock]\n\
       mode: output format\n\
             0   bus trace\n\
             1   PC trace and bus trace\n\
       StartClock: start trace cycle number\n\
       EndClock:   end trace cycle number\n\
                Conditions:0<=EndCycle<=StartCycle<=1,048,574\n\
\n\
example:\n\
(gdb)c33 otd 0 3000 100\n\
(gdb)c33 otd 0\n"

#define C33_OTF_HELPMSG    "\
c33 otf: Save bus trace contents                            [ICD3/ICD4/ICD6]\n\n\
usage: c33 otf mode filename [StartClock EndClock]\n\
       mode: output format\n\
             0   bus trace\n\
             1   PC trace and bus trace\n\
       filename: output file name\n\
       StartClock: start trace cycle number\n\
       EndClock:   end trace cycle number\n\
                Conditions:0<=EndCycle<=StartCycle<=1,048,574\n\
\n\
example:\n\
(gdb)c33 otf trace.txt 0 3000 100\n"

#define C33_DAS_HELPMSG    "\
c33 das: Debug unit address set                             [ICD3/ICD4/ICD6]\n\n\
usage: c33 das DebugROMaddress DebugRAMaddress [CPUtype]
       DebugROMaddress: Debug ROM start address(decimal or hexadecimal)\n\
       DebugRAMaddress: Debug RAM start address(decimal or hexadecimal)\n\
       CPUtype:         CPU type
                        0   Standard macro(default)\n\
                        1   PE\n\
                        3   Sdvanced macro\n\
\n\
example:\n\
(gdb)c33 das 0x60000 0x71000 3\n\
Specify 0x60000 for debug ROM start address,0x71000 for the debug RAM start address and advanced\n\
macro for the CPU type using this command,then set these conditions into the ICD using the target\n\
command.\n"


#define C33_SOURCE_HELPMSG  "\
source: Execute command file                   [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: source Filename\n\
       Filename: Name of command file.\n\n\
example:\n\
(gdb)source gwb33.cmd\n"

#define C33_TARGET_HELPMSG  "\
target: Connect target                         [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: target Type [Time]\n\
       Type:  One of the following symbols that specify the target\n\
              icd usb:   Connected with the S5U1C33001H(ICD3/ICD4) \n\
                         a USB interface(in ICD3/ICD4 mode).\n\
              icd usb2:  Connected with the S5U1C33001H(ICD3/ICD4)via \n\
                         a USB interface(in ICD3/ICD4 mode).\n\
                         It is used when connecting two sets of ICD3/4 to one set of PC.
                         The 1st debugger is \"target icd usb\" and the 2nd debugger is connected by \n\
                         \"target icd usb2.\" \n\
              icd6 usb:   Connected with the ICD6 \n\
                         a USB interface(in ICD6 mode).\n\
              icd6 usb2:  Connected with the ICD6 \n\
                         a USB interface(in ICD6 mode).\n\
                         It is used when connecting two sets of ICD6 to one set of PC.
                         The 1st debugger is \"target icd usb\" and the 2nd debugger is connected by \n\
                         \"target icd usb2.\" \n\
              icd com/V: Connected with the S5U1C33000H(ICD2)or \n\
                         S5U1C330M1D1(+S5U1C330M2S)via a serial intrface(in ICD2 or MON mode)\n\
                         ,where N denotes a port number that can be\n\
                         selected from com1 through com9. The debugger determines whether\n\
                          the S5U1C33000H or S5U1C330M1D1(+S5U1C330M2S)is connected.\n\
              sim:       Simulator started (in SIM mode).\n\
       Time:  Communication start delay time(in seconds,decimal,or hexadecimal)\n\
              When operating in ICD3/ICD4/ICD6 or simulator mode,this specification is unnecessary.\n\n\
example:\n\
(gdb)target sim\n\
(gdb)target icd com1 10\n\
(gdb)target icd usb\n"

#define C33_DETAUCH_HELPMSG "\
detach: disconnect target                     [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: detach\n\
       This command can be used to turn the S5U1C33000H or S5U1C33001H off \n\
       to switch between simulator mode and other modes,or perform operations \n\
       on the target board. You need not execute this command to terminate\n\
       debugging.\n\n\
example:\n\
(gdb)detach\n"

#define C33_PWD_HELPMSG "\
pwd: display current directory                 [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: pwd\n\
example:\n\
(gdb)pwd\n"

#define C33_CD_HELPMSG  "\
cd: change current directory                   [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: cd Directory\n\
       Disrectory: Character string used to specify a directory\n\n\
example:\n\
(gdb)cd /cygdrive/c/gnu33/sample/asm\n\
It is changed to \"c:\\gnu33\\sample\\asm\".\n\
A drive name must be specified in \"/cygdrive/drive name/\" format.\n\
Do not specifiy a drive name in \"c:\" format.\n"

#define C33_QUIT_HELPMSG    "\
quit: Quit debugger                            [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: quit\n\
       q       (abbreviated form)\n\n\
       You also can choose [Exit] from then [File]menu in the [source]window \n\
       or click the [Close]box in the [Source]window to quit the debugger.\n\n\
example:\n\
(gdb)q\n"

#define C33_FB_HELPMSG  "\
c33 fb: Fill area,in bytes                     [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: c33 fb StartAddr EndAddr Data\n\
       StartAddr: Start address of area to be filled(decimal,hexadecimal, or symbol)\n\
       EndAddr:   End address of area to be filled(decimal,hexadecimal, or symbol)\n\
       Data:      The data to write(decimal,hexadecimal)\n\
       Conditions: 0<=StartAddr<=EndAddr<=0xffffffff,0<=Data<=0xff\n\n\
example:\n\
(gdb)c33 fb 0x0 0xf 0x1\n\
The entry memory area from address 0x0 to address 0xf is rewritten with data 0x01(16 bytes).\n"

#define C33_FH_HELPMSG  "\
c33 fh: Fill area,in half words                [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: c33 fh StartAddr EndAddr Data\n\
       StartAddr: Start address of area to be filled(decimal,hexadecimal, or symbol)\n\
       EndAddr:   End address of area to be filled(decimal,hexadecimal, or symbol)\n\
       Data:      The data to write(decimal,hexadecimal)\n\
       Conditions: 0<=StartAddr<=EndAddr<=0xffffffff,0<=Data<=0xffff\n\n\
example:\n\
(gdb)c33 fh 0x0 0xf 0x1\n\
The entry memory area from address 0x0 to address 0xf is rewritten with data 0x0001(8 half words).\n"

#define C33_FW_HELPMSG  "\
c33 fw: Fill area,in words                     [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: c33 fw StartAddr EndAddr Data\n\
       StartAddr: Start address of area to be filled(decimal,hexadecimal, or symbol)\n\
       EndAddr:   End address of area to be filled(decimal,hexadecimal, or symbol)\n\
       Data:      The data to write(decimal,hexadecimal)\n\
       Conditions: 0<=StartAddr<=EndAddr<=0xffffffff,0<=Data<=0xffffffff\n\n\
example:\n\
(gdb)c33 fw 0x0 0xf 0x1\n\
The entry memory area from address 0x0 to address 0xf is rewritten with data 0x00000001(4 words).\n"

#define C33_MVB_HELPMSG "\
c33 mvb: Copy area,in bytes                    [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: c33 mvb SourceStart SourceEnd Distination\n\
       SourceStart: Start address of area from which to copy(decimal,hexadecimal, or symbol)\n\
       SourceEnd:   End address of area from which to copy(decimal,hexadecimal, or symbol)\n\
       Distination: Start address of area to which to copy(decimal,hexadecimal, or symbol)\n\
       Conditions: 0<=SourceStart<=SourceEnd<=0xffffffff, 0<=Distination<=0xffffffff\n\n\
example:\n\
(gdb)c33 mvb 0x0 0x7 0x8\n\
The contents of a memory area specified by address 0x0 and 0x7 \n\
is copied to an area beggining with address 0x8.\n\
(gdb)c33 mvb i i  i+4\n\
The contents of int variable i is copied to an area located one \n\
byte after that int variable.\n"

#define C33_MVH_HELPMSG "\
c33 mvh: Copy area,in half words               [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: c33 mvh SourceStart SourceEnd Distination\n\
       SourceStart: Start address of area from which to copy(decimal,hexadecimal, or symbol)\n\
       SourceEnd:   End address of area from which to copy(decimal,hexadecimal, or symbol)\n\
       Distination: Start address of area to which to copy(decimal,hexadecimal, or symbol)\n\
       Conditions: 0<=SourceStart<=SourceEnd<=0xffffffff, 0<=Distination<=0xffffffff\n\n\
example:\n\
(gdb)c33 mvh 0x0 0x7 0x8\n\
The contents of a memory area specified by address 0x0 and 0x7 is \n\
copied to an area beggining with address 0x8.\n\
(gdb)c33 mvh i i  i+4\n\
The contents of int variable i is copied to an area located \n\
one half word after that int variable.\n"

#define C33_MVW_HELPMSG "\
c33 mvw: Copy area,in words                    [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: c33 mvw SourceStart SourceEnd Distination\n\
       SourceStart: Start address of area from which to copy(decimal,hexadecimal, or symbol)\n\
       SourceEnd:   End address of area from which to copy(decimal,hexadecimal, or symbol)\n\
       Distination: Start address of area to which to copy(decimal,hexadecimal, or symbol)\n\
       Conditions: 0<=SourceStart<=SourceEnd<=0xffffffff, 0<=Distination<=0xffffffff\n\n\
example:\n\
(gdb)c33 mvw 0x0 0x7 0x8\n\
The contents of a memory area specified by address 0x0 and 0x7 is copied \n\
to an area beggining with address 0x8.\n\
(gdb)c33 mvw i i  i+4\n\
The contents of int variable i is copied to an area located one word after \n\
that int variable.\n"

#define C33_RM_HELPMSG "\
c33 rm: Read memory from target memory from debugger   [ICD2/ICD3/ICD4/ICD6]\n\n\
usage: c33 rm Address1 Address2\n\
       Address1: Start address of read area(decimal,hexadecimal, or symbol)\n\
       Address2: End   address of read area(decimal,hexadecimal, or symbol)\n\
       Conditions: 0<=Address1<=Address2<=0xffffffff\n\n\
example:\n\
(gdb)c33 rm 0x0 0x7fe\n\n"

#define C33_READMD_HELPMSG "\
c33 readmd: Set read memory mode                       [ICD3/ICD4/ICD6]\n\n\
usage: c33 readmd Mode\n\
       Mode: read mode(decimal,hexadecimal, or symbol)\n\
             0     Normal.\n\
             1     Exact boundary.\n\
example:\n\
(gdb)c33 readmd 1\n\n"

#define C33_DF_HELPMSG  "\
c33 df: Save memory contents                   [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: c33 df StartAddr EndAddr Type Filename Append\n\
       StartAddr: Start address of area to save(decimal,hexadecimal, or symbol)\n\
       EndAddr:   End address of area to save(decimal,hexadecimal, or symbol)\n\
       Type:      One of the following values that specify the type file\n\n\
                  1    Binary file.\n\
                  2    Text file.\n\
                  3    Motorola S3 file.\n\
       Filename:  File name\n\
       Append:    a    File append mode.\n\
                       Type=3: Not output footer record. \n\
                  f    File append mode.\n\
                       Type=3: Output footer record. \n\n\
       Conditions: 0<=StartAddr<=EndAddr<=0xffffffff\n\n\
example:\n\
(gdb)c33 df 0x0 0xf 2 dumo.txt\n\
(gdb)c33 df 0x80000 0x80103 3 dump.mot\n"

#define C33_CALLMD_HELPMSG  "\
c33 callmd: Set user funtion call mode         [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: c33 callmd Mode [FileName]\n\
       Mode:      One of the following numeric values that specify result output destination\n\n\
                  1    Display int the [Console]window(default).\n\
                  2    Write to a file.\n\
                  3    Display in the [Console]window and write to a file.\n\
       FileName:  Output file name (not effective when Mode=1)\n\n\
example:\n\
(gdb)c33 callmd 2 call.txt\n"

#define C33_CALL_HELPMSG    "\
c33 call: Call user funtion                    [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: c33 call Function [Arg1] [Arg2] [Arg3]\n\
       Function:  The function to be called(function name or decimal/hexadecimal start address)\n\
       Arg1-3:    Argument(decimal,hexadecimal,or symbol)\n\
       Conditions: Up to three arguments can be specified.\n\n\
example:\n\
(gdb)c33 call print_data 1 2 3\n\
The function print_data is called after specifying three arguments.\n"

#define C33_RSTC_HELPMSG    "\
c33 rstc: Cold reset                           [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: c33 rstc\n\n\
example:\n\
(gdb)c33 rstc\n\
The CPU is cold-reset.\n"

#define C33_RSTH_HELPMSG    "\
c33 rsth: Hot reset                            [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: c33 rsth\n\n\
example:\n\
(gdb)c33 rsth\n\
The CPU is hot-reset.\n"

#define C33_INT_HELPMSG "\
c33 int: Interrupt                             [SIM]\n\n\
usage: c33 int [Type Level]\n\
       Type:       Interrupt type(decimal,hexadecimal,or symbol)\n\
       Level:      Interrupt priority level(decimal,hexadecimal,or symbol)\n\
       Conditions: 0<=Type<=0x7fff,0<=Level<=15\n\n\
example:\n\
(gdb)c33 int\n\
If no parameters are specified,an NMI is generated\n\
(gdb)c33 int 3 6\n"

#define C33_RPF_HELPMSG "\
c33 rpf: Set map information                   [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: c33 rpf Filename\n\
       Filename:   Name of parameter file(with path also specifiable)\n\n\
example:\n\
(gdb)c33 rpf sample.par\n"

#define C33_MAP_HELPMSG "\
c33 map: Display map information               [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: c33 map\n\n\
example:\n\
(gdb)c33 map\n"

#define C33_FLS_HELPMSG "\
c33 fls: Set flash memory                      [ICD2/ICD3/ICD4/ICD6/MON]\n\n\
usage: c33 fls StartAddr EndAddr ErasePrg WritePrg\n\
       StartAddr: Start address of flash memory(decimal,hexadecimal, or symbol)\n\
       EndAddr:   End address of flash memory(decimal,hexadecimal, or symbol)\n\
       ErasePrg:  Start address of erase program(decimal,hexadecimal, or symbol)\n\
       WritePrg:  Start address of write program(decimal,hexadecimal, or symbol)\n\
       Conditions: 0<=StartAddr<=EndAddr<=0xffffffff\n\
                   0<=ErasePrg<=0xffffffff\n\
                   0<=WritePrg<=0xffffffff\n\n\
example:\n\
(gdb)c33 fls 0x200000 0x2fffff FLASH_ERASE FLASH_LOAD\n\
The flash memory area in the target system(0x200000 to 0x2fffff),\n\
and addresses of the erase and write routines are set.\n"

#define C33_FLE_HELPMSG "\
c33 fle: Erase flash memory                    [ICD2/ICD3/ICD4/ICD6/MON]\n\n\
usage: c33 fle ControlReg StartBlock EndBlock [Timer]\n\
       ControlReg: Start address set by c33 fls(decimal,hexadecimal, or symbol)\n\
       StartBlock: First block in erase range(decimal,hexadecimal, or symbol)\n\
       EndBlock:   Last block in erase range(decimal,hexadecimal, or symbol)\n\
                   When StartBlock=EndBlock=0,the entire area is erased.\n\
       Timer:      Flash memory erase timeout[sec](decimal,hexadecimal, or symbol)\n\
                   When this entry is omitted,the Timer is 150 sec. \n\n\
example:\n\
(gdb)c33 fle 0x200000 0 0\n\
The entire area of flash memory is erased.\n"

#define C33_TM_HELPMSG  "\
c33 tm: Trace mode                             [ICD2/ICD3/ICD4/ICD6/SIM]\n\n\
usage 1: ICD2/ICD3/ICD4/ICD6 mode\n\
       c33 tm Mode CycleInfo [Trigger1 Trigger2 Break Measure]\n\
       Mode:       Trace mode\n\
                   1  All trace mode,without overwrite\n\
                   2  All trace mode,with overwrite(default)\n\
                   3  Area trace mode\n\
       CycleInfo:  Content of number of run cycles displayd\n\
                   1  Cumulative cycles from start of a trace(default)\n\
                   2  Individual cycles for each instruction\n\
       Trigger1:   Start address of area trace(decimal,hexadecimal,or symbol)\n\
       Trigger2:   End address of area trace(decimal,hexadecimal,or symbol)\n\
       Break:      After area trace\n\
                   1  Do not break at Trigger2\n\
                   2  Break at Trigger2\n\
       Measure:    Area trace time measurement\n\
                   1  Measure execution time from start to break\n\
                   2  Measure execution time in trace area only\n\
       Conditions: 0<=Trigger1<=0xffffffff,0<=Trigger2<=0xffffffff\n\
       Always be sure to set Trigger1,Trigger2,Break and Measure for area trace mode.\n\
       Do not specify these parameters for all trace mode.\n\n\
example:\n\
(gdb)c33 tm 2 2\n\
(gdb)c33 tm 3 1 0x600000 0x600100 2 2\n\n\
usage 2: SIM mode\n\
       c33 tm on Mode [Filename]\n\
            Sets trace mode.\n\
       c33 tm off\n\n\
            Clears trace mode.\n\
       Mode:      Trace mode\n\
                  1  Normal display\n\
                  2  Register display\n\
                  3  Normal display including source information\n\
                  4  Register display including source information\n
       Filename:  Name of file to which trace information is output\n\
                  When a file name is specified,sampled trace information \n\
                  is output to the specified file, and not display in a window. \n\
                  When this entry is omitted,trace information is displayed in the [Trace]\n\
                  window,and not output to a file\n\n
example:\n\
(gdb)c33 tm on 4 trace.log\n\
(gdb)c33 tm off\n"

#define C33_TD_HELPMSG  "\
c33 td: Display trace content                  [ICD2/ICD3/ICD4/ICD6]\n\n\
usage: c33 td [StartCycle EndCycle]\n\
       StartCycle: Trace cycle number from which to start display(decimal or hexadecimal)\n\
       EndCycle:   Trace cycle number from which to complete display(decimal or hexadecimal)\n\
                   cycle number 0 is the lastest data.If StartCycle and EndCycle are \n\
                   omitted,about 10k trace cycles are displayed\n\
       Conditions: S5U1C33000H(ICD2)       0<=EndCycle<=StartCycle<=131,071\n\
                   S5U1C33001H(ICD3/ICD4/ICD6)  0<=EndCycle<=StartCycle<=1,048,574\n\n\
example:\n\
(gdb)c33 td\n\
All trace information currently in trace memory is displayed in the [Trace]window.\n\
(gdb)c33 td 1000 0\n\
Trace incormation from cycle number 1000 to the latest is displayed in the [Trace]window.\n"

#define C33_TS_HELPMSG  "\
c33 ts: Search trace content                   [ICD2/ICD3/ICD4/ICD6]\n\
usage: c33 ts Address [PreLine PostLine]\n\n\
       Address:    Executed address(decimal,hexadecimal,or symbol)\n\
                   Note that specifying 0xffffffff turns off search mode.\n\
       PreLine:    Number of lines to display and save before searched line(decimal or hexadecimal)\n\
       PostLine:   Number of lines to display and save after searched line(decimal or hexadecimal)\n\
       Conditions: 0<=Address<=0xffffffff\n\
                   0<=PreLine<=256, 0<=PostLine<=256\n\n\
example:\n\
(gdb)c33 ts 0x20100 5 5\n\
The trace information for only the cycle in which the instruction at address 0x20100 was\n
exected,and for the five cycles defore and after are displayed and saved.\n\
(gdb)c33 ts 0xffffffff\n\
Search mode is turned off.\n"

#define C33_TF_HELPMSG  "\
c33 tf: Save trace content                     [ICD2/ICD3/ICD4/ICD6]\n\n\
usage: c33 tf Filename [StartCycle EndCycle]\n\
       Filename:   Name of file in which to save trcae information\n\
       StartCycle: Trace cycle number from which to start saving(decimal or hexadecimal)\n\
       EndCycle:   Trace cycle number at which to complete saving(decimal or hexadecimal)\n\
                   Cycle number 0 is the latest data.If StartCycle and EndCycle are omitted,\n\
                   about 10k trace cycles are saved.\n\
       Conditions: S5U1C33000H(ICD2)       0<=EndCycle<=StartCycle<=131,071\n\
                   S5U1C33001H(ICD3/ICD4/ICD6)  0<=EndCycle<=StartCycle<=1,048,574\n\n\
example:\n\
(gdb)c33 tf trace.log\n\
All trace information currently in trace memory is saved to trace.log file.\n\
(gdb)c33 tf trace.log 1000 0\n\
Trace information from cycle number 1000 to the latest is saved to trace.log file.\n"

#define C33_STDIN_HELPMSG   "\
c33 stdin: Data input simulation               [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: c33 stdin Mode BreakAddr BufferAddr [Filename]\n\
       Mode:       1 Set data input simulation\n\
                   2 Clear data input simulation\n\
       BreakAddr:  Break address(decimal,hexadecimal,or symbol)\n\
       BufferAddr: Input buffer address(decimal,hexadecimal,or symbol)\n\
                   The buffer size is fixed to 65 bytes.
       Filename:   Name of input file.
                   When this entry is omitted,data entry from the[Simulated I/O]window is assumed.\n\
       Conditions: 0<=BreakAddr<=0xffffffff,0<=BufferAddr<=0xffffffff\n\n\
example:\n\
(gdb)c33 stdin 1 READ_FLASH READ_BUF input.txt\n\
Data entered from a file is set.\n\
(gdb)c33 stdin 1 READ_FLASH READ_BUF\n\
Data entered via the [Simulated I/O]window is set.\n\
(gdb)c33 stdin 2\n\
The data input simulation function is cleared.If data entered from a file was set,\n\
the specified file is closed.\n"

#define C33_STDOUT_HELPMSG  "\
c33 stdout: Data output simulation          [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: c33 stdout Mode BreakAddr BufferAddr [Filename]\n\
       Mode:       1 Set data output simulation\n\
                   2 Clear data output simulation\n\
       BreakAddr:  Break address(decimal,hexadecimal,or symbol)\n\
       BufferAddr: Output buffer address(decimal,hexadecimal,or symbol)\n\
                   The buffer size is fixed to 65 bytes.
       Filename:   Name of input file.
                   When this entry is omitted,data entry from the[Simulated I/O]window.\n\
       Conditions: 0<=BreakAddr<=0xffffffff,0<=BufferAddr<=0xffffffff\n\n\
example:\n\
(gdb)c33 stdout 1 WRITE_FLASH WRITE_BUF output.txt\n\
Data output a file is set.\n\
(gdb)c33 stdout 1 WRITE_FLASH WRITE_BUF\n\
Data output to the [Simulated I/O]window is set.\n\
(gdb)c33 stdout 2\n\
The data output simulation function is cleared.If data output to a file was set,\n\
the specified file is closed.\n"

#define C33_FWE_HELPMSG "\
c33 fwe: Erase program/data                 [ICD3/ICD4/ICD6]\n\n\
usage: c33 fwe Mode\n\
       Mode:       0 Erases write data\n\
                   1 Erases data/erase write program\n\n\
example:\n\
(gdb)c33 fwe 0\n\
The storage area for flash write data and the address information\n\
in the S5U1C33001H(ICD3/ICD4/ICD6) are erased.\n\
(gdb)c33 fwe 1\n\
The storage area for flash erase/write program and the entry information\n\
in the S5U1C33001H(ICD3/ICD4/ICD6) are erased.\n\n\
note:\n\
       This command can only be use in ICD3/ICD4/ICD6 mode.\n"

#define C33_FWLP_HELPMSG    "\
c33 fwlp: Load program                          [ICD3/ICD4/ICD6]\n\n\
usage: c33 fwlp Filename EraseEntryAddr WriteEntryAddr\n\
       Filename:        Name of data erase/write program file(Motorola S3 format file)\n\
       EraseEntryAddr:  Erase routine entry address\n\
                        (RAM address on the CPU side,in decimal or hexadecimal)\n\
       WriteEntryAddr:  Write routine entry address\n\
                        (RAM address on the CPU side,in decimal or hexadecimal)\n\
       Comment:         Comments to identify data/address information(may be omitted)\n\
       Conditions:      0<=EraseEntryAddr<=0xfffffffe(A0=0),0<=WriteEntryAddr<=0xfffffffe(A0=0),\n\
                        0<=comment size<=128 bytes\n\n\
example:\n\
(gdb)c33 fwlp writer.sa 0x90 0xb4\n\
The data erase/write program for the flash writer(prepared in file writer.sa)\n\
is loded in the S5U1C33001H(ICD3/ICD4/ICD6),and start addresses of the erase and write \n\
routines are set to 0x90 and 0xb4,respectively.\n\n\
note:\n\
       This command can only be use in ICD3/ICD4/ICD6 mode.\n\
       The data erase/write program is 8KB or less in size.\n"

#define C33_FWLD_HELPMSG    "\
c33 fwld: Load data                             [ICD3/ICD4/ICD6]\n\n\
usage: c33 fwld Filename EraseStartBlock EraseEndBlock EraseParam [Comment]\n\
       Filename:        Name of data file(Motorola S3 format file)\n\
       EraseStartBlock: Block at which to start erasing(flash on the CPU side,\n\
                        in decimal or hexadecimal)\n\
       EraseEndBlock:   Block at which to complete erasing(flash on the CPU side,\n\
                        in decimal or hexadecimal)\n\
       EraseParam:      Erase parameter\n\
       Comment:         Comments to identify data/address information(may be omitted)\n\
       Conditions:      0<=EraseStartBlock<=0xffffffff,0<=EraseEndBlock<=0xffffffff,\n\
                        If EraseStartBlock=EraseEndBlock=0,all blocks in flash memory\n\
                        are assumed to be erased.\n\
                        0<=comment size<=128 bytes\n\n\
example:\n\
(gdb)c33 fwld sample.sa 0 0 0x200000\n\
The range of flash memory to be erased(all blocks in flash memory)is set,\n\
and the flash write data prepared in file sample.sa is loaded in the S5U1C33001H(ICD3/ICD4/ICD6).\n\n\
note:\n\
       This command can only be use in ICD3/ICD4/ICD6 mode.\n\
       The data erase/write program is 8KB or less in size.\n"

#define C33_FWDC_HELPMSG    "\
c33 fwdc: Copy target memory                    [ICD3/ICD4/ICD6]\n\n\
usage: c33 fwdc SourceAddr Size EraseStartBlock EraseEnd Block EraseParam [Comment]\n\
       SourceAddr:      Target memory address from which to copy(flash on the CPU side,\n\
                        in decimal,hexadecimal,or symbol)\n\
       Size:            Number of bytes to copy(decimal or hexadecimal)\n\
       EraseStartBlock: Block at which to start erasing\n\
                        (flash on the CPU side,in decimal or hexadecimal)\n\
       EraseEndBlock:   Block at which to complete erasing\n\
                        (flash on the CPU side,in decimal or hexadecimal)\n\
       EraseParam:      Erase parameter\n\
                        A parameter passed to the erase routine in an external routine call\n\
       Comment:         Comments to identify data/address information(may be omitted)\n\
       Conditions:      0<=SourceAddr<=0xfffffffe(A0=0),0<=Size<=0xfffffffe(D0=0)\n\
                        0<=EraseStartBlock<=0xffffffff,0<=EraseEndBlock<=0xffffffff,\n\
                        If EraseStartBlock=EraseEndBlock=0,all blocks in flash memory \n\
                        are assumed to be erased.\n\
                        0<=comment size<=128 bytes\n\n\
example:\n\
(gdb)c33 fwdc FLASH_START 0x100000 0 0 0x200000\n\
The range of flash memory to be erased(all blocks in flash memory)is set,\n\
and 1MB of area is copied from\n\
FLASH_START in target memory to the S5U1C33001H(ICD3/ICD4/ICD6).\n\n\
note:\n\
       This command can only be use in ICD3/ICD4/ICD6 mode.\n\
       The data erase/write program is 8KB or less in size.\n"

#define C33_FWD_HELPMSG "\
c33 fwd: Display flash writer information       [ICD3/ICD4/ICD6]\n\n\
usage: c33 fwd\n\n\
example:\n\
(gdb)c33 fwd\n\n\
note:\n\
       This command can only be use in ICD3 mode.\n"

#define C33_LOG_HELPMSG "\
c33 log: Logging                                [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: c33 log Filename\n\
       Filename: Name of log file\n\
       starts logging\n\
usage: c33 log\n\
       completes logging\n\n\
example:\n\
(gdb)c33 log log.txt\n\
The commands to be entered hereafter and execution results are \n\
output to log.txt in text format.\n\
Log output remains on until the c33 log command is subsequently executed.\n\
(gdb)c33 log\n\
The log file is closed and log output terminated.\n"

#define C33_CLOCKMD_HELPMSG "\
c33 clockmd: Set execution counter mode     [ICD2/ICD3/ICD4/ICD6/SIM]\n\n\
usage: c33 clockmd Mode Function\n\
       Mode:       Counter mode
                   1 Reset mode\n\
                   2 Cumulating mode(default)\n\
       Function:   Unit of measurement\n\
                   1 Number of cycles(default) \n\
                     in simulator mode,only option 1 can be selected.\n\
                   2 Units of micro seconds\n\
                   3 Units of seconds\n\n
example:\n\
(gdb)c33 clockmd 2 1\n"

#define C33_CLOCK_HELPMSG   "\
c33 clock: Display execution counter            [ICD2/ICD3/ICD4/ICD6/SIM]\n\n\
usage: c33 clock\n\
       Displays the results counted during program execution.\n\n
example:\n\
(gdb)c33 clock\n"

#define C33_FIRMUPDATE_HELPMSG   "\
c33 firmupdate: ICDV3 Firmware update           [ICD3/ICD4/ICD6]\n\n\
usage: c33 firmupdate [Filename]\n\
       FileName:   ICDV3/V4 firmware file name(Motorola S3 format file)\n\
example:\n\
(gdb)c33 firmupdate icd33dmt.sa\n"

#define C33_HELP_HELPMSG    "\
c33 help: Displays a command description        [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: c33 help [Command]\n\
       Command:    group number of command or command\n\
                   When this entry is omitted,a list of command groups is displayed.\n\n\
example:\n\
(gdb)c33 help\n\
When you omit parameters,a list of command groups is displayed.\n\
(gdb)c33 help 5\n\
When you specify a command group number,a list of commands belonging to \n\
that group is displayed.\n\
(gdb)c33 help c33 rsth\n\
When you specify a command,a detailed description of that command is displayed.\n"

#define C33_MMU_HELPMSG "\
c33 mmu: Control MMU mode                       [ICD3/ICD4/ICD6]\n\n\
usage: c33 mmu [StackPointer]\n\
       StackPointer: The stack pointer(%%ssp) used with MMU exception\n\
                     at the time of the debugging mode.\n\
                     In the debugging mode, since %%ssp is 0x70ffc, \n\
                     please specify the other user RAM.\n\
                   When omitted,MMU control is not performed.(default)\n\
example:\n\
(gdb)c33 mmu 0x400\n\
     0x400 is used as a stack pointer, and MMU control function is confirmed.\n\
(gdb)c33 mmu\n\
     MMU control is not performed.\n\n\
Caution:
     The guarantee of operation of this command is not offered.\n\
     Please use this command \"AS IS\".\n"


#define C33_DCLK_HELPMSG "\
c33 dclk: Set DCLK                               [MON/ICD2/ICD3/ICD4/ICD6]\n\n\
usage: c33 dclk [number]\n\
       number: The ratio of a dclk cycle.\n\
               1  1/1 \n\
               2  1/2 \n\
               4  1/4 \n\
               8  1/8 \n\
example:\n\
(gdb)c33 dclk 2\n\
     The ratio of a dclk cycle is 1/2. \n"

#define C33_OSCWAIT_HELPMSG "\
c33 oscwait: Set OSC1 Wait Counter               [ICD3/ICD4/ICD6]\n\n\
usage: c33 oscwait [counter]\n\
      counter: Wait software counter of ICD firmware.\n\
               0<=counter<=0xffffff\n\
               When omitted,counter is setted 0x300.\n\

example:\n\
(gdb)c33 oscwait 0x40000\n"

#define C33_CACHEHIT_HELPMSG "\
c33 cachehit: Display the rate of cache hit      [ICD3/ICD4/ICD6]\n\n\
usage: c33 cachehit\n\

example:\n\
(gdb)c33 cachehit\n\
The rate of cache hit is 99.99745%%\n"

#define C33_LOGIANA_HELPMSG "\
c33 logiana: Set Logic analyzer mode             [ICD3/ICD4/ICD6]\n\n\
usage: c33 logiana [mode] [user input]\n\
      mode: on  logic analyzer mode on\n\
            off logic analyzer mode off\n\
      user input: 0  trcae external trace(TRC IN)\n\
                  1  trace target cpu trigger\n\
example:\n\
(gdb)c33 logiana off\n\
logic analyzer mode off.\n\
(gdb)c33 logiana on 1\n\
logic analyzer mode on.\n\
trace target CPU trigger signal.\n"

#define C33_RSTT_HELPMSG    "\
c33 rstt: Target Reset                           [ICD6]\n\n\
usage: c33 rstt\n\n\
example:\n\
(gdb)c33 rstt\n\
TARGET resetting ................... done.\n"

#define C33_TIMEBRK_HELPMSG    "\
c17 timebrk: Set timer break                     [ICD6]\n\n\
usage: c33 timebrk Timer\n\
        timer : timer break mode on \n\
           Conditions: 1<=Timer<=300000\n\
usage: c33 timebrk 0\n\
        timer break mode off \n\
example:\n\
(gdb)c17 timebrk 3000\n\
(gdb)cont\n\
(gdb)c17 timebrk 0\n"

/**********************/
/* "c33 help XXX"     */
/**********************/
const char c33_core_command_name[C33_MAX_CORE_COMMAND_NAME][C33_COMMAND_NAME_SIZE] = 
{
    "0","1","2","3","4","5","6","7","8","9","10","11","12","13",
    "continue","until","step","stepi","next","nexti","finish",
    "break","tbreak","hbreak","thbreak","watch","rwatch","awatch","delete","clear","enable","disable","ignore",
    "print","file","load","source","target","detach","pwd","cd","quit"
};
const char c33_core_command_usage[C33_MAX_CORE_COMMAND_NAME][C33_COMMAND_HELP_MSG_SIZE] = 
{
    C33_HELP_GROUP0_HELPMSG,
    C33_HELP_GROUP1_HELPMSG,
    C33_HELP_GROUP2_HELPMSG,
    C33_HELP_GROUP3_HELPMSG,
    C33_HELP_GROUP4_HELPMSG,
    C33_HELP_GROUP5_HELPMSG,
    C33_HELP_GROUP6_HELPMSG,
    C33_HELP_GROUP7_HELPMSG,
    C33_HELP_GROUP8_HELPMSG,
    C33_HELP_GROUP9_HELPMSG,
    C33_HELP_GROUP10_HELPMSG,
    C33_HELP_GROUP11_HELPMSG,
    C33_HELP_GROUP12_HELPMSG,
    C33_HELP_GROUP13_HELPMSG,
    C33_CONTINUE_HELPMSG,
    C33_UNTIL_HELPMSG,
    C33_STEP_HELPMSG,
    C33_STEPI_HELPMSG,
    C33_NEXT_HELPMSG,
    C33_NEXTI_HELPMSG,
    C33_FINISH_HELPMSG,
    C33_BREAK_HELPMSG,
    C33_TBREAK_HELPMSG,
    C33_HBREAK_HELPMSG,
    C33_THBREAK_HELPMSG,
    C33_WATCH_HELPMSG,
    C33_RWATCH_HELPMSG,
    C33_AWATCH_HELPMSG,
    C33_DELETE_HELPMSG,
    C33_CLEAR_HELPMSG,
    C33_ENABLE_HELPMSG,
    C33_DISABLE_HELPMSG,
    C33_IGNORE_HELPMSG,
    C33_PRINT_HELPMSG,
    C33_FILE_HELPMSG,
    C33_LOAD_HELPMSG,
    C33_SOURCE_HELPMSG,
    C33_TARGET_HELPMSG,
    C33_DETAUCH_HELPMSG,
    C33_PWD_HELPMSG,
    C33_CD_HELPMSG,
    C33_QUIT_HELPMSG
};
/**********************/
/* "c33 help c33 XXX" */
/**********************/
const char c33_command_name[C33_MAX_COMMAND_NAME][C33_COMMAND_NAME_SIZE] = 
{
    "fb","fh","fw","mvb","mvh","mvw","rm","df","callmd","call","rstc","rsth","int","rpf","map","fls","fle",
    "tm","td","ts","tf","stdin","stdout","fwe","fwlp","fwld","fwdc","fwd","log","clockmd","clock","firmupdate",
    "lpt","oab","obb","obt","otd","otf","das","readmd","help","mmu","dclk","oscwait","cachehit","logiana",
    "timebrk","rstt"
};
const char c33_command_usage[C33_MAX_COMMAND_NAME][C33_COMMAND_HELP_MSG_SIZE] = 
{
    C33_FB_HELPMSG,
    C33_FH_HELPMSG,
    C33_FW_HELPMSG,
    C33_MVB_HELPMSG,
    C33_MVH_HELPMSG,
    C33_MVW_HELPMSG,
    C33_RM_HELPMSG,
    C33_DF_HELPMSG,
    C33_CALLMD_HELPMSG,
    C33_CALL_HELPMSG,
    C33_RSTC_HELPMSG,
    C33_RSTH_HELPMSG,
    C33_INT_HELPMSG,
    C33_RPF_HELPMSG,
    C33_MAP_HELPMSG,
    C33_FLS_HELPMSG,
    C33_FLE_HELPMSG,
    C33_TM_HELPMSG,
    C33_TD_HELPMSG,
    C33_TS_HELPMSG,
    C33_TF_HELPMSG,
    C33_STDIN_HELPMSG,
    C33_STDOUT_HELPMSG,
    C33_FWE_HELPMSG,
    C33_FWLP_HELPMSG,
    C33_FWLD_HELPMSG,
    C33_FWDC_HELPMSG,
    C33_FWD_HELPMSG,
    C33_LOG_HELPMSG,
    C33_CLOCKMD_HELPMSG,
    C33_CLOCK_HELPMSG,
    C33_FIRMUPDATE_HELPMSG,
    C33_LPT_HELPMSG,
    C33_OAB_HELPMSG,
    C33_OBB_HELPMSG,
    C33_OBT_HELPMSG,
    C33_OTD_HELPMSG,
    C33_OTF_HELPMSG,
    C33_DAS_HELPMSG,
    C33_READMD_HELPMSG,
    C33_HELP_HELPMSG,
    C33_MMU_HELPMSG,
    C33_DCLK_HELPMSG,
    C33_OSCWAIT_HELPMSG,
    C33_CACHEHIT_HELPMSG,
    C33_LOGIANA_HELPMSG,
    C33_TIMEBRK_HELPMSG,
    C33_RSTT_HELPMSG
};

/**********************/
/* "c33 help set XXX" */
/**********************/
const char c33_set_command_name[C33_MAX_SET_COMMAND_NAME][C33_COMMAND_NAME_SIZE] = 
{
    "{}","$"
};
const char c33_set_command_usage[C33_MAX_SET_COMMAND_NAME][C33_COMMAND_HELP_MSG_SIZE] = 
{
"set {}: Data input                             [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: set {Size}Address=Data\n\
       Size:       one of the following symbols that specify data size\n\
                   char    In units of bytes\n\
                   short   In units of half words\n\
                   int     In units of words\n\
       Address:    Address to which to write data(decimal,hexadecmal,or  symbol)\n\
       Data:       The data to write(decimal,hexadecmal,or  symbol)\n\
       Conditions: 0<=Address<=0xffffffff,0<=Data<=0xff(set {char}),0<=Data<=0xffff(set {short}),\n\
                   0<=Data<=0xffffffff(set {int})\n\n\
example:\n\
(gdb)set {char}0x1000=0x55\n\
Byte data 0x55 is written tp address 0x1000.\n\
(gdb)set {short}0x1000=0x5555\n\
Half word data 0x5555 is written to address 0x1000.\n\
(gdb set {int}i=0x55555555\n\
Word data 0x55555555 is written to int variable i.\n",

"set $: Modify register                         [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: set $RegisterName=Value\n\
       RegisterName: Name of register to change(specified in lower letters)\n\
                     r0-r15,psr,sp,alr,ahr,lco,lsa,lea,sor,ttbr,dp,usp,ssp,pc\n\
       Value:        Word data to set in the register(decimal,hexadecmal,or  symbol)\n\
       Conditions:   0<=Value<=0xffffffff\n\n\
example:\n\
(gdb)set $r1=0x1000\n\
(gdb)set $pc=main\n\
In addition to numerials,symbols can also be used to set values.\n",

};
/***********************/
/* "c33 help info XXX" */
/***********************/
const char c33_info_command_name[C33_MAX_INFO_COMMAND_NAME][C33_COMMAND_NAME_SIZE] = 
{
    "reg","breakpoints","locals","var"
};
const char c33_info_command_usage[C33_MAX_INFO_COMMAND_NAME][C33_COMMAND_HELP_MSG_SIZE] = 
{
"info reg: Display register                     [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: info reg [RegisterName]\n\
       RegisterName: Name of register to change(specified in lower letters)\n\
                     r0-r15,psr,sp,alr,ahr,lco,lsa,lea,sor,ttbr,dp,usp,ssp,pc\n\
                     If the above is omitted,the contents of all registers are displayed.\n\n
example:\n\
(gdb)info reg r1\n\
(gdb)info reg ttbr\n\
(gdb)info reg\n",

"info breakpoints: Display breakpoint list      [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: info breakpoints\n\n\
example:\n\
(gdb)info breakpoints\n\
(gdb)info b\n",

"info locals: Display local symbol              [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: info locals\n\n\
example:\n\
(gdb)info locals\n\n
All local symbols defined in the function that includes the current PC address\n\
 are displayed along with symbol content.\n",

"info var: Display global symbol                [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: info var\n\n\
example:\n\
(gdb)info var\n\
All defined global and static variables are displayed in list from separately for each source file.\n\
Display under the headring \"Non-debugging symbols:\" are such global symbols as section symbols defined \n\
in other than the source file.\n"


};
/**********************/
/* "c33 help x /X"    */
/**********************/
#define C33_HELP_X_COMMAND_HELPMSG "\
x: Memory dump                                  [ICD2/ICD3/ICD4/ICD6/SIM/MON]\n\n\
usage: x [/[Length]Size] [Address]\n\
       Length:     Number of data items to display(in decimal)\n\
                   1 when omitted.\n\
       Size:       one of the following symbols that specify data size(in which units of data are displayed)\n\
                   b   In units of byts\n\
                   h   In units of half words\n\
                   w   In units of words\n\
       Address:    Address from which start displaying data8decimal,dexadecimal,or symbol)\n\
       Conditions: 0<=Length<=0xffffffff,0<=Address<=0xffffffff\n\n\
example:\n\
(gdb)x\n\
(gdb)x /b 0\n\
(gdb)x /b 1
(gdb)x /16h _START_text
(gdb)x /4w 0
(gdb)x /w &i
(gdb)x /w i \n"


#define C33_DEV_COMMAND_HELPMSG  "\
(1)c33 debug_icd number \n\
    number  : 1  display pc trace idle cycle. \n\
              2  start output USB logfile(\"c:/gdb_debug.txt\"). \n\
              4  stop output USB logfile. \n\
              6  display target CPU. \n\
              7  bus trace data dump enable. \n\
              8  bus trace data dump disable. \n\
              9  pc and bus trace data dump file(pcbustrc.dmp\") enable. When \"c33 otd 1\" command\n\
             10  pc and bus trace data dump file disable. \n\
             11  output bus trace dump file(\"bustrc.dmp\") enable. When \"c33 otd 0/1\" command\n\
             12  output bus trace dump file disable. \n\
             13  output program execution message display enable.\n\
             14  output program execution message display disable.\n\n\
(2)c33 mmu \n\
    Please refer \"c33 help c33 mmu\". \n"
             
/****************************************************************************/
/* Format        : void c33_Help(int iParamNum,char *pCmdName1,char *pCmdName2) */
/* Arguments                                                                */
/*  int iParamNum      : Parameter number                                   */
/*  char *pCmdName1    : Group number or "c33" or Command name              */
/*  char *pCmdName2    : c33 command                                        */
/* Return value  : void                                                     */
/* Explanation   : command usage display.                                   */
/*      (gdb)c33 help                                                       */
/*      (gdb)c33 help 0                                                     */
/*      (gdb)c33 help c33 rsth                                              */
/*      (gdb)c33 help target                                                */
/****************************************************************************/
void
c33_Help(int iParamNum,char *pCmdName1,char *pCmdName2)
{
    int i;

    /* Check number of parameter */
    if ( iParamNum != 1 && iParamNum != 2 && iParamNum != 3 )
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }
    if ( iParamNum == 1 ) {
        /* all group display */
        printf_filtered( C33_HELP_ALL_HELPMSG );
        return;
    }

    if( iParamNum == 2 )
    {
        /* x command */
        if (strcmp( pCmdName1,"x" ) == 0 )
        {
            printf_filtered( C33_HELP_X_COMMAND_HELPMSG );
            return;
        }
        /* set xxx commands */
        if (strcmp( pCmdName1,"set" ) == 0 )
        {
            printf_filtered( C33_HELP_SET_COMMAND_HELPMSG );
            return;
        }
        /* info xxx commands */
        if (strcmp( pCmdName1,"info" ) == 0 )
        {
            printf_filtered( C33_HELP_INFO_COMMAND_HELPMSG );
            return;
        }
        /* group and gdb core command */
        for( i = 0; i < C33_MAX_CORE_COMMAND_NAME; ++i )
        {
            if( strcmp( c33_core_command_name[i],pCmdName1 ) == 0 )
            {
                printf_filtered( c33_core_command_usage[i] );
                return;
            }
        }
        c33_error( C33_ERR_COMMAND_INVALID );
    }
    else
    {
        if( strcmp( pCmdName1,"c33" ) == 0 )
        {
            /* c33 command */
            for( i = 0; i < C33_MAX_COMMAND_NAME; ++i )
            {
                if( strcmp( c33_command_name[i],pCmdName2 ) == 0 )
                {
                    printf_filtered( c33_command_usage[i] );
                    return;
                }
            }
            c33_error( C33_ERR_COMMAND_INVALID );
        }
        else if (strcmp( pCmdName1,"set" ) == 0 )
        {
            /* set command */
            for( i = 0; i < C33_MAX_SET_COMMAND_NAME; ++i )
            {
                if( strcmp( c33_set_command_name[i],pCmdName2 ) == 0 )
                {
                    printf_filtered( c33_set_command_usage[i] );
                    return;
                }
            }
            c33_error( C33_ERR_COMMAND_INVALID );
        }
        else if (strcmp( pCmdName1,"info" ) == 0 )
        {
            /* info command */
            for( i = 0; i < C33_MAX_INFO_COMMAND_NAME; ++i )
            {
                if( strcmp( c33_info_command_name[i],pCmdName2 ) == 0 )
                {
                    printf_filtered( c33_info_command_usage[i] );
                    return;
                }
            }
            c33_error( C33_ERR_COMMAND_INVALID );
        }
        else
        {
            c33_error (C33_ERR_COMMAND_INVALID);
        }
    }
}
/****************************************************************************/
/* Format        : void c33_Help_dev(void);                                 */
/* Arguments                                                                */
/*  int iParamNum      : Parameter number                                   */
/*  char *pCmdName1    : Group number or "c33" or Command name              */
/*  char *pCmdName2    : c33 command                                        */
/* Return value  : void                                                     */
/* Explanation   : command for development usage display.                   */
/****************************************************************************/
void
c33_Help_dev()
{
    printf_filtered( C33_DEV_COMMAND_HELPMSG );
}
