/* ICD interface for the EPSON C33 for GDB, the GNU debugger.
   Copyright 1996, 1997, 1998, 1999, 2000, 2001, 2002
   Free Software Foundation, Inc.

   This file is part of GDB.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.	*/

//#define C33_DEBUG 1	/* if yor are debug command,undelete this. */

#include "defs.h"
#include "gdb_string.h"
#include "frame.h"
#include "symtab.h"
#include "inferior.h"
#include "breakpoint.h"
#include "symfile.h"
#include "target.h"
#include "objfiles.h"
#include "gdbcore.h"
#include "value.h"
#include "command.h"
#include "serial.h"
#include <string.h>
#include "gdb_assert.h"
#include <signal.h>

#include <tcl.h>
#include "c33common.h"
#include "c33icd.h"
#include <stdio.h>

/* Prototypes for local functions */
static void c33icd_files_info (struct target_ops *ignore);
static void c33icd_open (char *name, int from_tty);
static void c33icd_close (int quitting);
static void c33icd_store_registers (int regno);
static void c33icd_fetch_registers (int regno);
static void c33icd_prepare_to_store (void);
static ptid_t c33icd_wait (ptid_t pid, struct target_waitstatus *status);
static int	c33icd_Stdio(void);
static void c33icd_resume (ptid_t pid, int step,enum target_signal siggnal);
static void c33_SetBusMode(void);		/*add T.Tazaki 2004/04/19 */
static void c33_SetMMUMode(int iFlag);		/*add T.Tazaki 2004/08/03 */
static void c33icd_stop (void);
static void c33icd_kill (void);
static int	c33icd_insert_breakpoint (CORE_ADDR addr, char *contents_cache);
static int	c33icd_remove_breakpoint (CORE_ADDR addr, char *contents_cache);
static void c33icd_detach (char *args, int from_tty);
static void c33icd_mourn (void);
static int	c33icd_xfer_memory (CORE_ADDR memaddr, char *myaddr, int iLen, int iReadWrite, struct mem_attrib *attrib, struct target_ops *pTarget);
static void c33icd_ops_init (void);
static void c33_IcdStopCtrl_C (int iSignal);
static void c33icd_BusBreakCount_display(void);	/* add T.Tazaki 2004/04/21 */

static void c33_IcdMakeBCC(unsigned char *pucSendData);
static int c33_IcdMakeMessageID (int iID, unsigned char* pucSendData);
static int c33_IcdMakeMessageReset (int iID, unsigned char* pucSendData, int iType);
static int c33_IcdMakeMessageGo (int iID, unsigned char* pucSendData, int iFlag, int iAddr);
static int c33_IcdMakeMessageStep (int iID, unsigned char* pucSendData, int iFlag);
static int c33_IcdMakeMessageRegWrite (int iID, unsigned char* pucSendData);
static int c33_IcdMakeMessageSpRegWrite (int iID, unsigned char* pucSendData);
static int c33_IcdMakeMessageDataRead(int iID, unsigned char* pucSendData, int iAddr, int iLen ,int iUnit);
static int c33_IcdMakeMessageDataWrite(int iID, unsigned char* pucSendData, int iAddr, int iLen ,int iUnit, unsigned char* pMyAddr);
static int c33_IcdMakeMessageParlDataWrite(int iID, unsigned char* pucSendData, int iAddr, int iLen ); /* add T.Tazaki 2003/09/30 */
static int c33_IcdMakeMessageSoftPcBreakSet(int iID, unsigned char* pucSendData, int iAddr, int iNum);
static int c33_IcdMakeMessageSoftPcBreakClear(int iID, unsigned char* pucSendData, int iNum);
static int c33_IcdMakeMessageHardPcBreakSet(int iID, unsigned char* pucSendData, int iAddr);
static int c33_IcdMakeMessageHardPcBreak2(int iID, unsigned char* pucSendData, int iAddr);
static int c33_IcdMakeMessageDataFill(int iID, unsigned char* pucSendData, int iType, int iStartAddr, int iSize , int iData);
static int c33_IcdMakeMessageDataMove(int iID, unsigned char* pucSendData, int iType, int iSrc, int iDst, int iSize);
static int c33_IcdMakeMessageDataBreak(int iID, unsigned char* pucSendData, int iAddr, int iType);
static int c33_IcdMakeMessageFlashSet(int iID, unsigned char* pucSendData, int iFlashStartAddr, int iFlashEndAddr, int iEraseRoutineAddr, int iWriteRoutineAddr);
static int c33_IcdMakeMessageFlashErase(int iID, unsigned char* pucSendData, int iCtrlReg, int iStartBlock, int iEndBlock);
static int c33_IcdMakeMessageTraceReadData(int iID, unsigned char* pucSendData, unsigned long ulStartCycle, unsigned long ulReadSize);
static int c33_IcdMakeMessageTraceTrgMode(int iID, unsigned char* pucSendData, struct c33_icd_trace_mode* pc33TraceMode);
static int c33_IcdMakeMessageClockFunctionSet(int iID, unsigned char* pucSendData,int iClockFunction);
static int c33_IcdVer3MakeMessageFlashErase(int iID, unsigned char* pucSendData, int iEraseObject);
static int c33_IcdVer3MakeMessageFlashLoad(int iID, unsigned char* pucSendData, int iKind, unsigned long ulLoadAddress, unsigned long ulLen, unsigned char *pucLoadData);
static int c33_IcdVer3MakeMessageFlashCopy(int iID, unsigned char* pucSendData, unsigned long ulDestAddress, unsigned long ulSrcAddress,unsigned long ulSize);
static int c33_IcdVer3MakeMessageFlashDataInfoSet(int iID, unsigned char* pucSendData, unsigned long ulIcdDataAddress, unsigned long ulCpuDataAddress,unsigned long ulSize,
							unsigned long ulEraseStartBlock, unsigned long ulEraseEndBlock,unsigned long ulEraseParam, unsigned char *pComment );
static int c33_IcdVer3MakeMessageFlashEntryInfoSet(int iID, unsigned char* pucSendData, unsigned long ulIcdProgAddress, unsigned long ulCpuProgAddress,unsigned long ulProgSize,
							unsigned long ulEraseEntryAddress, unsigned long ulWriteEntryAddress, char *pComment );
static int c33_IcdVer3MakeMessageDebugBaseSet(int iID, unsigned char* pucSendData );
static int c33_IcdVer3MakeMessageOscWaitSet(int iID, unsigned char* pucSendData , unsigned long ulOscWait);	/* add T.Tazaki 2005/06/08 */
static int c33_IcdVer3MakeMessageAreaBreakSet(int iID, unsigned char* pucSendData );
static int c33_IcdVer3MakeMessageBusBreakSet(int iID, unsigned char* pucSendData );
static int c33_IcdVer3MakeMessageBusTraceSet(int iID, unsigned char* pucSendData );
static int c33_IcdMakeMessageBusTraceReadData(int iID, unsigned char* pucSendData, unsigned long ulStartCycle, unsigned long ulReadCycle);
static int c33_IcdMakeMessageSetTimerBreak(int iID, unsigned char* pucSendData, int iFlag, unsigned long ulTimer ); /* add T.Tazaki 2007/07/03 */

static int c33_IcdRecvVersionRead (int iID, unsigned char* pucRecvData);
static int c33_IcdRecvRegisterRead (int iID, unsigned char* pucRecvData);
static int c33_IcdRecvSpRegisterRead (int iID, unsigned char* pucRecvData);
static int c33_IcdRecvGo (int iID, unsigned char* pucRecvData);
static int c33_IcdRecvRunMon (int iID, unsigned char* pucRecvData);
static int c33_IcdRecvDataRead (int iID, unsigned char* pucRecvData, int iLen, unsigned char* pMyAddr);
static int c33_IcdRecvFlashErase (int iID, unsigned char* pucRecvData, int* piResult);
static int c33_IcdRecvGetTraceDataSize (int iID, unsigned char* pucRecvData, unsigned long* pulDataSize);
static int c33_IcdRecvTraceReadData (int iID, unsigned char* pucRecvData, unsigned long ulStartCycle, unsigned long ulReadSize, unsigned long ulCycleMax, unsigned char* pucDst);
static int c33_IcdRecvBusTraceReadData (int iID, unsigned char *pucRecvData, unsigned long ulStartCycle, unsigned long ulReadSize, unsigned long ulCycleMax, unsigned short *pusData);
static int c33_IcdStoreMessage(int iID, unsigned char* pucRecvData);
static int c33_IcdReadSerial(int iID, int iSize, unsigned char* pucRecvData);
static int c33_IcdRecvDataCheck(int iID, unsigned char* pucRecvData);
static int c33_IcdRecvFlashDataInfoGet (int iID, unsigned char* pucRecvData, struct c33_flash_writer_info *c33FlashWriterInfo);
static int c33_IcdRecvFlashEntryInfoGet (int iID, unsigned char* pucRecvData, struct c33_flash_writer_info *c33FlashWriterInfo);
static int c33_IcdRecvTargetReset (int iID, unsigned char* pucRecvData); /* add T.Tazaki 2007/07/03 */

static void 		 c33_IcdTraceModeDisplay(void);
static void 		 c33_IcdTraceInitData(void);
static int			 c33_IcdTraceGetData(void);
static int			 c33_IcdTraceAnalyze(void);
static int			 c33_IcdTraceGetDataSize(unsigned long *pulCycleMax);
static int			 c33_IcdTraceReadData(unsigned char* pucDst, unsigned long ulStartCycle, unsigned long ulReadSize, unsigned long ulCycleMax);
static int			 c33_IcdBusTraceGetData(void);		/* add T.Tazaki 2003/07/02 */
static int			 c33_IcdBusTraceReadData(unsigned short *pusData, unsigned long ulStartCycle, unsigned long ulReadCycle, unsigned long ulCycleMax);/* add T.Tazaki 2003/07/07 */
static void 		 c33_IcdTraceMake1Line(unsigned long ulTrcDataNumber, unsigned long ulCycleNumbar, char* pTraceStr);
static void			 c33_IcdBusMakeAddrData(unsigned long ulClock, char *cTraceAddr, char *cTraceData, char *cTraceInst, char *cTraceUnassem);	/* add T.Tazaki 2004/09/09 */
static int			 c33_IcdAreaBreakStatus_display(void);												/* add T.Tazaki 2004/04/22 */
static void 		 c33_IcdBusTraceMake1Line(unsigned long ulClock, char *cTraceStr);
static int 			 c33_IcdBusTraceCommon(FILE *fpTraceFile, int iMode, unsigned long ulStartCycle, unsigned long ulEndCycle);	/* add T.Tazaki 2004/04/22 */
static unsigned long c33_IcdTraceInstCycle(unsigned long ulCycle);
static void 		 c33_IcdTraceGetSourceCode(unsigned long ulAddr, char* pOut);
static void 		 c33_IcdTraceCalMaxNumber(void);
static int			 c33_IcdTraceAnalyzeDpco(void);
static int			 c33_IcdTraceAnalyzeAdvDpco(void);	/* add T.Tazaki 2003/07/04 */
static int			 c33_IcdTraceAnalyzeTrigger(void);
static int			 c33_IcdTraceAnalyzeRet(void);
static int			 c33_IcdTraceAnalyzeReti(void);
static int			 c33_IcdTraceAnalyzeMap(void);
static int			 c33_IcdTraceModify(void);			/* add T.Tazaki 2003/09/02 */
static int			 c33_IcdDebugDisp(void);			/* add T.Tazaki 2004/11/04 */
static void 		 c33_IcdTraceAnalyzeMakePcByDST(unsigned char ucMethod, unsigned long ulCycleCounter, unsigned char* ucDelayed, unsigned char* ucDpcoNum, unsigned long* ulPC, unsigned long* ulDelayedPC);
static unsigned long c33_IcdTrcComparePC(unsigned long ulPC, unsigned long ulCounter);
static unsigned long c33_IcdTrcCompCodeWithDST(unsigned long  ulCycleCounter, unsigned long ulMapTopAddr, unsigned long ulMapBottomAddr, unsigned char ucDpcoCounter, unsigned long *ulExpectNum);

static int c33_GetFileStartAddress( char *pFilename, unsigned long *ulFileStartAddress, unsigned long *ulMaxAddress, unsigned long *ulMaxLen );
static int SRecordToBin( char *buffer_address, unsigned char *dest_buf, unsigned long *ice_address, unsigned long *num );
static int htob( char *pointer );
static int CheckSum( char *buf_ptr, long length );
static void Type0Decode(char *pointer, char *end, unsigned char *string);
static void Type1Decode(char *pointer, char *end, unsigned char *dest, unsigned long *ice, unsigned long *num);
static void Type2Decode(char *pointer, char *end, unsigned char *dest, unsigned long *ice, unsigned long *num);
static void Type3Decode(char *pointer, char *end, unsigned char *dest, unsigned long *ice, unsigned long *num);

static int c33_IcdVer3AnalysisBusTraceData(void);											/* add T.Tazaki 2004/03/24 */
static void c33_Percent_Display( unsigned long ulMax,unsigned long ulCurrent );				/* add T.Tazaki 2004/10/15 */
static int c33_SetBusTraceData( unsigned long ulSetCycle, 									/* add T.Tazaki 2004/07/12 */
			 unsigned char ucBusMode, char cAddrData, unsigned long ulAddress, unsigned long ulData, int iAddressLength, int iDataLength,int iAddressComplete);

static void c33_IcdVer3BreakModeToToolbar(void);												/* add T.Tazaki 2005/05/18 */

static void c33_dummy( void );

static int			 c33_IcdUsbOpen(void);
static void 		 c33_IcdUsbClose(void);
static int			 c33_IcdUsbWrite(int iLen);
static int			 c33_IcdUsbRead(int iSize, int iTimeOut, unsigned char* pucRecvData);

static void c33_IcdProcessing(unsigned long ulWait);

static void c33_ClearAllSoftBreak( void );			/* add T.Tazaki 2005/05/11 */

/* static variable */
static struct target_ops c33icd_ops; /* Target function registration */
static struct serial *c33_desc; 	 /* Handle used to access serial I/O stream. */

static struct c33_icd_trace_data c33IcdTraceData; /* icd trace data structure */

static unsigned char ucSendData[C33_COMMAND_BUFFER_SIZE]; /* Command buffer */
static unsigned char ucRecvData[C33_COMMAND_BUFFER_SIZE]; /* Command buffer */

static int iDebugCommunication = 0;/* Protocol informaion output */
static int iDebugTrace = 0;/* Trace data dump display on console window , add T.Tazaki 2003/08/22 */
int g_iDebugRun = 0;  /* Program execution debug message output , add T.Tazaki 2005/04/27 */

static int iUsbLogCommunication = 0;/* Protocol informaion output  add T.Tazaki 2004/10/18 */

/* set default trace mode */
static int c33_IcdTraceOverwriteEnable(void);	/* add T.Tazaki 2004/04/05 */

/* global function */
extern unsigned long c33_CalcAbsAddr(unsigned long ulTargetAddr);

/* global variable */
extern struct c33_target_info c33TargetInfo; /* target information */
extern unsigned long g_ulClockCount[3];
extern unsigned long long g_ullClockCount;	/* add T.Tazaki 2007/07/03 for ICDV6 */

/* Save "c33 fls" parameter flash setup 	add T.Tazaki 2003/04/03 */
int save_iFlashSet = 0;
unsigned long save_ulFlashStartAddr;
unsigned long save_ulFlashEndAddr;
unsigned long save_ulEraseRoutineAddr;
unsigned long save_ulWriteRoutineAddr;

/* Not display watch command message by c33icd_wait 2'nd calling */
int g_resume_flag = 0;		/* 1=1'st call */

unsigned long g_ulCycleMax = 0;		/* add T.Tazaki 2005/01/31 */
int g_otx_1 = 0;					/* add T.Tazaki 2005/01/31 */

int g_iIcdLoadFirst = 0;	/* 1=High Speed program load external process routine write request add T.Tazaki 2003/07/29 */

int g_iParl_error_1st;		/* icdv2 parallel send timeout error message 1st display add T.Tazaki 2003/10/01 */

int g_iIcdSVflag = 0;		/* =%psr : SV flag	add T.Tazaki 2004/04/27 */

unsigned long g_save_start_pc = C33_INVALID_ADDR;
static unsigned long g_iTemp_brk_ng_addr = C33_INVALID_ADDR;	/* add T.Tazaki 2003/09/01 */
static unsigned long g_iSaveSkipAddr	 = C33_INVALID_ADDR;	/* add T.Tazaki 2003/10/14 */

/* >>>>> add iruma m.takeishi '03.12.03 */
static int iMonKeyBreak 				 = C33_MONSTATUS_OK;
/* <<<<< add iruma m.takeishi '03.12.03 */

static int g_iRegno;	/* add T.Tazaki 2003/12/05 */

static int g_iStepFirst = 0;	/* add T.Tazaki 2004/01/13 */
static unsigned long g_ulIdir;	/* save  %idir for Sp reg write 	add T.Tazaki 2004/01/19 */
static unsigned long g_ulDbbr;	/* save  %dbbr for Sp reg write 	add T.Tazaki 2004/01/19 */

unsigned long g_ulBus_Count_org = 0;	/* save ADV Bus Break Counter	add T.Tazaki 2004/04/21 */
long g_lBusBreakCnt = 0;				/* now Bus Break Counter		add T.Tazaki 2004/04/21 */

int	g_iID = 0;							/* add T.Tazaki 2004/05/11 : for USB STEP command hang up */
int g_iNow_OnUSBRead = 0;				/* add T.Tazaki 2004/05/11 : for USB STEP command hang up : =1 CheckCancel() by USBRead */
int g_iStep_Cancel = 0;					/* add T.Tazaki 2004/05/11 : for USB STEP command hang up : =1 Step canceled */
int g_reg_update_now = 0;				/* add T.Tazaki 2004/06/03 : 1=ICD update now (c33_reg_update) */

int 		  g_iMMU_Mode = 0;			/* add T.Tazaki 2004/08/03 : c33 mmu : = 1: mmu mode ON */
unsigned long g_ulMMU_SP =0;			/* add T.Tazaki 2004/08/03 : c33 mmu */
//int			g_iGoStepFirst = 1;		/* add T.Tazaki 2004/08/03 : c33 mmu */	/* del T.Tazaki 2005/02/07 */

/**********************************************/
/* Bus trace 1 line make					  */
/**********************************************/
static int g_bCompleteAddressFlag = C33_FALSE;			/* C33_TRUE:address complete output on	 add T.Tazaki 2004/07/13 */
static unsigned long g_ulSaveCompleteAddress = 0;		/* complete address 					 add T.Tazaki 2004/07/13 */

/**********************************************/ /* move 2004/11/04 T.Tazaki */
/* static delayed flag						  */
/**********************************************/
static char g_ucPC_Rel_Delayed = C33_FALSE;
static unsigned long g_ulSavePC;
static char g_ucPC_Abs_Delayed = C33_FALSE;
static unsigned long g_ulAbsSavePC;

/**********************************************/
/* Memory read mode 						  */
/**********************************************/
int g_iMemReadMode = 0;									/* 0=Normal,1=Exact  add T.Tazaki 2004/11/01 */

/**********************************************/
/* Target Reset Cancel						  */
/**********************************************/
int g_iSendCancel = 0;									/* 0=USB ID_GO No wait, 1=2 sec wait timeout */

/**********************************************/
/* time break mode 							  */
/**********************************************/	/* add T.Tazaki 2007/07/03 */
int g_iTimeBreak = 0;

/**********************************************/
/* Continue [Count],  ignore				  */
/**********************************************/

extern int g_iContinue_count;			/* add 2003/11/06 T.Tazaki */
extern int g_iIgnore_number;			/* add 2003/11/07 T.Tazaki */
extern int g_iIgnore_count;				/* add 2003/11/07 T.Tazaki */

/**********************************************/
/* Simulated I/O							  */
/**********************************************/

extern FILE *fpStdinfile;
extern FILE *fpStdoutfile;
extern int g_c33_std_in;				/* add 2003/03/10 T.Tazaki */
extern int g_c33_Index; 				/* add 2003/03/10 T.Tazaki */
extern char g_c33_stdin_string[1000];	/* add 2003/03/10 T.Tazaki */
extern int g_use_windows;				/* add 2003/03/10 T.Tazaki */
extern int g_c33_iSourceBreak;			/* add 2003/03/26 T.Tazaki */
extern int g_command_loop;				/* add 2003/03/26 T.Tazaki : log command ,1=command loop running */
extern int g_temp_brk;					/* add 2003/03/31 T.Tazaki : Temprary Soft PC break --> Hard PC break */
extern unsigned long g_ulTemp_brk_addr; 		   /* add 2003/03/31 T.Tazaki : Temprary Soft PC break --> Hard PC break */
extern int g_SoftPcBreakTable[ 510 ];	/* add 2003/04/01 T.Tazaki : Temprary Soft PC break --> Hard PC break */
extern int g_SoftPcBreakTable_Index;	/* add 2003/04/01 T.Tazaki : Temprary Soft PC break --> Hard PC break */
extern int giFrameClear;				/* add 2003/04/28 M.Takeishi : if giFrameClear = 1, and c33_Gdbtk_Update  */
										/*				  then, flush_cashed_frames() is called. */
extern int g_iSet_Command;				/* add T.Tazaki 2004/06/14 : set_command() in printcmd.c set $pc=xxxxx -> source window update flag */

/* add 2003/05/07 T.Tazaki for finish commnad Temprary break >>> */
extern void c33_GetBreakPoint(void);
extern int g_WatchNum;					/* add 2003/06/30 T.Tazaki */
extern char g_What[300];				/* add 2003/06/30 T.Tazaki */
extern int g_HardPcBreakTable[ 510 ];
extern int g_HardPcBreakTable_Index;
extern int g_WriteWatchBreakAddr;
extern int g_WriteWatchBreakAddrFlag;
extern int g_ReadWatchBreakAddr;
extern int g_ReadWatchBreakAddrFlag;
extern int g_WriteReadWatchBreakAddr;
extern int g_WriteReadWatchBreakAddrFlag;
/* add 2003/05/07 T.Tazaki for finish commnad Temprary break <<< */

extern void c33_delete_temp_breakpoint (unsigned long ulPC);	/* add 2003/07/08 T.Tazaki */

extern int g_iTraceMode;	/* 1=Trace mode ,c33_Disasm():c33-dis.c */ /* 2003/05/19 T.Tazaki */

extern int g_iSamePCMode;	/* add 2003/09/22 T.Tazaki */
extern int g_iNextCommand;	/* add 2003/11/06 T.Tazaki */

/* Tagget do not destroy Map information flag */
static int g_c33icd_open = 0;

/* Flash writer c33 fwld,fwld No comment default string */
static char g_NoCommentData[3] = "  ";

unsigned char g_ucAdvFlag;	/* =0x00:STD,=0x41:ADV,=0x50:PE  add T.Tazaki 2003/11/18 */

int g_c33icd_das = 0;		/* =1:"c33 das" command executed add T.Tazaki 2003/06/11 */

/* Paralell write */
int  g_iParallel_mode = 0;		  /* 1:icdv2 fast write by parallel port mode		  */
int  g_iParallel_LPTNo = 0; 	  /* lptX (1 or 2)									  */

/* Focus to Console window */
extern int g_iTargetOpen_end;			/* 1=Target command end (top.c): add T.Tazaki 2003/10/07 */

//extern int g_iSet_PE_reg_mode;			/* 1=PE mode set by Parameter file.   del 2007/07/12 T.Tazaki */

extern int g_ulCallPC;											 /* add T.Tazaki 2004/08/25 */
extern int g_ulStartPC; 										 /* add T.Tazaki 2004/08/25 */

/* PC + Normal Bus trace mode */
int g_iPC_Bus_Trace = 0;										/* add T.Tazaki 2004/09/13 */
int g_iPC_Bus_same = 0;											/* add T.Tazaki 2004/09/15 */
unsigned long g_ulClockBefore = 0;								/* add T.Tazaki 2004/09/15 */

static int g_iAddrCompeteIndex = (-1);							/* add T.Tazaki 2004/10/04 */
static unsigned int g_ulBeforePC = C33_ICD_TRACE_PC_INVALID;	/* add T.Tazaki 2004/10/04 */

extern void c33_InitCancel(void);
extern int c33_CheckCancel( int iCount );
extern void c33_EndCancel(void);

/**********************************************/
/* Percentage display						  */
/**********************************************/
int g_iPercent_First = 1;
int g_iPercentFlag[10];

/* add T.Tazaki 2004/12/01 >>> */
/**********************************************/
/* Trace default read cycle 				  */
/**********************************************/
static int g_iPC_TraceParam = 1;	/* change T.Tazaki 2005/01/12 */
static int g_iBUS_TraceParam = 1;	/* change T.Tazaki 2005/01/12 */
static int g_iCycleOver10000 = 0;
/* add T.Tazaki 2004/12/01 <<< */

/* add T.Tazaki 2004/12/14 >>> */
/**********************************************/
/* ICD SIM I/O On the Fly Error 			  */
/**********************************************/
int g_iMonErr = 0;
/* add T.Tazaki 2004/12/14 <<< */

static int g_iBusTraceRange = 0;	/* add T.Tazaki 2005/03/24 */

/**********************************************/
/* CPU Analysis for c33 das 				  */
/**********************************************/
int g_iDasCpu = 0;			/* = 1:set c33 das cpu type , =0:No set */	/* add T.Tazaki 2005/04/01 */

/**********************************************/
/* STEP + GO : for Current PC break setting	  */
/**********************************************/
extern int g_iSoftBreakPointIsSamePC;						/* add 2005/05/10 T.Tazaki */
extern unsigned long g_ulSoftBreakPointIsSamePC_addr;		/* add 2005/05/10 T.Tazaki */

/**********************************************/
/* gdb multi open							  */
/**********************************************/	/* add T.Tazaki 2005/05/16 */
int g_iUsbNo = 99;				/* 0="usb",1="usb2",2="usb3" */

int g_iStdin_EOF = 0;			/* =1:stdin file inout EOF */

/**********************************************/
/* PC Trace DIPSW 							  */
/**********************************************/	/* add T.Tazaki 2005/07/27 */
int g_iPCTrace = 1;

int g_iTargetResetDown = 0;				/* add T.Tazaki 2005/07/28 */

/**********************************************/
/* fle timer       							  */
/**********************************************/	/* add T.Tazaki 2006/05/16 */
static int g_iFleTimerFlag = 0;
static int g_iFleTimer = 0;

/**********************************************/
/* trace display / file mode				  */
/**********************************************/	/* add T.Tazaki 2007/07/25 */
static int g_iTraceDisplayMode = 1;

/**********************************************/
/* gnu33ide.dll 							  */
/**********************************************/
#ifdef __CYGWIN__				/* add T.Tazaki 2005/09/14 */
#include <windows.h>

void c33_OSMon(void);
static int  c33_DllMemRead( unsigned long ulAddress, unsigned long ulSize );
static int  c33_DllMemWrite( unsigned long ulAddress, unsigned long ulSize );
static int  c33_DllGetSymbol( unsigned long ulAddress );
static int  c33_DllGetIL( void );

WORD ( __stdcall *dll33_ClearCommandUpdate) ( void );/* add T.Tazaki 2006/01/12 */
WORD ( __stdcall *dll33_GdbEnable) ( void );
WORD ( __stdcall *dll33_GdbDisable) ( void );
WORD ( __stdcall *dll33_GdbRun) ( void );			/* add T.Tazaki 2005/12/08 */
WORD ( __stdcall *dll33_GdbStop) ( void );			/* add T.Tazaki 2005/12/08 */
WORD ( __stdcall *dll33_addGdbGoCounter) ( void );
WORD ( __stdcall *dll33_checkCommand) ( int *iCommand, unsigned char *pucAddress, unsigned long *pulAddress,
								  unsigned long *pulSize );
WORD ( __stdcall *dll33_setMem) ( unsigned long ulSize, unsigned char *pucData );
WORD ( __stdcall *dll33_getMem) ( unsigned long ulSize, unsigned char *pucData , int iParent);
WORD ( __stdcall *dll33_setSymbol) ( unsigned char *pucSymbol, int iResult );
WORD ( __stdcall *dll33_setIL) ( unsigned char ucIL );

HINSTANCE hIdeDll = NULL;
int g_GdbEnable = 0;
extern  unsigned long c33_CnvSymbolToAddress (char *exp);
extern  int c33_CnvAddressToSymbol (unsigned long ulAddress, unsigned char *pucSymbol );

int g_OSMon_Semaho = 0; /* add T.Tazaki 2005/10/27 */
#endif

/* for ICDV6 add T.Tazaki 2007/07/03 */
/**********************************************/
/* ICDV6 function							  */
/**********************************************/
static struct target_ops c33icd6_ops; /* Target function registration */
static void c33icd6_open (char *name, int from_tty);
static void c33icd6_ops_init (void);
int g_iICD_kind = C33_ICDV4;
static unsigned char g_ucTargetReset_result = 0;


/****************************************************************************/
/* Format		 : static void c33icd_6open (char *name, int from_tty)		*/
/* Arguments																*/
/*	char *name	 : Device name												*/
/*	int from_tty : 1 to indicate this is from the console.					*/
/* Return value  : void 													*/
/* Explanation	 : It connects with ICDV6 and ICDV6 is initialized.			*/
/****************************************************************************/
static void
c33icd6_open (char *name, int from_tty)
{
	g_iICD_kind = C33_ICDV6;
	c33icd_open ( name, from_tty);
}

/****************************************************************************/
/* Format		 : static void c33icd_open (char *name, int from_tty)		*/
/* Arguments																*/
/*	char *name	 : Device name												*/
/*	int from_tty : 1 to indicate this is from the console.					*/
/* Return value  : void 													*/
/* Explanation	 : It connects with ICD and ICD is initialized. 			*/
/****************************************************************************/
static void
c33icd_open (char *name, int from_tty)
{
	char cDevName[C33_COMMAND_MAX + 1];
	int iComWait = 0;
	int i_das_temp = 0;
	int iResult = C33_NO_ERR;
	unsigned long ulProcessorID;
	unsigned long ulTTBR;
	extern Tcl_Interp *gdbtk_interp;	

	/* Inspection of a device name. */
	if (name == 0)
	{
		c33_error (C33_ERR_MSG_DEVICE_NAME);
		return;
	}
	else
	{
		sscanf(name, "%s %d", &cDevName[0], &iComWait);
	}
	if (iComWait < 0 || iComWait > 20)
	{
		iComWait = 0;
	}

	g_c33icd_open = 1;		/* Do Not execute c33_ClearMap() */
	target_preopen (from_tty);
	if( g_iICD_kind == C33_ICDV6 )		/* for ICDV6 add T.Tazaki 2007/07/02 */
	{
		unpush_target (&c33icd6_ops);	/* for ICDV6 add T.Tazaki 2007/07/02 */
	}
	else
	{
		unpush_target (&c33icd_ops);
	}
	

	g_c33icd_open = 0;


	if (from_tty)
	{
		/* "C33 ICD33 debugging" message */
		puts_filtered (C33_MSG_TARGET_ICD_NAME);
	}

	if( g_iICD_kind == C33_ICDV6 )		/* for ICDV6 add T.Tazaki 2007/07/02 */
	{
		push_target (&c33icd6_ops);		/* for ICDV6 add T.Tazaki 2007/07/02 */
	}
	else
	{
		push_target (&c33icd_ops);	  /* Change of a target */
	}
	target_terminal_init ();

	/* add T.Tazaki 2005/05/16 >>>*/
	if ( strcmp(cDevName, "usb") == 0 )
	{
		g_iUsbNo = 0;
	}
	else if ( strcmp(cDevName, "usb2") == 0 )
	{
		g_iUsbNo = 1;
	}
	else if ( strcmp(cDevName, "usb3") == 0 )
	{
		g_iUsbNo = 2;
	}
	if ( g_iUsbNo <= 2 )
	{
	/* add T.Tazaki 2005/05/16 <<<*/
		if ( c33_IcdUsbOpen() != C33_NO_ERR )
		{
			c33_error (C33_ERR_MSG_USB_DRV);
			return;
		}
		/* A setup of a usb port was completed. */
		c33TargetInfo.iIcd_open = C33_USB_OPEN;
	}
	else
	{
		/* add T.Tazaki 2006/05/16 : change "comX" --> "/dev/comX" for new cygwin1.dll >>>*/
		if ( strcmp(cDevName, "com1") == 0 )
		{
			strcpy(cDevName,"/dev/com1");
		}
		else if ( strcmp(cDevName, "com2") == 0 )
		{
			strcpy(cDevName,"/dev/com2");
		}
		else if ( strcmp(cDevName, "com3") == 0 )
		{
			strcpy(cDevName,"/dev/com3");
		}
		else if ( strcmp(cDevName, "com4") == 0 )
		{
			strcpy(cDevName,"/dev/com4");
		}
		else if ( strcmp(cDevName, "com5") == 0 )
		{
			strcpy(cDevName,"/dev/com5");
		}
		else if ( strcmp(cDevName, "com6") == 0 )
		{
			strcpy(cDevName,"/dev/com6");
		}
		else if ( strcmp(cDevName, "com7") == 0 )
		{
			strcpy(cDevName,"/dev/com7");
		}
		else if ( strcmp(cDevName, "com8") == 0 )
		{
			strcpy(cDevName,"/dev/com8");
		}
		/* add T.Tazaki 2006/05/16 <<<*/

		
		/* Open and initialize the serial port. */
		c33_desc = serial_open (cDevName);

		if (c33_desc == NULL)
		{
			perror_with_name (cDevName);
		}

		/* Setup of a baud rate and setup of 2 stop bit */
		if (baud_rate == -1)
		{
			/* -nw mode only */
			baud_rate = 115200;   /* set defualt baud rate */
		}
		if ((baud_rate == 38400 || baud_rate == 115200) &&
			serial_setbaudrate(c33_desc, baud_rate) == 0 &&
			serial_setstopbits(c33_desc, SERIAL_1_AND_A_HALF_STOPBITS) == 0)
		{
		}
		else{
			serial_close (c33_desc);
			c33_error (C33_ERR_MSG_SERIAL_SETUP);
			return;
		}

		/* Setup of none parity and 8bit data */
		serial_raw (c33_desc);

		/* A setup of a serial port was completed. */
		c33TargetInfo.iIcd_open = C33_COM_OPEN;

		/* serial flush 2004/08/19 T.Tazaki >>> */
		serial_flush_input(c33_desc);
		serial_flush_output(c33_desc);
		/* serial flush 2004/08/19 T.Tazaki <<< */

	}

	/* Initialization of C33 */
	c33_wait_sec(iComWait);

	if (c33_Initialize() != C33_NO_ERR)
	{
		if (c33TargetInfo.iIcd_open == C33_COM_OPEN)
		{
			serial_close (c33_desc);
		}
		c33TargetInfo.iIcd_open = C33_NOT_OPEN;
		c33_error (C33_ERR_MSG_ICD_INIT);
		return;
	}
	else
	{
		/* Display icd software version */
		/* change T.Tazaki 2005/09/30 >>> */
//		printf_filtered (C33_MSG_TARGET_ICD_HARD_VER, c33TargetInfo.iVerRdIcdHardVer);
		printf_filtered (C33_MSG_TARGET_ICD_HARD_VER, (c33TargetInfo.iVerRdIcdHardVer & 0xf0) >> 4, c33TargetInfo.iVerRdIcdHardVer & 0xf);
		/* change T.Tazaki 2005/09/30 <<< */

		printf_filtered (C33_MSG_TARGET_ICD_SOFT_VER, (c33TargetInfo.iVerRdIcdFirmVer & 0xf0) >> 4, c33TargetInfo.iVerRdIcdFirmVer & 0xf);
	}

	c33TargetInfo.ucBrkFunc = 0;


	if ((c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) == C33_ICD_VER2 || 
		(c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) == C33_ICD_MON)	/* icd v2 or MON ? */
	{

		/*==============================================================*/
		/* ICD V2 /@MON  CPU Auto analysis	(STD/PE)					*/
		/*==============================================================*/

		c33_Cmd_ReadWrite(C33_READ, C33_ADDR_PRODUCT, C33_BYTE, C33_BYTE, &c33TargetInfo.ucAdvanced);

		c33TargetInfo.ucAdvanced &= C33_PRODUCT_ADV;
		if (c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV)	/* bit7=1:extend CPU=>PE ? */
		{
			c33TargetInfo.ucAdvanced = C33_PRODUCT_PE;		/* PE  */
//			c33_Cmd_ReadWrite(C33_READ, 0x20000000, C33_WORD, C33_WORD, (char*)&c33TargetInfo.ulBootAddr);
			c33_Cmd_ReadWrite(C33_READ, 0xc00000, C33_WORD, C33_WORD, (char*)&c33TargetInfo.ulBootAddr);	/* bug fix 2004/08/09 T.Tazaki */

			if( g_use_windows == 1 )	/* GUI Mode ? */
			{
				Tcl_SetVar( gdbtk_interp,"c33_reg_mode","PE",TCL_GLOBAL_ONLY );		
				/* Registers window update */
				g_reg_update_now = 1;	/* add T.Tazaki 2004/06/03 */
				Tcl_Eval( gdbtk_interp, "c33_reg_update" ); /* gdbtcl/srcbar.itcl */
				g_reg_update_now = 0;	/* add T.Tazaki 2004/06/03 */

				/* Display Target and CPU name : add T.Tazaki 2004/05/24 <<< */
				Tcl_SetVar( gdbtk_interp,"c33_target_name","ICD2:PE",TCL_GLOBAL_ONLY );	/* "ICD2+PE"-->"ICD2:PE" T.Tazaki 2005/03/01 */
				Tcl_Eval( gdbtk_interp, "c33_cpumode_set" ); 
				/* Display Target and CPU name : add T.Tazaki 2004/05/24 >>> */
			}
		}
		else
		{
			/* CPU is a standard macro */
			c33TargetInfo.ucAdvanced = C33_PRODUCT_STD;		/* STD */
			
			/* change T.Tazaki 2005/06/20 >>> */
//			c33_Cmd_ReadWrite(C33_READ, C33_ADDR_TTBR, C33_WORD, C33_WORD, (char*)&c33TargetInfo.ulBootAddr);
			c33_Cmd_ReadWrite(C33_READ, C33_ADDR_TTBR, C33_WORD, C33_WORD, (char*)&ulTTBR );
			c33_Cmd_ReadWrite(C33_READ, ulTTBR, C33_WORD, C33_WORD, (char*)&c33TargetInfo.ulBootAddr);
			/* change T.Tazaki 2005/06/20 <<> */

			if( g_use_windows == 1 )	/* GUI Mode ? */
			{
				/* Display Target and CPU name : add T.Tazaki 2004/05/24 <<< */
				if ((c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) == C33_ICD_VER2 )
				{
					Tcl_SetVar( gdbtk_interp,"c33_target_name","ICD2:STD",TCL_GLOBAL_ONLY );
				}
				else
				{
					Tcl_SetVar( gdbtk_interp,"c33_target_name","MON:STD",TCL_GLOBAL_ONLY );
				}
				Tcl_Eval( gdbtk_interp, "c33_cpumode_set" ); 
				/* Display Target and CPU name : add T.Tazaki 2004/05/24 >>> */
			}
		}
	}
	else if ((c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) == C33_ICD_VER3 )  /* icd v3 ? */
	{

//		if( g_c33icd_das == 0 )	/* c33 das no executeH 	del T.Tazaki 2005/04/01 */
		if( g_c33icd_das == 0 || g_iDasCpu == C33_AUTO )	/* c33 das no execute OR c33 das : no CPU? 	add T.Tazaki 2005/04/01 */
		{
			/* c33 das  */

			/*==============================================================*/
			/* ICD V3 		CPU Auto analysis	(STD/PE/ADV)				*/
			/*==============================================================*/
			c33_Cmd_ReadWrite(C33_READ, C33_ADDR_PRODUCT, C33_BYTE, C33_BYTE, &c33TargetInfo.ucAdvanced);

			c33TargetInfo.ucAdvanced &= C33_PRODUCT_ADV;
			if (c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV)
			{

				/* Special Register Read */
				
				/* Transmission of a message. */
				if (iResult == C33_NO_ERR){
					iResult = c33_SendMessage(C33_ID_SP_REG_READ, 0, 0, 0, 0, 0, 0, 0);
				}
				/* Reception of a message. */
				if (iResult == C33_NO_ERR){
					iResult = c33_RecvMessage(C33_ID_SP_REG_READ, 0, 0, 0, 0, 0);
				}
				if (iResult != C33_NO_ERR)
				{
					c33_ErrorMsg();
				}

				/* save %idir, %dbbr */
				g_ulIdir = read_register (IDIR_REGNUM);
				g_ulDbbr = read_register (DBBR_REGNUM);

				ulProcessorID = (read_register (IDIR_REGNUM) & 0xff000000 ) >> 24;

				/* update Regster window  */
				if ( ulProcessorID == C33_PROCESSOR_ADV )
				{
					if( g_use_windows == 1 )	/* GUI Mode ? */
					{
						Tcl_SetVar( gdbtk_interp,"c33_reg_mode","ADV",TCL_GLOBAL_ONLY );		
						/* Registers window update */
						g_reg_update_now = 1;	/* add T.Tazaki 2004/06/03 */
						Tcl_Eval( gdbtk_interp, "c33_reg_update" ); /* gdbtcl/srcbar.itcl */
						g_reg_update_now = 0;	/* add T.Tazaki 2004/06/03 */
					}
				}
				else if ( ulProcessorID == C33_PROCESSOR_PE )
				{
					c33TargetInfo.ucAdvanced = C33_PRODUCT_PE;

					if( g_use_windows == 1 )	/* GUI Mode ? */
					{
						Tcl_SetVar( gdbtk_interp,"c33_reg_mode","PE",TCL_GLOBAL_ONLY );		
						/* Registers window update */
						g_reg_update_now = 1;	/* add T.Tazaki 2004/06/03 */
						Tcl_Eval( gdbtk_interp, "c33_reg_update" ); /* gdbtcl/srcbar.itcl */
						g_reg_update_now = 0;	/* add T.Tazaki 2004/06/03 */
					}
				}
				
				/* CPU is an advance macro */
	//			c33TargetInfo.ulBootAddr = (read_register (PC_REGNUM) & C33_PC_MASK);
//	#if 0	/* del T.Tazaki 2004/06/29 : Do not get TTBR address */	/* add T.Tazaki 2005/06/20 */
				ulTTBR = read_register (TTBR_REGNUM);
				c33_Cmd_ReadWrite(C33_READ, ulTTBR, C33_WORD, C33_WORD, (char*)&c33TargetInfo.ulBootAddr);
//	#endif
//				c33TargetInfo.ulBootAddr = 0x78000;		/* set no user section address T.Tazaki 2004/06/29 */ /* del T.Tazaki 2005/06/20 */
				
				/* Get Info of bus trace, bus break, erea break */
				
//				if( g_c33icd_das == 0 )	/*del 2004/12/27 T.Tazaki */
//				{
					/* default */
					c33_Cmd_ReadWrite(C33_READ, C33_ADDR_BRK_FUNC, C33_BYTE, C33_BYTE, &c33TargetInfo.ucBrkFunc);
//				}
//				else
//				{	/*del 2004/12/27 T.Tazaki */
//					c33_Cmd_ReadWrite(C33_READ, c33TargetInfo.ulDebugBase_rom_addr + 0x1807c, C33_BYTE, C33_BYTE, &c33TargetInfo.ucBrkFunc);
//				}
			}
			else
			{
				/* CPU is a standard macro */
				c33TargetInfo.ucAdvanced = C33_PRODUCT_STD;		/* STD */
				/* change T.Tazaki 2005/06/20 >>> */
//				c33_Cmd_ReadWrite(C33_READ, C33_ADDR_TTBR, C33_WORD, C33_WORD, (char*)&c33TargetInfo.ulBootAddr);
				c33_Cmd_ReadWrite(C33_READ, C33_ADDR_TTBR, C33_WORD, C33_WORD, (char*)&ulTTBR );
				c33_Cmd_ReadWrite(C33_READ, ulTTBR, C33_WORD, C33_WORD, (char*)&c33TargetInfo.ulBootAddr);
				/* change T.Tazaki 2005/06/20 <<> */
			}
		}
		else
		{	
			/* add T.Tazaki 2004/12/27 >>> */
			
			/* c33 das L */

			if( c33TargetInfo.ucCpuType != C33_STD )
			{
				/* Special Register Read */
				
				/* Transmission of a message. */
				if (iResult == C33_NO_ERR){
					iResult = c33_SendMessage(C33_ID_SP_REG_READ, 0, 0, 0, 0, 0, 0, 0);
				}
				/* Reception of a message. */
				if (iResult == C33_NO_ERR){
					iResult = c33_RecvMessage(C33_ID_SP_REG_READ, 0, 0, 0, 0, 0);
				}
				if (iResult != C33_NO_ERR)
				{
					c33_ErrorMsg();
				}

				/* save %idir, %dbbr */
				g_ulIdir = read_register (IDIR_REGNUM);
				g_ulDbbr = read_register (DBBR_REGNUM);
				
				/* update Regster window  */
				if ( c33TargetInfo.ucCpuType == C33_ADV )
				{
					c33TargetInfo.ucAdvanced = C33_PRODUCT_ADV;

					if( g_use_windows == 1 )	/* GUI Mode ? */
					{
						Tcl_SetVar( gdbtk_interp,"c33_reg_mode","ADV",TCL_GLOBAL_ONLY );		
						/* Registers window update */
						g_reg_update_now = 1;	
						Tcl_Eval( gdbtk_interp, "c33_reg_update" ); /* gdbtcl/srcbar.itcl */
						g_reg_update_now = 0;	
					}
				}
				if ( c33TargetInfo.ucCpuType == C33_PE )
				{
					c33TargetInfo.ucAdvanced = C33_PRODUCT_PE;

					if( g_use_windows == 1 )	/* GUI Mode ? */
					{
						Tcl_SetVar( gdbtk_interp,"c33_reg_mode","PE",TCL_GLOBAL_ONLY );		
						/* Registers window update */
						g_reg_update_now = 1;	
						Tcl_Eval( gdbtk_interp, "c33_reg_update" ); /* gdbtcl/srcbar.itcl */
						g_reg_update_now = 0;	
					}
				}
				
				/* CPU is an advance macro */
//	#if 0	/* add T.Tazaki 2005/06/20 : Do not get TTBR address */
				ulTTBR = read_register (TTBR_REGNUM);
				c33_Cmd_ReadWrite(C33_READ, ulTTBR, C33_WORD, C33_WORD, (char*)&c33TargetInfo.ulBootAddr);
//	#endif
//				c33TargetInfo.ulBootAddr = 0x78000;		/* set no user section address T.Tazaki 2004/06/29 */ /* del T.Tazaki 2005/06/20 */

				/* Get Info of bus trace, bus break, erea break */
				c33_Cmd_ReadWrite(C33_READ, c33TargetInfo.ulDebugBase_rom_addr + 0x1807c, C33_BYTE, C33_BYTE, &c33TargetInfo.ucBrkFunc);
			}
			else
			{
				/* CPU is a standard macro */
				c33TargetInfo.ucAdvanced = C33_PRODUCT_STD;		/* STD */
				/* change T.Tazaki 2005/06/20 >>> */
//				c33_Cmd_ReadWrite(C33_READ, C33_ADDR_TTBR, C33_WORD, C33_WORD, (char*)&c33TargetInfo.ulBootAddr);
				c33_Cmd_ReadWrite(C33_READ, C33_ADDR_TTBR, C33_WORD, C33_WORD, (char*)&ulTTBR );
				c33_Cmd_ReadWrite(C33_READ, ulTTBR, C33_WORD, C33_WORD, (char*)&c33TargetInfo.ulBootAddr);
				/* change T.Tazaki 2005/06/20 <<> */
			}
			/* add T.Tazaki 2004/12/27 <<< */
		}
	}
	else
	{
		/* CPU is a standard macro */
		c33TargetInfo.ucAdvanced = C33_PRODUCT_STD;		/* STD */
		c33_Cmd_ReadWrite(C33_READ, C33_ADDR_TTBR, C33_WORD, C33_WORD, (char*)&c33TargetInfo.ulBootAddr);
	}
	
	if( c33TargetInfo.iIcd_open == C33_USB_OPEN )	/* ICD V3 ? */
	{
		/* set an exact model */
		if (c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV)
		{
			c33TargetInfo.ucCpuType = C33_ADV;	  /* Advanced macro */
			/* Display Target and CPU name : add T.Tazaki 2004/05/24 <<< */
			if( g_use_windows == 1 )	/* GUI Mode ? */
			{
				if( g_iICD_kind == C33_ICDV4 ) /* for ICDV6 add T.Tazaki 2007/07/05 */
				{
					Tcl_SetVar( gdbtk_interp,"c33_target_name","ICD3:ADV",TCL_GLOBAL_ONLY );
				}
				else
				{
					/* for ICDV6 add T.Tazaki 2007/07/05 */
					Tcl_SetVar( gdbtk_interp,"c33_target_name","ICD6:ADV",TCL_GLOBAL_ONLY );
				}
					
				Tcl_Eval( gdbtk_interp, "c33_cpumode_set" ); 

				/* area/bus break mode display: add T.Tazaki 2005/06/14 >>> */
				Tcl_SetVar( gdbtk_interp,"c33_break_mode","area:off bus:off",TCL_GLOBAL_ONLY );
				Tcl_Eval( gdbtk_interp, "c33_breakmode_set" ); 	/* call srcbar.itcl */
				/* Clear area/bus break mode : add T.Tazaki 2005/06/14 <<< */

			}
			/* Display Target and CPU name : add T.Tazaki 2004/05/24 >>> */

		}
		else if (c33TargetInfo.ucAdvanced == C33_PRODUCT_PE)
		{
			c33TargetInfo.ucCpuType = C33_PE;	 /* PE */
			/* Display Target and CPU name : add T.Tazaki 2004/05/24 <<< */
			if( g_use_windows == 1 )	/* GUI Mode ? */
			{
				if( g_iICD_kind == C33_ICDV4 ) /* for ICDV6 add T.Tazaki 2007/07/05 */
				{
					Tcl_SetVar( gdbtk_interp,"c33_target_name","ICD3:PE",TCL_GLOBAL_ONLY );
				}
				else
				{
					/* for ICDV6 add T.Tazaki 2007/07/05 */
					Tcl_SetVar( gdbtk_interp,"c33_target_name","ICD6:PE",TCL_GLOBAL_ONLY );
				}
				Tcl_Eval( gdbtk_interp, "c33_cpumode_set" ); 
			}
			/* Display Target and CPU name : add T.Tazaki 2004/05/24 >>> */
		}
		else
		{
			c33TargetInfo.ucCpuType = C33_STD;	  /* Standard macro */
			/* Display Target and CPU name : add T.Tazaki 2004/05/24 <<< */
			if( g_use_windows == 1 )	/* GUI Mode ? */
			{
				if( g_iICD_kind == C33_ICDV4 ) /* for ICDV6 add T.Tazaki 2007/07/05 */
				{
					Tcl_SetVar( gdbtk_interp,"c33_target_name","ICD3:STD",TCL_GLOBAL_ONLY );
				}
				else
				{
					/* for ICDV6 add T.Tazaki 2007/07/05 */
					Tcl_SetVar( gdbtk_interp,"c33_target_name","ICD6:STD",TCL_GLOBAL_ONLY );
				}
				Tcl_Eval( gdbtk_interp, "c33_cpumode_set" ); 
			}
			/* Display Target and CPU name : add T.Tazaki 2004/05/24 >>> */
		}
		i_das_temp = g_c33icd_das;
		g_c33icd_das = 1;


		/* CPU type & Debug Unit Base addres set */
		if (c33_Cmd_DebugAddr_Set() != C33_NO_ERR)
		{
			g_c33icd_das = i_das_temp;
			return;
		}
		g_c33icd_das = i_das_temp;
	}

	/* A target is reset. */ /* add T.Tazaki 2003/06/24 */
	if (c33_Cmd_Reset(C33_ICD_COLD_RESET) != C33_NO_ERR)
	{
		return;
	}

	/* Read all registers. */
	c33icd_fetch_registers(-1);

	write_register (PC_REGNUM, c33TargetInfo.ulBootAddr & C33_PC_MASK);

//	if (c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV || c33TargetInfo.ucAdvanced == C33_PRODUCT_PE)	/* del T.Tazaki 2005/06/20 */
//	{
//		printf_filtered (C33_MSG_TARGET_BOOT_ADDR_INFO, 0);
//	}
//	else
//	{
		printf_filtered (C33_MSG_TARGET_BOOT_ADDR_INFO, c33TargetInfo.ulBootAddr);
//	}

	inferior_ptid = pid_to_ptid (C33_PROCESS_ID);

#if 0	/* del T.Tazaki 2004/06/29 : Do not display */
	start_remote ();
#endif
	c33_IcdTraceInitData();

	/* trace mode initialize */
/*	c33TargetInfo.c33TraceMode.iTraceMode	= C33_ICD_TRACE_MODE_ALL_OVER_WRITE_NO; */
	c33TargetInfo.c33TraceMode.iTraceMode	= C33_ICD_TRACE_MODE_ALL_OVER_WRITE_YES;	/* Defaut Overwrite 2003/08/25 T.Tazaki */
	c33TargetInfo.c33TraceMode.iCounterMode = C33_ICD_TRACE_COUNTER_ACCUMULATE;
	c33TargetInfo.c33TraceMode.ulTrg1		= 0;
	c33TargetInfo.c33TraceMode.ulTrg2		= 0x100;
	c33TargetInfo.c33TraceMode.iBreak		= C33_ICD_TRACE_BREAK_OFF;
	c33TargetInfo.c33TraceMode.iMeasure 	= C33_ICD_TRACE_MEASURE_ALL;
	if (c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV || c33TargetInfo.ucAdvanced == C33_PRODUCT_PE )    /* add T.Tazaki 2003/11/21 >>> */
	{
		c33IcdTraceData.ucDpcoAddrFix = C33_ICD_DPCO_FIX32;
	}else{
		c33IcdTraceData.ucDpcoAddrFix = C33_ICD_DPCO_FIX29;
	}

	/* add T.Tazaki 2004/04/09 >>> */
	/* Default trace mode set */
	iResult = c33_IcdTraceOverwriteEnable();
	
	/* add T.Tazaki 2004/04/09 <<< */

	/* Target command after "c33 fls"			add T.Tazaki 2003/04/03 >>> */
	if( save_iFlashSet == 1 )
	{
		puts_filtered (C33_MSG_TARGET_ICD_FLS);

		/* re-set "c33 fls" */
		c33_IcdFlashSet( 5,save_ulFlashStartAddr,
						   save_ulFlashEndAddr,
						   save_ulEraseRoutineAddr,
						   save_ulWriteRoutineAddr);
	}
	/* Target command after "c33 fls"			add T.Tazaki 2003/04/03 <<< */

	/* call command mode initialize */
	c33TargetInfo.iFlashSet 			= 0;
	c33TargetInfo.iCallMode 			= 1;

	/* run,step cycle initialize */
	c33TargetInfo.iRunSec1 = 0;
	c33TargetInfo.iRunSec2 = 0;
	c33TargetInfo.iRunCycle = 0;

	/* add 04/07/01 T.Tazaki */
	if( (c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) != C33_ICD_MON)	/* icd v2 or icd v3 ? */
	{
		c33TargetInfo.iOnTheFly = C33_MD_ON_THE_FLY;
	}
	/* ICD V2,V3 setting */
	if( (c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) == C33_ICD_VER3 )
	{
		c33IcdTraceData.ulIcdTraceBufferSize = C33_ICD_TRACE_SIZE - 1;
/*		c33TargetInfo.iOnTheFly = C33_MD_ON_THE_FLY;		*/ /* ICDV2 = ON THE FLY 2004/07/01 T.Tazaki */
	}
	else
	{
		c33IcdTraceData.ulIcdTraceBufferSize = 0x20000 - 1;
	}

	/* add T.Tazaki 2003/06/16 >>> */
	if( c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV )	/* Parameter file is advanced mode ? */
	{
		if( g_ucAdvFlag != 0xff )	/* "file"command executed ? add T.Tazaki 2003/11/05 */
		{
			if( g_ucAdvFlag != 'A' )	/* check if ELF file is not Advanced macro. */
			{
				printf_filtered (C33_WARN_NOT_ADV);
			}
		}
	}
	/* add T.Tazaki 2003/11/18 >>> */
	else if( c33TargetInfo.ucAdvanced == C33_PRODUCT_PE )	/* Parameter file is PE mode ? */
	{
		if( g_ucAdvFlag != 0xff )	/* "file"command executed ? */
		{
			if( g_ucAdvFlag != 'P' )	/* check if ELF file is not PE macro. */
			{
				printf_filtered (C33_WARN_NOT_PE);
			}
		}
	}
	/* add T.Tazaki 2003/11/18 <<< */
	else
	{
		if( g_ucAdvFlag != 0xff )	/* "file"command executed ? add T.Tazaki 2003/11/05 */
		{
			if( g_ucAdvFlag != 0 )	/* check if ELF file is not Standard macro. */
			{
				printf_filtered (C33_WARN_NOT_STD);
			}
		}
	}
	/* add T.Tazaki 2003/06/16 <<< */

	/* OnChip bus flag clear */
	c33TargetInfo.iArea_Mode = 0;
	c33TargetInfo.iBus_Mode  = 0;
	
	/* OnChip Bus Trace default set */
//	c33TargetInfo.iBust_Mode = 1;			/* 1 Fixed */	/* change T.Tazaki 2004/09/29 */
	c33TargetInfo.iBust_Mode = 0;			/* trace off */ /* change T.Tazaki 2004/09/29 */
	c33TargetInfo.iBust_TraceMode	  = 0;	/* Normal Mode */
//	c33TargetInfo.iBust_BusSelect	  = 0;	/* HBCU data */ /* del 2004/06/25 bus select is common. */
	c33TargetInfo.iBus_BusSelect	 = 0;	/* HBCU data */ /* del 2004/06/25 bus select is common. */
	c33TargetInfo.cBust_BBCUBusSelect = 15;	/* all */

	c33TargetInfo.cBus_BBCUBusSelect = 15;	/* all : add T.Tazaki 2005/02/07 */

	c33TargetInfo.iBust_ReadWrite	  = 2;	/* Read/Write */
	c33TargetInfo.iBust_InstData	  = 2;	/* Inst/Data */
	c33TargetInfo.cBust_Access		  = 7;	/* W,H,B */
	c33TargetInfo.iBust_BusOutput	  = 2;	/* Address/Data */
	c33TargetInfo.iBust_AddrBusSize   = 3;	/* 4 byte */
	c33TargetInfo.iBust_Compress	  = 1;	/* ON */
	c33TargetInfo.iBust_UserInput	  = 0;	/* TRC IN */

	/* add T.Tazaki 2004/06/15 >>> */
	c33TargetInfo.iIcdLoad = 0;
	c33TargetInfo.iIcdRm = 0;
	/* add T.Tazaki 2004/06/15 <<< */

	/* Focus to Console window */
	g_iTargetOpen_end = 1;	/* already "target XXX" for focus to console window add T.Tazaki 2003/10/07 */

	c33TargetInfo.iLogiana_Mode = 0;	/* mode off */		/* add T.Tazaki 2005/09/22 */
	c33TargetInfo.iLogiana_UserIn = 0;						/* add T.Tazaki 2005/09/22 */

	/* >>>>> add iruma m.takeishi '03.12.03 */
	iMonKeyBreak = C33_MONSTATUS_OK;
	/* <<<<< add iruma m.takeishi '03.12.03 */

#ifdef __CYGWIN__	/* add T.Tazaki 2005/09/08 */
	if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
	{
		Tcl_Eval( gdbtk_interp, "c33_ide_event_start" ); /* gdbtcl/srcbar.itcl */

		/* gnu33ide.dll initialize */
		hIdeDll = LoadLibraryEx( "gnu33ide", NULL, 0 );
		if( hIdeDll != NULL )
		{
			dll33_ClearCommandUpdate  = (void ( _stdcall *) (WORD))GetProcAddress( hIdeDll,"ClearCommandUpdate" );/* add T.Tazaki 2006/01/12 */
			dll33_GdbEnable       = (void ( _stdcall *) (WORD))GetProcAddress( hIdeDll,"GdbEnable" );
			dll33_GdbDisable      = (void ( _stdcall *) (WORD))GetProcAddress( hIdeDll,"GdbDisable" );
			dll33_GdbRun          = (void ( _stdcall *) (WORD))GetProcAddress( hIdeDll,"GdbRun" );	/* add T.Tazaki 2005/12/08 */
			dll33_GdbStop         = (void ( _stdcall *) (WORD))GetProcAddress( hIdeDll,"GdbStop" );	/* add T.Tazaki 2005/12/08 */
			dll33_addGdbGoCounter = (void ( _stdcall *) (WORD))GetProcAddress( hIdeDll,"addGdbGoCounter" );
			dll33_checkCommand    = (void ( _stdcall *) (WORD))GetProcAddress( hIdeDll,"checkCommand" );
			dll33_setMem          = (void ( _stdcall *) (WORD))GetProcAddress( hIdeDll,"setMem" );
			dll33_getMem          = (void ( _stdcall *) (WORD))GetProcAddress( hIdeDll,"getMem" );	
			dll33_setSymbol       = (void ( _stdcall *) (WORD))GetProcAddress( hIdeDll,"setSymbol" );
			dll33_setIL           = (void ( _stdcall *) (WORD))GetProcAddress( hIdeDll,"setIL" );

			dll33_ClearCommandUpdate();/* add T.Tazaki 2006/01/12 */
			dll33_GdbEnable();
			g_GdbEnable = 1;
		}
	}
#endif

	return;
}
#ifdef __CYGWIN__	/* add T.Tazaki 2005/09/08 */
/****************************************************************************/
/* Format		 : void c33_OSMon( void )									*/
/* Arguments																*/
/* Explanation	 : execute IDE communication cpmmand.						*/
/****************************************************************************/
void
c33_OSMon()
{
	int iStatus = 0;
	int iCommand = 0;
	unsigned long ulAddress;
	unsigned long ulSize;
	unsigned char ucAddress[300];
	unsigned char *pucAddress;

    pucAddress = ucAddress;

	/* check if gdb detached already. */
	if( hIdeDll == NULL )			/* add T.Tazaki 2006/01/25 */
	{
		return;
	}

	if( g_OSMon_Semaho == 1 )		/* add T.Tazaki 2005/10/27 */
	{
		return;
	}

	g_OSMon_Semaho = 1;
	
	if( g_GdbEnable == 0 )	/* No connect target ? */
	{
		g_OSMon_Semaho = 0;
		return;
	}

	if( g_resume_flag == 0 )	/* Program not running ? */
	{
		iStatus = dll33_checkCommand( &iCommand, ucAddress, &ulAddress, &ulSize );	/* IDE command get */


		if( iStatus == 0 )	/* OK ? */
		{
			switch( iCommand )
			{
			case 0:	/* Memory R/W */
			case 1:
					if( *pucAddress != NULL )
					{
						/* convert symbol to binary */
						ulAddress = c33_CnvSymbolToAddress( pucAddress );
					}
					
					if( iCommand == 0 )
					{
						/* Memory Read ? */

						/* Read Memory to DLL buffer */
						iStatus = c33_DllMemRead( ulAddress, ulSize );
					}
					else
					{
						/* Memory Write ? */
						/* DLL buffer to Memory Write */
						iStatus = c33_DllMemWrite( ulAddress, ulSize );
					}
					break;
			case 2:	/* Get Address to Symbol */
					iStatus = c33_DllGetSymbol( ulAddress );
					break;
			case 3:	/* Get %psr : IL bit */
					iStatus = c33_DllGetIL();
					break;
			default:								/* add T.Tazaki 2007/07/12 */
					break;
			
			}
		}
	}

	g_OSMon_Semaho = 0;	/* add T.Tazaki 2005/10/27 */

}

/****************************************************************************/
/* Format		 : int c33_DllMemRead( unsigned long ulAddress, unsigned long ulSize ) */
/* Arguments																*/
/*	unsigned long  ulAddress	: read address								*/
/*	unsigned long  ulSize   	: read byte size							*/
/*	0	 : Success return													*/
/*	1 	 : Error return 													*/
/* Explanation	 : This function gets target memory data and set data to DLL.*/
/****************************************************************************/
static int
c33_DllMemRead( unsigned long ulAddress, unsigned long ulSize )
{
	int iResult = C33_NO_ERR;
	unsigned char szIdeBuffer[1024];
	
	/* read memory data */
	iResult = c33_Cmd_ReadWrite(C33_READ, ulAddress, ulSize, C33_BYTE, (unsigned char*)szIdeBuffer);
	if (iResult != C33_NO_ERR)
	{
//		c33_ErrorMsg();		/* del T.Tazaki 2006/01/11 */
		return iResult;		/* add T.Tazaki 2006/01/11 */

	}
	
	/* set memory data to DLL */
    iResult = dll33_setMem( ulSize, (unsigned char*)szIdeBuffer );

	return iResult;

}

/****************************************************************************/
/* Format		 : int c33_DllMemWrite( unsigned long ulAddress, unsigned long ulSize ) */
/* Arguments																*/
/*	unsigned long  ulAddress	: write address								*/
/*	unsigned long  ulSize   	: write byte size							*/
/*	0	 : Success return													*/
/*	1 	 : Error return 													*/
/* Explanation	 : This function writes data of dll to target memory.		*/
/****************************************************************************/
static int
c33_DllMemWrite( unsigned long ulAddress, unsigned long ulSize )
{
	int iResult = C33_NO_ERR;
	unsigned char szIdeBuffer[1024];
	
	/* get DLL Buffer */
    iResult = dll33_getMem( ulSize, (unsigned char*)szIdeBuffer, 1 );

	/* read memory data */
	iResult = c33_Cmd_ReadWrite(C33_WRITE, ulAddress, ulSize, C33_BYTE, (unsigned char*)szIdeBuffer);
	if (iResult != C33_NO_ERR)
	{
//		c33_ErrorMsg();		/* del T.Tazaki 2006/01/11 */
		return iResult;		/* add T.Tazaki 2006/01/11 */
	}

	return iResult;

}
/****************************************************************************/
/* Format		 : int c33_DllGetSymbol( unsigned long ulAddress ) 			*/
/* Arguments																*/
/*	unsigned long  ulAddress	: address of symbol							*/
/*	0	 : Success return													*/
/*	<>0	 : Error return 													*/
/* Explanation	 : This function sets symbol name to DLL.					*/
/****************************************************************************/
static int
c33_DllGetSymbol( unsigned long ulAddress )
{
	int iResult = C33_NO_ERR;
	int iStatus = 0;
	unsigned char szIdeBuffer[1024];

	iResult = c33_CnvAddressToSymbol( ulAddress, (unsigned char*)szIdeBuffer );
	if( iResult == C33_NO_ERR )
	{
		iStatus = 0;	/* conversion ok */
	}
	else
	{
		iStatus = 1;	/* conversion falied */
	}

	/* set symbol to DLL */
    iResult = dll33_setSymbol( (unsigned char*)szIdeBuffer, iStatus );

	return C33_NO_ERR;

}
/****************************************************************************/
/* Format		 : int c33_DllGetIL( void ) 								*/
/* Arguments																*/
/*	0	 : Success return													*/
/*	<>0	 : Error return 													*/
/* Explanation	 : This function sets %psr IL to DLL.						*/
/****************************************************************************/
static int
c33_DllGetIL()
{
	int iResult = C33_NO_ERR;
	unsigned char ucIL = NULL;
	extern unsigned long ulReg[];

	if( c33TargetInfo.iSim_open == 1 )
	{
		ucIL = (unsigned char)( ulReg[16] >> 8 ) & 0x0f;	/* 16 = PSR */
	}
	else
	{
		ucIL = (unsigned char)( registers[REGISTER_BYTE(PSR_REGNUM) + 1] & 0x0f );
	}
	
	/* set IL to DLL */
    iResult = dll33_setIL( ucIL );

	return C33_NO_ERR;

}
#endif

/****************************************************************************/
/* Format		 : static void c33icd_close (int quitting)					*/
/* Arguments																*/
/*	int quitting : 0-Connection close 1-Quit debugger						*/
/* Return value  : void 													*/
/* Explanation	 : Connection with ICD is closed.							*/
/****************************************************************************/
static void
c33icd_close (int quitting)
{
	/* Does it connect with ICD? */
	if (c33TargetInfo.iIcd_open == C33_COM_OPEN)
	{
		/* Connection with ICD is closed. */
		serial_close(c33_desc);
	}
	else if (c33TargetInfo.iIcd_open == C33_USB_OPEN)
	{
		c33_IcdUsbClose();
	}
	c33TargetInfo.iIcd_open = C33_NOT_OPEN;

	inferior_ptid = null_ptid;

	if( g_c33icd_open == 0 )
	{
		c33_ClearMap();
	}
	c33TargetInfo.iFlashSet = 0;

#ifdef __CYGWIN__	/* add T.Tazaki 2005/09/08 */
	if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
	{
		if( hIdeDll != NULL )
		{
			dll33_GdbDisable();
			FreeLibrary( hIdeDll );
			hIdeDll = NULL;
		}
	}
#endif
}


/****************************************************************************/
/* Format		 : void c33icd_WindowUpdate(void)							*/
/* Arguments	 : void 													*/
/* Return value  : void 													*/
/* Explanation	 : window update after RESET. called by c33common			*/
/****************************************************************************/
void
c33icd_WindowUpdate()
{
	c33icd_fetch_registers(-1);
	stop_pc = read_pc ();
	registers_changed ();

}

/****************************************************************************/
/* Format		 : static void c33icd_kill (void)							*/
/* Arguments	 : void 													*/
/* Return value  : void 													*/
/* Explanation	 : Execution of a program is maintained in ICD33.			*/
/****************************************************************************/
static void
c33icd_kill (void)
{
	target_mourn_inferior ();
	inferior_ptid = null_ptid;
	if( g_c33icd_open == 0 )
	{
		c33_ClearMap();
	}
#ifdef __CYGWIN__	/* add T.Tazaki 2005/09/08 */
	if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
	{
		if( hIdeDll != NULL )
		{
			dll33_GdbDisable();
			g_GdbEnable = 0;
			FreeLibrary( hIdeDll );
			hIdeDll = NULL;
		}
	}
#endif
}

/****************************************************************************/
/* Format		 : static void c33icd_mourn (void)							*/
/* Arguments	 : void 													*/
/* Return value  : void 													*/
/* Explanation	 : none 													*/
/****************************************************************************/
static void
c33icd_mourn (void)
{
}

/****************************************************************************/
/* Format		 : void _initialize_c33icd (void)							*/
/* Arguments	 : void 													*/
/* Return value  : void 													*/
/* Explanation	 : An icd target is added to a debugger.					*/
/*				   A makefile investigates a function name and generates init.c.*/
/****************************************************************************/
void
_initialize_c33icd (void)
{
	/* The definition of a target sub-routine. */
	c33icd_ops_init ();
	
	/* Add the icd target */
	add_target (&c33icd_ops);

	/* Add the icd commands */
	add_com (C33_COMMAND_NAME, class_obscure, c33_command,C33_COMMAND_HELP);

	/* Initialization of target information */
	memset(&c33TargetInfo,0x0,sizeof(c33TargetInfo));
}

/* for ICDV6 add 2007/07/02 T.Tazaki >>> */
/****************************************************************************/
/* Format		 : void _initialize_c33icd6 (void)							*/
/* Arguments	 : void 													*/
/* Return value  : void 													*/
/* Explanation	 : An icd target is added to a debugger.					*/
/*				   A makefile investigates a function name and generates init.c.*/
/****************************************************************************/
void
_initialize_c33icd6 (void)
{
	
	/* The definition of a target sub-routine. */
	c33icd6_ops_init ();
	
	/* Add the icd target */
	add_target (&c33icd6_ops);

	/* Add the icd commands */
	add_com (C33_COMMAND_NAME, class_obscure, c33_command,C33_COMMAND_HELP);

	/* Initialization of target information */
	memset(&c33TargetInfo,0x0,sizeof(c33TargetInfo));
}
/* for ICDV6 add 2007/07/02 T.Tazaki <<< */

/****************************************************************************/
/* Format		 : static void c33icd_files_info (struct target_ops * ignore)*/
/* Arguments																*/
/*	struct target_ops *ignore : Does not use								*/
/* Return value  : void 													*/
/* Explanation	 : The message to a command is displayed.					*/
/****************************************************************************/
static void
c33icd_files_info (struct target_ops *ignore)
{
	puts_filtered (C33_MSG_TARGET_ICD_FILE_INFO);
}

/****************************************************************************/
/* Format		 : static void	c33icd_stop (void)							*/
/* Arguments	 : void 													*/
/* Return value  : void 													*/
/* Explanation	 : ESC is sent to a target and it is stopped.				*/
/****************************************************************************/
static void
c33icd_stop (void)
{
	/* Set ESC break request */
	c33TargetInfo.iKeyBreak = 1;

	/* add 2003/03/26 T.Tazaki : Source command(or -x) Cancel flag ON >>> */
	if( g_command_loop == 1 )	/* now on source command loop ? */
	{
		g_c33_iSourceBreak = 1;
	}
	/* add 2003/03/26 T.Tazaki : Source command(or -x) Cancel flag ON <<< */
}

/****************************************************************************/
/* Format		 : static void c33icd_detach (char *args, int from_tty) 	*/
/* Arguments																*/
/*	char *args	 : The argument of a command								*/
/*	int from_tty : 1 to indicate this is from the console.					*/
/* Return value  : void 													*/
/* Explanation	 : A target is separated and debugging is ended.			*/
/****************************************************************************/
static void
c33icd_detach (char *args, int from_tty)
{
	if (args)
	{
		c33_error (C33_ERR_MSG_ICD_DETACH);
		return;
	}

	pop_target ();
	if (from_tty)
	{
		puts_filtered (C33_MSG_TARGET_END_DEBUG);
	}
}

/****************************************************************************/
/* Format		 : static void c33icd_fetch_registers (int iRegno)			*/
/* Arguments																*/
/*	int iRegno	 : Register number											*/
/* Return value  : void 													*/
/* Explanation	 : In the protocol of ICD33,								*/
/*				   since register specification cannot be performed.		*/
/****************************************************************************/
static void
c33icd_fetch_registers (int iRegno)
{
	int iResult = C33_NO_ERR;

	/* >>>>> add iruma m.takeishi '03.12.03 */
	if ( iMonKeyBreak == C33_MONSTATUS_NG ) {
		return;
	}
	/* <<<<< add iruma m.takeishi '03.12.03 */

	/* check if Now USB Reading.  add T.Tazaki 2004/05/10 >>> */
	if( g_iNow_OnUSBRead == 1 ) 
	{
		return;
	}
	/* check if Now USB Reading.  add T.Tazaki 2004/05/10 <<< */

	
	/* The read of a normal register. */
	if( iRegno == -1 || iRegno < LCO_REGNUM || iRegno >= SSP_REGNUM )	/* include SSP change T.Tazaki 2003/12/05 */
	{
		g_iRegno = iRegno;	/* add T.Tazaki 2003/12/05 */

		/* Transmission of a message. */
		if (iResult == C33_NO_ERR){
			iResult = c33_SendMessage(C33_ID_REG_READ, 0, 0, 0, 0, 0, 0, 0);
		}
		/* Reception of a message. */
		if (iResult == C33_NO_ERR){
			iResult = c33_RecvMessage(C33_ID_REG_READ, 0, 0, 0, 0, 0);
		}
		if (iResult == C33_NO_ERR)	/* add T.Tazaki 2003/08/05 */
		{
			g_iIcdLoadFirst = 1;	/* on */
		}
	}

	/* The read of a special register. */
	if( iRegno == -1 || (iRegno >= LCO_REGNUM && iRegno <= USP_REGNUM) )	/* No include SSP change T.Tazaki 2003/12/05 */
	{
		if ( (c33TargetInfo.ucAdvanced == C33_PRODUCT_PE || c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV)	
			 && (c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) == C33_ICD_VER3)
		{
			/* Transmission of a message. */
			if (iResult == C33_NO_ERR){
				iResult = c33_SendMessage(C33_ID_SP_REG_READ, 0, 0, 0, 0, 0, 0, 0);
			}
			/* Reception of a message. */
			if (iResult == C33_NO_ERR){
				iResult = c33_RecvMessage(C33_ID_SP_REG_READ, 0, 0, 0, 0, 0);
			}
		}
		else
		{
			iResult = c33_IcdRecvSpRegisterRead(C33_ID_SP_REG_RD_0, NULL);
		}
		if (iResult == C33_NO_ERR)	/* add T.Tazaki 2003/08/05 */
		{
			g_iIcdLoadFirst = 1;	/* on */
		}

	}

	if (iResult != C33_NO_ERR)
	{
		c33_ErrorMsg();
	}
}

/****************************************************************************/
/* Format		 : static void c33icd_store_registers (int iRegno)			*/
/* Arguments																*/
/*	int iRegno	 : Register number											*/
/* Return value  : void 													*/
/* Explanation	 : In the protocol of ICD33,								*/
/*				   since register specification cannot be performed.		*/
/****************************************************************************/
static void
c33icd_store_registers (int iRegno)
{
	extern Tcl_Interp *gdbtk_interp;	/*add T.Tazaki 2004/06/03 */
	int iResult = C33_NO_ERR;
	/* add T.Tazaki 2004/06/14 >>> */
	int iConsoleCommand = 0;
	char szbuf[100];
	char *pszbuf;

	pszbuf = szbuf;
	/* add T.Tazaki 2004/06/14 <<< */

	/* >>>>> add iruma m.takeishi '03.12.03 */
	if ( iMonKeyBreak == C33_MONSTATUS_NG ) {
		return;
	}
	/* <<<<< add iruma m.takeishi '03.12.03 */

	/* set $pc=xxxx + next bug fix  add T.Tazaki 2008/06/17 >>> */
	if( iRegno == PC_REGNUM && g_iSet_Command == 1)		/* set $pc=xxxx ? */	
	{
		c33_Cmd_Reset2( C33_ICD_HOT_RESET );	/* hot reset , but no message */
	}
	/* set $pc=xxxx + next bug fix  add T.Tazaki 2008/06/17 <<< */

	/* add T.Tazaki 2004/05/06 : set $sp=value	when %psr:sv=1 (USER MODE) >>> */
	if ( c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV && (c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) == C33_ICD_VER3)
	{
		if( iRegno == SP_REGNUM )
		{
			if( registers[REGISTER_BYTE(PSR_REGNUM) + 1] & 0x10 )	/* %psr : SV == 1 ? */
			{
				/* copy %sp to %usp */
				registers[REGISTER_BYTE(USP_REGNUM) + 3] = registers[REGISTER_BYTE(SP_REGNUM) + 3];
				registers[REGISTER_BYTE(USP_REGNUM) + 2] = registers[REGISTER_BYTE(SP_REGNUM) + 2];
				registers[REGISTER_BYTE(USP_REGNUM) + 1] = registers[REGISTER_BYTE(SP_REGNUM) + 1];
				registers[REGISTER_BYTE(USP_REGNUM) + 0] = registers[REGISTER_BYTE(SP_REGNUM) + 0];

				iResult = c33_SendMessage(C33_ID_SP_REG_WRITE, 0, 0, 0, 0, 0, 0, 0);
				/* Reception of a message. */
				if (iResult == C33_NO_ERR){
					iResult = c33_RecvMessage(C33_ID_SP_REG_WRITE, 0, 0, 0, 0, 0);
				}
				if (iResult != C33_NO_ERR)
				{
					c33_ErrorMsg();
				}
				/* add T.Tazaki 2004/11/01 >>> */
				if( g_use_windows == 1 )  /* GUI Mode ? */
				{
					c33_Gdbtk_Update (C33_UPDATE_REG);	/* add T.Tazaki 2005/09/21 */

#if 0		/* del T.Tazaki 2005/09/21 */
					g_reg_update_now = 1;

					registers_changed ();
					Tcl_Eval( gdbtk_interp, "c33_reg_update" ); /* gdbtcl/srcbar.itcl */
					g_reg_update_now = 0;
#endif
				}
				/* add T.Tazaki 2004/11/01 <<< */
				return;
			}
		}
	}
	/* add T.Tazaki 2004/05/06 : set $sp=value	>>> */


	/* The read of a normal register. */
	if( iRegno == -1 || iRegno < LCO_REGNUM || iRegno >= SSP_REGNUM )	/* include SSP change T.Tazaki 2003/12/05 */
	{
		g_iRegno = iRegno;	/* add T.Tazaki 2003/12/05 */
		
		/* Transmission of a message. */
		if (iResult == C33_NO_ERR){
			iResult = c33_SendMessage(C33_ID_REG_WRITE, 0, 0, 0, 0, 0, 0, 0);
		}
		/* Reception of a message. */
		if (iResult == C33_NO_ERR){
			iResult = c33_RecvMessage(C33_ID_REG_WRITE, 0, 0, 0, 0, 0);
		}
		if (iResult == C33_NO_ERR)	/* add T.Tazaki 2003/08/05 */
		{
			g_iIcdLoadFirst = 1;	/* on */
		}
	}

	/* The read of a special register. */
	if( iRegno == -1 || (iRegno >= LCO_REGNUM && iRegno <= USP_REGNUM) )	/* No include SSP change T.Tazaki 2003/12/05 */
	{
		if ( (c33TargetInfo.ucAdvanced == C33_PRODUCT_PE || c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV)
			  && (c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) == C33_ICD_VER3)
		{
			/* Transmission of a message. */
			if (iResult == C33_NO_ERR){
				iResult = c33_SendMessage(C33_ID_SP_REG_WRITE, 0, 0, 0, 0, 0, 0, 0);
			}
			/* Reception of a message. */
			if (iResult == C33_NO_ERR){
				iResult = c33_RecvMessage(C33_ID_SP_REG_WRITE, 0, 0, 0, 0, 0);
			}
		}
		else
		{
			c33_IcdMakeMessageSpRegWrite(C33_ID_SP_REG_WT_0, NULL);
		}
		if (iResult == C33_NO_ERR)	/* add T.Tazaki 2003/08/05 */
		{
			g_iIcdLoadFirst = 1;	/* on */
		}
	}

	if (iResult != C33_NO_ERR)
	{
		c33_ErrorMsg();
	}

	/* add update reg window	T.Tazaki 2004/06/14 >>> */
	if( g_use_windows == 1 )  /* GUI Mode ? */
	{
		g_reg_update_now = 1;

		registers_changed ();
//		if( iRegno == PC_REGNUM )		/* set $pc=xxxx ? */	/* del T.Tazaki 2005/09/21 */
//		{
			/* add update reg window	T.Tazaki 2004/08/09 >>> */
			if( g_command_loop == 0 && g_iSet_Command == 1)	/* Not command file && set $pc command ? */
			{
				g_iSet_Command = 0;
				/* source window update */
				giFrameClear = 1;
				c33_Gdbtk_Update (C33_UPDATE_REG);
				Tcl_SetVar( gdbtk_interp,"c33_set_pc_command","ON",TCL_GLOBAL_ONLY );
				if( g_iTargetOpen_end == 1 )	/* Target command end ? */
				{
					pszbuf = Tcl_GetVar(gdbtk_interp, "c33_console_cmd", TCL_GLOBAL_ONLY);	 
					iConsoleCommand = atoi( pszbuf );

					if( iConsoleCommand == 1 )
					{
						/* focus chenge to console window */
						Tcl_Eval( gdbtk_interp, "c33_focus_console" ); /* gdbtcl/srcbar.itcl */

						/* clear flag */
						Tcl_SetVar( gdbtk_interp,"c33_console_cmd","0",TCL_GLOBAL_ONLY ); /* gdbtcl/coonsole.itb */

					}
				}
				
				/* set $pc=boot -> next -> set $pc=boot -> next bug fix add T.Tazaki 2008/06/17 : DDTS gnu33v32-00025 >>> */ 
				init_wait_for_inferior ();
				clear_proceed_status ();
				/* set $pc=boot -> next -> set $pc=boot -> next bug fix add T.Tazaki 2008/06/17 : DDTS gnu33v32-00025 <<< */ 


				return_to_top_level (RETURN_QUIT);	/* return to console */
			}
			/* add update reg window	T.Tazaki 2004/08/09 <<< */
//		}
		Tcl_SetVar( gdbtk_interp,"c33_set_pc_command","OFF",TCL_GLOBAL_ONLY );
		g_reg_update_now = 0;
	}
	/* add update reg window	T.Tazaki 2004/06/14 <<< */
}

/****************************************************************************/
/* Format		 : static void c33icd_prepare_to_store (void)				*/
/* Arguments	 : void 													*/
/* Return value  : void 													*/
/* Explanation	 : It does not use in C33.									*/
/****************************************************************************/
static void
c33icd_prepare_to_store (void)
{
}

/****************************************************************************/
/* Format		 : static int c33icd_xfer_memory (CORE_ADDR memaddr, char *myaddr, int iLen, int iReadWrite, struct mem_attrib *attrib, struct target_ops *pTarget) */
/* Arguments																*/
/*	CORE_ADDR memaddr		   : Target address 							*/
/*	char *myaddr			   : Debugger address							*/
/*	int iLen				   : Data Length								*/
/*	int iReadWrite			   : Read or write								*/
/*	struct target_ops *pTarget : Does not use.								*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : The read or write of a target memory.					*/
/****************************************************************************/
static int
c33icd_xfer_memory (CORE_ADDR memaddr, char *myaddr, int iLen, int iReadWrite, struct mem_attrib *attrib, struct target_ops *pTarget)
{
	int iResult;
	int iUnit;/* Byte, Half word, Word */

	/* >>>>> add iruma m.takeishi '03.12.03 */
	if ( iMonKeyBreak == C33_MONSTATUS_NG ) {
		puts_filtered( C33_MSG_MON_TARGETBAD );
		return 0;
	}
	/* <<<<< add iruma m.takeishi '03.12.03 */

	/* The unit of r/w is created. */
	if (memaddr % C33_WORD == 0 && iLen % C33_WORD == 0)
	{
		iUnit = C33_WORD;
	}
	else if (memaddr % C33_HALF_WORD == 0 && iLen % C33_HALF_WORD == 0)
	{
		iUnit = C33_HALF_WORD;
	}
	else
	{
		iUnit = C33_BYTE;
	}

	/* check if icdv2 parallel load mode ? */
	if ( g_iParallel_mode == 1 )
	{
		if (iLen > C33_COMMAND_PARL_TEXT_SIZE )
		{
			iLen = C33_COMMAND_PARL_TEXT_SIZE;
		}
	}
	else
	{
		if (iLen > C33_COMMAND_TEXT_SIZE )
		{
			iLen = C33_COMMAND_TEXT_SIZE;
		}
	}
	
	iResult = c33_Cmd_ReadWrite(iReadWrite, memaddr, iLen, iUnit, myaddr);

	if (iResult != C33_NO_ERR)
	{
		return 0;
	}
	return iLen;
}

/****************************************************************************/
/* Format		 : static void c33icd_load (char *filename, int from_tty)	*/
/* Arguments																*/
/*	char *filename : filename												*/
/*	int from_tty   : 1 to indicate this is from the console.				*/
/* Return value  : void 													*/
/* Explanation	 : program is written in a target.							*/
/****************************************************************************/
static void
c33icd_load (char *filename, int from_tty)
{

	/* >>>>> add iruma m.takeishi '03.12.03 */
	if ( iMonKeyBreak == C33_MONSTATUS_NG ) {
		puts_filtered( C33_MSG_MON_TARGETBAD );
		return;
	}
	/* <<<<< add iruma m.takeishi '03.12.03 */

	c33TargetInfo.iIcdLoad = 1;
	
	g_iIcdLoadFirst = 1;	/* add T.Tazaki 2003/07/29 : ICD V3 high speed load */

	g_iParl_error_1st = 1;	/* icdv2 parallel time out error message : display 1 time. */
	
	c33_InitCancel(); /* Cancel Check Start add 2003/03/19 T.Tazaki  */

	generic_load (filename, from_tty);

	c33TargetInfo.iIcdLoad = 0;
	g_iIcdLoadFirst = 0;	/* add T.Tazaki 2003/07/29 : ICD V3 high speed load */

	c33_EndCancel();	/* Cancel Check End  add 2003/03/19 T.Tazaki */

	/* >>>>> add iruma m.takeishi '03.04.28 */
	giFrameClear = 1;
	/* <<<<< add iruma m.takeishi '03.04.28 */
	c33_Gdbtk_Update (C33_UPDATE_REG);
	stop_pc = read_pc ();
	set_current_frame (create_new_frame (read_fp (), stop_pc));
	select_frame (get_current_frame (), 0);
}

/****************************************************************************/
/* Format		 : static void c33icd_ops_init (void)						*/
/* Arguments	 : void 													*/
/* Return value  : void 													*/
/* Explanation	 : The sub routine is registered.							*/
/****************************************************************************/
static void
c33icd_ops_init (void)
{
	c33icd_ops.to_shortname = "icd";
	c33icd_ops.to_longname = "EPSON C33 ICD interface";
	c33icd_ops.to_doc = "Debug a system controlled by a EPSON C33 ICD.";
	c33icd_ops.to_open = c33icd_open;
	c33icd_ops.to_close = c33icd_close;
	c33icd_ops.to_attach = NULL;
	c33icd_ops.to_post_attach = NULL;
	c33icd_ops.to_require_attach = NULL;
	c33icd_ops.to_detach = c33icd_detach;
	c33icd_ops.to_require_detach = NULL;
	c33icd_ops.to_resume = c33icd_resume;
	c33icd_ops.to_wait = c33icd_wait;
	c33icd_ops.to_post_wait = NULL;
	c33icd_ops.to_fetch_registers = c33icd_fetch_registers;
	c33icd_ops.to_store_registers = c33icd_store_registers;
	c33icd_ops.to_prepare_to_store = c33icd_prepare_to_store;
	c33icd_ops.to_xfer_memory = c33icd_xfer_memory;
	c33icd_ops.to_files_info = c33icd_files_info;
	c33icd_ops.to_insert_breakpoint = c33icd_insert_breakpoint;
	c33icd_ops.to_remove_breakpoint = c33icd_remove_breakpoint;
	c33icd_ops.to_terminal_init = NULL;
	c33icd_ops.to_terminal_inferior = NULL;
	c33icd_ops.to_terminal_ours_for_output = NULL;
	c33icd_ops.to_terminal_ours = NULL;
	c33icd_ops.to_terminal_info = NULL;
	c33icd_ops.to_kill = c33icd_kill;
	c33icd_ops.to_load = c33icd_load;
	c33icd_ops.to_lookup_symbol = NULL;
	c33icd_ops.to_create_inferior = NULL;
	c33icd_ops.to_mourn_inferior = c33icd_mourn;
	c33icd_ops.to_can_run = 0;
	c33icd_ops.to_notice_signals = 0;
	c33icd_ops.to_thread_alive = NULL;
	c33icd_ops.to_stop = c33icd_stop;
	c33icd_ops.to_pid_to_exec_file = NULL;
	c33icd_ops.to_stratum = process_stratum;
	c33icd_ops.DONT_USE = NULL;
	c33icd_ops.to_has_all_memory = 1;
	c33icd_ops.to_has_memory = 1;
	c33icd_ops.to_has_stack = 1;
	c33icd_ops.to_has_registers = 1;
	c33icd_ops.to_has_execution = 1;
	c33icd_ops.to_sections = NULL;
	c33icd_ops.to_sections_end = NULL;
	c33icd_ops.to_magic = OPS_MAGIC;
}

/* for ICDV6 add T.Tazaki 2007/07/02 */
/****************************************************************************/
/* Format		 : static void c33icd6_ops_init (void)						*/
/* Arguments	 : void 													*/
/* Return value  : void 													*/
/* Explanation	 : The sub routine is registered.							*/
/*                 c33icd6_ops_initƂ̑_								*/
/*					to_shortname,to_longname,to_doc,to_open					*/
/****************************************************************************/
static void
c33icd6_ops_init (void)
{
	c33icd6_ops.to_shortname = "icd6";
	c33icd6_ops.to_longname = "EPSON C33 ICDV6 interface";
	c33icd6_ops.to_doc = "Debug a system controlled by a EPSON C33 ICDV6.";
	c33icd6_ops.to_open = c33icd6_open;
	c33icd6_ops.to_close = c33icd_close;
	c33icd6_ops.to_attach = NULL;
	c33icd6_ops.to_post_attach = NULL;
	c33icd6_ops.to_require_attach = NULL;
	c33icd6_ops.to_detach = c33icd_detach;
	c33icd6_ops.to_require_detach = NULL;
	c33icd6_ops.to_resume = c33icd_resume;
	c33icd6_ops.to_wait = c33icd_wait;
	c33icd6_ops.to_post_wait = NULL;
	c33icd6_ops.to_fetch_registers = c33icd_fetch_registers;
	c33icd6_ops.to_store_registers = c33icd_store_registers;
	c33icd6_ops.to_prepare_to_store = c33icd_prepare_to_store;
	c33icd6_ops.to_xfer_memory = c33icd_xfer_memory;
	c33icd6_ops.to_files_info = c33icd_files_info;
	c33icd6_ops.to_insert_breakpoint = c33icd_insert_breakpoint;
	c33icd6_ops.to_remove_breakpoint = c33icd_remove_breakpoint;
	c33icd6_ops.to_terminal_init = NULL;
	c33icd6_ops.to_terminal_inferior = NULL;
	c33icd6_ops.to_terminal_ours_for_output = NULL;
	c33icd6_ops.to_terminal_ours = NULL;
	c33icd6_ops.to_terminal_info = NULL;
	c33icd6_ops.to_kill = c33icd_kill;
	c33icd6_ops.to_load = c33icd_load;
	c33icd6_ops.to_lookup_symbol = NULL;
	c33icd6_ops.to_create_inferior = NULL;
	c33icd6_ops.to_mourn_inferior = c33icd_mourn;
	c33icd6_ops.to_can_run = 0;
	c33icd6_ops.to_notice_signals = 0;
	c33icd6_ops.to_thread_alive = NULL;
	c33icd6_ops.to_stop = c33icd_stop;
	c33icd6_ops.to_pid_to_exec_file = NULL;
	c33icd6_ops.to_stratum = process_stratum;
	c33icd6_ops.DONT_USE = NULL;
	c33icd6_ops.to_has_all_memory = 1;
	c33icd6_ops.to_has_memory = 1;
	c33icd6_ops.to_has_stack = 1;
	c33icd6_ops.to_has_registers = 1;
	c33icd6_ops.to_has_execution = 1;
	c33icd6_ops.to_sections = NULL;
	c33icd6_ops.to_sections_end = NULL;
	c33icd6_ops.to_magic = OPS_MAGIC;
}

/****************************************************************************/
/* Format		 : c33icd_resume (ptid_t pid, int step, enum target_signal siggnal) */
/* Arguments																*/
/*	ptid_t pid				   : Process information						*/
/*	int step				   : Step(1) or run(0)							*/
/*	enum target_signal siggnal : Does not use								*/
/* Return value  : void 													*/
/* Explanation	 : An execution system command is sent to a target. 		*/
/****************************************************************************/
static void
c33icd_resume (ptid_t pid, int step, enum target_signal siggnal)
{

	int iResult = C33_NO_ERR;
	int iCondition = 0;
	unsigned long ulCurrentPC;
	int i;
	int	iErrCount = 0;	/* add T.Tazaki 2005/06/22 */
    extern Tcl_Interp *gdbtk_interp;	/* bug fix : while() cannot stop : add T.Tazaki 2008/06/16 */


	c33TargetInfo.iKeyBreak = 0;	/* bug fix T.Tazaki 2004/11/10 */

	/* >>>>> add iruma m.takeishi '03.12.03 */
	if ( iMonKeyBreak == C33_MONSTATUS_NG ) {
		puts_filtered( C33_MSG_MON_TARGETBAD );
		return;
	}
	/* <<<<< add iruma m.takeishi '03.12.03 */

	/* Trace data clear */
	c33_IcdTraceInitData();

/* del T.Tazaki 2003/07/14 not reset when the program goes >>> */
#if 0
	/* Cold reset is carried out if current PC is a boot address. */
	if (read_register(PC_REGNUM) == (c33TargetInfo.ulBootAddr & C33_PC_MASK))
	{
		if (c33_Cmd_Reset(C33_ICD_COLD_RESET) != C33_NO_ERR )
		{
			return;
		}
	}
#endif
/* del T.Tazaki 2003/07/14 <<< */

    g_resume_flag = 1; /* add T.Tazaki 2005/12/08 */

#ifdef __CYGWIN__	/* add T.Tazaki 2005/12/08 */
	if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
	{
		if( hIdeDll != NULL )
		{
			dll33_addGdbGoCounter();		/* gnu33ide.dll : GO counter +1    */
			dll33_GdbRun();					/* gnu33ide.dll : GO flag ON   */
		}
	}
#endif
	
	if (step){
		if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
		{
			printf_filtered( "<STEP> \n" );
		}
		c33_SetMMUMode(0);	/* MMU OFF add T.Tazaki 2004/08/03 */

		/*==================================================*/
		/* Advanced : Set Area break, bus break, bus trace	*/ /* add T.Tazaki 2004/04/19 */
		/*==================================================*/
		c33_SetBusMode();

		/* step */

		/* bug fix : while() cannot stop : add T.Tazaki 2008/06/04 >>> */
		if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
		{
			
		   /* bug fix : while() cannot stop : add T.Tazaki 2008/06/17 >>> */
		   /* Memory Window update flag OFF */
			Sleep( 100 );
			
            Tcl_SetVar( gdbtk_interp,"c33_mem_update","OFF",TCL_GLOBAL_ONLY ); 
        	c33_Gdbtk_Update (0);
			c33_Gdbtk_Idle();
            /* Memory Window update flag ON */
            Tcl_SetVar( gdbtk_interp,"c33_mem_update","ON",TCL_GLOBAL_ONLY ); 

			Sleep( 100 );	

            /* Memory Window update flag OFF */
            Tcl_SetVar( gdbtk_interp,"c33_mem_update","OFF",TCL_GLOBAL_ONLY );
        	c33_Gdbtk_Update (0);
			c33_Gdbtk_Idle();
            /* Memory Window update flag ON */
            Tcl_SetVar( gdbtk_interp,"c33_mem_update","ON",TCL_GLOBAL_ONLY );
		   /* bug fix : while() cannot stop : add T.Tazaki 2008/06/17 <<< */



			if( c33TargetInfo.iKeyBreak == 1 )
			{
				return;
			}
		}
		/* bug fix : while() cannot stop : add T.Tazaki 2008/06/04 <<< */

		c33TargetInfo.iIcdExeStatus = C33_EXE_STATUS_STEP;
		iResult = c33_SendMessage(C33_ID_STEP, 0, 0, 0, 0, 0, 0, 0);
	
		g_iStepFirst = 1;
	}
	else {

		/* check if step break addess is Hard PC Break, and STEP + Continue 	add T.Tazaki 2004/01/19 >>> */
		if( g_iStepFirst == 1 )
		{
			g_iStepFirst = 0;

			ulCurrentPC = read_register(PC_REGNUM);

			c33_GetBreakPoint();


			/* check if current pc is hard pc break address. 		add T.Tazaki 2004/01/19 */
			if( g_HardPcBreakTable_Index == 1  )
			{
				if( g_HardPcBreakTable[ 0 ] == ulCurrentPC )
				{
					c33TargetInfo.iRunMonStatus = 0;
					return;
				}
			}
			else if( g_HardPcBreakTable_Index == 2 )
			{
				if( g_HardPcBreakTable[ 0 ] == ulCurrentPC || g_HardPcBreakTable[ 1 ] == ulCurrentPC )
				{
					c33TargetInfo.iRunMonStatus = 0;
					return;
				}
			}

			/* check if current pc is soft pc break address. 		add T.Tazaki 2004/01/23 */
			for( i = 0 ; i < g_SoftPcBreakTable_Index; ++i )
			{
				if( g_SoftPcBreakTable[ i ] == ulCurrentPC )
				{
					c33TargetInfo.iRunMonStatus = 0;
					return;
				}
			}
			
			/* check if current pc is temporary break address.  : add T.Tazaki 2005/05/12 >>> */
			if( g_temp_brk == 1 )
			{
				if( g_ulTemp_brk_addr == ulCurrentPC )
				{
					c33TargetInfo.iRunMonStatus = 0;
					return;
				}
			}
			/* add T.Tazaki 2005/05/12 >>> */
		}
		/* check if step break addess is Hard PC Break, and STEP + Continue 	add T.Tazaki 2004/01/19 <<< */


		g_iStepFirst = 0;
		/* add T.Tazaki 2004/01/13 <<< */

		if( c33TargetInfo.iClockMode == 1 ) /* counter reset mode ? */
		{
			/* Reset counters */
			g_ulClockCount[0] = 0;	/* cycle count total */
			g_ulClockCount[1] = 0;	/* 50ns time total */
			g_ulClockCount[2] = 0;	/* 1us time  total */
		}

		c33_SetMMUMode(0);	/* MMU OFF add T.Tazaki 2004/08/03 */

		/* add T.Tazaki 2005/05/10 >>> */
		/* chack if currect pc is same soft pc breakpoint. */
		if ( g_iSoftBreakPointIsSamePC == 1 )
		{
			if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
			{
				printf_filtered( "<c33_Cmd_SoftPcBreakSet(0x%08x)> \n",(int)g_ulSoftBreakPointIsSamePC_addr );
			}
			c33_Cmd_SoftPcBreakSet( g_ulSoftBreakPointIsSamePC_addr );	/* set currect pc breakpoint */
		}
		/* add T.Tazaki 2005/05/10 <<< */


		/*==================================================*/
		/* Advanced : Set Area break, bus break, bus trace	*/ /* add T.Tazaki 2004/04/19 */
		/*==================================================*/
		c33_SetBusMode();

//		  iCondition = 0x4;   /* execte condition bit 2 : BCLK/counter Clear */
		iCondition = 0x14;	 /* BCLK/counter Clear,Bus Break counter Clear(set again)  : change T.Tazaki 2004/10/28 */

		if( g_temp_brk == 1 )	/* temporary break ? 2003/03/31 T.Tazaki */
		{
			iCondition |= C33_BIT_ON_TEMP;
		}
		/* run */
		c33TargetInfo.iIcdExeStatus = C33_EXE_STATUS_RUN;
		if (c33TargetInfo.iOnTheFly == C33_MD_ON_THE_FLY)
		{
			iCondition |= C33_BIT_ON_THE_FLY;
		}
		if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
		{
			printf_filtered( "<RUN> g_temp_brk=[%d] iCondition=[%x] g_ulTemp_brk_addr=[%x]\n"
							,g_temp_brk,iCondition,(int)g_ulTemp_brk_addr);
		}

		/* GO */
		
		/* add T.Tazaki 2005/06/22 3 times error retry >>> */
		if ((c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) == C33_ICD_VER2 ) /* ICD2 ? */
		{
			iErrCount = 0;
			do
			{
				iResult = c33_SendMessage(C33_ID_GO, iCondition, g_ulTemp_brk_addr, 0, 0, 0, 0, 0);
				if (c33TargetInfo.iOnTheFly == C33_MD_ON_THE_FLY && iResult == C33_NO_ERR)
				{
					iResult = c33_RecvMessage(C33_ID_GO_ACK, 0, 0, 0, 0, 0);
				}
				if (iResult == C33_NO_ERR)
				{
					break;
				}
				
				/* Result Error */
				if( c33TargetInfo.iIcdErrorCode == C33_ID_ERR_CODE_RUN ) /* CPU is running error ? */
				{
					if( iErrCount > 3 )
					{
						c33_ErrorMsg();
						return;
					}
					++iErrCount;
				}
				else
				{
					break;
				}
			}
			while( 1 );
		}
		else
		{
		/* add T.Tazaki 2005/06/22 3 times error retry <<< */
			iResult = c33_SendMessage(C33_ID_GO, iCondition, g_ulTemp_brk_addr, 0, 0, 0, 0, 0);
			if (c33TargetInfo.iOnTheFly == C33_MD_ON_THE_FLY && iResult == C33_NO_ERR)
			{
				iResult = c33_RecvMessage(C33_ID_GO_ACK, 0, 0, 0, 0, 0);
			}
		}
	}

	if (iResult != C33_NO_ERR){
		c33_ErrorMsg();
	}

}
/****************************************************************************/
/* Format		 : static void c33_SetBusMode( void );						*/
/* Arguments																*/
/* Return value  : void 													*/
/* Explanation	 : Set area break or bus break or bus trace.				*/
/*                 Set PC Trace/Bus Trace (0x402ec)                         */
/****************************************************************************/
static void
c33_SetBusMode()
{
	unsigned char ucChar;		/* add T.Tazaki 2005/07/27 */
	int iResult = C33_NO_ERR;	/* add T.Tazaki 2005/07/27 */

	/* logiana mode ? add T.Tazaki 2005/09/26 >>> */
	if( c33TargetInfo.iLogiana_Mode == 1 )
	{
		c33TargetInfo.iBust_UserInput = c33TargetInfo.iLogiana_UserIn;
		
		/* Bus trcae set */
		iResult = c33_SendMessage(C33_ID_BUS_TRACE_SET, 0, 0, 0, 0, 0, 0, 0 );
		if (iResult == C33_NO_ERR)
		{
			iResult = c33_RecvMessage(C33_ID_BUS_TRACE_SET, 0, 0, 0, 0, 0);
		}
		if (iResult != C33_NO_ERR)
		{
			c33_ErrorMsg();
		}
		return;
	}
	/* add T.Tazaki 2005/09/26 <<< */

	
	/* Check if area break function is enable. */
	if( (c33TargetInfo.ucBrkFunc & C33_GDB_STS_AREA_BREAK) != 0 )
	{
		if( c33TargetInfo.iArea_Mode == 0 )
		{
			c33_IcdVer3AreaBreak_off();
		}
		else
		{
			c33_IcdVer3AreaBreak_on();
		}
	}
	/* Check if bus break function is enable. */
	if( (c33TargetInfo.ucBrkFunc & C33_GDB_STS_BUS_BREAK) != 0 )
	{
		if( c33TargetInfo.iBus_Mode == 0 )
		{
			c33_IcdVer3BusBreak_off();
		}
		else
		{
			c33_IcdVer3BusBreak_on();
		}
	}

	/* add PC Trace control T.Tazaki 2005/07/27 >>> */
	/* Check if PC trace is enable. */
	if( (c33TargetInfo.iVerRdIcdStatus & C33_DIPSW1_PC_TRACE) == 0 )	/* PC trace enabled ? */
	{
		g_iPCTrace = 1;	/* on */
	}
	else
	{
		g_iPCTrace = 0;	/* off */
	}
	/* add PC Trace control T.Tazaki 2005/07/27 <<< */

	/* Check if bus trace function is enable. */
	if( (c33TargetInfo.ucBrkFunc & C33_GDB_STS_BUS_TRACE) != 0 )
	{
		if( (c33TargetInfo.iVerRdIcdStatus & C33_DIPSW1_BUS_TRACE) == 0 )	/* bus trace enabled ?(DIPSW1:D4) add T.Tazaki 2004/05/31 */
		{
			if( c33TargetInfo.iBust_Mode == 0 )
			{
				c33_IcdVer3BusTrace_off();				/* add T.Tazaki 2004/09/29 */
			}
			else
			{
				c33_IcdVer3BusTrace_on();
			}
		}
		else
		{
			/* DIPSW1 ŃoXg[XݒłDebug Unit̖͍sB */
			if( c33TargetInfo.iBust_Mode == 0 )
			{
				c33_IcdVer3BusTrace_off();				/* add T.Tazaki 2004/09/29 */
			}
		}
	}
	/* add PC Trace control T.Tazaki 2005/07/27 >>> */
	else
	{
		ucChar = 0x59;			/* ݋ */
		iResult = c33_Cmd_ReadWrite(C33_WRITE, 0x402e8, 1, C33_BYTE, &ucChar);
		if (iResult == C33_NO_ERR)
		{
			if( g_iPCTrace == 1 )	/* add T.Tazaki 2005/07/27 */
			{
				ucChar = 0x01;	/* Bus trace off/ PC trace on */
			}
			else
			{
				ucChar = 0x00;  /* Bus trace off/ PC trace off */
			}
			iResult = c33_Cmd_ReadWrite(C33_WRITE, 0x402ec, 1, C33_BYTE, &ucChar);
		}
		if (iResult != C33_NO_ERR)
		{
			c33_ErrorMsg();
		}
	}
	/* add PC Trace control T.Tazaki 2005/07/27 <<< */
}

/****************************************************************************/
/* Format		 : static ptid_t c33icd_wait (ptid_t pid,struct target_waitstatus *status) */
/* Arguments																*/
/*	ptid_t pid						 : Process information					*/
/*	struct target_waitstatus *status : Target status						*/
/* Return value  : Process information										*/
/* Explanation	 : The target information after a break is					*/
/*				   received from icd, and it is stored. 					*/
/****************************************************************************/
static ptid_t
c33icd_wait (ptid_t pid,struct target_waitstatus *status)
{
	static void (*old_sigint) ();	/* Old signal-handler for sigint */
	int iResult = C33_NO_ERR;
	int iRet = 0;
	unsigned long ulPC;
	int i;
	int iCondition;   /* add T.Tazaki 2003/11/06 */
	int iType;		  /* add T.Tazaki 2003/11/07 */
	int iNum;		  /* add T.Tazaki 2003/11/07 */
	unsigned long ulStartTime;	  /* add T.Tazaki 2004/07/01 */
	int iWaitFirst = 1;			  /* add T.Tazaki 2004/07/01 */
	int iWaitTimer;				  /* add T.Tazaki 2004/07/01 */
	unsigned char ucSize;		  /* add T.Tazaki 2004/11/29 */
	int iIcdExeStatus_run;		  /* add T.Tazaki 2005/05/10 */
	unsigned short usCode;		  /* add T.Tazaki 2005/05/10 */

	if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
	{
 		   printf_filtered( "c33icd_wait \n" );
	}


	/* add T.Tazaki 2005/05/10 >>> for STEP --> soft pc break set --> CONT --> soft pc break clear */
	if (c33TargetInfo.iIcdExeStatus == C33_EXE_STATUS_RUN)
	{
		iIcdExeStatus_run = 1;
	}
	else
	{
		iIcdExeStatus_run = 0;

		/* bug fix : while() cannot stop : add T.Tazaki 2008/06/04 >>> */
		if( c33TargetInfo.iKeyBreak == 1 )
		{
			puts_filtered (C33_MSG_KEY_BREAK);
			printf_filtered( "\n");	
			g_resume_flag = 0;
			g_temp_brk = 0;
			c33TargetInfo.iKeyBreak = 0;
			status->value.sig = TARGET_SIGNAL_INT;
			c33TargetInfo.iIcdExeStatus = C33_EXE_STATUS_STOP;
			c33_SetMMUMode(1);	/* MMU ON */
			if( hIdeDll != NULL )
			{
				dll33_GdbStop();
			}
			return inferior_ptid;
		}
		/* bug fix : while() cannot stop : add T.Tazaki 2008/06/04 <<< */

	}
	/* add T.Tazaki 2005/05/10 <<< */

	do
	{
		
		g_iMonErr = 0;		/* add T.Tazaki 2004/12/14 ICD SIM I/O bug fix */


		/* Except ignores during execution. */
		if (c33TargetInfo.iIcdExeStatus != C33_EXE_STATUS_STOP)
		{
			if (c33TargetInfo.iIcdExeStatus == C33_EXE_STATUS_RUN)
			{
				if (c33TargetInfo.iOnTheFly == C33_MD_ON_THE_FLY)
				{
				
					/* On the fly mode */
					do
					{
					
#ifdef __CYGWIN__			/* if icdv2 then Wait add T.Tazaki 2004/07/01 */	/* ICDV2 needs the waiting.*/

						if ((c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) == C33_ICD_VER2 ) /* del T.Tazaki 2005/05/17 */
						{
							if( iWaitFirst == 1 )
							{
								iWaitTimer = 500;
								iWaitFirst = 0;
							}
							else
							{
								iWaitTimer = 200;
							}

							ulStartTime = GetTickCount();
							while( (ulStartTime + iWaitTimer) > GetTickCount() )
							{
							}
						}
						else
						{
							Sleep( 100 );		/* down the PC CPU used percent.  add T.Tazaki 2007/07/31 */
						}
#endif
//						c33_Gdbtk_Idle(); /* re-display */	/* del 2004/12/14 T.Tazaki Timing of RUNNING monitor  */

						iResult = c33_SendMessage(C33_ID_RUN_MON, 0, 0, 0, 0, 0, 0, 0);
						if ( iResult != C33_NO_ERR )
						{
							break;
						}
						/* An ESC break is carried out by the Ctrl-c. */
						old_sigint = signal (SIGINT, c33_IcdStopCtrl_C);
						iResult = c33_RecvMessage(C33_ID_RUN_MON, 0, 0, 0, 0, 0);
						signal (SIGINT, old_sigint);
						/* change 2004/10/13 T.Tazaki >>>*/

//						if ( iResult != C33_NO_ERR || ucRecvData[2] == C33_ID_GO )	
						if ( iResult != C33_NO_ERR )
						{
							g_iMonErr = 1;	/* add T.Tazaki 2004/12/14 ICD SIM I/O bug fix */
							break;
  						}	
						c33_Gdbtk_Idle(); /* re-display */	/* add 2004/12/14 T.Tazaki iming of RUNNING monitor */

						/* when the target is on reset, back to gdb by STOP bottun. add T.Tazaki 2005/08/02 >>> */
						/* monitor stats : RESET bit = 0 is check */

						if( c33TargetInfo.iKeyBreak == 1 )
						{
							if( (c33TargetInfo.iRunMonStatus & C33_BIT5) == 0 )	/* RESET  ? */
							{
								c33TargetInfo.iKeyBreak = 0;
								status->value.sig = TARGET_SIGNAL_INT;
								g_resume_flag = 0;
								g_iTargetResetDown = 1;
#ifdef __CYGWIN__	/* add T.Tazaki 2005/12/08 */
								if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
								{
									if( hIdeDll != NULL )
										dll33_GdbStop();
								}
#endif				
								return inferior_ptid;
							}
						}

						/*  when the target is on reset, back to gdb by STOP bottun.  add T.Tazaki 2005/08/02 <<< */

						if ( ucRecvData[2] == C33_ID_GO )
						{
							
							if ( (ucRecvData[3] & 0x08) != 0 || (ucRecvData[5] & 0x1b) != 0 || (ucRecvData[6] & 0xfc) != 0 )
							{
								ulPC = ucRecvData[12];
								ulPC <<= 8;
								ulPC += ucRecvData[13];
								ulPC <<= 8;
								ulPC += ucRecvData[14];
								ulPC <<= 8;
								ulPC += ucRecvData[15];
								
								if((ucRecvData[6] & 0x04) != 0 && g_ulTemp_brk_addr != ulPC )
								{
									/* GO */
									c33TargetInfo.iIcdExeStatus = C33_EXE_STATUS_RUN;
									iCondition = C33_BIT_ON_THE_FLY | C33_BIT_ON_TEMP;
									iResult = c33_SendMessage(C33_ID_GO, iCondition, g_ulTemp_brk_addr, 0, 0, 0, 0, 0);
									if (iResult == C33_NO_ERR)
									{
										iResult = c33_RecvMessage(C33_ID_GO_ACK, 0, 0, 0, 0, 0);
									}
									continue;
								}
								break;
							}
						}
						/* change 2004/10/13 T.Tazaki <<< */
					}
					while(c33TargetInfo.iKeyBreak == 0);

				}
				else
				{
					/* Break response mode */
					/* An ESC break is carried out by the Ctrl-c. */
					old_sigint = signal (SIGINT, c33_IcdStopCtrl_C);
					iResult = c33_RecvMessage(C33_ID_GO, 0, 0, 0, 0, 0);
					signal (SIGINT, old_sigint);
				}
			}
			else if(c33TargetInfo.iIcdExeStatus == C33_EXE_STATUS_STEP)
			{

				/* bug fix : while() cannot stop : add T.Tazaki 2008/06/04 >>> */
				if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
				{
					c33_Gdbtk_Idle();
				}
				/* bug fix : while() cannot stop : add T.Tazaki 2008/06/04 <<< */

				iResult = c33_RecvMessage(C33_ID_STEP, 0, 0, 0, 0, 0);
				/* for Update %sp	: Read Special Register 	add T.Tazaki 2004/08/26 >>> */
				if( iResult == C33_NO_ERR )
				{
					/* bug fix : while() cannot stop : add T.Tazaki 2008/05/26 >>> */
					if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
					{
						c33_Gdbtk_Idle();
						if( c33TargetInfo.iKeyBreak == 1 )
						{
							puts_filtered (C33_MSG_KEY_BREAK);
							printf_filtered( "\n");	
							g_resume_flag = 0;
							g_temp_brk = 0;
							c33TargetInfo.iKeyBreak = 0;
							status->value.sig = TARGET_SIGNAL_INT;
				  			c33TargetInfo.iIcdExeStatus = C33_EXE_STATUS_STOP;
							c33_SetMMUMode(1);	/* MMU ON */
							if( hIdeDll != NULL )
							{
								dll33_GdbStop();
							}
							return inferior_ptid;
						}
					}
					/* bug fix : while() cannot stop : add T.Tazaki 2008/05/26 <<< */
					
					/* bug fix : while() cannot stop : add T.Tazaki 2008/06/04 >>> */
					if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
					{
						c33_Gdbtk_Idle();
					}
					/* bug fix : while() cannot stop : add T.Tazaki 2008/06/04 <<< */

					if ( (c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV) 
						 && (c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) == C33_ICD_VER3)
					{
						/* Transmission of a message. */
						if (iResult == C33_NO_ERR){
							iResult = c33_SendMessage(C33_ID_SP_REG_READ, 0, 0, 0, 0, 0, 0, 0);
						}
						/* Reception of a message. */
						if (iResult == C33_NO_ERR){
							iResult = c33_RecvMessage(C33_ID_SP_REG_READ, 0, 0, 0, 0, 0); /* when %psr:SV=1, USP=>SP update */
						}
						ucRecvData[2] = C33_ID_STEP;
					}
				}
				/* for Update %sp	: Read Special Register 	add T.Tazaki 2004/08/26 <<< */
			}
			else
			{
				c33_error (C33_ERR_INTERNAL_ERR, "c33icd.c", "c33icd_wait()");
				return inferior_ptid;
			}

			/* One the Fly, ICDV2 STEP goto No On the Fly 	2004/07/12 T.Tazaki */
			if (c33TargetInfo.iOnTheFly == C33_MD_ON_THE_FLY &&
			   !( (c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) == C33_ICD_VER2 && c33TargetInfo.iIcdExeStatus == C33_EXE_STATUS_STEP) )
			{

				/*==================================================================================================*/
				/*	On The Fly 																						*/
				/*==================================================================================================*/
				/* add T.Tazaki 2004/05/12 >>> : check Step Cancel */
				if (g_iStep_Cancel == 1 )
				{
					c33_IcdUsbClose();
 					c33_IcdUsbOpen();
					g_iStep_Cancel = 0;

					/* ICD is running */
					iResult = c33_SendMessage(C33_ID_CAN, 0, 0, 0, 0, 0, 0, 0);
					if (iResult == C33_NO_ERR)
					{
						iResult = c33_RecvMessage(C33_ID_GO, 0, 0, 0, 0, 0);
						if (iResult == C33_NO_ERR)
						{
  							puts_filtered (C33_MSG_KEY_BREAK);
							printf_filtered( "\n");	
							c33_EndCancel();	
							g_resume_flag = 0;
				  			c33TargetInfo.iIcdExeStatus = C33_EXE_STATUS_STOP;
							status->value.sig = TARGET_SIGNAL_INT;
				  			
							g_iNow_OnUSBRead = 0;	/* Permit register read/write */

							c33_SetMMUMode(1);	/* MMU ON add T.Tazaki 2004/08/03 */
#ifdef __CYGWIN__	/* add T.Tazaki 2005/12/08 */
							if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
							{
								if( hIdeDll != NULL )
									dll33_GdbStop();
							}
#endif				
							return inferior_ptid; 
						}
						else
						{
							puts_filtered (C33_MSG_KEY_BREAK);
							printf_filtered( "\n");	
							c33_EndCancel();	
							g_resume_flag = 0;
							g_iNow_OnUSBRead = 0;	/* Permit register read/write */
							c33_SetMMUMode(1);	/* MMU ON add T.Tazaki 2004/08/03 */
#ifdef __CYGWIN__	/* add T.Tazaki 2005/12/08 */
							if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
							{
								if( hIdeDll != NULL )
									dll33_GdbStop();
							}
#endif				
							return inferior_ptid; 
						}
					}
				}
				/* add T.Tazaki 2004/05/12 <<< : check Step Cancel */

				else if ( ucRecvData[2] == C33_ID_RUN_MON && c33TargetInfo.iKeyBreak != 0 )
				{
					g_iSendCancel = 1;			/* add T.Tazaki 2004/11/01 for Target Reset hangup */
					iResult = c33_SendMessage(C33_ID_CAN, 0, 0, 0, 0, 0, 0, 0);
					if (iResult == C33_NO_ERR)
					{
						iResult = c33_RecvMessage(C33_ID_GO, 0, 0, 0, 0, 0);
					}
					g_iSendCancel = 0;			/* add T.Tazaki 2004/11/01 for Target Reset hangup */
				}
				else if( ucRecvData[2] == C33_ID_GO || ucRecvData[2] == C33_ID_STEP || iResult != C33_NO_ERR)
				{
					/* ICD is stop */
					/*	add T.Tazaki 2003/11/07 >>> */
					if(c33TargetInfo.iIcdExeStatus == C33_EXE_STATUS_RUN)
					{
						/* "cont [count]" command */
						if ( g_iContinue_count > 0 )
						{
							--g_iContinue_count;
							if( g_iContinue_count > 0 )
							{
								/* add T.Tazaki 2005/05/10 >>> */
								ulPC = read_pc();									 	   
								/* check if current pc is spft pc breakpoint. */
								iRet = c33_CheckSoftBreakPointIsSamePC( ulPC ); 	
								if( iRet == 1 )	/* yes ? */
								{
									c33_Cmd_SoftPcBreakClear( ulPC );

									/* step */
									iResult = c33_SendMessage(C33_ID_STEP, 0, 0, 0, 0, 0, 0, 0);
									if (iResult == C33_NO_ERR)
									{
										iResult = c33_RecvMessage(C33_ID_STEP, 0, 0, 0, 0, 0);
									}

									c33_Cmd_SoftPcBreakSet( ulPC );	
								}
								else
								{
								/* add T.Tazaki 2005/05/10 <<< */
									/* step */
									iResult = c33_SendMessage(C33_ID_STEP, 0, 0, 0, 0, 0, 0, 0);
									if (iResult == C33_NO_ERR)
									{
										iResult = c33_RecvMessage(C33_ID_STEP, 0, 0, 0, 0, 0);
									}
								}
								/* GO */
								c33TargetInfo.iIcdExeStatus = C33_EXE_STATUS_RUN;
//								iCondition = 0;   /* execte condition bit 2 : BCLK/counter Save */
								iCondition = 0x10;	 /* execte condition bit 4 : bus break counter : Clear add T.Tazaki 2004/10/29 */
								iCondition |= C33_BIT_ON_THE_FLY;
								iResult = c33_SendMessage(C33_ID_GO, iCondition, C33_INVALID_ADDR, 0, 0, 0, 0, 0);
								if (iResult == C33_NO_ERR)
								{
									iResult = c33_RecvMessage(C33_ID_GO_ACK, 0, 0, 0, 0, 0);
								}
								continue;
							}
						}
						/* "ignore" command */
						if ( g_iIgnore_number > 0 && g_iIgnore_count > 0 )
						{
							ulPC = read_pc();
							if ( c33TargetInfo.iRunBreakStatus & C33_BRK_STS_BRK_HARDPC )
							{
								iType = 2;
							}
							else if ( c33TargetInfo.iRunBreakStatus & C33_BRK_STS_BRK_DB )/* add T.Tazaki 2004/01/23 */
							{
								iType = 3;
							}else{
								iType = 1;
							}
							iNum = c33_GetBreakNumber( iType, ulPC );
							if( iNum == g_iIgnore_number )
							{
								--g_iIgnore_count;
								if( g_iIgnore_count > 0 )
								{
									/* add T.Tazaki 2005/05/10 >>> */
									ulPC = read_pc();									 	   
									/* PC == \tgu[N|CgH */
									iRet = c33_CheckSoftBreakPointIsSamePC( ulPC ); 	
									if( iRet == 1 )	/* yes ? */
									{
										c33_Cmd_SoftPcBreakClear( ulPC );

										/* step */
										iResult = c33_SendMessage(C33_ID_STEP, 0, 0, 0, 0, 0, 0, 0);
										if (iResult == C33_NO_ERR)
										{
											iResult = c33_RecvMessage(C33_ID_STEP, 0, 0, 0, 0, 0);
										}

										c33_Cmd_SoftPcBreakSet( ulPC );	
									}
									else
									{
									/* add T.Tazaki 2005/05/10 <<< */
										/* step */
										iResult = c33_SendMessage(C33_ID_STEP, 0, 0, 0, 0, 0, 0, 0);
										if (iResult == C33_NO_ERR)
										{
											iResult = c33_RecvMessage(C33_ID_STEP, 0, 0, 0, 0, 0);
										}
									}
									
									/* add T.Tazaki 2004/02/03 for awatch , case : Write inst(step) --> Read inst >>> */
									if ( c33TargetInfo.iRunBreakStatus & C33_BRK_STS_BRK_DB )
									{
										ulPC = read_pc();
										iType = 3;
										iNum = c33_GetBreakNumber( iType, ulPC );
										if( iNum == g_iIgnore_number )
										{
											--g_iIgnore_count;
										}
									}
									if( g_iIgnore_count > 0 )
									{
										/* add T.Tazaki 2004/02/03 <<< */
										/* GO */
										c33TargetInfo.iIcdExeStatus = C33_EXE_STATUS_RUN;
//										iCondition = 0;   /* execte condition bit 2 : BCLK/counter Save */
										iCondition = 0x10;	 /* execte condition bit 4 : bus break counter : Clear add T.Tazaki 2004/10/29 */
										iCondition |= C33_BIT_ON_THE_FLY;
										iResult = c33_SendMessage(C33_ID_GO, iCondition, C33_INVALID_ADDR, 0, 0, 0, 0, 0);
										if (iResult == C33_NO_ERR)
										{
											iResult = c33_RecvMessage(C33_ID_GO_ACK, 0, 0, 0, 0, 0);
										}
										continue;
									}
								}
							}
						}	
					}
					/* add T.Tazaki 2003/11/07 <<< */
				}
				else
				{
					c33_error (C33_ERR_INTERNAL_ERR, "c33icd.c", "c33icd_wait()");
					return inferior_ptid;
				}
			}
			else
			{
				/*==================================================================================================*/
				/* Not On The Fly 																					*/
				/*==================================================================================================*/
				if (iResult == C33_NO_ERR)
				{
					/* ICD is stopped */
					
					/*	add T.Tazaki 2003/11/07 >>> */
					if(c33TargetInfo.iIcdExeStatus == C33_EXE_STATUS_RUN)
					{
						/* "cont [count]" command */
						if ( g_iContinue_count > 0 )
						{
							--g_iContinue_count;
							if( g_iContinue_count > 0 )
							{
								c33TargetInfo.iIcdExeStatus = C33_EXE_STATUS_RUN;
								iCondition = 0;   /* execte condition bit 2 : BCLK/counter Save */
								iResult = c33_SendMessage(C33_ID_GO, iCondition, 0, 0, 0, 0, 0, 0);
								if (c33TargetInfo.iOnTheFly == C33_MD_ON_THE_FLY && iResult == C33_NO_ERR)
								{
									iResult = c33_RecvMessage(C33_ID_GO_ACK, 0, 0, 0, 0, 0);
								}
								continue;
							}
						}
						/* "ignore" command */
						if ( g_iIgnore_number > 0 && g_iIgnore_count > 0 )
						{
							ulPC = read_pc();
							if ( c33TargetInfo.iRunBreakStatus & C33_BRK_STS_BRK_HARDPC )
							{
								iType = 2;
							}
							else if ( c33TargetInfo.iRunBreakStatus & C33_BRK_STS_BRK_DB )	/* add T.Tazaki 2004/01/23 */
							{
								iType = 3;
							}else{
								iType = 1;
							}
							iNum = c33_GetBreakNumber( iType, ulPC );
							if( iNum == g_iIgnore_number )
							{
								--g_iIgnore_count;
								if( g_iIgnore_count > 0 )
								{
									c33TargetInfo.iIcdExeStatus = C33_EXE_STATUS_RUN;
									iCondition = 0;   /* execte condition bit 2 : BCLK/counter Save */
									iResult = c33_SendMessage(C33_ID_GO, iCondition, 0, 0, 0, 0, 0, 0);
									if (c33TargetInfo.iOnTheFly == C33_MD_ON_THE_FLY && iResult == C33_NO_ERR)
									{
										iResult = c33_RecvMessage(C33_ID_GO_ACK, 0, 0, 0, 0, 0);
									}
									continue;
								}
							}
						}	
					}
					/* add T.Tazaki 2003/11/07 <<< */

				}
				else if (iResult == C33_ERR && c33TargetInfo.iErrorCode == C33_ERROR_RS_KEY_BREAK)
				{
					if ( (c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) != C33_ICD_MON) {
						iResult = c33_SendMessage(C33_ID_CAN, 0, 0, 0, 0, 0, 0, 0);
						if (iResult == C33_NO_ERR)
						{
							iResult = c33_RecvMessage(C33_ID_GO, 0, 0, 0, 0, 0);
						}
					} else {
						puts_filtered (C33_MSG_MON_KEYBREAK_ERR);
						iMonKeyBreak = C33_MONSTATUS_NG;
					}
				}
			}

			c33TargetInfo.iIcdExeStatus = C33_EXE_STATUS_STOP;

			/* Print pc break over write error */
			if(c33TargetInfo.iRunSoftPcBreakStatus != 0)
			{
				for ( i = 0 ; i < C33_STDOUT_BREAK_NUM ; i++ )
				{
					if( (c33TargetInfo.iRunSoftPcClear & (1 << i)) != 0 )
					{
						printf_filtered (C33_MSG_GO_BREAK_OVER_WRITE, (int)c33TargetInfo.ulSoftPcBreak[i]);
					}
				}
			}

			/* Set break reason */
			status->kind = TARGET_WAITKIND_STOPPED;

			c33_GetBreakPoint();	/* get all breakpoint infomation : add T.Tazaki 2003/06/30*/

			if (c33TargetInfo.iKeyBreak != 0)
			{
				/* >>>>> change iruma m.takeishi '03.12.03 */
				/*
				puts_filtered (C33_MSG_KEY_BREAK);
				*/
				if ( iMonKeyBreak != C33_MONSTATUS_NG ) {
					puts_filtered (C33_MSG_KEY_BREAK);
				}
				/* <<<<< change iruma m.takeishi '03.12.03 */
				status->value.sig = TARGET_SIGNAL_INT;

				/* display bus break counter */
				c33icd_BusBreakCount_display();

			}
			
			/* add T.Tazaki 2003/06/30 >>> : Watch point display */

			else if ( c33TargetInfo.iRunBreakStatus & C33_BRK_STS_BRK_MRDB )
			{
				/* add T.Tazaki 2003/11/07 >>> */
				/* "continue [count]" command */
				if ( g_iContinue_count > 0 )
				{
					--g_iContinue_count;
					if( g_iContinue_count > 0 )
					{
						c33TargetInfo.iIcdExeStatus = C33_EXE_STATUS_RUN;
//						iCondition = 0;   /* execte condition bit 2 : BCLK/counter Save */
						iCondition = 0x10;	 /* execte condition bit 4 : bus break counter : Clear add T.Tazaki 2004/10/29 */
						if (c33TargetInfo.iOnTheFly == C33_MD_ON_THE_FLY)
						{
							iCondition |= C33_BIT_ON_THE_FLY;
						}
						iResult = c33_SendMessage(C33_ID_GO, iCondition, 0, 0, 0, 0, 0, 0);
						if (c33TargetInfo.iOnTheFly == C33_MD_ON_THE_FLY && iResult == C33_NO_ERR)
						{
							iResult = c33_RecvMessage(C33_ID_GO_ACK, 0, 0, 0, 0, 0);
						}
						continue;
					}
				}
				/* "ignore" command */
				if ( g_iIgnore_number > 0 && g_iIgnore_count > 0 )
				{
					ulPC = read_pc();
					iType = 3;
					iNum = c33_GetBreakNumber( iType, ulPC );
					if( iNum == g_iIgnore_number )
					{
						--g_iIgnore_count;
						if( g_iIgnore_count > 0 )
						{
							c33TargetInfo.iIcdExeStatus = C33_EXE_STATUS_RUN;
//							iCondition = 0;   /* execte condition bit 2 : BCLK/counter Save */
							iCondition = 0x10;	 /* execte condition bit 4 : bus break counter : Clear add T.Tazaki 2004/10/29 */
							/* add T.Tazaki 2004/10/29 >>> */
							if (c33TargetInfo.iOnTheFly == C33_MD_ON_THE_FLY)	
							{
								iCondition |= C33_BIT_ON_THE_FLY;
							}
							/* add T.Tazaki 2004/10/29 <<< */
							iResult = c33_SendMessage(C33_ID_GO, iCondition, 0, 0, 0, 0, 0, 0);
							if (c33TargetInfo.iOnTheFly == C33_MD_ON_THE_FLY && iResult == C33_NO_ERR)
							{
								iResult = c33_RecvMessage(C33_ID_GO_ACK, 0, 0, 0, 0, 0);
							}
							continue;
						}
					}
				}	
				/*	add T.Tazaki 2003/11/07 <<< */

				if( g_resume_flag == 1 )	/* Not display c33icd_wait 2'nd calling */
				{

					if( g_ReadWatchBreakAddrFlag == 1 )
					{
						printf_filtered( "Hardware read watchpoint %d: %s\n\n",g_WatchNum,g_What );
					}
					else if( g_WriteReadWatchBreakAddrFlag == 1 )
					{
						printf_filtered( "Hardware access (read/write) watchpoint %d: %s\n\n",g_WatchNum,g_What );
					}
					
					/* display bus break counter */
					c33icd_BusBreakCount_display();
				}
/*				status->value.sig = TARGET_SIGNAL_TRAP; */
				status->value.sig = TARGET_SIGNAL_INT;	/* change T.Tazaki 2003/07/31 */
				g_resume_flag = 0;
				c33_SetMMUMode(1);	/* MMU ON add T.Tazaki 2004/08/03 */
#ifdef __CYGWIN__	/* add T.Tazaki 2005/12/08 */
				if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
				{
					if( hIdeDll != NULL )
						dll33_GdbStop();
				}
#endif				
				return inferior_ptid;	/* add T.Tazaki 2003/07/31 */

			}
			else if ( c33TargetInfo.iRunBreakStatus & C33_BRK_STS_BRK_MWRB )
			{
				/*	add T.Tazaki 2003/11/07 >>> */
				/* "continue [count]" command */
				if ( g_iContinue_count > 0 )
				{
					--g_iContinue_count;
					if( g_iContinue_count > 0 )
					{
						c33TargetInfo.iIcdExeStatus = C33_EXE_STATUS_RUN;
//						iCondition = 0;   /* execte condition bit 2 : BCLK/counter Save */
						iCondition = 0x10;	 /* execte condition bit 4 : bus break counter : Clear add T.Tazaki 2004/10/29 */
						if (c33TargetInfo.iOnTheFly == C33_MD_ON_THE_FLY)
						{
							iCondition |= C33_BIT_ON_THE_FLY;
						}
						iResult = c33_SendMessage(C33_ID_GO, iCondition, 0, 0, 0, 0, 0, 0);
						if (c33TargetInfo.iOnTheFly == C33_MD_ON_THE_FLY && iResult == C33_NO_ERR)
						{
							iResult = c33_RecvMessage(C33_ID_GO_ACK, 0, 0, 0, 0, 0);
						}
						continue;
					}
				}
				/* "ignore" command */
				if ( g_iIgnore_number > 0 && g_iIgnore_count > 0 )
				{
					ulPC = read_pc();
					iType = 3;
					iNum = c33_GetBreakNumber( iType, ulPC );
					if( iNum == g_iIgnore_number )
					{
						--g_iIgnore_count;
						if( g_iIgnore_count > 0 )
						{
							c33TargetInfo.iIcdExeStatus = C33_EXE_STATUS_RUN;
//							iCondition = 0;   /* execte condition bit 2 : BCLK/counter Save */
							iCondition = 0x10;	 /* execte condition bit 4 : bus break counter : Clear add T.Tazaki 2004/10/29 */
							/* add T.Tazaki 2004/10/29 >>> */
							if (c33TargetInfo.iOnTheFly == C33_MD_ON_THE_FLY)	
							{
								iCondition |= C33_BIT_ON_THE_FLY;
							}
							/* add T.Tazaki 2004/10/29 <<< */
							iResult = c33_SendMessage(C33_ID_GO, iCondition, 0, 0, 0, 0, 0, 0);
							if (c33TargetInfo.iOnTheFly == C33_MD_ON_THE_FLY && iResult == C33_NO_ERR)
							{
								iResult = c33_RecvMessage(C33_ID_GO_ACK, 0, 0, 0, 0, 0);
							}
							continue;
						}
					}
				}	
				/* add T.Tazaki 2003/11/07 <<< */

				if( g_resume_flag == 1 )	/* Not display c33icd_wait 2'nd calling */
				{
					if( g_WriteWatchBreakAddrFlag == 1 )
					{
						printf_filtered( "Hardware write watchpoint %d: %s\n\n",g_WatchNum,g_What );
					}
					else if( g_WriteReadWatchBreakAddrFlag == 1 )
					{
						printf_filtered( "Hardware access (read/write) watchpoint %d: %s\n\n",g_WatchNum,g_What );
					}
					/* display bus break counter */
					c33icd_BusBreakCount_display();
				}
/*				status->value.sig = TARGET_SIGNAL_TRAP; */
				status->value.sig = TARGET_SIGNAL_INT;	/* change T.Tazaki 2003/07/31 */
				g_resume_flag = 0;
				c33_SetMMUMode(1);	/* MMU ON add T.Tazaki 2004/08/03 */
#ifdef __CYGWIN__	/* add T.Tazaki 2005/12/08 */
				if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
				{
					if( hIdeDll != NULL )
						dll33_GdbStop();
				}
#endif				
				return inferior_ptid;	/* add T.Tazaki 2003/07/31 */
			}
			/* add T.Tazaki 2003/06/30 <<< : Watch point display */

			/* add T.Tazaki 2004/04/22 : Area Break display >>> */
			else if (( c33TargetInfo.iRunBreakStatus & C33_BRK_STS_BRK_AB ) && ( (c33TargetInfo.ucBrkFunc & C33_GDB_STS_AREA_BREAK) != 0 ) )
			{
				/* "continue [count]" command */
				if ( g_iContinue_count > 0 )
				{
					--g_iContinue_count;
					if( g_iContinue_count > 0 )
					{
						c33TargetInfo.iIcdExeStatus = C33_EXE_STATUS_RUN;
//						iCondition = 0;   /* execte condition bit 2 : BCLK/counter Save */
						iCondition = 0x10;	 /* execte condition bit 4 : bus break counter : Clear add T.Tazaki 2004/10/29 */
						if (c33TargetInfo.iOnTheFly == C33_MD_ON_THE_FLY)
						{
							iCondition |= C33_BIT_ON_THE_FLY;
						}
						iResult = c33_SendMessage(C33_ID_GO, iCondition, 0, 0, 0, 0, 0, 0);
						if (c33TargetInfo.iOnTheFly == C33_MD_ON_THE_FLY && iResult == C33_NO_ERR)
						{
							iResult = c33_RecvMessage(C33_ID_GO_ACK, 0, 0, 0, 0, 0);
						}
						continue;
					}
				}
				if( g_resume_flag == 1 )	/* Not display c33icd_wait 2'nd calling */
				{
					printf_filtered( "\nHardware on chip Area Break!\n" );

					/* break detail display */
					iResult = c33_IcdAreaBreakStatus_display();
					if (iResult != C33_NO_ERR){
						c33_ErrorMsg();
						return inferior_ptid;	/* bug fix 2007/07/11 T.Tazaki */
					}
					/* display bus break counter */
					c33icd_BusBreakCount_display();	 /* add T.Tazaki 2004/10/29 */
					
				}
				status->value.sig = TARGET_SIGNAL_INT;
				g_resume_flag = 0;
				c33_SetMMUMode(1);	/* MMU ON add T.Tazaki 2004/08/03 */
#ifdef __CYGWIN__	/* add T.Tazaki 2005/12/08 */
				if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
				{
					if( hIdeDll != NULL )
						dll33_GdbStop();
				}
#endif				
				return inferior_ptid;
			}
			/* add T.Tazaki 2004/04/22 : Area Break display <<< */
			/* add T.Tazaki 2004/04/22 : Bus Break display >>> */
			else if (( c33TargetInfo.iRunBreakStatus & C33_BRK_STS_BRK_BB ) && ( (c33TargetInfo.ucBrkFunc & C33_GDB_STS_BUS_BREAK) != 0 ) )
			{
				/* "continue [count]" command */
				if ( g_iContinue_count > 0 )
				{
					--g_iContinue_count;
					if( g_iContinue_count > 0 )
					{
						c33TargetInfo.iIcdExeStatus = C33_EXE_STATUS_RUN;
//						iCondition = 0;   /* execte condition bit 2 : BCLK/counter Save */
						iCondition = 0x10;	 /* execte condition bit 4 : bus break counter : Clear add T.Tazaki 2004/10/29 */
						if (c33TargetInfo.iOnTheFly == C33_MD_ON_THE_FLY)
						{
							iCondition |= C33_BIT_ON_THE_FLY;
						}
						iResult = c33_SendMessage(C33_ID_GO, iCondition, 0, 0, 0, 0, 0, 0);
						if (c33TargetInfo.iOnTheFly == C33_MD_ON_THE_FLY && iResult == C33_NO_ERR)
						{
							iResult = c33_RecvMessage(C33_ID_GO_ACK, 0, 0, 0, 0, 0);
						}
						continue;
					}
				}
				if( g_resume_flag == 1 )	/* Not display c33icd_wait 2'nd calling */
				{
					printf_filtered( "\nHardware on chip Bus Break!\n" );
				}
					
				/* all soft break point clear 			add T.Tazaki 2005/05/11 */
				c33_ClearAllSoftBreak();

				status->value.sig = TARGET_SIGNAL_INT;
				g_resume_flag = 0;
				c33_SetMMUMode(1);	/* MMU ON add T.Tazaki 2004/08/03 */
#ifdef __CYGWIN__	/* add T.Tazaki 2005/12/08 */
				if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
				{
					if( hIdeDll != NULL )
						dll33_GdbStop();
				}
#endif				
				return inferior_ptid;
			}
			/* add T.Tazaki 2004/04/22 : Bus Break display <<< */
			
			else if ( c33TargetInfo.iRunBreakStatus & C33_BRK_STS_EXT_TRG )
			{
				puts_filtered (C33_MSG_EXTERNAL_BREAK);
				status->value.sig = TARGET_SIGNAL_INT;
				
				/* display bus break counter */
				c33icd_BusBreakCount_display();
			}
//			else if ( c33TargetInfo.iRunBreakStatus & C33_BRK_STS_BRK_INST )	/* del T.Tazaki 2004/12/14 ICD SIM I/O bug fix */
			else if ( (c33TargetInfo.iRunBreakStatus & C33_BRK_STS_BRK_INST ) || g_iMonErr == 1)/* add T.Tazaki 2004/12/14 ICD SIM I/O bug fix */
			{
				
				/* add T.Tazaki 2004/12/14 ICD SIM I/O bug fix >>> */
		  		if( g_iMonErr == 1 )			/* check if [CPU is not running] occured on the running monitor. */
				{
		 			iResult = C33_NO_ERR;		/* C33_ID_RUN_MON clear error */
				}
				/* add T.Tazaki 2004/12/14 ICD SIM I/O bug fix <<< */

				/* simulated i/o process */
				if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
				{
					printf_filtered( "c33icd_Stdio \n" );
				}

				iRet = c33icd_Stdio();
				if( iRet == 1 )
				{
					c33_Gdbtk_Update (0);	/* When te mode is SIM I/O, STOP button is disable bug fix. add T.Tazaki 2004/11/29 */
				}

				/* When te mode is SIM I/O, STOP button is disable bug fix. add T.Tazaki 2004/11/29 >>> */
				if( c33TargetInfo.iKeyBreak == 1 )
				{
					if ( iMonKeyBreak != C33_MONSTATUS_NG ) {
						puts_filtered (C33_MSG_KEY_BREAK);
					}
					status->value.sig = TARGET_SIGNAL_INT;

					/* display bus break counter */
					c33icd_BusBreakCount_display();

					
			 		ucSize = 0;
					iResult = c33_Cmd_ReadWrite(C33_WRITE, c33TargetInfo.ulWriteBufAddr, 1, C33_BYTE, (unsigned char*)&ucSize);

					break;
				}
				/* When te mode is SIM I/O, STOP button is disable bug fix. add T.Tazaki 2004/11/29 <<< */
				
				if( iRet == 1 )
				{
					iCondition = 0;
					
					/* add T.Tazaki 2005/05/13 >>> */
					ulPC =  read_pc();
					
			        /* back to currect pc break. */
				    if( c33TargetInfo.ulSoftPcBreak[C33_STDIN_BREAK_NUM-1] == ulPC )
			        {
				        usCode = c33TargetInfo.usSoftPcBreakOrgCode[C33_STDIN_BREAK_NUM-1];
					}
					else
					{
				        usCode = c33TargetInfo.usSoftPcBreakOrgCode[C33_STDOUT_BREAK_NUM-1];
					}
			        iResult = c33_Cmd_ReadWrite(C33_WRITE, ulPC, 2, C33_HALF_WORD, (unsigned char*)&usCode);
					if (iResult == C33_NO_ERR)
					{
						/* check if stdin is enabled.     add T.Tazaki 2005/06/15 */
						if( g_iStdin_EOF == 0 )
						{
							if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/05/13 */
							{
								printf_filtered( "STEP start PC=[%08x]\n",(int)ulPC );
							}
							/* step */
							iResult = c33_SendMessage(C33_ID_STEP, 0, 0, 0, 0, 0, 0, 0);
							if (iResult == C33_NO_ERR)
							{
								iResult = c33_RecvMessage(C33_ID_STEP, 0, 0, 0, 0, 0);
								if (iResult == C33_NO_ERR)
								{
							        usCode = SOFT_BREAK_INST;
							        iResult = c33_Cmd_ReadWrite(C33_WRITE, ulPC, 2, C33_HALF_WORD, (unsigned char*)&usCode);
							    }
							}
						}
						g_iStdin_EOF = 0; /* add T.Tazaki 2005/06/15 */
					}
					/* add T.Tazaki 2005/05/13 <<< */
//					}
//					else
//					{
#if 0	/* del T.Tazaki 2005/05/10 */
					if( c33TargetInfo.iIcd_open == C33_USB_OPEN )	/* ICD V3 ?  add T.Tazaki 2003/10/22 */
					{
						/* ICDV3 do not step, when current PC is break address. So gdb do step */
						iResult = c33_SendMessage(C33_ID_STEP, 0, 0, 0, 0, 0, 0, 0);
						if (iResult == C33_NO_ERR){
							iResult = c33_RecvMessage(C33_ID_STEP, 0, 0, 0, 0, 0);
						}
					}
#endif
					if (iResult == C33_NO_ERR){

						c33TargetInfo.iIcdExeStatus = C33_EXE_STATUS_RUN;
//						iCondition |= 0x14;   /* execte condition bit 4 : bus break counter : Clear add T.Tazaki 2004/10/29 */
						if (c33TargetInfo.iOnTheFly == C33_MD_ON_THE_FLY)
						{
							iCondition |= C33_BIT_ON_THE_FLY;
						}
						/* check if temporary break mode is ON. 			add T.Tazaki 2005/05/13 >>> */
						if ( g_temp_brk == 1 )
						{
							iCondition |= C33_BIT_ON_TEMP;
							iResult = c33_SendMessage(C33_ID_GO, iCondition, g_ulTemp_brk_addr, 0, 0, 0, 0, 0);
						}
						else
						{
						/* add T.Tazaki 2005/05/13 <<< */
							iResult = c33_SendMessage(C33_ID_GO, iCondition, 0, 0, 0, 0, 0, 0);
						}
						if (c33TargetInfo.iOnTheFly == C33_MD_ON_THE_FLY && iResult == C33_NO_ERR)
						{
							iResult = c33_RecvMessage(C33_ID_GO_ACK, 0, 0, 0, 0, 0);
						}
					}
					if (iResult != C33_NO_ERR){
						c33_ErrorMsg();
					}
					continue;
				}
				status->value.sig = TARGET_SIGNAL_TRAP;

				/* display bus break counter */
				c33icd_BusBreakCount_display();
			}
			else
			{
				status->value.sig = TARGET_SIGNAL_TRAP;
				if ( c33TargetInfo.iRunBreakStatus & C33_BRK_STS_BRK_HARDPC )	/* add T.Tazaki 2004/10/28 */
				{
					/* display bus break counter */
					c33icd_BusBreakCount_display();
				}
			}

			c33TargetInfo.iKeyBreak = 0;

			if( iResult == C33_ERR )
			{
				c33_ErrorMsg();
			}
		}
		
		break;	/* exit loop */
		
	}while( 1 );

	c33_SetMMUMode(1);	/* MMU ON add T.Tazaki 2004/08/03 */

	c33TargetInfo.iKeyBreak = 0;	/* add T.Tazaki 2004/11/09 */

	/* add T.Tazaki 2005/05/10 >>> */
	if (iIcdExeStatus_run == 1)	/* C33_EXE_STATUS_RUN ? */
	{
		if ( g_iSoftBreakPointIsSamePC == 1 )
		{
			if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
			{
				printf_filtered( "<c33_Cmd_SoftPcBreakClear(0x%08x)> \n",(int)g_ulSoftBreakPointIsSamePC_addr );
			}
			c33_Cmd_SoftPcBreakClear( g_ulSoftBreakPointIsSamePC_addr );
			g_iSoftBreakPointIsSamePC = 0;
		}
	}
	/* add T.Tazaki 2005/05/10 <<< */


	return inferior_ptid;
}

/* add T.Tazaki 2005/05/11 >>> */
/****************************************************************************/
/* Format		 : static void c33_ClearAllSoftBreak(void)					*/
/* Arguments																*/
/* Return value  : none														*/
/* Explanation	 : all soft break clear.        							*/
/****************************************************************************/
static void
c33_ClearAllSoftBreak()
{
	int i;
	int iResult = C33_NO_ERR;
	unsigned short usCode = 0;
	
    for (i = 0 ; i < C33_SOFT_PC_BREAK_MAX ; i++)
    {
        if ( c33TargetInfo.ulSoftPcBreak[ i ] != C33_BREAK_DISABLE )
        {
	        /* write Original code */
	        
	        usCode = c33TargetInfo.usSoftPcBreakOrgCode[i];
	        iResult = c33_Cmd_ReadWrite(C33_WRITE, c33TargetInfo.ulSoftPcBreak[i], 2, C33_HALF_WORD, (unsigned char*)&usCode);
			c33TargetInfo.ulSoftPcBreak[i] = C33_BREAK_DISABLE;
		}
	}
}

/* add T.Tazaki 2005/05/11 <<< */

/****************************************************************************/
/* Format		 : static void c33icd_BusBreakCount_display(void)			*/
/* Arguments																*/
/* Return value  : none														*/
/* Explanation	 : bus break counter display.								*/
/****************************************************************************/
static void
c33icd_BusBreakCount_display()
{
	int 	iResult = C33_NO_ERR;
	long	lBusBreakCnt = 0;

	/* Check if bus break function is enable. */
	if( (c33TargetInfo.ucBrkFunc & C33_GDB_STS_BUS_BREAK) != 0 )
	{
		/* check if bus break mode is on. */
		if( c33TargetInfo.iBus_Mode == 1 )
		{
			/*------------------------------------*/
			/* display bus remained break counter */
			/*------------------------------------*/
			iResult = c33_Cmd_ReadWrite(C33_READ, C33_BBCNT, C33_WORD, C33_WORD, &lBusBreakCnt);
			if( iResult == C33_NO_ERR )
			{
				/* lBusBreakCnt : 0xffffffff = END */
				/* lBusBreakCnt : 0x00000000 = c 1 */
				/* lBusBreakCnt : 0x00000001 = c 2 */
				
				if( lBusBreakCnt >= 0 )
				{
					if( g_ulBus_Count_org > 0 )
					{
						if( lBusBreakCnt != g_ulBus_Count_org )
						{
							printf_filtered( "*** Bus break counter <%u/%u> \n",lBusBreakCnt + 1, g_ulBus_Count_org + 1 );
						}
					}
				}
				/* save new counter */
				g_lBusBreakCnt = lBusBreakCnt;
			}
		}
	}
}
/****************************************************************************/
/* Format		 : static int c33icd_Stdio(void)							*/
/* Arguments																*/
/* Return value  : 1=Stdio break, 0=normal soft break						*/
/* Explanation	 : simulated i/o break process. 							*/
/****************************************************************************/
static int
c33icd_Stdio()
{
	unsigned long ulAddr;
	unsigned char ucSize;
	unsigned char ucChar;
	unsigned char szString[100];
	char	 szbuf[1000];
	char	 *pszbuf;
	int 	 iChar;
	int 	 iCnt;
	int 	 iRet = 0;
	int 	 iResult = C33_NO_ERR;
	int 	 i;
	int 	 iLen;
	int 	 k;

	extern Tcl_Interp *gdbtk_interp;

	pszbuf = szbuf;

	ulAddr = read_register(PC_REGNUM);

/* add T.Tazaki 2005/06/15 >>> */
    g_iStdin_EOF = 0;
/* add T.Tazaki 2005/06/15 <<< */

	/*===================*/
	/* check Stdin break */
	/*===================*/
	/* add T.Tazaki 2004/12/14 ICD SIM I/O bug fix >>> */
	if( g_iMonErr == 1 )
	{
		if( (c33TargetInfo.ulSoftPcBreak[C33_STDIN_BREAK_NUM-1] + 2 ) == ulAddr ) /* currect pc=READ_FLASH + 2 ? */
		{
			ulAddr -= 2;
		}
	}
	/* add T.Tazaki 2004/12/14 ICD SIM I/O bug fix >>> */

	if( c33TargetInfo.ulSoftPcBreak[C33_STDIN_BREAK_NUM-1] == ulAddr )
	{
		if( fpStdinfile != NULL )
		{
			/*-----------------------*/
			/* read buffer from file */
			/*-----------------------*/

			/* get buffer size */
			ulAddr = c33TargetInfo.ulReadBufAddr + 1;
			iCnt = 0;
			do
			{
				iChar = fgetc( fpStdinfile );
				if( iChar == EOF )
				{
/* add T.Tazaki 2005/06/15 >>> */
			        g_iStdin_EOF = 1;
/* add T.Tazaki 2005/06/15 <<< */
/* del T.Tazaki 2005/06/15 >>> */
//					fclose( fpStdinfile );
//					fpStdinfile = NULL;
/* del T.Tazaki 2005/06/15 <<< */
					break;
				}
				/* read buffer set */
				iResult = c33_Cmd_ReadWrite(C33_WRITE, ulAddr, 1, C33_BYTE, (unsigned char*)&iChar);
				if (iResult != C33_NO_ERR)
				{
					c33_ErrorMsg();
				}

				++iCnt;
				++ulAddr;
				if( iCnt >= 64 )
				{
					break;	/* read buffer full */
				}
			}while( 1 );
			
			/* size set */
			iResult = c33_Cmd_ReadWrite(C33_WRITE, c33TargetInfo.ulReadBufAddr, 1, C33_BYTE, (unsigned char*)&iCnt);
			if (iResult != C33_NO_ERR)
			{
				c33_ErrorMsg();
			}
		}
		else
		{
			/*--------------------------------------------------*/
			/* read buffer from Simulated I/O window key input	*/	/*	add 2003/03/10 T.Tazaki */
			/*--------------------------------------------------*/

			if( g_use_windows == 1 )  /* GUI Mode ? */
			{

			    /* TOOLBAR Disabled : add T.Tazaki 2008/02/26  bug fix  */
		        Tcl_Eval( gdbtk_interp, "c33_toolbar_disabled" ); /* gdbtcl/srcbar.itcl */

				/* wait key input "STRING" + ENTER in Simulated I/O window */

				if( g_c33_std_in == 0 )
				{
					/* add T.Tazaki 2003/09/25 >>> */

					/* Dummy : One space output for first Stdin(read_str) */
					Tcl_SetVar( gdbtk_interp,"c33_stdout_string"," ",TCL_GLOBAL_ONLY );
					Tcl_Eval( gdbtk_interp, "c33_stdout" ); /* gdbtcl/srcbar.itcl */

					/* Clear before TCL input string */
					Tcl_SetVar( gdbtk_interp,"c33_stdin_string","",TCL_GLOBAL_ONLY );	
					/* add T.Tazaki 2003/09/25 <<< */

					/* Get Simutaled I/O Window Key input String */

					c33TargetInfo.iKeyBreak = 0;	/* bug fix T.Tazaki 2004/11/10 */

					do
					{
						if(c33TargetInfo.iKeyBreak == 1)	/* add T.Tazaki 2004/06/22 */
						{
#if 0	/* del T.Tazaki 2004/11/29 */
							printf_filtered( C33_MSG_KEY_BREAK );	/* add T.Tazaki 2004/11/10 */
							printf_filtered( "\n" );				/* add T.Tazaki 2004/11/10 */
#endif
							break;
						}

						*pszbuf = (char *)NULL; /* string initilaize */
								
						Tcl_Eval( gdbtk_interp, "c33_stdin" ); /* gdbtcl/srcbar.itcl */

						pszbuf = Tcl_GetVar(gdbtk_interp, "c33_stdin_string", TCL_GLOBAL_ONLY);

						c33_Gdbtk_Idle ();
										
					}while( *pszbuf == (char *)NULL );

					g_c33_std_in = 1;	/* already keyinput */
					g_c33_Index = 0;
					strcpy( g_c33_stdin_string, pszbuf );

					/* Clear before TCL input string */
					Tcl_SetVar( gdbtk_interp,"c33_stdin_string","",TCL_GLOBAL_ONLY );

				}

				ulAddr = c33TargetInfo.ulReadBufAddr + 1;
				iCnt = 0;
				do
				{
					iChar = g_c33_stdin_string[g_c33_Index];
					if( iChar == NULL )
					{
						/* Initialize flag & read buffer index */
						g_c33_std_in = 0;
						g_c33_Index = 0;
						break;
					}
					/* read buffer set */
					iResult = c33_Cmd_ReadWrite(C33_WRITE, ulAddr, 1, C33_BYTE, (unsigned char*)&iChar);
					if (iResult != C33_NO_ERR)
					{
						c33_ErrorMsg();
					}

					++g_c33_Index;
					++iCnt;
					if( iCnt >= 64 )
					{
						break;	/* read buffer full */
					}
					++ulAddr;
					++pszbuf;
				}while( 1 );

				/* size set */
				iResult = c33_Cmd_ReadWrite(C33_WRITE, c33TargetInfo.ulReadBufAddr, 1, C33_BYTE, (unsigned char*)&iCnt);
				if (iResult != C33_NO_ERR)
				{
					c33_ErrorMsg();
				}

			}
		}
		
		iRet = 1;
	}
	
	/*====================*/
	/* check Stdout break */
	/*====================*/

	/* add T.Tazaki 2004/12/14 ICD SIM I/O bug fix >>> */
	if( g_iMonErr == 1 )
	{
		if( (c33TargetInfo.ulSoftPcBreak[C33_STDOUT_BREAK_NUM-1] + 2 ) == ulAddr ) /* currect pc=WRITE_FLASH + 2 ? */
		{
			ulAddr -= 2;
		}
	}
	/* add T.Tazaki 2004/12/14 ICD SIM I/O bug fix >>> */

	if( c33TargetInfo.ulSoftPcBreak[C33_STDOUT_BREAK_NUM-1] == ulAddr )
	{
		if( fpStdoutfile != NULL )
		{
			/*----------------------*/
			/* write buffer to file */
			/*----------------------*/

			/*  2004/12/20 T.Tazaki >>> */
#if 0
			/* get buffer size */
			iResult = c33_Cmd_ReadWrite(C33_READ, c33TargetInfo.ulWriteBufAddr, 1, C33_BYTE, (unsigned char*)&ucSize);
			if (iResult != C33_NO_ERR)
			{
				c33_ErrorMsg();
			}

			if( ucSize > 0 && ucSize < 65 ) /* size = max 64 */
			{
				for( ulAddr = c33TargetInfo.ulWriteBufAddr + 1; ulAddr <= (c33TargetInfo.ulWriteBufAddr+ucSize); ++ulAddr )
				{
					/* write buffer read */
					iResult = c33_Cmd_ReadWrite(C33_READ, ulAddr, 1, C33_BYTE, (unsigned char*)&ucChar);
					if (iResult != C33_NO_ERR)
					{
						c33_ErrorMsg();
					}

					if( fputc( ucChar, fpStdoutfile ) == EOF )
					{
						fclose( fpStdoutfile );
						fpStdoutfile = NULL;
						c33_error (C33_ERR_FILE_WRITE);
						return 1;
					}
				}
				
			}
#endif
			/* get buffer size */
			iResult = c33_Cmd_ReadWrite(C33_READ, c33TargetInfo.ulWriteBufAddr, 65, C33_BYTE, (unsigned char*)szbuf);
			if (iResult != C33_NO_ERR)
			{
				c33_ErrorMsg();
			}
			ucSize = szbuf[ 0 ];
			if( ucSize > 0 && ucSize < 65 ) /* size = max 64 */
			{
				for( i = 1; i < ucSize + 1; ++i )
				{
					ucChar = szbuf[ i ];

					if( fputc( ucChar, fpStdoutfile ) == EOF )
					{
						fclose( fpStdoutfile );
						fpStdoutfile = NULL;
						c33_error (C33_ERR_FILE_WRITE);
						return 1;
					}
				}
				/* Clear size */
				iCnt = 0;
				iResult = c33_Cmd_ReadWrite(C33_WRITE, c33TargetInfo.ulWriteBufAddr, 1, C33_BYTE, (unsigned char*)&iCnt);
				if (iResult != C33_NO_ERR)
				{
					c33_ErrorMsg();
				}

			}
			/*  2004/12/20 T.Tazaki <<< */

		}
		else
		{
			/*--------------------------------------*/
			/* write buffer to simulated I/O Window */	/*	add 2003/03/10 T.Tazaki */
			/*--------------------------------------*/

			/*  2004/12/20 T.Tazaki >>> */
#if 0
			/* get buffer size */
			iResult = c33_Cmd_ReadWrite(C33_READ, c33TargetInfo.ulWriteBufAddr, 1, C33_BYTE, (unsigned char*)&ucSize);
			if (iResult != C33_NO_ERR)
			{
				c33_ErrorMsg();
			}
			if( ucSize > 0 && ucSize < 65 ) /* size = max 64 */
			{

				i = 0;
				for( ulAddr = c33TargetInfo.ulWriteBufAddr + 1; ulAddr <= (c33TargetInfo.ulWriteBufAddr+ucSize); ++ulAddr )
				{
					/* write buffer read */
					iResult = c33_Cmd_ReadWrite(C33_READ, ulAddr, 1, C33_BYTE, (unsigned char*)&ucChar);
					if (iResult != C33_NO_ERR)
					{
						c33_ErrorMsg();
					}

					szString[ i ] = ucChar;
					++i;
				}
				szString[ i ] = 0x0;	/* NULL */
				
				if( g_use_windows == 1 )  /* GUI Mode ? */
				{
					/* change binary to "." : add T.Tazaki 2004/05/26 >>> */
					if( szString[0] != 0 )
					{
						iLen = strlen( szString );
						for( i = 0; i < iLen; ++i )
						{
							if( szString[i] <= 0x1f || (szString[i] >= 0x80 && szString[i] <= 0x9f) || szString[i] >= 0xe0)
							{
								if( szString[i] != 0x0a && szString[i] != 0x0d )
								{
									szString[i] = '.';
								}
							}
						}
						/* change binary to "." : add T.Tazaki 2004/05/26 <<< */

						/* Set TCL Global data and Call simulated I/O output pref */
						Tcl_SetVar( gdbtk_interp,"c33_stdout_string",szString,TCL_GLOBAL_ONLY );
						Tcl_Eval( gdbtk_interp, "c33_stdout" ); /* gdbtcl/srcbar.itcl */

					}
				}
				
			}
#endif

			/* get buffer size */
			iResult = c33_Cmd_ReadWrite(C33_READ, c33TargetInfo.ulWriteBufAddr, 65, C33_BYTE, (unsigned char*)szbuf);
			if (iResult != C33_NO_ERR)
			{
				c33_ErrorMsg();
			}

			ucSize = szbuf[ 0 ];
			if( ucSize > 0 && ucSize < 65 ) /* size = max 64 */
			{
				k = 0;
				for( i = 1; i < ucSize + 1; ++i )
				{
					szString[ k ] = szbuf[ i ];
					++k;
				}
				szString[ k ] = 0x0;	/* NULL */
				
				if( g_use_windows == 1 )  /* GUI Mode ? */
				{
					/* change binary to "." : add T.Tazaki 2004/05/26 >>> */
					if( szString[0] != 0 )
					{
						iLen = strlen( szString );
						for( i = 0; i < iLen; ++i )
						{
							if( szString[i] <= 0x1f || (szString[i] >= 0x80 && szString[i] <= 0x9f) || szString[i] >= 0xe0)
							{
								if( szString[i] != 0x0a && szString[i] != 0x0d )
								{
									szString[i] = '.';
								}
							}
						}
						/* change binary to "." : add T.Tazaki 2004/05/26 <<< */

						/* Set TCL Global data and Call simulated I/O output pref */
						Tcl_SetVar( gdbtk_interp,"c33_stdout_string",szString,TCL_GLOBAL_ONLY );
						Tcl_Eval( gdbtk_interp, "c33_stdout" ); /* gdbtcl/srcbar.itcl */

						/* Clear size */
						iCnt = 0;
						iResult = c33_Cmd_ReadWrite(C33_WRITE, c33TargetInfo.ulWriteBufAddr, 1, C33_BYTE, (unsigned char*)&iCnt);
						if (iResult != C33_NO_ERR)
						{
							c33_ErrorMsg();
						}

					}
				}
				
			}
			/*  2004/12/20 T.Tazaki <<< */
		}
		
		iRet = 1;
	}

	return iRet;
}

/****************************************************************************/
/* Format		 : static int c33icd_insert_breakpoint (CORE_ADDR addr, char *contents_cache) */
/* Arguments																*/
/*	CORE_ADDR addr		 : Break address									*/
/*	char *contents_cache : Does not use 									*/
/* Return value  : 0 Success return, 1 Error return 						*/
/* Explanation	 : A soft PC break is set up.								*/
/****************************************************************************/
static int
c33icd_insert_breakpoint (CORE_ADDR addr, char *contents_cache)
{

	int iResult;
	int i;
//	unsigned long ulCurrentPC;


	if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
	{
		printf_filtered( "1:<c33icd_insert_breakpoint> address=[%x] g_temp_brk=[%d] g_iSamePCMode=[%d]\n",(int)addr,g_temp_brk,g_iSamePCMode);
	}
	if( g_iSamePCMode == 1 )	/* duplicate c33_SetBreakPointIsSamePC() ON? */
	{							/* When soft pc break is setted in c33_SetBreakPointIsSamePC, skip */
		if( g_iNextCommand == 0 )	/* is this not by next command?  add T.Tazaki 2003/11/06 */
		{
			if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
			{
				printf_filtered( "2:<c33icd_insert_breakpoint> SKIP\n");
			}
			return 0;
		}
	}

#if 0	/* del T.Tazaki 2003/09/11 */
	if( (c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) == C33_ICD_VER3 )
	{
		ulCurrentPC = c33_pc_reg_get();
		if( addr == ulCurrentPC )	/* start pc == break point */
		{
			g_save_start_pc = addr;
			return 0;	/* do not set break point */
		}
	}
#endif

	/* if address is invalid address, skip this function.  add T.Tazaki 2003/10/14 */
	if ( addr == C33_INVALID_ADDR )
	{
		return 0;
	}

	/* Chech same sim in/out address */
	if( c33TargetInfo.ulSoftPcBreak[C33_STDIN_BREAK_NUM-1] == addr )
	{
		c33_error (C33_ERR_MSG_BRK_SOFT_MAX);
		return 1;
	}
	if( c33TargetInfo.ulSoftPcBreak[C33_STDOUT_BREAK_NUM-1] == addr )
	{
		c33_error (C33_ERR_MSG_BRK_SOFT_MAX);
		return 1;
	}

	if ( g_temp_brk == 1 )
	{
		c33_GetBreakPoint();
		
		/* search soft pc break same address */
		for( i = 0; i < g_SoftPcBreakTable_Index; ++i )
		{
			if( addr == g_SoftPcBreakTable[ i ] )
			{
				break;
			}
		}
		if (  i == g_SoftPcBreakTable_Index )
		{
			if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
			{
				printf_filtered( "3:<c33icd_insert_breakpoint> address=[%x]\n",(int)addr);
			}
			/* check if temp break is already setted. */ /* add T.Tazaki 2003/10/14 */
			if( g_ulTemp_brk_addr != C33_INVALID_ADDR )
			{
				g_iSaveSkipAddr = addr;
				if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
				{
					printf_filtered( "4:<c33icd_insert_breakpoint> SKIP:addr ==> g_iSaveSkipAddraddress=[%x]\n",(int)addr);
				}
				return 0;	/* skip */
			}

#if 1		/* Change 2007/07/12 T.Tazaki */
			g_ulTemp_brk_addr = addr;	 /* save temporary break address */
#else
			if( g_use_windows == 0 )	/* CUI ? */
			{
				if( g_ulTemp_brk_addr == C33_INVALID_ADDR )  /* T.Tazaki 2003/09/01 */
				{
					g_ulTemp_brk_addr = addr;	 /* save temporary break address */
				}
				else
				{
					g_iTemp_brk_ng_addr = addr;
				}
			}
			else
			{
				g_ulTemp_brk_addr = addr;	 /* save temporary break address */
			}
#endif
			return 0;

		}
	}

	iResult = c33_Cmd_SoftPcBreakSet(addr);
	if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
	{
		printf_filtered( "<c33_Cmd_SoftPcBreakSet> address=[%x]\n",(int)addr);
	}

	if ( iResult != C33_NO_ERR )
	{
		return 1;
	}
	return 0;
}

/****************************************************************************/
/* Format		 : static int c33icd_remove_breakpoint (CORE_ADDR addr, char *contents_cache) */
/* Arguments																*/
/*	CORE_ADDR addr		 : Break address									*/
/*	char *contents_cache : Does not use 									*/
/* Return value  : 0 Success return, 1 Error return 						*/
/* Explanation	 : A soft PC break is clear.								*/
/****************************************************************************/
static int
c33icd_remove_breakpoint (CORE_ADDR addr, char *contents_cache)
{
	int iResult;

	if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
	{
		printf_filtered( "1:<c33icd_remove_breakpoint> address=[%x] g_temp_brk=[%d]\n",(int)addr,g_temp_brk);
	}
	if( g_iSamePCMode == 1 )	/* duplicate c33_SetBreakPointIsSamePC() ON? */
	{							/* When soft pc break is setted in c33_SetBreakPointIsSamePC, skip */
		if( g_iNextCommand == 0 )	/* is this not by next command?  add T.Tazaki 2003/11/06 */
		{
			if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
			{
				printf_filtered( "2:<c33icd_remove_breakpoint> SKIP\n");
			}
			return 0;
		}
	}


#if 0	/* del T.Tazaki 2003/09/11 */
	if( (c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) == C33_ICD_VER3 )
	{
		if( g_save_start_pc == addr )	/* start pc == clear address ? */
		{
			g_save_start_pc = C33_INVALID_ADDR;
			return 0;		/* do not clear break point */
		}
	}
#endif

	/* if address is invalid address, skip this function.  add T.Tazaki 2003/10/14 */
	if ( addr == C33_INVALID_ADDR )
	{
		return 0;
	}

	if ( addr == g_iSaveSkipAddr )
	{
		g_iSaveSkipAddr = C33_INVALID_ADDR;
		if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
		{
			printf_filtered( "2:<c33icd_remove_breakpoint> SKIP:C33_INVALID_ADDR ==> g_iSaveSkipAddraddress\n");
		}
		return 0;
	}
	
	if ( g_use_windows == 0)
	{
		if( addr == g_iTemp_brk_ng_addr )
		{
#if 0 /* del T.Tazaki 2004/07/27  */
			g_temp_brk = 0; 		/* clear flag */
#endif
			g_ulTemp_brk_addr = C33_INVALID_ADDR;	/* clear temporary break address  2003/09/01 T.Tazaki */
			return 0;
		}
	}

	if ( g_temp_brk == 1 && g_ulTemp_brk_addr == addr )
	{
		if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
		{
			printf_filtered( "3:<c33icd_remove_breakpoint> address=[%x]\n",(int)addr);
		}
#if 0 /* del T.Tazaki 2004/07/27  */
		g_temp_brk = 0; 		/* clear flag */
#endif
		g_ulTemp_brk_addr = C33_INVALID_ADDR;	/* clear temporary break address  2003/09/01 T.Tazaki */
	}
	else
	{
		iResult = c33_Cmd_SoftPcBreakClear(addr);
		if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
		{
			printf_filtered( "<c33_Cmd_SoftPcBreakClear> address=[%x]\n",(int)addr);
		}

		if ( iResult != C33_NO_ERR )
		{
			return 1;
		}
	}
	return 0;
}

/****************************************************************************/
/* Format		 : static void c33_IcdStopCtrl_C (int iSignal)				*/
/* Arguments																*/
/*	int iSignal  : interrupt signal 										*/
/* Return value  : void 													*/
/* Explanation	 : Ctrl_C is sent to a target and it is stopped.			*/
/****************************************************************************/
static void
c33_IcdStopCtrl_C (int iSignal)
{
	/* Set ESC break request */
	c33TargetInfo.iKeyBreak = 1;

}




/*--------------------------------------------------------------------------*/
/* ICD protocol function													*/
/*--------------------------------------------------------------------------*/

/****************************************************************************/
/* Format		 : static int c33_IcdMakeMessageID (int iID, unsigned char* pucSendData) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucSendData : Message buffer 							*/
/* Return value 			   : Message length 							*/
/* Explanation	 : The transmitting message of only ID is created.			*/
/*				   The following is the specification of a message format.	*/
/*				   +-----+													*/
/*				   |	 |	Message length 8-15 bits						*/
/*				   +-----+													*/
/*				   |	 |	Message length 0-7 bits 						*/
/*				   +-----+													*/
/*				   |	 |	Message ID										*/
/*				   +-----+													*/
/*				   |	 |	BCC 											*/
/*				   +-----+													*/
/****************************************************************************/
static int
c33_IcdMakeMessageID (int iID, unsigned char* pucSendData)
{
	int iSendDataNum = 0; /* size(byte) of message size */
	unsigned char* pucSendDataTop = pucSendData;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add message length size(2byte) + bcc size(1byte) */

	return(iSendDataNum);
}

/****************************************************************************/
/* Format		 : static void c33_IcdMakeBCC (unsigned char* pucSendData)	*/
/* Arguments																*/
/*	unsigned char* pucSendData : Message buffer 							*/
/* Return value  : void 													*/
/* Explanation	 : BCC is added to transmitting data.						*/
/****************************************************************************/
static void
c33_IcdMakeBCC(unsigned char* pucSendData)
{
	int iMessageLength; 	  /* Message length */
	unsigned char ucBCC = 0;  /* BCC data */
	int i;

	iMessageLength =  ((int)(*pucSendData) << 8);
	pucSendData++;
	iMessageLength += ((int)(*pucSendData) << 0);
	pucSendData++;
	iMessageLength &= 0xffff;

	for( i = 0 ; i < iMessageLength ; i++){
		ucBCC ^= *pucSendData;/* calculate BCC */
		pucSendData++;
	}

	*pucSendData = ucBCC; /* set BCC */
}

/****************************************************************************/
/* Format		 : static int c33_IcdMakeMessageReset (int iID, unsigned char* pucSendData, int iType) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucSendData : Message buffer 							*/
/*	int iType				   : Reset type 								*/
/* Return value 			   : Message length 							*/
/* Explanation	 : Reset message data is created.							*/
/****************************************************************************/
static int
c33_IcdMakeMessageReset (int iID, unsigned char* pucSendData, int iType)
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)iType;
	pucSendData++;
	iSendDataNum++;

	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}

/****************************************************************************/
/* Format		 : static int c33_IcdMakeMessageGo (int iID, unsigned char* pucSendData, int iFlag, int iAddr) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucSendData : Message buffer 							*/
/*	int iFlag				   : Go condition								*/
/*	int iAddr				   : Temporary break address					*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Go message data is created.								*/
/****************************************************************************/
static int
c33_IcdMakeMessageGo (int iID, unsigned char* pucSendData, int iFlag, int iAddr )
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* Execution conditions are set up. */
	*pucSendData = (unsigned char)iFlag;
	pucSendData++;
	iSendDataNum++;

	/* A temporally break is set up. */
	*pucSendData = (unsigned char)((iAddr >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iAddr >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iAddr >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iAddr >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;


	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}

/****************************************************************************/
/* Format		 : static int c33_IcdMakeMessageStep (int iID, unsigned char* pucSendData, int iFlag) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucSendData : Message buffer 							*/
/*	int iFlag				   : Step condition 							*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Go message data is created.								*/
/****************************************************************************/
static int
c33_IcdMakeMessageStep (int iID, unsigned char* pucSendData, int iFlag)
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* Execution conditions are set up. */
	*pucSendData = (unsigned char)iFlag;
	pucSendData++;
	iSendDataNum++;

	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}

/****************************************************************************/
/* Format		 : static int c33_IcdMakeMessageRegWrite (int iID, unsigned char* pucSendData) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucSendData : Message buffer 							*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Register write message data is created.					*/
/****************************************************************************/
static int
c33_IcdMakeMessageRegWrite (int iID, unsigned char* pucSendData)
{
	int iReg;
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* Start register value write to buffers. (fixed) */
	*pucSendData = C33_COMMAND_START_REG_NUM;
	pucSendData++;
	iSendDataNum++;

	/* Standard register value write to buffers. */
	for (iReg = R0_REGNUM ; iReg < NUM_REGS_STD ; iReg++)
	{
		if (iReg == 0)
		{
			*(pucSendData + iReg * REGISTER_SIZE + 0) = registers[REGISTER_BYTE(PC_REGNUM) + 3];
			*(pucSendData + iReg * REGISTER_SIZE + 1) = registers[REGISTER_BYTE(PC_REGNUM) + 2];
			*(pucSendData + iReg * REGISTER_SIZE + 2) = registers[REGISTER_BYTE(PC_REGNUM) + 1];
			*(pucSendData + iReg * REGISTER_SIZE + 3) = registers[REGISTER_BYTE(PC_REGNUM) + 0] & C33_PC_MASK;
			registers[REGISTER_BYTE(PC_REGNUM) + 0] &= C33_PC_MASK;			/* add T.Tazaki 2005/06/27 */
		}
		else if (iReg >= 1 && iReg <= 16)
		{
			*(pucSendData + iReg * REGISTER_SIZE + 0) = registers[REGISTER_BYTE(iReg - 1) + 3];
			*(pucSendData + iReg * REGISTER_SIZE + 1) = registers[REGISTER_BYTE(iReg - 1) + 2];
			*(pucSendData + iReg * REGISTER_SIZE + 2) = registers[REGISTER_BYTE(iReg - 1) + 1];
			*(pucSendData + iReg * REGISTER_SIZE + 3) = registers[REGISTER_BYTE(iReg - 1) + 0];
		}
		else if (iReg == 17)
		{
			if( g_iRegno == SSP_REGNUM )	/* check if Write Register is %SSP ?	add T.Tazaki 2003/12/05 */
			{
				*(pucSendData + iReg * REGISTER_SIZE + 0) = registers[REGISTER_BYTE(SSP_REGNUM) + 3];
				*(pucSendData + iReg * REGISTER_SIZE + 1) = registers[REGISTER_BYTE(SSP_REGNUM) + 2];
				*(pucSendData + iReg * REGISTER_SIZE + 2) = registers[REGISTER_BYTE(SSP_REGNUM) + 1];
				*(pucSendData + iReg * REGISTER_SIZE + 3) = registers[REGISTER_BYTE(SSP_REGNUM) + 0] & C33_SP_MASK;
				registers[REGISTER_BYTE(SSP_REGNUM) + 0] &= C33_SP_MASK;	/* add T.Tazaki 2005/06/27 */
			}
			else
			{
				*(pucSendData + iReg * REGISTER_SIZE + 0) = registers[REGISTER_BYTE(SP_REGNUM) + 3];
				*(pucSendData + iReg * REGISTER_SIZE + 1) = registers[REGISTER_BYTE(SP_REGNUM) + 2];
				*(pucSendData + iReg * REGISTER_SIZE + 2) = registers[REGISTER_BYTE(SP_REGNUM) + 1];
				*(pucSendData + iReg * REGISTER_SIZE + 3) = registers[REGISTER_BYTE(SP_REGNUM) + 0] & C33_SP_MASK;
				registers[REGISTER_BYTE(SP_REGNUM) + 0] &= C33_SP_MASK;		/* add T.Tazaki 2005/06/27 */
			}
		}
		else if (iReg == 18)
		{
			*(pucSendData + iReg * REGISTER_SIZE + 0) = registers[REGISTER_BYTE(AHR_REGNUM) + 3];
			*(pucSendData + iReg * REGISTER_SIZE + 1) = registers[REGISTER_BYTE(AHR_REGNUM) + 2];
			*(pucSendData + iReg * REGISTER_SIZE + 2) = registers[REGISTER_BYTE(AHR_REGNUM) + 1];
			*(pucSendData + iReg * REGISTER_SIZE + 3) = registers[REGISTER_BYTE(AHR_REGNUM) + 0];
		}
		else if (iReg == 19)
		{
			*(pucSendData + iReg * REGISTER_SIZE + 0) = registers[REGISTER_BYTE(ALR_REGNUM) + 3];
			*(pucSendData + iReg * REGISTER_SIZE + 1) = registers[REGISTER_BYTE(ALR_REGNUM) + 2];
			*(pucSendData + iReg * REGISTER_SIZE + 2) = registers[REGISTER_BYTE(ALR_REGNUM) + 1];
			*(pucSendData + iReg * REGISTER_SIZE + 3) = registers[REGISTER_BYTE(ALR_REGNUM) + 0];
		}
		else if (iReg == 20)
		{
			*(pucSendData + iReg * REGISTER_SIZE + 0) = registers[REGISTER_BYTE(PSR_REGNUM) + 3];
			*(pucSendData + iReg * REGISTER_SIZE + 1) = registers[REGISTER_BYTE(PSR_REGNUM) + 2];
			*(pucSendData + iReg * REGISTER_SIZE + 2) = registers[REGISTER_BYTE(PSR_REGNUM) + 1];
			*(pucSendData + iReg * REGISTER_SIZE + 3) = registers[REGISTER_BYTE(PSR_REGNUM) + 0];
		}
	}
	pucSendData += NUM_REGS_STD * REGISTER_SIZE;
	iSendDataNum += NUM_REGS_STD * REGISTER_SIZE;

	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}

/****************************************************************************/
/* Format		 : static int c33_IcdRecvVersionRead (int iID, unsigned char* pucRecvData) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucRecvData : Message buffer 							*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : The received version read data is stored.				*/
/****************************************************************************/
static int
c33_IcdRecvVersionRead (int iID, unsigned char* pucRecvData)
{
	/* Receive data check */
	if( c33_IcdRecvDataCheck(iID,pucRecvData) != C33_NO_ERR)
	{
		return C33_ERR;
	}

	pucRecvData++;
	pucRecvData++;
	pucRecvData++;

	c33TargetInfo.iVerRdSioErr		= (int)*pucRecvData;
	pucRecvData++;
	c33TargetInfo.iVerRdSioTryCount = ((int)*(pucRecvData + 0) << 8) + ((int)*(pucRecvData + 1) << 0);
	pucRecvData++;
	pucRecvData++;
	c33TargetInfo.iVerRdIcdStatus	= (int)*pucRecvData;
	pucRecvData++;
	c33TargetInfo.iVerRdIcdHardVer	= (int)*pucRecvData;
	pucRecvData++;
	c33TargetInfo.iVerRdIcdFirmVer	= (int)*pucRecvData;
	pucRecvData++;
	c33TargetInfo.iVerRdEmuMemNum	= (int)*pucRecvData;
	pucRecvData++;
	c33TargetInfo.iVerRdDIAG		= (int)*pucRecvData;
	pucRecvData++;
	c33TargetInfo.iVerRdTrgtDown	= (int)*pucRecvData;
	pucRecvData++;
	c33TargetInfo.iVerRdMonStatus	= ((int)*(pucRecvData + 0) << 8) + ((int)*(pucRecvData + 1) << 0);

	return C33_NO_ERR;
}

/****************************************************************************/
/* Format		 : static int c33_IcdMakeMessageDataWrite(int iID, unsigned char* pucSendData, int iAddr, int iLen, int iUnit, unsigned char* pMyAddr) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucSendData : Message buffer 							*/
/*	int iAddr				   : Target address 							*/
/*	int iLen				   : Data Length								*/
/*	int iUnit				   : C33_BYTE,C33_HALF_WORD,C33_WORD			*/
/*	unsigned char* pMyAdd r    : Debugger address							*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Data write message data is created.						*/
/****************************************************************************/
static int
c33_IcdMakeMessageDataWrite(int iID, unsigned char* pucSendData, int iAddr, int iLen, int iUnit, unsigned char* pMyAddr)
{
	int iSize;
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* The target address is set up. */
	*pucSendData = (unsigned char)((iAddr >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iAddr >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iAddr >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iAddr >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	if( iID == C33_ID_DATA_WRITE_FAST ) /* add 2003/07/29 T.Tazaki */
	{
		/* External process routine write flag is set up. */
		if( g_iIcdLoadFirst == 1 )
		{
			*pucSendData = 0;		/* first : write  */
		}
		else
		{
			*pucSendData = 0x80;	/* second... : do not write <-- speed up */
		}
	}
	else
	{
		/* Data unit is set up. */
		if (iUnit == C33_WORD)
		{
			*pucSendData = 3;
		}
		else if (iUnit == C33_HALF_WORD)
		{
			*pucSendData = 2;
		}
		else
		{
			*pucSendData = 1;
		}
	}
	pucSendData++;
	iSendDataNum++;

	/* Data size is set up. */
	for(iSize = 0 ; iSize < iLen ; iSize++)
	{
		*pucSendData = *pMyAddr;
		pucSendData++;
		pMyAddr++;
		iSendDataNum++;
	}

	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}
/****************************************************************************/
/* Format		 : static int c33_IcdMakeMessageParlDataWrite(int iID, unsigned char* pucSendData, int iAddr, int iLen) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucSendData : Message buffer 							*/
/*	int iAddr				   : Target address 							*/
/*	int iLen				   : Data Length								*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Parallel Data write message data is created. 			*/
/****************************************************************************/
static int
c33_IcdMakeMessageParlDataWrite(int iID, unsigned char* pucSendData, int iAddr, int iLen)
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* The target address is set up. */
	*pucSendData = (unsigned char)((iAddr >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iAddr >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iAddr >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iAddr >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* Data size is set up. */
	*pucSendData = (unsigned char)((iLen >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iLen >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}

/****************************************************************************/
/* Format		 : static int c33_IcdMakeMessageDataRead(int iID, unsigned char* pucSendData, int iAddr, int iLen, int iUnit) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucSendData : Message buffer 							*/
/*	int iAddr				   : Target address 							*/
/*	int iLen				   : Data Length								*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Data read message data is created.						*/
/****************************************************************************/
static int
c33_IcdMakeMessageDataRead(int iID, unsigned char* pucSendData, int iAddr, int iLen, int iUnit)
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* The target address is set up. */
	*pucSendData = (unsigned char)((iAddr >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iAddr >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iAddr >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iAddr >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* Data unit is set up. */
	if (iUnit == C33_WORD)
	{
		*pucSendData = 3;
	}
	else if (iUnit == C33_HALF_WORD)
	{
		*pucSendData = 2;
	}
	else
	{
		*pucSendData = 1;
	}
	pucSendData++;
	iSendDataNum++;

	/* Data size is set up. */
	*pucSendData = (unsigned char)((iLen >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iLen >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}

/****************************************************************************/
/* Format		 : static int c33_IcdMakeMessageSoftPcBreakSet(int iID, unsigned char* pucSendData, int iAddr, int iNum) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucSendData : Message buffer 							*/
/*	int iAddr				   : Break address								*/
/*	int iNum				   : Break number								*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Soft PC break set message data is created.				*/
/****************************************************************************/
static int
c33_IcdMakeMessageSoftPcBreakSet(int iID, unsigned char* pucSendData, int iAddr, int iNum)
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* The break number is set up. */
	*pucSendData = (unsigned char)((iNum >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* The break address is set up. */
	*pucSendData = (unsigned char)((iAddr >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iAddr >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iAddr >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iAddr >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);

}

/****************************************************************************/
/* Format		 : static int c33_IcdMakeMessageSoftPcBreakClear(int iID, unsigned char* pucSendData,int iNum) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucSendData : Message buffer 							*/
/*	int iNum				   : Break number								*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Soft PC break clear message data is created. 			*/
/****************************************************************************/
static int
c33_IcdMakeMessageSoftPcBreakClear(int iID, unsigned char* pucSendData, int iNum)
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* The break number is set up. */
	*pucSendData = (unsigned char)((iNum >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}

/****************************************************************************/
/* Format		 : static int c33_IcdMakeMessageHardPcBreakSet(int iID, unsigned char* pucSendData, int iAddr) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucSendData : Message buffer 							*/
/*	int iAddr				   : Break address								*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Hard PC break set message data is created.				*/
/****************************************************************************/
static int
c33_IcdMakeMessageHardPcBreakSet(int iID, unsigned char* pucSendData, int iAddr)
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* The break address is set up. */
	*pucSendData = (unsigned char)((iAddr >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iAddr >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iAddr >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iAddr >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}

/****************************************************************************/
/* Format		 : static int c33_IcdMakeMessageHardPcBreak2(int iID, unsigned char* pucSendData, int iAddr) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucSendData : Message buffer 							*/
/*	int iAddr				   : Break address								*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Hard PC break 2 set/clear message data is created.		*/
/****************************************************************************/
static int
c33_IcdMakeMessageHardPcBreak2(int iID, unsigned char* pucSendData, int iAddr)
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* The break address is set up. */
	*pucSendData = (unsigned char)((iAddr >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iAddr >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iAddr >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iAddr >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* reserved 4 bytes */
	*pucSendData = 0xff;
	pucSendData++;
	iSendDataNum++;
	*pucSendData = 0xff;
	pucSendData++;
	iSendDataNum++;
	*pucSendData = 0xff;
	pucSendData++;
	iSendDataNum++;
	*pucSendData = 0xff;
	pucSendData++;
	iSendDataNum++;

	/* set or clear */
	if ( iAddr == C33_BREAK_DISABLE )
	{
		*pucSendData = 0;
	}
	else
	{
		*pucSendData = 1;
	}
	pucSendData++;
	iSendDataNum++;

	/* reserved 2 bytes */
	*pucSendData = 0xff;
	pucSendData++;
	iSendDataNum++;
	*pucSendData = 0xff;
	pucSendData++;
	iSendDataNum++;
	
	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}

/****************************************************************************/
/* Format		 : static int c33_IcdMakeMessageDataFill(int iID, unsigned char* pucSendData, int iType, int iStartAddr, int iSize , int iData) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucSendData : Message buffer 							*/
/*	int iType				   : Byte or half word or word					*/
/*	int iStartAddr			   : Start address								*/
/*	int iSize				   : Fill size									*/
/*	int iData				   : Fill data									*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Data fill message data is created.						*/
/****************************************************************************/
static int
c33_IcdMakeMessageDataFill(int iID, unsigned char* pucSendData, int iType, int iStartAddr, int iSize , int iData)
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* Set start address */
	*pucSendData = (unsigned char)((iStartAddr >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iStartAddr >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iStartAddr >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iStartAddr >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* Set fill size */
	*pucSendData = (unsigned char)((iSize >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iSize >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iSize >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iSize >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* Set fill type */
	if ( iType == C33_BYTE )
	{
		*pucSendData = 1;
	}
	else if ( iType == C33_HALF_WORD)
	{
		*pucSendData = 2;
	}
	else
	{
		*pucSendData = 3;
	}
	pucSendData++;
	iSendDataNum++;

	/* Set fill data */
	*pucSendData = (unsigned char)((iData >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iData >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iData >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iData >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;

	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}

/****************************************************************************/
/* Format		 : static int c33_IcdMakeMessageDataMove(int iID, unsigned char* pucSendData, int iType, int iSrc, int iDst, int iSize) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucSendData : Message buffer 							*/
/*	int iType				   : Byte or half word or word					*/
/*	int iSrc				   : Source address 							*/
/*	int iDst				   : Destination address						*/
/*	int iSize				   : Move size									*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Data move message data is created.						*/
/****************************************************************************/
static int
c33_IcdMakeMessageDataMove(int iID, unsigned char* pucSendData, int iType, int iSrc, int iDst, int iSize)
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* Set source address */
	*pucSendData = (unsigned char)((iSrc >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iSrc >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iSrc >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iSrc >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* Set move size */
	*pucSendData = (unsigned char)((iSize >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iSize >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iSize >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iSize >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* Set destination address */
	*pucSendData = (unsigned char)((iDst >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iDst >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iDst >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iDst >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* Set fill type */
	if ( iType == C33_BYTE )
	{
		*pucSendData = 1;
	}
	else if ( iType == C33_HALF_WORD)
	{
		*pucSendData = 2;
	}
	else
	{
		*pucSendData = 3;
	}
	pucSendData++;
	iSendDataNum++;

	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}

/****************************************************************************/
/* Format		 : static int c33_IcdMakeMessageDataBreak(int iID, unsigned char* pucSendData, int iAddr, int iType) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucSendData : Message buffer 							*/
/*	int iAddr				   : Break address								*/
/*	int iType				   : Break type 								*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Data break message data is created.						*/
/****************************************************************************/
static int
c33_IcdMakeMessageDataBreak(int iID, unsigned char* pucSendData, int iAddr, int iType)
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* Set Break type */
	if ( iType == hw_write )
	{
		*pucSendData = 1;
	}
	else if ( iType == hw_read )
	{
		*pucSendData = 0;
	}
	else
	{
		*pucSendData = 2;
	}

	*pucSendData = (unsigned char)((iType >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* Set Break address */
	*pucSendData = (unsigned char)((iAddr >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iAddr >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iAddr >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iAddr >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}

/****************************************************************************/
/* Format		 : static int c33_IcdMakeMessageFlashSet(int iID, unsigned char* pucSendData, int iFlashStartAddr, int iFlashEndAddr, int iEraseRoutineAddr, int iWriteRoutineAddr) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucSendData : Message buffer 							*/
/*	int iFlashStartAddr 	   : Flash start address						*/
/*	int iFlashEndAddr		   : Flash end address							*/
/*	int iEraseRoutineAddr	   : Erase routine address						*/
/*	int iWriteRoutineAddr	   : Write routine address						*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Flash set message data is created.						*/
/****************************************************************************/
static int
c33_IcdMakeMessageFlashSet(int iID, unsigned char* pucSendData, int iFlashStartAddr, int iFlashEndAddr, int iEraseRoutineAddr, int iWriteRoutineAddr)
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* Set flash start address */
	*pucSendData = (unsigned char)((iFlashStartAddr >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iFlashStartAddr >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iFlashStartAddr >>	8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iFlashStartAddr >>	0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* Set flash end address */
	*pucSendData = (unsigned char)((iFlashEndAddr >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iFlashEndAddr >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iFlashEndAddr >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iFlashEndAddr >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* Set erase routine address */
	*pucSendData = (unsigned char)((iEraseRoutineAddr >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iEraseRoutineAddr >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iEraseRoutineAddr >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iEraseRoutineAddr >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* Set write routine address */
	*pucSendData = (unsigned char)((iWriteRoutineAddr >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iWriteRoutineAddr >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iWriteRoutineAddr >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iWriteRoutineAddr >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}

/****************************************************************************/
/* Format		 : static int c33_IcdMakeMessageFlashErase(int iID, unsigned char* pucSendData, int iCtrlReg, int iStartBlock, int iEndBlock) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucSendData : Message buffer 							*/
/*	int iCtrlReg		: Flash start address								*/
/*	int iStartBlock 		 : Flash end address							*/
/*	int iEndBlock	   : Write routine address								*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Flash erase message data is created. 					*/
/****************************************************************************/
static int
c33_IcdMakeMessageFlashErase(int iID, unsigned char* pucSendData, int iCtrlReg, int iStartBlock, int iEndBlock)
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* Set control register address */
	*pucSendData = (unsigned char)((iCtrlReg >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iCtrlReg >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iCtrlReg >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iCtrlReg >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* Set start block address */
	*pucSendData = (unsigned char)((iStartBlock >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iStartBlock >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iStartBlock >>	8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iStartBlock >>	0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* Set end block address */
	*pucSendData = (unsigned char)((iEndBlock >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iEndBlock >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iEndBlock >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((iEndBlock >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}

/****************************************************************************/
/* Format		 : static int c33_IcdMakeMessageTraceReadData(int iID, unsigned char* pucSendData, unsigned long ulStartCycle, unsigned long ulReadSize) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucSendData : Message buffer 							*/
/*	unsigned long ulStartCycle : Start cycle								*/
/*	unsigned long ulReadSize   : Read size									*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Trace read data message data is created. 				*/
/****************************************************************************/
static int
c33_IcdMakeMessageTraceReadData(int iID, unsigned char* pucSendData, unsigned long ulStartCycle, unsigned long ulReadSize)
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* Set start cycle number */
	*pucSendData = (unsigned char)((ulStartCycle >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulStartCycle >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulStartCycle >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulStartCycle >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* Set read size */
	*pucSendData = (unsigned char)((ulReadSize >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulReadSize >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}

/****************************************************************************/
/* Format		 : static int c33_IcdMakeMessageBusTraceReadData(int iID, unsigned char* pucSendData, unsigned long ulStartCycle, unsigned long ulReadCycle) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucSendData : Message buffer 							*/
/*	unsigned long ulStartCycle : Start cycle								*/
/*	unsigned long ulReadCycle  : Read cycle 								*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Bus Trace read data message data is created. 			*/
/****************************************************************************/
static int
c33_IcdMakeMessageBusTraceReadData(int iID, unsigned char* pucSendData, unsigned long ulStartCycle, unsigned long ulReadCycle)
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* Set start cycle number */
	*pucSendData = (unsigned char)((ulStartCycle >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulStartCycle >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulStartCycle >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulStartCycle >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* Set read size */
	*pucSendData = (unsigned char)((ulReadCycle >>	8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulReadCycle >>	0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}

/****************************************************************************/
/* Format		 : static int c33_IcdMakeMessageTraceTrgMode(int iID, unsigned char* pucSendData, struct c33_icd_trace_mode* pc33TraceMode) */
/* Arguments																*/
/*	int iID 								 : Message index				*/
/*	unsigned char* pucSendData				 : Message buffer				*/
/*	struct c33_icd_trace_mode* pc33TraceMode : Treace mode address			*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Trace mode setting message data is created.				*/
/****************************************************************************/
static int
c33_IcdMakeMessageTraceTrgMode(int iID, unsigned char* pucSendData, struct c33_icd_trace_mode* pc33TraceMode)
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* The trigger1 address is set up. */
	*pucSendData = (unsigned char)((pc33TraceMode->ulTrg1 >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((pc33TraceMode->ulTrg1 >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((pc33TraceMode->ulTrg1 >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((pc33TraceMode->ulTrg1 >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* The trigger1 break condition. */
	*pucSendData = 0; /* break disable */
	pucSendData++;
	iSendDataNum++;

	/* The trigger2 address is set up. */
	*pucSendData = (unsigned char)((pc33TraceMode->ulTrg2 >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((pc33TraceMode->ulTrg2 >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((pc33TraceMode->ulTrg2 >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((pc33TraceMode->ulTrg2 >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* The trigger1 break condition. */
	if ( pc33TraceMode->iBreak == C33_ICD_TRACE_BREAK_ON )
	{
		*pucSendData = 1; /* break enable */
	}
	else
	{
		*pucSendData = 0; /* break disable */
	}
	pucSendData++;
	iSendDataNum++;

	/* Measure mode is set up. */
	if ( pc33TraceMode->iMeasure == C33_ICD_TRACE_MEASURE_ALL )
	{
		*pucSendData = 0; /* all */
	}
	else
	{
		*pucSendData = 1; /* area */
	}
	pucSendData++;
	iSendDataNum++;

	/* Trace mode is set up. */
	if ( pc33TraceMode->iTraceMode == C33_ICD_TRACE_MODE_ALL_OVER_WRITE_YES )
	{
		*pucSendData = 2; /* all trace, over write yes */
	}
	else if ( pc33TraceMode->iTraceMode == C33_ICD_TRACE_MODE_ALL_OVER_WRITE_NO )
	{
		*pucSendData = 0; /* all trace, over write no */
	}
	else
	{
		*pucSendData = 1; /* area trace, over write yes */
	}
	pucSendData++;
	iSendDataNum++;

	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}

/****************************************************************************/
/* Format		 : static int c33_IcdMakeMessageClockFunctionSet(int iID, unsigned char* pucSendData,int iClockFunction) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucSendData : Message buffer 							*/
/*	int iClockFunction		   : clock function 							*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : The time measurement unit message data is created.		*/
/****************************************************************************/
static int
c33_IcdMakeMessageClockFunctionSet(int iID, unsigned char* pucSendData, int iClockFunction)
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* The time measurement unit is set up. */
	*pucSendData = (unsigned char)((iClockFunction >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}

/****************************************************************************/
/* Format		 : static int c33_IcdVer3MakeMessageFlashErase(int iID, unsigned char* pucSendData,int iEraseObject) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucSendData : Message buffer 							*/
/*	int iEraseObject		   : 0 data/address info erase					*/
/*							   : 1 program/entry info erase 				*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Flash writer : data,program erase.						*/
/****************************************************************************/
static int
c33_IcdVer3MakeMessageFlashErase(int iID, unsigned char* pucSendData, int iEraseObject)
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* The erase object is set up. */
	*pucSendData = (unsigned char)((iEraseObject >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}
/****************************************************************************/
/* Format		 : static int c33_IcdVer3MakeMessageFlashLoad(int iID, unsigned char* pucSendData, int iKind, unsigned long ulLoadAddress, unsigned long ulLen, unsigned char *pucLoadData) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucSendData : Message buffer 							*/
/*	int iKind				   : 0 load data								*/
/*							   : 1 load program 							*/
/*	unsigned long ulLoadAddress: load offset address(ICD flash) 			*/
/*	unsigned long ulLen 	   : load data length							*/
/*	unsigned char *pucLoadData : load data									*/
/*	Return value															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Flash writer : load data,program from HOST to ICD.		*/
/****************************************************************************/
static int
c33_IcdVer3MakeMessageFlashLoad(int iID, unsigned char* pucSendData, int iKind, unsigned long ulLoadAddress, unsigned long ulLen, unsigned char *pucLoadData)
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;
	unsigned long ulCnt;

	/* The elf file data is set up. 1 packet MAX 0x400 */

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* The load kind is set up. */
	*pucSendData = (unsigned char)((iKind >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* The load address is set up. */
	*pucSendData = (unsigned char)((ulLoadAddress >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulLoadAddress >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulLoadAddress >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulLoadAddress >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;


	/* The load data is set up. */
	for( ulCnt = 0; ulCnt < ulLen; ++ulCnt )
	{
		*pucSendData = *pucLoadData;
		pucSendData++;
		pucLoadData++;
		iSendDataNum++;
	}

	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}
/****************************************************************************/
/* Format		 : static int c33_IcdVer3MakeMessageFlashCopy(int iID, unsigned char* pucSendData, unsigned long ulDestAddress, unsigned long ulSrcAddress,unsigned long ulSize) */
/* Arguments																*/
/*	int iID 					: Message index 							*/
/*	unsigned char* pucSendData	: Message buffer							*/
/*	unsigned long ulDestAddress : Copy destination address (ICD flash)		*/
/*	unsigned long ulSrcAddress	: Copy source	   address (CPU flash)		*/
/*	unsigned long ulSize		: Copy byte size							*/
/*	Return value															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Flash writer : Copy data from target to ICD. 			*/
/****************************************************************************/
static int
c33_IcdVer3MakeMessageFlashCopy(int iID, unsigned char* pucSendData, unsigned long ulDestAddress, unsigned long ulSrcAddress,unsigned long ulSize)
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* (reserved) */		/* add T.Tazaki 2005/01/14 bug fix >>> */
	*pucSendData = 0;
	pucSendData++;
	iSendDataNum++;		/* add T.Tazaki 2005/01/14 bug fix <<< */

	/* The copy destination address is set up. */
	*pucSendData = (unsigned char)((ulDestAddress >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulDestAddress >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulDestAddress >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulDestAddress >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* The copy source address is set up. */
	*pucSendData = (unsigned char)((ulSrcAddress >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulSrcAddress >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulSrcAddress >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulSrcAddress >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* The copy byte size is set up. */
	*pucSendData = (unsigned char)((ulSize >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulSize >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulSize >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulSize >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;


	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}
/****************************************************************************/
/* Format		 : static int c33_IcdVer3MakeMessageFlashDataInfoSet(int iID, unsigned char* pucSendData, unsigned long ulIcdDataAddress, unsigned long ulCpuDataAddress,unsigned long ulSize, */
/*							  unsigned long ulEraseStartBlock, unsigned long ulEraseEndBlock,unsigned long ulEraseParam,char *pComment */
/* Arguments																*/
/*	int iID 					   : Message index							*/
/*	unsigned char* pucSendData	   : Message buffer 						*/
/*	unsigned long ulIcdDataAddress : ICD destination address(ICD flash) 	*/
/*	unsigned long ulCpuDataAddress : Write/Verify address	(CPU flash) 	*/
/*	unsigned long ulSize		   : Write/Verify byte size 				*/
/*	unsigned long ulEraseStartBlock: Erase start block						*/
/*	unsigned long ulEraseEndBlock  : Erase end block						*/
/*	unsigned long ulEraseParam	   : Erase parameter						*/
/*	unsigned char *pComment 	   : Comment								*/
/*	Return value															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Flash writer : Address infomation set.					*/
/****************************************************************************/
static int
c33_IcdVer3MakeMessageFlashDataInfoSet(int iID, unsigned char* pucSendData, unsigned long ulIcdDataAddress, unsigned long ulCpuDataAddress,unsigned long ulSize,
							unsigned long ulEraseStartBlock, unsigned long ulEraseEndBlock,unsigned long ulEraseParam,unsigned char *pComment )
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;
	int i;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* area count */
	*pucSendData = 1;
	pucSendData++;
	iSendDataNum++;

	/* (reserved) */
	*pucSendData = 0;
	pucSendData++;
	iSendDataNum++;

	/* The ICD destination address is set up. */
	*pucSendData = (unsigned char)((ulIcdDataAddress >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulIcdDataAddress >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulIcdDataAddress >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulIcdDataAddress >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* The Write/Verify address is set up. */
	*pucSendData = (unsigned char)((ulCpuDataAddress >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulCpuDataAddress >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulCpuDataAddress >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulCpuDataAddress >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* The Write/Verify byte size is set up. */
	*pucSendData = (unsigned char)((ulSize >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulSize >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulSize >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulSize >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* The Erase start block is set up. */
	*pucSendData = (unsigned char)((ulEraseStartBlock >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulEraseStartBlock >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulEraseStartBlock >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulEraseStartBlock >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* The Erase end block is set up. */
	*pucSendData = (unsigned char)((ulEraseEndBlock >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulEraseEndBlock >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulEraseEndBlock >>	8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulEraseEndBlock >>	0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* The Erase parameter is set up. */
	*pucSendData = (unsigned char)((ulEraseParam >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulEraseParam >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulEraseParam >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulEraseParam >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	if( pComment == 0 )
	{
		*pucSendData = 0;
		pucSendData++;
		iSendDataNum++;
	}
	else
	{
		/* The comment is set up. */
		for( i = 0; i < 128; ++i )
		{
			if( *pComment == 0 )
			{
				*pucSendData = 0;
				pucSendData++;
				iSendDataNum++;
				break;
			}
			*pucSendData = *pComment;
			pucSendData++;
			iSendDataNum++;
			++pComment;
		}
	}
	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}

/****************************************************************************/
/* Format		 : static int c33_IcdVer3MakeMessageFlashEntryInfoSet(int iID, unsigned char* pucSendData, unsigned long ulIcdProgAddress, unsigned long ulCpuProgAddress,unsigned long ulProgSize, */
/*							  unsigned long ulEraseEntryAddress, unsigned long ulWriteEntryAddress, char *pComment) */
/* Arguments																*/
/*	int iID 						  : Message index						*/
/*	unsigned char* pucSendData		  : Message buffer						*/
/*	unsigned long ulIcdProgAddress	  : ICD destination address(ICD flash)	*/
/*	unsigned long ulCpuProgAddress	  : CPU destination address(CPU flash)	*/
/*	unsigned long ulProgSize		  : Program byte size					*/
/*	unsigned long ulEraseEntryAddress : CPU Erase entry address(CPU RAM)	*/
/*	unsigned long ulWriteEntryAddress : CPU Write entry address(CPU RAM)	*/
/*	char *pComment					  : Comment 							*/
/*	Return value															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Flash writer : Write/Erase routin Entry infomation set.	*/
/****************************************************************************/
static int
c33_IcdVer3MakeMessageFlashEntryInfoSet(int iID, unsigned char* pucSendData, unsigned long ulIcdProgAddress, unsigned long ulCpuProgAddress,unsigned long ulProgSize,
							unsigned long ulEraseEntryAddress, unsigned long ulWriteEntryAddress, char *pComment )
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;
	int i;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* entry count */
	*pucSendData = 1;
	pucSendData++;
	iSendDataNum++;

	/* (reserved) */
	*pucSendData = 0;
	pucSendData++;
	iSendDataNum++;

	/* The ICD destination address is set up. */
	*pucSendData = (unsigned char)((ulIcdProgAddress >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulIcdProgAddress >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulIcdProgAddress >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulIcdProgAddress >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* The CPU destination address is set up. */
	*pucSendData = (unsigned char)((ulCpuProgAddress >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulCpuProgAddress >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulCpuProgAddress >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulCpuProgAddress >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* The program byte size is set up. */
	*pucSendData = (unsigned char)((ulProgSize >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulProgSize >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulProgSize >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulProgSize >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* The Erase entry address is set up. */
	*pucSendData = (unsigned char)((ulEraseEntryAddress >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulEraseEntryAddress >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulEraseEntryAddress >>	8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulEraseEntryAddress >>	0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* The Write entry address is set up. */
	*pucSendData = (unsigned char)((ulWriteEntryAddress >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulWriteEntryAddress >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulWriteEntryAddress >>	8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulWriteEntryAddress >>	0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* (reserved) */
	*pucSendData = 0;
	pucSendData++;
	iSendDataNum++;
	*pucSendData = 0;
	pucSendData++;
	iSendDataNum++;
	*pucSendData = 0;
	pucSendData++;
	iSendDataNum++;
	*pucSendData = 0;
	pucSendData++;
	iSendDataNum++;

	if( pComment == 0 )
	{
		*pucSendData = 0;
		pucSendData++;
		iSendDataNum++;
	}
	else
	{
		/* The comment is set up. */
		for( i = 0; i < 128; ++i )
		{
			if( *pComment == 0 )
			{
				*pucSendData = 0;
				pucSendData++;
				iSendDataNum++;
				break;
			}
			*pucSendData = *pComment;
			pucSendData++;
			iSendDataNum++;
			++pComment;
		}
	}
	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}
/****************************************************************************/
/* Format		 : static int c33_IcdVer3MakeMessageDebugBaseSet(int iID, unsigned char* pucSendData ) */
/* Arguments																*/
/*	int iID 						  : Message index						*/
/*	unsigned char* pucSendData		  : Message buffer						*/
/*	Return value															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Set Cpu type and debug unit base address to ICD V3.		*/
/****************************************************************************/
static int
c33_IcdVer3MakeMessageDebugBaseSet(int iID, unsigned char* pucSendData )
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* (reserved) */
	*pucSendData = 0;
	pucSendData++;
	iSendDataNum++;

	/* ROM address is set up. */
	*pucSendData = (unsigned char)((c33TargetInfo.ulDebugBase_rom_addr >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((c33TargetInfo.ulDebugBase_rom_addr >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((c33TargetInfo.ulDebugBase_rom_addr >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((c33TargetInfo.ulDebugBase_rom_addr >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* RAM address is set up. */
	*pucSendData = (unsigned char)((c33TargetInfo.ulDebugBase_ram_addr >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((c33TargetInfo.ulDebugBase_ram_addr >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((c33TargetInfo.ulDebugBase_ram_addr >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((c33TargetInfo.ulDebugBase_ram_addr >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* CPU type is set up. */
	*pucSendData = c33TargetInfo.ucCpuType;
	pucSendData++;
	iSendDataNum++;

	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}
/****************************************************************************/
/* Format		 : static int c33_IcdVer3MakeMessageOscWaitSet(int iID, unsigned char* pucSendData, unsigned long ulOscWait) */
/* Arguments																*/
/*	int iID 						  : Message index						*/
/*	unsigned char* pucSendData		  : Message buffer						*/
/*  unsigned long ulOscWait           : Osc wait counter					*/
/*	Return value															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Set OSC wait counter for ICD V3/4 firmware.        		*/
/****************************************************************************/
static int
c33_IcdVer3MakeMessageOscWaitSet(int iID, unsigned char* pucSendData, unsigned long ulOscWait )
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* (reserved) */
	*pucSendData = 0;
	pucSendData++;
	iSendDataNum++;

	/* Wait counter is set up. */
	*pucSendData = (unsigned char)((ulOscWait >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulOscWait >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulOscWait >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulOscWait >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}
/****************************************************************************/
/* Format		 : static int c33_IcdVer3MakeMessageAreaBreakSet(int iID, unsigned char* pucSendData ) */
/* Arguments																*/
/*	int iID 						  : Message index						*/
/*	unsigned char* pucSendData		  : Message buffer						*/
/*	Return value															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Set area break condition to ICD V3. advanced macro.		*/
/****************************************************************************/
static int
c33_IcdVer3MakeMessageAreaBreakSet(int iID, unsigned char* pucSendData )
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* (reserved) */
	*pucSendData = 0;
	pucSendData++;
	iSendDataNum++;

	/* BBCU bus select */
	*pucSendData = (unsigned char)(c33TargetInfo.cArea_BBCUBusSelect);
	pucSendData++;
	iSendDataNum++;
	
	/* (reserved) */
	*pucSendData = 0;
	pucSendData++;
	iSendDataNum++;

	/* Read/Write */
	*pucSendData = (unsigned char)(c33TargetInfo.iArea_ReadWrite & 0xff);
	pucSendData++;
	iSendDataNum++;
	
	/* Area(0-7)  */
	*pucSendData = (unsigned char)(c33TargetInfo.ucArea1);
	pucSendData++;
	iSendDataNum++;

	/* Area(8-15) */
	*pucSendData = (unsigned char)(c33TargetInfo.ucArea2);
	pucSendData++;
	iSendDataNum++;
	
	/* Area(16-22)*/
	*pucSendData = (unsigned char)(c33TargetInfo.ucArea3);
	pucSendData++;
	iSendDataNum++;

	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}
/****************************************************************************/
/* Format		 : static int c33_IcdVer3MakeMessageBusBreakSet(int iID, unsigned char* pucSendData ) */
/* Arguments																*/
/*	int iID 						  : Message index						*/
/*	unsigned char* pucSendData		  : Message buffer						*/
/*	Return value															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Set bus	break condition to ICD V3. advanced macro.		*/
/****************************************************************************/
static int
c33_IcdVer3MakeMessageBusBreakSet(int iID, unsigned char* pucSendData )
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* bus select */
	*pucSendData = (unsigned char)(c33TargetInfo.iBus_BusSelect & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* BBCU bus select */
//	  *pucSendData = (unsigned char)(c33TargetInfo.cArea_BBCUBusSelect);	/* bug fix 2004/06/28 T.Tazaki */
	*pucSendData = (unsigned char)(c33TargetInfo.cBus_BBCUBusSelect);
	pucSendData++;
	iSendDataNum++; 	

	/* Trig/Break */
	*pucSendData = (unsigned char)(c33TargetInfo.iBus_TrigBreak & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* Read/Write */
	*pucSendData = (unsigned char)(c33TargetInfo.iBus_ReadWrite & 0xff);
	pucSendData++;
	iSendDataNum++;
	
	/* Inst/Data */
	*pucSendData = (unsigned char)(c33TargetInfo.iBus_InstData & 0xff);
	pucSendData++;
	iSendDataNum++;
	
	/* Bus access */
	*pucSendData = (unsigned char)(c33TargetInfo.cBus_Access);
	pucSendData++;
	iSendDataNum++;

	/* interrupt condition */
	*pucSendData = (unsigned char)(c33TargetInfo.iBus_Int & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* interrupt level */
	*pucSendData = (unsigned char)(c33TargetInfo.iBus_IntLevel & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* data compare */
	*pucSendData = (unsigned char)(c33TargetInfo.iBus_DataComp & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* break address */
	*pucSendData = (unsigned char)((c33TargetInfo.ulBus_Address >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((c33TargetInfo.ulBus_Address >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((c33TargetInfo.ulBus_Address >>	8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((c33TargetInfo.ulBus_Address >>	0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* break address mask */
	*pucSendData = (unsigned char)((c33TargetInfo.ulBus_AddressMask >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((c33TargetInfo.ulBus_AddressMask >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((c33TargetInfo.ulBus_AddressMask >>	8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((c33TargetInfo.ulBus_AddressMask >>	0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* break data */
	*pucSendData = (unsigned char)((c33TargetInfo.ulBus_Data >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((c33TargetInfo.ulBus_Data >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((c33TargetInfo.ulBus_Data >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((c33TargetInfo.ulBus_Data >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* break data mask */
	*pucSendData = (unsigned char)((c33TargetInfo.ulBus_DataMask >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((c33TargetInfo.ulBus_DataMask >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((c33TargetInfo.ulBus_DataMask >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((c33TargetInfo.ulBus_DataMask >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* break counter */
	*pucSendData = (unsigned char)((c33TargetInfo.ulBus_Count >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((c33TargetInfo.ulBus_Count >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((c33TargetInfo.ulBus_Count >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((c33TargetInfo.ulBus_Count >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}
/****************************************************************************/
/* Format		 : static int c33_IcdVer3MakeMessageBusTraceSet(int iID, unsigned char* pucSendData ) */
/* Arguments																*/
/*	int iID 						  : Message index						*/
/*	unsigned char* pucSendData		  : Message buffer						*/
/*	Return value															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Set bus trace  condition to ICD V3. advanced macro.		*/
/****************************************************************************/
static int
c33_IcdVer3MakeMessageBusTraceSet(int iID, unsigned char* pucSendData )
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* bus select */
//	  *pucSendData = (unsigned char)(c33TargetInfo.iBust_BusSelect & 0xff);
	*pucSendData = (unsigned char)(c33TargetInfo.iBus_BusSelect & 0xff);  /* change bus select is common T.Tazaki 2004/06/25 */
	pucSendData++;
	iSendDataNum++;

	/* BBCU bus select */
	*pucSendData = (unsigned char)(c33TargetInfo.cBust_BBCUBusSelect);
	pucSendData++;
	iSendDataNum++;

	/* Trace mode select */
	*pucSendData = (unsigned char)(c33TargetInfo.iBust_TraceMode);
	pucSendData++;
	iSendDataNum++;

	/* Read/Write */
	*pucSendData = (unsigned char)(c33TargetInfo.iBust_ReadWrite & 0xff);
	pucSendData++;
	iSendDataNum++;
	
	/* Inst/Data */
	*pucSendData = (unsigned char)(c33TargetInfo.iBust_InstData & 0xff);
	pucSendData++;
	iSendDataNum++;
	
	/* Bus access */
	*pucSendData = (unsigned char)(c33TargetInfo.cBust_Access);
	pucSendData++;
	iSendDataNum++;

	/* bus output condition */
	*pucSendData = (unsigned char)(c33TargetInfo.iBust_BusOutput & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* address bus size */
	*pucSendData = (unsigned char)(c33TargetInfo.iBust_AddrBusSize & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* data/address compress */
	*pucSendData = (unsigned char)(c33TargetInfo.iBust_Compress & 0xff);
	pucSendData++;
	iSendDataNum++;

	/* user input */
	*pucSendData = (unsigned char)(c33TargetInfo.iBust_UserInput & 0xff);
	pucSendData++;
	iSendDataNum++;


	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}
/* add T.Tazaki 2007/07/03 :ICDV6 new command "c33 timebrk" >>> */
/****************************************************************************/
/* Format		 : static int c33_IcdMakeMessageSetTimerBreak(int iID, unsigned char* pucSendData,  int iFlag, unsigned long ulTimer ) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucSendData : Message buffer 							*/
/*	int iFlag				   : break timer flag on(=1), off(=0)			*/
/*	unsigned long ulTimer	   : break timer (32bit counter)				*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Set / Clear Break timer.          					    */
/****************************************************************************/
static int
c33_IcdMakeMessageSetTimerBreak(int iID, unsigned char* pucSendData, int iFlag, unsigned long ulTimer )
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;

	pucSendData++;
	pucSendData++;
	*pucSendData = (unsigned char)iID;
	pucSendData++;
	iSendDataNum++;

	/* Set break status flag */
	*pucSendData = (unsigned char)iFlag;
	pucSendData++;
	iSendDataNum++;

	/* Set break timer */
	*pucSendData = 0;
	pucSendData++;
	iSendDataNum++;
	*pucSendData = 0;
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulTimer >> 24) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulTimer >> 16) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulTimer >>  8) & 0xff);
	pucSendData++;
	iSendDataNum++;
	*pucSendData = (unsigned char)((ulTimer >>  0) & 0xff);
	pucSendData++;
	iSendDataNum++;

	*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
	*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

	c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

	iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */

	return(iSendDataNum);
}
/* add T.Tazaki 2007/07/03 <<< */

/****************************************************************************/
/* Format		 : static int c33_IcdRecvGo (int iID, unsigned char* pucRecvData) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucRecvData : Message buffer 							*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : The execution result of a go command is stored.			*/
/****************************************************************************/
static int
c33_IcdRecvGo (int iID, unsigned char* pucRecvData)
{
	unsigned char ucRegTmp[REGISTER_SIZE];
	int iReg;
	unsigned long ulTmp;
	int iUnit;			/* Pʔԍ(0:Cycle,1:50ns,2:us) for ICDV6 add T.Tazaki 2007/07/03 */
	
	/* Receive data check */
	if( c33_IcdRecvDataCheck(iID,pucRecvData) != C33_NO_ERR)
	{
		return C33_ERR;
	}

	pucRecvData++;
	pucRecvData++;
	pucRecvData++;

	/* Preservation of ICD status. */
	c33TargetInfo.iRunMonStatus 		= ((int)*(pucRecvData + 0) << 8) + ((int)*(pucRecvData + 1) << 0);
	pucRecvData++;
	pucRecvData++;
	c33TargetInfo.iRunBreakStatus		= ((int)*(pucRecvData + 0) << 8) + ((int)*(pucRecvData + 1) << 0);
	pucRecvData++;
	pucRecvData++;
	c33TargetInfo.iRunSoftPcBreakStatus = ((int)*(pucRecvData + 0) << 0);
	pucRecvData++;
	c33TargetInfo.iRunSoftPcClear		= ((int)*(pucRecvData + 0) << 24) + ((int)*(pucRecvData + 1) << 16) + ((int)*(pucRecvData + 2) << 8) + ((int)*(pucRecvData + 3) << 0);
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;

	/* Preservation of register data.  */
	for ( iReg = R0_REGNUM ; iReg < NUM_REGS_STD ; iReg++ )
	{
		ucRegTmp[3] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[2] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[1] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[0] = *pucRecvData;
		pucRecvData++;
		if (iReg == 0)
		{
			ucRegTmp[0] &= C33_PC_MASK;
			supply_register(PC_REGNUM,ucRegTmp);
		}
		else if (iReg == 17)
		{
			ucRegTmp[0] &= C33_SP_MASK;
			supply_register(SP_REGNUM,ucRegTmp);
		}
		else if (iReg == 18)
		{
			supply_register(AHR_REGNUM,ucRegTmp);
		}
		else if (iReg == 19)
		{
			supply_register(ALR_REGNUM,ucRegTmp);
		}
		else if (iReg == 20)
		{
			supply_register(PSR_REGNUM,ucRegTmp);
		}
		else if (iReg >= 1 && iReg <= 16)
		{
			supply_register(iReg - 1,ucRegTmp);
		}
	}

	if(c33TargetInfo.iIcd_open == C33_USB_OPEN && g_iICD_kind == C33_ICDV6 ) /* for ICDV6 add T.Tazaki 2007/07/23 >>> */
	{
		/* sԑP */
		iUnit = ((int)*(pucRecvData + 0) << 24) + ((int)*(pucRecvData + 1) << 16) + ((int)*(pucRecvData + 2) << 8) + ((int)*(pucRecvData + 3) << 0);
		pucRecvData++;
		pucRecvData++;
		pucRecvData++;
		pucRecvData++;

		/* Counter 31bit*/
		c33TargetInfo.iRunCounter	= ((int)*(pucRecvData + 0) << 24) + ((int)*(pucRecvData + 1) << 16) + ((int)*(pucRecvData + 2) << 8) + ((int)*(pucRecvData + 3) << 0);
		pucRecvData++;
		pucRecvData++;
		pucRecvData++;
		pucRecvData++;

		/* Counter 32bit*/
		c33TargetInfo.iRunCounter <<= 32;
		c33TargetInfo.iRunCounter += (((int)*(pucRecvData + 0) << 24) + ((int)*(pucRecvData + 1) << 16) + ((int)*(pucRecvData + 2) << 8) + ((int)*(pucRecvData + 3) << 0));
		pucRecvData++;
		pucRecvData++;
		pucRecvData++;
		pucRecvData++;

		ulTmp =  (unsigned long) c33TargetInfo.iRunCounter;	
		/* for ICDV6 add T.Tazaki 2007/07/23 <<< */
	}
	else
	{
		/* Preservation of execution time. */
		c33TargetInfo.iRunSec1				= ((int)*(pucRecvData + 0) << 24) + ((int)*(pucRecvData + 1) << 16) + ((int)*(pucRecvData + 2) << 8) + ((int)*(pucRecvData + 3) << 0);
		pucRecvData++;
		pucRecvData++;
		pucRecvData++;
		pucRecvData++;
		c33TargetInfo.iRunSec2				= ((int)*(pucRecvData + 0) << 24) + ((int)*(pucRecvData + 1) << 16) + ((int)*(pucRecvData + 2) << 8) + ((int)*(pucRecvData + 3) << 0);
		pucRecvData++;
		pucRecvData++;
		pucRecvData++;
		pucRecvData++;
		c33TargetInfo.iRunCycle 			= ((int)*(pucRecvData + 0) << 24) + ((int)*(pucRecvData + 1) << 16) + ((int)*(pucRecvData + 2) << 8) + ((int)*(pucRecvData + 3) << 0);
		pucRecvData++;
		pucRecvData++;
		pucRecvData++;
		pucRecvData++;

		/* Preservation of parallel dumping data. */
		c33TargetInfo.iParallelDump1		= ((int)*(pucRecvData + 0) << 24) + ((int)*(pucRecvData + 1) << 16) + ((int)*(pucRecvData + 2) << 8) + ((int)*(pucRecvData + 3) << 0);
		pucRecvData++;
		pucRecvData++;
		pucRecvData++;
		pucRecvData++;
		c33TargetInfo.iParallelDump2		= ((int)*(pucRecvData + 0) << 24) + ((int)*(pucRecvData + 1) << 16) + ((int)*(pucRecvData + 2) << 8) + ((int)*(pucRecvData + 3) << 0);
		pucRecvData++;
		pucRecvData++;
		pucRecvData++;
		pucRecvData++;
		c33TargetInfo.iParallelDump3		= ((int)*(pucRecvData + 0) << 24) + ((int)*(pucRecvData + 1) << 16) + ((int)*(pucRecvData + 2) << 8) + ((int)*(pucRecvData + 3) << 0);
		pucRecvData++;
		pucRecvData++;
		pucRecvData++;
		pucRecvData++;
		c33TargetInfo.iParallelDump4		= ((int)*(pucRecvData + 0) << 24) + ((int)*(pucRecvData + 1) << 16) + ((int)*(pucRecvData + 2) << 8) + ((int)*(pucRecvData + 3) << 0);
	
		/* add clock counter */
		switch( c33TargetInfo.iClockFunction )
		{
			case 2: /* Time micro sec display */
					ulTmp =  (unsigned long) c33TargetInfo.iRunSec1;	/* Unit:50ns */
					break;
			case 3: /* Time Sec display */
					ulTmp =  (unsigned long) c33TargetInfo.iRunSec2;	/* Unit:1us */
					break;
			case 1: /* Cycle display */
			default:
					ulTmp =  (unsigned long) c33TargetInfo.iRunCycle;	/* Unit:cycle */
					ulTmp *= 4;	/* bug fix	add T.Tazaki 2004/06/14 */
					break;
		}
	}

	if( c33TargetInfo.iClockMode == 1 ) /* counter reset mode ? */
	{
		g_ulClockCount[c33TargetInfo.iClockFunction - 1] = ulTmp;
	}
	else
	{
		g_ulClockCount[c33TargetInfo.iClockFunction - 1] += ulTmp;
	}
	return C33_NO_ERR;
}

/****************************************************************************/
/* Format		 : static int c33_IcdRecvRunMon (int iID, unsigned char* pucRecvData) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucRecvData : Message buffer 							*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : The execution result of a on the fly command is stored.	*/
/****************************************************************************/
static int
c33_IcdRecvRunMon (int iID, unsigned char* pucRecvData)
{
	unsigned char ucRegTmp[REGISTER_SIZE];
	int iReg;
	
	/* Receive data check */
	if( c33_IcdRecvDataCheck(iID,pucRecvData) != C33_NO_ERR)
	{
		return C33_ERR;
	}

	pucRecvData++;
	pucRecvData++;
	pucRecvData++;

	/* add T.Tazaki 2005/08/02 */
	c33TargetInfo.iRunMonStatus = ((int)*(pucRecvData + 0) << 8) + ((int)*(pucRecvData + 1) << 0);

	/* Skip break information. */
	pucRecvData += 9;

	/* Skip register information. */
	pucRecvData += NUM_REGS_STD * REGISTER_SIZE;

	/* Preservation of execution time. */
	c33TargetInfo.iRunSec1	= ((int)*(pucRecvData + 0) << 24) + ((int)*(pucRecvData + 1) << 16) + ((int)*(pucRecvData + 2) << 8) + ((int)*(pucRecvData + 3) << 0);
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;
	c33TargetInfo.iRunSec2	= ((int)*(pucRecvData + 0) << 24) + ((int)*(pucRecvData + 1) << 16) + ((int)*(pucRecvData + 2) << 8) + ((int)*(pucRecvData + 3) << 0);
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;
	c33TargetInfo.iRunCycle = ((int)*(pucRecvData + 0) << 24) + ((int)*(pucRecvData + 1) << 16) + ((int)*(pucRecvData + 2) << 8) + ((int)*(pucRecvData + 3) << 0);
	/*
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;
	*/
	return C33_NO_ERR;
}

/****************************************************************************/
/* Format		 : static int c33_IcdRecvRegisterRead (int iID, unsigned char* pucRecvData) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucRecvData : Message buffer 							*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : The received register read data is stored.				*/
/****************************************************************************/
static int
c33_IcdRecvRegisterRead (int iID, unsigned char* pucRecvData)
{
	int iReg;
	unsigned char ucRegTmp[REGISTER_SIZE];
	unsigned char ucRegSPTmp[REGISTER_SIZE];	/* Save %SP   add T.Tazaki 2003/12/05 */
	int iSVflag = 0;							/* add T.Tazaki 2003/12/05 */

	/* Receive data check */
	if( c33_IcdRecvDataCheck(iID,pucRecvData) != C33_NO_ERR)
	{
		return C33_ERR;
	}

	pucRecvData++;
	pucRecvData++;
	pucRecvData++;

	/* Skip monitor starus */
	pucRecvData++;
	pucRecvData++;

	/* Preservation of register data.  */
	for ( iReg = R0_REGNUM ; iReg < NUM_REGS_STD ; iReg++ )
	{
		ucRegTmp[3] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[2] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[1] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[0] = *pucRecvData;
		pucRecvData++;
		if (iReg == 0)
		{
			ucRegTmp[0] &= C33_PC_MASK;
			supply_register(PC_REGNUM,ucRegTmp);
		}
		else if (iReg == 17)
		{
			ucRegTmp[0] &= C33_SP_MASK;
			supply_register(SP_REGNUM,ucRegTmp);
			
			/* Save %SP   add T.Tazaki 2003/12/05 >>> */
			ucRegSPTmp[0] = ucRegTmp[0];
			ucRegSPTmp[1] = ucRegTmp[1];
			ucRegSPTmp[2] = ucRegTmp[2];
			ucRegSPTmp[3] = ucRegTmp[3];
			/* Save %SP   add T.Tazaki 2003/12/05 <<< */
		}
		else if (iReg == 18)
		{
			supply_register(AHR_REGNUM,ucRegTmp);
		}
		else if (iReg == 19)
		{
			supply_register(ALR_REGNUM,ucRegTmp);
		}
		else if (iReg == 20)
		{
			supply_register(PSR_REGNUM,ucRegTmp);
			
			/* check %PSR  SV FLAG	add T.Tazaki 2003/12/05 >>> */
			if ( ucRegTmp[1] & 0x10 ){
				iSVflag = 1;			/* User Mode */
			}
			else
			{
				iSVflag = 0;			/* Supervisor Mode */
			}
			g_iIcdSVflag = iSVflag;		/* add T.Tazaki 2004/04/27 : ICDV3 Adv , GUI mode RegWindow ssp,usp */
			/* check %PSR  SV FLAG	add T.Tazaki 2003/12/05 <<< */
			
		}
		else if (iReg >= 1 && iReg <= 16)
		{
			supply_register(iReg - 1,ucRegTmp);
		}
	}


   supply_register(SSP_REGNUM,ucRegSPTmp);	/* add T.Tazaki 2004/05/06 */

#if 0 /* del T.Tazaki 2004/05/06 >>> */
	/* Set %ssp or %usp   add T.Tazaki 2003/12/05 >>> */
	if( iSVflag == 0 )	
	{
		supply_register(SSP_REGNUM,ucRegSPTmp);
	}
	else
	{
		supply_register(USP_REGNUM,ucRegSPTmp);/* del T.Tazaki 2004/05/06 */
	}
	/* Set %ssp or %usp   add T.Tazaki 2003/12/05 <<< */
#endif /* del T.Tazaki 2004/05/06 <<< */
	return C33_NO_ERR;
}

/****************************************************************************/
/* Format		 : static int c33_IcdRecvDataRead (int iID, unsigned char* pucRecvData, int iLen, unsigned char* pMyAddr) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucRecvData : Message buffer 							*/
/*	int iLen				   : Data Length								*/
/*	unsigned char* pMyAddr	   : Debugger address							*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : The received data read data is stored.					*/
/****************************************************************************/
static int
c33_IcdRecvDataRead (int iID, unsigned char* pucRecvData, int iLen, unsigned char* pMyAddr)
{
	int iSize;
	/* Receive data check */
	if( c33_IcdRecvDataCheck(iID,pucRecvData) != C33_NO_ERR)
	{
		return C33_ERR;
	}

	pucRecvData++;
	pucRecvData++;
	pucRecvData++;

	for(iSize = 0 ; iSize < iLen ; iSize++)
	{
		*pMyAddr = *pucRecvData;
		pMyAddr++;
		pucRecvData++;
	}

	return C33_NO_ERR;
}

/****************************************************************************/
/* Format		 : static int c33_IcdRecvFlashErase (int iID, unsigned char* pucRecvData, int* piResult) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucRecvData : Message buffer 							*/
/*	int* piResult			   : Flash erase result 						*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : The received flash erase data is stored. 				*/
/****************************************************************************/
static int
c33_IcdRecvFlashErase (int iID, unsigned char* pucRecvData, int* piResult)
{
	/* Receive data check */
	if( c33_IcdRecvDataCheck(iID,pucRecvData) != C33_NO_ERR)
	{
		return C33_ERR;
	}

	pucRecvData++;
	pucRecvData++;
	pucRecvData++;

	*piResult = ((int)*(pucRecvData + 0) << 24) + ((int)*(pucRecvData + 1) << 16) + ((int)*(pucRecvData + 2) << 8) + ((int)*(pucRecvData + 3) << 0);

	return C33_NO_ERR;
}

/****************************************************************************/
/* Format		 : static int c33_IcdRecvGetTraceDataSize (int iID, unsigned char* pucRecvData, unsigned long* pulDataSize) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucRecvData : Message buffer 							*/
/*	unsigned long* pulDataSize : Data size receive pointer					*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : The received data size is stored in pulDataSize. 		*/
/****************************************************************************/
static int
c33_IcdRecvGetTraceDataSize (int iID, unsigned char* pucRecvData, unsigned long* pulDataSize)
{
	/* Receive data check */
	if( c33_IcdRecvDataCheck(iID,pucRecvData) != C33_NO_ERR)
	{
		return C33_ERR;
	}

	pucRecvData++;
	pucRecvData++;
	pucRecvData++;

	*pulDataSize = ((unsigned long)*(pucRecvData + 0) << 24) + ((unsigned long)*(pucRecvData + 1) << 16) + ((unsigned long)*(pucRecvData + 2) << 8) + ((unsigned long)*(pucRecvData + 3) << 0);

	return C33_NO_ERR;
}

/****************************************************************************/
/* Format		 : static int c33_IcdRecvTraceReadData (int iID, unsigned char* pucRecvData, unsigned long ulStartCycle, unsigned long ulReadSize, unsigned long ulCycleMax, unsigned char* pucDst) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucRecvData : Message buffer 							*/
/*	unsigned long ulStartCycle : Start cycle								*/
/*	unsigned long ulReadSize   : Read size									*/
/*	unsigned long ulCycleMax   : cycle max number							*/
/*	unsigned char* pucDst	   : Data receive pointer						*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : The received trace data is stored in pucDst. 			*/
/****************************************************************************/
static int
c33_IcdRecvTraceReadData (int iID, unsigned char* pucRecvData, unsigned long ulStartCycle, unsigned long ulReadSize, unsigned long ulCycleMax, unsigned char* pucDst)
{
	unsigned char ucHi; /* hi bit or low bit */
	unsigned long l;

	/* Receive data check */
	if( c33_IcdRecvDataCheck(iID,pucRecvData) != C33_NO_ERR)
	{
		return C33_ERR;
	}

	pucRecvData++;
	pucRecvData++;
	pucRecvData++;

	ucHi = 1;	
	for(l = 0 ; l < ulReadSize ; l++)
	{
		if(ucHi == 1)
		{
			pucDst[ulCycleMax - ulStartCycle - l - 1] = *pucRecvData >> 4;
			ucHi = 0;
		}
		else
		{
			pucDst[ulCycleMax - ulStartCycle - l - 1] = *pucRecvData & 0xf;
			ucHi = 1;
			pucRecvData++;
		}
	}

	return C33_NO_ERR;
}
/****************************************************************************/
/* Format		 : static int c33_IcdRecvBusTraceReadData (int iID, unsigned char* pucRecvData, unsigned long ulStartCycle, unsigned long ulReadCycle, unsigned long ulCycleMax, unsigned short* pusData) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucRecvData : Message buffer 							*/
/*	unsigned long ulStartCycle : Start cycle								*/
/*	unsigned long ulReadCycle  : Read cycle 								*/
/*	unsigned long ulCycleMax   : cycle max number							*/
/*	unsigned short* pusData    : Data receive pointer						*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : The received Bus trace data is stored in pusData.		*/
/****************************************************************************/
static int
c33_IcdRecvBusTraceReadData (int iID, unsigned char* pucRecvData, unsigned long ulStartCycle, unsigned long ulReadCycle, unsigned long ulCycleMax, unsigned short* pusData)
{
	unsigned long l;
	unsigned short usRecvData;

	/* Receive data check */
	if( c33_IcdRecvDataCheck(iID,pucRecvData) != C33_NO_ERR)
	{
		return C33_ERR;
	}

	pucRecvData++;
	pucRecvData++;
	pucRecvData++;

	for(l = 0 ; l < ulReadCycle; ++l)
	{
		usRecvData = *pucRecvData << 8;
		pucRecvData++;
		usRecvData &= 0xff00;
		usRecvData |= *pucRecvData;
		pucRecvData++;
		
		pusData[ulCycleMax - ulStartCycle - l - 1] = usRecvData;
	}

	return C33_NO_ERR;
}

/****************************************************************************/
/* Format		 : static int c33_IcdRecvFlashDataInfoGet (int iID, unsigned char* pucRecvData, struct c33_flash_writer_info *c33FlashWriterInfo) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucRecvData : Message buffer 							*/
/*	struct c33_flash_writer_info *c33FlashWriterInfo : data infomatin struct pointer */
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : The received flash writer address data information is stored. */
/****************************************************************************/
static int
c33_IcdRecvFlashDataInfoGet (int iID, unsigned char* pucRecvData, struct c33_flash_writer_info *c33FlashWriterInfo)
{
	int iAreaCount;
	int i;
	
	/* Receive data check */
	if( c33_IcdRecvDataCheck(iID,pucRecvData) != C33_NO_ERR)
	{
		return C33_ERR;
	}

	pucRecvData++;
	pucRecvData++;
	pucRecvData++;

	iAreaCount = (int)*pucRecvData;
	if( iAreaCount == 0 )	/* not set area infomation */
	{
		return C33_ERR;
	}
	pucRecvData++;

	/* (reserved) */
	pucRecvData++;

	c33FlashWriterInfo->ulIcdDataAddress = ((unsigned long)*(pucRecvData + 0) << 24) + ((unsigned long)*(pucRecvData + 1) << 16) + ((unsigned long)*(pucRecvData + 2) << 8) + ((unsigned long)*(pucRecvData + 3) << 0);
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;

	c33FlashWriterInfo->ulCpuDataAddress = ((unsigned long)*(pucRecvData + 0) << 24) + ((unsigned long)*(pucRecvData + 1) << 16) + ((unsigned long)*(pucRecvData + 2) << 8) + ((unsigned long)*(pucRecvData + 3) << 0);
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;

	c33FlashWriterInfo->ulDataSize = ((unsigned long)*(pucRecvData + 0) << 24) + ((unsigned long)*(pucRecvData + 1) << 16) + ((unsigned long)*(pucRecvData + 2) << 8) + ((unsigned long)*(pucRecvData + 3) << 0);
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;

	c33FlashWriterInfo->ulEraseParam = ((unsigned long)*(pucRecvData + 0) << 24) + ((unsigned long)*(pucRecvData + 1) << 16) + ((unsigned long)*(pucRecvData + 2) << 8) + ((unsigned long)*(pucRecvData + 3) << 0);
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;

	c33FlashWriterInfo->ulEraseStartBlock = ((unsigned long)*(pucRecvData + 0) << 24) + ((unsigned long)*(pucRecvData + 1) << 16) + ((unsigned long)*(pucRecvData + 2) << 8) + ((unsigned long)*(pucRecvData + 3) << 0);
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;

	c33FlashWriterInfo->ulEraseEndBlock = ((unsigned long)*(pucRecvData + 0) << 24) + ((unsigned long)*(pucRecvData + 1) << 16) + ((unsigned long)*(pucRecvData + 2) << 8) + ((unsigned long)*(pucRecvData + 3) << 0);
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;


	for( i = 0; i < 128; ++i )
	{
		c33FlashWriterInfo->szDataComment[i] = *pucRecvData;
		if( *pucRecvData == 0 )
		{
			break;
		}
		pucRecvData++;
	}
	c33FlashWriterInfo->szDataComment[i] = 0;
	
	return C33_NO_ERR;
}
/****************************************************************************/
/* Format		 : static int c33_IcdRecvFlashEntryInfoGet (int iID, unsigned char* pucRecvData, struct c33_flash_writer_info *c33FlashWriterInfo) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucRecvData : Message buffer 							*/
/*	struct c33_flash_writer_info *c33FlashWriterInfo : data infomatin struct pointer */
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : The received flash writer address data information is stored. */
/****************************************************************************/
static int
c33_IcdRecvFlashEntryInfoGet (int iID, unsigned char* pucRecvData, struct c33_flash_writer_info *c33FlashWriterInfo)
{
	int iEntryCount;
	int i;
	
	
	/* Receive data check */
	if( c33_IcdRecvDataCheck(iID,pucRecvData) != C33_NO_ERR)
	{
		return C33_ERR;
	}

	pucRecvData++;
	pucRecvData++;
	pucRecvData++;

	iEntryCount = (int)*pucRecvData;
	if( iEntryCount == 0 )	/* not set entry infomation */
	{
		return C33_ERR;
	}
	pucRecvData++;

	/* (reserved) */
	pucRecvData++;

	c33FlashWriterInfo->ulIcdProgAddress = ((unsigned long)*(pucRecvData + 0) << 24) + ((unsigned long)*(pucRecvData + 1) << 16) + ((unsigned long)*(pucRecvData + 2) << 8) + ((unsigned long)*(pucRecvData + 3) << 0);
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;

	c33FlashWriterInfo->ulCpuProgAddress = ((unsigned long)*(pucRecvData + 0) << 24) + ((unsigned long)*(pucRecvData + 1) << 16) + ((unsigned long)*(pucRecvData + 2) << 8) + ((unsigned long)*(pucRecvData + 3) << 0);
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;

	c33FlashWriterInfo->ulProgSize = ((unsigned long)*(pucRecvData + 0) << 24) + ((unsigned long)*(pucRecvData + 1) << 16) + ((unsigned long)*(pucRecvData + 2) << 8) + ((unsigned long)*(pucRecvData + 3) << 0);
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;

	c33FlashWriterInfo->ulEraseEntryAddress = ((unsigned long)*(pucRecvData + 0) << 24) + ((unsigned long)*(pucRecvData + 1) << 16) + ((unsigned long)*(pucRecvData + 2) << 8) + ((unsigned long)*(pucRecvData + 3) << 0);
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;

	c33FlashWriterInfo->ulWriteEntryAddress = ((unsigned long)*(pucRecvData + 0) << 24) + ((unsigned long)*(pucRecvData + 1) << 16) + ((unsigned long)*(pucRecvData + 2) << 8) + ((unsigned long)*(pucRecvData + 3) << 0);
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;

	/* (reserved) */
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;
	pucRecvData++;

	for( i = 0; i < 128; ++i )
	{
		c33FlashWriterInfo->szProgComment[i] = *pucRecvData;
		if( *pucRecvData == 0 )
		{
			break;
		}
		pucRecvData++;
	}
	c33FlashWriterInfo->szProgComment[i] = 0;
	
	return C33_NO_ERR;
}
/* add T.Tazaki 2007/07/03 >>> */
/****************************************************************************/
/* Format		 : static int c33_IcdRecvTargetReset (int iID, unsigned char* pucRecvData) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucRecvData : Message buffer 							*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : The received Ext Jump is stored.			 				*/
/****************************************************************************/
static int
c33_IcdRecvTargetReset (int iID, unsigned char* pucRecvData)
{
	/* Receive data check */
	if( c33_IcdRecvDataCheck(iID,pucRecvData) != C33_NO_ERR)
	{
		return C33_ERR;
	}

	pucRecvData++;
	pucRecvData++;
	pucRecvData++;

	g_ucTargetReset_result = *pucRecvData;

	return C33_NO_ERR;
}
/* add T.Tazaki 2007/07/03 <<< */

/****************************************************************************/
/* Format		 : int c33_IcdSendMessage (int iID, int iParam1, int iParam2, int iParam3, int iParam4, void *pvParam5) */
/* Arguments																*/
/*	int iID 	 : Command index											*/
/*	int iParam1  : Command parameter1										*/
/*	int iParam2  : Command parameter2										*/
/*	int iParam3  : Command parameter3										*/
/*	int iParam4  : Command parameter4										*/
/*	void *pvPraram5: Command parameter5 									*/
/*	int iParam6  : Command parameter6										*/
/*	int iParam7  : Command parameter7										*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : The specified command is transmitted.					*/
/****************************************************************************/
int
c33_IcdSendMessage (int iID, int iParam1, int iParam2, int iParam3, int iParam4, void *pvParam5, int iParam6, int iParam7)
{
	int iSendLength;	/* Command length */
	int iResult = C33_ERR;

	/* Target open? */
	if(c33TargetInfo.iIcd_open == C33_NOT_OPEN)
	{
		/* Target closed */
		return C33_ERR;
	}
	else if(c33TargetInfo.iIcd_open == C33_COM_OPEN)
	{
		serial_flush_output(c33_desc);
		c33TargetInfo.iErrorCode = C33_NO_ERR;
	}
	else if(c33TargetInfo.iIcd_open == C33_USB_OPEN)
	{
		c33TargetInfo.iErrorCode = C33_NO_ERR;
	}

	/* area trace mode don't use hard pc break */
	if(c33TargetInfo.c33TraceMode.iTraceMode == C33_ICD_TRACE_MODE_AREA_OVER_WRITE_NO)
	{
		if( iID == C33_ID_HARD_PC_BREAK_SET || iID == C33_ID_HARD_PC_BREAK2 || iID == C33_ID_HARD_PC_BREAK_CLR)
		{
			return C33_NO_ERR;
		}
	}

	switch (iID)
	{
		case C33_ID_OK:
		case C33_ID_CAN:
			ucSendData[0] = (unsigned char)iID;
			iSendLength = 1;
			break;
		case C33_ID_VER_READ:
		case C33_ID_INITIALIZE:
		case C33_ID_REG_READ:
		case C33_ID_SP_REG_READ:
		case C33_ID_RUN_MON:
		case C33_ID_HARD_PC_BREAK_CLR:
		case C33_ID_DATA_BREAK_CLR:
		case C33_ID_TRACE_READ_DATA_SIZE:
		case C33_ID_TRACE_RESUME:
		case C33_ID_AREA_BREAK_CLR:
		case C33_ID_BUS_BREAK_CLR:
		case C33_ID_FIRM_UPDATE:
		case C33_ID_FRASH_ADDR_INFO_GET:
		case C33_ID_FRASH_ENTRY_INFO_GET:
		case C33_ID_TARGET_RESET:		/* add T.Tazaki 2007/07/03 */
			iSendLength = c33_IcdMakeMessageID(iID, ucSendData);
			break;
		case C33_ID_RESET:
			iSendLength = c33_IcdMakeMessageReset(iID, ucSendData, iParam1);
			break;
		case C33_ID_GO:
			/* Initialize trace data */
			iSendLength = c33_IcdMakeMessageGo(iID, ucSendData, iParam1, iParam2 );
			break;
		case C33_ID_STEP:
			/* Initialize trace data */
			iSendLength = c33_IcdMakeMessageStep(iID, ucSendData, iParam1);
			break;
		case C33_ID_REG_WRITE:
			iSendLength = c33_IcdMakeMessageRegWrite(iID, ucSendData);
			break;
		case C33_ID_SP_REG_WRITE:
			iSendLength = c33_IcdMakeMessageSpRegWrite(iID, ucSendData);
			break;
		case C33_ID_DATA_WRITE_FAST:
		case C33_ID_DATA_WRITE:
			iSendLength = c33_IcdMakeMessageDataWrite(iID, ucSendData, iParam1, iParam2, iParam3, pvParam5);
			break;
		case C33_ID_ICD_MEM_WRITE:		/* add icdv3 memory write  T.Tazaki 2004/02/26 */
			iSendLength = c33_IcdMakeMessageDataWrite(iID, ucSendData, iParam1, iParam2, iParam3, pvParam5);
			break;
		case C33_ID_DATA_PARALLEL_WRITE:	/* add parallel load T.Tazaki 2003/09/30 */
			iSendLength = c33_IcdMakeMessageParlDataWrite(iID, ucSendData, iParam1, iParam2);
			break;
		case C33_ID_DATA_READ_EXACT:
		case C33_ID_DATA_READ:
			iSendLength = c33_IcdMakeMessageDataRead(iID, ucSendData, iParam1, iParam2, iParam3);
			break;
		case C33_ID_ICD_MEM_READ:		/* add icdv3 memory read  T.Tazaki 2004/02/26 */
			iSendLength = c33_IcdMakeMessageDataRead(iID, ucSendData, iParam1, iParam2, iParam3);
			break;
		case C33_ID_SOFT_PC_BREAK_SET:
			iSendLength = c33_IcdMakeMessageSoftPcBreakSet(iID, ucSendData, iParam1, iParam2);
			break;
		case C33_ID_SOFT_PC_BREAK_CLR:
			iSendLength = c33_IcdMakeMessageSoftPcBreakClear(iID, ucSendData, iParam1);
			break;
		case C33_ID_HARD_PC_BREAK_SET:
			iSendLength = c33_IcdMakeMessageHardPcBreakSet(iID, ucSendData, iParam1);
			break;
		case C33_ID_HARD_PC_BREAK2:
			iSendLength = c33_IcdMakeMessageHardPcBreak2(iID, ucSendData, iParam1);
			break;
		case C33_ID_DATA_FILL:
			iSendLength = c33_IcdMakeMessageDataFill(iID, ucSendData, iParam1, iParam2, iParam3, iParam4);
			break;
		case C33_ID_DATA_MOVE:
			iSendLength = c33_IcdMakeMessageDataMove(iID, ucSendData, iParam1, iParam2, iParam3, iParam4);
			break;
		case C33_ID_DATA_BREAK_SET:
			iSendLength = c33_IcdMakeMessageDataBreak(iID, ucSendData, iParam1, iParam2);
			break;
		case C33_ID_FLASH_SET:
			iSendLength = c33_IcdMakeMessageFlashSet(iID, ucSendData, iParam1, iParam2, iParam3, iParam4);
			break;
		case C33_ID_FLASH_ERASE:
			iSendLength = c33_IcdMakeMessageFlashErase(iID, ucSendData, iParam1, iParam2, iParam3);
			break;
		case C33_ID_TRACE_READ_DATA:
			iSendLength = c33_IcdMakeMessageTraceReadData(iID, ucSendData, iParam1, iParam2);
			break;
		case C33_ID_TRACE_TRIGGER:
			iSendLength = c33_IcdMakeMessageTraceTrgMode(iID, ucSendData, pvParam5);
			break;
		case C33_ID_ICE_COMMAND:
			iSendLength = iParam1;
			break;
		case C33_ID_MEASURE_UNIT:
			iSendLength = c33_IcdMakeMessageClockFunctionSet(iID, ucSendData, iParam1);
			break;
		case C33_ID_FRASH_ERASE:
			iSendLength = c33_IcdVer3MakeMessageFlashErase(iID, ucSendData, iParam1);
			break;
		case C33_ID_FRASH_LOAD:
			iSendLength = c33_IcdVer3MakeMessageFlashLoad(iID, ucSendData, iParam1, iParam2, iParam3, pvParam5);
			break;
		case C33_ID_FRASH_COPY:
			iSendLength = c33_IcdVer3MakeMessageFlashCopy(iID, ucSendData, iParam1, iParam2, iParam3);
			break;
		case C33_ID_FRASH_ADDR_INFO_SET:
			iSendLength = c33_IcdVer3MakeMessageFlashDataInfoSet(iID, ucSendData, iParam1, iParam2, iParam3, iParam4, iParam6, iParam7, pvParam5);
			break;
		case C33_ID_FRASH_ENTRY_INFO_SET:
			iSendLength = c33_IcdVer3MakeMessageFlashEntryInfoSet(iID, ucSendData, iParam1, iParam2, iParam3, iParam4, iParam6, pvParam5);
			break;
		case C33_ID_SET_DEBUG_BASE:
			iSendLength = c33_IcdVer3MakeMessageDebugBaseSet(iID, ucSendData);
			break;
		case C33_ID_OSCWAIT_COUNT_SET:
			iSendLength = c33_IcdVer3MakeMessageOscWaitSet(iID, ucSendData, iParam1);
			break;
		case C33_ID_AREA_BREAK_SET:
			iSendLength = c33_IcdVer3MakeMessageAreaBreakSet(iID, ucSendData);
			break;
		case C33_ID_BUS_BREAK_SET:
			iSendLength = c33_IcdVer3MakeMessageBusBreakSet(iID, ucSendData);
			break;
		case C33_ID_BUS_TRACE_SET:
			iSendLength = c33_IcdVer3MakeMessageBusTraceSet(iID, ucSendData);
			break;
		case C33_ID_BUS_TRACE_READ_DATA:
			iSendLength = c33_IcdMakeMessageBusTraceReadData(iID, ucSendData, iParam1, iParam2);
			break;
		case C33_ID_TIME_BREAK:		/* add ICDV6 set Timer break  T.Tazaki 2007/07/03 */
			iSendLength = c33_IcdMakeMessageSetTimerBreak(iID, ucSendData, iParam1, iParam2);
			break;
		default:
			c33_error (C33_ERR_MSG_COM_INVALID_ID, iID);
			return C33_ERR;
	}

	/* protocol monitor */
	if(iDebugCommunication == 1)
	{
		c33_IcdDebug(3,0,0,0);
	}
	
	/* USB monitor 		add T.Tazaki 2004/10/18 */
	if(iUsbLogCommunication == 1)
	{
		c33_UsbLog(2,0,0);
	}

	/* Transmission of a command. */
	immediate_quit++;
	if(c33TargetInfo.iIcd_open == C33_COM_OPEN)
	{
		iResult = serial_write(c33_desc, ucSendData, iSendLength);
	}
	else if(c33TargetInfo.iIcd_open == C33_USB_OPEN)
	{
		iResult = c33_IcdUsbWrite(iSendLength);
	}
	immediate_quit--;

	if(iResult)
	{
		c33TargetInfo.iErrorCode = C33_ERROR_RS_SEND;
		iResult = C33_ERR;
	}
	else
	{
		iResult = C33_NO_ERR;
	}

	return iResult;
}

/****************************************************************************/
/* Format		 : int c33_IcdRecvMessage (int iID, int iParam1, int iParam2, int iParam3, int iParam4, void *pvParam5) */
/* Arguments																*/
/*	int iID 	 : Command index											*/
/*	int iParam1  : Command parameter1										*/
/*	int iParam2  : Command parameter2										*/
/*	int iParam3  : Command parameter3										*/
/*	int iParam4  : Command parameter4										*/
/*	void *pvPraram5: Command parameter5 									*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : The specified command received.							*/
/****************************************************************************/
int
c33_IcdRecvMessage (int iID, int iParam1, int iParam2, int iParam3, int iParam4, void *pvParam5)
{
	int iResult = C33_NO_ERR;
	extern Tcl_Interp *gdbtk_interp;	/* add T.Tazaki 2006/01/12  */

	/* Target open? */
	if(c33TargetInfo.iIcd_open == C33_NOT_OPEN)
	{
		/* Target closed */
		return C33_ERR;
	}

	/* area trace mode don't use hard pc break */
	if(c33TargetInfo.c33TraceMode.iTraceMode == C33_ICD_TRACE_MODE_AREA_OVER_WRITE_NO)
	{
		if( iID == C33_ID_HARD_PC_BREAK_SET || iID == C33_ID_HARD_PC_BREAK2 || iID == C33_ID_HARD_PC_BREAK_CLR)
		{
			return C33_NO_ERR;
		}
	}

	/* Receiving data is stored in a buffer. */
	iResult = c33_IcdStoreMessage(iID, ucRecvData);/* read return massage from ICE */

	if (iResult != C33_NO_ERR)
	{
#ifdef __CYGWIN__	/* add T.Tazaki 2006/01/16  */
		if( g_use_windows == 1 && g_GdbEnable == 1)  /* GUI Mode and Not command file ? */	
		{
			g_OSMon_Semaho = 0;
			dll33_ClearCommandUpdate();
			Tcl_Eval( gdbtk_interp, "c33_ide_event_start" ); /* gdbtcl/srcbar.itcl */
		}

#endif
		iResult = C33_ERR;
	}
	else
	{
		/* Every command is processed. */
		switch (iID)
		{
			case C33_ID_VER_READ:
				iResult = c33_IcdRecvVersionRead(iID, ucRecvData);
				break;
			case C33_ID_DATA_READ_EXACT:
			case C33_ID_DATA_READ:
				iResult = c33_IcdRecvDataRead(iID, ucRecvData, iParam2, pvParam5);
				break;
			case C33_ID_ICD_MEM_READ:		/* add icdv3 memory read  T.Tazaki 2004/02/26 */
				iResult = c33_IcdRecvDataRead(iID, ucRecvData, iParam2, pvParam5);
				break;
			case C33_ID_REG_READ:
				iResult = c33_IcdRecvRegisterRead(iID, ucRecvData);
				break;
			case C33_ID_SP_REG_READ:
				iResult = c33_IcdRecvSpRegisterRead(iID, ucRecvData);
				break;
			case C33_ID_GO:
			case C33_ID_STEP:
				iResult = c33_IcdRecvGo(iID, ucRecvData);
				break;
			case C33_ID_RUN_MON:
				if ( ucRecvData[2] == C33_ID_GO )
				{
					iResult = c33_IcdRecvGo(C33_ID_GO, ucRecvData);
				}
				else
				{
					iResult = c33_IcdRecvRunMon(iID, ucRecvData);
				}
				break;
			case C33_ID_FLASH_ERASE:
				iResult = c33_IcdRecvFlashErase(iID, ucRecvData, pvParam5);
				break;
			case C33_ID_TRACE_READ_DATA_SIZE:
				iResult = c33_IcdRecvGetTraceDataSize(iID, ucRecvData, pvParam5);
				break;
			case C33_ID_TRACE_READ_DATA:
				iResult = c33_IcdRecvTraceReadData(iID, ucRecvData, iParam1, iParam2, iParam3, pvParam5);
				break;
			case C33_ID_FRASH_ADDR_INFO_GET:
				iResult = c33_IcdRecvFlashDataInfoGet(iID, ucRecvData, pvParam5);
				break;
			case C33_ID_FRASH_ENTRY_INFO_GET:
				iResult = c33_IcdRecvFlashEntryInfoGet(iID, ucRecvData, pvParam5);
				break;
			case C33_ID_TARGET_RESET:		/* add T.Tazaki 2007/07/03 */
				iResult = c33_IcdRecvTargetReset(iID, ucRecvData);
				break;
			case C33_ID_OK:
			case C33_ID_INITIALIZE:
			case C33_ID_RESET:
			case C33_ID_REG_WRITE:
			case C33_ID_SP_REG_WRITE:
			case C33_ID_DATA_WRITE:
			case C33_ID_ICD_MEM_WRITE:		/* add icdv3 memory write  T.Tazaki 2004/02/26 */
			case C33_ID_DATA_PARALLEL_WRITE:
			case C33_ID_GO_ACK:
			case C33_ID_SOFT_PC_BREAK_SET:
			case C33_ID_SOFT_PC_BREAK_CLR:
			case C33_ID_HARD_PC_BREAK_CLR:
			case C33_ID_HARD_PC_BREAK_SET:
			case C33_ID_HARD_PC_BREAK2:
			case C33_ID_DATA_FILL:
			case C33_ID_DATA_MOVE:
			case C33_ID_DATA_BREAK_CLR:
			case C33_ID_DATA_BREAK_SET:
			case C33_ID_FLASH_SET:
			case C33_ID_TRACE_RESUME:
			case C33_ID_TRACE_TRIGGER:
			case C33_ID_MEASURE_UNIT:
			case C33_ID_AREA_BREAK_CLR:
			case C33_ID_AREA_BREAK_SET:
			case C33_ID_BUS_BREAK_CLR:
			case C33_ID_BUS_BREAK_SET:
			case C33_ID_BUS_TRACE_SET:
			case C33_ID_DATA_WRITE_FAST:
			case C33_ID_FIRM_UPDATE:
			case C33_ID_FRASH_ERASE:
			case C33_ID_FRASH_LOAD:
			case C33_ID_FRASH_COPY:
			case C33_ID_FRASH_ADDR_INFO_SET:
			case C33_ID_FRASH_ENTRY_INFO_SET:
			case C33_ID_SET_DEBUG_BASE:
			case C33_ID_OSCWAIT_COUNT_SET:
			case C33_ID_ICE_COMMAND:
			case C33_ID_TIME_BREAK:		/* add ICDV6 set Timer break  T.Tazaki 2007/07/03 */
				break;
			case C33_ID_BUS_TRACE_READ_DATA:
				iResult = c33_IcdRecvBusTraceReadData(iID, ucRecvData, iParam1, iParam2, iParam3, pvParam5);
				break;
			default:
				c33_error (C33_ERR_MSG_COM_INVALID_ID, iID);
				return C33_ERR;
				break;
		}
	}

	/* protocol monitor */
	if(iDebugCommunication == 1)
	{
		c33_IcdDebug(3,1,iResult,0);
	}

	/* USB monitor 			add T.Tazaki 2004/11/28 */
	if(iUsbLogCommunication == 1)
	{
		if( iID != C33_ID_REG_READ && iID != C33_ID_SP_REG_READ && iID != C33_ID_GO && iID != C33_ID_RUN_MON 
		  && iID != C33_ID_DATA_READ && iID != C33_ID_DATA_READ_EXACT)	
		{
			c33_UsbLog(2,1,0);
		}
	}

	if(iResult != C33_NO_ERR)
	{
		if (c33TargetInfo.iIcd_open == C33_COM_OPEN)
		{
			/* It waits for 1 second. */
			c33_wait_sec(1);
			/* The flash of a serial port */
			serial_flush_input(c33_desc);
		}
		else if (c33TargetInfo.iIcd_open == C33_USB_OPEN)
		{
		}
	}

	return iResult;
}

/****************************************************************************/
/* Format		 : static int c33_IcdReadSerial (int iID, int iSize, unsigned char* pucRecvData) */
/* Arguments																*/
/*	int iID 				   : Command index								*/
/*	int iSize				   : Read size									*/
/*	unsigned char* pucRecvData : Store buffer								*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Data is read from a serial port. 						*/
/****************************************************************************/
static int
c33_IcdReadSerial (int iID, int iSize, unsigned char* pucRecvData)
{
	int iData;
	int iTimeOut;
	int iFlag;

	g_iID = iID;	/* add T.Tazaki 2004/05/11 : for STEP command hang up */

	switch (iID)
	{
		case C33_ID_STEP:
			/* add T.Tazaki 2004/05/19 : for STEP command hang up >>> */
			if ( c33TargetInfo.iIcd_open == C33_USB_OPEN )
			{
				c33_InitCancel();
				iTimeOut = 0;	/* USB : No wait */
			}
			else
			{
				iTimeOut = 1;	/* COM */
			}
			break;
			/* add T.Tazaki 2004/05/19 : for STEP command hang up <<< */
		case C33_ID_GO:
		case C33_ID_GO_ACK:
		case C33_ID_RUN_MON:
		case C33_ID_OK:
			if ( c33TargetInfo.iIcd_open == C33_USB_OPEN )
			{
				iTimeOut = 0;	/* USB : No Wait */
			}
			else
			{
				iTimeOut = 1;	/* COM */
			}
			break;
		case C33_ID_CAN:
			iTimeOut = 3;
			break;
		case C33_ID_VER_READ:
		case C33_ID_INITIALIZE:
		case C33_ID_RESET:
		case C33_ID_REG_READ:
		case C33_ID_SP_REG_READ:
		case C33_ID_REG_WRITE:
		case C33_ID_SP_REG_WRITE:
		case C33_ID_DATA_WRITE:
		case C33_ID_DATA_PARALLEL_WRITE:
		case C33_ID_DATA_READ:
		case C33_ID_ICD_MEM_READ:		/* add icdv3 memory read  T.Tazaki 2004/02/26 */
		case C33_ID_ICD_MEM_WRITE:		/* add icdv3 memory write  T.Tazaki 2004/02/26 */
		case C33_ID_SOFT_PC_BREAK_SET:
		case C33_ID_SOFT_PC_BREAK_CLR:
		case C33_ID_HARD_PC_BREAK_SET:
		case C33_ID_HARD_PC_BREAK_CLR:
		case C33_ID_HARD_PC_BREAK2:
		case C33_ID_DATA_BREAK_CLR:
		case C33_ID_DATA_BREAK_SET:
		case C33_ID_FLASH_SET:
		case C33_ID_DEF_TIMEOUT:
		case C33_ID_TRACE_READ_DATA_SIZE:
		case C33_ID_TRACE_READ_DATA:
		case C33_ID_TRACE_RESUME:
		case C33_ID_TRACE_TRIGGER:
		case C33_ID_MEASURE_UNIT:
		case C33_ID_AREA_BREAK_CLR:
		case C33_ID_AREA_BREAK_SET:
		case C33_ID_BUS_BREAK_CLR:
		case C33_ID_BUS_BREAK_SET:
		case C33_ID_BUS_TRACE_SET:
		case C33_ID_BUS_TRACE_READ_DATA:
		case C33_ID_DATA_WRITE_FAST:
		case C33_ID_DATA_READ_EXACT:
		case C33_ID_FRASH_ADDR_INFO_SET:
		case C33_ID_FRASH_ADDR_INFO_GET:
		case C33_ID_FRASH_ENTRY_INFO_SET:
		case C33_ID_FRASH_ENTRY_INFO_GET:
		case C33_ID_SET_DEBUG_BASE: 	/* add T.Tazaki 2003/06/26 */
		case C33_ID_OSCWAIT_COUNT_SET: 	/* add T.Tazaki 2003/06/26 */
		case C33_ID_TARGET_RESET:		/* add ICDV6 Target reset     T.Tazaki 2007/07/03 */
		case C33_ID_TIME_BREAK:			/* add ICDV6 set Timer break  T.Tazaki 2007/07/03 */
			iTimeOut = 6;
			break;
		case C33_ID_DATA_MOVE:
		case C33_ID_DATA_FILL:
		/* add 2003/03/18 T.Tazaki >>> */
			/* check if ICE is not ICDV3. add T.Tazaki 2003/10/15 */
			if ( (c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) != C33_ICD_VER3 )
			{
				c33_InitCancel();
				iTimeOut = 1;
			}
			else
			{
				iTimeOut = 150;	/* add T.Tazaki 2005/01/18 bug fix */
			}
//			iTimeOut = 1;		/* del T.Tazaki 2005/01/18 bug fix */
			break;
		/* add 2003/03/18 T.Tazaki <<< */
		case C33_ID_ICE_COMMAND:
//		case C33_ID_FLASH_ERASE:		/* del T.Tazaki 2006/05/16 */
		case C33_ID_FIRM_UPDATE:
		case C33_ID_FRASH_ERASE:
		case C33_ID_FRASH_LOAD:
		case C33_ID_FRASH_COPY:
			iTimeOut = 150;
			break;
			
		/* add 2006/05/16 T.Tazaki >>> */
		case C33_ID_FLASH_ERASE:
			if( g_iFleTimerFlag == 0 )
			{
				iTimeOut = 150;
			}
			else
			{
				iTimeOut = g_iFleTimer;
			}
			break;
		/* add 2006/05/16 T.Tazaki <<< */
			
		default:
			c33_error (C33_ERR_MSG_COM_INVALID_ID, iID);
			return C33_ERR;
			break;
	}

	/* It reads the specified number of times. */
	while (iSize > 0)
	{
		if ( c33TargetInfo.iIcd_open == C33_COM_OPEN )
		{
			immediate_quit++;
			iData = serial_readchar (c33_desc, iTimeOut);
			immediate_quit--;

			switch (iData)
			{
				case SERIAL_EOF:
					c33TargetInfo.iErrorCode = C33_ERROR_RS_CLOSE;
					return C33_ERR;
				case SERIAL_ERROR:
					c33TargetInfo.iErrorCode = C33_ERROR_RS_COMMUNICATION;
					return C33_ERR;
				case SERIAL_TIMEOUT:
					if (iID == C33_ID_GO || iID == C33_ID_STEP)
					{
						if ( iMonKeyBreak == C33_MONSTATUS_NG )
						{
							return C33_ERR;
						}
						else if (c33TargetInfo.iKeyBreak == 1)
						{
							c33TargetInfo.iErrorCode = C33_ERROR_RS_KEY_BREAK;
							return C33_ERR;
						}
						else
						{
							continue;
						}
					}
					/* add 2003/03/24 T.Tazaki : Check STOP button or Escape Key ON >>>  */
					if (iID == C33_ID_DATA_MOVE || iID == C33_ID_DATA_FILL)
					{
						/* check if ICE is not ICDV3. add T.Tazaki 2003/10/15 */
						if ( (c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) != C33_ICD_VER3 )
						{
							iFlag = c33_CheckCancel( 0 );
							if (c33TargetInfo.iKeyBreak == 1 || iFlag == 1 )
							{
								/* Cancel */
								c33TargetInfo.iErrorCode = C33_ERROR_RS_KEY_BREAK;
								c33_EndCancel();	/* Cancel Check End */
								return C33_ERR;
							}
							else
							{
								continue;
							}
						}
					}
					/* add 2003/03/24 T.Tazaki : Check STOP button or Escape Key ON <<<  */
					c33TargetInfo.iErrorCode = C33_ERROR_RS_TIMEOUT;
					c33TargetInfo.iIcdErrorCode = iID;
					return C33_ERR;

				default:								/* add T.Tazaki 2007/07/12 */
					break;
			}
			*pucRecvData = (unsigned char)iData;
			pucRecvData++;
			iSize--;
		}
		else if ( c33TargetInfo.iIcd_open == C33_USB_OPEN )
		{
			immediate_quit--;
			if( c33_IcdUsbRead(iSize, iTimeOut, pucRecvData) != C33_NO_ERR )
			{
				c33TargetInfo.iErrorCode = C33_ERROR_USB_RECV;
				immediate_quit--;
				/* add T.Tazaki 2004/05/11 : for STEP command hang up >>> */
			   	if( iID == C33_ID_STEP ) /* check Step command */
			   	{
					if ( (c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) == C33_ICD_VER3 )
					{
						/* Cancel */
						c33TargetInfo.iErrorCode = C33_ERROR_RS_KEY_BREAK;
					}
				}
				/* add T.Tazaki 2004/05/11 : for STEP command hang up <<< */
				return C33_ERR;
			}
			else
			{
				iSize = 0;
			}
			immediate_quit--;
		}
	}

	/* add 2003/03/19 T.Tazaki : Check Escape Key ON >>>  */
	switch (iID)
	{
		case C33_ID_DATA_MOVE:
		case C33_ID_DATA_FILL:
			/* check if ICE is not ICDV3. add T.Tazaki 2003/10/15 */
			if ( (c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) != C33_ICD_VER3 )
			{
				c33_EndCancel();	/* Cancel Check End */
			}
			break;
		/* add T.Tazaki 2004/05/11 for Step command hang up >>> */
		case C33_ID_STEP:					
			if ( (c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) == C33_ICD_VER3 )
			{
				c33_EndCancel();	/* Cancel Check End */
			}
			break;
		default:								/* add T.Tazaki 2007/07/12 */
			break;
		/* add T.Tazaki 2004/05/11 for Step command hang up <<< */
	}
	/* add 2003/03/19 T.Tazaki : Check Escape Key ON <<<  */

	return C33_NO_ERR;
}

/****************************************************************************/
/* Format		 : static int c33_IcdStoreMessage (int iID, unsigned char* pucRecvData) */
/* Arguments																*/
/*	int iID 				   : Command index								*/
/*	unsigned char* pucRecvData : Store buffer								*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Receiving data is stored in a buffer.					*/
/****************************************************************************/
static int
c33_IcdStoreMessage(int iID, unsigned char* pucRecvData)
{
	int iMessageSize; /* Size of message */
	unsigned char ucTemp;

	if (c33_IcdReadSerial (iID, 1, pucRecvData) != C33_NO_ERR )
	{
		return C33_ERR;
	}
	

	if (*pucRecvData >= C33_ICD_ACK)
	{

		/* Received control code */
		if (*pucRecvData == C33_ICD_ACK)
		{
			switch (iID)
			{
				case C33_ID_INITIALIZE:
				case C33_ID_RESET:
				case C33_ID_REG_WRITE:
				case C33_ID_SP_REG_WRITE:
				case C33_ID_DATA_WRITE:
				case C33_ID_DATA_PARALLEL_WRITE:
				case C33_ID_GO_ACK:
				case C33_ID_SOFT_PC_BREAK_SET:
				case C33_ID_SOFT_PC_BREAK_CLR:
				case C33_ID_HARD_PC_BREAK_SET:
				case C33_ID_HARD_PC_BREAK_CLR:
				case C33_ID_HARD_PC_BREAK2:
				case C33_ID_DATA_FILL:
				case C33_ID_DATA_MOVE:
				case C33_ID_DATA_BREAK_CLR:
				case C33_ID_DATA_BREAK_SET:
				case C33_ID_FLASH_SET:
				case C33_ID_TRACE_RESUME:
				case C33_ID_TRACE_TRIGGER:
				case C33_ID_MEASURE_UNIT:
				case C33_ID_AREA_BREAK_CLR:
				case C33_ID_AREA_BREAK_SET:
				case C33_ID_BUS_BREAK_CLR:
				case C33_ID_BUS_BREAK_SET:
				case C33_ID_BUS_TRACE_SET:
				case C33_ID_DATA_WRITE_FAST:
				case C33_ID_FIRM_UPDATE:
				case C33_ID_FRASH_ERASE:
				case C33_ID_FRASH_LOAD:
				case C33_ID_FRASH_COPY:
				case C33_ID_FRASH_ADDR_INFO_SET:
				case C33_ID_FRASH_ADDR_INFO_GET:
				case C33_ID_FRASH_ENTRY_INFO_SET:
				case C33_ID_FRASH_ENTRY_INFO_GET:
				case C33_ID_SET_DEBUG_BASE:
				case C33_ID_OSCWAIT_COUNT_SET:
				case C33_ID_ICE_COMMAND:
				case C33_ID_TIME_BREAK:			/* add ICDV6 set Timer break  T.Tazaki 2007/07/03 */
					break;
				default:
					c33_error (C33_ERR_MSG_COM_INVALID_ID, iID);
					return C33_ERR;
					break;
			}
		}
		else if(*pucRecvData == C33_ICD_ERR)
		{

			pucRecvData++;
			c33TargetInfo.iErrorCode = C33_ERROR_RS_ERR;
			if (c33_IcdReadSerial (C33_ID_DEF_TIMEOUT, 1, pucRecvData) != C33_NO_ERR )
			{
				return C33_ERR;
			}
			c33TargetInfo.iIcdErrorCode = *pucRecvData;
			pucRecvData++;
			if (c33TargetInfo.iIcdErrorCode == C33_ICD_ERR_PC_BREAK || c33TargetInfo.iIcdErrorCode == C33_ICD_ERR_FLASH_WRITE)
			{
				iMessageSize = 9;
			}
			else
			{
				iMessageSize = 4;
			}
			if (c33_IcdReadSerial (C33_ID_DEF_TIMEOUT, iMessageSize, pucRecvData) != C33_NO_ERR )
			{
				return C33_ERR;
			}
			memcpy(&c33TargetInfo.ucIcdErrorInfo[0],pucRecvData,iMessageSize);
			return C33_ERR;
		}
		else if( (*pucRecvData & C33_ICD_STS) == C33_ICD_STS )
		{
			if(iID == C33_ID_OK)
			{
				if (*pucRecvData & C33_BIT0)
				{
					c33TargetInfo.iErrorCode = C33_ERROR_RS_STS_BUSY;
					return C33_ERR;
				}
				else if (*pucRecvData & C33_BIT2)
				{
					c33TargetInfo.iErrorCode = C33_ERROR_RS_STS_RUN;
					return C33_ERR;
				}
				else
				{
					/* Success */
				}
			}
			else
			{

				/* add T.Tazaki 2004/05/19 : STEP Cancel >>> */
				/* 1.key break STEP 2.send CANCEL 3.when receive responce,fist 0x00(Size high) is nothing.*/
				/* Received message */
				ucTemp = *pucRecvData;
				iMessageSize = *pucRecvData;
				iMessageSize &= 0xffff;
				iMessageSize += 1; /* Add bcc size */

				*pucRecvData = 0x00;
				pucRecvData++;
				*pucRecvData = ucTemp;
				pucRecvData++;
				if (iMessageSize > (C33_COMMAND_BUFFER_SIZE - 2 - 1) )
				{
					c33TargetInfo.iErrorCode = C33_ERROR_RS_MAX_SIZE_OVER;
					return C33_ERR;
				}
				if (c33_IcdReadSerial (C33_ID_DEF_TIMEOUT, iMessageSize, pucRecvData) != C33_NO_ERR )
				{
					return C33_ERR;
				}
				return C33_NO_ERR;

				c33TargetInfo.iErrorCode = C33_ERROR_RS_RECV_STS;
				return C33_ERR;
				/* add T.Tazaki 2004/05/19 : STEP Cancel <<< */

			}
		}
		else if( (*pucRecvData & C33_ICD_NAK) == C33_ICD_NAK )
		{
			if (*pucRecvData & C33_BIT0)
			{
				c33TargetInfo.iErrorCode = C33_ERROR_RS_NAK_FLEMMING;
				return C33_ERR;
			}
			else if (*pucRecvData & C33_BIT1)
			{
				c33TargetInfo.iErrorCode = C33_ERROR_RS_NAK_BCC;
				return C33_ERR;
			}
			else if (*pucRecvData & C33_BIT2)
			{
				c33TargetInfo.iErrorCode = C33_ERROR_RS_NAK_OVERRUN;
				return C33_ERR;
			}
			else
			{
				c33TargetInfo.iErrorCode = C33_ERROR_RS_NAK_TIMEOUT;
				return C33_ERR;
			}
		}
	}
	else
	{

		/* Received message */
		iMessageSize = ((int)*pucRecvData) << 8;
		pucRecvData++;
		if (c33_IcdReadSerial (C33_ID_DEF_TIMEOUT, 1, pucRecvData) != C33_NO_ERR )
		{
			return C33_ERR;
		}
		iMessageSize += ((int)*pucRecvData) << 0;
		iMessageSize &= 0xffff;
		iMessageSize += 1; /* Add bcc size */
		pucRecvData++;
		if (iMessageSize > (C33_COMMAND_BUFFER_SIZE - 2 - 1) )
		{
			c33TargetInfo.iErrorCode = C33_ERROR_RS_MAX_SIZE_OVER;
			return C33_ERR;
		}
		if (c33_IcdReadSerial (C33_ID_DEF_TIMEOUT, iMessageSize, pucRecvData) != C33_NO_ERR )
		{
			return C33_ERR;
		}
	}
	return C33_NO_ERR;
}

/****************************************************************************/
/* Format		 : static int c33_IcdRecvDataCheck (int iID, unsigned char* pucRecvData) */
/* Arguments																*/
/*	int iID 				   : Command index								*/
/*	unsigned char* pucRecvData : Store buffer								*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : The BCC check of receiving data. 						*/
/****************************************************************************/
static int
c33_IcdRecvDataCheck(int iID, unsigned char* pucRecvData)
{
	int iRecvLen;		  /* text data length */
	unsigned char ucBcc;  /* bcc code */

	/* Get text data length */
	iRecvLen = *pucRecvData << 8;
	pucRecvData++;
	iRecvLen += *pucRecvData;
	iRecvLen &= 0xffff;
	pucRecvData++;

	/* Get bcc */
	ucBcc = 0;
	while(iRecvLen != 0)
	{
		ucBcc ^= *pucRecvData;
		pucRecvData++;
		iRecvLen--;
	}

	/* compare bcc */
	if ( ucBcc != *pucRecvData )
	{
		c33TargetInfo.iErrorCode = C33_ERROR_RS_BCC_CODE;
		return C33_ERR;
	}

	return C33_NO_ERR;
}

/****************************************************************************/
/* Format		 : void c33_IcdFlashSet(int iParamNum, unsigned long ulFlashStartAddr, unsigned long ulFlashEndAddr, unsigned long ulEraseRoutineAddr, unsigned long ulWriteRoutineAddr) */
/* Arguments																*/
/*	int iParamNum					 : Parameter number 					*/
/*	unsigned long ulFlashStartAddr	 : Flash start address					*/
/*	unsigned long ulFlashEndAddr	 : Flash end address					*/
/*	unsigned long ulEraseRoutineAddr : Flash erase routine address			*/
/*	unsigned long ulWriteRoutineAddr : Flash write routine address			*/
/* Return value  : void 													*/
/* Explanation	 : Flash memory setting.									*/
/****************************************************************************/
void
c33_IcdFlashSet(int iParamNum, unsigned long ulFlashStartAddr, unsigned long ulFlashEndAddr, unsigned long ulEraseRoutineAddr, unsigned long ulWriteRoutineAddr)
{
	int iResult;

	/* Check number of parameter */
	if ( iParamNum != 5 )
	{
		c33_error (C33_ERR_COMMAND_PARAM);
		return;
	}

	/* Check address */
	if ( ulFlashStartAddr > ulFlashEndAddr )
	{
		c33_error (C33_ERR_COMMAND_ADDR);
		return;
	}

	/* boundary matching */
	ulFlashStartAddr &= 0xfffffff0;
	ulFlashEndAddr |= 0xf;
	ulWriteRoutineAddr &= 0xfffffffe;
	ulEraseRoutineAddr &= 0xfffffffe;

	/* save parameters for detach(destroy "c33 fls" */
	save_iFlashSet			= 1;   /* ON */
	save_ulFlashStartAddr	= ulFlashStartAddr;
	save_ulFlashEndAddr 	= ulFlashEndAddr;
	save_ulEraseRoutineAddr = ulEraseRoutineAddr;
	save_ulWriteRoutineAddr = ulWriteRoutineAddr;

	/* save parameters for call command */
	c33TargetInfo.iFlashSet 		 = 1;	/* ON */
	c33TargetInfo.ulFlashStartAddr	 = ulFlashStartAddr;
	c33TargetInfo.ulFlashEndAddr	 = ulFlashEndAddr;
	c33TargetInfo.ulEraseRoutineAddr = ulEraseRoutineAddr;
	c33TargetInfo.ulWriteRoutineAddr = ulWriteRoutineAddr;

	printf_filtered (C33_MSG_FLS, (int)ulFlashStartAddr, (int)ulFlashEndAddr, (int)ulEraseRoutineAddr, (int)ulWriteRoutineAddr);

	iResult = c33_SendMessage(C33_ID_FLASH_SET, ulFlashStartAddr, ulFlashEndAddr, ulEraseRoutineAddr, ulWriteRoutineAddr, 0, 0, 0);
	if (iResult == C33_NO_ERR)
	{
		iResult = c33_RecvMessage(C33_ID_FLASH_SET, 0, 0, 0, 0, 0);
	}

	if (iResult == C33_NO_ERR)
	{
		puts_filtered (C33_MSG_TARGET_DONE);
	}
	else
	{
		puts_filtered (C33_MSG_TARGET_FAILURE);
		c33_ErrorMsg();
	}
}

/****************************************************************************/
/* Format		 : void c33_IcdFlashErase(int iParamNum, int iCtrlReg, unsigned long ulStartBlock, unsigned long ulEndBlock,int iTimer) */
/* Arguments																*/
/*	int iParamNum			   : Parameter number							*/
/*	int iCtrlReg			   : Flash control register address 			*/
/*	unsigned long ulStartBlock : Start block								*/
/*	unsigned long ulEndBlock   : End block									*/
/* int iTimer                  : Flash memory erase timeout value.(sec)     */ /* add T.Tazaki 2006/05/16 */
/* Return value  : void 													*/
/* Explanation	 : Flash memory setting.									*/
/****************************************************************************/
void
c33_IcdFlashErase(int iParamNum, int iCtrlReg, unsigned long ulStartBlock, unsigned long ulEndBlock,int iTimer )
{
	int iResult;
	int iExitCode;

	/* Check number of parameter */
//	if ( iParamNum != 4 )						/* change T.Tazaki 2006/05/16 */
	if ( iParamNum != 4 && iParamNum != 5 )		/* change T.Tazaki 2006/05/16 */
	{
		c33_error (C33_ERR_COMMAND_PARAM);
		return;
	}
	
	/* add T.Tazaki 2006/05/16 >>> */
	if ( iParamNum == 4 )
	{
		g_iFleTimerFlag = 0;
		g_iFleTimer = 0;
	}
	else
	{
		g_iFleTimerFlag = 1;
		g_iFleTimer = iTimer;
	}
	/* add T.Tazaki 2006/05/16 <<< */


	printf_filtered (C33_MSG_FLE, iCtrlReg, (int)ulStartBlock, (int)ulEndBlock);

	iResult = c33_SendMessage(C33_ID_FLASH_ERASE, iCtrlReg, ulStartBlock, ulEndBlock, 0, 0, 0, 0);
	if (iResult == C33_NO_ERR)
	{
		iResult = c33_RecvMessage(C33_ID_FLASH_ERASE, 0, 0, 0, 0, &iExitCode);
	}

	if (iResult == C33_NO_ERR)
	{
		printf_filtered (C33_MSG_FLE_END, iExitCode);
	}
	else
	{
		puts_filtered (C33_MSG_TARGET_FAILURE);
		c33_ErrorMsg();
	}
}
/****************************************************************************/
/* Format		 : void c33_IcdCall(unsigned long ulStartAddr,unsigned long ulArg1,unsigned long ulArg2, unsigned long ulArg3) */
/* Arguments																*/
/*	unsigned long ulStartAddr : Function address							*/
/*	unsigned long ulArg1	  : Argument1									*/
/*	unsigned long ulArg2	  : Argument2									*/
/*	unsigned long ulArg3	  : Argument3									*/
/* Return value  : void 													*/
/* Explanation	 : call subroutine and result output.						*/
/****************************************************************************/
void
c33_IcdCall(unsigned long ulStartAddr, unsigned long ulArg1,unsigned long ulArg2, unsigned long ulArg3)
{
	int 	 iResult = C33_NO_ERR;
	unsigned long ulReturnPacketAddr = 0;
	unsigned long ulSize = 0;
	unsigned long i;
	unsigned long ul_loop;
	char	szData[0x400];
	unsigned long ulDataTopAddr;
	unsigned char ucChar;		/* add T.Tazaki 2004/09/28 */
	FILE* fpCallfile = NULL;

	/* set call routine address by flash set message */
	iResult = c33_SendMessage(C33_ID_FLASH_SET, 0, 0, ulStartAddr, 0, 0, 0, 0);
	if (iResult == C33_NO_ERR)
	{
		iResult = c33_RecvMessage(C33_ID_FLASH_SET, 0, 0, 0, 0, 0);
	}

	if (iResult == C33_NO_ERR)
	{
		/* Function call execution by erase command */
		iResult = c33_SendMessage(C33_ID_FLASH_ERASE, ulArg1, ulArg2, ulArg3, 0, 0, 0, 0);
	}
	if (iResult == C33_NO_ERR)
	{
		iResult = c33_RecvMessage(C33_ID_FLASH_ERASE, 0, 0, 0, 0, &ulReturnPacketAddr);
	}
	
	if (iResult == C33_NO_ERR)
	{
		/* Packet pointer == -1 ? */
//		  if( ulReturnPacketAddr != 0xffffffffUL )
		if( ulReturnPacketAddr != 0xffffffffUL && g_iMMU_Mode == 0)	/* check "c33 mmu" mode OFF ?	T.Tazaki 2004/08/03 */
		{
			if( c33TargetInfo.iCallMode == 2 || c33TargetInfo.iCallMode == 3 )
			{
				/* output to file */
				fpCallfile = fopen(c33TargetInfo.cCallFileName, "wb");
				if (fpCallfile == NULL)
				{
					/* file open error */
					c33_error (C33_ERR_FILE_OPEN, c33TargetInfo.cCallFileName);
					return;
				}
			}

			/* Get Packet size */
			iResult = c33_Cmd_ReadWrite(C33_READ, ulReturnPacketAddr, C33_WORD, C33_WORD, (unsigned char*)&ulSize);
			if (iResult != C33_NO_ERR)
			{
			   c33_ErrorMsg();
			}

			/* Set Data Top address */
			ulDataTopAddr = ulReturnPacketAddr + 4;

			/* Data Output loop */
			while( 1 )
			{
				if( ulSize == 0 ){		/* add T.Tazaki 2007/07/11 */
					break;	//exit while
				}

				if( ulSize >= 0x400 ){
					ul_loop = 0x400;
					ulSize -= 0x400;
				}else{
					ul_loop = ulSize;
					ulSize = 0;
				}

				iResult = c33_Cmd_ReadWrite(C33_READ, ulDataTopAddr, ul_loop, C33_BYTE, (unsigned char*)szData );
				if (iResult != C33_NO_ERR)
				{
				   c33_ErrorMsg();
				}

				ulDataTopAddr += ul_loop;		//update read address

				/* Get Packet data */
				for ( i = 0; i < ul_loop; ++i )
				{
					/* No ANK, change to '.'	add T.Tazaki 2004/09/28 >>> */
					ucChar = (unsigned char)szData[i];
					if( (ucChar >= 0 && ucChar <= 0x1f) || (ucChar >= 0x7f && ucChar <= 0xa0) || (ucChar >= 0xe0 && ucChar <= 0xff) )
					{
						if( ucChar != 0x0a )	/* add T.Tazaki 2005/05/11 : for Multi line data */
						{
							szData[i] = '.';
						}
					}
					/* add T.Tazaki 2004/09/28 <<< */
					
					/* output return string */
					if( c33TargetInfo.iCallMode == 1 || c33TargetInfo.iCallMode == 3 )
					{
						/* output to window */
						printf_filtered( "%c",szData[i] );
					}
					if( c33TargetInfo.iCallMode == 2 || c33TargetInfo.iCallMode == 3 )
					{
						/* output to file */
						if (fputc(szData[i] , fpCallfile) == EOF)
						{
							fclose( fpCallfile );
							c33_error ( C33_ERR_FILE_WRITE);
							return;
						}
					}
				}
				if( ulSize == 0 ){
					break;	//exit while
				}
			}

			/* output return string */
			if( c33TargetInfo.iCallMode == 1 || c33TargetInfo.iCallMode == 3 )
			{
				/* output to window */
				puts_filtered( "\n" );
			}
			if( c33TargetInfo.iCallMode == 2 || c33TargetInfo.iCallMode == 3 )
			{
				if (fpCallfile != NULL)
				{
					fclose( fpCallfile );
				}
			}
		}
		
		/* Check if falsh set is already setted , back to current setting */
		if( c33TargetInfo.iFlashSet == 1 )
		{
			/* current flash set */
			iResult = c33_SendMessage(C33_ID_FLASH_SET, c33TargetInfo.ulFlashStartAddr,
														c33TargetInfo.ulFlashEndAddr,
														c33TargetInfo.ulEraseRoutineAddr,
														c33TargetInfo.ulWriteRoutineAddr, 0, 0, 0);

			if (iResult == C33_NO_ERR)
			{
				iResult = c33_RecvMessage(C33_ID_FLASH_SET, 0, 0, 0, 0, 0);
			}
		}
	}

	if (iResult != C33_NO_ERR)
	{
		c33_ErrorMsg();
	}
}
/****************************************************************************/
/* Format		 : void c33_IcdClockMode(int iClockFunction)				*/
/* Arguments																*/
/*	int iClockFunction		  : clock function 1:cycle,2:time usec,3:time sec */
/* Return value  : void 													*/
/* Explanation	 : Set clock function										*/
/****************************************************************************/
void
c33_IcdClockMode(int iClockFunction)
{
	int iResult;

	/* set call routine address by flash set message */
	iResult = c33_SendMessage(C33_ID_MEASURE_UNIT, (iClockFunction - 1), 0, 0, 0, 0, 0, 0);
	if (iResult == C33_NO_ERR)
	{
		iResult = c33_RecvMessage(C33_ID_MEASURE_UNIT, 0, 0, 0, 0, 0);
	}
	if (iResult != C33_NO_ERR)
	{
		c33_ErrorMsg();
	}
}
/****************************************************************************/
/* Format		 : void c33_IcdIceCommand(int iParamNum, char* pFilename)	*/
/* Arguments																*/
/*	int iParamNum	: Parameter number										*/
/*	char* pFilename : File name 											*/
/* Return value  : void 													*/
/* Explanation	 : Send and receive ice Command.							*/
/****************************************************************************/
void
c33_IcdIceCommand(int iParamNum, char* pFilename)
{
	FILE *fpTraceFile = NULL;		/* pointer to parameter file */
	unsigned char ucBuf[C33_COMMAND_BUFFER_SIZE];
	unsigned char* pucSendData = ucSendData;
	int iSendDataNum; /* size(byte) of sending data */
	int iRecvDataNum;
	int iTemp,i;

	/* Check number of parameter */
	if ( iParamNum == 2 )
	{
	}
	else
	{
		c33_error (C33_ERR_COMMAND_PARAM);
		return;
	}
	if ( pFilename[0] == 0 )
	{
		c33_error (C33_ERR_FILE_OPEN, pFilename);
		return;
	}

	/* Open command file */
	fpTraceFile = fopen(pFilename, "r");
	if (fpTraceFile == NULL)
	{
		/* file open error */
		c33_error (C33_ERR_FILE_OPEN, pFilename);
		return;
	}

	iTemp = 0;
	iSendDataNum = 0;
	pucSendData++;
	pucSendData++;

	while(fgets(ucBuf,C33_COMMAND_BUFFER_SIZE,fpTraceFile) != 0)
	{
		sscanf(ucBuf,"%x",&iTemp);
		iSendDataNum++;
		if(iSendDataNum == 1 && (iTemp == C33_ID_OK || iTemp == C33_ID_CAN))
		{
			ucSendData[0] = (unsigned char)iTemp;
			break;
		}
		else
		{
			*pucSendData = (unsigned char)iTemp;
			pucSendData++;
		}
	}

	if(iSendDataNum == 1 && (iTemp == C33_ID_OK || iTemp == C33_ID_CAN))
	{
	}
	else
	{
		/* set message size & bcc code */
		ucSendData[0] = (unsigned char)((iSendDataNum >>  8) & 0xff);
		ucSendData[1] = (unsigned char)((iSendDataNum >>  0) & 0xff);
		c33_IcdMakeBCC(ucSendData);/* set BCC data */
		iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */
	}

	/* file close */
	if (fpTraceFile != NULL)
	{
		fclose(fpTraceFile);
	}
	
	if (c33_SendMessage(C33_ID_ICE_COMMAND, iSendDataNum, 0, 0, 0, 0, 0, 0) != C33_NO_ERR)
	{
		c33_error (C33_ERR_MSG_ICE_COMMAND_SEND);
		return;
	}
	else
	{
		c33_IcdRecvMessage(C33_ID_ICE_COMMAND, 0, 0, 0, 0, 0);
		if(ucRecvData[0] >= C33_ICD_ACK)
		{
			printf_filtered("Control code = %02X\n",(int)ucRecvData[0]);
			if(ucRecvData[0] == C33_ICD_ERR)
			{
				printf_filtered("Error code   = %02X\n",(int)ucRecvData[1]);
				printf_filtered("Error address= ");
				printf_filtered("%02X",(int)ucRecvData[2]);
				printf_filtered("%02X",(int)ucRecvData[3]);
				printf_filtered("%02X",(int)ucRecvData[4]);
				printf_filtered("%02X",(int)ucRecvData[5]);
				if(ucRecvData[1] == C33_ICD_ERR_PC_BREAK || ucRecvData[1] == C33_ICD_ERR_FLASH_WRITE)
				{
					printf_filtered("%02X",(int)ucRecvData[6]);
					printf_filtered("%02X",(int)ucRecvData[7]);
					printf_filtered("%02X",(int)ucRecvData[8]);
					printf_filtered("%02X",(int)ucRecvData[9]);
					printf_filtered("%02X",(int)ucRecvData[10]);
				}
				printf_filtered("\n");
			}
		}
		else
		{
			iRecvDataNum  = ucRecvData[0] << 8;
			iRecvDataNum += ucRecvData[1];
			printf_filtered("Data size = %04X\n",iRecvDataNum);
			printf_filtered("ID        = %02X\n",(int)ucRecvData[2]);
			printf_filtered("Data      = ");
			for(i = 1 ; i < iRecvDataNum ; i++)
			{
				if( (i & 0xf) == 1 && i != 1)
				{
					printf_filtered("\n            ");
				}
				printf_filtered("%02X ", (int)ucRecvData[i + 2]);
			}
			printf_filtered("\n");
			printf_filtered("BCC       = %02X\n",(int)ucRecvData[i + 2]);
		}
	}
}

/* add T.Tazaki 2003/07/29 */
/****************************************************************************/
/* Format		 : void c33_IcdReadMemoey(int iParamNum,unsigned long ulStartAddr,unsigned long ulEndAddr) */
/* Arguments																*/
/*	int iParamNum				   : Parameter number						*/
/*	unsigned long ulStartAddr	   : Read Target start address				*/
/*	unsigned long ulEndAddr 	   : Read Target end   address				*/
/* Return value  : void 													*/
/* Explanation	 : Read memory for virtual code trace.						*/
/****************************************************************************/
void
c33_IcdReadMemory(int iParamNum,unsigned long ulStartAddr,unsigned long ulEndAddr)
{
	int iResult;
	char szData[1025];
	unsigned long ulAddr;
	unsigned long ulSize;
	int iCancel = 0;

	/* Check number of parameter */
	if ( iParamNum != 3 )
	{
		c33_error (C33_ERR_COMMAND_PARAM);
		return;
	}
	if( ulStartAddr > ulEndAddr )
	{
		c33_error (C33_ERR_COMMAND_ADDR);
		return;
	}

	puts_filtered(C33_MSG_READ_MEMORY);

	c33_InitCancel();	/* Cancel initialize */ /* add T.Tazaki 2003/07/10 */

	c33TargetInfo.iIcdRm = 1;		/* c33 rm ON : add T.Tazaki 2004/06/15 */

	for( ulAddr = ulStartAddr; ulAddr <= ulEndAddr; ulAddr += ulSize)
	{
		ulSize = ulEndAddr - ulAddr + 1;
		if( ulSize > 1024 )
		{
			ulSize = 1024;
		}
		/* target memory read to buffer */
		iResult = c33_Cmd_ReadWrite(C33_READ, ulAddr, ulSize, C33_BYTE, (unsigned char*)szData );
		if( iResult == C33_NO_ERR )
		{
			/* save code */
			c33_ReadWriteMapMem(C33_WRITE, ulAddr, ulSize, C33_BYTE, (unsigned char*)szData);
		}
		else
		{
			c33TargetInfo.iIcdRm = 0;		/* c33 rm OFF : add T.Tazaki 2004/06/15 */
			puts_filtered (C33_MSG_TARGET_FAILURE);
			c33_ErrorMsg();
		}

		if (c33TargetInfo.iKeyBreak == 1 || c33_CheckCancel( 10 ) == 1 )
		{
			/* Cancel */
			iCancel = 1;
			printf_filtered( C33_MSG_TARGET_CANCEL);
			break;
		}
	}

	c33TargetInfo.iIcdRm = 0;		/* c33 rm OFF : add T.Tazaki 2004/06/15 */

	c33_EndCancel();

	if( iCancel == 0 )
	{
		puts_filtered (C33_MSG_TARGET_DONE);
	}
}

/****************************************************************************/
/* Format        : static void c33_CallMode(int iParamNum, int iCallMode , char *cFileName ) */
/* Arguments                                                                */
/*  int iParamNum             : Parameter number                            */
/*  int iCallMode             : call mode number 1:window,2:file,3:window & file  */
/*  char *cFileName           : call result output file name                */
/* Return value  : void                                                     */
/* Explanation   : Set call mode.                                           */
/****************************************************************************/
static void
c33_CallMode(int iParamNum, int iCallMode , char *cFileName )
{
    /* Check number of parameter */
    if ( iParamNum != 2 && iParamNum != 3 )
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }

    /* Check mode number */
    if( iCallMode < 1 || iCallMode > 3 )
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }

    if( strlen( cFileName ) > 256 )
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }
    
    if( iCallMode == 2 || iCallMode == 3 )
    {
        if( iParamNum != 3 )
        {
            c33_error (C33_ERR_COMMAND_PARAM);
            return;
        }
        /* Check file name length */
        if( strlen( cFileName ) > 255 )
        {
            c33_error (C33_ERR_COMMAND_PARAM);
            return;
        }
        /* Set output file name */
        strcpy( c33TargetInfo.cCallFileName,cFileName );
    }
    
    /* Set call mode number */
    c33TargetInfo.iCallMode = iCallMode;
}

/* add T.Tazaki 2005/05/24 */
/****************************************************************************/
/* Format        : void c33_Dclock(int iParamNum, int iDclock)              */
/* Arguments                                                                */
/*  int iParamNum             : Parameter number                            */
/*  int iDclock               : devision number 1,2,4,8                     */
/* Return value  : void                                                     */
/* Explanation   : Function call execution.                                 */
/****************************************************************************/
void
c33_Dclock(int iParamNum, int iDclock)
{
    int iResult = C33_NO_ERR;
    unsigned long ulAddr;
    unsigned char ucData;

    /* Check number of parameter */
    if ( iParamNum != 2 )
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }
	if( iDclock != 1 && iDclock != 2 && iDclock != 4 && iDclock != 8 )
	{
        c33_error ("C33 command error, invalid parameter.\nc33 dclk [number] :  number = 1,2,4,8");
        return;
    }
    
    ulAddr = c33TargetInfo.ulDebugBase_ram_addr + 0x07067;	/* DCLKS register */
    switch( iDclock )
    {
		case 1: ucData = 2;	break;
		case 2: ucData = 1;	break;
		case 4: ucData = 0;	break;
		case 8: ucData = 3;	break;
		default:ucData = 2;	break;							/* add T.Tazaki 2007/07/12 */
	}
	
    iResult = c33_SendMessage(C33_ID_DATA_WRITE, ulAddr, 1, C33_BYTE, 0, &ucData, 0, 0);
    if (iResult == C33_NO_ERR)
    {
    	iResult = c33_RecvMessage(C33_ID_DATA_WRITE, ulAddr, 1, C33_BYTE, 0, &ucData);
    }
    if (iResult != C33_NO_ERR)
    {
        c33_ErrorMsg();
        return;
    }

	printf_filtered( "set DCLK 1/%d.\n",iDclock );
}
/* add T.Tazaki 2005/06/08 */
/****************************************************************************/
/* Format        : void c33_IcdOscWait(int iParamNum, int iWaitCount)       */
/* Arguments                                                                */
/*  int iParamNum             : Parameter number                            */
/*  int iWaitCount            : Delay count                                 */
/* Return value  : void                                                     */
/* Explanation   : Set delay counter of ICD3/4 Firmware OSC1 slow clock mode. */
/****************************************************************************/
void
c33_IcdOscWait(int iParamNum, int iWaitCount)
{
    int iResult = C33_NO_ERR;

    /* Check number of parameter */
    if ( iParamNum != 1 && iParamNum != 2 )
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }
    
    if( iParamNum == 1 )
    {
		iWaitCount = 0x300;
	}
	else
	{
		if( iWaitCount < 0 || iWaitCount > 0xffffff )
		{
	        c33_error ("C33 command error, invalid parameter.\nc33 oscwait [counter] :  0<=counter<=0xffffff");
	        return;
	    }
    }
	iResult = c33_IcdSendMessage(C33_ID_OSCWAIT_COUNT_SET, iWaitCount, 0, 0, 0, 0, 0, 0);
	if( iResult == C33_NO_ERR)
	{
		iResult = c33_IcdRecvMessage(C33_ID_OSCWAIT_COUNT_SET, 0, 0, 0, 0, 0);
	}
    if (iResult != C33_NO_ERR)
    {
        c33_ErrorMsg();
        return;
    }

	printf_filtered( "set OSC1 wait counter 0x%x.\n",iWaitCount );
    
}

/* add T.Tazaki 2005/06/23 */
/****************************************************************************/
/* Format        : void c33_IcdVer3CacheHit(void)                           */
/* Arguments                                                                */
/* Return value  : void                                                     */
/* Explanation   : Dislpay the rate of cache hit. when cpu is adv.          */
/****************************************************************************/
void
c33_IcdVer3CacheHit()
{
    int iResult = C33_NO_ERR;
    unsigned long  ulAccessCounter;
    unsigned short usAccessCounter_high;
    unsigned short usAccessCounter_low;
    unsigned long  ulMissHitCounter;
    unsigned short usMissHitCounter_high;
    unsigned short usMissHitCounter_low;
    double dCacheHitRate;


	iResult = c33_Cmd_ReadWrite(C33_READ, 0x48356, 2, C33_HALF_WORD, (unsigned char*)&usAccessCounter_high);
	if (iResult != C33_NO_ERR)
	{
		c33_ErrorMsg();
	}
	if (iResult == C33_NO_ERR)
	{
		iResult = c33_Cmd_ReadWrite(C33_READ, 0x48354, 2, C33_HALF_WORD, (unsigned char*)&usAccessCounter_low);
	}
	if (iResult == C33_NO_ERR)
	{
		iResult = c33_Cmd_ReadWrite(C33_READ, 0x4835a, 2, C33_HALF_WORD, (unsigned char*)&usMissHitCounter_high);
	}
	if (iResult == C33_NO_ERR)
	{
		iResult = c33_Cmd_ReadWrite(C33_READ, 0x48358, 2, C33_HALF_WORD, (unsigned char*)&usMissHitCounter_low);
	}
	if (iResult != C33_NO_ERR)
	{
		c33_ErrorMsg();
	}

	ulAccessCounter  = usAccessCounter_high  * 0x10000 + usAccessCounter_low;
	ulMissHitCounter = usMissHitCounter_high * 0x10000 + usMissHitCounter_low;

	if( ulAccessCounter == 0 )
	{
		printf_filtered( "The rate of cache hit is 0.00000%%\n" );
	}
	else
	{
		dCacheHitRate = 1.0 - (double)( (double)ulMissHitCounter / (double)ulAccessCounter );
		dCacheHitRate *= 100;
		printf_filtered( "The rate of cache hit is %8.5f%%\n",dCacheHitRate );
	}


}

/****************************************************************************/
/* Format        : void c33_LogianaMode(int iParamNum, char *cMode, int iUser_in) */
/* Arguments                                                                */
/*  int iParamNum             : Parameter number                            */
/*  char *cMode               : "on"  logiana mode on                       */
/*                            : "off" logiana mode off                      */
/*  int iUser_in              : 0=external trcae input signal               */
/*                            : 1=target CPU trigger sugnal                 */
/* Return value  : void                                                     */
/* Explanation   : set logic analyser mode.                                 */
/****************************************************************************/
void
c33_LogianaMode(int iParamNum, char *cMode, int iUser_in)
{

    /* Check number of parameter */
    if ( iParamNum != 2 && iParamNum != 3 )
    {
        c33_error (C33_ERR_COMMAND_PARAM);
        return;
    }

	if( strcmp( cMode,"on" ) != 0 && strcmp( cMode,"off" ) != 0 )
	{
		c33_error (C33_ERR_COMMAND_INVALID_PARAM);
        return;
	}

    if( iParamNum == 2 )
    {
		if( strcmp( cMode,"off" ) != 0 )
		{
			c33_error (C33_ERR_COMMAND_PARAM);
	        return;
		}
		c33TargetInfo.iLogiana_Mode = 0;	/* mode off */
		c33TargetInfo.iBust_Mode = 0;

		printf_filtered( "logic analyzer mode off.\n");
	}
	else
	{
		if( strcmp( cMode,"on" ) != 0 )
		{
			c33_error (C33_ERR_COMMAND_PARAM);
	        return;
		}
		if( iUser_in != 0 && iUser_in != 1 )
		{
			c33_error (C33_ERR_COMMAND_INVALID_PARAM);
	        return;
		}
		c33TargetInfo.iLogiana_Mode   = 1;	/* mode on */
		c33TargetInfo.iLogiana_UserIn = iUser_in;
		c33TargetInfo.iBust_Mode = 1;		/* bus trace ON */
		c33TargetInfo.iBust_TraceMode = 1;	/* user trace mode */
		
		printf_filtered( "logic analyzer mode on.\n");
		if( iUser_in == 0 )
		{
			printf_filtered( "  external trace(TRC IN).\n");
		}
		else
		{
			printf_filtered( "  target CPU trigger signal.\n");
		}
    }
}


/****************************************************************************/
/* Format		 : static int c33_IcdTraceGetDataSize(unsigned long *pulCycleMax) */
/* Arguments																*/
/*	unsigned long* pulCycleMax : Data size receive pointer					*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Get trace data size. 									*/
/****************************************************************************/
static int
c33_IcdTraceGetDataSize(unsigned long *pulCycleMax)
{
	int iResult;
	*pulCycleMax = 0;

	/* Transmission of a message. */
	iResult = c33_SendMessage(C33_ID_TRACE_READ_DATA_SIZE, 0, 0, 0, 0, 0, 0, 0);

	/* Reception of a message. */
	if (iResult == C33_NO_ERR){
		iResult = c33_RecvMessage(C33_ID_TRACE_READ_DATA_SIZE, 0, 0, 0, 0, pulCycleMax);
	}

	return iResult;
}

/****************************************************************************/
/* Format		 : static int c33_IcdTraceReadData(unsigned char* pucDst, unsigned long ulStartCycle, unsigned long ulReadSize, unsigned long ulCycleMax) */
/* Arguments																*/
/*	unsigned char* pucDst	   : Data receive pointer						*/
/*	unsigned long ulStartCycle : Start cycle								*/
/*	unsigned long ulReadSize   : Read data size 							*/
/*	unsigned long ulCycleMax   : Trace max data size						*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Get trace data from icd33.								*/
/****************************************************************************/
static int
c33_IcdTraceReadData(unsigned char* pucDst, unsigned long ulStartCycle, unsigned long ulReadSize, unsigned long ulCycleMax)
{

	int iResult;

	/* Transmission of a message. */
	iResult = c33_SendMessage(C33_ID_TRACE_READ_DATA, ulStartCycle, ulReadSize, 0, 0, 0, 0, 0);

	/* Reception of a message. */
	if (iResult == C33_NO_ERR){
		iResult = c33_RecvMessage(C33_ID_TRACE_READ_DATA, ulStartCycle, ulReadSize, ulCycleMax, 0, pucDst);
	}

	return iResult;
}

/****************************************************************************/
/* Format		 : static void c33_IcdTraceGetData(void)				   */
/* Arguments	 : void 													*/
/* Return value  : void 													*/
/* Explanation	 : Read trace data from ICD33.								*/
/****************************************************************************/
static int
c33_IcdTraceGetData(void)
{
	unsigned long ulCycleMax = 0;					  /* trace data size from icd33 */
	unsigned long ulReadSize = C33_COMMAND_TEXT_SIZE; /* read data size */
	unsigned char ucTrg;							  /* 1:trigger1 2:trigger2 */
	unsigned long ulIdleNum = 0;					  /* idle data number */
	unsigned long l;
	unsigned char szbuf[200];	/* trace data dump for debug */

	/* Was trace data already read? */	
	if(c33IcdTraceData.ulValidStart != C33_ICD_TRACE_INVALID){
		return C33_NO_ERR;
	}

	puts_filtered (C33_MSG_TRC_READ);

	/* Trace data all clear */
	for (l = 0 ; l < C33_ICD_TRACE_SIZE ; l++)
	{
		c33IcdTraceData.ucDst[l] = 0;
		c33IcdTraceData.ucMethod[l] = 0;
		c33IcdTraceData.ucDpcoNum[l] = 0;
		c33IcdTraceData.ulPC[l] = C33_ICD_TRACE_PC_INVALID;
		c33IcdTraceData.usCode[l] = 0;
	}
	c33IcdTraceData.ucSearchFlag = 0;  /* 0:trace mode 1:trace search mode */
	c33IcdTraceData.ucAnalyzeFlag = 0; /* 0:yet analysis 1:analysis finished */
	g_ulBeforePC = C33_ICD_TRACE_PC_INVALID;			/* add T.Tazaki 2004/10/04 */

	/* Get trace data size */
	if ( g_otx_1 == 1 )		/* is this called "c33 otd 1 or otf 1 mode"H 	add T.Tazaki 2005/01/31 */
	{
										/* not read 2 times. */
		ulCycleMax = g_ulCycleMax;		/* add T.Tazaki 2005/01/31 */
	}
	else if (c33_IcdTraceGetDataSize(&ulCycleMax) != C33_NO_ERR)
	{
		/* change T.Tazaki 2003/06/18 >>> */
//		puts_filtered (C33_MSG_TRC_FINISHED);
		puts_filtered (C33_MSG_TARGET_FAILURE);
		/* change T.Tazaki 2003/06/18 <<< */
		return C33_ERR;
	}

	c33_InitCancel();	/* Cancel initialize */ /* add T.Tazaki 2004/09/13 */

	/* add T.Tazaki 2004/12/01 >>> */
	g_iCycleOver10000 = 0;
	if( g_iPC_TraceParam == 0 )
	{
		if( ulCycleMax > 10000 )
		{
			ulCycleMax = 10000;
			g_iCycleOver10000 = 1;
		}
	}
	/* add T.Tazaki 2004/12/01 <<< */

	/* Read trace data from icd33 */
	for(l = 0 ; l < ulCycleMax ; l += ulReadSize)
	{
		if(ulCycleMax < l + C33_COMMAND_TEXT_SIZE)
		{
			ulReadSize = ulCycleMax - l;
		}
		if (c33_IcdTraceReadData(c33IcdTraceData.ucDst ,l ,ulReadSize ,ulCycleMax) != C33_NO_ERR)
		{
		/* change T.Tazaki 2003/06/18 >>> */
//			puts_filtered (C33_MSG_TRC_FINISHED);
			puts_filtered (C33_MSG_TARGET_FAILURE);
		/* change T.Tazaki 2003/06/18 <<< */
			return C33_ERR;
		}
		c33_IcdProcessing(3000);

		/* Check cancel */
		if (c33TargetInfo.iKeyBreak == 1 || c33_CheckCancel( 20 ) == 1 ) /* 10 --> 20 add T.Tazaki 2004/11/29 */
		{
			/* Cancel */
			printf_filtered( C33_MSG_TARGET_CANCEL);
			return C33_ERR;
		}
	}

	c33_EndCancel();	/* add T.Tazaki 2004/09/13 */

	/* Cut last dummy cycle */
	if (c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV || c33TargetInfo.ucAdvanced == C33_PRODUCT_PE ) /* add PE T.Tazaki 2003/11/21 */
	{
		if(ulCycleMax > 1)
		{
			ulCycleMax--;
		}
	}

	/* Delete dst signal data of retd ,all trace only.(33208 problem) */
	/* retd can not latch case */
	if(c33IcdTraceData.stTraceMode.iTraceMode != C33_ICD_TRACE_MODE_AREA_OVER_WRITE_NO )
	{
		if(ulCycleMax < c33IcdTraceData.ulIcdTraceBufferSize)
		{
			if( (c33IcdTraceData.ucDst[0] & C33_ICD_TRACE_DST0_1) == C33_ICD_TRACE_DST_ABS_JP )
			{
				for(l = 0 ; (l + 1) < ulCycleMax ; l++)
				{
					c33IcdTraceData.ucDst[l] = c33IcdTraceData.ucDst[l + 1];
				}
				if(ulCycleMax > 1)
				{
					ulCycleMax--;
				}
			}
		}
		else
		{
			if( (c33IcdTraceData.ucDst[0] & C33_ICD_TRACE_DST0_1) != C33_ICD_TRACE_DST_ABS_JP && (c33IcdTraceData.stTraceMode.iTraceMode == C33_ICD_TRACE_MODE_ALL_OVER_WRITE_NO) )
			{
				/* c33IcdTraceData.ucDst[ulCycleMax - 1]; */
				if(ulCycleMax > 1){
					ulCycleMax--;
				}
			}
			c33IcdTraceData.ucDst[0] = C33_ICD_TRACE_DST_IDLE;
		}
	}

	if( iDebugTrace == 1 )	/* debug trace mode ?	 add T.Tazaki 2003/08/22 */
	{


		printf_filtered( "\n" );
		for (l = 0 ; l < ulCycleMax ; l++)
		{
			sprintf( szbuf,"%5d: DPCO[%d] DST[%d%d%d]  ",l,  (c33IcdTraceData.ucDst[l] & 0x08) >> 3,
															 (c33IcdTraceData.ucDst[l] & 0x04) >> 2,
															 (c33IcdTraceData.ucDst[l] & 0x02) >> 1,
															 (c33IcdTraceData.ucDst[l] & 0x01 ) );

			if( (c33IcdTraceData.ucDst[l] & 0x07) == 0x07 )
			{
				strcat(szbuf,"BREAK AREA");
			}
			else
			{
				if ((c33IcdTraceData.ucDst[l] & 0x03) == 0x0 )
				{
					strcat(szbuf,"NORMAL    ");
				}
				else if ((c33IcdTraceData.ucDst[l] & 0x03) == 0x1 )
				{
					strcat(szbuf,"PC REL    ");
				}
				else if ((c33IcdTraceData.ucDst[l] & 0x03) == 0x2 )
				{
					strcat(szbuf,"PC ABS    ");
				}
				else if ((c33IcdTraceData.ucDst[l] & 0x03)== 0x3 )
				{
					strcat(szbuf,"IDLE      ");
				}

				if( (c33IcdTraceData.ucDst[l] & 0x04) == 0x04 )
				{
					strcat(szbuf," +BREAK");
				}
			}
			printf_filtered( "%s\n",szbuf );
		}
	}

	/* Make trace trigger bit and idle data number count */
	ucTrg = 1;
	for (l = 0 ; l < ulCycleMax ; l++)
	{
		if(c33IcdTraceData.stTraceMode.iTraceMode == C33_ICD_TRACE_MODE_AREA_OVER_WRITE_NO)
		{
			if(c33IcdTraceData.ucDst[l] & C33_ICD_TRACE_DST2)
			{
				if(ucTrg == 1)
				{
					ucTrg = 2;
					c33IcdTraceData.ucDst[l] |= C33_ICD_TRACE_TRG1_HIT;
				}
				else
				{
					ucTrg = 1;

					/* icd33 SSRAM bug fix */
					if( (l + 1) < ulCycleMax && (c33IcdTraceData.ucDst[l+1] & C33_ICD_TRACE_DST2) == 0)
					{
						c33IcdTraceData.ucDst[l] |= C33_ICD_TRACE_TRG1_HIT;
					}
					else
					{
						c33IcdTraceData.ucDst[l] |= C33_ICD_TRACE_TRG2_HIT;
					}

				}
			}
		}
		if( (c33IcdTraceData.ucDst[l] & C33_ICD_TRACE_DST0_1) == C33_ICD_TRACE_DST_IDLE )
		{
			ulIdleNum++;
		}
	}

	/* Save data size */
	c33IcdTraceData.ulValidStart = 0;
	c33IcdTraceData.ulValidEnd = ulCycleMax;

	/* Accumulation calculate to use offset */
	c33IcdTraceData.ulCycleOffset = 0;
	while( (c33IcdTraceData.ucDst[c33IcdTraceData.ulCycleOffset] & C33_ICD_TRACE_DST0_1 ) == C33_ICD_TRACE_DST_IDLE )
	{
		c33IcdTraceData.ulCycleOffset++;
		if(c33IcdTraceData.ulCycleOffset >= c33IcdTraceData.ulValidEnd)
		{/* data is idle only */
			c33IcdTraceData.ulCycleOffset = 0;
			break;
		}
	}

	puts_filtered (C33_MSG_TRC_FINISHED);

	return C33_NO_ERR;
}

/****************************************************************************/
/* Format		 : static unsigned long c33_IcdTraceInstCycle(unsigned long ulCycle) */
/* Arguments																*/
/*	unsigned long ulCycle : Cycle number									*/
/* Return value  : Cycle number of next instruction 						*/
/* Explanation	 : Get cycle number of next instruction.					*/
/****************************************************************************/
static unsigned long
c33_IcdTraceInstCycle(unsigned long ulCycle)
{
	unsigned long ulClock = 0;
	do
	{
		ulClock++;
		ulCycle++;
		/* Check data end */
		if(c33IcdTraceData.ulValidEnd <= ulCycle)
		{
			break;
		}
	}while( (c33IcdTraceData.ucDst[ulCycle] & C33_ICD_TRACE_DST0_1 ) == C33_ICD_TRACE_DST_IDLE );
	return(ulClock);
}

/****************************************************************************/
/* Format		 : static void c33_IcdTraceGetSourceCode(unsigned long ulAddr, char* pOut) */
/* Arguments																*/
/*	unsigned long ulAddr : Address											*/
/*	char* pOut			 : Buffer of source information 					*/
/* Return value  : void 													*/
/* Explanation	 : Get source code from address.							*/
/****************************************************************************/
static void
c33_IcdTraceGetSourceCode(unsigned long ulAddr, char* pOut)
{
	unsigned long ulLineNo;
	char cTemp[C33_TRACE_DATA_LINE_MAX + 1];
	unsigned short usCodeWork;

	/* add T.Tazaki 2003/06/17 >>> */
	if (c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV || c33TargetInfo.ucAdvanced == C33_PRODUCT_PE ) /* add PE T.Tazaki 2003/11/21 */
	{
		/* Advanced macro : ext instruction address skip */
		/*	ext imm13	   : <------ set address   */
		/*	ext imm13	   :					   */
		/*	ld.w %rs,imm5  : input address		   */

		if( ulAddr >= 2 )
		{
			ulAddr -= 2;
			
			do
			{
				c33_ReadWriteMapMem(C33_READ, ulAddr, C33_HALF_WORD, C33_HALF_WORD, (unsigned char*)&usCodeWork);

				if( ( usCodeWork & 0xe000 ) == 0xc000 )  /* ext imm13 */
				{
					if( ulAddr >= 2 )
					{
						ulAddr -= 2;
					}
					else
					{
						break;
					}
				}
				else
				{
					ulAddr += 2;
					break;
				}
			}while( 1 );
		}
	}
	/* add T.Tazaki 2003/06/17 <<< */

	pOut[0] = 0;

	/* Get filename and line number */
	if (c33_FindFilename(ulAddr, cTemp, &ulLineNo) == C33_NO_ERR )
	{
		strcat(pOut, "(" );
#if 0	/* modify T.Tazaki 2005/02/15 >>> */
		strcat(pOut, cTemp );
		if (strlen (pOut) < 16)
		{
			strcat(pOut, ")" );
		}
		else
		{
			pOut[16] = ')';
			pOut[17] = 0;
		}
		/* Position adjustment */
		while(strlen(pOut) < 18){
			strcat(pOut, " " );
		}
#endif
		if (strlen (cTemp) < 16)
		{
			strcat(pOut, cTemp );
			strcat(pOut, ")" );
			/* Position adjustment */
			while(strlen(pOut) < 18){
				strcat(pOut, " " );
			}
		}
		else
		{
			strncat(pOut, cTemp+(strlen(cTemp)-15),15 );
			strcat(pOut, ") " );
		}
/* modify T.Tazaki 2005/02/15 <<< */

		sprintf(cTemp, "%05u  ", (int)ulLineNo);
		strcat(pOut, cTemp);

		/* Get source code */
		c33_print_source_lines (ulAddr, cTemp);
		
		strcat(pOut, cTemp);
	}
}

/****************************************************************************/
/* Format		 : static void c33_IcdTraceMake1Line(unsigned long ulTrcDataNumber, unsigned long ulCycleNumbar, char* pTraceStr) */
/* Arguments																*/
/*	unsigned long ulTrcDataNumber : Trace data number						*/
/*	unsigned long ulCycleNumbar   : Cycle number							*/
/*	char* pTraceStr 			  : Storing pointer 						*/
/* Return value  : void 													*/
/* Explanation	 : Get trace data string.									*/
/****************************************************************************/
static void
c33_IcdTraceMake1Line(unsigned long ulTrcDataNumber, unsigned long ulCycleNumbar, char* pTraceStr)
{
	char cTemp[C33_TRACE_DATA_LINE_MAX + 1];
	char cTemp2[C33_TRACE_DATA_LINE_MAX + 1];				/* add T.Tazaki 2005/03/02 */
	char cTempAsm[C33_TRACE_DATA_LINE_MAX + 1];
	char cTraceStr[C33_TRACE_DATA_LINE_MAX + 1];			/* add T.Tazaki 2004/09/15 */
	int i;
	int iLenExt = 0;										/* add T.Tazaki 2005/02/21 */
	unsigned short usCode;									/* add T.Tazaki 2005/02/21 */
	unsigned long  ulPC;  									/* add T.Tazaki 2005/02/21 */
	int iResult = C33_NO_ERR;								/* add T.Tazaki 2005/02/21 */
	pTraceStr[0] = 0;										/* add T.Tazaki 2005/02/21 */

	/* add T.Tazaki 2005/02/23 >>>	: ADV or PE, OUTPUT ext. */
	cTemp[0] = 0;
	if (c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV || c33TargetInfo.ucAdvanced == C33_PRODUCT_PE )
	{

		/* ext 0x1	<--̗p			*/
		/* ext 0x1					*/
		/* ext 0x1					*/
		/* ext 0x1					*/
		/* ext 0x1	<--̗p			*/
		/* ld.w %r1,[%r2]			*/

//		for( ulPC = c33IcdTraceData.ulPC[ulTrcDataNumber] - 4; ulPC >= 0; ulPC -= 2 )	/* everytime true Change 2007/07/12 T.Tazaki */
		for( ulPC = c33IcdTraceData.ulPC[ulTrcDataNumber] - 4; ; ulPC -= 2 )			/* not everytime true Change 2007/07/12 T.Tazaki */
		{
			iResult = c33_ReadWriteMapMem(C33_READ, ulPC, C33_HALF_WORD, C33_HALF_WORD, (unsigned char*)&usCode);
			if( iResult != C33_NO_ERR )
			{
				if( ulPC != (c33IcdTraceData.ulPC[ulTrcDataNumber] - 4) )
				{
					ulPC += 2;
				}
				break;
			}
			if( ( ( usCode & 0xe000 ) == 0xc000 ) ||  /* ext imm13					  */
				( ( usCode & 0xff00 ) == 0x3b00 ) ||  /* ext op,imm2  ext cont		  */
				( ( usCode & 0xff00 ) == 0x3f00 ) )   /* ext %rs	  ext %rs,op,imm2 */
			{
				if( ulPC == 0 )
				{
					break;
				}
				continue;
			}
			else
			{
				if( ulPC != (c33IcdTraceData.ulPC[ulTrcDataNumber] - 4) )
				{
					ulPC += 2;
				}
				break;
			}
		}
		
		iResult = c33_ReadWriteMapMem(C33_READ, ulPC, C33_HALF_WORD, C33_HALF_WORD, (unsigned char*)&usCode);
		if( iResult == C33_NO_ERR )
		{
			if( ( ( usCode & 0xe000 ) == 0xc000 ) ||  /* ext imm13					  */
				( ( usCode & 0xff00 ) == 0x3b00 ) ||  /* ext op,imm2  ext cont		  */
				( ( usCode & 0xff00 ) == 0x3f00 ) )   /* ext %rs	  ext %rs,op,imm2 */
			{
				c33_Disasm( usCode,cTempAsm); /* get disassemble code */ 
#if 1			/* Change T.Tazaki 2007/07/25 */
				if( g_iTraceDisplayMode == 1 )
				{
					sprintf( cTemp,"           %08X %04X %s\n",ulPC,usCode,cTempAsm);
				}
				else
				{
					sprintf( cTemp,"           %08X %04X %s",ulPC,usCode,cTempAsm);
					c33_AddCRLF(cTemp);
				}
#else
/* T.Tazaki 2006/07/13 bug fix >>> */ 
//				sprintf( cTemp,"           %08X %04X %s\n",ulPC,usCode,cTempAsm);	/* add T.Tazaki 2005/03/02 */
				sprintf( cTemp,"           %08X %04X %s",ulPC,usCode,cTempAsm);		
/* T.Tazaki 2006/07/13 bug fix <<< */ 
				c33_AddCRLF(cTemp);			/* add T.Tazaki 2006/07/13 bug fix */
#endif
			}
		}
		iResult = c33_ReadWriteMapMem(C33_READ, c33IcdTraceData.ulPC[ulTrcDataNumber] - 2, C33_HALF_WORD, C33_HALF_WORD, (unsigned char*)&usCode);
		if( iResult == C33_NO_ERR )
		{
			if( ( ( usCode & 0xe000 ) == 0xc000 ) ||  /* ext imm13					  */
				( ( usCode & 0xff00 ) == 0x3b00 ) ||  /* ext op,imm2  ext cont		  */
				( ( usCode & 0xff00 ) == 0x3f00 ) )   /* ext %rs	  ext %rs,op,imm2 */
			{
				c33_Disasm( usCode,cTempAsm); /* get disassemble code */ 
#if 1			/* Change T.Tazaki 2007/07/25 */
				if( g_iTraceDisplayMode == 1 )
				{
					sprintf( cTemp2,"           %08X %04X %s\n",c33IcdTraceData.ulPC[ulTrcDataNumber] - 2,usCode,cTempAsm); /* add T.Tazaki 2005/03/02 */
				}
				else
				{
					sprintf( cTemp2,"           %08X %04X %s",c33IcdTraceData.ulPC[ulTrcDataNumber] - 2,usCode,cTempAsm); /* add T.Tazaki 2005/03/02 */
					c33_AddCRLF(cTemp2);
				}
#else
/* T.Tazaki 2006/07/13 bug fix >>> */ 
//				sprintf( cTemp2,"           %08X %04X %s\n",c33IcdTraceData.ulPC[ulTrcDataNumber] - 2,usCode,cTempAsm); /* add T.Tazaki 2005/03/02 */
				sprintf( cTemp2,"           %08X %04X %s",c33IcdTraceData.ulPC[ulTrcDataNumber] - 2,usCode,cTempAsm); /* add T.Tazaki 2005/03/02 */
/* T.Tazaki 2006/07/13 bug fix <<< */ 
				c33_AddCRLF(cTemp2);		/* add T.Tazaki 2006/07/13 bug fix */
#endif
				strcat( cTemp,cTemp2 );
			}
			else
			{
				cTemp[0] = 0;
			}
		}
		else
		{
			cTemp[0] = 0;
		}
		iLenExt = strlen( cTemp );
	}
	/* add T.Tazaki 2005/02/23 <<< */

	/* Make trigger string */	
	if( (c33IcdTraceData.ucDst[ulTrcDataNumber] & C33_ICD_TRACE_TRG1_HIT) == C33_ICD_TRACE_TRG1_HIT ){
//		strcpy(cTemp, "T1 ");/* modify T.Tazaki 2005/02/21 */
		strcat(cTemp, "T1 ");/* modify T.Tazaki 2005/02/21 */
	}
	else if( (c33IcdTraceData.ucDst[ulTrcDataNumber] & C33_ICD_TRACE_TRG2_HIT) == C33_ICD_TRACE_TRG2_HIT ){
//		strcpy(cTemp, "T2 ");/* modify T.Tazaki 2005/02/21 */
		strcat(cTemp, "T2 ");/* modify T.Tazaki 2005/02/21 */
	}
	else {
//		strcpy(cTemp, "   ");/* modify T.Tazaki 2005/02/21 */
		strcat(cTemp, "   ");/* modify T.Tazaki 2005/02/21 */
	}
	strcat(pTraceStr, cTemp);

	if (c33IcdTraceData.ucDebugIdleDsp != 0)
	{
		sprintf(cTemp, "%02X %02X %05u %02d", c33IcdTraceData.ucMethod[ulTrcDataNumber], c33IcdTraceData.ucDst[ulTrcDataNumber], (unsigned int)ulTrcDataNumber, c33IcdTraceData.ucDpcoNum[ulTrcDataNumber]);
		strcat(pTraceStr, cTemp);
	}

	/* Make cycle string */
	sprintf(cTemp, "%07lu", ulCycleNumbar);
	strcat(pTraceStr, cTemp);

	/* Make search flag string */
	if(c33IcdTraceData.ucSearchFlag == 1 && (c33IcdTraceData.ucDst[ulTrcDataNumber] & C33_ICD_TRACE_SEARCH_HIT) == C33_ICD_TRACE_SEARCH_HIT)
	{
		strcat(pTraceStr, "*");
	}
	else
	{
		strcat(pTraceStr, " ");
	}

	/* Check interrupt */
	
	c33IcdTraceData.ulPC[ulTrcDataNumber] &= 0xfffffffe;	/* A0 = 0 : add T.Tazaki 2005/02/15 */
	
	if( (c33IcdTraceData.ucMethod[ulTrcDataNumber] & C33_ICD_TRACE_INTERRUPT) == C33_ICD_TRACE_INTERRUPT )
	{
		strcat(pTraceStr, "interrupt");
	}
	else
	{
		if( g_ulBeforePC != c33IcdTraceData.ulPC[ulTrcDataNumber] )	/* is the same before address? */
		{
			/* Make address string */
			if(c33IcdTraceData.ulPC[ulTrcDataNumber] == C33_ICD_TRACE_PC_INVALID)
			{
				/* Analyze error case */
				strcat(pTraceStr, "********");
			}
			else
			{
				for(i = 7 ; i >= 0 ; i--)
				{
					if( (c33IcdTraceData.ucDpcoNum[ulTrcDataNumber] <  (i + 1) * 4 + 2) && (c33IcdTraceData.ucDpcoNum[ulTrcDataNumber] != c33IcdTraceData.ucDpcoAddrFix) )
					{
						sprintf(cTemp, "*");
					}
					else
					{
						sprintf(cTemp, "%X", (unsigned int)(c33IcdTraceData.ulPC[ulTrcDataNumber] >> (i * 4) & 0x0f) );
					}
					
					strcat(pTraceStr, cTemp);
					g_ulBeforePC = c33IcdTraceData.ulPC[ulTrcDataNumber] ;	/* update before PC */
				}
			}

			/* Make code string and disassemble string */
			if( (c33IcdTraceData.ucMethod[ulTrcDataNumber] & C33_ICD_TRACE_FIX_PC_BIT) == C33_ICD_TRACE_FIX_PC_BIT )
			{
				sprintf(cTemp, " %04X ", c33IcdTraceData.usCode[ulTrcDataNumber]);
				g_iTraceMode = 1;		 /* ld.w %r0,0x2a (xld.w %r0,0xffffffea) --> (ld.w %r0,0x2a):Not extend */
				c33_Disasm( c33IcdTraceData.usCode[ulTrcDataNumber],cTempAsm); /* get disassemble code */ 
				g_iTraceMode = 0; 
				strcat(cTemp, cTempAsm);
			}
			else
			{
				sprintf(cTemp, " ---- ----------               ");
			}
			strcat(pTraceStr, cTemp);
		}
		else
		{
			/* POƓPCȂ̂ŁAo͂ȂB */
//			sprintf(cTemp, "                               ");	/* del 2005/02/15 T.Tazaki */
			pTraceStr[0] = 0;				/* add T.Tazaki 2005/02/22 */
			return;							/* add T.Tazaki 2005/02/22 */
		}
	}
	/* Complete the string length */
	strcat(pTraceStr, "                                 ");/* space length is " ---- ----------               " */
//	pTraceStr[51] = 0;			/* modify T.Tazaki 2005/02/21 */
	pTraceStr[51+iLenExt] = 0;	/* modify T.Tazaki 2005/02/21 */

	/* Make clock number string */
	if( c33TargetInfo.c33TraceMode.iCounterMode == C33_ICD_TRACE_COUNTER_INST )
	{
		/* Normal mode */
		sprintf(cTemp, "%07lu ", c33_IcdTraceInstCycle(ulTrcDataNumber));
		if( (c33IcdTraceData.ucDst[ulTrcDataNumber] & C33_ICD_TRACE_TRG2_HIT) != 0 || ulCycleNumbar == 0)
		{
			sprintf(cTemp, "------- ");
		}
	}
	else
	{
		/* Accumulation mode */
/* ClḱAItZbg\߂BT.Tazaki 2005/03/23 >>> */
//		sprintf(cTemp, "%07lu ", ulTrcDataNumber - c33IcdTraceData.ulCycleOffset);
		sprintf(cTemp, "%07lu ", ulTrcDataNumber );
/* ClḱAItZbg\߂BT.Tazaki 2005/03/23 <<< */
	}
	strcat(pTraceStr, cTemp);

	/* check if PC and Bus trace display mode	  add T.Tazaki 2004/09/15 >>> */
	if( g_iPC_Bus_Trace == 1 )
	{
		if( c33TargetInfo.iBust_TraceMode == 0 )  /* Normal Mode ? */
		{
			if(c33IcdTraceData.cBusDataExist[ ulTrcDataNumber ] == 1) 
			{
				/* PC + Normal Mode */
				g_iPC_Bus_same = 1;
				c33_IcdBusTraceMake1Line( ulTrcDataNumber, cTraceStr);	/* make BUS Trace data */
				g_iPC_Bus_same = 0;
				strcat(pTraceStr, cTraceStr);
			}
			else
			{
				strcat(pTraceStr, "                                               ");
			}
		}
		else
		{
			/* PC + User Mode */
			g_iPC_Bus_same = 1;
			c33_IcdBusTraceMake1Line( ulTrcDataNumber, cTraceStr);	/* make BUS Trace data */
			g_iPC_Bus_same = 0;
			strcat(pTraceStr, cTraceStr);
		}
	}

	/* check if PC and Bus trace display mode	  add T.Tazaki 2004/09/15 <<< */
		
	/* Make method string */
	if( c33IcdTraceData.ulPC[ulTrcDataNumber] == C33_ICD_TRACE_PC_INVALID && c33IcdTraceData.ucDpcoNum[ulTrcDataNumber] == c33IcdTraceData.ucDpcoAddrFix)
	{
		/* analyze error */
		strcat(pTraceStr, "ERR ");
	}
	else if( (c33IcdTraceData.ucMethod[ulTrcDataNumber] & C33_ICD_TRACE_FIX_DPCO) == C33_ICD_TRACE_FIX_DPCO )
	{
		/* analyze to dpco */
		strcat(pTraceStr, "DPC ");
	}
	else if( (c33IcdTraceData.ucMethod[ulTrcDataNumber] & C33_ICD_TRACE_FIX_RET) == C33_ICD_TRACE_FIX_RET )
	{
		/* analyze to ret instruction */
		strcat(pTraceStr, "RET ");
	}
	else if( (c33IcdTraceData.ucMethod[ulTrcDataNumber] & C33_ICD_TRACE_FIX_RETI) == C33_ICD_TRACE_FIX_RETI )
	{
		/* analyze to reti instruction */
		strcat(pTraceStr, "RTI ");
	}
	else if( (c33IcdTraceData.ucMethod[ulTrcDataNumber] & C33_ICD_TRACE_FIX_MAP) == C33_ICD_TRACE_FIX_MAP )
	{
		/* analyze to map information */
		strcat(pTraceStr, "MAP ");
	}
	else if( (c33IcdTraceData.ucMethod[ulTrcDataNumber] & C33_ICD_TRACE_FIX_TIRGGER) == C33_ICD_TRACE_FIX_TIRGGER )
	{
		/* analyze to trigger information */
		strcat(pTraceStr, "TRG ");
	}
	else if( (c33IcdTraceData.ucMethod[ulTrcDataNumber] & C33_ICD_TRACE_FIX_START_PC) == C33_ICD_TRACE_FIX_START_PC )
	{
		/* analyze to trigger information */
		strcat(pTraceStr, "SPC ");
	}
	else
	{
		strcat(pTraceStr, "--- ");
	}

	/* Make filename and source */
	if( (c33IcdTraceData.ucMethod[ulTrcDataNumber] & C33_ICD_TRACE_FIX_PC_BIT) == C33_ICD_TRACE_FIX_PC_BIT && ((c33IcdTraceData.ucMethod[ulTrcDataNumber] & C33_ICD_TRACE_INTERRUPT) == 0) )
	{
		c33_IcdTraceGetSourceCode(c33IcdTraceData.ulPC[ulTrcDataNumber], cTemp);
		strcat(pTraceStr, cTemp);
	}
}

/****************************************************************************/
/* Format		 : static void c33_IcdTraceCalMaxNumber(void)				*/
/* Arguments	 : void 													*/
/* Return value  : void 													*/
/* Explanation	 : This function is calculate data number.					*/
/*				   Data number is cycle number minus idle number.			*/
/****************************************************************************/
static void
c33_IcdTraceCalMaxNumber(void)
{
	unsigned long ulCycleCounter;
	unsigned long ulIdleNum = 0;  /* idle data number */

	for( ulCycleCounter = c33IcdTraceData.ulValidStart ; ulCycleCounter < c33IcdTraceData.ulValidEnd ; ulCycleCounter++)
	{
		if( (c33IcdTraceData.ucDst[ulCycleCounter] & C33_ICD_TRACE_DST0_1) == C33_ICD_TRACE_DST_IDLE )
		{
			ulIdleNum++;
		}
	}

	/* Save enable data size */
	if (c33IcdTraceData.ucDebugIdleDsp != 0)
	{
		c33IcdTraceData.ulDataMaxNumber = c33IcdTraceData.ulValidEnd;
	}
	else
	{
		c33IcdTraceData.ulDataMaxNumber = c33IcdTraceData.ulValidEnd - ulIdleNum ;
	}
}

/****************************************************************************/
/* Format		 : static void c33_IcdTraceInitData(void)					*/
/* Arguments	 : void 													*/
/* Return value  : void 													*/
/* Explanation	 : Initialize trace data.									*/
/****************************************************************************/
static void
c33_IcdTraceInitData(void)
{
	/* Set initial value */
	c33IcdTraceData.ulValidStart  = C33_ICD_TRACE_INVALID;
	c33IcdTraceData.ulValidEnd	  = C33_ICD_TRACE_INVALID;
	c33IcdTraceData.ucSearchFlag  = 0;	/* 0:trace mode 1:trace search mode */
	c33IcdTraceData.ucAnalyzeFlag = 0; /* 0:yet analysis 1:analysis finished */

	c33IcdTraceData.ulValidBusStart  = C33_ICD_TRACE_INVALID;

	if (c33TargetInfo.iIcdExeStatus == C33_EXE_STATUS_STOP)
	{
		/* Target is not running. */
		c33IcdTraceData.ulStartPcAddr = read_register (PC_REGNUM) & C33_PC_MASK;
		/* store trace mode */
		c33IcdTraceData.stTraceMode = c33TargetInfo.c33TraceMode;
	}
	else
	{
		/* Target is running. */
		c33IcdTraceData.ulStartPcAddr = C33_ICD_TRACE_PC_INVALID;/* Don't use */
	}

}

/****************************************************************************/
/* Format		 : void c33_IcdTraceDisplay(int iParamNum, unsigned long ulStartCycle, unsigned long ulEndCycle) */
/* Arguments																*/
/*	int iParamNum			   : Parameter number							*/
/*	unsigned long ulStartCycle : Start cycle								*/
/*	unsigned long ulEndCycle   : End cycle									*/
/* Return value  : void 													*/
/* Explanation	 : Display trace data on trace window.						*/
/****************************************************************************/
void
c33_IcdTraceDisplay(int iParamNum, unsigned long ulStartCycle, unsigned long ulEndCycle)
{
	unsigned long ulCycleNumbar;	/* Cycle number */
	unsigned long ulTrcDataNumber;	/* trace data number */
	unsigned long l;
	char cErrMsg[C33_TRACE_DATA_LINE_MAX + 1];
	char cTraceStr[C33_TRACE_DATA_LINE_MAX + 1];
	int  iCancel = 0;
	
	extern Tcl_Interp *gdbtk_interp;

	cErrMsg[0] = 0;

	if( g_use_windows == 0 )	 /* CUI mode ? */
	{
		return;
	}


	/* Check number of parameter */
	if ( iParamNum == 1 )
	{
		/* bug fix 2005/01/12 T.Tazaki >>> */
		if(c33IcdTraceData.ucSearchFlag == 0) /* Not "c33 ts" mode */
		{
			if( g_iPC_TraceParam == 1 )	
			{
				c33IcdTraceData.ulValidStart  = C33_ICD_TRACE_INVALID;
			}
			g_iPC_TraceParam = 0;	
		}
		/* bug fix 2005/01/12 T.Tazaki <<< */

		ulStartCycle = c33IcdTraceData.ulIcdTraceBufferSize;
		ulEndCycle = 0;
	}
	else if ( iParamNum == 3 )
	{
		/* bug fix 2005/01/12 T.Tazaki >>> */
		if(c33IcdTraceData.ucSearchFlag == 0) /* Not "c33 ts" mode */
		{
			if( g_iPC_TraceParam == 0 )	
			{
				c33IcdTraceData.ulValidStart  = C33_ICD_TRACE_INVALID;
			}
			g_iPC_TraceParam = 1;
		}
		/* bug fix 2005/01/12 T.Tazaki <<< */
		if (ulStartCycle > c33IcdTraceData.ulIcdTraceBufferSize || ulEndCycle > c33IcdTraceData.ulIcdTraceBufferSize)
		{
			c33_error (C33_ERR_COMMAND_TRC_SIZE);
			return;
		}
		if (ulEndCycle > ulStartCycle)
		{
			c33_error (C33_ERR_COMMAND_CYCLE);
			return;
		}
	}
	else
	{
		c33_error (C33_ERR_COMMAND_PARAM);
		return;
	}

	if (c33TargetInfo.pMapInfo == 0)
	{
		c33_error (C33_ERR_NO_PARAMETER_FILE);
		return;
	}


	c33_IcdProcessing(0);/*  timer of "..." is reset */

	/* read icd33 trace data */
	if (c33_IcdTraceGetData() != C33_NO_ERR){
		c33_EndCancel();	/* add T.Tazaki 2004/09/13 */
		c33_ErrorMsg();
		return;
	}

	/* analyze trace data */
	if (c33_IcdTraceAnalyze() != C33_NO_ERR){
		c33_ErrorMsg();
		return;
	}

	/* trace window clear */ /* add T.Tazaki 2003/07/28 */
	Tcl_Eval( gdbtk_interp, "c33_clear_trace_win" );

	/* Write trace header */
	sprintf(cTraceStr, C33_MSG_TRC_HEADER);

	Tcl_SetVar( gdbtk_interp,"c33_trace_string",cTraceStr,TCL_GLOBAL_ONLY );
	Tcl_Eval( gdbtk_interp, "c33_trace_out" );


	c33_InitCancel();	/* Cancel initialize */ /* add T.Tazaki 2003/07/10 */
	iCancel = 0;

	/* Valid data? */	
	if (c33IcdTraceData.ulValidStart != C33_ICD_TRACE_INVALID)
	{
		/* Write file */
		ulTrcDataNumber = 0;
		ulCycleNumbar = (c33IcdTraceData.ulDataMaxNumber - 1);
		l = ulEndCycle;
		g_ulBeforePC = C33_ICD_TRACE_PC_INVALID;			/* add T.Tazaki 2005/02/15 */
		while(l <= ulStartCycle)
		{
			/* Check over run */
			if( ulTrcDataNumber > c33IcdTraceData.ulValidEnd )
			{
				break;
			}
			/* pass over idle cycle */
			if( (c33IcdTraceData.ucDst[ulTrcDataNumber] & C33_ICD_TRACE_DST0_1) != C33_ICD_TRACE_DST_IDLE || c33IcdTraceData.ucDebugIdleDsp != 0)
			{
				if(c33IcdTraceData.ucSearchFlag == 0 || (c33IcdTraceData.ucDst[ulTrcDataNumber] & C33_ICD_TRACE_HIT_PRE_POST) != 0 )
				{
					if( ulCycleNumbar <= ulStartCycle && ulCycleNumbar >= ulEndCycle )
					{
						g_iTraceDisplayMode = 1;	/* Trace displayu mode set for c33_IcdTraceMake1Line() LF code  add T.Tazaki 2007/07/25 */
						c33_IcdTraceMake1Line(ulTrcDataNumber, ulCycleNumbar, cTraceStr);
						
						if( cTraceStr[0] != 0 )	/* same address continue?	add T.Tazaki 2005/02/22 */
						{
							Tcl_SetVar( gdbtk_interp,"c33_trace_string",cTraceStr,TCL_GLOBAL_ONLY );
							Tcl_Eval( gdbtk_interp, "c33_trace_out" );
						}
						l++;

						/* Check cancel add T.Tazaki 2004/11/29 >>> speed up */
						if (c33TargetInfo.iKeyBreak == 1 || c33_CheckCancel( 100 ) == 1 ) 
						{
							/* Cancel */
							printf_filtered( C33_MSG_TARGET_CANCEL);
							c33_EndCancel();
							return;
						}
						/* Check cancel add T.Tazaki 2004/11/29 <<< */
					}
				}
				ulCycleNumbar--;
			}
			ulTrcDataNumber++;

#if 0	/* del T.Tazaki 2004/11/29 speed up */
			/* Check cancel */
			if (c33TargetInfo.iKeyBreak == 1 || c33_CheckCancel( 500 ) == 1 )	
			{
				/* Cancel */
				iCancel = 1;
				printf_filtered( C33_MSG_TARGET_CANCEL);
				break;
			}
#endif

		}
	}

	c33_EndCancel(); /* add T.Tazaki 2003/07/10 */

	/* Check cancel */
	if (iCancel == 0 )	/* Not cancel ? */
	{
		/* Print error message */
		if ( cErrMsg[0] != 0 )
		{
			c33_error (cErrMsg);
			return;
		}
	}
}

/****************************************************************************/
/* Format		 : void c33_IcdTraceFile(int iParamNum, char* pFilename, unsigned long ulStartCycle, unsigned long ulEndCycle) */
/* Arguments																*/
/*	int iParamNum			   : Parameter number							*/
/*	char* pFilename 		   : File name									*/
/*	unsigned long ulStartCycle : Start cycle								*/
/*	unsigned long ulEndCycle   : End cycle									*/
/* Return value  : void 													*/
/* Explanation	 : Make trace data file.									*/
/****************************************************************************/
void
c33_IcdTraceFile(int iParamNum, char* pFilename, unsigned long ulStartCycle, unsigned long ulEndCycle)
{
	unsigned long ulCycleNumbar;	/* Cycle number */
	unsigned long ulTrcDataNumber;	/* trace data number */
	unsigned long l;
	FILE *fpTraceFile = NULL;		/* pointer to parameter file */
	char cErrMsg[C33_TRACE_DATA_LINE_MAX + 1];
	char cTraceStr[C33_TRACE_DATA_LINE_MAX + 1];
	int  iCancel = 0;				/* Cancel flag 2003/03/25 T.Tazaki*/
	cErrMsg[0] = 0;


	/* Check number of parameter */
	if ( iParamNum == 2 )
	{
		/* bug fix 2005/01/12 T.Tazaki >>> */
		if(c33IcdTraceData.ucSearchFlag == 0) /* Not "c33 ts" mode */
		{
			if( g_iPC_TraceParam == 1 )
			{
				c33IcdTraceData.ulValidStart  = C33_ICD_TRACE_INVALID;
			}
			g_iPC_TraceParam = 0;
		}
		/* bug fix 2005/01/12 T.Tazaki <<< */
		ulStartCycle = c33IcdTraceData.ulIcdTraceBufferSize;
		ulEndCycle = 0;
	}
	else if ( iParamNum == 4 )
	{
		/* bug fix 2005/01/12 T.Tazaki >>> */
		if(c33IcdTraceData.ucSearchFlag == 0) /* Not "c33 ts" mode */
		{
			if( g_iPC_TraceParam == 0 )
			{
				c33IcdTraceData.ulValidStart  = C33_ICD_TRACE_INVALID;
			}
			g_iPC_TraceParam = 1;
		}
		/* bug fix 2005/01/12 T.Tazaki <<< */
		if (ulStartCycle > c33IcdTraceData.ulIcdTraceBufferSize || ulEndCycle > c33IcdTraceData.ulIcdTraceBufferSize)
		{
			c33_error (C33_ERR_COMMAND_TRC_SIZE);
			return;
		}
		if (ulEndCycle > ulStartCycle)
		{
			c33_error (C33_ERR_COMMAND_CYCLE);
			return;
		}
	}
	else
	{
		c33_error (C33_ERR_COMMAND_PARAM);
		return;
	}
	if ( pFilename[0] == 0 )
	{
		c33_error (C33_ERR_FILE_OPEN, pFilename);
		return;
	}

	if (c33TargetInfo.pMapInfo == 0)
	{
		c33_error (C33_ERR_NO_PARAMETER_FILE);
		return;
	}

	c33_IcdProcessing(0);/*  timer of "..." is reset */

	/* read icd33 trace data */
	if (c33_IcdTraceGetData() != C33_NO_ERR){
		c33_EndCancel();	/* add T.Tazaki 2004/09/13 */
		c33_ErrorMsg();
		return;
	}

	/* analyze trace data */
	if (c33_IcdTraceAnalyze() != C33_NO_ERR){
		c33_ErrorMsg();
		return;
	}

	/* display message "Making file." */
	puts_filtered (C33_MSG_TRC_MAKING_FILE);

	/* Open parameter file */
	fpTraceFile = fopen(pFilename, "wb");		/* "w" --> "wb" modify T.Tazaki 2003/05/19 */
	if (fpTraceFile == NULL)
	{
		/* file open error */
		c33_error (C33_ERR_FILE_OPEN, pFilename);
		return;
	}

	/* Write trace header */
	sprintf(cTraceStr, C33_MSG_TRC_HEADER);
	c33_AddCRLF(cTraceStr);

	c33_InitCancel();	/* Cancel initialize */ /* add T.Tazaki 2003/09/02 */
	iCancel = 0;

	if (fputs(cTraceStr, fpTraceFile) == EOF)
	{
		sprintf(cErrMsg, C33_ERR_FILE_WRITE);
	}
	else
	{
		/* Valid data? */	
		if (c33IcdTraceData.ulValidStart != C33_ICD_TRACE_INVALID)
		{

			/* Write file */
			ulTrcDataNumber = 0;
			ulCycleNumbar = (c33IcdTraceData.ulDataMaxNumber - 1);
			l = ulEndCycle;
			g_ulBeforePC = C33_ICD_TRACE_PC_INVALID;			/* add T.Tazaki 2005/02/15 */
			while(l <= ulStartCycle)
			{
				/* Check over run */
				if( ulTrcDataNumber > c33IcdTraceData.ulValidEnd )
				{
					break;
				}
				/* pass over idle cycle */
				if( (c33IcdTraceData.ucDst[ulTrcDataNumber] & C33_ICD_TRACE_DST0_1) != C33_ICD_TRACE_DST_IDLE || c33IcdTraceData.ucDebugIdleDsp != 0)
				{
					if(c33IcdTraceData.ucSearchFlag == 0 || (c33IcdTraceData.ucDst[ulTrcDataNumber] & C33_ICD_TRACE_HIT_PRE_POST) != 0 )
					{
						if( ulCycleNumbar <= ulStartCycle && ulCycleNumbar >= ulEndCycle )
						{
							g_iTraceDisplayMode = 0;	/* Trace displayu mode set for c33_IcdTraceMake1Line() LF code  add T.Tazaki 2007/07/25 */
							c33_IcdTraceMake1Line(ulTrcDataNumber, ulCycleNumbar, cTraceStr);
							if( cTraceStr[0] != 0 )	//* same address continue?	add T.Tazaki 2005/02/22 */
							{
								c33_AddCRLF(cTraceStr);
								c33_IcdProcessing(3000);
								if (fputs( cTraceStr, fpTraceFile) == EOF) 
								{
									/* file write error */
									sprintf(cErrMsg, C33_ERR_FILE_WRITE);
									break;
								}
							}
							l++;
							/* Check cancel add T.Tazaki 2004/11/30 >>> speed up */
							if (c33TargetInfo.iKeyBreak == 1 || c33_CheckCancel( 2000 ) == 1 ) 
							{
								/* Cancel */
								printf_filtered( C33_MSG_TARGET_CANCEL);
								c33_EndCancel();
								/* file close */
								if (fpTraceFile != NULL)
								{
									fclose(fpTraceFile);
								}
								return;
							}
							/* Check cancel add T.Tazaki 2004/11/30 <<< */
						}
					}
					ulCycleNumbar--;
				}
				ulTrcDataNumber++;

#if 0	/* del T.Tazaki 2004/11/29 speed up */
				/* Check cancel */
				if (c33TargetInfo.iKeyBreak == 1 || c33_CheckCancel( 2000 ) == 1 ) /* add T.Tazaki 2003/09/02 */
				{
					/* Cancel */
					iCancel = 1;
					printf_filtered( C33_MSG_TARGET_CANCEL);
					break;
				}
#endif
			}
		}
	}

	c33_EndCancel();	/* add T.Tazaki 2004/11/29 */
	
	/* file close */
	if (fpTraceFile != NULL)
	{
		fclose(fpTraceFile);
	}

	/* Check cancel 2003/03/25 T.Tazaki */
	if (iCancel == 0 )	/* Not cancel ? */
	{
		/* Print error message */
		if ( cErrMsg[0] != 0 )
		{
			c33_error (cErrMsg);
			return;
		}
	
		puts_filtered (C33_MSG_TRC_FINISHED);
	}
}

/****************************************************************************/
/* Format		 : void c33_IcdTraceSearch(int iParamNum, unsigned long ulSearchAddr, unsigned long ulPreLine, unsigned long ulPostLine) */
/* Arguments																*/
/*	int iParamNum			   : Parameter number							*/
/*	unsigned long ulSearchAddr : Search address 							*/
/*	unsigned long ulPreLine    : Pre line									*/
/*	unsigned long ulPostLine   : Post line									*/
/* Return value  : void 													*/
/* Explanation	 : Make trace search data file. 							*/
/****************************************************************************/
void c33_IcdTraceSearch(int iParamNum, unsigned long ulSearchAddr, unsigned long ulPreLine, unsigned long ulPostLine)
{
	unsigned long ulPreLineWork;				   /* pre lines */
	unsigned long ulPostLineWork;				   /* post lines */
	unsigned long ulSearchHitCounter = 0;		   /* search hit counter */
	unsigned long ulSearchDataCounter = 0;		   /* search data counter */
	unsigned long ulCounter;					   /* loop counter */
	int  iCancel = 0;							   /* Cancel flag 2003/04/07 T.Tazaki*/

//	if ( iParamNum != 4 )	/* bug fix 2004/12/01 T.Tazaki */
	if ( iParamNum != 2 && iParamNum != 4 )	/* bug fix 2004/12/01 T.Tazaki */
	{
		c33_error (C33_ERR_COMMAND_PARAM);
		return;
	}

	if ( ulPreLine > C33_PRE_POST_LINE_MAX || ulPostLine > C33_PRE_POST_LINE_MAX )
	{
		c33_error (C33_ERR_COMMAND_TRC_LINE);
		return;
	}
	
	if (ulSearchAddr == 0xffffffff)
	{
		c33IcdTraceData.ucSearchFlag = 0;
		return;
	}

	c33_IcdProcessing(0);/*  timer of "..." is reset */

	/* read icd33 trace data */
	if (c33_IcdTraceGetData() != C33_NO_ERR){
		c33_EndCancel();	/* add T.Tazaki 2004/09/13 */
		c33_ErrorMsg();
		return;
	}

	/* analyze trace data */
	if (c33_IcdTraceAnalyze() != C33_NO_ERR){
		c33_ErrorMsg();
		return;
	}

	/* display message "Search processing." */
	puts_filtered (C33_MSG_TRC_SEARCH_PROCESSING);

	/* data is valid ? */	
	if (c33IcdTraceData.ulValidStart != C33_ICD_TRACE_INVALID)
	{
		/* set search hit flag and post line flag */
		ulPostLineWork =0;
		for (ulCounter = 0;ulCounter < c33IcdTraceData.ulValidEnd;ulCounter++)
		{
			/* pass over idle cycle */
			if( (c33IcdTraceData.ucDst[ulCounter] & C33_ICD_TRACE_DST0_1) != C33_ICD_TRACE_DST_IDLE )
			{
				c33_IcdProcessing(3000);
				/* set post line flag */
				c33IcdTraceData.ucDst[ulCounter] &= (~C33_ICD_TRACE_HIT_PRE_POST & 0xff);/* old search flag clear */
				if(ulPostLineWork != 0)
				{
					c33IcdTraceData.ucDst[ulCounter] |= C33_ICD_TRACE_SEARCH_PRE_POST;
					ulPostLineWork--;
				}
				/* set search flag */
				if(c33IcdTraceData.ulPC[ulCounter] == ulSearchAddr && (c33IcdTraceData.ucMethod[ulCounter] & C33_ICD_TRACE_FIX_PC_BIT) == C33_ICD_TRACE_FIX_PC_BIT)
				{
					c33IcdTraceData.ucDst[ulCounter] |= C33_ICD_TRACE_SEARCH_HIT;
					ulSearchHitCounter++;
					ulPostLineWork = ulPostLine;
				}
			}
		}
		/* set pre line flag and calculate search data count */
		ulPreLineWork = 0;
		for (ulCounter = 1;ulCounter <= c33IcdTraceData.ulValidEnd;ulCounter++)
		{
			/* pass over idle cycle */
			if( (c33IcdTraceData.ucDst[c33IcdTraceData.ulValidEnd - ulCounter] & C33_ICD_TRACE_DST0_1) != C33_ICD_TRACE_DST_IDLE )
			{
				c33_IcdProcessing(3000);
				/* set pre line */
				if(ulPreLineWork != 0)
				{
					c33IcdTraceData.ucDst[c33IcdTraceData.ulValidEnd - ulCounter] |= C33_ICD_TRACE_SEARCH_PRE_POST;
					ulPreLineWork--;
				}
				else
				{
					/* null */
					/* clear is finished  by set search hit transaction */
				}
				if(c33IcdTraceData.ucDst[c33IcdTraceData.ulValidEnd - ulCounter] & C33_ICD_TRACE_SEARCH_HIT)
				{
					ulPreLineWork = ulPreLine;
				}
				/* calculate search data count */
				if( (c33IcdTraceData.ucDst[c33IcdTraceData.ulValidEnd - ulCounter] & C33_ICD_TRACE_HIT_PRE_POST) != 0 )
				{
					ulSearchDataCounter++;
				}
			}
		}
	}
	/* set search mode */
	c33IcdTraceData.ucSearchFlag = 1;
	c33IcdTraceData.ulSearchMaxNumber = ulSearchDataCounter;

	/* Check cancel 2003/04/07 T.Tazaki */
	if (iCancel == 0 )	/* Not cancel ? */
	{
		puts_filtered (C33_MSG_TRC_FINISHED);
	}
	return;
}
/****************************************************************************/
/* Format		 : int c33_IcdTraceOverwriteEnable(void) 					*/
/* Arguments																*/
/* Return value  : void 													*/
/* Explanation	 : Set Default Trace mode.									*/
/****************************************************************************/
static int
c33_IcdTraceOverwriteEnable()
{
	int iResult = C33_NO_ERR;
	struct c33_icd_trace_mode stTrcModeTmp;

	/* Check if PC trace is enable.   bug fix 2008/06/10 T.Tazaki */
	if( (c33TargetInfo.iVerRdIcdStatus & C33_DIPSW1_PC_TRACE) == 0 )	/* PC trace enabled ? bug fix add 2008/06/10 T.Tazaki */
	{

		stTrcModeTmp.iTraceMode   = C33_ICD_TRACE_MODE_ALL_OVER_WRITE_YES;
		stTrcModeTmp.iCounterMode = C33_ICD_TRACE_COUNTER_ACCUMULATE;
		stTrcModeTmp.ulTrg1 	  = C33_BREAK_DISABLE;
		stTrcModeTmp.ulTrg2 	  = C33_BREAK_DISABLE;
		stTrcModeTmp.iBreak 	  = C33_ICD_TRACE_BREAK_OFF;
		stTrcModeTmp.iMeasure	  = C33_ICD_TRACE_MEASURE_ALL;

		/* Transmission of a message. */
		iResult = c33_SendMessage(C33_ID_TRACE_TRIGGER, 0, 0, 0, 0, &stTrcModeTmp, 0, 0);
		/* Reception of a message. */
		if (iResult == C33_NO_ERR){
			iResult = c33_RecvMessage(C33_ID_TRACE_TRIGGER, 0, 0, 0, 0, 0);

		}
#if 0
		if( iResult != C33_NO_ERR )
		{
			printf_filtered (C33_WARN_DEFAULT_TRACE_MODE_SET);
		}
#endif

	}

	return iResult;
}
/****************************************************************************/
/* Format		 : void c33_IcdTraceMode(int iParamNum, int iTrcMode, int iCounterMode, int iTrg1, int iTrg2, int iBrk, int iMeasure) */
/* Arguments																*/
/*	int iParamNum	 : Parameter number 									*/
/*	int iTrcMode	 : Trace mode											*/
/*	int iCounterMode : Counter mode (ACCUMULATE,INSTRUCTION)				*/
/*	int iTrg1		 : Trigger address1 									*/
/*	int iTrg2		 : Trigger address2 									*/
/*	int iBrk		 : Break flag											*/
/*	int iMeasure	 : Measure mode (all,area)								*/
/* Return value  : void 													*/
/* Explanation	 : Make trace data file.									*/
/****************************************************************************/
void
c33_IcdTraceMode(int iParamNum, int iTrcMode, int iCounterMode, int iTrg1, int iTrg2, int iBrk, int iMeasure)
{
	int iResult = C33_NO_ERR;
	struct c33_icd_trace_mode stTrcModeTmp;

	/* Check number of parameter */
	if ( iParamNum == 1 )
	{
		c33_IcdTraceModeDisplay();
	}
	else
	{
		if ( iCounterMode != C33_ICD_TRACE_COUNTER_ACCUMULATE && iCounterMode != C33_ICD_TRACE_COUNTER_INST )
		{
			c33_error (C33_ERR_COMMAND_PARAM);
			return;
		}

		if ( iParamNum == 3 && (iTrcMode == C33_ICD_TRACE_MODE_ALL_OVER_WRITE_YES || iTrcMode == C33_ICD_TRACE_MODE_ALL_OVER_WRITE_NO) )
		{
		}
		else if ( iParamNum == 7 && iTrcMode == C33_ICD_TRACE_MODE_AREA_OVER_WRITE_NO )
		{
			if ( iBrk != C33_ICD_TRACE_BREAK_OFF && iBrk != C33_ICD_TRACE_BREAK_ON )
			{
				c33_error (C33_ERR_COMMAND_PARAM);
				return;
			}
			if ( iMeasure != C33_ICD_TRACE_MEASURE_ALL && iMeasure != C33_ICD_TRACE_MEASURE_AREA )
			{
				c33_error (C33_ERR_COMMAND_PARAM);
				return;
			}
			iTrg1 &= 0xfffffffe;
			iTrg2 &= 0xfffffffe;
			if ( c33_BreakAddrCheck(iTrg1, C33_TRACE_TRGGER_BREAK) == C33_ERR || c33_BreakAddrCheck(iTrg2, C33_TRACE_TRGGER_BREAK) == C33_ERR)
			{
				/* Display error message and return error at c33_BreakAddrCheck() */
			}
		}
		else
		{
			c33_error (C33_ERR_COMMAND_PARAM);
			return;
		}

		stTrcModeTmp = c33TargetInfo.c33TraceMode;
		stTrcModeTmp.iTraceMode   = iTrcMode;
		stTrcModeTmp.iCounterMode = iCounterMode;
		if ( iTrcMode == C33_ICD_TRACE_MODE_AREA_OVER_WRITE_NO )
		{
			stTrcModeTmp.ulTrg1 	  = iTrg1;
			stTrcModeTmp.ulTrg2 	  = iTrg2;
			stTrcModeTmp.iBreak 	  = iBrk;
			stTrcModeTmp.iMeasure	  = iMeasure;
		}
		/* add T.Tazaki 2003/12/16 >>> */
		else
		{
			stTrcModeTmp.ulTrg1 	  = C33_BREAK_DISABLE;
			stTrcModeTmp.ulTrg2 	  = C33_BREAK_DISABLE;
			stTrcModeTmp.iBreak 	  = C33_ICD_TRACE_BREAK_OFF;
			stTrcModeTmp.iMeasure	  = iMeasure;
		}
		/* add T.Tazaki 2003/12/16 <<< */

		/* Transmission of a message. */
		iResult = c33_SendMessage(C33_ID_TRACE_TRIGGER, 0, 0, 0, 0, &stTrcModeTmp, 0, 0);
		/* Reception of a message. */
		if (iResult == C33_NO_ERR){
			iResult = c33_RecvMessage(C33_ID_TRACE_TRIGGER, 0, 0, 0, 0, 0);
		}

		if (iResult == C33_NO_ERR){
			/* store trace mode */
			c33TargetInfo.c33TraceMode = stTrcModeTmp;
			c33_IcdTraceModeDisplay();
		}
		else
		{
			c33_ErrorMsg();
		}
	}
}

/****************************************************************************/
/* Format		 : static void c33_IcdTraceModeDisplay						*/
/* Arguments	 : void 													*/
/* Return value  : void 													*/
/* Explanation	 : Display trace mode.										*/
/****************************************************************************/
static void
c33_IcdTraceModeDisplay(void)
{
	/* Print trace mode */
	if( c33TargetInfo.c33TraceMode.iTraceMode == C33_ICD_TRACE_MODE_ALL_OVER_WRITE_YES )
	{
		printf_filtered (C33_MSG_TRC_OVER_WRITE_YES);
	}
	else if( c33TargetInfo.c33TraceMode.iTraceMode == C33_ICD_TRACE_MODE_ALL_OVER_WRITE_NO )
	{
		printf_filtered (C33_MSG_TRC_OVER_WRITE_NO);
	}
	else if( c33TargetInfo.c33TraceMode.iTraceMode == C33_ICD_TRACE_MODE_AREA_OVER_WRITE_NO )
	{
		printf_filtered (C33_MSG_TRC_AREA);
	}
	/* Print clock display */
	if( c33TargetInfo.c33TraceMode.iCounterMode == C33_ICD_TRACE_COUNTER_ACCUMULATE )
	{
		printf_filtered (C33_MSG_TRC_ACCUMULATE);
	}
	else if( c33TargetInfo.c33TraceMode.iCounterMode == C33_ICD_TRACE_COUNTER_INST )
	{
		printf_filtered (C33_MSG_TRC_INST);
	}
	/* area trace mode only */
	if( c33TargetInfo.c33TraceMode.iTraceMode == C33_ICD_TRACE_MODE_AREA_OVER_WRITE_NO )
	{
		printf_filtered (C33_MSG_TRC_TRIGGER1_ADDR, (int)c33TargetInfo.c33TraceMode.ulTrg1);
		printf_filtered (C33_MSG_TRC_TRIGGER2_ADDR, (int)c33TargetInfo.c33TraceMode.ulTrg2);

		if( c33TargetInfo.c33TraceMode.iBreak == C33_ICD_TRACE_BREAK_OFF )
		{
			printf_filtered (C33_MSG_TRC_BREAK_NO);
		}
		else if( c33TargetInfo.c33TraceMode.iBreak == C33_ICD_TRACE_BREAK_ON )
		{
			printf_filtered (C33_MSG_TRC_BREAK_YES);
		}

		if( c33TargetInfo.c33TraceMode.iMeasure == C33_ICD_TRACE_MEASURE_ALL )
		{
			printf_filtered (C33_MSG_TRC_MEASURE_ALL);
		}
		else if( c33TargetInfo.c33TraceMode.iMeasure == C33_ICD_TRACE_MEASURE_AREA )
		{
			printf_filtered (C33_MSG_TRC_MEASURE_AREA);
		}
		printf_filtered(C33_MSG_TRC_HARD_PC_BREAK_OFF);
	}
}

/****************************************************************************/
/* Format		 : static int c33_IcdTraceAnalyze(void) 					*/
/* Arguments	 : void 													*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : This function analyze trace data.						*/
/*				   Aanalyze sequence										*/
/*					1.analyze by start PC and trigger address				*/
/*					2.analyze by DPCO										*/
/*					3.analyze by ret instruction							*/
/*					4.analyze by map information							*/
/*					5.analyze by reti instruction							*/
/****************************************************************************/
static int
c33_IcdTraceAnalyze(void)
{
	int iRet = C33_NO_ERR;

	/* check and set analyze completed flag */
	if(c33IcdTraceData.ucAnalyzeFlag == 1){ /* 0:yet analysis 1:analysis finished */
		return iRet;						/* analize comleted allready */
	}
	else{
		c33IcdTraceData.ucAnalyzeFlag = 1;
	}

	puts_filtered (C33_MSG_TRC_ANALYZING);

	/* analyze trace data used DPCO signel */
	if(c33_IcdTraceAnalyzeTrigger() == C33_ERR)
	{
		iRet = C33_ERR;
	}
	/* analyze trace data used PC ABS */
	else if(c33_IcdTraceAnalyzeDpco() == C33_ERR)
	{
		iRet = C33_ERR;
	}
	/* analyze trace data used DPCO H31+L31 */ /* Analysis new DPCO add T.Tazaki 2005/03/01 */
	else if(c33_IcdTraceAnalyzeAdvDpco() == C33_ERR)
	{
		iRet = C33_ERR;
	}
	/* analyze trace data used return instruction */
	else if(c33_IcdTraceAnalyzeRet() == C33_ERR)
	{
		iRet = C33_ERR;
	}
	/* analyze trace data used map information */
	else if(c33_IcdTraceAnalyzeMap() == C33_ERR)
	{
		iRet = C33_ERR;
	}
	/* analyze trace data used return interrupt instruction */
	else if(c33_IcdTraceAnalyzeReti() == C33_ERR)
	{
		iRet = C33_ERR;
	}
	/* delayed instruction trace data modify */ /* add T.Tazaki 2004/11/04 */
	else if(c33_IcdTraceModify() == C33_ERR)
	{
		iRet = C33_ERR;
	}
#if 0	/* del 2004/04/09 new dpco analysis move */
	/* analyze trace data used ABS PC header */ /* add T.Tazaki 2003/07/04 */
	else if(c33_IcdTraceAnalyzeAdvDpco() == C33_ERR)
	{
		iRet = C33_ERR;
	}
#endif

	/* delayed instruction trace data modify */ /* add T.Tazaki 2004/11/04 */
	else if(c33_IcdDebugDisp() == C33_ERR)
	{
		iRet = C33_ERR;
	}
	/* Set data munber of trace data structure */
	c33_IcdTraceCalMaxNumber();

	puts_filtered (C33_MSG_TRC_FINISHED);

	return iRet;
}

/****************************************************************************/
/* Format		 : static int c33_IcdTraceAnalyzeTrigger(void)				*/
/* Arguments	 : void 													*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : This function analyze by start PC and trigger address.	*/
/****************************************************************************/
static int
c33_IcdTraceAnalyzeTrigger(void)
{
	unsigned long ulPC = C33_ICD_TRACE_PC_INVALID;		  /* execution PC */	
	unsigned long ulDelayedPC = C33_ICD_TRACE_PC_INVALID; /* delayed instruction PC */
	unsigned char ucDelayed = C33_FALSE;				  /* delayed instruction flag */
	unsigned long ulCycleCounter;						  /* cycle counter */
	unsigned char ucDpcoCounter = c33IcdTraceData.ucDpcoAddrFix;	 /* DPCO counter */
	unsigned char ucDpcoDumy;							  /* DPCO dumy */
	unsigned char ucTrcValid;							  /* analyze flag */

	/* analize by start pc */
	/* unusable case  1.not over write mode and trace data is full
					  2.area trace mode
					  3.realtime trace mode(except 1th time)
	*/
	
	if( g_iCycleOver10000 == 0 )	/* Not trace default cycle read mode ?	add T.Tazaki 2004/12/01 */
	{

		if(c33IcdTraceData.stTraceMode.iTraceMode != C33_ICD_TRACE_MODE_AREA_OVER_WRITE_NO) /* Not area trace mode */
		{
			if(c33IcdTraceData.stTraceMode.iTraceMode == C33_ICD_TRACE_MODE_ALL_OVER_WRITE_NO || c33IcdTraceData.ulValidEnd < c33IcdTraceData.ulIcdTraceBufferSize) /* in runnning data is smoller then 0x100000 on over write mode */
			{
				if(c33IcdTraceData.ulStartPcAddr != C33_ICD_TRACE_PC_INVALID) /* Start pc enable */
				{
					ulPC = c33IcdTraceData.ulStartPcAddr;/* start PC saved when cpu running */
					for( ulCycleCounter = c33IcdTraceData.ulValidStart ; ulCycleCounter < c33IcdTraceData.ulValidEnd ; ulCycleCounter++)
					{
						if( ucDpcoCounter != c33IcdTraceData.ucDpcoAddrFix && ucDelayed == C33_FALSE){
							/* analyze finished already or execute absolute jump instruntion */
							break;
						}
						else
						{
							/* analyze by dst */
							c33_IcdTraceAnalyzeMakePcByDST(C33_ICD_TRACE_FIX_START_PC, ulCycleCounter, &ucDelayed, &ucDpcoCounter, &ulPC, &ulDelayedPC);
						}
					}
				}
			}
		}

	}

	/* trigger mode check */
	if(c33IcdTraceData.stTraceMode.iTraceMode == C33_ICD_TRACE_MODE_AREA_OVER_WRITE_NO)
	{
		ucTrcValid = C33_FALSE;
		ucDpcoCounter = 0;
		/* used trigger address */
		for( ulCycleCounter = c33IcdTraceData.ulValidStart ; ulCycleCounter < c33IcdTraceData.ulValidEnd ; ulCycleCounter++)
		{
			/* search trigger hit data */	
			if( (c33IcdTraceData.ucDst[ulCycleCounter] & C33_ICD_TRACE_TRG1_HIT ) != 0 )
			/* if( (c33IcdTraceData.ucDst[ulCycleCounter] & (C33_ICD_TRACE_TRG1_HIT | C33_ICD_TRACE_TRG2_HIT) ) != 0 ){ */
			{
				/* next is not continuance data.clear flag */
				ucDelayed = C33_FALSE;
				ucDpcoCounter = 0;
				ucTrcValid = C33_FALSE;
				if( (c33IcdTraceData.ucDst[ulCycleCounter] & C33_ICD_TRACE_TRG1_HIT) == C33_ICD_TRACE_TRG1_HIT)
				{
					ucDpcoCounter = c33IcdTraceData.ucDpcoAddrFix;
					ulPC = c33IcdTraceData.stTraceMode.ulTrg1;
					ucTrcValid = C33_TRUE;
				}
			}

			/* delayed instruction case */
			if(ucDelayed == C33_TRUE)
			{
				ucDpcoDumy = c33IcdTraceData.ucDpcoAddrFix;
				c33_IcdTraceAnalyzeMakePcByDST(C33_ICD_TRACE_FIX_TIRGGER,ulCycleCounter,&ucDelayed,&ucDpcoDumy,&ulPC,&ulDelayedPC);
			}
			else if( (c33IcdTraceData.ucMethod[ulCycleCounter] & C33_ICD_TRACE_FIX_PC_BIT) == C33_ICD_TRACE_FIX_PC_BIT || ucDpcoCounter != c33IcdTraceData.ucDpcoAddrFix )
			{
				/* analyze finished already or execute absolute jump instruntion */
				ucTrcValid = C33_FALSE;
			}
			else
			{
				/* analyze by dst */
				c33_IcdTraceAnalyzeMakePcByDST(C33_ICD_TRACE_FIX_TIRGGER,ulCycleCounter,&ucDelayed,&ucDpcoCounter,&ulPC,&ulDelayedPC);
			}
		}
	}
	else{
		/* not used trigger address */
	}

	return C33_NO_ERR;
}

/****************************************************************************/
/* Format		 : static int c33_IcdTraceAnalyzeDpco(void) 				*/
/* Arguments	 : void 													*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : This function analyze trace data used DPCO signel.		*/
/****************************************************************************/
static int
c33_IcdTraceAnalyzeDpco(void)
{
	unsigned long  ulPC = C33_ICD_TRACE_PC_INVALID; 	   /* execution PC */	
	unsigned long  ulDelayedPC = C33_ICD_TRACE_PC_INVALID; /* delayed instruction PC */
	unsigned char  ucDpcoNum = 0;						   /* DPCO output counter */
	unsigned char  ucDelayed = C33_FALSE;				   /* delayed instruction flag */
	unsigned long  ulCycleCounter;						   /* cycle counter */
	unsigned long  ulUpstreamCounter;					   /* cycle counter of upstream analyze */

	g_ucPC_Rel_Delayed = C33_FALSE;		/* add T.Tazaki 2005/03/01 */
	g_ucPC_Abs_Delayed = C33_FALSE;		/* add T.Tazaki 2005/03/01 */

	/* make PC from DPCO signel */	
	if(c33IcdTraceData.ulValidEnd < 1)
	{
		return C33_NO_ERR;
	}
	
	for(ulCycleCounter = c33IcdTraceData.ulValidStart;ulCycleCounter < c33IcdTraceData.ulValidEnd - 1;ulCycleCounter++)
	{
		/* check continuance data */
		if( (c33IcdTraceData.ucDst[ulCycleCounter] & C33_ICD_TRACE_TRG1_HIT) != 0 )
		{
			ucDpcoNum = 0;
			ucDelayed = C33_FALSE;
		}

		if (c33TargetInfo.ucAdvanced == C33_PRODUCT_STD)	/* add T.Tazaki 2003/04/24 >>> */
		{
			/* Standard macro */

			/* transaction by DPCO counter */
			/* case 0: find absolute jump case */
			/* case 28: just 28bit case */
			/* case 29: output is end case */
			/* default : putting dpco case */
			switch(ucDpcoNum)
			{
				case 0:
					if( (c33IcdTraceData.ucDst[ulCycleCounter] & C33_ICD_TRACE_DST0_1) == C33_ICD_TRACE_DST_ABS_JP )
					{
						/* fined absolute instruction */
						ucDpcoNum = C33_DPCO_TIMING;
						ulPC = 0;
					}
					break;
				case 28:
					ulPC += ((c33IcdTraceData.ucDst[ulCycleCounter + 1] & C33_ICD_TRACE_DPCO_BIT) >> 3) << (ucDpcoNum - 1);/* make PC from DPCO */	
					ucDpcoNum++;/* DPCO output counter renewal */
					/* -27 is absolute instruction position */
					/* -26 is adle cycle or delayed instruction position .*/ /* ! analyze default case .skip this case */	
					/* makeup PC is after -25 potition */	
					ulUpstreamCounter = ulCycleCounter - C33_ICD_DPCO_25;
					/* set pc from dpco start to dpco end */
					while(ulCycleCounter >= ulUpstreamCounter)
					{
						/* analyze by dst */
						c33_IcdTraceAnalyzeMakePcByDST(C33_ICD_TRACE_FIX_DPCO,ulUpstreamCounter,&ucDelayed,&ucDpcoNum,&ulPC,&ulDelayedPC);
						ulUpstreamCounter++;
					}
					break;
				case C33_ICD_DPCO_FIX29:
					/* analyze by dst */
					c33_IcdTraceAnalyzeMakePcByDST(C33_ICD_TRACE_FIX_DPCO,ulCycleCounter,&ucDelayed,&ucDpcoNum,&ulPC,&ulDelayedPC);
					break;
				default:
					ulPC += ((c33IcdTraceData.ucDst[ulCycleCounter + 1] & C33_ICD_TRACE_DPCO_BIT) >> 3) << (ucDpcoNum - 1);/* make PC from DPCO */	
					ucDpcoNum++;/* DPCO output counter renewal */
					/* analyze by dst */
					c33_IcdTraceAnalyzeMakePcByDST(C33_ICD_TRACE_FIX_DPCO,ulCycleCounter,&ucDelayed,&ucDpcoNum,&ulPC,&ulDelayedPC);
					if( (c33IcdTraceData.ucDst[ulCycleCounter] & C33_ICD_TRACE_DST0_1) == C33_ICD_TRACE_DST_ABS_JP )
					{
						/* find absolute instruction */
						ucDpcoNum = C33_DPCO_TIMING;
						ulPC = 0;
					}
					break;
			}
		}
		else
		{
			/* Advanced macro */
			
			/* transaction by DPCO counter */
			/* case 0: find absolute jump case */
			/* case 31: just 32bit case */
			/* case 32: output is end case */
			/* default : putting dpco case */
			switch(ucDpcoNum)
			{
				case 0:
					if( (c33IcdTraceData.ucDst[ulCycleCounter] & C33_ICD_TRACE_DST0_1) == C33_ICD_TRACE_DST_ABS_JP )
					{
						/* fined absolute instruction */
						ucDpcoNum = 1;
						ulPC = 0;
					}
					break;
				case 31:
					ulPC += ((c33IcdTraceData.ucDst[ulCycleCounter] & C33_ICD_TRACE_DPCO_BIT) >> 3) << ucDpcoNum;/* make PC from DPCO */	
					ucDpcoNum++;/* DPCO output counter renewal */
					/* -31 is absolute instruction position */
					/* -30 is adle cycle or delayed instruction position .*/ /* ! analyze default case .skip this case */	
					/* makeup PC is after -30 potition */	
					ulUpstreamCounter = ulCycleCounter - 30;	/* - 28 --> - 30 T.Tazaki 2003/06/18 */
					/* set pc from dpco start to dpco end */
					while(ulCycleCounter >= ulUpstreamCounter)
					{
						/* analyze by dst */
						c33_IcdTraceAnalyzeMakePcByDST(C33_ICD_TRACE_FIX_DPCO,ulUpstreamCounter,&ucDelayed,&ucDpcoNum,&ulPC,&ulDelayedPC);
						ulUpstreamCounter++;
					}
					break;
				case C33_ICD_DPCO_FIX32:
					/* analyze by dst */
					c33_IcdTraceAnalyzeMakePcByDST(C33_ICD_TRACE_FIX_DPCO,ulCycleCounter,&ucDelayed,&ucDpcoNum,&ulPC,&ulDelayedPC);
					break;
				default:
					ulPC += ((c33IcdTraceData.ucDst[ulCycleCounter] & C33_ICD_TRACE_DPCO_BIT) >> 3) << ucDpcoNum;/* make PC from DPCO */	
					ucDpcoNum++;/* DPCO output counter renewal */
					/* analyze by dst */
					c33_IcdTraceAnalyzeMakePcByDST(C33_ICD_TRACE_FIX_DPCO,ulCycleCounter,&ucDelayed,&ucDpcoNum,&ulPC,&ulDelayedPC);
					if( (c33IcdTraceData.ucDst[ulCycleCounter] & C33_ICD_TRACE_DST0_1) == C33_ICD_TRACE_DST_ABS_JP )
					{
						/* find absolute instruction */
						ucDpcoNum = 1;
						ulPC = 0;
					}
					break;
			}
		}
		
	}
	if(ucDpcoNum != 0)
	{
		c33_IcdTraceAnalyzeMakePcByDST(C33_ICD_TRACE_FIX_DPCO,ulCycleCounter,&ucDelayed,&ucDpcoNum,&ulPC,&ulDelayedPC);
	}

	return C33_NO_ERR;
}

/* add T.Tazaki 2003/07/03 >>> */

/****************************************************************************/
/* Format		 : static int c33_IcdTraceAnalyzeAdvDpco(void)				*/
/* Arguments	 : void 													*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : This function analyze trace data used DPCO signel.		*/
/****************************************************************************/
static int
c33_IcdTraceAnalyzeAdvDpco(void)
{
	unsigned long  ulPC = C33_ICD_TRACE_PC_INVALID; 	   /* execution PC */	
	unsigned long  ulPC_wk = C33_ICD_TRACE_PC_INVALID;	   /* execution PC	 temporary		 add T.Tazaki 2005/03/02 */ 	
	unsigned long  ulDelayedPC = C33_ICD_TRACE_PC_INVALID; /* delayed instruction PC */
	unsigned char  ucDpcoNum = 0;						   /* DPCO output counter */
	unsigned char  ucDelayed = C33_FALSE;				   /* delayed instruction flag */
	unsigned long  ulCycleCounter;						   /* cycle counter */
	unsigned long  i;
	unsigned long  ulCycle;								   
	unsigned long  ulCycle_wk;							   /* ulCycle temporary 	  add T.Tazaki 2005/03/02 */
	int 		   iHighCnt;							   /* DPCO = 1 counter	max 31 */
	int 		   iLowCnt; 							   /* DPCO = 0 counter	max 31 */
	unsigned short	usCode;								   /* read instruction code   add T.Tazaki 2005/02/17 */
	

#if 0	/* del T.Tazaki 2005/02/17 >>> */

	/*===========================================================================*/
	/* Analyze by DPCO header : [ '1' * 31bit ]+[ '0' * 31bit ]+[ ABS PC 31bit ] */ 
	/*===========================================================================*/

	if (c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV || c33TargetInfo.ucAdvanced == C33_PRODUCT_PE ) /* add PE T.Tazaki 2003/11/21 */
	{
		/* counters initialize */
		iHighCnt = 0;
		iLowCnt = 0;
		i = 0xffffffff;
		ulPC = 0;

		for(ulCycleCounter = c33IcdTraceData.ulValidStart;ulCycleCounter < c33IcdTraceData.ulValidEnd; ulCycleCounter++)
		{
			if( iHighCnt >= 31 && iLowCnt == 31 )
			{
				if( i == 0xffffffff )
				{
					i = 1;
				}
				/* make PC from dpco */
				if( i < 32 )
				{
					ulPC += ((c33IcdTraceData.ucDst[ ulCycleCounter ] & C33_ICD_TRACE_DPCO_BIT) >> 3) << i;
					++i;
					
					if( i >= 32 )
					{
						/* Update PC ABS to ( High 31 <---> Address 31 ), not IDLE Cycle */
						for ( ulCycle = ulSaveulCycleCounter; ulCycle < ulCycleCounter; ++ulCycle )
						{
							/* check if cycle is not IDLE */
							if ( (c33IcdTraceData.ucDst[ulCycle] & C33_ICD_TRACE_DST0_1) != C33_ICD_TRACE_DST_IDLE )
							{
								/* check if method is not TRACE FIX */
								if ( (c33IcdTraceData.ucMethod[ulCycle] & C33_ICD_TRACE_FIX_PC_BIT) != C33_ICD_TRACE_FIX_PC_BIT)
								{
									/* set trace data */
									c33IcdTraceData.ucDst[ulCycle] = (c33IcdTraceData.ucDst[ulCycle] & ~C33_ICD_TRACE_DST0_1) | C33_ICD_TRACE_DST_ABS_JP;
									c33IcdTraceData.ucDpcoNum[ulCycle] = C33_ICD_DPCO_FIX32;
									c33IcdTraceData.ulPC[ulCycle] = ulPC;
									c33IcdTraceData.ucMethod[ulCycle] = (C33_ICD_TRACE_FIX_PC_BIT | C33_ICD_TRACE_FIX_DPCO);
									/* save code */
									c33_ReadWriteMapMem(C33_READ, ulPC, C33_HALF_WORD, C33_HALF_WORD, (unsigned char*)&c33IcdTraceData.usCode[ulCycle]);
								}
							}
						}
						/* counters initialize */
						iHighCnt = 0;
						iLowCnt = 0;
						i = 0xffffffff;
						ulPC = 0;
					}
				}
			}
			else if( ( (c33IcdTraceData.ucDst[ulCycleCounter] & C33_ICD_TRACE_DPCO_BIT) >> 3) == 1 )
			{
				if( iHighCnt == 0 )
				{
					ulSaveulCycleCounter = ulCycleCounter;		/* Save DPCO Start Cycle */
				}
				iHighCnt++; 	/* count up '1' */
				iLowCnt = 0;
			}
			else
			{
				iLowCnt++;		/* count up '0' */
				if( iHighCnt < 31 )
				{
					iHighCnt = 0;
				}
			}
		}
	}

#endif

	/* add T.Tazaki 2005/02/28 >>> */

	/*===========================================================================*/
	/* Analyze by DPCO header : [ HIGH 31bit ]+[ LOW 31bit ]+[ ADDR 31bit ] 	 */ 
	/*				ADDR is LOW 31bit.											 */
	/*===========================================================================*/

	if (c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV || c33TargetInfo.ucAdvanced == C33_PRODUCT_PE ) 
	{
		/* counters initialize */
		iHighCnt = 0;
		iLowCnt = 0;

		ucDpcoNum = 0;						/* add T.Tazaki 2005/03/01 */
		ucDelayed = C33_FALSE;				/* add T.Tazaki 2005/03/01 */
		g_ucPC_Rel_Delayed = C33_FALSE;		/* add T.Tazaki 2005/03/01 */
		g_ucPC_Abs_Delayed = C33_FALSE;		/* add T.Tazaki 2005/03/01 */

		for(ulCycleCounter = c33IcdTraceData.ulValidStart;ulCycleCounter < c33IcdTraceData.ulValidEnd; ulCycleCounter++)
		{
			if( iHighCnt >= 31 && iLowCnt == 31 )
			{
				ulPC = 0;
				i = 1;

	  			for ( ulCycle = ulCycleCounter; ulCycle < (ulCycleCounter + 31); ++ulCycle )
				{
					/* check if cycle count is over trace size. */
					if( ulCycle >= c33IcdTraceData.ulValidEnd )
					{
						g_ucPC_Rel_Delayed = C33_FALSE;		/* add T.Tazaki 2005/03/01 */
						g_ucPC_Abs_Delayed = C33_FALSE;		/* add T.Tazaki 2005/03/01 */
						return C33_NO_ERR;
					}
					/* check if ABS jump is exist. */
					if( (c33IcdTraceData.ucDst[ulCycle] & C33_ICD_TRACE_DST0_1) == C33_ICD_TRACE_DST_ABS_JP )
					{
						break;	/* exit for ulCycle */
					}
					ulPC += (((c33IcdTraceData.ucDst[ ulCycle ] & C33_ICD_TRACE_DPCO_BIT) >> 3) << i);
					++i;
				}
				if( ulCycle < (ulCycleCounter + 31) )	/* check if ABS jump is exist. */
				{
					/* counters initialize */
					iHighCnt = 0;
					iLowCnt = 0;
					ulCycleCounter = ulCycle;	/* Search H31+L31 from the next ABS jump cycle */
					continue;
				}

				/*==========================================================*/
				/* Distinction of a definite detection start cycle			*/
				/*==========================================================*/
				
			
				if( (c33IcdTraceData.ucDst[ulCycleCounter - 1] & C33_ICD_TRACE_DST0_1) == C33_ICD_TRACE_DST_IDLE )
				{
					/*********************************************/
					/* check LOW[31]==IDLE or normal             */
					/*********************************************/


					/* A instruntion of [ulPC] distinguishes in the time of ext instruntion, and the time which is not so. */
					
					/* read instruntion */
					c33_ReadWriteMapMem(C33_READ, ulPC, C33_HALF_WORD, C33_HALF_WORD, (unsigned char*)&usCode);
					if( ( ( usCode & 0xe000 ) == 0xc000 ) ||  /* ext imm13					  */
						( ( usCode & 0xff00 ) == 0x3b00 ) ||  /* ext op,imm2  ext cont		  */
						( ( usCode & 0xff00 ) == 0x3f00 ) )   /* ext %rs	  ext %rs,op,imm2 */
					{
						/* [LOW28] NORMAL  */
						/* [LOW29] IDLE    */
						/* [LOW30] IDLE    */
						/* [LOW31]*IDLE    ext */
						/* [addr1] IDLE   <-- start analysis */
						/* [addr2] NORMAL  inst after ext  */
						ulCycle = ulCycleCounter;
					}
					else
					{
						/* [LOW28] NORMAL <---start analysis  */
						/* [LOW29] IDLE   */
						/* [LOW30] IDLE   */
						/* [LOW31]*IDLE   not ext */
						/* [addr1] IDLE   */
						/* [addr2] NORMAL */
						
						/* searchs except IDLE upward*/
						
						for( ulCycle = ulCycleCounter - 2; ulCycle > 0; --ulCycle )
						{
							if( (c33IcdTraceData.ucDst[ulCycle] & C33_ICD_TRACE_DST0_1) != C33_ICD_TRACE_DST_IDLE )
							{
								break;
							}
						}
						if( (c33IcdTraceData.ucDst[ulCycle] & C33_ICD_TRACE_DST0_1) == C33_ICD_TRACE_DST_NORM )
						{
							/* [LOW23] IDLE  (ext command is possible.)	<--- ulPC is here */
							/* [LOW24] IDLE  (ext command is possible.)	*/
							/* [LOW25] PC REL(delayed jumpj	<---start analysis	*/
							/* [LOW26] IDLE   */
							/* [LOW27] IDLE   */
							/* [LOW28] NORMAL(delayed jumpj */
							/* [LOW29] IDLE   */
							/* [LOW30] IDLE   */
							/* [LOW31]*IDLE   except ext */
							/* [addr1] IDLE   */
							/* [addr2] NORMAL */
							
							
							/* searchs except PC REL upward*/
							for( ulCycle_wk = ulCycle - 1; ulCycle_wk > 0; --ulCycle_wk )
							{
								if( (c33IcdTraceData.ucDst[ulCycle_wk] & C33_ICD_TRACE_DST0_1) != C33_ICD_TRACE_DST_IDLE )
								{
									 /* REL JUMP ? */
									if( (c33IcdTraceData.ucDst[ulCycle_wk] & C33_ICD_TRACE_DST0_1) == C33_ICD_TRACE_DST_JP ) /* PC REL ? */
									{
										/* Delayed inst? ? */
										c33_ReadWriteMapMem(C33_READ, ulPC - 2, C33_HALF_WORD, C33_HALF_WORD, (unsigned char*)&usCode);
										if(	( (usCode & 0xff00) == 0x0900 ) || 	/* jrgt.d sign8  */
											( (usCode & 0xff00) == 0x0b00 ) || 	/* jrge.d sign8  */
											( (usCode & 0xff00) == 0x0d00 ) || 	/* jrlt.d sign8  */
											( (usCode & 0xff00) == 0x0f00 ) || 	/* jrle.d sign8  */
											( (usCode & 0xff00) == 0x1100 ) || 	/* jrugt.d sign8 */
											( (usCode & 0xff00) == 0x1300 ) || 	/* jruge.d sign8 */
											( (usCode & 0xff00) == 0x1500 ) || 	/* jrult.d sign8 */
											( (usCode & 0xff00) == 0x1700 ) || 	/* jrule.d sign8 */
											( (usCode & 0xff00) == 0x1900 ) || 	/* jreq.d sign8  */
											( (usCode & 0xff00) == 0x1b00 ) || 	/* jrne.d sign8  */
											( (usCode & 0xff00) == 0x1d00 ) || 	/* call.d sign8  */
											( (usCode & 0xff00) == 0x1f00 ) ) 	/* jp.d sign8	 */
										{
											ulCycle = ulCycle_wk;		/* Set analizing start cycle */

											/* searchs EXT upward */
											for( ulPC_wk = ulPC - 4 ; ulPC_wk >= 2 ; ulPC_wk -= 2 )
											{
												c33_ReadWriteMapMem(C33_READ, ulPC_wk, C33_HALF_WORD, C33_HALF_WORD, (unsigned char*)&usCode);
												if( ( usCode & 0xe000 ) != 0xc000 )  /* ext imm13 ? */
												{
													ulPC_wk += 2;
													break;
												}
											}
											ulPC = ulPC_wk;
										}
										break;
									}
									else
									{
										break;
									}
								}
							}
						}
					}
				}
				else
				{
					/*********************************************/
					/* When LOW[31]<>IDLE, Normal inst           */
					/*********************************************/

					/* [LOW28] NORMAL */
					/* [LOW29] IDLE   */
					/* [LOW30] IDLE   */
					/* [LOW31]*NORMAL <---start analisys */
					/* [addr1] IDLE   */
					/* [addr2] IDLE   */
					ulCycle = ulCycleCounter - 1;
				}

				if( iDebugTrace == 1 )	/* debug mode ? */
				{
					printf_filtered( "\n OUT %d --->  ulPC=[%08x]\n",ulCycle, ulPC);
				}


				/* Make result of analysis to PC ABS or c33IcdTraceData.ulValidEnd.  */

				for ( ;  ; ++ulCycle )														
				{
					if( ulCycle == c33IcdTraceData.ulValidEnd )	/* Make to PC ABS but, Was the loop carried out to the last? */
					{
						ulCycleCounter = c33IcdTraceData.ulValidEnd;	/* End of analysis */
						break;
					}
					/* check if cycle is PC ABS */
					if( (c33IcdTraceData.ucDst[ulCycle] & C33_ICD_TRACE_DST0_1) == C33_ICD_TRACE_DST_ABS_JP )
					{
						break;	/* exit for ulCycle */
					}
					if( iDebugTrace == 1 )	/* debug mode ? */
					{
						if( ulPC != C33_ICD_TRACE_PC_INVALID )
						{
							if( (c33IcdTraceData.ucDst[ulCycle] & C33_ICD_TRACE_DST0_1) != C33_ICD_TRACE_DST_IDLE )
							{
								if( (c33IcdTraceData.ucMethod[ulCycle] & C33_ICD_TRACE_FIX_PC_BIT) != C33_ICD_TRACE_FIX_PC_BIT)
								{
									printf_filtered( "---- Cycle=%d,ulPC=[%08x]iHighCnt=[%d],iLowCnt=[%d],i=[%d] \n",ulCycle,ulPC,iHighCnt,iLowCnt,i );
								}
							}
						}
					}
					/* analyze by dst */
					ucDpcoNum = c33IcdTraceData.ucDpcoAddrFix;
					c33_IcdTraceAnalyzeMakePcByDST(C33_ICD_TRACE_FIX_DPCO,ulCycle,&ucDelayed,&ucDpcoNum,&ulPC,&ulDelayedPC);
				}

				if( ulCycleCounter == c33IcdTraceData.ulValidEnd)	/* End of analysis */ /* add T.Tazaki 2005/03/03 */
				{
					break;
				}
					
				/* counters initialize */
				iHighCnt = 0;
				iLowCnt = 0;

				ucDpcoNum = 0;						/* add T.Tazaki 2005/03/01 */
				ucDelayed = C33_FALSE;				/* add T.Tazaki 2005/03/01 */
				g_ucPC_Rel_Delayed = C33_FALSE;		/* add T.Tazaki 2005/03/01 */
				g_ucPC_Abs_Delayed = C33_FALSE;		/* add T.Tazaki 2005/03/01 */

			}
			else if( ( (c33IcdTraceData.ucDst[ulCycleCounter] & C33_ICD_TRACE_DPCO_BIT) >> 3) == 1 )
			{
				if( iLowCnt > 0 )
				{
					iHighCnt = 0;
				}
				iHighCnt++; 	/* count up '1' */
				iLowCnt = 0;
			}
			else
			{
				iLowCnt++;		/* count up '0' */
				if( iHighCnt < 31 )
				{
					iHighCnt = 0;
				}
			}
		}
	}
	/* add T.Tazaki 2005/02/28 <<< */

	g_ucPC_Rel_Delayed = C33_FALSE;		/* add T.Tazaki 2005/03/01 */
	g_ucPC_Abs_Delayed = C33_FALSE;		/* add T.Tazaki 2005/03/01 */

	return C33_NO_ERR;
}


/****************************************************************************/
/* Format		 : static int c33_IcdTraceAnalyzeRet(void)					*/
/* Arguments	 : void 													*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : This function analyze by ret instruction.				*/
/*				   1.Search return instruncton. 							*/
/*				   2.Search call instruncton.The call is a pair of return.	*/
/*				   3.Next return instruction address is call instruction address + 2. */
/****************************************************************************/
static int
c33_IcdTraceAnalyzeRet(void)
{
	unsigned long  ulPC = C33_ICD_TRACE_PC_INVALID; 	   /* execution PC */	
	unsigned long  ulDelayedPC = C33_ICD_TRACE_PC_INVALID; /* delayed instruction PC */
	unsigned char  ucDelayed = C33_FALSE;				   /* delayed instruction flag */
	unsigned char  ucDpcoCounter = 0;					   /* DPCO counter */
	unsigned char  ucDpcoDumy;							   /* DPCO dumy */
	unsigned long  ulCycleCounter;						   /* cycle counter */
	unsigned long  ulUpstreamCounter = 0;				   /* cycle counter of upstream analyze */
	unsigned long  ulCallNum = 0;						   /* call instruction counter */
	unsigned char  ucAnalyzeControl = 0;				   /* analyze process control flag */
														   /* 0: stage is search ret instruction */
														   /* 1: stage is search delayed instruction of ret */
														   /* 2: stage is search next instruction of ret or ret.d */
														   /* 3:4: space */
														   /* 5: stage is search call instruction */
														   /* 6: stage is set address of PC */


	g_ucPC_Rel_Delayed = C33_FALSE;		/* add T.Tazaki 2005/03/01 */
	g_ucPC_Abs_Delayed = C33_FALSE;		/* add T.Tazaki 2005/03/01 */

	/* analize by ret instruction */
	for(ulCycleCounter = c33IcdTraceData.ulValidStart;ulCycleCounter < c33IcdTraceData.ulValidEnd;ulCycleCounter++)
	{
		/* check continuance data */
		if( (c33IcdTraceData.ucDst[ulCycleCounter] & C33_ICD_TRACE_TRG1_HIT) != 0 )
		{
			ucDpcoCounter = 0;
			ucDelayed = C33_FALSE;
			ucAnalyzeControl = 0;
		}

		/* delayed instruction case */
		if(ucDelayed == C33_TRUE)
		{
			/* jp.d or ret.d case only */
			ucDpcoDumy = c33IcdTraceData.ucDpcoAddrFix;
			c33_IcdTraceAnalyzeMakePcByDST(C33_ICD_TRACE_FIX_RET,ulCycleCounter,&ucDelayed,&ucDpcoDumy,&ulPC,&ulDelayedPC);
		}

		/* analyze process is search delayed instruction */
		if(ucAnalyzeControl == 1)
		{
			if( (c33IcdTraceData.ucDst[ulCycleCounter] & C33_ICD_TRACE_DST0_1) != C33_ICD_TRACE_DST_IDLE )
			{
				ucAnalyzeControl = 2;
			}
		}
		/* analyze process is search next instruction */
		else if(ucAnalyzeControl == 2)
		{
			if( (c33IcdTraceData.ucDst[ulCycleCounter] & C33_ICD_TRACE_DST0_1) != C33_ICD_TRACE_DST_IDLE )
			{
				ucAnalyzeControl = 5;
			}
		}

		/* analyze process is search call instruction */
		if(ucAnalyzeControl == 5)
		{
			if( (c33IcdTraceData.ucMethod[ulCycleCounter] & C33_ICD_TRACE_FIX_PC_BIT) == C33_ICD_TRACE_FIX_PC_BIT )
			{	/* address fix */
				/* analyze finished already */
				ucAnalyzeControl = 0;
			}
			else
			{
				/* search call instruction */
				ucAnalyzeControl = 6;
				ulCallNum = 1;
				while(ulUpstreamCounter < 0xffffffff)
				{
					/* check continuance data */
					if( (c33IcdTraceData.ucDst[ulCycleCounter] & C33_ICD_TRACE_TRG1_HIT) != 0 )
					{
						break;
					}

					if( (c33IcdTraceData.ucMethod[ulUpstreamCounter] & C33_ICD_TRACE_FIX_PC_BIT) == C33_ICD_TRACE_FIX_PC_BIT )
					{/* address fix */
						if( (c33IcdTraceData.usCode[ulUpstreamCounter] & 0xfe00) == 0x1c00 || (c33IcdTraceData.usCode[ulUpstreamCounter] & 0xfef0) == 0x0600 )
						{
							/* call sign , call %rd */
							ulCallNum--;
							if(ulCallNum == 0)
							{
								/* find call */
								if( (c33IcdTraceData.usCode[ulUpstreamCounter] & C33_DELAYED_BIT) == C33_DELAYED_BIT)
								{
									ulPC = c33IcdTraceData.ulPC[ulUpstreamCounter] + 4;
								}
								else
								{
									ulPC = c33IcdTraceData.ulPC[ulUpstreamCounter] + 2;
								}
								/* compare this PC with dpco */
								if(c33_IcdTrcComparePC(ulPC,ulCycleCounter) != 0)
								{
									/* This program can not analyze by ret instruction */
									ulCallNum = 1;
								}
								break;
							}
						}
						else if( (c33IcdTraceData.usCode[ulUpstreamCounter] & 0xfeff) == 0x0640 )
						{
							/* ret */
							ulCallNum++;
						}
					}
					else if( (c33IcdTraceData.ucDst[ulUpstreamCounter] & C33_ICD_TRACE_DST0_1) == C33_ICD_TRACE_DST_IDLE)
					{
					}
					else
					{
						break;
					}
					ulUpstreamCounter--;
				}
				if(ulCallNum == 0)
				{
					ucDpcoCounter = c33IcdTraceData.ucDpcoAddrFix;
				}
				else
				{
					ucAnalyzeControl = 0;
				}
			}
		}

		/* analyze process is set PC */
		if(ucAnalyzeControl == 6 && ucDpcoCounter == c33IcdTraceData.ucDpcoAddrFix)
		{
			/* analyze by dst */
			c33_IcdTraceAnalyzeMakePcByDST(C33_ICD_TRACE_FIX_RET,ulCycleCounter,&ucDelayed,&ucDpcoCounter,&ulPC,&ulDelayedPC);
			
			/* execute jump instruction */
			if(ucDpcoCounter != c33IcdTraceData.ucDpcoAddrFix)
			{
				ucAnalyzeControl = 0;
			}
		}

		/* analyze process is search ret or retd or reti instruction */
		if(ucAnalyzeControl == 0)
		{
			if( (c33IcdTraceData.ucMethod[ulCycleCounter] & C33_ICD_TRACE_FIX_PC_BIT) == C33_ICD_TRACE_FIX_PC_BIT )
			{/* address fix */
				if( (c33IcdTraceData.ucDst[ulCycleCounter] & C33_ICD_TRACE_DST0_1) == C33_ICD_TRACE_DST_ABS_JP )
				{/* absolute jump */
					/* check ret or ret.d or reti */	
					if( c33IcdTraceData.usCode[ulCycleCounter] == C33_RET_INSTRUCTION )
					{
						ucAnalyzeControl = 2;
					}
					else if(c33IcdTraceData.usCode[ulCycleCounter] == C33_RET_DELAY_INSTRUCTION)
					{
						ucAnalyzeControl = 1;
					}
					ulUpstreamCounter = ulCycleCounter - 1;/* search call instruction befor ulUpstreamCounter */
				}
			}
		}
	}

	return C33_NO_ERR;
}

/****************************************************************************/
/* Format		 : static int c33_IcdTraceAnalyzeReti(void) 				*/
/* Arguments	 : void 													*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : This function is analyze interrupt.						*/
/*				   1.Search interrupt piont and set interrupt flag. 		*/
/*				   2.Search reti instruction and anayze return address. 	*/
/****************************************************************************/
static int
c33_IcdTraceAnalyzeReti(void)
{
	unsigned long  ulPC = C33_ICD_TRACE_PC_INVALID; 	   /* execution PC */	
	unsigned long  ulDelayedPC = C33_ICD_TRACE_PC_INVALID; /* delayed instruction PC */
	unsigned char  ucDelayed = C33_FALSE;				   /* delayed instruction flag */
	unsigned long  ulCycleCounter;						   /* cycle counter */
	unsigned short usCode = 0;							   /* analyze code */
	unsigned char  ucDpcoCounter = 0;					   /* DPCO counter */
	unsigned long  ulUpstreamCounter;					   /* cycle counter of upstream analyze */
	unsigned char  ucKindCode;							   /* kind of code */	
	unsigned long  ulRetiNum = 0;						   /* interrupt counter */

	g_ucPC_Rel_Delayed = C33_FALSE;		/* add T.Tazaki 2005/03/01 */
	g_ucPC_Abs_Delayed = C33_FALSE;		/* add T.Tazaki 2005/03/01 */

	/* analyze interrupt point */
	for(ulCycleCounter = c33IcdTraceData.ulValidStart;ulCycleCounter < c33IcdTraceData.ulValidEnd;ulCycleCounter++)
	{
		/* check continuance data */
		if( (c33IcdTraceData.ucDst[ulCycleCounter] & C33_ICD_TRACE_TRG1_HIT) != 0 )
		{
			ucDpcoCounter = 0;
			ucDelayed = C33_FALSE;
		}
		/* search absolute jump point */	
		if( (c33IcdTraceData.ucDst[ulCycleCounter] & C33_ICD_TRACE_DST0_1) == C33_ICD_TRACE_DST_ABS_JP && (c33IcdTraceData.ucMethod[ulCycleCounter] & C33_ICD_TRACE_FIX_PC_BIT) == C33_ICD_TRACE_FIX_PC_BIT)
		{
			/* it is not jump inst, but when DST=ABS_JP,It is regarded as interruption. */
			usCode = c33IcdTraceData.usCode[ulCycleCounter];
			usCode &= 0xfeff;/* clear delayed bit */
			ucKindCode = C33_ICD_TRACE_DST_NORM;
			/* instrction or interrupt check */
			if( (usCode >> 4) == 0x060)
			{/* call %rd */
				ucKindCode = C33_ICD_TRACE_DST_ABS_JP;
			}
			else if( (usCode >> 4) == 0x068)
			{/* jp %rd */
				ucKindCode = C33_ICD_TRACE_DST_ABS_JP;
			}
			else if( (usCode >> 0) == 0x0640)
			{/* ret */
				ucKindCode = C33_ICD_TRACE_DST_ABS_JP;
			}
			else if( (usCode >> 0) == 0x04c0)
			{/* reti */
				ucKindCode = C33_ICD_TRACE_DST_ABS_JP;
			}
			else if( (usCode >> 2) == 0x120)
			{/* int */
				ucKindCode = C33_ICD_TRACE_DST_ABS_JP;
			}
			/* set interrupt flag */
			if(ucKindCode == C33_ICD_TRACE_DST_NORM)
			{
				if (c33TargetInfo.ucAdvanced == C33_PRODUCT_STD)	/* add T.Tazaki 2004/03/23 Except ADV:loop inst */
				{
					c33IcdTraceData.ucMethod[ulCycleCounter] |= C33_ICD_TRACE_INTERRUPT;
				}
			}
		}

		/* analize by dst */
		if(ucDpcoCounter == c33IcdTraceData.ucDpcoAddrFix && (c33IcdTraceData.ucDst[ulCycleCounter] & C33_ICD_TRACE_DST0_1) != C33_ICD_TRACE_DST_IDLE)
		{
			if( (c33IcdTraceData.ucMethod[ulCycleCounter] & C33_ICD_TRACE_FIX_PC_BIT) == C33_ICD_TRACE_FIX_PC_BIT )
			{
				ucDpcoCounter = 0;/* finished already */
			}
			else
			{
				c33_IcdTraceAnalyzeMakePcByDST(C33_ICD_TRACE_FIX_RETI,ulCycleCounter,&ucDelayed,&ucDpcoCounter,&ulPC,&ulDelayedPC);
			}
		}

		/* search reti instruction */
		if( (c33IcdTraceData.ucDst[ulCycleCounter] & C33_ICD_TRACE_DST0_1) == C33_ICD_TRACE_DST_ABS_JP && (c33IcdTraceData.ucMethod[ulCycleCounter] & C33_ICD_TRACE_FIX_PC_BIT) == C33_ICD_TRACE_FIX_PC_BIT)
		{
			if( (usCode >> 0) == 0x04c0)
			{/* reti */
				ulUpstreamCounter = ulCycleCounter -1;
				ulRetiNum = 1;
				while(ulUpstreamCounter < 0xffffffff)
				{
					if( (c33IcdTraceData.ucMethod[ulUpstreamCounter] & C33_ICD_TRACE_FIX_PC_BIT) == C33_ICD_TRACE_FIX_PC_BIT )
					{/* address fix */
						/* check continuance data */
						if( (c33IcdTraceData.ucDst[ulCycleCounter] & C33_ICD_TRACE_TRG1_HIT) != 0 )
						{
							break;
						}
						if( (c33IcdTraceData.ucMethod[ulUpstreamCounter] & C33_ICD_TRACE_INTERRUPT) == C33_ICD_TRACE_INTERRUPT || (c33IcdTraceData.usCode[ulUpstreamCounter] >> 2) == 0x120 )
						{
							/* interrupt */
							ulRetiNum--;
							if(ulRetiNum == 0)
							{
								/* find interrupt */
								if(c33IcdTraceData.ucDpcoNum[ulUpstreamCounter] != c33IcdTraceData.ucDpcoAddrFix)
								{
									ulRetiNum = 1;/* pc is unknown */
								}
								ulPC = c33IcdTraceData.ulPC[ulUpstreamCounter];
								/* int instruction case,return address is next instruction */
								if( (c33IcdTraceData.usCode[ulUpstreamCounter] >> 2) == 0x120 )
								{
									ulPC +=2;
								}
								/* compare this PC with dpco */
								if(c33_IcdTrcComparePC(ulPC,ulCycleCounter + 1) != 0)
								{
									/* This program can not analyze by reti instruction */
									ulRetiNum = 1;
								}
								break;
							}
						}
						else if( c33IcdTraceData.usCode[ulUpstreamCounter] == 0x04c0 )
						{
							/* reti */
							ulRetiNum++;
						}
					}
					else if( (c33IcdTraceData.ucDst[ulUpstreamCounter] & C33_ICD_TRACE_DST0_1) == C33_ICD_TRACE_DST_IDLE)
					{
					}
					else
					{
						break;
					}
					ulUpstreamCounter--;
				}
				if(ulRetiNum == 0)
				{
					ucDpcoCounter = c33IcdTraceData.ucDpcoAddrFix;
				}
				else
				{
					ucDpcoCounter = 0;
				}
			}
		}
	}
	return C33_NO_ERR;
}

/****************************************************************************/
/* Format		 : static int c33_IcdTraceAnalyzeMap(void)					*/
/* Arguments	 : void 													*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : This function analyze by map information.				*/
/*				   1.Make expect PC from map information.					*/
/*				   2.Compare DST signal with code of expect PC. 			*/
/*				   3.Match PC is only one if this PC is answer. 			*/
/****************************************************************************/
static int
c33_IcdTraceAnalyzeMap(void)
{
	unsigned long  ulPC = C33_ICD_TRACE_PC_INVALID; 		   /* execution PC */	
	unsigned long  ulDelayedPC = C33_ICD_TRACE_PC_INVALID;	   /* delayed instruction PC */
	unsigned char  ucDelayed = C33_FALSE;					   /* delayed instruction flag */
	unsigned char  ucDpcoCounter = 0;						   /* DPCO counter */
	unsigned char  ucDpcoDumy;								   /* dumy */
	unsigned long  ulCycleCounter;							   /* cycle counter */
	unsigned long  ulUpstreamCounter = 0;					   /* cycle counter of upstream analyze */
	unsigned long  ulWorkPC;								   /* work PC */
	unsigned long  ulExpectNum; 							   /* expectation number */
	struct c33_map_info *pstMapInfo;						   /* map information pointer */

	g_ucPC_Rel_Delayed = C33_FALSE;		/* add T.Tazaki 2005/03/01 */
	g_ucPC_Abs_Delayed = C33_FALSE;		/* add T.Tazaki 2005/03/01 */

	/* analize by map information */
	for(ulCycleCounter = c33IcdTraceData.ulValidStart;ulCycleCounter < c33IcdTraceData.ulValidEnd;ulCycleCounter++)
	{

		/* delayed instruction case */
		if(ucDelayed == C33_TRUE)
		{
			ucDpcoDumy = c33IcdTraceData.ucDpcoAddrFix;
			c33_IcdTraceAnalyzeMakePcByDST(C33_ICD_TRACE_FIX_MAP,ulCycleCounter,&ucDelayed,&ucDpcoDumy,&ulPC,&ulDelayedPC);
		}

		/* search absolute jump */
		if( (((c33IcdTraceData.ucDst[ulCycleCounter] & C33_ICD_TRACE_DST0_1) == C33_ICD_TRACE_DST_ABS_JP) && ((c33IcdTraceData.ucMethod[ulCycleCounter] & C33_ICD_TRACE_FIX_PC_BIT) != C33_ICD_TRACE_FIX_PC_BIT)) ||	
			((ulCycleCounter == c33IcdTraceData.ulValidEnd - 1) && c33IcdTraceData.ucDpcoNum[ulCycleCounter] != c33IcdTraceData.ucDpcoAddrFix))
		{
			ucDpcoCounter = c33IcdTraceData.ucDpcoNum[ulCycleCounter];
			if(ucDpcoCounter > 9)
			{/* over 1byte data */
				/* make expect PC from map information */
				ulExpectNum = 0;
				pstMapInfo = (struct c33_map_info *)c33TargetInfo.pMapInfo;
				while (pstMapInfo)
				{
					if (pstMapInfo->iMapAttr == C33_ROM || pstMapInfo->iMapAttr == C33_RAM || pstMapInfo->iMapAttr == C33_IO)
					{
						/* compare DST with code */
						ulWorkPC = c33_IcdTrcCompCodeWithDST(ulCycleCounter,pstMapInfo->ulStartAddr,pstMapInfo->ulEndAddr,ucDpcoCounter,&ulExpectNum);
						if(ulWorkPC != C33_ICD_TRACE_PC_INVALID)
						{
							/* save match PC */
							ulPC = ulWorkPC;
						}
					}
					if(ulExpectNum >= 2)
					{
						break;
					}
					pstMapInfo = (struct c33_map_info *)pstMapInfo->pMapNext;
				}
				/* match pc is only one */
				if(ulExpectNum == 1)
				{
					if (c33TargetInfo.ucAdvanced == C33_PRODUCT_STD)	/* add T.Tazaki 2004/03/22	*/
					{
						ulUpstreamCounter = ulCycleCounter - ucDpcoCounter + 3;/* ulUpstreamCounter is DPCO output start point + 2 */
					}else{
						/* add T.Tazaki 2004/03/22	bug fix */
						ulUpstreamCounter = ulCycleCounter - ucDpcoCounter + 2;/* ulUpstreamCounter is DPCO output start point + 1 */
					}
																		   /*  DPCO output start point is idle or delayed instruction */
					/* set pc from dpco start to dpco end */
					while(ulCycleCounter >= ulUpstreamCounter)
					{
						/* analyze by dst */
						ucDpcoDumy = c33IcdTraceData.ucDpcoAddrFix;
						c33_IcdTraceAnalyzeMakePcByDST(C33_ICD_TRACE_FIX_MAP,ulUpstreamCounter,&ucDelayed,&ucDpcoDumy,&ulPC,&ulDelayedPC);
						ulUpstreamCounter++;
					}
				}
				/* match pc is nothing if interrupt point */
				/* else if(ulExpectNum == 0)
				{
					c33IcdTraceData.ucMethod[ulCycleCounter] |= (C33_ICD_TRACE_FIX_PC_BIT | C33_ICD_TRACE_FIX_MAP | C33_ICD_TRACE_INTERRUPT);
				}*/ /* 1998/12/15 C.Nakayama the probability of bug is high */	
			}
		}
	}
	return C33_NO_ERR;
}

/* add T.Tazaki 2004/11/04 >>> */

/****************************************************************************/
/* Format		 : static int c33_IcdTraceModify(void)						*/
/* Arguments	 : void 													*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : advanced macro call.d %rb + delayed insn output process. */
/****************************************************************************/
static int
c33_IcdTraceModify(void)
{
	unsigned long  ulCycleCounter;						   /* cycle counter */
	unsigned char ucDpcoNum1;
	unsigned long ulPC1;
	unsigned char ucMethod1;
	unsigned short usCode1;
	unsigned char ucDpcoNum2;
	unsigned long ulPC2;
	unsigned char ucMethod2;
	unsigned short usCode2;
	unsigned long ulCounter1;
	unsigned long ulCounter2;
	int iFlag;
	unsigned long  ulCallNum = 0;						   /* call instruction counter */
	
	/* call.d %rb + delayed inst modify */
	/* PC ABS( <31 bitj + PC ABS(call.d %rb) Ffor Cannot trace delayed inst of "call.d %rb" */
	/* 	 xld.w %r4,CALL1	*/
	/*		 call %r4			*/
	/*		 call.d %4			*/
	/*		 ld.w  %r3,0   <--- trace  */
	/*		 nop				*/
	/*							*/
	/*	CALL1: nop				*/
	/*		   nop				*/
	/*		   ret	   :PC ABS	*/
	/*							*/

	
	if (c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV)
	{
		g_ucPC_Abs_Delayed = C33_FALSE;

		/* ALL cycle */
		for(ulCycleCounter = c33IcdTraceData.ulValidStart;ulCycleCounter < c33IcdTraceData.ulValidEnd;ulCycleCounter++)
		{
			if( (c33IcdTraceData.ucMethod[ulCycleCounter] & C33_ICD_TRACE_FIX_PC_BIT) == C33_ICD_TRACE_FIX_PC_BIT )
			{
				/* search call.d %rb */
				if( (c33IcdTraceData.ucMethod[ulCycleCounter] & C33_ICD_TRACE_FIX_RET)	== C33_ICD_TRACE_FIX_RET  ||	/* Analysis RET  ? */
					(c33IcdTraceData.ucMethod[ulCycleCounter] & C33_ICD_TRACE_FIX_RETI) == C33_ICD_TRACE_FIX_RETI ||	/* Analysis RETI ? */
					(c33IcdTraceData.ucMethod[ulCycleCounter] & C33_ICD_TRACE_FIX_MAP)	== C33_ICD_TRACE_FIX_MAP)		/* Analysis MAP  ? */
				{
					if( (c33IcdTraceData.usCode[ulCycleCounter] & 0xfff0) == 0x0700 )	/* find ? */
					{
						/* Insert delayed inst after "call.d" */
						
						/* Search NORMAL */
						for( ulCounter1 = ulCycleCounter + 1; ulCounter1 < c33IcdTraceData.ulValidEnd; ulCounter1++)
						{
							if( (c33IcdTraceData.ucDst[ulCounter1] & C33_ICD_TRACE_DST0_1) == C33_ICD_TRACE_DST_NORM )
							{
								break;
							}
						}
						if( ulCounter1 != c33IcdTraceData.ulValidEnd )	/* find ? */
						{
							/* save temporary data before setting */
							ucDpcoNum1 = c33IcdTraceData.ucDpcoNum[ulCounter1];
							ulPC1	   = c33IcdTraceData.ulPC[ulCounter1];
							ucMethod1  = c33IcdTraceData.ucMethod[ulCounter1];
							usCode1    = c33IcdTraceData.usCode[ulCounter1];
							iFlag = 1;
							
							/* Overwrite delayed inst */
							c33IcdTraceData.ucDpcoNum[ulCounter1] = c33IcdTraceData.ucDpcoAddrFix;
							c33IcdTraceData.ulPC[ulCounter1]	  = c33IcdTraceData.ulPC[ulCycleCounter]  + 2;
							c33IcdTraceData.ucMethod[ulCounter1]  = c33IcdTraceData.ucMethod[ulCycleCounter];
							/* save code */
							c33_ReadWriteMapMem(C33_READ, c33IcdTraceData.ulPC[ulCounter1], C33_HALF_WORD, C33_HALF_WORD, (unsigned char*)&c33IcdTraceData.usCode[ulCounter1]);


							ulCallNum = 0;
							
							/* Every one data which already FIXed is back moved to ret. */
							for( ulCounter2 = ulCounter1 + 1; ulCounter2 < c33IcdTraceData.ulValidEnd; ++ulCounter2 )
							{
								if( (c33IcdTraceData.ucDst[ulCounter2] & C33_ICD_TRACE_DST0_1) != C33_ICD_TRACE_DST_IDLE )
								{
									if( iFlag == 1 )
									{
										/* The data before a setup is saved temporarily. */
										ucDpcoNum2 = c33IcdTraceData.ucDpcoNum[ulCounter2];
										ulPC2	   = c33IcdTraceData.ulPC[ulCounter2];
										ucMethod2  = c33IcdTraceData.ucMethod[ulCounter2];
										usCode2    = c33IcdTraceData.usCode[ulCounter2];

										/* Overwrite temporary data */
										c33IcdTraceData.ucDpcoNum[ulCounter2] = ucDpcoNum1;
										c33IcdTraceData.ulPC[ulCounter2]	  = ulPC1;
										c33IcdTraceData.ucMethod[ulCounter2]  = ucMethod1;
										c33IcdTraceData.usCode[ulCounter2]	  = usCode1;
										iFlag = 2;

										/* call nesting ? */
										if ( (usCode1 & 0xfff0) == INSN_CALL_RB  || (usCode1 & 0xfff0) == INSN_CALL_RB_D ||
											 (usCode1 & 0xff00) == INSN_CALL_IMM || (usCode1 & 0xff00) == INSN_CALL_IMM_D )
										{
									 		ulCallNum++;
									 	}
									 	else if( usCode1 == C33_RET_INSTRUCTION )	/* ret ? */
										{
											if( ulCallNum == 0 )
											{
//												/* overwrite "ret". and search next "call.d" */
//												ulCycleCounter = ulCounter2;
												break;
											}
											else
											{
										 		ulCallNum--;
											}
										}
									}
									else
									{
										/* The data before a setup is saved temporarily. */
										ucDpcoNum1 = c33IcdTraceData.ucDpcoNum[ulCounter2];
										ulPC1	   = c33IcdTraceData.ulPC[ulCounter2];
										ucMethod1  = c33IcdTraceData.ucMethod[ulCounter2];
										usCode1    = c33IcdTraceData.usCode[ulCounter2];

										/* Overwrite temporary data */
										c33IcdTraceData.ucDpcoNum[ulCounter2] = ucDpcoNum2;
										c33IcdTraceData.ulPC[ulCounter2]	  = ulPC2;
										c33IcdTraceData.ucMethod[ulCounter2]  = ucMethod2;
										c33IcdTraceData.usCode[ulCounter2]	  = usCode2;
										iFlag = 1;

										/* call nesting ? */
										if ( (usCode2 & 0xfff0) == INSN_CALL_RB  || (usCode2 & 0xfff0) == INSN_CALL_RB_D ||
											 (usCode2 & 0xff00) == INSN_CALL_IMM || (usCode2 & 0xff00) == INSN_CALL_IMM_D )
										{
									 		ulCallNum++;
									 	}
									 	else if( usCode2 == C33_RET_INSTRUCTION )	/* ret ? */
										{
											if( ulCallNum == 0 )
											{
//												/* overwrite "ret". and search next "call.d" */
//												ulCycleCounter = ulCounter2;
												break;
											}
											else
											{
										 		ulCallNum--;
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}
	
	
	
	return C33_NO_ERR;

}
/* add T.Tazaki 2004/11/04 <<< */

/****************************************************************************/
/* Format		 : static int c33_IcdDebugDisp(void)						*/
/* Arguments	 : void 													*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : gdb debug routine. for PC trace. 						*/
/****************************************************************************/
static int
c33_IcdDebugDisp(void)
{
	unsigned long  ulCycleCounter;						   /* cycle counter */
	char szbuf[100];
	char szbuf2[300];

	
	if( iDebugTrace == 1 )	/* debug trace mode ?	 add T.Tazaki 2003/08/22 */
	{

		printf_filtered("\n");
		for(ulCycleCounter = c33IcdTraceData.ulValidStart; ulCycleCounter < c33IcdTraceData.ulValidEnd; ulCycleCounter++)
		{
			sprintf( szbuf,"%5d: DPCO[%d] DST[%d%d%d]  ",ulCycleCounter,	
															 (c33IcdTraceData.ucDst[ulCycleCounter] & 0x08) >> 3,
															 (c33IcdTraceData.ucDst[ulCycleCounter] & 0x04) >> 2,
															 (c33IcdTraceData.ucDst[ulCycleCounter] & 0x02) >> 1,
															 (c33IcdTraceData.ucDst[ulCycleCounter] & 0x01 ) );
			if( (c33IcdTraceData.ucDst[ulCycleCounter] & 0x07) == 0x07 )
			{
				strcat(szbuf,"BREAK AREA");
			}
			else
			{
				if ((c33IcdTraceData.ucDst[ulCycleCounter] & 0x03) == 0x0 )
				{
					strcat(szbuf,"NORMAL    ");
				}
				else if ((c33IcdTraceData.ucDst[ulCycleCounter] & 0x03) == 0x1 )
				{
					strcat(szbuf,"PC REL    ");
				}
				else if ((c33IcdTraceData.ucDst[ulCycleCounter] & 0x03) == 0x2 )
				{
					strcat(szbuf,"PC ABS    ");
				}
				else if ((c33IcdTraceData.ucDst[ulCycleCounter] & 0x03)== 0x3 )
				{
					strcat(szbuf,"IDLE      ");
				}

				if( (c33IcdTraceData.ucDst[ulCycleCounter] & 0x04) == 0x04 )
				{
					strcat(szbuf," +BREAK");
				}
			}
			sprintf( szbuf2,"%s METHOD[%02x] DPCONUM[%02x] PC[%08x] CODE[%04x]\n",
															 szbuf,
															  c33IcdTraceData.ucMethod[ulCycleCounter],
															  c33IcdTraceData.ucDpcoNum[ulCycleCounter],
															  c33IcdTraceData.ulPC[ulCycleCounter],
															  c33IcdTraceData.usCode[ulCycleCounter]
															  );
			printf_filtered(szbuf2);
		}
	}
	return C33_NO_ERR;

}

/****************************************************************************/
/* Format		 : static unsigned long c33_IcdTrcCompCodeWithDST(unsigned long  ulCycleCounter, unsigned long ulMapTopAddr, unsigned long ulMapBottomAddr, unsigned char ucDpcoCounter, unsigned long *ulExpectNum) */
/* Arguments																*/
/*	unsigned long  ulCycleCounter : Cycle counter							*/
/*	unsigned long ulMapTopAddr	  : Map top address 						*/
/*	unsigned long ulMapBottomAddr : Map bottom address						*/
/*	unsigned char ucDpcoCounter   : DPCO output counter 					*/
/*	unsigned long *ulExpectNum	  : Expectation number						*/
/* Return value 															*/
/*	C33_ICD_TRACE_PC_INVALID : Error return 								*/
/*	Other					 : Success return								*/
/* Explanation	 : This function is compare code and DST signel.			*/
/****************************************************************************/
static unsigned long
c33_IcdTrcCompCodeWithDST(unsigned long  ulCycleCounter, unsigned long ulMapTopAddr, unsigned long ulMapBottomAddr, unsigned char ucDpcoCounter, unsigned long *ulExpectNum)

{
	unsigned short	usCode; 						   /* target code */
	unsigned char	ucKindCode; 					   /* kind of code */
	unsigned long  ulStart; 						   /* map loop work */
	unsigned long  ulEnd;							   /* map loop work */
	unsigned long  ulMapAddrCounter;				   /* map address loop counter */
	unsigned long  ulExpectPC;						   /* expectation PC */
	unsigned char	ucMatch;						   /* match flag */
	unsigned long  ulUpstreamCounter;				   /* cycle counter of upstream analyze */
	unsigned long  ulPC = C33_ICD_TRACE_PC_INVALID;    /* execution PC */	
	unsigned char	ucDelayed = C33_FALSE;			   /* delayed instruction flag */

	ulStart = ulMapTopAddr >> (ucDpcoCounter - 1);
	ulEnd = ulMapBottomAddr >> (ucDpcoCounter - 1);
	for(ulMapAddrCounter = ulStart;ulMapAddrCounter <= ulEnd;ulMapAddrCounter++)
	{
		ulExpectPC = (ulMapAddrCounter << (ucDpcoCounter - 1)) + c33IcdTraceData.ulPC[ulCycleCounter];
		if(ulExpectPC <= ulMapBottomAddr && ulExpectPC >= ulMapTopAddr)
		{
			/* compare DST signal with parogram code */
			if (c33TargetInfo.ucAdvanced == C33_PRODUCT_STD)	/* add T.Tazaki 2003/05/26 >>> */
			{
				ulUpstreamCounter = ulCycleCounter - ucDpcoCounter + 3;/* ulUpstreamCounter is DPCO output start point + 2 */
			}
			else
			{
				ulUpstreamCounter = ulCycleCounter - ucDpcoCounter + 2;/* ulUpstreamCounter is DPCO output start point + 1 */
			}
																   /*  DPCO output start point is idle or delayed instruction */
			/* set delayed instruction flag */
			if( (c33IcdTraceData.ucMethod[ulUpstreamCounter - 1] & C33_ICD_TRACE_FIX_PC_BIT) == C33_ICD_TRACE_FIX_PC_BIT)
			{
				if( (c33IcdTraceData.usCode[ulUpstreamCounter - 1] & C33_DELAYED_BIT) == C33_DELAYED_BIT)
				{
					ucDelayed = C33_TRUE;						/* delayed instruction flag */
				}
			}
			ucMatch = C33_TRUE;
			while( ulUpstreamCounter <=  ulCycleCounter )
			{
				c33_ReadWriteMapMem(C33_READ, ulExpectPC, C33_HALF_WORD, C33_HALF_WORD, (unsigned char*)&usCode);
				usCode &= 0xfeff;/* clear delayed bit */
				
				if( (usCode >> 4) == 0x060)
				{/* call %rd */
					ucKindCode = C33_ICD_TRACE_DST_ABS_JP;
				}
				else if( (usCode >> 4) == 0x068)
				{/* jp %rd */
					ucKindCode = C33_ICD_TRACE_DST_ABS_JP;
				}
				else if( (usCode >> 0) == 0x0640)
				{/* ret */
					ucKindCode = C33_ICD_TRACE_DST_ABS_JP;
				}
				else if( (usCode >> 0) == 0x04c0)
				{/* reti */
					ucKindCode = C33_ICD_TRACE_DST_ABS_JP;
				}
				else if( (usCode >> 2) == 0x120)
				{/* int */
					ucKindCode = C33_ICD_TRACE_DST_ABS_JP;
				}
				else if( (usCode >> 11) == 1)
				{/* jrgt,jrge,jrlt,jrle */
					ucKindCode = C33_ICD_TRACE_DST_JP;
				}
				else if( (usCode >> 11) == 2)
				{/* jrugt,jruge,jrult,jrule */
					ucKindCode = C33_ICD_TRACE_DST_JP;
				}
				else if( (usCode >> 10) == 6)
				{/* jreq,jrne */
					ucKindCode = C33_ICD_TRACE_DST_JP;
				}
				else if( (usCode >> 10) == 7)
				{/* call sign8,jp sign8 */
					ucKindCode = C33_ICD_TRACE_DST_JP_ALWAYS;
				}
				else{
					ucKindCode = C33_ICD_TRACE_DST_NORM;
				}

				/* idle case */
				/* delayed case */
				if( (c33IcdTraceData.ucDst[ulUpstreamCounter] & C33_ICD_TRACE_DST0_1) == C33_ICD_TRACE_DST_IDLE || ucDelayed == C33_TRUE)
				{
					if(ucDelayed == C33_TRUE)
					{
						if ( (c33IcdTraceData.ucDst[ulUpstreamCounter] & C33_ICD_TRACE_DST0_1) == C33_ICD_TRACE_DST_IDLE)
						{
							ucDelayed = C33_FALSE;
						}
						else
						{
							ucDelayed = C33_FALSE;
							ucMatch = C33_FALSE;
							break;
						}
					}
					/* idle only is null */
				}
				/* normal case */
				else if ( (c33IcdTraceData.ucDst[ulUpstreamCounter] & C33_ICD_TRACE_DST0_1) == C33_ICD_TRACE_DST_NORM)
				{
					if(ucKindCode == C33_ICD_TRACE_DST_ABS_JP || ucKindCode == C33_ICD_TRACE_DST_JP_ALWAYS)
					{
						ucMatch = C33_FALSE;
						break;
					}
					else
					{
						ulExpectPC += 2;
					}
				}
				/* absolute case */
				else if ( (c33IcdTraceData.ucDst[ulUpstreamCounter] & C33_ICD_TRACE_DST0_1) == C33_ICD_TRACE_DST_ABS_JP)
				{
					if (c33TargetInfo.ucAdvanced == C33_PRODUCT_STD)	/* add T.Tazaki 2004/03/23 Except ADV:loop inst */
					{
						if(ucKindCode != C33_ICD_TRACE_DST_ABS_JP)
						{
							ucMatch = C33_FALSE;
							break;
						}
					}
				}
				/* jump case */
				else
				{
					if(ucKindCode == C33_ICD_TRACE_DST_JP || ucKindCode == C33_ICD_TRACE_DST_JP_ALWAYS)
					{
						c33_ReadWriteMapMem(C33_READ, ulExpectPC, C33_HALF_WORD, C33_HALF_WORD, (unsigned char*)&usCode);
						if( (usCode & C33_DELAYED_BIT) == C33_DELAYED_BIT)
						{
							ucDelayed = C33_TRUE;
						}
						ulExpectPC = c33_CalcAbsAddr(ulExpectPC);
						if(ulExpectPC == C33_ICD_TRACE_PC_INVALID)
						{/* can not analyze program */
							ucMatch = C33_FALSE;
							break;
						}
					}
					else
					{
						ucMatch = C33_FALSE;
						break;
					}
				}
				ulUpstreamCounter++;
			}
			if(ucMatch == C33_TRUE)
			{
				(*ulExpectNum)++;
				ulPC = (ulMapAddrCounter << (ucDpcoCounter - 1)) + c33IcdTraceData.ulPC[ulCycleCounter];
			}
		}
	}
	return (ulPC);
}


/****************************************************************************/
/* Format		 : static void c33_IcdTraceAnalyzeMakePcByDST(unsigned char ucMethod, unsigned long ulCycleCounter, unsigned char* ucDelayed, unsigned char* ucDpcoNum, unsigned long* ulPC, unsigned long* ulDelayedPC) */
/* Arguments																*/
/*	unsigned char ucMethod		 : Analyze method							*/
/*	unsigned long ulCycleCounter : Cycle counter							*/
/*	unsigned char *ucDelayed	 : Delayed instruction flag 				*/
/*	unsigned char *ucDpcoNum	 : Output DPCO number						*/
/*	unsigned long *ulPC 		 : Make pc from DPCO						*/
/*	unsigned long *ulDelayedPC	 : Delayed instruntion PC address			*/
/* Return value  : void 													*/
/* Explanation	 : This function analyze DST and renewal icd trace structure.*/
/****************************************************************************/

static void
c33_IcdTraceAnalyzeMakePcByDST(
	unsigned char ucMethod, 				   /* analyze method */
	unsigned long ulCycleCounter,			   /* cycle counter */
	unsigned char *ucDelayed,				   /* delayed instruction flag */
	unsigned char *ucDpcoNum,				   /* output DPCO number */
	unsigned long *ulPC,					   /* make pc from DPCO */
	unsigned long *ulDelayedPC) 			   /* delayed instruntion PC address */
{
	unsigned short usCodeWork = 0;
	unsigned short usAsmCode = 0;			   /* ADV : ABS_JP delayed code check */
	
	c33_IcdProcessing(3000);

	if( (c33IcdTraceData.ucMethod[ulCycleCounter] & C33_ICD_TRACE_FIX_PC_BIT) == C33_ICD_TRACE_FIX_PC_BIT)
	{
		return;/* analyze finished already */
	}


	/* save dpco */
	c33IcdTraceData.ucDpcoNum[ulCycleCounter] = *ucDpcoNum;
	/* save PC */
	c33IcdTraceData.ulPC[ulCycleCounter] = *ulPC;


	/* check cpu bug */
	if( (c33IcdTraceData.ucDst[ulCycleCounter] & C33_ICD_TRACE_DST0_1) != C33_ICD_TRACE_DST_IDLE && *ucDelayed == C33_TRUE)
	{
		/* ASSERT(0); */
	}
	
	/*==============================*/
	/* Check IDLE */
	/*==============================*/
	if( (c33IcdTraceData.ucDst[ulCycleCounter] & C33_ICD_TRACE_DST0_1) == C33_ICD_TRACE_DST_IDLE )
	{
		if (c33TargetInfo.ucAdvanced == C33_PRODUCT_STD) /* STD don't have delayed cycle add T.Tazaki 2003/09/03 */
		{
			if(*ucDelayed == C33_TRUE)
			{
				c33IcdTraceData.ucDst[ulCycleCounter] = (c33IcdTraceData.ucDst[ulCycleCounter] & ~C33_ICD_TRACE_DST0_1) | C33_ICD_TRACE_DST_NORM;
				c33IcdTraceData.ulDataMaxNumber++;
				c33IcdTraceData.ucDpcoNum[ulCycleCounter] = c33IcdTraceData.ucDpcoAddrFix;
				c33IcdTraceData.ulPC[ulCycleCounter] = *ulDelayedPC;
				c33IcdTraceData.ucMethod[ulCycleCounter] |= (C33_ICD_TRACE_FIX_PC_BIT | ucMethod);
				/* save code */
				c33_ReadWriteMapMem(C33_READ, *ulDelayedPC, C33_HALF_WORD, C33_HALF_WORD, (unsigned char*)&c33IcdTraceData.usCode[ulCycleCounter]);/* if pc==0xffffffff then code=0xffff */
				*ucDelayed = C33_FALSE;
			}
		}
	}
	else
	{
		if(*ulPC != C33_ICD_TRACE_PC_INVALID)
		{/* no map or can not analyze program check */
			if (*ucDelayed == C33_TRUE)
			{/* can not analyze program check */
				*ucDelayed = C33_FALSE;
			}
			if(*ucDpcoNum != c33IcdTraceData.ucDpcoAddrFix)
			{
				/* pc is not fix */
			}
			else
			{
				/* save code */
				c33_ReadWriteMapMem(C33_READ, *ulPC, C33_HALF_WORD, C33_HALF_WORD, (unsigned char*)&c33IcdTraceData.usCode[ulCycleCounter]);

				c33IcdTraceData.ucMethod[ulCycleCounter] |= (C33_ICD_TRACE_FIX_PC_BIT | ucMethod);


				/* add T.Tazaki 2003/06/16 >>>> */
				/* ALL DST */
				if (c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV || c33TargetInfo.ucAdvanced == C33_PRODUCT_PE ) /* add PE T.Tazaki 2003/11/21 */
				{
					/* Advanced macro : ext instruction address skip */
					/*	ext imm13	   : skip	*/
					/*	ext imm13	   : skip	*/
					/*	ld.w %rs,imm5  : set PC */
					do
					{
						c33_ReadWriteMapMem(C33_READ, *ulPC, C33_HALF_WORD, C33_HALF_WORD, (unsigned char*)&usCodeWork);

						if( ( ( usCodeWork & 0xe000 ) == 0xc000 ) ||  /* ext imm13					  */
							( ( usCodeWork & 0xff00 ) == 0x3b00 ) ||  /* ext op,imm2  ext cont		  */
							( ( usCodeWork & 0xff00 ) == 0x3f00 ) )   /* ext %rs	  ext %rs,op,imm2 */
						{
								*ulPC += 2;
						}
						else
						{
							c33IcdTraceData.usCode[ulCycleCounter] = usCodeWork;
							c33IcdTraceData.ulPC[ulCycleCounter] = *ulPC;	/* set target insn PC address */
							break;
						}
					}while( 1 );
				}
				/* add T.Tazaki 2003/06/16 <<< */
				/*==============================*/
				/* Check NORMAL 				*/
				/*==============================*/
				/* normal instruction case */
				if( (c33IcdTraceData.ucDst[ulCycleCounter] & C33_ICD_TRACE_DST0_1) == C33_ICD_TRACE_DST_NORM )
				{

					if (c33TargetInfo.ucAdvanced == C33_PRODUCT_STD)	
					{
						*ulPC += 2;
					}
					else
					{
						if( g_ucPC_Rel_Delayed == C33_TRUE )
						{
							*ulPC = g_ulSavePC; 	/* set new jump PC */
							g_ucPC_Rel_Delayed = C33_FALSE;
						}
						else if( g_ucPC_Abs_Delayed == C33_TRUE )
						{
							c33IcdTraceData.ulPC[ulCycleCounter] = g_ulAbsSavePC;	/* set ABS delayed inst address */
							c33_ReadWriteMapMem(C33_READ, g_ulAbsSavePC, C33_HALF_WORD, C33_HALF_WORD, (unsigned char*)&c33IcdTraceData.usCode[ulCycleCounter]);
							g_ucPC_Abs_Delayed = C33_FALSE;
						}
						else
						{
							*ulPC += 2;
						}
					}
				}
				/*==============================*/
				/* Check PC ABS 				*/
				/*==============================*/
				/* absolute jump instruction case */
				else if( (c33IcdTraceData.ucDst[ulCycleCounter] & C33_ICD_TRACE_DST0_1) == C33_ICD_TRACE_DST_ABS_JP )
				{
					if (c33TargetInfo.ucAdvanced == C33_PRODUCT_STD)	
					{
						if( (c33IcdTraceData.usCode[ulCycleCounter] & C33_DELAYED_BIT) == C33_DELAYED_BIT)
						{
							*ulDelayedPC = *ulPC + 2;
							*ucDelayed = C33_TRUE;
						}
					}
					else
					{
						/* Advanced have delayed instrunction cycle add T.Tazaki 2004/03/17 */
						/*	(sample)				   DST		   PC	 */
						/*	Branch	   call.d %r0	   PC ABS  : 0x10000 */
						/*			 				   IDLE    : 0x10200 */
						/*			 				   IDLE    : 0x10200 */
						/*			 				   IDLE    : 0x10200 */
						/*			   add %r1,1	   NORMAL  : 0x10200 <-- 0x10002 is correct. overwrite g_ulAbsSavePC. */
						/*			 				   IDLE    : 0x10200 */
						/*			 				   IDLE    : 0x10200 */
						/*			 				   IDLE    : 0x10200 */
						
						/* bug fix T.Tazaki 2004/08/20 >>> */
//del					if( (c33IcdTraceData.usCode[ulCycleCounter] & C33_DELAYED_BIT) == C33_DELAYED_BIT)
						usAsmCode = c33IcdTraceData.usCode[ulCycleCounter];
						if( ( (usAsmCode & 0xfff0) == 0x03c0 ) || 	/* jpr.d  %rb	 */
							( (usAsmCode & 0xfff0) == 0x0700 ) || 	/* call.d %rb	 */
							( usAsmCode == 0x0740 )			   || 	/* ret.d  %rb	 */
							( (usAsmCode & 0xfff0) == 0x0780 ) ) 	/* jp.d   %rb	 */
						/* bug fix T.Tazaki 2004/08/20 <<< */						
						{
							g_ulAbsSavePC = *ulPC + 2;			/* save delayed address */
							g_ucPC_Abs_Delayed = C33_TRUE;
						}
					}
					
					*ulPC = 0;
					if (c33TargetInfo.ucAdvanced == C33_PRODUCT_STD)	
					{
						*ucDpcoNum = C33_DPCO_TIMING;
					}
					else
					{
						*ucDpcoNum = 1;
					}
				}
				/*==============================*/
				/* Check PC REL 				*/
				/*==============================*/
				/* not absolute jump instruction case */
				else
				{
					if (c33TargetInfo.ucAdvanced == C33_PRODUCT_STD)	
					{
						/* PC go 2 if this instruction is delayed */
						if( (c33IcdTraceData.usCode[ulCycleCounter] & C33_DELAYED_BIT) == C33_DELAYED_BIT)
						{
							/* save jump address */
							*ulDelayedPC = *ulPC + 2;
							*ucDelayed = C33_TRUE;
						}
						*ulPC = c33_CalcAbsAddr(*ulPC);
					}
					else
					{
						/* Advanced have delayed instrunction cycle add T.Tazaki 2003/09/03 */
						/*							   DST	  */
						/*	Branch	   jreq.d XXX	   PC REL */
						/*			   add %r1,1	   NORMAL */
						/*	No Branch  jreq.d XXX	   NORMAL */
						/*			   add %r1,1	   NORMAL */
						/* bug fix T.Tazaki 2004/08/20 >>> */
//del					if( (c33IcdTraceData.usCode[ulCycleCounter] & C33_DELAYED_BIT) == C33_DELAYED_BIT)
						usAsmCode = c33IcdTraceData.usCode[ulCycleCounter];
						if(	( (usAsmCode & 0xff00) == 0x0900 ) || 	/* jrgt.d sign8  */
							( (usAsmCode & 0xff00) == 0x0b00 ) || 	/* jrge.d sign8  */
							( (usAsmCode & 0xff00) == 0x0d00 ) || 	/* jrlt.d sign8  */
							( (usAsmCode & 0xff00) == 0x0f00 ) || 	/* jrle.d sign8  */
							( (usAsmCode & 0xff00) == 0x1100 ) || 	/* jrugt.d sign8 */
							( (usAsmCode & 0xff00) == 0x1300 ) || 	/* jruge.d sign8 */
							( (usAsmCode & 0xff00) == 0x1500 ) || 	/* jrult.d sign8 */
							( (usAsmCode & 0xff00) == 0x1700 ) || 	/* jrule.d sign8 */
							( (usAsmCode & 0xff00) == 0x1900 ) || 	/* jreq.d sign8  */
							( (usAsmCode & 0xff00) == 0x1b00 ) || 	/* jrne.d sign8  */
							( (usAsmCode & 0xff00) == 0x1d00 ) || 	/* call.d sign8  */
							( (usAsmCode & 0xff00) == 0x1f00 ) ) 	/* jp.d sign8	 */
						/* bug fix T.Tazaki 2004/08/20 <<< */						
						{
							g_ulSavePC = c33_CalcAbsAddr(*ulPC);		/* get absolote jump address */
							g_ucPC_Rel_Delayed = C33_TRUE;
							*ulPC += 2;
						}
						else
						{
							*ulPC = c33_CalcAbsAddr(*ulPC);
						}
					}
				}
			}
		}
		else
		{
			*ucDpcoNum = 0;
			*ucDelayed = C33_FALSE;
		}
	}
}

/****************************************************************************/
/* Format		 : static unsigned long c33_IcdTrcComparePC(unsigned long ulPC, unsigned long ulCounter) */
/* Arguments																*/
/*	unsigned long ulPC		: Target PC 									*/
/*	unsigned long ulCounter : DPCO data counter 							*/
/* Return value  : 0 : Match	Other : Not match							*/
/* Explanation	 : This function is compare target PC with DPCO PC. 		*/
/****************************************************************************/
static unsigned long
c33_IcdTrcComparePC(unsigned long ulPC, unsigned long ulCounter)
{
	unsigned long ulMask = 0xffffffff;	   /* mask data */

	/* get DPCO PC */
	if(ulCounter >= c33IcdTraceData.ulValidEnd)
	{
		return(1);/* not match */
	}

	while( (c33IcdTraceData.ucDst[ulCounter] & C33_ICD_TRACE_DST0_1) != C33_ICD_TRACE_DST_ABS_JP)
	{
		/* check continuance data */
		if( (c33IcdTraceData.ucDst[ulCounter] & C33_ICD_TRACE_TRG2_HIT) != 0 )
		{
			break;
		}

		if( (ulCounter + 1) < c33IcdTraceData.ulValidEnd)
		{
			ulCounter++;
		}
		else
		{
			break;
		}
	}

	/* make mask data */
	ulMask = ulMask << (c33IcdTraceData.ucDpcoNum[ulCounter] - 1);
	ulMask = ~ulMask;

	/* return differrence */
	return( (ulPC & ulMask) - (c33IcdTraceData.ulPC[ulCounter] & ulMask) );
}

/****************************************************************************/
/* Format		 : void c33_IcdVer3FlashErase(int iParamNum, int EraseObject) */
/* Arguments																*/
/*	int iParamNum	  : Parameter number									*/
/*	int EraseObject   : Flash Erase object	0=data/address info 			*/
/*											1=program/entry info			*/
/* Return value  : void 													*/
/* Explanation	 : Flash Writer : Erase. & "NO DATA" Info set.				*/
/****************************************************************************/
void
c33_IcdVer3FlashErase(int iParamNum, int iEraseObject)
{
	int iResult;

	/* Check number of parameter */
	if ( iParamNum != 2 )
	{
		c33_error (C33_ERR_COMMAND_PARAM);
		return;
	}

	/* Check address */
	if ( iEraseObject > 1 )
	{
		c33_error (C33_ERR_COMMAND_ADDR);
		return;
	}

	if( iEraseObject == C33_DATA_LOAD )
	{
		printf_filtered (C33_MSG_FLS3_ERASE_DATA);
	}else{
		printf_filtered (C33_MSG_FLS3_ERASE_PROGRAM);
	}
	
	iResult = c33_SendMessage(C33_ID_FRASH_ERASE, iEraseObject, 0, 0, 0, 0, 0, 0);
	if (iResult == C33_NO_ERR)
	{
		iResult = c33_RecvMessage(C33_ID_FRASH_ERASE, 0, 0, 0, 0, 0);
	}

	if (iResult == C33_NO_ERR)
	{
		if( iEraseObject == C33_DATA_LOAD )
		{
			/*------------------------------------------------------*/
			/* Falsh (NO DATA)Data information Set					*/
			/*------------------------------------------------------*/
			
			iResult = c33_SendMessage(C33_ID_FRASH_ADDR_INFO_SET, 0x0,	0x0, 0x0,
									  0x0, "NO DATA", 0x0, 0x0 );
			if (iResult == C33_NO_ERR)
			{
				iResult = c33_RecvMessage(C33_ID_FRASH_ADDR_INFO_SET, 0, 0, 0, 0, 0);
			}

			if (iResult != C33_NO_ERR)
			{
				puts_filtered (C33_MSG_TARGET_FAILURE);
				c33_ErrorMsg();
			}
		}
		else
		{
			/*------------------------------------------------------*/
			/* Falsh (NO DATA)Entry information Set 				*/
			/*------------------------------------------------------*/

			iResult = c33_SendMessage(C33_ID_FRASH_ENTRY_INFO_SET, 0x0,  0x0, 0x0,
									  0x0, "NO DATA", 0x0, 0 );
			if (iResult == C33_NO_ERR)
			{
				iResult = c33_RecvMessage(C33_ID_FRASH_ENTRY_INFO_SET, 0, 0, 0, 0, 0);
			}

			if (iResult != C33_NO_ERR)
			{
				puts_filtered (C33_MSG_TARGET_FAILURE);
				c33_ErrorMsg();
			}
		}
		puts_filtered (C33_MSG_TARGET_DONE);
	}
	else
	{
		puts_filtered (C33_MSG_TARGET_FAILURE);
		c33_ErrorMsg();
	}
}

/****************************************************************************/
/* Format		 : void c33_IcdVer3FlashProgLoad(int iParamNum, char* pFilename, unsigned long ulEraseEntryAddress, */
/*												 unsigned long ulWriteEntryAddress, char *pComment */
/* Arguments																*/
/*	int iParamNum					  : Parameter number					*/
/*	char* pFilename 				  : Load program file name				*/
/*	unsigned long ulEraseEntryAddress : CPU Erase entry address(CPU RAM)	*/
/*	unsigned long ulWriteEntryAddress : CPU Write entry address(CPU RAM)	*/
/*	char *pComment					  : Comment 							*/
/* Return value  : void 													*/
/* Explanation	 : Flash Writer : flash erase & Load program HOST to ICD.	*/
/****************************************************************************/
void
c33_IcdVer3FlashProgLoad(int iParamNum, char* pFilename, unsigned long ulEraseEntryAddress,
						 unsigned long ulWriteEntryAddress, char *pComment )
{
	int iResult;
	unsigned char ucLine[512];
	unsigned char ucRecordData[512];
	FILE *fpLoadFile = NULL;
	unsigned long ulLen = 0;
	char szBuf[300];
	int iFirstFileAddress = 1;
	unsigned long ulAddress;
	unsigned long ulFileStartAddress = 0;
	unsigned long ulTmpLoadAddress;
	unsigned long ulMaxAddress = 0;
	unsigned long ulMaxLen = 0;
	unsigned long ulSize = 0;
	int iCnt;
	int iCancel;

	/* Check number of parameter */
	if ( iParamNum != 4 && iParamNum != 5 )
	{
		c33_error (C33_ERR_COMMAND_PARAM);
		return;
	}

	if( iParamNum == 4 )
	{
		pComment = (char *)g_NoCommentData; /* dummy string set */
	}

	fpLoadFile = fopen(pFilename, "r");
	if (fpLoadFile == NULL)
	{
		/* file open error */
		sprintf( szBuf,C33_ERR_FILE_OPEN,pFilename );
		c33_error (szBuf);
		return;
	}
	fclose(fpLoadFile);

	/* Get file minimum addr, maximum addr, length & file format check */
	iResult = c33_GetFileStartAddress( pFilename, &ulFileStartAddress, &ulMaxAddress, &ulMaxLen );
	if (iResult != C33_NO_ERR)
	{
		return;
	}

	/* Check size limit */
	if( ( ulMaxAddress + ulMaxLen - ulFileStartAddress ) > 0x1fff )
	{
		sprintf( szBuf,C33_ERR_LOAD_ADDR_OVER,0x1fff );
		c33_error (szBuf);
		return;
	}

	/*------------------------------------------------------*/
	/* Flash Program Erase									*/
	/*------------------------------------------------------*/
	 printf_filtered (C33_MSG_FLS3_ERASE_DATA);

	iResult = c33_SendMessage(C33_ID_FRASH_ERASE, C33_PROGRAM_LOAD, 0, 0, 0, 0, 0, 0);
	if (iResult == C33_NO_ERR)
	{
		iResult = c33_RecvMessage(C33_ID_FRASH_ERASE, 0, 0, 0, 0, 0);
	}

	if (iResult != C33_NO_ERR)
	{
		puts_filtered (C33_MSG_TARGET_FAILURE);
		c33_ErrorMsg();
	}
	puts_filtered (C33_MSG_TARGET_DONE);

	/*------------------------------------------------------*/
	/* Falsh Entry information Set							*/
	/*------------------------------------------------------*/

	printf_filtered (C33_MSG_FLS3_PROG_LOAD);

	/* boundary matching */
	ulEraseEntryAddress &= 0xfffffffe;
	ulWriteEntryAddress &= 0xfffffffe;

	ulSize = ulMaxAddress + ulMaxLen - ulFileStartAddress;

	/* ICD Flash start address = 0x00000000 FIXED */
	iResult = c33_SendMessage(C33_ID_FRASH_ENTRY_INFO_SET, 0x0,  ulFileStartAddress, ulSize,
							  ulEraseEntryAddress, pComment, ulWriteEntryAddress, 0 );
	if (iResult == C33_NO_ERR)
	{
		iResult = c33_RecvMessage(C33_ID_FRASH_ENTRY_INFO_SET, 0, 0, 0, 0, 0);
	}

	if (iResult != C33_NO_ERR)
	{
		puts_filtered (C33_MSG_TARGET_FAILURE);
		c33_ErrorMsg();
	}


	/*------------------------------------------------------*/
	/* Motolora file read & load							*/
	/*------------------------------------------------------*/

	fpLoadFile = fopen(pFilename, "r");
	if (fpLoadFile == NULL)
	{
		/* file open error */
		puts_filtered (C33_MSG_TARGET_FAILURE);
		sprintf( szBuf,C33_ERR_FILE_OPEN,pFilename );
		c33_error (szBuf);
		return;
	}

	iCnt = 0;

	c33_InitCancel();	/* Cancel initialize */
	iCancel = 0;

	while(fgets(ucLine,500,fpLoadFile) != 0)
	{
		if (SRecordToBin(ucLine, ucRecordData, &ulAddress, &ulLen) == C33_REC_DOWNLOAD )
		{

			ulTmpLoadAddress = ulAddress - ulFileStartAddress;

			/* program/data load command*/
			iResult = c33_SendMessage(C33_ID_FRASH_LOAD, C33_PROGRAM_LOAD, ulTmpLoadAddress, ulLen, 0, ucRecordData, 0, 0);
			if (iResult == C33_NO_ERR)
			{
				iResult = c33_RecvMessage(C33_ID_FRASH_LOAD, 0, 0, 0, 0, 0);
			}

			if (iResult != C33_NO_ERR)
			{
				fclose( fpLoadFile );
				c33_EndCancel();	/* Cancel Check End */
				puts_filtered (C33_MSG_TARGET_FAILURE);
				c33_ErrorMsg();
			}
			
			/* Check cancel */
			if (c33TargetInfo.iKeyBreak == 1 || c33_CheckCancel( 50 ) == 1 )
			{
				/* Cancel */
				iCancel = 1;
				printf_filtered( C33_MSG_TARGET_CANCEL);
				break;
			}

			++iCnt;
			if( iCnt == 5000 )
			{
				puts_filtered( "." );
				iCnt = 0;
			}
		}
	}

	fclose(fpLoadFile);

	c33_EndCancel();	/* Cancel Check End */
	
	if (iCancel == 0 )	/* Not cancel ? */
	{
		puts_filtered (C33_MSG_TARGET_DONE);
	}

}

/****************************************************************************/
/* Format		 : void c33_IcdVer3FlashDataLoad(int iParamNum, char* pFilename, unsigned long ulEraseStartBlock,  */
/*												 unsigned long ulEraseEndBlock, unsigned long ulEraseParam, char *pComment) */
/* Arguments																*/
/*	int iParamNum				   : Parameter number						*/
/*	char* pFilename 			   : Load program file name 				*/
/*	unsigned long ulEraseStartBlock: Erase start block						*/
/*	unsigned long ulEraseEndBlock  : Erase end block						*/
/*	unsigned long ulEraseParam	   : Erase parameter						*/
/*	char *pComment				   : Comment								*/
/* Return value  : void 													*/
/* Explanation	 : Flash Writer : Load data from HOST to ICD.				*/
/****************************************************************************/
void
c33_IcdVer3FlashDataLoad(int iParamNum, char* pFilename, unsigned long ulEraseStartBlock,
						 unsigned long ulEraseEndBlock, unsigned long ulEraseParam, char *pComment )
{
	int iResult;
	unsigned char ucLine[512];
	unsigned char ucRecordData[512];
	FILE *fpLoadFile = NULL;
	unsigned long ulLen = 0;
	char szBuf[300];
	int iFirstFileAddress = 1;
	unsigned long ulAddress;
	unsigned long ulFileStartAddress = 0;
	unsigned long ulTmpLoadAddress;
	unsigned long ulMaxAddress = 0;
	unsigned long ulMaxLen = 0;
	unsigned long ulSize = 0;
	int iCnt;
	int iCancel;

	/* Check number of parameter */
	if ( iParamNum != 5 && iParamNum != 6 )
	{
		c33_error (C33_ERR_COMMAND_PARAM);
		return;
	}

	if( iParamNum == 5 )
	{
		pComment = (char *)g_NoCommentData; /* dummy string set */
	}
	

	fpLoadFile = fopen(pFilename, "r");
	if (fpLoadFile == NULL)
	{
		/* file open error */
		sprintf( szBuf,C33_ERR_FILE_OPEN,pFilename );
		c33_error (szBuf);
		return;
	}
	fclose(fpLoadFile);


	/* Get file minimum addr, maximum addr, length & file format check */
	iResult = c33_GetFileStartAddress( pFilename, &ulFileStartAddress, &ulMaxAddress, &ulMaxLen );
	if (iResult != C33_NO_ERR)
	{
		return;
	}

	if( g_iICD_kind == C33_ICDV6 )	/* ICDV6 ΉFICDV6 max 4MB Flash ROM  T.Tazaki 2007/07/25 */
	{
		/* Check size limit */
		if( ( ulMaxAddress + ulMaxLen - ulFileStartAddress ) > 0x3fffff )
		{
			sprintf( szBuf,C33_ERR_LOAD_ADDR_OVER, 0x3fffff );
			c33_error (szBuf);
			return;
		}
	}
	else
	{
		/* Check size limit */
		if( ( ulMaxAddress + ulMaxLen - ulFileStartAddress ) > 0x7fffff )
		{
			sprintf( szBuf,C33_ERR_LOAD_ADDR_OVER, 0x7fffff );
			c33_error (szBuf);
			return;
		}
	}
	
	/*------------------------------------------------------*/
	/* Flash Data Erase 									*/
	/*------------------------------------------------------*/
	 printf_filtered (C33_MSG_FLS3_ERASE_DATA);

	iResult = c33_SendMessage(C33_ID_FRASH_ERASE, C33_DATA_LOAD, 0, 0, 0, 0, 0, 0);
	if (iResult == C33_NO_ERR)
	{
		iResult = c33_RecvMessage(C33_ID_FRASH_ERASE, 0, 0, 0, 0, 0);
	}

	if (iResult != C33_NO_ERR)
	{
		puts_filtered (C33_MSG_TARGET_FAILURE);
		c33_ErrorMsg();
	}
	puts_filtered (C33_MSG_TARGET_DONE);

	/*------------------------------------------------------*/
	/* Flash Data information Set							*/
	/*------------------------------------------------------*/

	printf_filtered (C33_MSG_FLS3_DATA_LOAD);

	ulSize = ulMaxAddress + ulMaxLen - ulFileStartAddress;

	iResult = c33_SendMessage(C33_ID_FRASH_ADDR_INFO_SET, 0x0,	ulFileStartAddress, ulSize,
							  ulEraseParam, pComment, ulEraseStartBlock, ulEraseEndBlock );
	if (iResult == C33_NO_ERR)
	{
		iResult = c33_RecvMessage(C33_ID_FRASH_ADDR_INFO_SET, 0, 0, 0, 0, 0);
	}

	if (iResult != C33_NO_ERR)
	{
		puts_filtered (C33_MSG_TARGET_FAILURE);
		c33_ErrorMsg();
	}
	

	/*------------------------------------------------------*/
	/* Motolora file read & load							*/
	/*------------------------------------------------------*/

	/* Motolora file read & load */
	fpLoadFile = fopen(pFilename, "r");
	if (fpLoadFile == NULL)
	{
		/* file open error */
		puts_filtered (C33_MSG_TARGET_FAILURE);
		sprintf( szBuf,C33_ERR_FILE_OPEN,pFilename );
		c33_error (szBuf);
		return;
	}

	iCnt = 0;

	c33_InitCancel();	/* Cancel initialize */
	iCancel = 0;

	while(fgets(ucLine,500,fpLoadFile) != 0)
	{
		if (SRecordToBin(ucLine, ucRecordData, &ulAddress, &ulLen) == C33_REC_DOWNLOAD )
		{
			ulTmpLoadAddress = ulAddress - ulFileStartAddress;

			/* program/data load command*/
			iResult = c33_SendMessage(C33_ID_FRASH_LOAD, C33_DATA_LOAD, ulTmpLoadAddress, ulLen, 0, ucRecordData, 0, 0);
			if (iResult == C33_NO_ERR)
			{
				iResult = c33_RecvMessage(C33_ID_FRASH_LOAD, 0, 0, 0, 0, 0);
			}
			if (iResult != C33_NO_ERR)
			{
				fclose( fpLoadFile );
				c33_EndCancel();	/* Cancel Check End */
				puts_filtered (C33_MSG_TARGET_FAILURE);
				c33_ErrorMsg();
			}
			/* Check cancel */
			if (c33TargetInfo.iKeyBreak == 1 || c33_CheckCancel( 1 ) == 1 )
			{
				/* Cancel */
				iCancel = 1;
				printf_filtered( C33_MSG_TARGET_CANCEL);
				break;
			}
			++iCnt;
			if( iCnt == 5000 )
			{
				puts_filtered( "." );
				iCnt = 0;
			}
		}
	}

	fclose(fpLoadFile);
	
	c33_EndCancel();	/* Cancel Check End */
	
	if (iCancel == 0 )	/* Not cancel ? */
	{
		puts_filtered (C33_MSG_TARGET_DONE);
	}

}

/****************************************************************************/
/* Format		 : static int c33_GetFileStartAddress( char *pFilename, unsigned long *ulFileStartAddress, unsigned long *ulMaxAddress, unsigned long *ulMaxLen ) */
/* Arguments																*/
/*	char* pFilename 				  : Load program file name				*/
/*	unsigned long *ulFileStartAddress : file start address pointer			*/
/*	unsigned long *ulMaxAddress 	  : file max   address pointer			*/
/*	unsigned long *ulMaxLen 		  : file max address record data length */
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Get MOT file minimum start record address.				*/
/****************************************************************************/
static int
c33_GetFileStartAddress( char *pFilename, unsigned long *ulFileStartAddress, unsigned long *ulMaxAddress, unsigned long *ulMaxLen )
{
	unsigned char ucLine[512];
	unsigned char ucRecordData[512];
	FILE *fpLoadFile = NULL;
	unsigned long ulAddress;
	unsigned long ulLen;

	*ulFileStartAddress = 0xffffffff;
	*ulMaxAddress = 0;
	
	/* Open command file */
	fpLoadFile = fopen(pFilename, "r");
	if (fpLoadFile == NULL)
	{
		/* file open error */
		c33_error (C33_ERR_FILE_OPEN, pFilename);
		return C33_NO_ERR;	/* 0 --> C33_NO_ERR T.Tazaki 2007/07/12 */
	}

	while(fgets(ucLine,500,fpLoadFile) != 0)
	{
		switch(SRecordToBin(ucLine, ucRecordData, &ulAddress, &ulLen))
		{
			case C33_REC_DOWNLOAD:
				if( ulAddress < *ulFileStartAddress )
				{
					*ulFileStartAddress = ulAddress;/* update minimum address */
				}
				if( ulAddress > *ulMaxAddress )
				{
					*ulMaxAddress = ulAddress;		/* update maximum address */
					*ulMaxLen = ulLen;
				}
				break;

			case C33_REC_IGNORE:
				break;

			case C33_REC_NAME:
				break;

			case C33_REC_ERROR:
			default:
				fclose(fpLoadFile);
				c33_error ( C33_ERR_FILE_FORMAT, pFilename );
//				break;					/* Change 2007/07/12 T.Tazaki */
				return C33_ERR;			/* Change 2007/07/12 T.Tazaki */

		}
	}

	fclose(fpLoadFile);
	
	return C33_NO_ERR;
}

/****************************************************************************/
/* Format		 : static int SRecordToBin( char *buffer_address, unsigned char *dest_buf, unsigned long *ice_address, unsigned long *num ) */
/* Arguments																*/
/*	char *buffer_address		   : address of source (S_record)			*/
/*	unsigned char *dest_buf 	   : destination for binary data			*/
/*	unsigned long *ice_address	   : address for modification				*/
/*	unsigned long *num			   : byte count for modification			*/
/* Return value  : Record Type												*/
/* Explanation	 : Subroutine to convert Motorola S_record format files to binary data.  */
/****************************************************************************/
static int
SRecordToBin( char *buffer_address, unsigned char *dest_buf, unsigned long *ice_address, unsigned long *num )
{
	char		*buf_ptr;
	char		*data_end;
	char		s_type;
	int 		type;
	int 		length = 0;
	
	buf_ptr = buffer_address;
	
	s_type = *( ++buf_ptr );

	length = ( htob( ++buf_ptr ) << 4 );
	length += ( htob( ++buf_ptr ) );
	data_end = buf_ptr + ( ( length - 1 ) * 2 );

	if ( !CheckSum( buf_ptr, (long)length ) ) {
		return C33_REC_ERROR;
	}

	switch(s_type) {
	case '0':
		Type0Decode(++buf_ptr, data_end, dest_buf);
		type = C33_REC_NAME;
		break;

	case '1':
		Type1Decode(++buf_ptr, data_end, dest_buf, ice_address, num);
		type = C33_REC_DOWNLOAD;
		break;

	case '2':
		Type2Decode(++buf_ptr, data_end, dest_buf, ice_address, num);
		type = C33_REC_DOWNLOAD;
		break;

	case '3':
		Type3Decode(++buf_ptr, data_end, dest_buf, ice_address, num);
		type = C33_REC_DOWNLOAD;
		break;

		// Should set PC to this address
	case '7':		// 4 byte entry address
	case '8':		// 3 byte entry address
	case '9':		// 2 byte entry address
		type = C33_REC_IGNORE;
		break;

	default:
		type = C33_REC_ERROR;
	}
	return type;
}

/****************************************************************************/
/* Format		 : static int htob( char *pointer ) 						*/
/* Arguments																*/
/*	char *pointer				   : pointer to string containing ASCII character  */
/* Return value  : binary													*/
/* Explanation	 : Hexadecimal ascii to binary value conversion subroutine. */
/****************************************************************************/
static int
htob( char *pointer )
{
	if ( ( *pointer >= '0' ) && ( *pointer <= '9' ) )
	{
		   return ( *pointer - '0' );
	}
	if ( ( *pointer >= 'a' ) && ( *pointer <= 'f' ) )
	{
	   return ( *pointer - 'a' + 10 );
	}
	if ( ( *pointer >= 'A' ) && ( *pointer <= 'F' ) )
	{
		   return ( *pointer - 'A' + 10 );
	}
	return 0;
}

/****************************************************************************/
/* Format		 : static int CheckSum( char *buf_ptr, long length )		*/
/* Arguments																*/
/*	char *buf_ptr	  : current pointer into source buffer in PC.			*/
/*	long length 	  : number of data bytes in record. 					*/
/* Return value  : TRUE if valid else FALSE.								*/
/* Explanation	 : Calculate & compare checksum.							*/
/****************************************************************************/
static int
CheckSum( char *buf_ptr, long length )
{
	char	checksum = 0;
	char	s_check;
	char	*pointer;
	int 	count;

	pointer  = buf_ptr - 2;
	
	for ( count = (int)length; count > 0; count-- ) {
		checksum = (unsigned char)( checksum + ( htob( ++pointer ) << 4 ) );
		checksum = (unsigned char)( checksum + htob( ++pointer ) );
	}
	checksum = (unsigned char)(0xff - checksum);

	// Convert ASCII representation of checksum byte to hex value.
	s_check = (unsigned char)( htob( ++pointer ) << 4 );
	s_check = (unsigned char)( s_check + htob( ++pointer ) );

	if (checksum == s_check) {
		return C33_TRUE;
	} else {
		return C33_FALSE;
	}
}
/****************************************************************************/
/* Format		 : static void Type0Decode(char *pointer, char *end, unsigned char *string) */
/* Arguments																*/
/*	char *pointer		  : current pointer into source buffer in PC.		*/
/*	char *end			  : end address of source buffer.					*/
/*	unsigned char *string : pointer to comment string.						*/
/* Return value  : void 													*/
/* Explanation	 : Extract comment field from S0 record.					*/
/****************************************************************************/
static void
Type0Decode(char *pointer, char *end, unsigned char *string)
{
	unsigned char	 *comment;

	comment = string;

	// Skip location address fields.
	pointer += 4;							

	while ( pointer < end ) {
	// Convert ascii hex into ascii and store in comment string.
		*comment = (char)( htob( pointer++ ) << 4 );
		*comment = (char)( *comment + htob( pointer++ ) );
		comment++;
	}
	// Set end of string to NULL.
	*comment = '\0';

	return;
}
/****************************************************************************/
/* Format		 : static void Type1Decode(char *pointer, char *end, unsigned char *dest, unsigned long *ice, unsigned long *num */
/* Arguments																*/
/*	char *pointer		  : current pointer into source buffer in PC.		*/
/*	char *end			  : end address of source buffer.					*/
/*	unsigned char *dest   : pointer to destination buffer.					*/
/*	unsigned char *ice	  : destination address in ICD memory.				*/
/*	unsigned char *num	  : number of bytes (to be returned to caller)g.	*/
/* Return value  : void 													*/
/* Explanation	 : Extract data from S1 record. 							*/
/****************************************************************************/
static void
Type1Decode(char *pointer, char *end, unsigned char *dest, unsigned long *ice, unsigned long *num)
{
	unsigned char  *destination;

	destination = dest;

	*ice  = 0;
	// Assemble ICE destination address from component ASCII characters
	// in location address fields, into long word value
	*ice  = ((long)htob(pointer++ ) << 12); 
	*ice += ((long)htob(pointer++ ) <<	8); 
	*ice += ((long)htob(pointer++ ) <<	4); 
	*ice +=  (long)htob(pointer++ );
	
   *num = 0;
	
	while (pointer < end) {
		// Convert ASCII hex to bin data in destination bufr.
		*destination = (char)(htob(pointer++) << 4);
		*destination = (char)(*destination + htob(pointer++));
		destination++;
		(*num)++;
	}

	return;
}

/****************************************************************************/
/* Format		 : static void Type2Decode(char *pointer, char *end, unsigned char *dest, unsigned long *ice, unsigned long *num */
/* Arguments																*/
/*	char *pointer		  : current pointer into source buffer in PC.		*/
/*	char *end			  : end address of source buffer.					*/
/*	unsigned char *dest   : pointer to destination buffer.					*/
/*	unsigned char *ice	  : destination address in ICD memory.				*/
/*	unsigned char *num	  : number of bytes (to be returned to caller)g.	*/
/* Return value  : void 													*/
/* Explanation	 : Extract data from S2 record. 							*/
/****************************************************************************/
static void
Type2Decode(char *pointer, char *end, unsigned char *dest, unsigned long *ice, unsigned long *num)
{
	unsigned char  *destination;

	destination = dest;

	*ice  = 0;
	// Assemble ICE destination address from component ASCII characters
	// in location address fields, into long word value.
	*ice  = ((long)htob(pointer++) << 20);
	*ice += ((long)htob(pointer++) << 16);
	*ice += ((long)htob(pointer++) << 12);
	*ice += ((long)htob(pointer++) <<  8);
	*ice += ((long)htob(pointer++) <<  4);
	*ice +=  (long)htob(pointer++);
	
	*num = 0;
	
	while(pointer < end) {
	// Convert ASCII hex to bin data in destination bufr.
		*destination = (char)(htob(pointer++) << 4);
		*destination = (char)(*destination + htob(pointer++));
		destination++;
		(*num)++;
	}

	return;
}


/****************************************************************************/
/* Format		 : static void Type3Decode(char *pointer, char *end, unsigned char *dest, unsigned long *ice, unsigned long *num */
/* Arguments																*/
/*	char *pointer		  : current pointer into source buffer in PC.		*/
/*	char *end			  : end address of source buffer.					*/
/*	unsigned char *dest   : pointer to destination buffer.					*/
/*	unsigned char *ice	  : destination address in ICD memory.				*/
/*	unsigned char *num	  : number of bytes (to be returned to caller)g.	*/
/* Return value  : void 													*/
/* Explanation	 : Extract data from S3 record. 							*/
/****************************************************************************/
static void
Type3Decode(char *pointer, char *end, unsigned char *dest, unsigned long *ice, unsigned long *num)
{
	unsigned char  *destination;

	destination = dest;

	*ice  = 0;
	// Assemble ICE destination address from component ASCII characters
	// in location address fields, into long word value.
	*ice  = ((long)htob(pointer++) << 28);
	*ice += ((long)htob(pointer++) << 24);
	*ice += ((long)htob(pointer++) << 20);
	*ice += ((long)htob(pointer++) << 16);
	*ice += ((long)htob(pointer++) << 12);
	*ice += ((long)htob(pointer++) <<  8);
	*ice += ((long)htob(pointer++) <<  4);
	*ice +=  (long)htob(pointer++);
	
	*num = 0;
	
	while (pointer < end) {
	// Convert ASCII hex to bin data in destination bufr.
		*destination = (char)(htob(pointer++) << 4);
		*destination = (char)(*destination + htob(pointer++));
		destination++;
		(*num)++;
	}

	return;
}


/****************************************************************************/
/* Format		 : void c33_IcdVer3FlashDataCopy(int iParamNum, unsigned long ulSrcAddress,unsigned long ulSize,) */
/*													unsigned long ulEraseStartBlock, unsigned long ulEraseEndBlock,unsigned long ulEraseParam,char *pComment) */
/* Arguments																*/
/*	int iParamNum				   : Parameter number						*/
/*	unsigned long ulSrcAddress	   : Copy source	  address (CPU flash)	*/
/*	unsigned long ulSize		   : Copy byte size 						*/
/*	unsigned long ulEraseStartBlock: Erase start block						*/
/*	unsigned long ulEraseEndBlock  : Erase end block						*/
/*	unsigned long ulEraseParam	   : Erase parameter						*/
/*	char *pComment				   : Comment								*/
/* Return value  : void 													*/
/* Explanation	 : Flash Writer : Data Erase & Data Information set &		*/
/*								  Data copy from target to ICD. 			*/
/*				   ICD flash address = 0x00000000 FIXED.					*/
/****************************************************************************/
void
c33_IcdVer3FlashDataCopy(int iParamNum, unsigned long ulSrcAddress, unsigned long ulSize,
						 unsigned long ulEraseStartBlock, unsigned long ulEraseEndBlock,unsigned long ulEraseParam,char *pComment)
{
	int iResult;


	/* Check number of parameter */
	if ( iParamNum != 6 && iParamNum != 7 )
	{
		c33_error (C33_ERR_COMMAND_PARAM);
		return;
	}

	if ( iParamNum == 6 )
	{
		pComment = (char *)g_NoCommentData; /* dummy string set */
	}


	/*------------------------------------------------------*/
	/* Flash Data Erase 									*/
	/*------------------------------------------------------*/
	 printf_filtered (C33_MSG_FLS3_ERASE_DATA);

	iResult = c33_SendMessage(C33_ID_FRASH_ERASE, C33_DATA_LOAD, 0, 0, 0, 0, 0, 0);
	if (iResult == C33_NO_ERR)
	{
		iResult = c33_RecvMessage(C33_ID_FRASH_ERASE, 0, 0, 0, 0, 0);
	}

	if (iResult != C33_NO_ERR)
	{
		puts_filtered (C33_MSG_TARGET_FAILURE);
		c33_ErrorMsg();
	}
	puts_filtered (C33_MSG_TARGET_DONE);

	/*------------------------------------------------------*/
	/* Flash Data Information set							*/
	/*------------------------------------------------------*/
	/* Data copy start title display */
	printf_filtered (C33_MSG_FLS3_COPY, 0x0, (int)ulSrcAddress, (int)ulSize);

	/* boundary matching */
	ulSrcAddress  &= 0xfffffffe;
	
	/* Set Data address information */

	iResult = c33_SendMessage(C33_ID_FRASH_ADDR_INFO_SET, 0x0,	ulSrcAddress, ulSize,
							  ulEraseParam, pComment, ulEraseStartBlock, ulEraseEndBlock );
	if (iResult == C33_NO_ERR)
	{
		iResult = c33_RecvMessage(C33_ID_FRASH_ADDR_INFO_SET, 0, 0, 0, 0, 0);
	}

	if (iResult != C33_NO_ERR)
	{
		puts_filtered (C33_MSG_TARGET_FAILURE);
		c33_ErrorMsg();
	}


	/*------------------------------------------------------*/
	/* Flash Data Copy										*/
	/*------------------------------------------------------*/

	iResult = c33_SendMessage(C33_ID_FRASH_COPY, 0x0, ulSrcAddress, ulSize, 0, 0, 0, 0);
	if (iResult == C33_NO_ERR)
	{
		iResult = c33_RecvMessage(C33_ID_FRASH_COPY, 0, 0, 0, 0, 0);
	}

	if (iResult == C33_NO_ERR)
	{
		puts_filtered (C33_MSG_TARGET_DONE);
	}
	else
	{
		puts_filtered (C33_MSG_TARGET_FAILURE);
		c33_ErrorMsg();
	}
}
/****************************************************************************/
/* Format		 : void c33_IcdVer3FlashInfoDisplay(int iParamNum ) 		*/
/* Arguments																*/
/*	int iParamNum	   : Parameter number									*/
/* Return value  : void 													*/
/* Explanation	 : Flash Writer : Program/Data Address information display. */
/****************************************************************************/
void
c33_IcdVer3FlashInfoDisplay(int iParamNum)
{
	int iResult;
	struct c33_flash_writer_info c33FlashWriterInfo;
	
	/* Check number of parameter */
	if ( iParamNum != 1 )
	{
		c33_error (C33_ERR_COMMAND_PARAM);
		return;
	}

	iResult = c33_SendMessage(C33_ID_FRASH_ADDR_INFO_GET, 0, 0, 0, 0, 0, 0, 0 );
	if (iResult == C33_NO_ERR)
	{
		iResult = c33_RecvMessage(C33_ID_FRASH_ADDR_INFO_GET, 0, 0, 0, 0, &c33FlashWriterInfo);
	}

	if (iResult != C33_NO_ERR)
	{
		c33_ErrorMsg();
	}
	
	/* address information display */
	
//	  printf_filtered ( C33_MSG_FLS3_DATA_INFO1, (unsigned int)c33FlashWriterInfo.ulIcdDataAddress ); //del 2003/06/16 T.Tazaki
	printf_filtered ( C33_MSG_FLS3_DATA_INFO2, (unsigned int)c33FlashWriterInfo.ulCpuDataAddress );
	printf_filtered ( C33_MSG_FLS3_DATA_INFO3, (unsigned int)c33FlashWriterInfo.ulDataSize );
	printf_filtered ( C33_MSG_FLS3_DATA_INFO4, (unsigned int)c33FlashWriterInfo.ulEraseStartBlock );
	printf_filtered ( C33_MSG_FLS3_DATA_INFO5, (unsigned int)c33FlashWriterInfo.ulEraseEndBlock );
	printf_filtered ( C33_MSG_FLS3_DATA_INFO6, (unsigned int)c33FlashWriterInfo.ulEraseParam );
	printf_filtered ( C33_MSG_FLS3_DATA_INFO7, c33FlashWriterInfo.szDataComment );
	printf_filtered ( "\n" );

	iResult = c33_SendMessage(C33_ID_FRASH_ENTRY_INFO_GET, 0, 0, 0, 0, 0, 0, 0 );
	if (iResult == C33_NO_ERR)
	{
		iResult = c33_RecvMessage(C33_ID_FRASH_ENTRY_INFO_GET, 0, 0, 0, 0, &c33FlashWriterInfo);
	}

	if (iResult != C33_NO_ERR)
	{
		c33_ErrorMsg();
	}
	
	/* address information display */
	
//	  printf_filtered ( C33_MSG_FLS3_ENTRY_INFO1, (unsigned int)c33FlashWriterInfo.ulIcdProgAddress ); //del 2003/06/16 T.Tazaki
	printf_filtered ( C33_MSG_FLS3_ENTRY_INFO2, (unsigned int)c33FlashWriterInfo.ulCpuProgAddress );
	printf_filtered ( C33_MSG_FLS3_ENTRY_INFO3, (unsigned int)c33FlashWriterInfo.ulProgSize );
	printf_filtered ( C33_MSG_FLS3_ENTRY_INFO4, (unsigned int)c33FlashWriterInfo.ulEraseEntryAddress );
	printf_filtered ( C33_MSG_FLS3_ENTRY_INFO5, (unsigned int)c33FlashWriterInfo.ulWriteEntryAddress );
	printf_filtered ( C33_MSG_FLS3_ENTRY_INFO6, c33FlashWriterInfo.szProgComment );
}
/****************************************************************************/
/* Format		 : void c33_IcdVer3FirmUpdate(int iParamNum, char* pFilename)*/
/* Arguments																*/
/*	int iParamNum					  : Parameter number					*/
/*	char* pFilename 				  : Firmware file name(motolora S3 type */
/* Return value  : void 													*/
/* Explanation	 : ICDV3 firmware update.									*/
/****************************************************************************/
void
c33_IcdVer3FirmUpdate(int iParamNum, char* pFilename)
{
	int iResult;
	unsigned char ucLine[512];
	unsigned char ucRecordData[512];
	FILE *fpLoadFile = NULL;
	unsigned long ulLen = 0;
	char szBuf[300];
	int iFirstFileAddress = 1;
	unsigned long ulAddress;
	unsigned long ulFileStartAddress = 0;
	unsigned long ulTmpLoadAddress;
	unsigned long ulMaxAddress = 0;
	unsigned long ulMaxLen = 0;
	int iCnt;
	int iCancel;

	/* Check number of parameter */
	if ( iParamNum != 2 )
	{
		c33_error (C33_ERR_COMMAND_PARAM);
		return;
	}

	fpLoadFile = fopen(pFilename, "r");
	if (fpLoadFile == NULL)
	{
		/* file open error */
		sprintf( szBuf,C33_ERR_FILE_OPEN,pFilename );
		c33_error (szBuf);
		return;
	}
	fclose(fpLoadFile);


	/* Get file minimum addr, maximum addr, length & file format check */
	iResult = c33_GetFileStartAddress( pFilename, &ulFileStartAddress, &ulMaxAddress, &ulMaxLen );
	if (iResult != C33_NO_ERR)
	{
		return;
	}


	/* Check size limit */
	if( g_iICD_kind == C33_ICDV6 )	/* ICDV6 ΉFICDV6 max 4MB Flash ROM  T.Tazaki 2007/07/25 */
	{
		if( ( ulMaxAddress + ulMaxLen - ulFileStartAddress ) > 0x3fffff )
		{
			sprintf( szBuf,C33_ERR_LOAD_ADDR_OVER, 0x3fffff );
			c33_error (szBuf);
			return;
		}
	}
	else
	{
		if( ( ulMaxAddress + ulMaxLen - ulFileStartAddress ) > 0x7fffff )
		{
			sprintf( szBuf,C33_ERR_LOAD_ADDR_OVER, 0x7fffff );
			c33_error (szBuf);
			return;
		}
	}

	/*------------------------------------------------------*/
	/* Flash Data Erase 									*/
	/*------------------------------------------------------*/
	 printf_filtered (C33_MSG_FLS3_ERASE_DATA);

	iResult = c33_SendMessage(C33_ID_FRASH_ERASE, C33_DATA_LOAD, 0, 0, 0, 0, 0, 0);
	if (iResult == C33_NO_ERR)
	{
		iResult = c33_RecvMessage(C33_ID_FRASH_ERASE, 0, 0, 0, 0, 0);
	}

	if (iResult != C33_NO_ERR)
	{
		puts_filtered (C33_MSG_TARGET_FAILURE);
		c33_ErrorMsg();
	}
	puts_filtered (C33_MSG_TARGET_DONE);



	/*------------------------------------------------------*/
	/* Motolora file read & load							*/
	/*------------------------------------------------------*/
	printf_filtered (C33_MSG_FLS3_DATA_LOAD);

	/* Motolora file read & load */
	fpLoadFile = fopen(pFilename, "r");
	if (fpLoadFile == NULL)
	{
		/* file open error */
		puts_filtered (C33_MSG_TARGET_FAILURE);
		sprintf( szBuf,C33_ERR_FILE_OPEN,pFilename );
		c33_error (szBuf);
		return;
	}

	iCnt = 0;

	c33_InitCancel();	/* Cancel initialize */
	iCancel = 0;

	while(fgets(ucLine,500,fpLoadFile) != 0)
	{
		if (SRecordToBin(ucLine, ucRecordData, &ulAddress, &ulLen) == C33_REC_DOWNLOAD )
		{

			ulTmpLoadAddress = ulAddress - ulFileStartAddress;

			/* program/data load command*/
			iResult = c33_SendMessage(C33_ID_FRASH_LOAD, C33_DATA_LOAD, ulTmpLoadAddress, ulLen, 0, ucRecordData, 0, 0);
			if (iResult == C33_NO_ERR)
			{
				iResult = c33_RecvMessage(C33_ID_FRASH_LOAD, 0, 0, 0, 0, 0);
			}

			if (iResult != C33_NO_ERR)
			{
				fclose( fpLoadFile );
				c33_EndCancel();	/* Cancel Check End */
				puts_filtered (C33_MSG_TARGET_FAILURE);
				c33_ErrorMsg();
			}
			/* Check cancel */
			if (c33TargetInfo.iKeyBreak == 1 || c33_CheckCancel( 50 ) == 1 )
			{
				/* Cancel */
				iCancel = 1;
				printf_filtered( C33_MSG_TARGET_CANCEL);
				break;
			}
			++iCnt;
			if( iCnt == 5000 )
			{
				puts_filtered( "." );
				iCnt = 0;
			}
		}
	}

	fclose(fpLoadFile);
	
	c33_EndCancel();	/* Cancel Check End */

	puts_filtered (C33_MSG_TARGET_DONE);

	/*------------------------------------------------------*/
	/* Firmware update										*/
	/*------------------------------------------------------*/

	printf_filtered (C33_MSG_FIRM_UPDATE);

	iResult = c33_SendMessage(C33_ID_FIRM_UPDATE, 0, 0, 0, 0, 0, 0, 0);
	if (iResult == C33_NO_ERR)
	{
		iResult = c33_RecvMessage(C33_ID_FIRM_UPDATE, 0, 0, 0, 0, 0);
	}

	if (iResult != C33_NO_ERR)
	{
		puts_filtered (C33_MSG_TARGET_FAILURE);
		c33_ErrorMsg();
		return;
	}
	/* wait firmware reset process is done.*/
	puts_filtered (C33_MSG_TARGET_DONE);

	/*------------------------------------------------------*/
	/* Message diaplay										*/
	/*------------------------------------------------------*/
	
	printf_filtered( C33_MSG_FIRM_UPDATE_END);
}
/* add T.Tazaki 2007/07/03 new command >>> */
/****************************************************************************/
/* Format		 : void c33_Cmd_TargetReset(void)							*/
/* Arguments																*/
/* Return value  : void 													*/
/* Explanation	 : Target reset.             								*/
/****************************************************************************/
void
c33_Cmd_TargetReset()
{
	int iResult;

	puts_filtered (C33_MSG_TARGET_RESET_TARGET);

	iResult = c33_SendMessage(C33_ID_TARGET_RESET, 0, 0, 0, 0, 0, 0, 0);
	if (iResult == C33_NO_ERR)
	{
		iResult = c33_RecvMessage(C33_ID_TARGET_RESET, 0, 0, 0, 0, 0);
	}
	if (iResult != C33_NO_ERR)
	{
		c33_ErrorMsg();
	}
	if( g_ucTargetReset_result == 0 )
	{
		puts_filtered (C33_MSG_TARGET_DONE);
	}
	else
	{
		puts_filtered (C33_MSG_TARGET_FAILURE);
	}
}
/****************************************************************************/
/* Format		 : void c33_IcdVer6TimerBreak(int iParamNum, unsigned long ulTimer)	*/
/* Arguments																*/
/*	int iParamNum					  : Parameter number					*/
/*	unsigned long ulTimer			  : Timer Count(0`300000)		        */
/* Return value  : void 													*/
/* Explanation	 : ICDmini set Timer break. 								*/
/****************************************************************************/
void
c33_IcdVer6TimerBreak(int iParamNum, unsigned long ulTimer)
{
	int iResult;
	int iFlag = 0;
    long double  ldTmp;		

	/* Check number of parameter */
	if ( iParamNum != 2 )
	{
		c33_error (C33_ERR_COMMAND_PARAM);
		return;
	}

	
	/* Check timer off */
	if( ulTimer == 0 )
	{
		iFlag = 0;
		printf_filtered( "    Timer break off.\n" );
	}
	else if ( ulTimer > C33_TIMBRK_MAX )		/* MAX over */
	{
		c33_error( C33_ERR_TIMER_OVER );	/* change message T.Tazaki 2007/06/04 */
		return;
	}
	else
	{
        printf_filtered( "    Timer break on. [%d ms]\n", ulTimer );
	
		ldTmp = ulTimer * 1000;		/* MS --> uS  */
 		ldTmp /= C33_TIMER_US;   	/* uS --> counter */
		ulTimer = ldTmp;

		iFlag = 1;
	}
	
	g_ullClockCount = 0;
	
	iResult = c33_SendMessage(C33_ID_TIME_BREAK, iFlag, ulTimer, 0, 0, 0, 0, 0);
	if (iResult == C33_NO_ERR)
	{
		iResult = c33_RecvMessage(C33_ID_TIME_BREAK, 0, 0, 0, 0, 0);
	}
	if (iResult != C33_NO_ERR)
	{
		c33_ErrorMsg();
	}
	g_iTimeBreak = iFlag;
	
}
/* add T.Tazaki 2007/07/03 new command <<< */

/****************************************************************************/
/* Format		 : void c33_IcdVer3ReadMode(int iParamNum, int iMode)		*/
/* Arguments																*/
/*	int iParamNum					  : Parameter number					*/
/*	int iMode						  : 0=Normal,1=Exact read				*/
/* Return value  : void 													*/
/* Explanation	 : ICDV3 select memory read mode.							*/
/****************************************************************************/
void
c33_IcdVer3ReadMode(int iParamNum, int iMode)
{
	/* Check number of parameter */
	if ( iParamNum != 2 )
	{
		c33_error (C33_ERR_COMMAND_PARAM);
		return;
	}

	if( iMode != 0 && iMode != 1 )
	{
		c33_error( C33_ERR_COMMAND_INVALID_PARAM );
		return;
	}
	
	g_iMemReadMode = iMode;
}

/****************************************************************************/
/* Format		 : void c33_IcdVer3DebugBaseSet(int iParamNum,unsigned long ulRomAddr,unsigned long ulRamAddr,unsigned char ucCpuType) */
/* Arguments																*/
/*	int iParamNum				   : Parameter number						*/
/*	unsigned long ulRomAddr 	   : Debug unit ROM base address			*/
/*	unsigned long ulRamAddr 	   : Debug unit RAM base address			*/
/*	unsigned char ucCpuType 	   : Cpu type								*/
/* Return value  : void 													*/
/****************************************************************************/
void
c33_IcdVer3DebugBaseSet(int iParamNum,unsigned long ulRomAddr,unsigned long ulRamAddr,unsigned char ucCpuType)
{
	int iResult = C33_NO_ERR;

	/* Check number of parameter */
	if ( iParamNum != 3 && iParamNum != 4 )
	{
		c33_error (C33_ERR_COMMAND_PARAM);
		return;
	}


	/* check if das command is first.   add T.Tazaki 2005/06/23 */
	if( g_c33icd_das == 0 )
	{
		if ( iParamNum == 4 )
		{
			/* check CPU type */
			if( ucCpuType != C33_STD && ucCpuType != C33_PE && ucCpuType != C33_ADV )
			{
				c33_error (C33_ERR_COMMAND_PARAM);
				return;
			}
			/* set CPU type */
			c33TargetInfo.ucCpuType = ucCpuType;

			g_iDasCpu = ucCpuType;	/* add T.Tazaki 2005/04/01 */

		}
		else
		{
			/* set CPU type : default */
			c33TargetInfo.ucCpuType = C33_STD;	  /* standard macro */

			g_iDasCpu = C33_AUTO;	/* add T.Tazaki 2005/04/01 */
		}

		/* set Debug unit address */
		c33TargetInfo.ulDebugBase_rom_addr = ulRomAddr;
		c33TargetInfo.ulDebugBase_ram_addr = ulRamAddr;

		g_c33icd_das = 1;

	}
	else
	{
		/* 2nd c33 das command : send message DebugBaseSet        add T.Tazaki 2005/06/23 >>> */

	    puts_filtered (C33_MSG_TARGET_SET_DEBUG_BASE);

		/* set Debug unit address */
		c33TargetInfo.ulDebugBase_rom_addr = ulRomAddr;
		c33TargetInfo.ulDebugBase_ram_addr = ulRamAddr;

	    /* Transmission of a message. */
	    iResult = c33_SendMessage(C33_ID_SET_DEBUG_BASE, 0, 0, 0, 0, 0, 0, 0);

	    /* Reception of a message. */
	    if (iResult == C33_NO_ERR){
	        iResult = c33_RecvMessage(C33_ID_SET_DEBUG_BASE, 0, 0, 0, 0, 0);
	    }

	    if (iResult == C33_NO_ERR)
	    {
	        puts_filtered (C33_MSG_TARGET_DONE);
	    }
	    else
	    {
	        puts_filtered (C33_MSG_TARGET_FAILURE);
	    }
		/* 2nd c33 das command : send message DebugBaseSet        add T.Tazaki 2005/06/23 <<< */
	}
}
/****************************************************************************/
/* Format		 : void c33_IcdVer3AreaBreak(int iParamNum, char *pMode, char cBBCUBusSelect, char *pReadWrite, unsigned long ulArea ) */
/* Arguments																*/
/*	int iParamNum		 : Parameter number 								*/
/*	char *pMode 		 : Break mode "on" or "off" 						*/
/*	char cBBCUBusSelect  : bit3:USR bit2:DMA bit1:CCU bit0:HBCU 			*/
/*	char *pReadWrite	 : read/write set  "r":read "w":write "rw":read/write */
/*	unsigned long ulArea : CE area	bit0-bit22 (1:ON,0:OFF) 				*/
/* Return value  : void 													*/
/****************************************************************************/
void
c33_IcdVer3AreaBreak(int iParamNum, char *pMode, char cBBCUBusSelect, char *pReadWrite, unsigned long ulArea )
{
	int iResult = C33_NO_ERR;
	char szBuf[50];
	char szBuf2[50];
	unsigned long ulAreaAnd;
	int i;
	int iLen;
	int iReadWrite = 0;


	/* Check if area break function is enable. */
	if( (c33TargetInfo.ucBrkFunc & C33_GDB_STS_AREA_BREAK) == 0 )
	{
		c33_error (C33_ERR_MSG_TARGET_AREA_BRK_DIS);
	}

	/* Check number of parameter */
	if ( iParamNum != 2 && iParamNum != 5 )
	{
		c33_IcdVer3AreaBreak_usage();
		return;
	}

	if( strcmp(pMode,"off") == 0 )
	{
		if ( iParamNum != 2 )
		{
			c33_IcdVer3AreaBreak_usage();
			return;
		}

		c33TargetInfo.iArea_Mode = 0;
		
		printf_filtered ( C33_MSG_AREA_BREAK_OFF );

		c33_IcdVer3BreakModeToToolbar();	/* add T.Tazaki 2005/05/18 */
	}
	else
	{
		if( strcmp(pMode,"on") == 0 )
		{
			if ( iParamNum != 5 )
			{
				c33_IcdVer3AreaBreak_usage();
				return;
			}
			
			/* parameter check */
			if( cBBCUBusSelect > 0x0f || cBBCUBusSelect <= 0 )
			{
				c33_IcdVer3AreaBreak_usage();
				return;
			}
			if( strcmp(pReadWrite,"r") == 0 )
			{
				iReadWrite = 0;
			}
			else if ( strcmp(pReadWrite,"w") == 0 )
			{
				iReadWrite = 1;
			}
			else if ( strcmp(pReadWrite,"rw") == 0 )
			{
				iReadWrite = 2;
			}
			else
			{
				c33_IcdVer3AreaBreak_usage();
				return;
			}
			if( ulArea > 0x007fffff || ulArea == 0)
			{
				c33_IcdVer3AreaBreak_usage();
				return;
			}


			c33TargetInfo.iArea_Mode = 1;
			c33TargetInfo.cArea_BBCUBusSelect = cBBCUBusSelect;
			c33TargetInfo.iArea_ReadWrite	  = iReadWrite;
			c33TargetInfo.ucArea1		 = (unsigned char)( ulArea & 0x000000ff );
			c33TargetInfo.ucArea2		 = (unsigned char)((ulArea & 0x0000ff00 ) >> 8 );
			c33TargetInfo.ucArea3		 = (unsigned char)((ulArea & 0x007f0000 ) >> 16 );

			printf_filtered ( C33_MSG_AREA_BREAK_ON );

			/***** Parameter display *****/
			
			/* BBCU Bus Select */
			
			szBuf[0] = 0;
			if( cBBCUBusSelect & C33_BBCU_USR )
			{
				strcat( szBuf,"USER," );
			}
			if( cBBCUBusSelect & C33_BBCU_DMA )
			{
				strcat( szBuf,"DMA," );
			}
			if( cBBCUBusSelect & C33_BBCU_CCU )
			{
				strcat( szBuf,"CACHE," );
			}
			if( cBBCUBusSelect & C33_BBCU_HBCU )
			{
				strcat( szBuf,"CPU " );
			}
			iLen = strlen(szBuf);
			szBuf[iLen-1] = 0;	/* comma cut */
			printf_filtered ( C33_MSG_AREA_BREAK_BBCU_SELECT, szBuf );

			/* Read/Write */
			switch( iReadWrite )
			{
				case 0: strcpy( szBuf,"Read" );
						break;
				case 1: strcpy( szBuf,"Write" );
						break;
				case 2: strcpy( szBuf,"Read/Write" );
						break;
				default:strcpy( szBuf,"   " );			/* add T.Tazaki 2007/07/12 */
						break;						
			}
			printf_filtered ( C33_MSG_AREA_BREAK_RW, szBuf );
			
			/* Area */
			
			szBuf[0] = 0;
			ulAreaAnd = 0x00000001;
			for( i = 0; i < 23; ++i )
			{
				if( ulArea & ulAreaAnd )
				{
					sprintf( szBuf2,"%d,",i );
					strcat ( szBuf, szBuf2 );
				}
				ulAreaAnd <<= 1;
			}
			iLen = strlen(szBuf);
			szBuf[iLen-1] = 0;	/* comma cut */
			printf_filtered ( C33_MSG_AREA_BREAK_AREA, szBuf );

			c33_IcdVer3BreakModeToToolbar();	/* add T.Tazaki 2005/05/18 */

		}
		else
		{
			c33_IcdVer3AreaBreak_usage();
			return;
		}
	}
}
/****************************************************************************/
/* Format		 : void c33_IcdVer3AreaBreak_on(void) 						*/
/* Arguments																*/
/* Return value  : void 													*/
/****************************************************************************/
void
c33_IcdVer3AreaBreak_on()
{
	int iResult = C33_NO_ERR;
	
	/* Area break set */
	iResult = c33_SendMessage(C33_ID_AREA_BREAK_SET, 0, 0, 0, 0, 0, 0, 0 );
	if (iResult == C33_NO_ERR)
	{
		iResult = c33_RecvMessage(C33_ID_AREA_BREAK_SET, 0, 0, 0, 0, 0);
	}
	if (iResult != C33_NO_ERR)
	{
		c33_ErrorMsg();
	}
}
/****************************************************************************/
/* Format		 : void c33_IcdVer3AreaBreak_off(void) 						*/
/* Arguments																*/
/* Return value  : void 													*/
/****************************************************************************/
void
c33_IcdVer3AreaBreak_off()
{
	int iResult = C33_NO_ERR;
	
	/* Area break clear */
	iResult = c33_SendMessage(C33_ID_AREA_BREAK_CLR, 0, 0, 0, 0, 0, 0, 0 );
	if (iResult == C33_NO_ERR)
	{
		iResult = c33_RecvMessage(C33_ID_AREA_BREAK_CLR, 0, 0, 0, 0, 0);
	}
	if (iResult != C33_NO_ERR)
	{
		c33_ErrorMsg();
	}
}
/****************************************************************************/
/* Format		 : void c33_IcdVer3AreaBreak_usage( void )					*/
/* Arguments																*/
/* Return value  : void 													*/
/****************************************************************************/
void
c33_IcdVer3AreaBreak_usage( void )
{
	puts_filtered( "Usage : c33 oab ( On chip area break )\n");
	puts_filtered( "  c33 oab <mode> <BBCU bus master select> <r/w condition> <area>\n\n");
	puts_filtered( "          mode:\"on\"  area break mode on\n");
	puts_filtered( "               \"off\" area break mode off\n");
	puts_filtered( "          BBCU bus master select: 0x01-0x0f\n");
	puts_filtered( "                     bit3=1 memory access by user.\n");
	puts_filtered( "                     bit2=1 memory access by DMA.\n");
	puts_filtered( "                     bit1=1 memory access by cache.\n");
	puts_filtered( "                     bit0=1 memory access by cpu.\n");
	puts_filtered( "          r/w condition:\n");
	puts_filtered( "                     \"r\"  Read\n");
	puts_filtered( "                     \"w\"  Write\n");
	puts_filtered( "                     \"rw\" Read or write\n");
	puts_filtered( "          Area: 0x1-0x7fffffh\n");
	puts_filtered( "                     bit0-bit22 are equivalent to the area 0 - the area 22.\n");
	puts_filtered( "                     Effective at 1,Invalid at 0\n");

	c33_error (C33_ERR_COMMAND_INVALID_PARAM);
}

/****************************************************************************/
/* Format		 : void c33_IcdVer3BusBreak(int iParamNum, char *pMode, int iBusSelect ,char cBBCUBusSelect, int iTrigBreak, */
/*					 char *pReadWrite, int iInstData, char cAccess, int iInt, int iIntLevel, int iDataComp, 					*/
/*					 unsigned long ulAddress, unsigned long ulAddressMask, unsigned long ulData, unsigned long ulDataMask,	  */
/*					 unsigned long ulCount,int iCntClrSave )				*/
/* Arguments																*/
/*	int iParamNum		 : Parameter number 								*/
/*	char *pMode 		 : Break mode "on" or "off" 						*/
/*	int  iBusSelect 	 : Bus select 0:HBCU data 1:HBCU inst 2:BBCU		*/
/*	char cBBCUBusSelect  : bit3:USR bit2:DMA bit1:CCU bit0:HBCU 			*/
/*	int  iTrigBreak 	 : 0:Trig out only 1:break							*/
/*	char *pReadWrite	 : read/write set  "r":read "w":write "rw":read/write */
/*	int  iInstData		 : 0:Inst 1:data 2:both 							*/
/*	char cAccess		 : bit0:BYTE bit1:HALF bit2:WORD					*/
/*	int  iInt			 : 0:Break by int level disable 1:enable			*/
/*	int  iIntLevel		 : 0:Trig out only 1:break							*/
/*	int  iDataComp		 : 0:32bit compare 1:high16bit/low 16bit compare	*/
/*	unsigned long ulAddress 	: break address 							*/
/*	unsigned long ulAddressMask : break address mask bit					*/
/*	unsigned long ulData		: break data								*/
/*	unsigned long ulDataMask	: break data mask bit						*/
/*	unsigned long ulCount		: break counter 							*/
/*	int  iCntClrSave	 : break counter clear/save flag 1:clear,0:save 	*/
/* Return value  : void 													*/
/****************************************************************************/
void
c33_IcdVer3BusBreak(int iParamNum, char *pMode, int iBusSelect ,char cBBCUBusSelect, int iTrigBreak,
	 char *pReadWrite, int iInstData, char cAccess, int iInt, int iIntLevel, int iDataComp,
	 unsigned long ulAddress, unsigned long ulAddressMask, unsigned long ulData, unsigned long ulDataMask,
	 unsigned long ulCount,int iCntClrSave )
{
	int iResult = C33_NO_ERR;
	char szBuf[50];
	int iLen;
	int iReadWrite = 0;

	/* Check if bus break function is enable. */
	if( (c33TargetInfo.ucBrkFunc & C33_GDB_STS_BUS_BREAK) == 0 )
	{
		c33_error (C33_ERR_MSG_TARGET_BUS_BRK_DIS);
	}

	/* Check number of parameter */
	if ( iParamNum != 2 && iParamNum != 17 )
	{
		c33_IcdVer3BusBreak_usage();
		return;
	}

	if( strcmp(pMode,"off") == 0 )
	{
		if ( iParamNum != 2 )
		{
			c33_IcdVer3BusBreak_usage();
			return;
		}

		c33TargetInfo.iBus_Mode = 0;

		printf_filtered ( C33_MSG_BUS_BREAK_OFF );

		c33_IcdVer3BreakModeToToolbar();	/* add T.Tazaki 2005/05/18 */

	}
	else
	{
		if( strcmp(pMode,"on") == 0 )
		{
			if ( iParamNum != 17 )
			{
				c33_IcdVer3BusBreak_usage();
				return;
			}

			/* parameter check */
			if( iBusSelect < 0 || iBusSelect > 2 )
			{
				c33_IcdVer3BusBreak_usage();
				return;
			}
			if( iBusSelect == 1 )	/* change gdb param to farm packet */
			{
				iBusSelect = 2;
			}else if ( iBusSelect == 2 )
			{
				iBusSelect = 1;
			}
			
//	  		if( cBBCUBusSelect > 0x0f || cBBCUBusSelect < 0 )		/* bug fix T.Tazaki 2004/10/19 */
			if( iBusSelect == 1 )	/* BBCU ?	when only BBCU,check BUS MASTER. add T.Tazaki 2004/10/19 */
			{
				if( cBBCUBusSelect > 0x0f || cBBCUBusSelect <= 0 )
				{
					c33_IcdVer3BusBreak_usage();
					return;
				}
			}
			/* add T.Tazaki 2004/10/05 >>> */
			if( iBusSelect == 1 && cBBCUBusSelect == 0 )	/* BBCU and BUS MASTER = 0 ? */
			{
				c33_IcdVer3BusBreak_usage();
				return;
			}
			/* add T.Tazaki 2004/10/05 <<< */

//			if( iTrigBreak < 0 || iTrigBreak > 2 )	/* bug fix T.Tazaki 2004/10/19 */
			if( iTrigBreak < 0 || iTrigBreak > 1 )	/* bug fix T.Tazaki 2004/10/19 */
			{
				c33_IcdVer3BusBreak_usage();
				return;
			}
			if( strcmp(pReadWrite,"r") == 0 )
			{
				iReadWrite = 0;
			}
			else if ( strcmp(pReadWrite,"w") == 0 )
			{
				iReadWrite = 1;
			}
			else if ( strcmp(pReadWrite,"rw") == 0 )
			{
				iReadWrite = 2;
			}
			else
			{
				c33_IcdVer3BusBreak_usage();
				return;
			}
			if( iInstData < 0 || iInstData > 2 )
			{
				c33_IcdVer3BusBreak_usage();
				return;
			}
			if( cAccess < 0 || cAccess > 0x07 )
			{
				c33_IcdVer3BusBreak_usage();
				return;
			}
			if( iInt < 0 || iInt > 1 )
			{
				c33_IcdVer3BusBreak_usage();
				return;
			}
			if( iIntLevel < 0 || iIntLevel > 15 )
			{
				c33_IcdVer3BusBreak_usage();
				return;
			}
			if( iDataComp < 0 || iDataComp > 1 )
			{
				c33_IcdVer3BusBreak_usage();
				return;
			}
			if( ulCount < 1 || ulCount > 0x7fffffffUL )	/* change 2004/04/21 0x800000000UL -->	 T.Tazaki */
			{
				c33_IcdVer3BusBreak_usage();
				return;
			}
			--ulCount;
			
			if( iCntClrSave < 0 || iCntClrSave > 1 )
			{
				c33_IcdVer3BusBreak_usage();
				return;
			}

			c33TargetInfo.iBus_Mode = 1;
			c33TargetInfo.iBus_BusSelect	 = iBusSelect;
			c33TargetInfo.cBus_BBCUBusSelect = cBBCUBusSelect;
			c33TargetInfo.iBus_TrigBreak	 = iTrigBreak;
			c33TargetInfo.iBus_ReadWrite	 = iReadWrite;
			c33TargetInfo.iBus_InstData 	 = iInstData;
			c33TargetInfo.cBus_Access		 = cAccess;
			c33TargetInfo.iBus_Int			 = iInt;
			c33TargetInfo.iBus_IntLevel 	 = iIntLevel;
			c33TargetInfo.iBus_DataComp 	 = iDataComp;
			c33TargetInfo.ulBus_Address 	 = ulAddress;
			c33TargetInfo.ulBus_AddressMask  = ulAddressMask;
			c33TargetInfo.ulBus_Data		 = ulData;
			c33TargetInfo.ulBus_DataMask	 = ulDataMask;
			c33TargetInfo.ulBus_Count		 = (unsigned long)g_lBusBreakCnt = g_ulBus_Count_org = ulCount;
			c33TargetInfo.iBus_iCntClrSave	 = iCntClrSave;

			printf_filtered ( C33_MSG_BUS_BREAK_ON );

			/***** Parameter display *****/
			
			/* Bus Select */
			switch( iBusSelect )
			{
				case 0: strcpy( szBuf,"CPU data bus" );
						break;
				case 1: strcpy( szBuf,"BBCU" );
						break;
				case 2: strcpy( szBuf,"CPU instruction bus" );
						break;
				default:strcpy( szBuf,"   " );	/* add T.Tazaki 2007/07/12 */
						break;								

			}
			printf_filtered ( C33_MSG_BUS_BREAK_BUS_SELECT, szBuf );
			
			/* BBCU Bus Select */
			szBuf[0] = 0;
			if( iBusSelect == 1 )	/* BBCUnH 			add T.Tazaki 2004/11/22 */
			{
				if( cBBCUBusSelect & C33_BBCU_USR )
				{
					strcat( szBuf,"USER," );
				}
				if( cBBCUBusSelect & C33_BBCU_DMA )
				{
					strcat( szBuf,"DMA," );
				}
				if( cBBCUBusSelect & C33_BBCU_CCU )
				{
					strcat( szBuf,"CACHE," );
				}
				if( cBBCUBusSelect & C33_BBCU_HBCU )
				{
					strcat( szBuf,"CPU " );
				}
			}
			else
			{
				strcat( szBuf,"--- " );		/* add T.Tazaki 2004/11/22 */
			}
			iLen = strlen(szBuf);
			szBuf[iLen-1] = 0;	/* comma cut */
			printf_filtered ( C33_MSG_BUS_BREAK_BBCU_SELECT, szBuf );

			/* Trigger / Break */
			if( iTrigBreak == 0 )
			{
				strcpy( szBuf,"Trigger only" );
			}
			else
			{
				strcpy( szBuf,"Break" );
			}
			printf_filtered ( C33_MSG_BUS_BREAK_TRIG_BREAK, szBuf );

			/* Read / Write */
			switch( iReadWrite )
			{
				case 0: strcpy( szBuf,"Read" );
						break;
				case 1: strcpy( szBuf,"Write" );
						break;
				case 2: strcpy( szBuf,"Read/Write" );
						break;
				default:strcpy( szBuf,"   " );	/* add T.Tazaki 2007/07/12 */
						break;								
			}
			printf_filtered ( C33_MSG_BUS_BREAK_RW, szBuf );

			/* Instruction / Data */
			switch( iInstData )
			{
				case 0: strcpy( szBuf,"Instruction" );
						break;
				case 1: strcpy( szBuf,"Data" );
						break;
				case 2: strcpy( szBuf,"Instruction/Data" );
						break;
				default:strcpy( szBuf,"   " );	/* add T.Tazaki 2007/07/12 */
						break;								
			}
			printf_filtered ( C33_MSG_BUS_BREAK_INST_DATA, szBuf );
			
			/* Access bus size */
			szBuf[0] = 0;
			if( cAccess & 0x01 )	/* BYTE ? */
			{
				strcat( szBuf,"Byte," );
			}
			if( cAccess & 0x02 )	/* HALF WORD ? */
			{
				strcat( szBuf,"Half word," );
			}
			if( cAccess & 0x04 )	/* WORD ? */
			{
				strcat( szBuf,"Word," );
			}
			iLen = strlen(szBuf);
			szBuf[iLen-1] = 0;	/* comma cut */
			printf_filtered ( C33_MSG_BUS_BREAK_ACCESS_SIZE, szBuf );

			/* Interrupt permission */
			if( iInt == 0 )
			{
				strcpy( szBuf,"Disable" );
			}
			else
			{
				strcpy( szBuf,"Enable" );
			}
			printf_filtered ( C33_MSG_BUS_BREAK_INT, szBuf );
			
			/* Interrupt Level */
			printf_filtered ( C33_MSG_BUS_BREAK_INT_LEVEL, iIntLevel );
			
			/* Data compare */
			if( iDataComp == 0 )
			{
				strcpy( szBuf,"32bit" );
			}
			else
			{
				strcpy( szBuf,"high 16bit/low 16bit" );
			}
			printf_filtered ( C33_MSG_BUS_BREAK_DATA_COMP, szBuf );
			
			/* break address */
			printf_filtered ( C33_MSG_BUS_BREAK_ADDRESS, (unsigned int)ulAddress );
			printf_filtered ( C33_MSG_BUS_BREAK_ADDRESS_MASK, (unsigned int)ulAddressMask );

			/* break data */
			printf_filtered ( C33_MSG_BUS_BREAK_DATA, (unsigned int)ulData );
			printf_filtered ( C33_MSG_BUS_BREAK_DATA_MASK, (unsigned int)ulDataMask );
			
			/* break counter */
			printf_filtered ( C33_MSG_BUS_BREAK_COUNT, (unsigned int)ulCount + 1 );

			/* break counter clear/save */
			if( iCntClrSave == 0 )
			{
				strcpy( szBuf,"Save" );
			}
			else
			{
				strcpy( szBuf,"Clear" );
			}
			printf_filtered ( C33_MSG_BUS_BREAK_COUNT_CLR_SAVE, szBuf );

			/* Warning display add T.Tazaki 2005/02/07 >>> */
			/* Bus Select */
			switch( iBusSelect )
			{
				case 0:	printf_filtered ( "Warning : Bus selection of bus trace was also changed into \"CPU data bus\".\n"	);
						break;
				case 1: printf_filtered ( "Warning : Bus selection of bus trace was also changed into \"BBCU\".\n"	);
						break;
				case 2: printf_filtered ( "Warning : Bus selection of bus trace was also changed into \"CPU instruction bus\".\n"  );
						break;
				default:printf_filtered ( " \n");	/* add T.Tazaki 2007/07/12 */
						break;								
			}
			/* Warning display add T.Tazaki 2005/02/07 <<< */

			c33_IcdVer3BreakModeToToolbar();	/* add T.Tazaki 2005/05/18 */

		}
		else
		{
			c33_IcdVer3BusBreak_usage();
			return;
		}
	}
}
/****************************************************************************/
/* Format		 : void c33_IcdVer3BusBreak(void) 		  					*/
/* Arguments																*/
/* Return value  : void 													*/
/****************************************************************************/
void
c33_IcdVer3BusBreak_on()
{
	int iResult = C33_NO_ERR;

	if( c33TargetInfo.iBus_iCntClrSave == C33_COUNT_SAVE )
	{
		if( g_lBusBreakCnt >= 0 )
		{
			/* set update counter */
			c33TargetInfo.ulBus_Count = (unsigned long)g_lBusBreakCnt;
		}
		else
		{
			/* set initial counter */
			c33TargetInfo.ulBus_Count = g_ulBus_Count_org;
		}
	}
	else
	{
		/* set initial counter */
		c33TargetInfo.ulBus_Count = g_ulBus_Count_org;
	}

	 /* Bus break set */
	iResult = c33_SendMessage(C33_ID_BUS_BREAK_SET, 0, 0, 0, 0, 0, 0, 0 );
	if (iResult == C33_NO_ERR)
	{
		iResult = c33_RecvMessage(C33_ID_BUS_BREAK_SET, 0, 0, 0, 0, 0);
	}
	if (iResult != C33_NO_ERR)
	{
		c33_ErrorMsg();
	}
}
/****************************************************************************/
/* Format		 : void c33_IcdVer3BusBreak_off(void) 						*/
/* Arguments																*/
/* Return value  : void 													*/
/****************************************************************************/
void
c33_IcdVer3BusBreak_off()
{
	int iResult = C33_NO_ERR;
	
	c33TargetInfo.ulBus_Count = g_ulBus_Count_org;

	/* Bus break clear */
	iResult = c33_SendMessage(C33_ID_BUS_BREAK_CLR, 0, 0, 0, 0, 0, 0, 0 );
	if (iResult == C33_NO_ERR)
	{
		iResult = c33_RecvMessage(C33_ID_BUS_BREAK_CLR, 0, 0, 0, 0, 0);
	}
	if (iResult != C33_NO_ERR)
	{
		c33_ErrorMsg();
	}
}
/****************************************************************************/
/* Format		 : void c33_IcdVer3BusBreak_usage( void )					*/
/* Arguments																*/
/* Return value  : void 													*/
/****************************************************************************/
void
c33_IcdVer3BusBreak_usage( void )
{
	puts_filtered( "Usage : c33 obb ( On chip bus break )\n");
	puts_filtered( "  c33 obb <mode> <bus select> <BBCU bus master select> <trig/break> <r/w condition>\n");
	puts_filtered( "          <inst/data> <access> <int> <int level> <data compare> <address> <address mask>\n");
	puts_filtered( "          <data> <data mask> <counter> <counter clr/save>\n\n");
	puts_filtered( "          mode:\"on\"  bus break mode on\n");
	puts_filtered( "               \"off\" bus break mode off\n");
	puts_filtered( "          bus select:0:CPU data bus\n");
	puts_filtered( "                     1:CPU instruction bus\n");
	puts_filtered( "                     2:BBCU\n");
	puts_filtered( "          BBCU bus master select:\n");
	puts_filtered( "                     bit3=1 memory access by user.\n");
	puts_filtered( "                     bit2=1 memory access by DMA.\n");
	puts_filtered( "                     bit1=1 memory access by cache.\n");
	puts_filtered( "                     bit0=1 memory access by cpu.\n");
	puts_filtered( "          trig/brea: 0:trigger output only\n");
	puts_filtered( "                     1:break\n");
	puts_filtered( "          r/w condition:\n");
	puts_filtered( "                     \"r\"  Read\n");
	puts_filtered( "                     \"w\"  Write\n");
	puts_filtered( "                     \"rw\" Read or write\n");
	puts_filtered( "          inst/data: 0:instruction\n");
	puts_filtered( "                     1:data\n");
	puts_filtered( "                     2:instruction/data\n");
	puts_filtered( "          access:    bit2=1 WORD access\n");
	puts_filtered( "                     bit1=1 HALF WORD access\n");
	puts_filtered( "                     bit0=1 BYTE access\n");
	puts_filtered( "          int:       0:break by interrupt level is disable\n");
	puts_filtered( "                     1:break by interrupt level is enable\n");
	puts_filtered( "          int level: 0-15\n");
	puts_filtered( "                     interrupt level\n");
	puts_filtered( "          data compare:\n");
	puts_filtered( "                     0:32bit\n");
	puts_filtered( "                     1:high 16bit and low 16bit\n");
	puts_filtered( "          address:   break address  0x0-0xffffffff\n");
	puts_filtered( "          address mask: break address mask bits 0x0-0xffffffff\n");
	puts_filtered( "          data:      break data  0x0-0xffffffff\n");
	puts_filtered( "          data mask: break data mask bits  0x0-0xffffffff\n");
	puts_filtered( "          counter:   break counter( 0x1-0x7fffffff )\n");
	puts_filtered( "          counter clr/save:\n");
	puts_filtered( "                     0:clear break counter by program run\n");
	puts_filtered( "                     1:save break counter by program run\n");

	c33_error (C33_ERR_COMMAND_INVALID_PARAM);
}
/*add T.Tazaki 2005/05/18 >>> */
/****************************************************************************/
/* Format		 : static void c33_IcdVer3BreakModeToToolbar(void) 			*/
/* Arguments																*/
/* Return value  : area/bus break mode change on source window toolbar.		*/
/****************************************************************************/
static void
c33_IcdVer3BreakModeToToolbar()
{
	extern Tcl_Interp *gdbtk_interp;

	if( g_use_windows == 1 )	/* GUI Mode ? */
	{
		if( c33TargetInfo.iArea_Mode == 0 && c33TargetInfo.iBus_Mode == 0 )
		{
			Tcl_SetVar( gdbtk_interp,"c33_break_mode","area:off bus:off",TCL_GLOBAL_ONLY );
		}
		if( c33TargetInfo.iArea_Mode == 1 && c33TargetInfo.iBus_Mode == 0 )
		{
			Tcl_SetVar( gdbtk_interp,"c33_break_mode","area:on  bus:off",TCL_GLOBAL_ONLY );
		}
		if( c33TargetInfo.iArea_Mode == 0 && c33TargetInfo.iBus_Mode == 1 )
		{
			Tcl_SetVar( gdbtk_interp,"c33_break_mode","area:off bus:on ",TCL_GLOBAL_ONLY );
		}
		if( c33TargetInfo.iArea_Mode == 1 && c33TargetInfo.iBus_Mode == 1 )
		{
			Tcl_SetVar( gdbtk_interp,"c33_break_mode","area:on  bus:on ",TCL_GLOBAL_ONLY );
		}

		Tcl_Eval( gdbtk_interp, "c33_breakmode_set" ); 	/* call srcbar.itcl */

	}
}
/*add T.Tazaki 2005/05/18 >>> */
/****************************************************************************/
/* Format		 : void c33_IcdVer3BusTrace(int iParamNum, int iTraceMode,int iBusSelect ,char cBBCUBusSelect, char *pReadWrite, */
/*					 int iInstData, char cAccess, int iBusOutput, int iAddrBusSize, int iCompress, int iUserInput ) */
/* Arguments																*/
/*	int iParamNum		 : Parameter number 								*/
/*	int  iTraceMode 	 : 0:Normal 1:User 10:Do not set bus trace,when GO/STEP */
/*	int  iBusSelect 	 : Bus select 0:HBCU data 1:HBCU inst 2:BBCU		*/
/*	char cBBCUBusSelect  : bit3:USR bit2:DMA bit1:CCU bit0:HBCU 			*/
/*	char *pReadWrite	 : read/write set  "r":read "w":write "rw":read/write */
/*	int  iInstData		 : 0:Inst 1:data 2:both 							*/
/*	char cAccess		 : bit0:BYTE bit1:HALF bit2:WORD					*/
/*	int  iBusOutput 	 : 0:Address bus,1:Data bus 2:both					*/
/*	int  iAddrBusSize	 : bus size 1-4 									*/
/*	int  iCompress		 : 0:NO data/address compressh 1:compress			*/
/*	int  iUserInput 	 : 0:external trace input 1:target CPU tigger input */
/* Return value  : void 													*/
/****************************************************************************/
void
c33_IcdVer3BusTrace(int iParamNum, int iTraceMode,int iBusSelect ,char cBBCUBusSelect, char *pReadWrite,
	 int iInstData, char cAccess, int iBusOutput, int iAddrBusSize, int iCompress, int iUserInput )
{
	char szBuf[50];
	int iLen;
	int iReadWrite = 0;

	/* Check if bus trace function is enable. */
	if( (c33TargetInfo.ucBrkFunc & C33_GDB_STS_BUS_TRACE) == 0 )
	{
		c33_error (C33_ERR_MSG_TARGET_BUS_TRACE_DIS);
	}

	/* check Bus Trace mode OFF : add T.Tazaki 2004/09/29 >>> */
	if( iParamNum == 2 )
	{
		if( iTraceMode == 2 )	/* Trace OFF ? */
		{
			c33TargetInfo.iBust_Mode = 0;
			printf_filtered ( C33_MSG_BUS_TRC_MODE_DISABLE );
		}
		else if( iTraceMode == 10 )	/* "c33 obt 10" Jp[^? */
		{
			c33TargetInfo.iBust_Mode = 10;
		}
		else
		{
			c33_IcdVer3BusTrace_usage();
		}
		return;
	}
	/* check Bus Trace mode OFF : add T.Tazaki 2004/09/29 <<< */


	/* Check number of parameter */
	if ( iParamNum != 11 )
	{
		c33_IcdVer3BusTrace_usage();
		return;
	}

	/* parameter check */
	if( iTraceMode < 0 || iTraceMode > 1)
	{
		c33_IcdVer3BusTrace_usage();
		return;
	}
	if( iBusSelect < 0 || iBusSelect > 2 )
	{
		c33_IcdVer3BusTrace_usage();
		return;
	}
	if( iBusSelect == 1 )	/* change gdb param to farm packet */
	{
		iBusSelect = 2;
	}else if ( iBusSelect == 2 )
	{
		iBusSelect = 1;
	}
			
//	if( cBBCUBusSelect > 0x0f || cBBCUBusSelect < 0 )		/* bug fix T.Tazaki 2004/10/19 */
	if( iBusSelect == 1 )	/* BBCU ? When only BBCU, check BUS MASTER.  add T.Tazaki 2004/10/19 */
	{
   		if( cBBCUBusSelect > 0x0f || cBBCUBusSelect <= 0 )
		{
			c33_IcdVer3BusTrace_usage();
			return;
		}
	}
	/* add T.Tazaki 2004/10/05 >>> */
	if( iBusSelect == 1 && cBBCUBusSelect == 0 )	/* BBCU and BUS MASTER = 0 ? */
	{
		c33_IcdVer3BusTrace_usage();
		return;
	}
	/* add T.Tazaki 2004/10/05 <<< */
	
	if( strcmp(pReadWrite,"r") == 0 )
	{
		iReadWrite = 0;
	}
	else if ( strcmp(pReadWrite,"w") == 0 )
	{
		iReadWrite = 1;
	}
	else if ( strcmp(pReadWrite,"rw") == 0 )
	{
		iReadWrite = 2;
	}
	else
	{
		c33_IcdVer3BusTrace_usage();
		return;
	}

	if( iInstData < 0 || iInstData > 2 )
	{
		c33_IcdVer3BusTrace_usage();
		return;
	}
	if( cAccess < 0 || cAccess > 0x07 )
	{
		c33_IcdVer3BusTrace_usage();
		return;
	}
	if( iBusOutput < 0 || iBusOutput > 2 )
	{
		c33_IcdVer3BusTrace_usage();
		return;
	}
	if( iAddrBusSize < 1 || iAddrBusSize > 4 )
	{
		c33_IcdVer3BusTrace_usage();
		return;
	}
	if( iCompress < 0 || iCompress > 1 )
	{
		c33_IcdVer3BusTrace_usage();
		return;
	}
	if( iUserInput < 0 || iUserInput > 1 )
	{
		c33_IcdVer3BusTrace_usage();
		return;
	}

	c33TargetInfo.iBust_Mode = 1;
	c33TargetInfo.iBust_TraceMode	  = iTraceMode;
//	c33TargetInfo.iBust_BusSelect	  = iBusSelect;	/* bus select is common. T.Tazaki 2004/06/25 */
	c33TargetInfo.iBus_BusSelect	  = iBusSelect;	/* bus select is common. T.Tazaki 2004/06/25 */
	c33TargetInfo.cBust_BBCUBusSelect = cBBCUBusSelect;
	c33TargetInfo.iBust_ReadWrite	  = iReadWrite;
	c33TargetInfo.iBust_InstData	  = iInstData;
	c33TargetInfo.cBust_Access		  = cAccess;
	c33TargetInfo.iBust_BusOutput	  = iBusOutput;
	c33TargetInfo.iBust_AddrBusSize   = iAddrBusSize - 1;
	c33TargetInfo.iBust_Compress	  = iCompress;
	c33TargetInfo.iBust_UserInput	  = iUserInput;

	printf_filtered ( C33_MSG_SET_BUS_TRACE );
	
	/***** Parameter display *****/
			
	/* Trace Mode */
	if( iTraceMode == 0 )
	{
		strcpy( szBuf,"Normal mode" );
	}
	else
	{
		strcpy( szBuf,"User mode" );
	}
	printf_filtered ( C33_MSG_BUS_TRACE_TRACE_MODE, szBuf );

	/* Bus Select */
	switch( iBusSelect )
	{
		case 0: strcpy( szBuf,"CPU data bus" );
				break;
		case 1: strcpy( szBuf,"BBCU" );
				break;
		case 2: strcpy( szBuf,"CPU instruction bus" );
				break;
		default:strcpy( szBuf,"   " );	/* add T.Tazaki 2007/07/12 */
				break;								
	}
	printf_filtered ( C33_MSG_BUS_BREAK_BUS_SELECT, szBuf );

	/* BBCU Bus master Select */
	szBuf[0] = 0;
	if( iBusSelect == 1 )	/* BBCUnH 			add T.Tazaki 2004/11/22 */
	{
		if( cBBCUBusSelect & C33_BBCU_USR )
		{
			strcat( szBuf,"USER," );
		}
		if( cBBCUBusSelect & C33_BBCU_DMA )
		{
			strcat( szBuf,"DMA," );
		}
		if( cBBCUBusSelect & C33_BBCU_CCU )
		{
			strcat( szBuf,"CACHE," );
		}
		if( cBBCUBusSelect & C33_BBCU_HBCU )
		{
			strcat( szBuf,"CPU " );
		}
	}
	else
	{
		strcat( szBuf,"--- " );		/* add T.Tazaki 2004/11/22 */
	}
	iLen = strlen(szBuf);
	szBuf[iLen-1] = 0;	/* comma cut */
	printf_filtered ( C33_MSG_BUS_BREAK_BBCU_SELECT, szBuf );


	/* Read/Write */
	switch( iReadWrite )
	{
		case 0: strcpy( szBuf,"Read" );
				break;
		case 1: strcpy( szBuf,"Write" );
				break;
		case 2: strcpy( szBuf,"Read/Write" );
				break;
		default:strcpy( szBuf,"   " );	/* add T.Tazaki 2007/07/12 */
				break;								
	}
	printf_filtered ( C33_MSG_BUS_TRACE_RW, szBuf );

	/* Instruction / Data */
	switch( iInstData )
	{
		case 0: strcpy( szBuf,"Instruction" );
				break;
		case 1: strcpy( szBuf,"Data" );
				break;
		case 2: strcpy( szBuf,"Instruction/Data" );
				break;
		default:strcpy( szBuf,"   " );	/* add T.Tazaki 2007/07/12 */
				break;								
	}
	printf_filtered ( C33_MSG_BUS_TRACE_INST_DATA, szBuf );
	
	/* Access bus size */
	szBuf[0] = 0;
	if( cAccess & 0x01 )	/* BYTE ? */
	{
		strcat( szBuf,"Byte," );
	}
	if( cAccess & 0x02 )	/* HALF WORD ? */
	{
		strcat( szBuf,"Half word," );
	}
	if( cAccess & 0x04 )	/* WORD ? */
	{
		strcat( szBuf,"Word," );
	}
	iLen = strlen(szBuf);
	szBuf[iLen-1] = 0;	/* comma cut */
	printf_filtered ( C33_MSG_BUS_TRACE_ACCESS_SIZE, szBuf );
	
	
	/* Bus output condition */
	switch( iBusOutput )
	{
		case 0: strcpy( szBuf,"Address bus" );
				break;
		case 1: strcpy( szBuf,"Data bus" );
				break;
		case 2: strcpy( szBuf,"Address bus / Data bus" );
				break;
		default:strcpy( szBuf,"   " );	/* add T.Tazaki 2007/07/12 */
				break;								
	}
	printf_filtered ( C33_MSG_BUS_TRACE_BUS_OUTPUT, szBuf );
	
	/* address bus size */
	printf_filtered ( C33_MSG_BUS_TRACE_BUS_SIZE, iAddrBusSize );
	
	/* compress */
	if( iCompress == 0 )
	{
		strcpy( szBuf,"Disable" );
	}
	else
	{
		strcpy( szBuf,"Enable" );
	}
	printf_filtered ( C33_MSG_BUS_TRACE_COMPRESS, szBuf );
	
	/* user input */
	if( iUserInput == 0 )
	{
		strcpy( szBuf,"TRC IN signal" );
	}
	else
	{
		strcpy( szBuf,"DBT signal" );
	}
	printf_filtered ( C33_MSG_BUS_TRACE_USER_INPUT, szBuf );

	/* Warning display add T.Tazaki 2005/02/07 >>> */
	/* Bus Select */
	switch( iBusSelect )
	{
 	   case 0: printf_filtered ( "Warning : Bus selection of bus break was also changed into \"CPU data bus\".\n"  );
			   break;
	   case 1: printf_filtered ( "Warning : Bus selection of bus break was also changed into \"BBCU\".\n"  );
			   break;
	   case 2: printf_filtered ( "Warning : Bus selection of bus break was also changed into \"CPU instruction bus\".\n"  );
			   break;
	   default:printf_filtered ( "\n" );	/* add T.Tazaki 2007/07/12 */
				break;								
	}
	/* Warning display add T.Tazaki 2005/02/07 <<< */

}
/****************************************************************************/
/* Format		 : void c33_IcdVer3BusTrace_on(void) 	  					*/
/* Arguments																*/
/* Return value  : void 													*/
/****************************************************************************/
void
c33_IcdVer3BusTrace_on()
{
	int iResult = C33_NO_ERR;
	unsigned char ucChar;

	/* check Bus Trace mode ON : add T.Tazaki 2004/09/29 >>> */

	/* ADV Debug Unit : bus trace enable */
	ucChar = 0x59;			/* ݋ */
	iResult = c33_Cmd_ReadWrite(C33_WRITE, 0x402e8, 1, C33_BYTE, &ucChar);
	if (iResult == C33_NO_ERR)
	{
		if( g_iPCTrace == 1 )	/* add T.Tazaki 2005/07/27 */
		{
			ucChar = 0x03;	/* Bus trace on / PC trace on */
		}
		else
		{
			ucChar = 0x02;  /* Bus trace on / PC trace off */
		}
		iResult = c33_Cmd_ReadWrite(C33_WRITE, 0x402ec, 1, C33_BYTE, &ucChar);
	}
	/* check Bus Trace mode ON : add T.Tazaki 2004/09/29 <<< */

	if (iResult == C33_NO_ERR)
	{
		if( c33TargetInfo.iBust_Mode != 10 )	/* Not Secret parameter? 	 add T.Tazaki 2004/09/29 */
		{
			/* Bus trcae set */
			iResult = c33_SendMessage(C33_ID_BUS_TRACE_SET, 0, 0, 0, 0, 0, 0, 0 );
			if (iResult == C33_NO_ERR)
			{
				iResult = c33_RecvMessage(C33_ID_BUS_TRACE_SET, 0, 0, 0, 0, 0);
			}
		}
	}
	if (iResult != C33_NO_ERR)
	{
		c33_ErrorMsg();
	}
}

/* check Bus Trace mode OFF : add T.Tazaki 2004/09/29 >>> */
/****************************************************************************/
/* Format		 : void c33_IcdVer3BusTrace_off(void) 	  					*/
/* Arguments																*/
/* Return value  : void 													*/
/****************************************************************************/
void
c33_IcdVer3BusTrace_off()
{
	int iResult = C33_NO_ERR;
	unsigned char ucChar;

	/* ADV Debug Unit : bus trace disable */
	ucChar = 0x59;			/* ݋ */
	iResult = c33_Cmd_ReadWrite(C33_WRITE, 0x402e8, 1, C33_BYTE, &ucChar);
	if (iResult == C33_NO_ERR)
	{
		if( g_iPCTrace == 1 )	/* add T.Tazaki 2005/07/27 */
		{
			ucChar = 0x01;	/* Bus trace off/ PC trace on */
		}
		else
		{
			ucChar = 0x00;  /* Bus trace off/ PC trace off */
		}
		iResult = c33_Cmd_ReadWrite(C33_WRITE, 0x402ec, 1, C33_BYTE, &ucChar);
	}
	if (iResult != C33_NO_ERR)
	{
		c33_ErrorMsg();
	}
}
/* check Bus Trace mode OFF : add T.Tazaki 2004/09/29 <<< */
/****************************************************************************/
/* Format		 : void c33_IcdVer3BusTrace_usage( void )					*/
/* Arguments																*/
/* Return value  : void 													*/
/****************************************************************************/
void
c33_IcdVer3BusTrace_usage( void )
{
	puts_filtered( "Usage : c33 obt ( On chip bus trace )\n");
	puts_filtered( "  c33 obt <trace mode> <bus select> <BBCU bus master select> <r/w condition> <inst/data>\n");
	puts_filtered( "          <access> <bus output> <address bus size> <compress> <user input>\n\n");
	puts_filtered( "          trace mode:0:normal\n");
	puts_filtered( "                     1:user\n");
	puts_filtered( "                     2:off\n");		/* add T.Tazaki 2004/09/29 */
	puts_filtered( "          bus select:0:CPU data bus\n");
	puts_filtered( "                     1:CPU instruction bus\n");
	puts_filtered( "                     2:BBCU\n");
	puts_filtered( "          BBCU bus master select:\n");
	puts_filtered( "                     bit3=1 memory access by user.\n");
	puts_filtered( "                     bit2=1 memory access by DMA.\n");
	puts_filtered( "                     bit1=1 memory access by cache.\n");
	puts_filtered( "                     bit0=1 memory access by cpu.\n");
	puts_filtered( "          r/w condition:\n");
	puts_filtered( "                     \"r\"  Read\n");
	puts_filtered( "                     \"w\"  Write\n");
	puts_filtered( "                     \"rw\" Read or write\n");
	puts_filtered( "          inst/data: 0:instruction\n");
	puts_filtered( "                     1:data\n");
	puts_filtered( "                     2:instruction/data\n");
	puts_filtered( "          access:    bit2=1 WORD access\n");
	puts_filtered( "                     bit1=1 HALF WORD access\n");
	puts_filtered( "                     bit0=1 BYTE access\n");
	puts_filtered( "          bus output:0:address bus\n");
	puts_filtered( "                     1:data bus\n");
	puts_filtered( "                     2:address/data bus\n");
	puts_filtered( "          address bus size:\n");
	puts_filtered( "                     address bus size (1-4byte)\n");
	puts_filtered( "          compress:  0:address/data compress is disable\n");
	puts_filtered( "                     1:address/data compress is enable\n");
	puts_filtered( "          user input:0:external trace signal(TRC IN)\n");
	puts_filtered( "                     1:debug trace trigger signal(DBT)\n");

	c33_error (C33_ERR_COMMAND_INVALID_PARAM);
}
/****************************************************************************/
/* Format		 : void c33_IcdVer3TraceDisplay(int iParamNum, int iMode, unsigned long ulStartCycle, unsigned long ulEndCycle)    */
/* Arguments																*/
/*	int iParamNum			   : Parameter number							*/
/*	int iMode				   : Output Mode =0:bus trace,1=pc + bus trace	*/
/*	unsigned long ulStartCycle : Start cycle								*/
/*	unsigned long ulEndCycle   : End cycle									*/
/* Return value  : void 													*/
/* Explanation	 : Display onchip bus trcae data.							*/
/****************************************************************************/
void
c33_IcdVer3TraceDisplay(int iParamNum, int iMode, unsigned long ulStartCycle, unsigned long ulEndCycle)
{
	char cErrMsg[C33_TRACE_DATA_LINE_MAX + 1];
	int  iCancel = 0;	
	
	cErrMsg[0] = 0;


	/* Check number of parameter */
	if ( iParamNum == 2 )	/* no range specificationH */
	{
		/* bug fix T.Tazaki 2005/01/12 >>> */
		if( g_iBUS_TraceParam != 0 )	/* Last time, bus trace is range specification? */
		{
			c33IcdTraceData.ulValidBusStart  = C33_ICD_TRACE_INVALID;
		}
	   	g_iBUS_TraceParam = 0;
		if( iMode == 1 )	/* PC+BUS trace ? */
		{
			if( g_iPC_TraceParam != 0 )	/* Last time, PC trace is range specification? */
			{
				c33IcdTraceData.ulValidStart  = C33_ICD_TRACE_INVALID;
			}
			g_iPC_TraceParam = 0;	
		}
		/* bug fix T.Tazaki 2005/01/12 <<< */
		ulStartCycle = c33IcdTraceData.ulIcdTraceBufferSize;
		ulEndCycle = 0;
		
		g_iBusTraceRange = 0;			/* add T.Tazaki 2005/03/24 */
	}
	else if ( iParamNum == 4 )	/* range specificationH */
	{
		/* bug fix T.Tazaki 2005/01/12 >>> */
		if( g_iBUS_TraceParam == 0 )	/* Last time, bus trace is no range specification? */
		{
			c33IcdTraceData.ulValidBusStart  = C33_ICD_TRACE_INVALID;
		}
	   	g_iBUS_TraceParam = 1;
		if( iMode == 1 )	/* PC+BUS traceH */
		{
			if( g_iPC_TraceParam == 0 )	/* Last time, PC trace is no range specification? */
			{
				c33IcdTraceData.ulValidStart  = C33_ICD_TRACE_INVALID;
			}
			g_iPC_TraceParam = 1;	
		}
		/* bug fix T.Tazaki 2005/01/12 <<< */
		
		if (ulStartCycle > c33IcdTraceData.ulIcdTraceBufferSize || ulEndCycle > c33IcdTraceData.ulIcdTraceBufferSize)
		{
			c33_error (C33_ERR_COMMAND_TRC_SIZE);
			return;
		}
		/* Normal mode Cycle check */
		if (ulEndCycle > ulStartCycle)
		{
			c33_error (C33_ERR_COMMAND_CYCLE);
			return;
		}

		g_iBusTraceRange = 1;			/* add T.Tazaki 2005/03/24 */
	}
	else
	{
		c33_error (C33_ERR_COMMAND_PARAM);
		return;
	}

	if( iMode != 0 && iMode != 1 )
	{
		c33_error (C33_ERR_COMMAND_OUTPUT_MODE);
		return;
	}
	c33IcdTraceData.iOutputMode = iMode;

	/* Get trace data size   add T.Tazaki 2005/09/29 >>> */
	if (c33_IcdTraceGetDataSize(&g_ulCycleMax) != C33_NO_ERR)
	{
		puts_filtered (C33_MSG_TARGET_FAILURE);
		return;
	}

	if( g_ulCycleMax == 0 )
	{
		return;
	}
	/* add T.Tazaki 2005/09/29 <<< */

	if( c33TargetInfo.iBust_TraceMode == 1 )  /* User Mode ? */
	{ 
		if( iParamNum == 2 )
		{
			ulStartCycle = g_ulCycleMax - 1;
			ulEndCycle	 = 0;
		}
	}

	/* Get Trace & Output */
	g_iTraceDisplayMode = 1;	/* Trace displayu mode set for c33_IcdTraceMake1Line() LF code add T.Tazaki 2007/07/25 */

	iCancel = c33_IcdBusTraceCommon(NULL, iMode, ulStartCycle, ulEndCycle);

	/* Check cancel */
	if (iCancel == 0 )	/* Not cancel ? */
	{
		/* Print error message */
		if ( cErrMsg[0] != 0 )
		{
			c33_error (cErrMsg);
			return;
		}
	}
	
}
/****************************************************************************/
/* Format		 : void c33_IcdVer3TraceFile(int iParamNum, int iMode, char* pFilename, unsigned long ulStartCycle, unsigned long ulEndCycle)  */
/* Arguments																*/
/*	int iParamNum			   : Parameter number							*/
/*	int iMode				   : Output Mode =0:bus trace,1=pc + bus trace	*/
/*	char* pFilename 		   : File name									*/
/*	unsigned long ulStartCycle : Start cycle								*/
/*	unsigned long ulEndCycle   : End cycle									*/
/* Return value  : void 													*/
/* Explanation	 : Make on chip bus trace data file.						*/
/****************************************************************************/
void
c33_IcdVer3TraceFile(int iParamNum, int iMode, char* pFilename, unsigned long ulStartCycle, unsigned long ulEndCycle)
{
	
	FILE *fpTraceFile = NULL;		/* pointer to parameter file */
	char cErrMsg[C33_TRACE_DATA_LINE_MAX + 1];
	int  iCancel = 0;				/* Cancel flag 2003/03/25 T.Tazaki*/
	cErrMsg[0] = 0;

	/* Check number of parameter */
	if ( iParamNum == 3 )	/* no range specificationH */
	{
		/* bug fix T.Tazaki 2005/01/12 >>> */
		if( g_iBUS_TraceParam != 0 )	/* Last time, bus trace is range specification? */
		{
			c33IcdTraceData.ulValidBusStart  = C33_ICD_TRACE_INVALID;
		}
	   	g_iBUS_TraceParam = 0;
		if( iMode == 1 )	/* PC+BUSg[XH */
		{
			if( g_iPC_TraceParam != 0 )	/* Last time, PC trace is range specification? */
			{
				c33IcdTraceData.ulValidStart  = C33_ICD_TRACE_INVALID;
			}
			g_iPC_TraceParam = 0;	
		}
		/* bug fix T.Tazaki 2005/01/12 <<< */
		ulStartCycle = c33IcdTraceData.ulIcdTraceBufferSize;
		ulEndCycle = 0;
		g_iBusTraceRange = 0;			/* add T.Tazaki 2005/03/24 */
	}
	else if ( iParamNum == 5 )	/* range specificationH */
	{
		/* bug fix T.Tazaki 2005/01/12 >>> */
		if( g_iBUS_TraceParam == 0 )	/* Last time, bus trace is no range specification? */
		{
			c33IcdTraceData.ulValidBusStart  = C33_ICD_TRACE_INVALID;
		}
	   	g_iBUS_TraceParam = 1;
		if( iMode == 1 )	/* PC+BUSg[XH */
		{
			if( g_iPC_TraceParam == 0 )	/* Last time, PC trace is no range specification? */
			{
				c33IcdTraceData.ulValidStart  = C33_ICD_TRACE_INVALID;
			}
			g_iPC_TraceParam = 1;	
		}
		/* bug fix T.Tazaki 2005/01/12 <<< */
		if (ulStartCycle > c33IcdTraceData.ulIcdTraceBufferSize || ulEndCycle > c33IcdTraceData.ulIcdTraceBufferSize)
		{
			c33_error (C33_ERR_COMMAND_TRC_SIZE);
			return;
		}
		if (ulEndCycle > ulStartCycle)
		{
			c33_error (C33_ERR_COMMAND_CYCLE);
			return;
		}
		g_iBusTraceRange = 1;			/* add T.Tazaki 2005/03/24 */
	}
	else
	{
		c33_error (C33_ERR_COMMAND_PARAM);
		return;
	}
	if ( pFilename[0] == 0 )
	{
		c33_error (C33_ERR_FILE_OPEN, pFilename);
		return;
	}

	if( iMode != 0 && iMode != 1 )
	{
		c33_error (C33_ERR_COMMAND_OUTPUT_MODE);
		return;
	}
	c33IcdTraceData.iOutputMode = iMode;

	/* Get trace data size   add T.Tazaki 2005/09/29 >>> */
	if (c33_IcdTraceGetDataSize(&g_ulCycleMax) != C33_NO_ERR)
	{
		puts_filtered (C33_MSG_TARGET_FAILURE);
		return;
	}

	if( g_ulCycleMax == 0 )
	{
		return;
	}
	/* add T.Tazaki 2005/09/29 <<< */

	/* Open parameter file */
	fpTraceFile = fopen(pFilename, "wb");
	if (fpTraceFile == NULL)
	{
		/* file open error */
		c33_error (C33_ERR_FILE_OPEN, pFilename);
		return;
	}

	if( c33TargetInfo.iBust_TraceMode == 1 )  /* User Mode ? */
	{ 
		if( iParamNum == 3 )
		{
			ulStartCycle = g_ulCycleMax - 1;
			ulEndCycle	 = 0;
		}
	}
	/* Get Trace & Output */
	g_iTraceDisplayMode = 0;	/* Trace displayu mode set for c33_IcdTraceMake1Line() LF code  add T.Tazaki 2007/07/25 */

	iCancel = c33_IcdBusTraceCommon(fpTraceFile, iMode, ulStartCycle, ulEndCycle);

	/* file close */
	if (fpTraceFile != NULL)
	{
		fclose(fpTraceFile);
	}

	/* Check cancel */
	if (iCancel == 0 )	/* Not cancel ? */
	{
		/* Print error message */
		if ( cErrMsg[0] != 0 )
		{
			c33_error (cErrMsg);
			return;
		}
	
		puts_filtered (C33_MSG_TRC_FINISHED);
	}
	

}

/****************************************************************************/
/* Format		 : static int c33_IcdBusTraceCommon(FILE *fpTraceFile, int iMode, unsigned long ulStartCycle, unsigned long ulEndCycle)		*/
/* Arguments																*/
/*	FILE* fpTraceFile		   : Trace file pointer, NULL=Display			*/
/*	int iMode				   : Output Mode =0:bus trace,1=pc + bus trace	*/
/*	unsigned long ulStartCycle : Start cycle								*/
/*	unsigned long ulEndCycle   : End cycle									*/
/* Return value  : int iCancel												*/
/* Explanation	 : Bus trace main routine. (for c33 otd, c33 otf common)	*/
/****************************************************************************/
static int
c33_IcdBusTraceCommon(FILE *fpTraceFile, int iMode, unsigned long ulStartCycle, unsigned long ulEndCycle)
{
	extern Tcl_Interp *gdbtk_interp;
	
	unsigned long ulCycleNumbar;	/* Cycle number */
	unsigned long ulTrcDataNumber;	/* trace data number */
	unsigned long l;
	char cTraceStr[C33_TRACE_DATA_LINE_MAX + 1];
	char cTraceStr2[C33_TRACE_DATA_LINE_MAX + 1];
	char cTraceAddr[10];
	char cTraceData[10];
	char cTraceInst[10];		/* add T.Tazaki 2004/09/02 */
	char cTraceUnassem[256];	/* add T.Tazaki 2004/09/09 */

	int  iCancel = 0;	
	int	 iPC_OutputFlag = 0;
	unsigned long ulClock = 0;
	unsigned long ulStartClock = 0;
	unsigned long ulCycle = 0;
	int iFirstFlag = 1;
	static char szAccess[] = { 'r','w' };
	static char szSize[] = { ' ','B','H',' ','W' };
	FILE *fpPcBusTraceFile = NULL;		/* add T.Tazaki 2005/03/22 */

	c33_IcdProcessing(0);/*  timer of "..." is reset */

	g_ulBeforePC = C33_ICD_TRACE_PC_INVALID;			/* add T.Tazaki 2004/10/04 */

	/* read BUS Trace data */
	if (c33_IcdBusTraceGetData() != C33_NO_ERR){
		/* add T.Tazaki 2004/09/13 >>> */
		if( c33TargetInfo.iKeyBreak == 1 )
		{
			c33_EndCancel();	
			return 1;
		}
		/* add T.Tazaki 2004/09/13 <<<> */
		c33_ErrorMsg();
		return 0;
	}

	/* Analysis bus trace */
	if( c33_IcdVer3AnalysisBusTraceData() != C33_NO_ERR){
		/* add T.Tazaki 2004/09/13 >>> */
		if( c33TargetInfo.iKeyBreak == 1 )
		{
			c33_EndCancel();	
			return 1;
		}
		/* add T.Tazaki 2004/09/13 <<<> */
		c33_ErrorMsg();
		return 0;
	}

	c33IcdTraceData.ulValidBusStart = ulStartCycle;

	c33_InitCancel();	/* Cancel initialize */ 
	iCancel = 0;

	g_bCompleteAddressFlag = C33_FALSE;	/* add T.Tazaki 2004/08/30 <<< */

	g_otx_1 = 0;		/* add T.Tazaki 2005/01/31 */

	if( iMode == 1 )
	{
		/*==============================================*/
		/* Output PC Trace and Bus Trace 				*/
		/*==============================================*/

		c33_IcdProcessing(0);/*  timer of "..." is reset */

		g_otx_1 = 1;	/* "c33 otd or otf flag ON : Do not get trace size 2 times. add T.Tazaki 2005/01/31 */
		/* read icd33 trace data */
		if (c33_IcdTraceGetData() != C33_NO_ERR){
			if( c33TargetInfo.iKeyBreak == 1 )
			{
				c33_EndCancel();
				return 1;
			}
			c33_ErrorMsg();
			return 0;
		}

		/* analyze trace data */
		if (c33_IcdTraceAnalyze() != C33_NO_ERR){
			c33_ErrorMsg();
			return iCancel;
		}

		if( iDebugTrace == 2 )	/* PC + BUS trace dump mode ?  */
		{
/* change console --> file output  T.Tazaki 2005/03/22 >>> */
			/* Open parameter file */
			fpPcBusTraceFile = fopen("pcbustrc.dmp", "w");
			if (fpPcBusTraceFile == NULL)
			{
				c33_EndCancel();	/* add T.Tazaki 2004/06/11 */
				/* file open error */
				c33_error (C33_ERR_FILE_OPEN, "pcbustrc.dmp");
				return iCancel;	/* bug fix 2007/07/11 T.Tazaki */
			}
			fprintf (fpPcBusTraceFile,"\nClk     DPCO DST  DTD  DTS   USERIN\n");
			//					0000000  1	 111  XX   00000   1   
			for (l = 0; l < g_ulCycleMax; l++)
			{
				fprintf (fpPcBusTraceFile, "%07d  %d   %d%d%d  %02x   %d%d%d%d%d   %d  \n",
									l,
									(c33IcdTraceData.ucDst[l] & 0x08) >> 3,
									(c33IcdTraceData.ucDst[l] & 0x04) >> 2,
									(c33IcdTraceData.ucDst[l] & 0x02) >> 1,
									(c33IcdTraceData.ucDst[l] & 0x01 ),
									c33IcdTraceData.usBusTraceData[l] >> 8,
									(c33IcdTraceData.usBusTraceData[l] & 0x0010) >> 4,
									(c33IcdTraceData.usBusTraceData[l] & 0x0008) >> 3,
									(c33IcdTraceData.usBusTraceData[l] & 0x0004) >> 2,
									(c33IcdTraceData.usBusTraceData[l] & 0x0002) >> 1,
									(c33IcdTraceData.usBusTraceData[l] & 0x0001),
									(c33IcdTraceData.usBusTraceData[l] & 0x0020) >> 5
								);
			}

			/* file close */
			if (fpPcBusTraceFile != NULL)
			{
				fclose(fpPcBusTraceFile);
			}

#if 0
			printf_filtered ("\nClk     DPCO DST  DTD  DTS   USERIN\n");
			//					0000000  1	 111  XX   00000   1   
			for (l = 0; l < g_ulCycleMax; l++)
			{
				printf_filtered( "%07d  %d   %d%d%d  %02x   %d%d%d%d%d   %d  \n",
									l,
									(c33IcdTraceData.ucDst[l] & 0x08) >> 3,
									(c33IcdTraceData.ucDst[l] & 0x04) >> 2,
									(c33IcdTraceData.ucDst[l] & 0x02) >> 1,
									(c33IcdTraceData.ucDst[l] & 0x01 ),
									c33IcdTraceData.usBusTraceData[l] >> 8,
									(c33IcdTraceData.usBusTraceData[l] & 0x0010) >> 4,
									(c33IcdTraceData.usBusTraceData[l] & 0x0008) >> 3,
									(c33IcdTraceData.usBusTraceData[l] & 0x0004) >> 2,
									(c33IcdTraceData.usBusTraceData[l] & 0x0002) >> 1,
									(c33IcdTraceData.usBusTraceData[l] & 0x0001),
									(c33IcdTraceData.usBusTraceData[l] & 0x0020) >> 5
								);
			}
#endif
/* change console --> file output  T.Tazaki 2005/03/22 <<< */
		}

		if( fpTraceFile == NULL )
		{
			/* trace window clear */ 
			Tcl_Eval( gdbtk_interp, "c33_clear_trace_win" );
		}
		else
		{
			/* display message "Making file." */
			puts_filtered (C33_MSG_TRC_MAKING_FILE);
			c33_IcdProcessing(0);	/* add T.Tazaki 2004/09/13 */
			g_iPercent_First = 1;	/* add T.Tazaki 2004/10/15 */
		}
		
		/* Write trace header */
		if( c33TargetInfo.iBust_TraceMode == 0 )  /* Normal Mode ? */
		{
			sprintf(cTraceStr, C33_MSG_PC_BUS_NORMAL_TRC_HEADER);
		}
		else
		{
			sprintf(cTraceStr, C33_MSG_PC_BUS_USER_TRC_HEADER);
		}
		if( fpTraceFile == NULL )
		{
			Tcl_SetVar( gdbtk_interp,"c33_trace_string",cTraceStr,TCL_GLOBAL_ONLY );
			Tcl_Eval( gdbtk_interp, "c33_trace_out" );
		}
		else
		{
			fprintf( fpTraceFile,"%s\n",cTraceStr);
		}

		/* Valid data? */ 
		if (c33IcdTraceData.ulValidStart != C33_ICD_TRACE_INVALID)
		{
			ulTrcDataNumber = 0;
			ulCycleNumbar = (c33IcdTraceData.ulDataMaxNumber - 1);
			l = ulEndCycle;
			g_ulBeforePC = C33_ICD_TRACE_PC_INVALID;			/* add T.Tazaki 2005/02/15 */

			c33_InitCancel();	/* Cancel initialize   add T.Tazaki 2004/09/13 */ 

			while(l <= ulStartCycle)
			{
				/* Check over run */
				if( ulTrcDataNumber > c33IcdTraceData.ulValidEnd )
				{
					break;
				}
				/* pass over idle cycle */
				if( (c33IcdTraceData.ucDst[ulTrcDataNumber] & C33_ICD_TRACE_DST0_1) != C33_ICD_TRACE_DST_IDLE || c33IcdTraceData.ucDebugIdleDsp != 0)
				{
					if(c33IcdTraceData.ucSearchFlag == 0 || (c33IcdTraceData.ucDst[ulTrcDataNumber] & C33_ICD_TRACE_HIT_PRE_POST) != 0 )
					{
						/* add T.Tazaki 2004/08/30 >>> : check address complete output from first cycle. */
						for( ulClock = 0; ulClock < ulStartClock; ++ulClock )
						{
							if( (c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_ONLY || c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_DATA ) &&	/* add T.Tazaki 2004/07/22 */
								c33IcdTraceData.cAddrComplete[ ulClock ] == C33_ADDRESS_COMPLETE )
							{
								/* Save high 16 bit */
								g_bCompleteAddressFlag = C33_TRUE;
								g_ulSaveCompleteAddress = (c33IcdTraceData.ulAddress[ ulClock ] & 0xffff0000 ) >> 16;
							}
						}
						/* add T.Tazaki 2004/08/30 <<< */

						/* range cycleH */
						if( ulCycleNumbar <= ulStartCycle && ulCycleNumbar >= ulEndCycle )
						{
							if( iPC_OutputFlag == 1 )
							{
								/* Output Bus Trace */
								for( ulClock = ulStartClock; ulClock < g_ulCycleMax; ++ulClock )
								{
									if(ulClock >= ulTrcDataNumber)
									{
										ulStartClock = ulClock;
										break;	/* exit for ulClock */
									}
									
									if( c33TargetInfo.iBust_TraceMode == 0 )  /* Normal Mode ? */
									{
										if(c33IcdTraceData.cBusDataExist[ ulClock ] == 1)
										{
											if(ulClock >= c33IcdTraceData.ulCycleOffset && ulClock < ulTrcDataNumber)
											{
												if( ulClock != g_ulClockBefore )	/* 	add 2004/09/15 T.Tazaki */
												{
													/* PC + Normal Mode */
													c33_IcdBusTraceMake1Line( ulClock, cTraceStr);
													if( fpTraceFile == NULL )
													{
														Tcl_SetVar( gdbtk_interp,"c33_trace_string",cTraceStr,TCL_GLOBAL_ONLY );
														Tcl_Eval( gdbtk_interp, "c33_trace_out" );
													}
													else
													{
														fprintf( fpTraceFile,"%s\n",cTraceStr);
													}
												}
											}
										}
									}
									else
									{
										if(ulClock >= c33IcdTraceData.ulCycleOffset && ulClock < ulTrcDataNumber)
										{
											if( ulClock != g_ulClockBefore )	/* add 2004/09/15 T.Tazaki */
											{
												/* PC + User Mode */
												c33_IcdBusTraceMake1Line( ulClock, cTraceStr);
												if( fpTraceFile == NULL )
												{
													Tcl_SetVar( gdbtk_interp,"c33_trace_string",cTraceStr,TCL_GLOBAL_ONLY );
													Tcl_Eval( gdbtk_interp, "c33_trace_out" );
												}
												else
												{
													fprintf( fpTraceFile,"%s\n",cTraceStr);
												}
											}
										}
									}
								}
							}
							
							/* Outout PC Trace */
							if( iFirstFlag == 1 )
							{
								ulStartClock = ulTrcDataNumber;
								
								/* add T.Tazaki 2005/03/24 >>> */
								/*==========================================================================================================*/
								/* When CLK=0 ---> BUS TRACE <--- ulStartClock, output.														*/
								/*==========================================================================================================*/
								/*	IDLE							<--(1)																	*/
								/*	IDLE		BUSTRACE data				<--Bus Trace Start												*/
								/*	IDLE		BUSTRACE data																				*/
								/*	IDLE		BUSTRACE data																				*/
								/*	IDLE																									*/
								/*	IDLE																									*/
								/*	IDLE							<--(2)																	*/
								/*	NORMAL							<--PC Trace Start														*/
								/*	IDLE																									*/
								/*																											*/
								/*		output bus trace (1)--(2)																			*/

								if( c33TargetInfo.iBust_TraceMode == 0 )  /* Normal Mode ? */
								{
									/*==============================================*/
									/* Normal Mode Output 							*/
									/*==============================================*/
									if( g_iBusTraceRange == 0 )	/* no range? */
									{
										if( g_ulCycleMax < 10000 )	/* no default MAX Over? */
										{
											for( ulClock = 0; ulClock < ulStartClock; ++ulClock )
											{
												if(c33IcdTraceData.cBusDataExist[ ulClock ] == 1)
												{
													/*	check address complete output from first cycle. */
													if( (c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_ONLY || c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_DATA ) &&
														c33IcdTraceData.cAddrComplete[ ulClock ] == C33_ADDRESS_COMPLETE )
													{
														/* save high 16bit */
														g_bCompleteAddressFlag = C33_TRUE;
														g_ulSaveCompleteAddress = (c33IcdTraceData.ulAddress[ ulClock ] & 0xffff0000 ) >> 16;
													}
													if( c33TargetInfo.iBus_BusSelect == 0 || c33TargetInfo.iBus_BusSelect == 2 )
													{
														sprintf(cTraceStr,"                                                   %07d CPU        ",ulClock);
													}
													else
													{
														/* BBCU */
														if( c33IcdTraceData.cBus[ ulClock ] != C33_DMAUSR || c33IcdTraceData.cBus[ ulClock ] == C33_INST)
														{
															sprintf(cTraceStr,"                                                   %07d CPU/CACHE  ",ulClock);
														}
														else
														{
															sprintf(cTraceStr,"                                                   %07d DMA/USR    ",ulClock);
														}
													}
													/* make address and data string */
													c33_IcdBusMakeAddrData( ulClock, cTraceAddr, cTraceData, cTraceInst, cTraceUnassem );
													
													if(c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_DATA )
													{
														strcat( cTraceStr,cTraceAddr );
														strcat( cTraceStr,cTraceData );
													}
													else if(c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_ONLY )
													{
														strcat( cTraceStr,cTraceAddr );
														strcat( cTraceStr,"           " );
													}
													else
													{
														strcat( cTraceStr,"         " );
														strcat( cTraceStr,cTraceData );
													}
													
													strcat( cTraceStr,cTraceInst );	
													
													sprintf(cTraceStr2,"%c  %c   %d",
																		szAccess[c33IcdTraceData.cAccess[ ulClock ]],	/* r or w */
																		szSize[c33IcdTraceData.cSize[ ulClock ]],		/* B or H or W */
																		c33IcdTraceData.cUserIn[ ulClock ]	);
													
													strcat( cTraceStr,cTraceStr2 );
													
													if( fpTraceFile == NULL )
													{
														Tcl_SetVar( gdbtk_interp,"c33_trace_string",cTraceStr,TCL_GLOBAL_ONLY );
														Tcl_Eval( gdbtk_interp, "c33_trace_out" );
													}
													else
													{
														fprintf( fpTraceFile,"%s\n",cTraceStr);
													}
												}
											}
										}
									}
								}
								else
								{
									/*==============================================*/
									/* User   Mode Output 							*/
									/*==============================================*/
									
									if( g_iBusTraceRange == 0 )	/* no range? */
									{
										if( g_ulCycleMax < 10000 )	/* no default MAX Over?  */
										{
											for( ulClock = 0; ulClock < ulStartClock; ++ulClock )
											{
												sprintf( cTraceStr,"                                                   %07d %d%d%d%d%d%d%d%d   %d%d%d%d%d   %d     ",
																						 ulClock,
																						 ((c33IcdTraceData.usBusTraceData[ulClock] >> 8) & 0x80) >> 7,
																						 ((c33IcdTraceData.usBusTraceData[ulClock] >> 8) & 0x40) >> 6,
																						 ((c33IcdTraceData.usBusTraceData[ulClock] >> 8) & 0x20) >> 5,
																						 ((c33IcdTraceData.usBusTraceData[ulClock] >> 8) & 0x10) >> 4,
																						 ((c33IcdTraceData.usBusTraceData[ulClock] >> 8) & 0x08) >> 3,
																						 ((c33IcdTraceData.usBusTraceData[ulClock] >> 8) & 0x04) >> 2,
																						 ((c33IcdTraceData.usBusTraceData[ulClock] >> 8) & 0x02) >> 1,
																						  (c33IcdTraceData.usBusTraceData[ulClock] >> 8) & 0x01,
																						 
																						 (c33IcdTraceData.usBusTraceData[ulClock] & 0x10) >> 4,
																						 (c33IcdTraceData.usBusTraceData[ulClock] & 0x08) >> 3,
																						 (c33IcdTraceData.usBusTraceData[ulClock] & 0x04) >> 2,
																						 (c33IcdTraceData.usBusTraceData[ulClock] & 0x02) >> 1,
																						 (c33IcdTraceData.usBusTraceData[ulClock] & 0x01),
																						 
																						 (c33IcdTraceData.usBusTraceData[ulClock] & 0x20) >> 5 );
												if( fpTraceFile == NULL )
												{
													Tcl_SetVar( gdbtk_interp,"c33_trace_string",cTraceStr,TCL_GLOBAL_ONLY );
													Tcl_Eval( gdbtk_interp, "c33_trace_out" );
												}
												else
												{
													fprintf( fpTraceFile,"%s\n",cTraceStr);
												}
											}
										}
									}
								}
								/* add T.Tazaki 2005/03/24 <<< */
								iFirstFlag = 0;
							}
							g_iPC_Bus_Trace = 1;
							c33_IcdTraceMake1Line(ulTrcDataNumber, ulCycleNumbar, cTraceStr);
							g_iPC_Bus_Trace = 0;
							if( cTraceStr[0] != 0 )	/* same PC address continue?	add T.Tazaki 2005/02/22 */
							{
								if( fpTraceFile == NULL )
								{
									Tcl_SetVar( gdbtk_interp,"c33_trace_string",cTraceStr,TCL_GLOBAL_ONLY );
									Tcl_Eval( gdbtk_interp, "c33_trace_out" );
								}
								else
								{
									fprintf( fpTraceFile,"%s\n",cTraceStr);
									c33_IcdProcessing(8000);
								}
								iPC_OutputFlag = 1;		/* first PC Trace data output */
							}
							l++;

#if 0		/* del T.Tazaki 2005/03/24 */
							/* Check cancel  speed up	add T.Tazaki 2004/11/29 >>> */
							if (c33TargetInfo.iKeyBreak == 1 || c33_CheckCancel( 100 ) == 1 ) /* add T.Tazaki 2003/07/10 */
							{
								/* Cancel */
								iCancel = 1;
								printf_filtered( C33_MSG_TARGET_CANCEL);
								break;
							}
							/* Check cancel  speed up	add T.Tazaki 2004/11/29 >>> */
#endif
						}
					}
					ulCycleNumbar--;
				}
				ulTrcDataNumber++;

//#if 0		/* del T.Tazaki 2004/11/29 speed up */ /* add T.Tazaki 2005/03/24 */
				/* Check cancel */
				if (c33TargetInfo.iKeyBreak == 1 || c33_CheckCancel( 1000 ) == 1 ) /* add T.Tazaki 2003/07/10 */
				{
					/* Cancel */
					iCancel = 1;
					printf_filtered( C33_MSG_TARGET_CANCEL);
					break;
				}
//#endif
			}
		}
	}
	else
	{
		/*==============================================*/
		/* Output Bus Trace 							*/
		/*==============================================*/

		if( fpTraceFile == NULL )
		{
			/* trace window clear */ 
			Tcl_Eval( gdbtk_interp, "c33_clear_trace_win" );
		}
		else
		{
			/* display message "Making file." */
			puts_filtered (C33_MSG_TRC_MAKING_FILE);
		}

		if( c33TargetInfo.iBust_TraceMode == 0 )  /* Normal Mode ? */
		{

			/*==============================================*/
			/* Normal Mode Output 							*/
			/*==============================================*/
			/* Write bus trace header */
			strcpy(cTraceStr, C33_MSG_BUS_NORMAL_TRC_HEADER);

			if( fpTraceFile == NULL )
			{
				Tcl_SetVar( gdbtk_interp,"c33_trace_string",cTraceStr,TCL_GLOBAL_ONLY );
				Tcl_Eval( gdbtk_interp, "c33_trace_out" );
			}
			else
			{
				fprintf( fpTraceFile,"%s\n",cTraceStr);
			}

			//	Output Format
			//		   Cycle   Clk	   BM	Address  Data	  I/D RW SZ UIN Unassemble
			//		   0000029 0000006 CPU			 00000000  i  r  W	 0	ld.w	 %r1, %r2
			//																ld.w	 %r1, %ahr

			if( c33IcdTraceData.ulBusTraceCycle > 1 )
			{
				ulCycle = c33IcdTraceData.ulBusTraceCycle - 1;
			}

			c33_InitCancel();	/* Cancel initialize   add T.Tazaki 2004/09/13 */ 
			c33_IcdProcessing(0);/*  timer of "..." is reset */
			g_iPercent_First = 1;	/* add T.Tazaki 2004/10/15 */
			
			for( ulClock = 0; ulClock < g_ulCycleMax; ++ulClock )
			{
				if(c33IcdTraceData.cBusDataExist[ ulClock ] == 1)
				{
					/* add T.Tazaki 2004/08/30 >>> : check address complete output from first cycle. */
					if( (c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_ONLY || c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_DATA ) &&	/* add T.Tazaki 2004/07/22 */
						c33IcdTraceData.cAddrComplete[ ulClock ] == C33_ADDRESS_COMPLETE )
					{
						/* save high 16 bit */
						g_bCompleteAddressFlag = C33_TRUE;
						g_ulSaveCompleteAddress = (c33IcdTraceData.ulAddress[ ulClock ] & 0xffff0000 ) >> 16;
					}
					/* add T.Tazaki 2004/08/30 <<< */
					if( ulCycle <= ulStartCycle && ulCycle >= ulEndCycle)
					{
						if( c33TargetInfo.iBus_BusSelect == 0 || c33TargetInfo.iBus_BusSelect == 2 )
						{
							sprintf(cTraceStr,"   %07d %07d CPU       ",ulCycle,ulClock);
						}
						else
						{
							/* BBCU */
							if( c33IcdTraceData.cBus[ ulClock ] != C33_DMAUSR || c33IcdTraceData.cBus[ ulClock ] == C33_INST)
							{
								sprintf(cTraceStr,"   %07d %07d CPU/CACHE ",ulCycle,ulClock);
							}
							else
							{
								sprintf(cTraceStr,"   %07d %07d DMA/USR   ",ulCycle,ulClock);
							}
						}
						/* make address and data string */
						c33_IcdBusMakeAddrData( ulClock, cTraceAddr, cTraceData, cTraceInst, cTraceUnassem );	/* change T.Tazaki 2004/09/09 */
						
						if(c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_DATA )
						{
							strcat( cTraceStr,cTraceAddr );
							strcat( cTraceStr,cTraceData );
						}
						else if(c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_ONLY )
						{
							strcat( cTraceStr,cTraceAddr );
							strcat( cTraceStr,"           " );
						}
						else
						{
							strcat( cTraceStr,"         " );
							strcat( cTraceStr,cTraceData );
						}
						
						strcat( cTraceStr,cTraceInst );				/* add I/D data T.Tazaki 2004/09/02 */
						
						sprintf(cTraceStr2,"%c  %c   %d  ",
											szAccess[c33IcdTraceData.cAccess[ ulClock ]],	/* r or w */
											szSize[c33IcdTraceData.cSize[ ulClock ]],		/* B or H or W */
											c33IcdTraceData.cUserIn[ ulClock ]	);
						
						strcat( cTraceStr,cTraceStr2 );
						strcat( cTraceStr,cTraceUnassem );			/* add Unassem T.Tazaki 2004/09/09 */
						
						if( fpTraceFile == NULL )
						{
							Tcl_SetVar( gdbtk_interp,"c33_trace_string",cTraceStr,TCL_GLOBAL_ONLY );
							Tcl_Eval( gdbtk_interp, "c33_trace_out" );
						}
						else
						{
							fprintf( fpTraceFile,"%s\n",cTraceStr);
						}

						/* Check cancel */
						if (c33TargetInfo.iKeyBreak == 1 || c33_CheckCancel( 500 ) == 1 ) 
						{
							/* Cancel */
							iCancel = 1;
							printf_filtered( C33_MSG_TARGET_CANCEL);
							break;
						}
					}

					--ulCycle;
				}

				if( fpTraceFile != NULL )
				{
					c33_IcdProcessing(3000);
//					c33_Percent_Display(g_ulCycleMax, ulClock);		/* percentage display T.Tazaki 2004/10/15 */
				}
			}
		}
		else
		{
			/*==============================================*/
			/* User   Mode Output 							*/
			/*==============================================*/

			/* Write bus trace header */
			strcpy(cTraceStr, C33_MSG_BUS_USER_TRC_HEADER);

			if( fpTraceFile == NULL )
			{
				Tcl_SetVar( gdbtk_interp,"c33_trace_string",cTraceStr,TCL_GLOBAL_ONLY );
				Tcl_Eval( gdbtk_interp, "c33_trace_out" );
			}
			else
			{
				fprintf( fpTraceFile,"%s\n",cTraceStr);
			}

			c33_InitCancel();	/* Cancel initialize   add T.Tazaki 2004/09/13 */ 
			c33_IcdProcessing(0);/*  timer of "..." is reset */
			g_iPercent_First = 1;	/* add T.Tazaki 2004/10/15 */

			//	Output Format
			//	   Cycle   Clk		DTD 	  DTS	 UIN
			//	   0000074 0000000	00000000  00000   1
			//	   0000073 0000001	00010000  00000   1
			//	   0000072 0000002	00000000  10010   1

//			for (ulCycle = g_ulCycleMax - 1; ulCycle >= 0 ; ulCycle--)	/*  Change 2007/07/12 T.Tazaki */
			for (ulCycle = g_ulCycleMax - 1;  ; ulCycle--)				/*  Change 2007/07/12 T.Tazaki */
			{
				if( ulCycle <= ulStartCycle && ulCycle >= ulEndCycle)
				{
					sprintf( cTraceStr,"   %07d %07d %d%d%d%d%d%d%d%d  %d%d%d%d%d   %d     ",
															 ulCycle, 
															 g_ulCycleMax - ulCycle - 1,
															 ((c33IcdTraceData.usBusTraceData[g_ulCycleMax - ulCycle - 1] >> 8) & 0x80) >> 7,
															 ((c33IcdTraceData.usBusTraceData[g_ulCycleMax - ulCycle - 1] >> 8) & 0x40) >> 6,
															 ((c33IcdTraceData.usBusTraceData[g_ulCycleMax - ulCycle - 1] >> 8) & 0x20) >> 5,
															 ((c33IcdTraceData.usBusTraceData[g_ulCycleMax - ulCycle - 1] >> 8) & 0x10) >> 4,
															 ((c33IcdTraceData.usBusTraceData[g_ulCycleMax - ulCycle - 1] >> 8) & 0x08) >> 3,
															 ((c33IcdTraceData.usBusTraceData[g_ulCycleMax - ulCycle - 1] >> 8) & 0x04) >> 2,
															 ((c33IcdTraceData.usBusTraceData[g_ulCycleMax - ulCycle - 1] >> 8) & 0x02) >> 1,
															  (c33IcdTraceData.usBusTraceData[g_ulCycleMax - ulCycle - 1] >> 8) & 0x01,
															 
															 (c33IcdTraceData.usBusTraceData[g_ulCycleMax - ulCycle - 1] & 0x10) >> 4,
															 (c33IcdTraceData.usBusTraceData[g_ulCycleMax - ulCycle - 1] & 0x08) >> 3,
															 (c33IcdTraceData.usBusTraceData[g_ulCycleMax - ulCycle - 1] & 0x04) >> 2,
															 (c33IcdTraceData.usBusTraceData[g_ulCycleMax - ulCycle - 1] & 0x02) >> 1,
															 (c33IcdTraceData.usBusTraceData[g_ulCycleMax - ulCycle - 1] & 0x01),
															 
															 (c33IcdTraceData.usBusTraceData[g_ulCycleMax - ulCycle - 1] & 0x20) >> 5 );
					if( fpTraceFile == NULL )
					{
						Tcl_SetVar( gdbtk_interp,"c33_trace_string",cTraceStr,TCL_GLOBAL_ONLY );
						Tcl_Eval( gdbtk_interp, "c33_trace_out" );
					}
					else
					{
						fprintf( fpTraceFile,"%s\n",cTraceStr);
					}

					/* Check cancel */
					if (c33TargetInfo.iKeyBreak == 1 || c33_CheckCancel( 500 ) == 1 ) 
					{
						/* Cancel */
						iCancel = 1;
						printf_filtered( C33_MSG_TARGET_CANCEL);
						break;
					}
				}
				if( ulCycle == 0 )
				{
					break;
				}
				if( fpTraceFile != NULL )
				{
					c33_IcdProcessing(3000);
//					c33_Percent_Display(g_ulCycleMax, g_ulCycleMax - ulClock);		/* percentage display T.Tazaki 2004/10/15 */
				}
			}
		}
	}

	g_otx_1 = 0;		/* add T.Tazaki 2005/01/31 */

	c33_EndCancel();
	
	return iCancel;
}
/****************************************************************************/
/* Format		 : static void c33_IcdBusTraceMake1Line(unsigned long ulClock, char *cTraceStr) */
/* Arguments																*/
/*	unsigned long ulClock		  : Bus trace clock 						*/
/*	char* cTraceStr 			  : Output string pointer					*/
/* Return value  : void 													*/
/* Explanation	 : Make on chip bus trace 1Line.							*/
/****************************************************************************/
static void
c33_IcdBusTraceMake1Line(unsigned long ulClock, char *cTraceStr)
{
	char cTraceStr2[C33_TRACE_DATA_LINE_MAX + 1];
	char cTraceAddr[10];
	char cTraceData[10];
	char cTraceInst[10];		/* add T.Tazaki 2004/09/02 */
	char cTraceUnassem[256];	/* add T.Tazaki 2004/09/09 : dummy */

	static char szAccess[] = { 'r','w' };
	static char szSize[] = { ' ','B','H',' ','W' };
	
	if( c33TargetInfo.iBust_TraceMode == 0 )  /* Normal Mode ? */
	{
		/*==========================================================*/
		/* Make Normal Mode 1Line									*/
		/*==========================================================*/
		
		/* T.Tazaki 2004/09/15 */
		if( g_iPC_Bus_Trace == 1 && g_iPC_Bus_same == 1 )	/* PC + BUS  Same cycle ? */
		{
			if( c33TargetInfo.iBus_BusSelect == 0 || c33TargetInfo.iBus_BusSelect == 2 )
			{
				sprintf(cTraceStr,"CPU        ");
			}
			else
			{
				/* BBCU */
				if( c33IcdTraceData.cBus[ ulClock ] != C33_DMAUSR || c33IcdTraceData.cBus[ ulClock ] == C33_INST)
				{
					sprintf(cTraceStr,"CPU/CACHE  ");
				}
				else
				{
					sprintf(cTraceStr,"DMA/USR    ");
				}
			}
			g_ulClockBefore = ulClock;		/* save cycle number, it is for DONOT output same bus trace cycle. */
											/*	PC trace ------------ 000050 bus trace -------------------- */
											/*						  000050 bus trace	<--DONOT output		*/
											/*						  000062 bus trace						*/
		}
		else
		{
			if( c33TargetInfo.iBus_BusSelect == 0 || c33TargetInfo.iBus_BusSelect == 2 )
			{
				sprintf(cTraceStr,"                                                   %07d CPU        ",
				/* Clk do not display offset. T.Tazaki 2005/03/23 >>> */
				//					ulClock - c33IcdTraceData.ulCycleOffset );
									ulClock );
				/* Clk do not display offset. T.Tazaki 2005/03/23 <<< */
			}
			else
			{
				/* BBCU */
				if( c33IcdTraceData.cBus[ ulClock ] != C33_DMAUSR || c33IcdTraceData.cBus[ ulClock ] == C33_INST)
				{
					sprintf(cTraceStr,"                                                   %07d CPU/CACHE  ",
					/* Clk do not display offset. T.Tazaki 2005/03/23 >>> */
					//				ulClock - c33IcdTraceData.ulCycleOffset );
									ulClock  );
					/* Clk do not display offset. T.Tazaki 2005/03/23 <<< */
				}
				else
				{
					/* Clk do not display offset. T.Tazaki 2005/03/23 >>> */
					sprintf(cTraceStr,"                                                   %07d DMA/USR    ",
					//				ulClock - c33IcdTraceData.ulCycleOffset );
									ulClock );
					/* Clk do not display offset. T.Tazaki 2005/03/23 <<< */
				}
			}
		}
		
		/* make address and data string */
//		c33_IcdBusMakeAddrData( ulClock, cTraceAddr, cTraceData );
		c33_IcdBusMakeAddrData( ulClock, cTraceAddr, cTraceData, cTraceInst, cTraceUnassem );	/* change T.Tazaki 2004/09/09 */

		if(c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_DATA )
		{
			strcat( cTraceStr,cTraceAddr );
			strcat( cTraceStr,cTraceData );
		}
		else if(c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_ONLY )
		{
			strcat( cTraceStr,cTraceAddr );
			strcat( cTraceStr,"           " );
		}
		else
		{
			strcat( cTraceStr,"         " );
			strcat( cTraceStr,cTraceData );
		}

		strcat( cTraceStr,cTraceInst );				/* add I/D data T.Tazaki 2004/09/02 */
		
		sprintf(cTraceStr2,"%c  %c   %d     ",
							szAccess[c33IcdTraceData.cAccess[ ulClock ]],	/* r or w */
							szSize[c33IcdTraceData.cSize[ ulClock ]],		/* B or H or W */
							c33IcdTraceData.cUserIn[ ulClock ]	);
		strcat( cTraceStr,cTraceStr2 );
	}
	else
	{
		/*==========================================================*/
		/* Make User Mode 1Line.  									*/
		/*==========================================================*/

		/* T.Tazaki 2004/09/15 */
		if( g_iPC_Bus_Trace == 1 && g_iPC_Bus_same == 1 )	/* PC + BUS  Same cycle ? */
		{
			sprintf(cTraceStr,
			"%d%d%d%d%d%d%d%d   %d%d%d%d%d   %d     ",
									((c33IcdTraceData.usBusTraceData[ulClock] >> 8) & 0x80) >> 7,	/* DTD */
									((c33IcdTraceData.usBusTraceData[ulClock] >> 8) & 0x40) >> 6,
									((c33IcdTraceData.usBusTraceData[ulClock] >> 8) & 0x20) >> 5,
									((c33IcdTraceData.usBusTraceData[ulClock] >> 8) & 0x10) >> 4,
									((c33IcdTraceData.usBusTraceData[ulClock] >> 8) & 0x08) >> 3,
									((c33IcdTraceData.usBusTraceData[ulClock] >> 8) & 0x04) >> 2,
									((c33IcdTraceData.usBusTraceData[ulClock] >> 8) & 0x02) >> 1,
									(c33IcdTraceData.usBusTraceData[ulClock] >> 8) & 0x01,
														 
									(c33IcdTraceData.usBusTraceData[ulClock] & 0x10) >> 4,			/* DTS */
									(c33IcdTraceData.usBusTraceData[ulClock] & 0x08) >> 3,
									(c33IcdTraceData.usBusTraceData[ulClock] & 0x04) >> 2,
									(c33IcdTraceData.usBusTraceData[ulClock] & 0x02) >> 1,
									(c33IcdTraceData.usBusTraceData[ulClock] & 0x01),
														
									(c33IcdTraceData.usBusTraceData[ulClock] & 0x20) >> 5 );		/* USERIN */
			g_ulClockBefore = ulClock;		/* save cycle number, it is for DONOT output same bus trace cycle. */
											/*	PC trace ------------ 000050 bus trace -------------------- */
											/*						  000050 bus trace	<--DONOT output		*/
											/*						  000062 bus trace						*/
		}                                   
		else
		{
			sprintf(cTraceStr,
			"                                                   %07d %d%d%d%d%d%d%d%d   %d%d%d%d%d   %d     ",
					/* Clk do not display offset. T.Tazaki 2005/03/23 >>> */
					//				ulClock - c33IcdTraceData.ulCycleOffset,
									ulClock,
					/* Clk do not display offset. T.Tazaki 2005/03/23 <<< */
									((c33IcdTraceData.usBusTraceData[ulClock] >> 8) & 0x80) >> 7,	/* DTD */
									((c33IcdTraceData.usBusTraceData[ulClock] >> 8) & 0x40) >> 6,
									((c33IcdTraceData.usBusTraceData[ulClock] >> 8) & 0x20) >> 5,
									((c33IcdTraceData.usBusTraceData[ulClock] >> 8) & 0x10) >> 4,
									((c33IcdTraceData.usBusTraceData[ulClock] >> 8) & 0x08) >> 3,
									((c33IcdTraceData.usBusTraceData[ulClock] >> 8) & 0x04) >> 2,
									((c33IcdTraceData.usBusTraceData[ulClock] >> 8) & 0x02) >> 1,
									(c33IcdTraceData.usBusTraceData[ulClock] >> 8) & 0x01,
														 
									(c33IcdTraceData.usBusTraceData[ulClock] & 0x10) >> 4,			/* DTS */
									(c33IcdTraceData.usBusTraceData[ulClock] & 0x08) >> 3,
									(c33IcdTraceData.usBusTraceData[ulClock] & 0x04) >> 2,
									(c33IcdTraceData.usBusTraceData[ulClock] & 0x02) >> 1,
									(c33IcdTraceData.usBusTraceData[ulClock] & 0x01),
														
									(c33IcdTraceData.usBusTraceData[ulClock] & 0x20) >> 5 );		/* USERIN */
		}
	}
}
/****************************************************************************/
/* Format		 : static void c33_IcdBusMakeAddrData(unsigned long ulClock, char *cTraceAddr, char *cTraceData, char *cTraceInst, char *cTraceUnassem) */
/* Arguments																*/
/*	unsigned long ulClock		  : Bus trace clock 						*/
/*	char* cTraceAddr			  : Output address string pointer			*/
/*	char* cTraceData			  : Output data    string pointer			*/
/*	char* cTraceInst			  : Output inst    string pointer			*/
/*	char* cTraceUnassem 		  : Output disasm  string pointer			*/
/* Return value  : void 													*/
/* Explanation	 : Make on chip bus trace 1Line.							*/
/****************************************************************************/
static void
c33_IcdBusMakeAddrData(unsigned long ulClock, char *cTraceAddr, char *cTraceData, char *cTraceInst, char *cTraceUnassem)
{
	unsigned short usInst_high;
	unsigned short usInst_low;
	char szBuf[100];
	char szSpace[50];
	
	/*======================================================================================*/
	/* address 																				*/
	/*======================================================================================*/

	/* when getting address length is 0,dusplay space.						*/
	/* when getting address is less than specific address,display "*".		*/

	/* The case of complete address											   */
	/* 1001c584				.cAddrComplete[]=1	.cAddrSame[] = 0  --> 1001c584 */
	/* ******78				.cAddrComplete[]=0	.cAddrSame[] = 1  --> 1001**78 */
	/* ******7c				.cAddrComplete[]=0	.cAddrSame[] = 1  --> 1001**7c */
	/* 1001c580 comnpress	.cAddrComplete[]=0	.cAddrSame[] = 1  --> 1001c580 */
	
	/* The case of no complete address										   */
	/* ****c584				.cAddrComplete[]=0	.cAddrSame[] = 1  --> ++++c584 */
	/* ******78				.cAddrComplete[]=0	.cAddrSame[] = 1  --> ++++**78 */
	/* ******7c				.cAddrComplete[]=0	.cAddrSame[] = 1  --> ++++**7c */
	/* 1001c580 comnpress	.cAddrComplete[]=0	.cAddrSame[] = 1  --> ++++c580 */

	if( (c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_ONLY || c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_DATA ) &&	/* add T.Tazaki 2004/07/22 */
		c33IcdTraceData.cAddrComplete[ ulClock ] == C33_ADDRESS_COMPLETE )
	{
		/* save high 16bit */
		g_bCompleteAddressFlag = C33_TRUE;
		g_ulSaveCompleteAddress = (c33IcdTraceData.ulAddress[ ulClock ] & 0xffff0000 ) >> 16;
	}
	
	if( c33IcdTraceData.cAddressLength[ ulClock ] == 0 )
	{
		/* Get address length is 0. */
		sprintf(cTraceAddr,"         " );
	}
	else if( c33TargetInfo.iBust_AddrBusSize == 0 )	/* address size = 1 byte ? */
	{
		/* Get address length is 1-4. */
		if( c33IcdTraceData.cAddrSame[ ulClock ] == 1 &&
			(c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_ONLY || c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_DATA ) )	/* add T.Tazaki 2004/09/01 */
		{
			sprintf(cTraceAddr,"++++**%02X ",c33IcdTraceData.ulAddress[ ulClock ] & 0x000000ff );
		}
		else
		{
			sprintf(cTraceAddr,"******%02X ",c33IcdTraceData.ulAddress[ ulClock ] & 0x000000ff );
		}
	}
	else if( c33TargetInfo.iBust_AddrBusSize == 1 )	/* address size = 2 byte ? */
	{
		if( c33IcdTraceData.cAddressLength[ ulClock ] == 1 )
		{
			/* Get address length is 1. */
			if( c33IcdTraceData.cAddrSame[ ulClock ] == 1 &&
				(c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_ONLY || c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_DATA ) )	/* add T.Tazaki 2004/09/01 */
			{
				sprintf(cTraceAddr,"++++**%02X ",c33IcdTraceData.ulAddress[ ulClock ] & 0x000000ff);
			}
			else
			{
				sprintf(cTraceAddr,"******%02X ",c33IcdTraceData.ulAddress[ ulClock ] & 0x000000ff );
			}
		}
		else
		{
			/* Get address length is 2-4. */
			if( c33IcdTraceData.cAddrSame[ ulClock ] == 1 &&
				(c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_ONLY || c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_DATA ) )	/* add T.Tazaki 2004/09/01 */
			{
				sprintf(cTraceAddr,"++++%04X ",c33IcdTraceData.ulAddress[ ulClock ] & 0x0000ffff );
			}
			else
			{
				sprintf(cTraceAddr,"****%04X ",c33IcdTraceData.ulAddress[ ulClock ] & 0x0000ffff );
			}
		}
	}
	else if( c33TargetInfo.iBust_AddrBusSize == 2 )	/* address size = 3 byte ? */
	{
		if( c33IcdTraceData.cAddressLength[ ulClock ] == 1 )
		{
			/* Get address length is 1. */
			if( c33IcdTraceData.cAddrSame[ ulClock ] == 1 &&
				(c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_ONLY || c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_DATA ) )	/* add T.Tazaki 2004/09/01 */
			{
				if( g_bCompleteAddressFlag == C33_TRUE )	/* add T.Tazaki 2004/09/01 */
				{
					sprintf(cTraceAddr,"++%02X**%02X ",g_ulSaveCompleteAddress & 0x00ff,c33IcdTraceData.ulAddress[ ulClock ] & 0x000000ff );	/* add T.Tazaki 2004/09/01 */
				}
				else
				{
					sprintf(cTraceAddr,"++++**%02X ",c33IcdTraceData.ulAddress[ ulClock ] & 0x000000ff );
				}
			}
			else
			{
				sprintf(cTraceAddr,"******%02X ",c33IcdTraceData.ulAddress[ ulClock ] & 0x000000ff );
			}
		}
		else if( c33IcdTraceData.cAddressLength[ ulClock ] == 2 )
		{
			/* Get address length is 2. */
			if( c33IcdTraceData.cAddrSame[ ulClock ] == 1 &&
				(c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_ONLY || c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_DATA ) )	/* add T.Tazaki 2004/09/01 */
			{
				if( g_bCompleteAddressFlag == C33_TRUE )	/* add T.Tazaki 2004/09/01 */
				{
					sprintf(cTraceAddr,"++%02X%04X ", g_ulSaveCompleteAddress & 0x00ff,c33IcdTraceData.ulAddress[ ulClock ] & 0x0000ffff );	/* add T.Tazaki 2004/09/01 */
				}
				else
				{
					sprintf(cTraceAddr,"++++%04X ", c33IcdTraceData.ulAddress[ ulClock ] & 0x0000ffff );
				}
			}
			else
			{
				sprintf(cTraceAddr,"****%04X ", c33IcdTraceData.ulAddress[ ulClock ] & 0x0000ffff );
			}
		}
		else 
		{
			/* Get address length is 3-4. */
			if( c33IcdTraceData.cAddrSame[ ulClock ] == 1 &&
				(c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_ONLY || c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_DATA ) )	/* add T.Tazaki 2004/09/01 */
			{
				sprintf(cTraceAddr,"++%06X ", c33IcdTraceData.ulAddress[ ulClock ] & 0x00ffffff );
			}
			else
			{
				sprintf(cTraceAddr,"**%06X ", c33IcdTraceData.ulAddress[ ulClock ] & 0x00ffffff );
			}
		}
	}
	else
	{
		/* address size = 4 byte */
		if( c33IcdTraceData.cAddressLength[ ulClock ] == 1 )
		{
			/* Get address length is 1. */
			if( c33IcdTraceData.cAddrSame[ ulClock ] == 1 &&
				(c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_ONLY || c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_DATA ) )	/* add T.Tazaki 2004/07/22 */
			{
				if( g_bCompleteAddressFlag == C33_TRUE )
				{
					sprintf(cTraceAddr,"%04X**%02X ",g_ulSaveCompleteAddress & 0xffff, c33IcdTraceData.ulAddress[ ulClock ] & 0x000000ff );
				}
				else
				{
					sprintf(cTraceAddr,"++++**%02X ", c33IcdTraceData.ulAddress[ ulClock ] & 0x000000ff );
				}
			}
			else
			{
				sprintf(cTraceAddr,"******%02X ", c33IcdTraceData.ulAddress[ ulClock ] & 0x000000ff );
			}
		}
		else if( c33IcdTraceData.cAddressLength[ ulClock ] == 2 )
		{
			/* Get address length is 2. */
			if( c33IcdTraceData.cAddrSame[ ulClock ] == 1 &&
				(c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_ONLY || c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_DATA ) )	/* add T.Tazaki 2004/07/23 */
			{
				if( g_bCompleteAddressFlag == C33_TRUE )
				{
					sprintf(cTraceAddr,"%04X%04X ",g_ulSaveCompleteAddress & 0xffff, c33IcdTraceData.ulAddress[ ulClock ] & 0x0000ffff );
				}
				else
				{
					sprintf(cTraceAddr,"++++%04X ", c33IcdTraceData.ulAddress[ ulClock ] & 0x0000ffff );
				}
			}
			else
			{
				sprintf(cTraceAddr,"****%04X ", c33IcdTraceData.ulAddress[ ulClock ] & 0x0000ffff );
			}
		}
		else if( c33IcdTraceData.cAddressLength[ ulClock ] == 3 )
		{
			/* Get address length is 3. */
			if( c33IcdTraceData.cAddrSame[ ulClock ] == 1 &&
				(c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_ONLY || c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_DATA ) )	/* add T.Tazaki 2004/07/22 */

			{
				if( g_bCompleteAddressFlag == C33_TRUE )
				{
					sprintf(cTraceAddr,"%04X%04X ", g_ulSaveCompleteAddress & 0xffff, c33IcdTraceData.ulAddress[ ulClock ] & 0x0000ffff );
				}
				else
				{
					sprintf(cTraceAddr,"++%06X ", c33IcdTraceData.ulAddress[ ulClock ] & 0x00ffffff );
				}
			}
			else
			{
				sprintf(cTraceAddr,"**%06X ", c33IcdTraceData.ulAddress[ ulClock ] & 0x00ffffff );
			}
		}
		else
		{
			/* Get address length is 4. */
			if( c33IcdTraceData.cAddrSame[ ulClock ] == 1 &&
				(c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_ONLY || c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_DATA ) )	/* add T.Tazaki 2004/07/22 */

			{
				if( g_bCompleteAddressFlag == C33_TRUE )
				{
					sprintf(cTraceAddr,"%04X%04X ", g_ulSaveCompleteAddress & 0xffff, c33IcdTraceData.ulAddress[ ulClock ] & 0x0000ffff );
				}
				else
				{
					/* Compression coincidence, but no complete address */
//					sprintf(cTraceAddr,"%08x ",c33IcdTraceData.ulAddress[ ulClock ] );					/* del 2004/08/31 T.Tazaki */
					sprintf(cTraceAddr,"++++%04X ",c33IcdTraceData.ulAddress[ ulClock ] & 0x0000ffff ); /* add 2004/08/31 T.Tazaki */
				}
			}
			else
			{
				sprintf(cTraceAddr,"%08X ",c33IcdTraceData.ulAddress[ ulClock ] );
			}
		}
	}
	
	/*======================================================================================*/
	/* data 																				*/
	/*======================================================================================*/
	if( c33IcdTraceData.cDataLength[ ulClock ] == 0 )
	{
		/* add T.Tazaki 2005/01/31 >>> */
		if( c33IcdTraceData.cAddrData[ ulClock ] == C33_ADDRESS_DATA)
		{
			sprintf(cTraceData,"********   " );
		}
		else
		{
		/* add T.Tazaki 2005/01/31 <<< */
			sprintf(cTraceData,"           " );
		}
	}
	else if( c33IcdTraceData.cSize[ ulClock ] == C33_BYTE )
	{
		sprintf(cTraceData,"      %02X   ", c33IcdTraceData.ulData[ ulClock ] & 0x000000ff );
	}
	else if(  c33IcdTraceData.cSize[ ulClock ] == C33_HALF_WORD )
	{
		if( c33IcdTraceData.cDataLength[ ulClock ] == 1 )
		{
			sprintf(cTraceData,"    **%02X   ", c33IcdTraceData.ulData[ ulClock ] & 0x000000ff );
		}
		else
		{
			sprintf(cTraceData,"    %04X   ", c33IcdTraceData.ulData[ ulClock ] & 0x0000ffff );
		}
	}
	else
	{
		if( c33IcdTraceData.cDataLength[ ulClock ] == 1 )
		{
			/* inst data? HBCU(INST) or BBCU(INST) ?	 add T.Tazaki 2004/08/23 */
			if( c33TargetInfo.iBus_BusSelect == 2 || (c33TargetInfo.iBus_BusSelect == 1 && c33IcdTraceData.cBus[ ulClock ] == C33_INST ) )
			{
				/* X***X***    I2 15:12, I1 15:12 */
				sprintf(cTraceData,"%01X***%01X***   ", (c33IcdTraceData.ulData[ ulClock ] & 0xf0000000)>>28, (c33IcdTraceData.ulData[ ulClock ] & 0x0000f000)>>12 );
			}
			else
			{
				sprintf(cTraceData,"******%02X   ",c33IcdTraceData.ulData[ ulClock ] & 0x000000ff );
			}
		}
		else if( c33IcdTraceData.cDataLength[ ulClock ] == 2 )
		{
			/* inst data? HBCU(INST) or BBCU(INST) ? add T.Tazaki 2004/08/23 */
			if( c33TargetInfo.iBus_BusSelect == 2 || (c33TargetInfo.iBus_BusSelect == 1 && c33IcdTraceData.cBus[ ulClock ] == C33_INST ) )
			{
				/* XX**XX**    I2 15:8, I1 15:8 */
				sprintf(cTraceData,"%02X**%02X**   ", (c33IcdTraceData.ulData[ ulClock ] & 0xff000000)>>24, (c33IcdTraceData.ulData[ ulClock ] & 0x0000ff00)>>8 );
			}
			else
			{
				sprintf(cTraceData,"****%04X   ", c33IcdTraceData.ulData[ ulClock ] & 0x0000ffff );
			}
		}
		else if( c33IcdTraceData.cDataLength[ ulClock ] == 3 )
		{
			/* inst data? HBCU(INST) or BBCU(INST) ? add T.Tazaki 2004/08/23 */
			if( c33TargetInfo.iBus_BusSelect == 2 || (c33TargetInfo.iBus_BusSelect == 1 && c33IcdTraceData.cBus[ ulClock ] == C33_INST ) )
			{
				/* XXX*XXX*    I2 15:4, I1 15:4 */
				sprintf(cTraceData,"%03X*%03X*   ", (c33IcdTraceData.ulData[ ulClock ] & 0xfff00000)>>20,(c33IcdTraceData.ulData[ ulClock ] & 0x0000fff0)>>4 );
			}
			else
			{
				sprintf(cTraceData,"**%06X   ", c33IcdTraceData.ulData[ ulClock ] & 0x00ffffff );
			}
		}
		else
		{
			sprintf(cTraceData,"%08X   ",c33IcdTraceData.ulData[ ulClock ] );
		}
	}
	/*======================================================================================*/
	/* I/D	 	inst or data																*/
	/*======================================================================================*/
	if( c33IcdTraceData.cBus[ ulClock ] == C33_INST )
	{
		sprintf(cTraceInst,"i  " );
	}
	else
	{
		sprintf(cTraceInst,"d  " );
	}
	/*======================================================================================*/
	/* Unassemble													add 2004/09/13 T.Tazaki */
	/*======================================================================================*/
	*cTraceUnassem = 0;
	if( c33IcdTraceData.cBus[ ulClock ] == C33_INST )
	{
		if( c33IcdTraceData.cDataLength[ ulClock ] == 4 )
		{
			usInst_high = (c33IcdTraceData.ulData[ ulClock ] & 0xffff0000) >> 16;
			usInst_low	= (c33IcdTraceData.ulData[ ulClock ] & 0x0000ffff);
			g_iTraceMode = 1;	/* output base inst */
			c33_Disasm (usInst_low, szBuf);
			strcpy( szSpace, "                         ");
			szSpace[25 - strlen(szBuf)] = 0;
			sprintf(cTraceUnassem,"%s%s  ",szBuf, szSpace  );
			c33_Disasm (usInst_high,  szBuf);
			g_iTraceMode = 0;
			sprintf(cTraceUnassem,"%s\n                                                              %s",cTraceUnassem,szBuf);
		}
	}


}

/****************************************************************************/
/* Format		 : static int c33_IcdBusTraceReadData(unsigned short *pusDatat, unsigned long ulStartCycle, unsigned long ulReadCycle, unsigned long ulCycleMax) */
/* Arguments																*/
/*	unsigned char* pucDst	   : Data receive pointer						*/
/*	unsigned long ulStartCycle : Start cycle								*/
/*	unsigned long ulReadCycle  : Read cycle 								*/
/*	unsigned long ulCycleMax   : Trace max data size						*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Get bus trace data from ICD V3.							*/
/****************************************************************************/
static int
c33_IcdBusTraceReadData(unsigned short *pusData, unsigned long ulStartCycle, unsigned long ulReadCycle, unsigned long ulCycleMax)
{

	int iResult;

	/* Transmission of a message. */
	iResult = c33_SendMessage(C33_ID_BUS_TRACE_READ_DATA, ulStartCycle, ulReadCycle, 0, 0, 0, 0, 0);

	/* Reception of a message. */
	if (iResult == C33_NO_ERR){
		iResult = c33_RecvMessage(C33_ID_BUS_TRACE_READ_DATA, ulStartCycle, ulReadCycle, ulCycleMax, 0, pusData);
	}

	return iResult;
}

/****************************************************************************/
/* Format		 : static void c33_IcdBusTraceGetData(void) 				*/
/* Arguments	 : void 													*/
/* Return value  : void 													*/
/* Explanation	 : Read bus trace data from ICD V3. 						*/
/****************************************************************************/
static int
c33_IcdBusTraceGetData(void)
{
	unsigned long ulCycleMax = 0;					  /* trace data size from icd33 */
	unsigned long ulReadCycle = C33_COMMAND_BUS_TEXT_SIZE; /* read data size */	/* change 2004/06/07 T.Tazaki */
	unsigned long l;
	FILE *fpBusTraceFile = NULL;
	
	/* Was trace data already read? */	
	if(c33IcdTraceData.ulValidBusStart != C33_ICD_TRACE_INVALID){
		return C33_NO_ERR;
	}

	puts_filtered (C33_MSG_BUS_TRC_READ);

	c33IcdTraceData.ulBusTraceCycle = 0;

	/* Trace data all clear */
	for (l = 0; l < C33_ICD_TRACE_SIZE; l++)
	{
		c33IcdTraceData.usBusTraceData[l] = 0;

		c33IcdTraceData.cBusDataExist[l] = 0;
		c33IcdTraceData.cAddrData[l] = 0;
		c33IcdTraceData.ulAddress[l] = 0;
		c33IcdTraceData.ulData[l]	 = 0;
		c33IcdTraceData.cSize[l]	 = 0;
		c33IcdTraceData.cAccess[l]	 = 0;
		c33IcdTraceData.cBus[l] 	 = 0;
		c33IcdTraceData.cUserIn[l]	 = 0;
		c33IcdTraceData.cAddressLength[l] = 0;
		c33IcdTraceData.cDataLength[l]	  = 0;
		c33IcdTraceData.cAddrComplete[l]  = C33_ADDRESS_NOT_COMPLETE;
		c33IcdTraceData.cAddrSame[l]  = 0;
		
	}

	/* Get trace data size */
	if (c33_IcdTraceGetDataSize(&ulCycleMax) != C33_NO_ERR)
	{
		puts_filtered (C33_MSG_TARGET_FAILURE);
		return C33_ERR;
	}

	/* add T.Tazaki 2004/12/01 >>> */
	if( g_iBUS_TraceParam == 0 )
	{
		if( ulCycleMax > 10000 )
		{
			ulCycleMax = 10000;
		}
	}
	/* add T.Tazaki 2004/12/01 <<< */


	g_ulCycleMax = ulCycleMax;



	c33_InitCancel();	/* Cancel initialize  add T.Tazaki 2004/06/11 */ 


	g_iPercent_First = 1;

	/* Read trace data from icd33 */
	for(l = 0 ; l < ulCycleMax ; l += ulReadCycle)
	{
//		if(ulCycleMax < l + C33_COMMAND_TEXT_SIZE )		/* change 2004/06/07 T.Tazaki */
		if(ulCycleMax < l + C33_COMMAND_BUS_TEXT_SIZE )	/* change 2004/06/07 T.Tazaki */
		{
			ulReadCycle = ulCycleMax - l;
		}
		if (c33_IcdBusTraceReadData(c33IcdTraceData.usBusTraceData, l ,ulReadCycle ,ulCycleMax) != C33_NO_ERR)
		{
			puts_filtered (C33_MSG_TARGET_FAILURE);
			return C33_ERR;
		}
		c33_IcdProcessing(3000);

//	   	c33_Percent_Display(g_ulCycleMax, l);	/* percentage display T.Tazaki 2004/10/15 */

		/* Check cancel */
		if (c33TargetInfo.iKeyBreak == 1 || c33_CheckCancel( 20 ) == 1 ) /* 10 --> 20 add T.Tazaki 2004/11/29 */
		{
			/* Cancel */
			printf_filtered( C33_MSG_TARGET_CANCEL);
//			c33_EndCancel();	/* del T.Tazaki 2004/09/13 */
			return C33_ERR;
		}
	}

	puts_filtered (C33_MSG_TRC_FINISHED);

	if( iDebugTrace == 1 )	/* trace dump mode ?  */
	{
		puts_filtered ("\nCycle    DTD    DTS     USERIN\n");
		
		for (l = 0; l < g_ulCycleMax; l++)
		{
			//				0000000  00 	00000	  0
			printf_filtered( "%07d  %02x     %d%d%d%d%d     %d\n",
					l,
					c33IcdTraceData.usBusTraceData[l] >> 8,
					(c33IcdTraceData.usBusTraceData[l] & 0x0010) >> 4,
					(c33IcdTraceData.usBusTraceData[l] & 0x0008) >> 3,
					(c33IcdTraceData.usBusTraceData[l] & 0x0004) >> 2,
					(c33IcdTraceData.usBusTraceData[l] & 0x0002) >> 1,
					(c33IcdTraceData.usBusTraceData[l] & 0x0001),
					(c33IcdTraceData.usBusTraceData[l] & 0x0020) >> 5);
		}
	}
	/* add Bus Trace dump output file : 2004/05/17 T.Tazaki >>> */
	else if( iDebugTrace == 3 )
	{
		/* Open parameter file */
		fpBusTraceFile = fopen("bustrc.dmp", "w");
		if (fpBusTraceFile == NULL)
		{
			c33_EndCancel();	/* add T.Tazaki 2004/06/11 */
			/* file open error */
			c33_error (C33_ERR_FILE_OPEN, "bustrc.dmp");
			return C33_ERR;	/* bug fix 2007/07/11 T.Tazaki */
		}

		fprintf( fpBusTraceFile,"Number DTD  DTS    USERIN\n");
	//						 "00000: XX   11111    1
		for (l = 0 ; l < g_ulCycleMax ; l++)
		{
				fprintf( fpBusTraceFile,"%5d: %02x   %d%d%d%d%d    %d\n",
					l, 
					(c33IcdTraceData.usBusTraceData[l] >> 8),
					(c33IcdTraceData.usBusTraceData[l] & 0x0010) >> 4,
					(c33IcdTraceData.usBusTraceData[l] & 0x0008) >> 3,
					(c33IcdTraceData.usBusTraceData[l] & 0x0004) >> 2,
					(c33IcdTraceData.usBusTraceData[l] & 0x0002) >> 1,
					(c33IcdTraceData.usBusTraceData[l] & 0x0001),
					(c33IcdTraceData.usBusTraceData[l] & 0x0020) >> 5);
		}

		/* file close */
		if (fpBusTraceFile != NULL)
		{
			fclose(fpBusTraceFile);
		}
	}
	/* add Bus Trace dump output file : 2004/05/17 T.Tazaki <<< */

	c33_EndCancel();

	return C33_NO_ERR;
}

/****************************************************************************/
/* Format		 : static int c33_IcdVer3AnalysisBusTraceData(void)			*/
/* Arguments																*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Analysis and Make bus trace data.						*/
/* Output : 				 												*/
/*				c33IcdTraceData.cAddrData[ ulCycle ]						*/
/*				c33IcdTraceData.ulAddress[ ulCycle ]						*/
/*				c33IcdTraceData.ulData[ ulCycle ]	 						*/
/*				c33IcdTraceData.cSize[ ulCycle ]	 						*/
/*				c33IcdTraceData.cAccess[ ulCycle ]	 						*/
/*				c33IcdTraceData.cBus[ ulCycle ] 	 						*/
/*				c33IcdTraceData.cUserIn[ ulCycle ]	 						*/
/*				c33IcdTraceData.cAddressLength[ ulCycle ]					*/
/*				c33IcdTraceData.cDataLength[ ulCycle ]	  					*/
/*				c33IcdTraceData.cAddrComplete[ ulCycle ]  					*/
/*				c33IcdTraceData.cAddrSame[ ulCycle ]  						*/
/****************************************************************************/
static int
c33_IcdVer3AnalysisBusTraceData()
{
	unsigned long ulAddress    = 0;
	unsigned long ulOldAddress = 0;
	unsigned long ulData	   = 0;
	unsigned long ulStartCycle = 0;
	unsigned long i = 0;
	unsigned char ucBusMode = C33_IDLE;
	int		bAddressGetMode = C33_FALSE;
	int		bDataGetMode	= C33_FALSE;
	int		iBusAddressCount = 0;
	int		iBusDataCount	 = 0;
	int		bAddressAnalysis_flag = C33_FALSE;
	int 	bAddressCompress = C33_FALSE;

	/* Was trace data already read? */	
	if(c33IcdTraceData.ulValidBusStart != C33_ICD_TRACE_INVALID){
		return C33_NO_ERR;
	}

	puts_filtered (C33_MSG_BUS_TRC_ANALYZING);

	g_iAddrCompeteIndex = -1; /* initialize compelete address cycle position. add T.Tazaki 2004/10/04 */
	
	c33_InitCancel();
	c33_IcdProcessing(0);
	g_iPercent_First = 1;

	if ( g_ulCycleMax > 0 )
	{
		if ( c33TargetInfo.iBust_TraceMode == 0 )	/* Trace Mode == Normal */
		{
			/*==============================================================*/
			/* Normal Mode													*/
			/*==============================================================*/

//			if( c33TargetInfo.iBust_BusSelect == 1 )	/* BBCU ? */
			if( c33TargetInfo.iBus_BusSelect == 1 )		/* T.Tazaki 2004/06/25 */
			{
				/*==============================================================*/
				/* BBCU Trace 													*/
				/*==============================================================*/
				
				/* skip until start status */
				
				for (i = 0 ; i < g_ulCycleMax ; i++)
				{
					if( ( (c33IcdTraceData.usBusTraceData[i] & 0x1f) == DMAUSR_START_B_WR ) || 
						( (c33IcdTraceData.usBusTraceData[i] & 0x1f) == DMAUSR_START_B_RD ) || 
						( (c33IcdTraceData.usBusTraceData[i] & 0x1f) == DMAUSR_START_H_WR ) || 
						( (c33IcdTraceData.usBusTraceData[i] & 0x1f) == DMAUSR_START_H_RD ) || 
						( (c33IcdTraceData.usBusTraceData[i] & 0x1f) == DMAUSR_START_W_WR ) || 
						( (c33IcdTraceData.usBusTraceData[i] & 0x1f) == DMAUSR_START_W_RD ) ||
						( (c33IcdTraceData.usBusTraceData[i] & 0x1f) == DATA_START_B_WR   ) || 
						( (c33IcdTraceData.usBusTraceData[i] & 0x1f) == DATA_START_B_RD   ) || 
						( (c33IcdTraceData.usBusTraceData[i] & 0x1f) == DATA_START_H_WR   ) || 
						( (c33IcdTraceData.usBusTraceData[i] & 0x1f) == DATA_START_H_RD   ) || 
						( (c33IcdTraceData.usBusTraceData[i] & 0x1f) == DATA_START_W_WR   ) || 
						( (c33IcdTraceData.usBusTraceData[i] & 0x1f) == DATA_START_W_RD   ) ||
						( (c33IcdTraceData.usBusTraceData[i] & 0x1f) == INST_START		  ) )
					{
						break;
					}
				}

				/* Analysis Trace Data */
				
				for ( ; i < g_ulCycleMax; ++i )
				{

					switch ( c33IcdTraceData.usBusTraceData[i] & 0x1f )
					{
					case C33_IDLE:
							if( bAddressGetMode == C33_TRUE )
							{
								/* del 2005/01/31 T.Tazaki */
//								c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, iBusAddressCount, 0, C33_ADDRESS_NOT_COMPLETE );
								/* add	bug fix 2005/01/31 T.Tazaki >>> */
			   					switch( c33TargetInfo.iBust_BusOutput )
			   					{							
			   					case 0: c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, iBusAddressCount, 0, C33_ADDRESS_NOT_COMPLETE );
			   							break;
								case 2:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, 0,	   iBusAddressCount, 0, C33_ADDRESS_NOT_COMPLETE );
			   							break;
								}
								/* bug fix 2005/01/31 T.Tazaki <<< */
							}
							else if( bDataGetMode == C33_TRUE )
							{
								if( bAddressCompress == C33_TRUE )
								{
									/* del bug fix 2005/01/31 T.Tazaki */
//									c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPRESS );
									/* add	bug fix 2005/01/31 T.Tazaki >>> */
				   					switch( c33TargetInfo.iBust_BusOutput )
				   					{							
				   					case 0: c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPRESS );
				   							break;
									case 1:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY,			0, ulData, 0, iBusDataCount, C33_ADDRESS_COMPRESS );
				   							break;
									case 2:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPRESS );
				   							break;
									}
									/* bug fix 2005/01/31 T.Tazaki <<< */
									bAddressCompress = C33_FALSE;
								}
								else
								{
									/* del bug fix 2005/01/31 T.Tazaki */
//									c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPLETE );
									/* add	bug fix 2005/01/31 T.Tazaki >>> */
				   					switch( c33TargetInfo.iBust_BusOutput )
				   					{							
				   					case 0: c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPLETE );
				   							break;
									case 1:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY,	0,		   ulData, 0, iBusDataCount, C33_ADDRESS_COMPLETE );
				   							break;
									case 2:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPLETE );
				   							break;
									}
									/* bug fix 2005/01/31 T.Tazaki <<< */

								}
								ulOldAddress = ulAddress & 0xffff0000;
							}
							if( bAddressGetMode == C33_TRUE || bDataGetMode == C33_TRUE )
							{
								ulAddress = 0;
								ulData = 0;
								bAddressGetMode = C33_FALSE;
								bDataGetMode	= C33_FALSE;
								iBusAddressCount = 0;
								iBusDataCount = 0;
								bAddressAnalysis_flag = C33_FALSE;
							}
							break;

					case DMAUSR_START_B_WR:
					case DMAUSR_START_B_RD:
					case DMAUSR_START_H_WR:
					case DMAUSR_START_H_RD:
					case DMAUSR_START_W_WR:
					case DMAUSR_START_W_RD:
					case DATA_START_B_WR:
					case DATA_START_B_RD:
					case DATA_START_H_WR:
					case DATA_START_H_RD:
					case DATA_START_W_WR:
					case DATA_START_W_RD:
					case INST_START:

							/* start */
							
							if( bAddressGetMode == C33_TRUE )
							{
								/* del 2005/01/31 T.Tazaki */
//								c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, iBusAddressCount, 0, C33_ADDRESS_NOT_COMPLETE );
								/* add	bug fix 2005/01/31 T.Tazaki >>> */
			   					switch( c33TargetInfo.iBust_BusOutput )
			   					{							
			   					case 0: c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, 0, iBusAddressCount, 0, C33_ADDRESS_NOT_COMPLETE );
			   							break;
								case 2:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, 0, iBusAddressCount, 0, C33_ADDRESS_NOT_COMPLETE );
			   							break;
								}
								/* bug fix 2005/01/31 T.Tazaki <<< */
							}
							else if( bDataGetMode == C33_TRUE )
							{
								if( bAddressCompress == C33_TRUE )
								{
									/* del bug fix 2005/01/31 T.Tazaki */
//									c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPRESS );
									/* add	bug fix 2005/01/31 T.Tazaki >>> */
				   					switch( c33TargetInfo.iBust_BusOutput )
				   					{							
				   					case 0: c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPRESS );
				   							break;
									case 1:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY,	0,		   ulData, 0, iBusDataCount, C33_ADDRESS_COMPRESS );
				   							break;
									case 2:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPRESS );
				   							break;
									}
									/* bug fix 2005/01/31 T.Tazaki <<< */
									bAddressCompress = C33_FALSE;
								}
								else
								{
									/* del bug fix 2005/01/31 T.Tazaki */
//									c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPLETE );
									/* add	bug fix 2005/01/31 T.Tazaki >>> */
				   					switch( c33TargetInfo.iBust_BusOutput )
				   					{							
				   					case 0: c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPLETE );
				   							break;
									case 1:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY,	0,		   ulData, 0, iBusDataCount, C33_ADDRESS_COMPLETE );
				   							break;
									case 2:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPLETE );
				   							break;
									}
									/* bug fix 2005/01/31 T.Tazaki <<< */
								}
								ulOldAddress = ulAddress & 0xffff0000;
							}
							ulAddress = 0;
							ulData = 0;
							bAddressGetMode = C33_FALSE;
							bDataGetMode	= C33_FALSE;
							iBusAddressCount = 0;
							iBusDataCount = 0;
							
							ucBusMode = c33IcdTraceData.usBusTraceData[i] & 0x1f;		/* update current status */

//							if( ucBusMode == INST_START || c33TargetInfo.iBust_BusOutput == 0 || c33TargetInfo.iBust_BusOutput == 2 )	/* bug fix 2004/12/07 T.Tazaki */
		   					if( c33TargetInfo.iBust_BusOutput == 0 || c33TargetInfo.iBust_BusOutput == 2 )								/* bug fix 2004/12/07 T.Tazaki */
							{
								/* address start */
								bAddressGetMode = C33_TRUE;
								bDataGetMode	= C33_FALSE;
								ulAddress = ( c33IcdTraceData.usBusTraceData[i] & 0xff00 ) >> 8;	// [A0]
								iBusAddressCount = 1;
								iBusDataCount = 0;
								ulData = 0;

								if( iBusAddressCount == (c33TargetInfo.iBust_AddrBusSize + 1) )
								{
									if( c33TargetInfo.iBust_BusOutput == 0 )	/* Output Bus : Address only ? */
									{
										c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, iBusAddressCount, 0, C33_ADDRESS_COMPLETE );	/* COMPLETE */
										ulOldAddress = ulAddress & 0xffff0000;
										ulAddress = 0;
										ulData = 0;
										bAddressGetMode = C33_FALSE;
										iBusAddressCount = 0;
									}
									else
									{
										bAddressGetMode = C33_FALSE;
										bDataGetMode	= C33_TRUE;
										iBusAddressCount = 0;
									}
								}
							}
							else
							{
								/* output data bus is Data only */
								/* data start */
								bAddressGetMode = C33_FALSE;
								bDataGetMode	= C33_TRUE;
								ulData = ( c33IcdTraceData.usBusTraceData[i] & 0xff00 ) >> 8;	   // [D0]
								iBusAddressCount = 0;
								iBusDataCount = 1;
								ulAddress = 0;
							}
							ulStartCycle = i;		/* save start cycle */
							break;

					case DMAUSR_CONT:
					case DATA_CONT:
					case INST_CONT:
							/* continue */
							if( bAddressGetMode == C33_TRUE )
							{
								/* AhX */
								if( iBusAddressCount < (c33TargetInfo.iBust_AddrBusSize + 1) )
								{
									ulAddress += ( c33IcdTraceData.usBusTraceData[i] & 0xff00 ) << ((iBusAddressCount - 1) * 8 ); // [A1]/[A2]/[A3]
									++iBusAddressCount;
									if( iBusAddressCount == (c33TargetInfo.iBust_AddrBusSize + 1) )
									{
										if( c33TargetInfo.iBust_BusOutput == 0 )	/* Output Bus : Address only ? */
										{
											/* address only */
											c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, iBusAddressCount, 0, C33_ADDRESS_COMPLETE );	/* COMPLETE */

											ulOldAddress = ulAddress & 0xffff0000;
											ulAddress = 0;
											ulData = 0;
											bAddressGetMode = C33_FALSE;
											iBusAddressCount = 0;
										}
										else
										{
											/* address and data */
											bAddressGetMode = C33_FALSE;
											bDataGetMode	= C33_TRUE;
											iBusAddressCount = 0;
										}
									}
								}
							}
							else if ( bDataGetMode == C33_TRUE )
							{
								switch( ucBusMode )
								{
								case DMAUSR_START_B_WR:
								case DMAUSR_START_B_RD:
								case DATA_START_B_WR:
								case DATA_START_B_RD:
									/* byte data */
									ulData = ( c33IcdTraceData.usBusTraceData[i] & 0xff00 ) >> 8;	// [D0]
		 							if( c33TargetInfo.iBust_BusOutput == 0 || c33TargetInfo.iBust_BusOutput == 2 )	/* add 2004/12/07 T.Tazaki */
		 							{
										if( bAddressCompress == C33_TRUE )
										{
											c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 1, C33_ADDRESS_COMPRESS );	/* COMPLETE */
											bAddressCompress = C33_FALSE;
										}
										else
										{
											c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 1, C33_ADDRESS_COMPLETE );	/* COMPLETE */
										}
		 							}
		  							else
		 							{
										/* BBCU  add 2004/12/07 T.Tazaki */
		 								c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY, 0, ulData, 0, 1, C33_ADDRESS_COMPLETE );	/* COMPLETE */
		 							}
									ulOldAddress = ulAddress & 0xffff0000;
									ulAddress = 0;
									ulData = 0;
									bDataGetMode = C33_FALSE;
									iBusDataCount = 0;
									break;
								
								case DMAUSR_START_H_WR:
								case DMAUSR_START_H_RD:
								case DATA_START_H_WR:
								case DATA_START_H_RD:
									/* half word data */
									if ( iBusDataCount == 0 )
									{
										++iBusDataCount;
										ulData = ( c33IcdTraceData.usBusTraceData[i] & 0xff00 ) >> 8; // [D0]
									}
									else if( iBusDataCount == 1 )
									{
										ulData += ( c33IcdTraceData.usBusTraceData[i] & 0xff00 );	  // [D1]
		 								if( c33TargetInfo.iBust_BusOutput == 0 || c33TargetInfo.iBust_BusOutput == 2 )	/* add 2004/12/07 T.Tazaki */
		 								{
											if( bAddressCompress == C33_TRUE )
											{
												c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 2, C33_ADDRESS_COMPRESS );	/* COMPLETE */
												bAddressCompress = C33_FALSE;
											}
											else
											{
												c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 2, C33_ADDRESS_COMPLETE );	/* COMPLETE */
											}
		 								}
		 								else
		 								{
											/* BBCU  add 2004/12/07 T.Tazaki */
		 									c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY, ulAddress, ulData, 0, 2, C33_ADDRESS_COMPLETE );	/* COMPLETE */
		 								}
										ulOldAddress = ulAddress & 0xffff0000;
										ulAddress = 0;
										ulData = 0;
										bDataGetMode = C33_FALSE;
										iBusDataCount = 0;
									}
									break;
									
								case DMAUSR_START_W_WR:
								case DMAUSR_START_W_RD:
								case DATA_START_W_WR:
								case DATA_START_W_RD:
								case INST_START:
									/* word data */
									if( iBusDataCount == 0 )
									{
										++iBusDataCount;
										ulData = ( c33IcdTraceData.usBusTraceData[i] & 0xff00 ) >> 8;	  // [D0]
									}
									else if( iBusDataCount < 4 )
									{
										++iBusDataCount;
										ulData += ( c33IcdTraceData.usBusTraceData[i] & 0xff00 ) << ((iBusDataCount - 2) * 8 ); // [D1]/[D2]/[D3]
										if( iBusDataCount == 4 )
										{
		 									if( c33TargetInfo.iBust_BusOutput == 0 || c33TargetInfo.iBust_BusOutput == 2 )/* add 2004/12/07 T.Tazaki */
		  									{
												if( bAddressCompress == C33_TRUE )
												{
													c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPRESS );	/* COMPLETE */
													bAddressCompress = C33_FALSE;
												}
												else
												{
													c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPLETE );	/* COMPLETE */
												}
		 									}
		 									else
		  									{
												/* BBCU  add 2004/12/07 T.Tazaki */
		 										c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY, 0, ulData, 0, 4, C33_ADDRESS_COMPLETE );	/* COMPLETE */
		   									}
											ulOldAddress = ulAddress & 0xffff0000;
											ulAddress = 0;
											ulData = 0;
											bDataGetMode = C33_FALSE;
											iBusDataCount = 0;
										}
									}
									break;
								default:								/* add T.Tazai 2007/07/12 */
									break;
								}
							}
							break;

					case DMAUSR_COMPR_MATCH:
					case DATA_COMPR_MATCH:
					case INST_COMPR_MATCH:
							/* kAhXv */
							if( bAddressGetMode == C33_TRUE )
							{
								/* AhX */
//del 2004/07/22				ulAddress &= 0x0000ffff;
//del 2004/07/22				ulAddress |= ulOldAddress;	/*  [A0]+[A2][A3] */
								ulAddress += ( c33IcdTraceData.usBusTraceData[i] & 0xff00 ); // +[A1]
/*add 2004/08/30 */				ulAddress &= 0x0000ffff;
/*add 2004/08/30 */				ulAddress |= ulOldAddress;	/*  [A0]+[A2][A3] */
								if( c33TargetInfo.iBust_BusOutput == 0 )	/* Output Bus : Address only ? */
								{
									c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, 4, 0, C33_ADDRESS_COMPRESS );	/* COMPLETE */
									bAddressCompress = C33_FALSE;

									ulOldAddress = ulAddress & 0xffff0000;
									ulAddress = 0;
									ulData = 0;
									bAddressGetMode = C33_FALSE;
									iBusAddressCount = 0;
								}
								else
								{
									bAddressGetMode = C33_FALSE;
									bDataGetMode	= C33_TRUE;
									iBusAddressCount = 0;

									/* address compress flag on. */
									bAddressCompress = C33_TRUE;
								}
							}
							break;

					case DMAUSR_COMPR_0:
					case DATA_COMPR_0:
							/* Compress 0 */
							if ( bDataGetMode == C33_TRUE )
							{
								/* high 16bit data : 0 data   [D0]+[D1]+[00]+[00] */
								ulData += ( c33IcdTraceData.usBusTraceData[i] & 0xff00 ); // +[D1]
								ulData &= 0x0000ffff;
								if( bAddressCompress == C33_TRUE )
								{
									/* del bug fix 2005/01/31 T.Tazaki */
//									c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPRESS );	/* COMPLETE */
									/* add	bug fix 2005/01/31 T.Tazaki >>> */
				   					switch( c33TargetInfo.iBust_BusOutput )
				   					{							
				   					case 0: c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPRESS );
				   							break;
									case 1:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY,	0,		   ulData, 0, 4, C33_ADDRESS_COMPRESS );
				   							break;
									case 2:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPRESS );
				   							break;
									}
									/* bug fix 2005/01/31 T.Tazaki <<< */
									bAddressCompress = C33_FALSE;
								}
								else
								{
									/* del bug fix 2005/01/31 T.Tazaki */
//									c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPLETE );	/* COMPLETE */
									/* add	bug fix 2005/01/31 T.Tazaki >>> */
				   					switch( c33TargetInfo.iBust_BusOutput )
				   					{							
				   					case 0: c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPLETE );
				   							break;
									case 1:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY,	0,		   ulData, 0, 4, C33_ADDRESS_COMPLETE );
				   							break;
									case 2:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPLETE );
				   							break;
									}
									/* bug fix 2005/01/31 T.Tazaki <<< */
								}

								ulOldAddress = ulAddress & 0xffff0000;
								ulAddress = 0;
								ulData = 0;
								bDataGetMode = C33_FALSE;
								iBusDataCount = 0;
								bAddressGetMode = C33_FALSE;
							}
							break;
					
					case DMAUSR_COMPR_1:
					case DATA_COMPR_1:
							/* Compress 1 */
							if ( bDataGetMode == C33_TRUE )
							{
								/* high 16bit : 1 data   [D0]+[D1]+[ff]+[ff] */
								ulData += ( c33IcdTraceData.usBusTraceData[i] & 0xff00 ); // +[D1]
								ulData |= 0xffff0000;
								if( bAddressCompress == C33_TRUE )
								{
									/* del bug fix 2005/01/31 T.Tazaki */
//									c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPRESS );	/* COMPLETE */
									/* add	bug fix 2005/01/31 T.Tazaki >>> */
				   					switch( c33TargetInfo.iBust_BusOutput )
				   					{							
				   					case 0: c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPRESS );
				   							break;
									case 1:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY,	0,		   ulData, 0, 4, C33_ADDRESS_COMPRESS );
				   							break;
									case 2:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPRESS );
				   							break;
									}
									/* bug fix 2005/01/31 T.Tazaki <<< */
									bAddressCompress = C33_FALSE;
								}
								else
								{
									/* del bug fix 2005/01/31 T.Tazaki */
//									c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPLETE );	/* COMPLETE */
									/* add	bug fix 2005/01/31 T.Tazaki >>> */
				   					switch( c33TargetInfo.iBust_BusOutput )
				   					{							
				   					case 0: c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPLETE );
				   							break;
									case 1:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY,	0,		   ulData, 0, 4, C33_ADDRESS_COMPLETE );
				   							break;
									case 2:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPLETE );
				   							break;
									}
									/* bug fix 2005/01/31 T.Tazaki <<< */
								}

								ulOldAddress = ulAddress & 0xffff0000;	 /* kvpɏ16rbgۑ */
								ulAddress = 0;
								ulData = 0;
								bDataGetMode = C33_FALSE;
								iBusDataCount = 0;
								bAddressGetMode = C33_FALSE;
							}
							break;
					default:								/* add T.Tazai 2007/07/12 */
							break;
					
					}

					/* Check cancel */
					if (c33TargetInfo.iKeyBreak == 1 || c33_CheckCancel( 20000 ) == 1 ) /* change 500 --> 20000 T.Tazaki 2004/12/01 */
					{
						/* Cancel */
						printf_filtered( C33_MSG_TARGET_CANCEL);
						return C33_ERR;
					}
//					c33_IcdProcessing(3000);
					c33_Percent_Display(g_ulCycleMax,i);  /* add T.Tazaki 2004/10/15 */

				} // for i
			}
			else
			{
				/*==============================================================*/
				/* HBCU Trace 													*/
				/*==============================================================*/

				/* skip until start status */
				for (i = 0 ; i < g_ulCycleMax ; i++)
				{
					if( ( (c33IcdTraceData.usBusTraceData[i] & 0x1f) == RDDATA_START_B_RD ) || 
						( (c33IcdTraceData.usBusTraceData[i] & 0x1f) == RDDATA_START_H_RD ) || 
						( (c33IcdTraceData.usBusTraceData[i] & 0x1f) == RDDATA_START_W_RD ) || 
						( (c33IcdTraceData.usBusTraceData[i] & 0x1f) == DATA_START_B_WR   ) || 
						( (c33IcdTraceData.usBusTraceData[i] & 0x1f) == DATA_START_B_RD   ) || 
						( (c33IcdTraceData.usBusTraceData[i] & 0x1f) == DATA_START_H_WR   ) || 
						( (c33IcdTraceData.usBusTraceData[i] & 0x1f) == DATA_START_H_RD   ) || 
						( (c33IcdTraceData.usBusTraceData[i] & 0x1f) == DATA_START_W_WR   ) || 
						( (c33IcdTraceData.usBusTraceData[i] & 0x1f) == DATA_START_W_RD   ) ||
						( (c33IcdTraceData.usBusTraceData[i] & 0x1f) == INST_START		  ) )
					{
						break;
					}
				}

				/* Analysis Trace Data */
				
				for ( ; i < g_ulCycleMax; ++i )
				{
					switch ( c33IcdTraceData.usBusTraceData[i] & 0x1f )
					{
					case C33_IDLE:
							if( bAddressGetMode == C33_TRUE )
							{
								/* del bug fix 2005/01/31 T.Tazaki */
//								c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, iBusAddressCount, 0, C33_ADDRESS_NOT_COMPLETE );
								/* add	bug fix 2005/01/31 T.Tazaki >>>  */
			   					switch( c33TargetInfo.iBust_BusOutput )
			   					{							
			   					case 0: c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, iBusAddressCount, 0, C33_ADDRESS_NOT_COMPLETE );
			   							break;
								case 2:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, 0,	   iBusAddressCount, 0, C33_ADDRESS_NOT_COMPLETE );
			   							break;
								}
								/* bug fix 2005/01/31 T.Tazaki <<< */
							}
							else if( bDataGetMode == C33_TRUE )
							{
								if( bAddressAnalysis_flag == C33_TRUE )
								{
									if( bAddressCompress == C33_TRUE )
									{
//										c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPRESS ); /* del bug fix 2005/01/31 T.Tazaki */
										/* add	bug fix 2005/01/31 T.Tazaki >>> */
					   					switch( c33TargetInfo.iBust_BusOutput )
					   					{							
					   					case 0: c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPRESS );
					   							break;
										case 1:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY,	0,		   ulData, 0, iBusDataCount, C33_ADDRESS_COMPRESS );
					   							break;
										case 2:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPRESS );
					   							break;
										}
										/* bug fix 2005/01/31 T.Tazaki <<< */
										bAddressCompress = C33_FALSE;
									}
									else
									{
//										c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPLETE ); /* del bug fix 2005/01/31 T.Tazaki */
										/* add	bug fix 2005/01/31 T.Tazaki >>> */
					   					switch( c33TargetInfo.iBust_BusOutput )
					   					{							
					   					case 0: c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPLETE );
					   							break;
										case 1:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY,	0,		   ulData, 0, iBusDataCount, C33_ADDRESS_COMPLETE );
					   							break;
										case 2:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPLETE );
					   							break;
										}
										/* bug fix 2005/01/31 T.Tazaki <<< */
									}
									ulOldAddress = ulAddress & 0xffff0000;
								}
								else
								{
									c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY, ulAddress, ulData, 0, iBusDataCount, C33_ADDRESS_NOT_COMPLETE );
								}
							}
							if( bAddressGetMode == C33_TRUE || bDataGetMode == C33_TRUE )
							{
								ulAddress = 0;
								ulData = 0;
								bAddressGetMode = C33_FALSE;
								bDataGetMode	= C33_FALSE;
								iBusAddressCount = 0;
								iBusDataCount = 0;
								bAddressAnalysis_flag = C33_FALSE;
							}
							break;

					case DATA_START_B_WR:
					case DATA_START_B_RD:
					case DATA_START_H_WR:
					case DATA_START_H_RD:
					case DATA_START_W_WR:
					case DATA_START_W_RD:
					case INST_START:
							/* start */

							if( bAddressGetMode == C33_TRUE )
							{
								/* del bug fix 2005/01/31 T.Tazaki */
//								c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, iBusAddressCount, 0, C33_ADDRESS_NOT_COMPLETE );
								/* add	bug fix 2005/01/31 T.Tazaki >>>  */
			   					switch( c33TargetInfo.iBust_BusOutput )
			   					{							
			   					case 0: c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, iBusAddressCount, 0, C33_ADDRESS_NOT_COMPLETE );
			   							break;
								case 2:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, 0,	   iBusAddressCount, 0, C33_ADDRESS_NOT_COMPLETE );
			   							break;
								}
								/* bug fix 2005/01/31 T.Tazaki <<< */
							}
							else if( bDataGetMode == C33_TRUE )
							{
								if( bAddressAnalysis_flag == C33_TRUE )
								{
									if( bAddressCompress == C33_TRUE )
									{
										/* del bug fix 2005/01/31 T.Tazaki */
//										c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPRESS );
										/* add	bug fix 2005/01/31 T.Tazaki >>> */
					   					switch( c33TargetInfo.iBust_BusOutput )
					   					{							
					   					case 0: c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPRESS );
					   							break;
										case 1:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY,	0,		   ulData, 0, iBusDataCount, C33_ADDRESS_COMPRESS );
					   							break;
										case 2:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPRESS );
					   							break;
										}
										/* bug fix 2005/01/31 T.Tazaki <<< */
										bAddressCompress = C33_FALSE;
									}
									else
									{
										/* del bug fix 2005/01/31 T.Tazaki */
//										c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPLETE );
										/* add	bug fix 2005/01/31 T.Tazaki >>> */
					   					switch( c33TargetInfo.iBust_BusOutput )
					   					{							
					   					case 0: c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPLETE );
					   							break;
										case 1:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY,	0,		   ulData, 0, iBusDataCount, C33_ADDRESS_COMPLETE );
					   							break;
										case 2:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPLETE );
					   							break;
										}
										/* bug fix 2005/01/31 T.Tazaki <<< */
									}
									ulOldAddress = ulAddress & 0xffff0000;
								}
								else
								{
									c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY, ulAddress, ulData, 0, iBusDataCount, C33_ADDRESS_NOT_COMPLETE );
								}
							}
							ulAddress = 0;
							ulData = 0;
							bAddressGetMode = C33_FALSE;
							bDataGetMode	= C33_FALSE;
							iBusAddressCount = 0;
							iBusDataCount = 0;
							bAddressAnalysis_flag = C33_FALSE;

							ucBusMode = c33IcdTraceData.usBusTraceData[i] & 0x1f;

							/* check if inst is start or output bus is Address only or Address+Data? */
							if( ucBusMode == INST_START || c33TargetInfo.iBust_BusOutput == 0 || c33TargetInfo.iBust_BusOutput == 2 )
							{
								/* AhXJn */
								bAddressGetMode = C33_TRUE;
								bDataGetMode	= C33_FALSE;
								ulAddress = ( c33IcdTraceData.usBusTraceData[i] & 0xff00 ) >> 8;	// [A0]
								iBusAddressCount = 1;
								iBusDataCount = 0;
								ulData = 0;

								if( iBusAddressCount == (c33TargetInfo.iBust_AddrBusSize + 1) )
								{
									if( c33TargetInfo.iBust_BusOutput == 0 )	/* Output Bus : Address only ? */
									{
										c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, iBusAddressCount, 0, C33_ADDRESS_COMPLETE );

										ulOldAddress = ulAddress & 0xffff0000;
										ulAddress = 0;
										ulData = 0;
										bAddressGetMode = C33_FALSE;
										iBusAddressCount = 0;

										bAddressAnalysis_flag = C33_FALSE;
									}
									else
									{
										bAddressGetMode = C33_FALSE;
										bDataGetMode	= C33_TRUE;
										iBusAddressCount = 0;

										bAddressAnalysis_flag = C33_TRUE;
									}
								}
							}
							else
							{
								/* data start */
								bAddressGetMode = C33_FALSE;
								bDataGetMode	= C33_TRUE;
								ulData = ( c33IcdTraceData.usBusTraceData[i] & 0xff00 ) >> 8;	   // [D0]
								iBusAddressCount = 0;
								iBusDataCount = 1;
								ulAddress = 0;

								bAddressAnalysis_flag = C33_FALSE;
							}
							ulStartCycle = i;		/* save start cycle */
							break;

					case RDDATA_START_B_RD:
					case RDDATA_START_H_RD:
					case RDDATA_START_W_RD:
							/* start read data */

							if( bAddressGetMode == C33_TRUE )
							{
								/* del 2005/01/31 T.Tazaki */
//								c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, iBusAddressCount, 0, C33_ADDRESS_NOT_COMPLETE );
								/* add	bug fix 2005/01/31 T.Tazaki >>>  */
			   					switch( c33TargetInfo.iBust_BusOutput )
			   					{							
			   					case 0: c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, 0, iBusAddressCount, 0, C33_ADDRESS_NOT_COMPLETE );
			   							break;
								case 2:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, 0, iBusAddressCount, 0, C33_ADDRESS_NOT_COMPLETE );
			   							break;
								}
								/* bug fix 2005/01/31 T.Tazaki <<< */
							}
							else if( bDataGetMode == C33_TRUE )
							{
								if( bAddressAnalysis_flag == C33_TRUE )
								{
									if( bAddressCompress == C33_TRUE )
									{
										/* del bug fix 2005/01/31 T.Tazaki */
//										c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPRESS );
										/* add	bug fix 2005/01/31 T.Tazaki >>> */
					   					switch( c33TargetInfo.iBust_BusOutput )
					   					{							
					   					case 0: c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPRESS );
					   							break;
										case 1:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY,	0,		   ulData, 0, iBusDataCount, C33_ADDRESS_COMPRESS );
					   							break;
										case 2:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPRESS );
					   							break;
										default:								/* add T.Tazai 2007/07/12 */
												break;
										}
										/* bug fix 2005/01/31 T.Tazaki <<< */
										bAddressCompress = C33_FALSE;
									}
									else
									{
										/* del bug fix 2005/01/31 T.Tazaki */
//										c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPLETE );
										/* add	bug fix 2005/01/31 T.Tazaki >>> */
					   					switch( c33TargetInfo.iBust_BusOutput )
					   					{							
					   					case 0: c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPLETE );
					   							break;
										case 1:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY,	0,		   ulData, 0, iBusDataCount, C33_ADDRESS_COMPLETE );
					   							break;
										case 2:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, iBusDataCount, C33_ADDRESS_COMPLETE );
					   							break;
										default:								/* add T.Tazai 2007/07/12 */
												break;
										}
										/* bug fix 2005/01/31 T.Tazaki <<< */
									}
									ulOldAddress = ulAddress & 0xffff0000;
								}
								else
								{
									c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY, ulAddress, ulData, 0, iBusDataCount, C33_ADDRESS_NOT_COMPLETE );
								}
							}
							ulAddress = 0;
							ulData = 0;
							bAddressGetMode = C33_FALSE;
							bDataGetMode	= C33_FALSE;
							iBusAddressCount = 0;
							iBusDataCount = 0;
							bAddressAnalysis_flag = C33_FALSE;


							ucBusMode = c33IcdTraceData.usBusTraceData[i] & 0x1f;	/* update mode */
							ulStartCycle = i;		/* start start cycle */

							ulData = ( c33IcdTraceData.usBusTraceData[i] & 0xff00 ) >> 8;	   // [D0]
							bAddressGetMode = C33_FALSE;
							iBusAddressCount = 0;
							bAddressAnalysis_flag = C33_FALSE;

							if( ucBusMode == RDDATA_START_B_RD )
							{
								if( bAddressCompress == C33_TRUE )
								{
									c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY, ulAddress, ulData, 0, 1, C33_ADDRESS_COMPRESS );
									bAddressCompress = C33_FALSE;
								}
								else
								{
									c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY, ulAddress, ulData, 0, 1, C33_ADDRESS_COMPLETE );
								}

								ulOldAddress = ulAddress & 0xffff0000;
								ulAddress = 0;
								ulData = 0;
								bDataGetMode = C33_FALSE;
								iBusDataCount = 0;
							}
							else
							{
								bDataGetMode = C33_TRUE;
								iBusDataCount = 1;
							}
							break;

					case RDDATA_CONT:
					case DATA_CONT:
					case INST_CONT:
							/* continue */
							if( bAddressGetMode == C33_TRUE )
							{
								/* ADDRESS */
								if( iBusAddressCount < (c33TargetInfo.iBust_AddrBusSize + 1) )
								{
									ulAddress += ( c33IcdTraceData.usBusTraceData[i] & 0xff00 ) << ((iBusAddressCount - 1) * 8 ); // [A1]/[A2]/[A3]
									++iBusAddressCount;
									if( iBusAddressCount == (c33TargetInfo.iBust_AddrBusSize + 1) )
									{
										if ( ucBusMode == DATA_START_B_RD ||		/* READ or INST ? */
											 ucBusMode == DATA_START_H_RD ||
											 ucBusMode == DATA_START_W_RD ||
											 ucBusMode == INST_START	  ||
											 c33TargetInfo.iBust_BusOutput == 0 )	/* Output Bus : Address only ? */
										{
											/* Only address division,COMPLETE */
											c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, iBusAddressCount, 0, C33_ADDRESS_COMPLETE );	/* COMPLETE */

											ulOldAddress = ulAddress & 0xffff0000;
											ulAddress = 0;
											ulData = 0;
											bAddressGetMode = C33_FALSE;
											iBusAddressCount = 0;
											bAddressAnalysis_flag = C33_FALSE;
										}
										else
										{
											/* address and data */
											bAddressGetMode = C33_FALSE;
											bDataGetMode	= C33_TRUE;
											iBusAddressCount = 0;

											bAddressAnalysis_flag = C33_TRUE;
										}
									}
								}
							}
							else if ( bDataGetMode == C33_TRUE )
							{
								switch( ucBusMode )
								{
								case DATA_START_B_WR:
								case DATA_START_B_RD:
									/* byte data */
									ulData = ( c33IcdTraceData.usBusTraceData[i] & 0xff00 ) >> 8;	  // [D0]
									if( bAddressAnalysis_flag == C33_TRUE )
									{
										if( bAddressCompress == C33_TRUE )
										{
											/* del bug fix 2005/01/31 T.Tazaki */
//											c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 1, C33_ADDRESS_COMPRESS );	/* COMPLETE */
											/* add	bug fix 2005/01/31 T.Tazaki >>> */
						   					switch( c33TargetInfo.iBust_BusOutput )
						   					{							
						   					case 0: c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, 4, 1, C33_ADDRESS_COMPRESS );
						   							break;
											case 1:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY,	0,		   ulData, 0, 1, C33_ADDRESS_COMPRESS );
						   							break;
											case 2:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 1, C33_ADDRESS_COMPRESS );
						   							break;
											default:								/* add T.Tazai 2007/07/12 */
													break;
											}
											/* bug fix 2005/01/31 T.Tazaki <<< */
											bAddressCompress = C33_FALSE;
										}
										else
										{
											/* del bug fix 2005/01/31 T.Tazaki */
//											c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 1, C33_ADDRESS_COMPLETE );	/* COMPLETE */
											/* add	bug fix 2005/01/31 T.Tazaki >>> */
						   					switch( c33TargetInfo.iBust_BusOutput )
						   					{							
						   					case 0: c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, 4, 1, C33_ADDRESS_COMPLETE );
						   							break;
											case 1:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY,	0,		   ulData, 0, 1, C33_ADDRESS_COMPLETE );
						   							break;
											case 2:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 1, C33_ADDRESS_COMPLETE );
						   							break;
											default:								/* add T.Tazai 2007/07/12 */
													break;
											}
											/* bug fix 2005/01/31 T.Tazaki <<< */
										}
									}
									else
									{
										c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY, ulAddress, ulData, 0, 1, C33_ADDRESS_COMPLETE );		/* COMPLETE */
									}

									ulOldAddress = ulAddress & 0xffff0000;
									ulAddress = 0;
									ulData = 0;
									bDataGetMode = C33_FALSE;
									iBusDataCount = 0;
									bAddressAnalysis_flag = C33_FALSE;
									break;
								
								case RDDATA_START_H_RD:
								case DATA_START_H_WR:
								case DATA_START_H_RD:
									/* half word data */
									if( iBusDataCount == 0 )
									{
										++iBusDataCount;
										ulData = ( c33IcdTraceData.usBusTraceData[i] & 0xff00 ) >> 8;	  // [D0]
									}
									else if( iBusDataCount == 1 )
									{
										ulData += ( c33IcdTraceData.usBusTraceData[i] & 0xff00 );		  // [D1]
										if( bAddressAnalysis_flag == C33_TRUE )
										{
											if( bAddressCompress == C33_TRUE )
											{
												/* del bug fix 2005/01/31 T.Tazaki */
//												c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 2, C33_ADDRESS_COMPRESS );	/* COMPLETE */
												/* add	bug fix 2005/01/31 T.Tazaki >>> */
							   					switch( c33TargetInfo.iBust_BusOutput )
							   					{							
							   					case 0: c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, 4, 2, C33_ADDRESS_COMPRESS );
							   							break;
												case 1:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY,	0,		   ulData, 0, 2, C33_ADDRESS_COMPRESS );
							   							break;
												case 2:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 2, C33_ADDRESS_COMPRESS );
							   							break;
												default:								/* add T.Tazai 2007/07/12 */
														break;
												}
												/* bug fix 2005/01/31 T.Tazaki <<< */
												bAddressCompress = C33_FALSE;
											}
											else
											{
												/* del bug fix 2005/01/31 T.Tazaki */
//												c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 2, C33_ADDRESS_COMPLETE );	/* COMPLETE */
												/* add	bug fix 2005/01/31 T.Tazaki >>> */
							   					switch( c33TargetInfo.iBust_BusOutput )
							   					{							
							   					case 0: c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, 4, 2, C33_ADDRESS_COMPLETE );
							   							break;
												case 1:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY,	0,		   ulData, 0, 2, C33_ADDRESS_COMPLETE );
							   							break;
												case 2:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 2, C33_ADDRESS_COMPLETE );
							   							break;
												default:								/* add T.Tazai 2007/07/12 */
														break;
												}
												/* bug fix 2005/01/31 T.Tazaki <<< */
											}
										}
										else
										{
											c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY, ulAddress, ulData, 0, 2, C33_ADDRESS_COMPLETE );		/* COMPLETE */
										}

										if( ucBusMode != RDDATA_START_H_RD )
										{
											ulOldAddress = ulAddress & 0xffff0000;
										}
										ulAddress = 0;
										ulData = 0;
										bDataGetMode = C33_FALSE;
										iBusDataCount = 0;
										bAddressAnalysis_flag = C33_FALSE;
									}
									break;
									
								case RDDATA_START_W_RD:
								case DATA_START_W_WR:
								case DATA_START_W_RD:
								case INST_START:
									/* [hf[^ */
									if( iBusDataCount == 0 )
									{
										++iBusDataCount;
										ulData = ( c33IcdTraceData.usBusTraceData[i] & 0xff00 ) >> 8;	  // [D0]
									}
									else if( iBusDataCount < 4 )
									{
										++iBusDataCount;
										ulData += ( c33IcdTraceData.usBusTraceData[i] & 0xff00 ) << ((iBusDataCount - 2) * 8 ); // [D1]/[D2]/[D3]
										if( iBusDataCount == 4 )
										{
											if( bAddressAnalysis_flag == C33_TRUE )
											{
												if( bAddressCompress == C33_TRUE )
												{
													/* del bug fix 2005/01/31 T.Tazaki */
//													c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPRESS );	/* COMPLETE */
													/* add	bug fix 2005/01/31 T.Tazaki >>> */
								   					switch( c33TargetInfo.iBust_BusOutput )
								   					{							
								   					case 0: c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPRESS );
								   							break;
													case 1:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY,	0,		   ulData, 0, 4, C33_ADDRESS_COMPRESS );
								   							break;
													case 2:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPRESS );
								   							break;
													default:								/* add T.Tazai 2007/07/12 */
															break;
													}
													/* bug fix 2005/01/31 T.Tazaki <<< */
													bAddressCompress = C33_FALSE;
												}
												else
												{
													/* del bug fix 2005/01/31 T.Tazaki */
//													c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPLETE );	/* COMPLETE */
													/* add	bug fix 2005/01/31 T.Tazaki >>> */
								   					switch( c33TargetInfo.iBust_BusOutput )
								   					{							
								   					case 0: c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPLETE );
								   							break;
													case 1:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY,	0,		   ulData, 0, 4, C33_ADDRESS_COMPLETE );
								   							break;
													case 2:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPLETE );
								   							break;
													default:								/* add T.Tazai 2007/07/12 */
															break;
													}
													/* bug fix 2005/01/31 T.Tazaki <<< */
												}
											}
											else
											{
//del 2004/08/31 T.Tazaki						c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY, ulAddress, ulData, 0, 4, C33_ADDRESS_COMPLETE );		/* COMPLETE */
/*add 2004/08/31 T.Tazaki */					c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY, ulAddress, ulData, 0, 4, C33_ADDRESS_NOT_COMPLETE );		/* COMPLETE */
											}
											if( ucBusMode != RDDATA_START_W_RD )
											{
												ulOldAddress = ulAddress & 0xffff0000;
											}
											ulAddress = 0;
											ulData = 0;
											bDataGetMode = C33_FALSE;
											iBusDataCount = 0;
											bAddressAnalysis_flag = C33_FALSE;
										}
									}
									break;
								default:								/* add T.Tazai 2007/07/12 */
									break;
								}
							}
							break;

					case DATA_COMPR_MATCH:
					case INST_COMPR_MATCH:
							/* match compress address */
							if( bAddressGetMode == C33_TRUE )
							{
								/* ADDRESS */
//del 2004/07/22 T.Tazaki		ulAddress &= 0x0000ffff;
//del 2004/07/22 T.Tazaki		ulAddress |= ulOldAddress;	/*  [A0]+[A2][A3] */
								ulAddress += ( c33IcdTraceData.usBusTraceData[i] & 0xff00 ); // +[A1]
/*add 2004/08/30 T.Tazaki */	ulAddress &= 0x0000ffff;
/*add 2004/08/30 T.Tazaki */	ulAddress |= ulOldAddress;	/*  [A0]+[A2][A3] */
								
								if( ucBusMode == DATA_START_B_RD ||			/* READ or INST ?  */
									ucBusMode == DATA_START_H_RD ||
									ucBusMode == DATA_START_W_RD ||
									ucBusMode == INST_START 	 ||
									c33TargetInfo.iBust_BusOutput == 0 )	/* Output Bus : Address only ? */
								{
									/* Only address division,COMPLETE */
									c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, 4, 0, C33_ADDRESS_COMPRESS );		/* COMPLETE */
									ulOldAddress = ulAddress & 0xffff0000;
									ulAddress = 0;
									ulData = 0;
									bAddressGetMode = C33_FALSE;
									iBusAddressCount = 0;
									bAddressAnalysis_flag = C33_FALSE;
								}
								else
								{
									/* address and data */
									bAddressGetMode = C33_FALSE;
									bDataGetMode	= C33_TRUE;
									iBusAddressCount = 0;
									bAddressAnalysis_flag = C33_TRUE;
									bAddressCompress = C33_TRUE;
								}
							}
							break;

					case RDDATA_COMPR_0:
					case DATA_COMPR_0:
							/* compress 0 */
							if ( bDataGetMode == C33_TRUE )
							{
								/* high 16bit : 0 data   [D0]+[D1]+[00]+[00] */
								ulData += ( c33IcdTraceData.usBusTraceData[i] & 0xff00 ); // +[D1]
								ulData &= 0x0000ffff;
								if( bAddressAnalysis_flag == C33_TRUE )
								{
									if( bAddressCompress == C33_TRUE )
									{
										/* del bug fix 2005/01/31 T.Tazaki */
//										c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPRESS );	/* COMPLETE */
										/* add	bug fix 2005/01/31 T.Tazaki >>> */
					   					switch( c33TargetInfo.iBust_BusOutput )
					   					{							
					   					case 0: c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPRESS );
					   							break;
										case 1:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY,	0,		   ulData, 0, 4, C33_ADDRESS_COMPRESS );
					   							break;
										case 2:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPRESS );
					   							break;
										default:								/* add T.Tazai 2007/07/12 */
												break;
										}
										/* bug fix 2005/01/31 T.Tazaki <<< */
										bAddressCompress = C33_FALSE;
									}
									else
									{
										/* del bug fix 2005/01/31 T.Tazaki */
//										c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPLETE );	/* COMPLETE */
										/* add	bug fix 2005/01/31 T.Tazaki >>> */
					   					switch( c33TargetInfo.iBust_BusOutput )
					   					{							
					   					case 0: c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPLETE );
					   							break;
										case 1:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY,	0,		   ulData, 0, 4, C33_ADDRESS_COMPLETE );
					   							break;
										case 2:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPLETE );
					   							break;
										default:								/* add T.Tazai 2007/07/12 */
												break;
										}
										/* bug fix 2005/01/31 T.Tazaki <<< */
									}
								}
								else
								{
									c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY, ulAddress, ulData, 0, 4, C33_ADDRESS_COMPLETE );		/* COMPLETE */
								}
								ulOldAddress = ulAddress & 0xffff0000;
								ulAddress = 0;
								ulData = 0;
								bDataGetMode = C33_FALSE;
								iBusDataCount = 0;
								bAddressAnalysis_flag = C33_FALSE;
							}
							break;

					case RDDATA_COMPR_1:
					case DATA_COMPR_1:
							/* compress 1 */
							if ( bDataGetMode == C33_TRUE )
							{
								/* high 16bit : 1 data  [D0]+[D1]+[ff]+[ff] */
								ulData += ( c33IcdTraceData.usBusTraceData[i] & 0xff00 ); // +[D1]
								ulData |= 0xffff0000;
								if( bAddressAnalysis_flag == C33_TRUE )
								{
									if( bAddressCompress == C33_TRUE )
									{
										/* del bug fix 2005/01/31 T.Tazaki */
//										c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPRESS );	/* COMPLETE */
										/* add	bug fix 2005/01/31 T.Tazaki >>> */
					   					switch( c33TargetInfo.iBust_BusOutput )
					   					{							
					   					case 0: c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPRESS );
					   							break;
										case 1:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY,	0,		   ulData, 0, 4, C33_ADDRESS_COMPRESS );
					   							break;
										case 2:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPRESS );
					   							break;
										default:								/* add T.Tazai 2007/07/12 */
												break;
										}
										/* bug fix 2005/01/31 T.Tazaki <<< */
										bAddressCompress = C33_FALSE;
									}
									else
									{
										/* del bug fix 2005/01/31 T.Tazaki */
//										c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPLETE );	/* COMPLETE */
										/* add	bug fix 2005/01/31 T.Tazaki >>> */
					   					switch( c33TargetInfo.iBust_BusOutput )
					   					{							
					   					case 0: c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_ONLY, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPLETE );
					   							break;
										case 1:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY,	0,		   ulData, 0, 4, C33_ADDRESS_COMPLETE );
					   							break;
										case 2:	c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_ADDRESS_DATA, ulAddress, ulData, 4, 4, C33_ADDRESS_COMPLETE );
					   							break;
										default:								/* add T.Tazai 2007/07/12 */
												break;
										}
										/* bug fix 2005/01/31 T.Tazaki <<< */
									}
								}
								else
								{
									c33_SetBusTraceData( ulStartCycle, ucBusMode, C33_DATA_ONLY, ulAddress, ulData, 0, 4, C33_ADDRESS_COMPLETE );		/* COMPLETE */
								}
								ulOldAddress = ulAddress & 0xffff0000;
								ulAddress = 0;
								ulData = 0;
								bDataGetMode = C33_FALSE;
								iBusDataCount = 0;
								bAddressAnalysis_flag = C33_FALSE;
							}
							break;
					default:								/* add T.Tazai 2007/07/12 */
							break;
					}

					/* Check cancel */
					if (c33TargetInfo.iKeyBreak == 1 || c33_CheckCancel( 20000 ) == 1 ) /* change 500 --> 20000 T.Tazaki 2004/12/01 */
					{
						/* Cancel */
						printf_filtered( C33_MSG_TARGET_CANCEL);
						return C33_ERR;
					}
//					c33_IcdProcessing(3000);
					
					c33_Percent_Display(g_ulCycleMax,i);  /* add T.Tazaki 2004/10/15 */

				}
			}
		}
	}

	c33_EndCancel();
	
	puts_filtered( ".100%...");
	puts_filtered (C33_MSG_TRC_FINISHED);

	return C33_NO_ERR;

}

/****************************************************************************/
/* Format		 : static c33_Percent_Display( unsigned long ulMax,unsigned long ulCurrent ) */
/* Arguments																*/
/*	unsigned long ulMax 	: MAX											*/
/*	unsigned long ulCurrent : current value 								*/
/* Return value 															*/
/* Explanation	 : Progress display.   										*/
/****************************************************************************/
static
void c33_Percent_Display( unsigned long ulMax,unsigned long ulCurrent )
{
	double dPercent;
	double dMax;
	double dCurrent;
	int i;
	
	if( g_iPercent_First == 1 )
	{
		for( i = 0; i < 10; ++i )
		{
			g_iPercentFlag[ i ] = 0;
		}
		g_iPercent_First = 0;
	}
	
	dMax = ulMax;
	dCurrent = ulCurrent;
	dPercent = (dCurrent / dMax) * 100;
	
	if( dPercent >= 10 )
	{
		if( g_iPercentFlag[0] == 0 )
		{
			printf_filtered( ".10%%.");
			g_iPercentFlag[0] = 1;
		}
	}
	if( dPercent >= 20 )
	{
		if( g_iPercentFlag[1] == 0 )
		{
			printf_filtered( ".20%%.");
			g_iPercentFlag[1] = 1;
		}
	}
	if( dPercent >= 30 )
	{
		if( g_iPercentFlag[2] == 0 )
		{
			printf_filtered( ".30%%.");
			g_iPercentFlag[2] = 1;
		}
	}
	if( dPercent >= 40 )
	{
		if( g_iPercentFlag[3] == 0 )
		{
			printf_filtered( ".40%%.");
			g_iPercentFlag[3] = 1;
		}
	}
	if( dPercent >= 50 )
	{
		if( g_iPercentFlag[4] == 0 )
		{
			printf_filtered( ".50%%.");
			g_iPercentFlag[4] = 1;
		}
	}
	if( dPercent >= 60 )
	{
		if( g_iPercentFlag[5] == 0 )
		{
			printf_filtered( ".60%%.");
			g_iPercentFlag[5] = 1;
		}
	}
	if( dPercent >= 70 )
	{
		if( g_iPercentFlag[6] == 0 )
		{
			printf_filtered( ".70%%.");
			g_iPercentFlag[6] = 1;
		}
	}
	if( dPercent >= 80 )
	{
		if( g_iPercentFlag[7] == 0 )
		{
			printf_filtered( ".80%%.");
			g_iPercentFlag[7] = 1;
		}
	}
	if( dPercent >= 90 )
	{
		if( g_iPercentFlag[8] == 0 )
		{
			printf_filtered( ".90%%.");
			g_iPercentFlag[8] = 1;
		}
	}
}

/****************************************************************************/
/* Format		 : static int c33_SetBusTraceData( unsigned long ulSetCycle,		 */
/*				   char cAddrData, unsigned char ucBusMode, unsigned long ulAddress, */
/*					unsigned long ulData, int iAddressLength, int iDataLength,		 */
/*					int iAddressComplete )									*/
/* Arguments																*/
/*	unsigned long ulSetCycle	 : Set	 cycle								*/
/*	unsigned char ucBusMode 	 : DTS( mode )								*/
/*	char		  cAddrData 	 : 0=Address Only, 1=Data Only, 2=Address/Data */
/*	unsigned long ulAddress 	 : Address bus								*/
/*	unsigned long ulData		 : Data bus 								*/
/*	int			  iAddressLength : get address length						*/
/*	int			  iDataLength	 : get data    length						*/
/*	int			  iAddressComplete	: 0:Not Complete,1:Complete,2:Compress	*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Make bus trace data. 									*/
/****************************************************************************/
static int
c33_SetBusTraceData( unsigned long ulSetCycle, unsigned char ucBusMode,
					 char cAddrData, unsigned long ulAddress, unsigned long ulData, int iAddressLength, int iDataLength,int iAddressComplete)
{
	int iSize = 0;
	int iAccess = 0;
	int iBus = 0;
	unsigned char ucData0,ucData1,ucData2,ucData3;
	int i;

	switch( ucBusMode )
	{
	case DMAUSR_START_B_WR:
				iSize = C33_BYTE;		iAccess = C33_WRITE; iBus = C33_DMAUSR;
				break;
	case DMAUSR_START_B_RD:
				iSize = C33_BYTE;		iAccess = C33_READ;  iBus = C33_DMAUSR;
				break;
	case DMAUSR_START_H_WR:
				iSize = C33_HALF_WORD;	iAccess = C33_WRITE; iBus = C33_DMAUSR;
				break;
	case DMAUSR_START_H_RD:
				iSize = C33_HALF_WORD;	iAccess = C33_READ;  iBus = C33_DMAUSR;
				break;
	case DMAUSR_START_W_WR:
				iSize = C33_WORD;		iAccess = C33_WRITE; iBus = C33_DMAUSR;
				break;
	case DMAUSR_START_W_RD:
				iSize = C33_WORD;		iAccess = C33_READ;  iBus = C33_DMAUSR;
				break;
	case DATA_START_B_WR:
				iSize = C33_BYTE;		iAccess = C33_WRITE; iBus = C33_DATA;
				break;
	case DATA_START_B_RD:
	case RDDATA_START_B_RD:
				iSize = C33_BYTE;		iAccess = C33_READ;  iBus = C33_DATA;
				break;
	case DATA_START_H_WR:
				iSize = C33_HALF_WORD;	iAccess = C33_WRITE; iBus = C33_DATA;
				break;
	case DATA_START_H_RD:
	case RDDATA_START_H_RD:
				iSize = C33_HALF_WORD;	iAccess = C33_READ;  iBus = C33_DATA;
				break;
	case DATA_START_W_WR:
				iSize = C33_WORD;		iAccess = C33_WRITE; iBus = C33_DATA;
				break;
	case DATA_START_W_RD:
	case RDDATA_START_W_RD:
				iSize = C33_WORD;		iAccess = C33_READ;  iBus = C33_DATA;
				break;
	case INST_START:
				iSize = C33_WORD;		iAccess = C33_READ;  iBus = C33_INST;
				break;
	default:								/* add T.Tazai 2007/07/12 */
				break;
	}

	/* set trace data from bus trace start cycle to current cycle */

	/* set ONLY Set Cycle data */
	
	++c33IcdTraceData.ulBusTraceCycle;
	
	
	c33IcdTraceData.cBusDataExist[ ulSetCycle ]  = 1;
	c33IcdTraceData.cAddrData[ ulSetCycle ] 	 = cAddrData;
	c33IcdTraceData.ulAddress[ ulSetCycle ] 	 = ulAddress;
	c33IcdTraceData.cAddressLength[ ulSetCycle ] = (char)iAddressLength;
	c33IcdTraceData.cAddrComplete[ ulSetCycle ]  = (char)iAddressComplete;

	if( iAddressComplete == C33_ADDRESS_COMPLETE )
	{
		g_iAddrCompeteIndex = ulSetCycle;		/* Update compete address posision for Speed Up */
	}

	if( cAddrData == C33_ADDRESS_ONLY || cAddrData == C33_ADDRESS_DATA )	/* add T.Tazaki 2004/07/22 */
	{
		if( iAddressComplete == C33_ADDRESS_COMPRESS )
		{
			/* Output complete address 											 */
			/* 1001c584				.cAddrComplete[i+0]=1	.cAddrSame[i+0] = 0  */
			/* ******78				.cAddrComplete[i+1]=0	.cAddrSame[i+1] <= 1 */
			/* ******7c				.cAddrComplete[i+2]=0	.cAddrSame[i+2] <= 1 */
			/* 1001c580 compress	.cAddrComplete[i+3]=2	.cAddrSame[i+3] <= 1 */
			
			/* no Output complete address								 		 */
			/* ****c584				.cAddrComplete[0]=0		.cAddrSame[0] <= 1	 */
			/* ******78				.cAddrComplete[1]=0		.cAddrSame[1] <= 1	 */
			/* ******7c				.cAddrComplete[2]=0		.cAddrSame[2] <= 1	 */
			/* 1001c580 compress	.cAddrComplete[3]=2		.cAddrSame[3] <= 1	 */

			/* search complete address output cycle */
			if( ulSetCycle > 0 )
			{
#if 0		/* del T.Tazaki 2004/10/04	 search is slow. */
				for( i = ulSetCycle - 1; i >= 0; --i )
				{
					if( c33IcdTraceData.cBusDataExist[ ulSetCycle ] == 1 )	/* add T.Tazaki 2004/07/22 */
					{
						if( c33IcdTraceData.cAddrComplete[ i ] == C33_ADDRESS_COMPLETE )
						{
							iFound = 1;
							break;
						}
					}
				}
				if( iFound == 1 )
				{
					i = i + 1;
				}else{
					i = 0;
				}
#endif
				if( g_iAddrCompeteIndex == -1 )
				{
					i = 0;
				}
				else
				{
					i = g_iAddrCompeteIndex + 1;
				}
				
				/* set same before address flag */
				for( ; i <= ulSetCycle; ++i )
				{
					if( c33IcdTraceData.cBusDataExist[ i ] == 1 )	/* add T.Tazaki 2004/07/23 */
					{
						if( c33IcdTraceData.cAddrData[ i ] == C33_ADDRESS_ONLY || c33IcdTraceData.cAddrData[ i ] == C33_ADDRESS_DATA )	/* add T.Tazaki 2004/07/23 */
						{
							c33IcdTraceData.cAddrSame[ i ] = 1;	/* on */
						}
					}
				}
			}
		}
	}

	/* inst address + data cycle(BBCU) or HBCU inst bus ? */
//	if( (cAddrData == C33_ADDRESS_DATA && iBus == C33_INST) || (cAddrData == C33_DATA_ONLY && c33TargetInfo.iBust_BusSelect == 2 ) )
//	if( (cAddrData == C33_ADDRESS_DATA && iBus == C33_INST) || (cAddrData == C33_DATA_ONLY && c33TargetInfo.iBus_BusSelect == 2 ) ) /* 2004/06/25 T.Tazaki */
	if( (cAddrData == C33_ADDRESS_DATA && iBus == C33_INST) ||
		(cAddrData == C33_DATA_ONLY && c33TargetInfo.iBus_BusSelect == 2 ) ||
		(cAddrData == C33_DATA_ONLY && iBus == C33_INST ))				/* add BBCU 2004/12/07 T.Tazaki */
	{
		if( iSize == C33_WORD )
		{
			/* chenge bit data : 0x12345678 => 0x75318642 */
			ucData3 = (unsigned char)(ulData & 0x000000ff);
			ucData2 = (unsigned char)((ulData & 0x0000ff00) >>	8);
			ucData1 = (unsigned char)((ulData & 0x00ff0000) >> 16);
			ucData0 = (unsigned char)((ulData & 0xff000000) >> 24);
			
			ulData = (ucData3 & 0xf0) + ((ucData2 & 0xf0) >> 4);
			ulData <<= 8;
			ulData += ((ucData1 & 0xf0) + ((ucData0 & 0xf0) >> 4));
			ulData <<= 8;
			ulData += (((ucData3 & 0x0f) << 4) + (ucData2 & 0x0f));
			ulData <<= 8;
			ulData += (((ucData1 & 0x0f) << 4) + (ucData0 & 0x0f));
		}
	}

	c33IcdTraceData.ulData[ ulSetCycle ]	= ulData;
	c33IcdTraceData.cDataLength[ ulSetCycle ] = (char)iDataLength;
	c33IcdTraceData.cSize[ ulSetCycle ] 	= (char)iSize;
	c33IcdTraceData.cAccess[ ulSetCycle ]	= (char)iAccess;

	if( c33TargetInfo.iBus_BusSelect == 2 )		/* HBCU:inst ? */
	{
		c33IcdTraceData.cBus[ ulSetCycle ] = C33_INST;
	}
	else
	{
		c33IcdTraceData.cBus[ ulSetCycle ] = (char)iBus;
	}
	c33IcdTraceData.cUserIn[ ulSetCycle ]	= (c33IcdTraceData.usBusTraceData[ ulSetCycle ] & 0x20) >> 5;

	if( iDebugTrace == 1 )	/* trace dump mode ?  */
	{
		printf_filtered( "SetClock[%08d]: ",ulSetCycle);

		printf_filtered( "AddrData[%d] ",cAddrData);

		printf_filtered( "Address[%08x]Length[%d]  Data[%08x]Length[%d] ",ulAddress,iAddressLength,ulData,iDataLength);
	
		if( iSize == C33_BYTE )
		{
			printf_filtered( "[BYTE] ");
		}
		else if( iSize == C33_HALF_WORD )
		{
			printf_filtered( "[HALF] ");
		}
		else
		{
			printf_filtered( "[WORD] ");
		}
			
		if( iAccess == C33_READ )
		{
			printf_filtered( "[READ ] ");
		}
		else
		{
			printf_filtered( "[WRITE] ");
		}
		if( iBus == C33_INST )
		{
			printf_filtered( "[INST] ");
		}
		else if ( iBus == C33_DMAUSR )
		{
			printf_filtered( "[DMA ] ");
		}
		else
		{
			printf_filtered( "[DATA] ");
		}
		if( c33IcdTraceData.cUserIn[ ulSetCycle ] == 1 )
		{
			printf_filtered( "UserIN[1]");
		}
		else
		{
			printf_filtered( "UserIN[0]");
		}
		printf_filtered( "\n" );
	}
	
	return C33_NO_ERR;
}
/****************************************************************************/
/* Format		 : void c33_IcdFinishCommand(void)							*/
/* Arguments	 : void 													*/
/* Return value  : void 													*/
/* Explanation	 : finish command. called by infcmd.c						*/
/****************************************************************************/
#define INSN_RET	  0x0640
#define INSN_RET_D	  0x0740
#define INSN_RETI	  0x04c0

void
c33_IcdFinishCommand()
{
	static void (*old_sigint) ();	/* Old signal-handler for sigint */
	unsigned long ulCurrentPC;
	unsigned long ulPC;
	unsigned long ulTempBreakAddr;
	unsigned long ulAddr;
	unsigned long ulSP;
	unsigned short insn_word;
	int iCondition;
	int iResult;
	int i;
	int iPcBreakFlag = 0;
	int iID;
	int iType = 0;
	int iAddr = 0;
	unsigned long ulStartTime;	  	/* add T.Tazaki 2004/07/01 */
	int iWaitFirst = 1;				/* add T.Tazaki 2004/07/01 */
	int iWaitTimer;					/* add T.Tazaki 2004/07/01 */
	/* add T.Tazaki 2005/05/06 >>> */
	unsigned short usCode;
	/* add T.Tazaki 2005/05/06 <<< */
	int iRet = 0;  					/* add T.Tazaki 2005/05/11 */

	c33_GetBreakPoint();

	/* Soft PC Break Count Check */
	if( g_SoftPcBreakTable_Index > C33_SOFT_PC_BREAK_MAX )
	{
		c33_error (C33_ERR_MSG_BRK_SOFT_MAX);
		return;
	}
	/* Hard PC Break Count Check */
	if( g_HardPcBreakTable_Index > C33_HARD_PC_BREAK_MAX )
	{
		c33_error (C33_ERR_MSG_BRK_HARD_MAX);
		return;
	}

	ulCurrentPC = ulTempBreakAddr = read_pc();


	/* Get Instruction Code on current PC */
	insn_word = read_memory_unsigned_integer (ulCurrentPC, 2);
	
	if (insn_word != INSN_RET && insn_word != INSN_RET_D && insn_word != INSN_RETI)
	{

		/* Current PC is same Soft or Hard pc break address ?  */
		iPcBreakFlag = c33_CheckPcBreak( ulCurrentPC );
		if( iPcBreakFlag == 1  )
		{
			/* check if current pc is spft pc breakpoint. add T.Tazaki 2005/05/11 >>> */
			iRet = c33_CheckSoftBreakPointIsSamePC( ulPC ); 	
			if( iRet == 1 )
			{
				c33_Cmd_SoftPcBreakClear( ulCurrentPC ); 
			}
			/* add T.Tazaki 2005/05/11 <<< */

			/*************************************************************/
			/* 1 STEP													 */
			/*************************************************************/
			iResult = c33_SendMessage(C33_ID_STEP, 0, 0, 0, 0, 0, 0, 0);
			if( iResult == C33_NO_ERR )
			{
				iResult = c33_RecvMessage(C33_ID_STEP, 0, 0, 0, 0, 0);
				if (iResult != C33_NO_ERR){
					c33_ErrorMsg();
					return;
				}
				/* add T.Tazaki 2005/05/11 >>> */
				if( iRet == 1 )
				{
					c33_Cmd_SoftPcBreakSet( ulCurrentPC );		/*  add T.Tazaki 2005/05/11 */
				}
				/* add T.Tazaki 2005/05/11 <<< */

				/* get current PC */
				ulCurrentPC = ulTempBreakAddr = read_pc();
			}
		}

		iPcBreakFlag = c33_CheckPcBreak( ulTempBreakAddr );

		insn_word = read_memory_unsigned_integer (ulCurrentPC, 2);
		if (insn_word != INSN_RET && insn_word != INSN_RET_D && insn_word != INSN_RETI)
		{
			/* Search Return code */
			while(1)
			{
				if( ulCurrentPC >= 0xfffffffeUL )
				{
					return; /* address over ==> NOP */
				}
				ulCurrentPC += 2;
				insn_word = read_memory_unsigned_integer (ulCurrentPC, 2);
				if (insn_word == INSN_RET || insn_word == INSN_RET_D || insn_word == INSN_RETI)
				{
					/* found ! */
					ulTempBreakAddr = ulCurrentPC;
					break;
				}
			}
			
			/*==========================================*/
			/* All Breakpoint Insert					*/
			/*==========================================*/
			
			/* Soft PC Break Insert */
			for( i = 0; i < g_SoftPcBreakTable_Index; ++i )
			{
				if( i < C33_SOFT_PC_BREAK_MAX )
				{
					if( ulTempBreakAddr != g_SoftPcBreakTable[ i ] )
					{
						/* change T.Tazaki 2005/05/06 >>> */

						/* read original code */
						iResult = c33_Cmd_ReadWrite(C33_READ, g_SoftPcBreakTable[ i ], 2, C33_HALF_WORD, (unsigned char*)&usCode );
						if( iResult == C33_NO_ERR )
						{
							/* save code */
							c33TargetInfo.usSoftPcBreakOrgCode[i] = usCode;
							c33TargetInfo.ulSoftPcBreak[i]		  = g_SoftPcBreakTable[ i ];
							
							/* write "BRK" */
							usCode = SOFT_BREAK_INST;
							iResult = c33_Cmd_ReadWrite(C33_WRITE, g_SoftPcBreakTable[ i ], 2, C33_HALF_WORD, (unsigned char*)&usCode);
						}

#if 0
						iResult = c33_SendMessage(C33_ID_SOFT_PC_BREAK_SET, g_SoftPcBreakTable[ i ], i + 1, 0, 0, 0, 0, 0);
						if (iResult == C33_NO_ERR)
						{
							iResult = c33_RecvMessage(C33_ID_SOFT_PC_BREAK_SET, 0, 0, 0, 0, 0);
						}
#endif
						/* change T.Tazaki 2005/05/06 <<< */
						if ( iResult != C33_NO_ERR )
						{
							c33_ErrorMsg();
							return;
						}
					}
					else
					{
						iPcBreakFlag = 1;
					}
				}
			}
			
			/* Hard PC Break Insert */
			for( i = 0; i < g_HardPcBreakTable_Index; ++i )
			{
				if( ulTempBreakAddr != g_HardPcBreakTable[ i ] )
				{
					if ( i == 0 )
					{
						/* Set hard PC break 1 */
						iID = C33_ID_HARD_PC_BREAK_SET;
					}
					else
					{
						/* Set hard PC break 2 */
						iID = C33_ID_HARD_PC_BREAK2;
					}
					iResult = c33_SendMessage(iID, g_HardPcBreakTable[ i ], 0, 0, 0, 0, 0, 0);
					if (iResult == C33_NO_ERR)
					{
						iResult = c33_RecvMessage(iID, 0, 0, 0, 0, 0);
					}
					if ( iResult != C33_NO_ERR )
					{
						c33_ErrorMsg();
						return;
					}
				}
				else
				{
					iPcBreakFlag = 1;
				}
			}
			
			/* Write Watch Break Insert */
			if( g_WriteWatchBreakAddrFlag == 1 || g_ReadWatchBreakAddrFlag == 1 || g_WriteReadWatchBreakAddrFlag == 1 )
			{
				if( g_WriteWatchBreakAddrFlag == 1 )
				{
					iType = hw_write;	/* WRITE */
					iAddr = g_WriteWatchBreakAddr;
				}
				else if( g_ReadWatchBreakAddrFlag == 1 )
				{
					iType = hw_read;	/* READ */
					iAddr = g_ReadWatchBreakAddr;
				}
				else if( g_WriteReadWatchBreakAddrFlag == 1 )
				{
					iType = hw_access;	/* READ/WRITE */
					iAddr = g_WriteReadWatchBreakAddr;
				}
				iResult = c33_SendMessage(C33_ID_DATA_BREAK_SET, iAddr, iType, 0, 0, 0, 0, 0);
				if (iResult == C33_NO_ERR)
				{
					iResult = c33_RecvMessage(C33_ID_DATA_BREAK_SET, 0, 0, 0, 0, 0);
				}
				if ( iResult != C33_NO_ERR )
				{
					c33_ErrorMsg();
					return;
				}
			}
			
			c33_SetMMUMode(0);	/* MMU OFF add T.Tazaki 2004/08/03 */

			/*==================================================*/
			/* Advanced : Set Area break, bus break, bus trace	*/ /* add T.Tazaki 2004/04/22 */
			/*==================================================*/
			c33_SetBusMode();

			/*==========================================*/
			/* GO to ret address						*/
			/*==========================================*/
			iCondition = C33_BIT_ON_TEMP;
			c33TargetInfo.iIcdExeStatus = C33_EXE_STATUS_RUN;
			
			iCondition |= 0x10;   /* execte condition bit 4 : bus break counter : Clear add T.Tazaki 2004/10/29 */
			if (c33TargetInfo.iOnTheFly == C33_MD_ON_THE_FLY)
			{
			   iCondition |= C33_BIT_ON_THE_FLY;
			}
			iResult = c33_SendMessage(C33_ID_GO, iCondition, ulTempBreakAddr, 0, 0, 0, 0, 0);
			if (c33TargetInfo.iOnTheFly == C33_MD_ON_THE_FLY)
			{
				/* On the fly mode */
				iResult = c33_RecvMessage(C33_ID_GO_ACK, 0, 0, 0, 0, 0);
				if (iResult != C33_NO_ERR){
					c33_ErrorMsg();
					return;
				}

				/* wait break */
				do
				{
#ifdef __CYGWIN__		/* if icdv2 then Wait add T.Tazaki 2004/07/01 */
					if ((c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) == C33_ICD_VER2 ) 
					{
						if( iWaitFirst == 1 )
						{
							iWaitTimer = 500;
							iWaitFirst = 0;
						}
						else
						{
							iWaitTimer = 200;
						}
						ulStartTime = GetTickCount();
						while( (ulStartTime + iWaitTimer) > GetTickCount() )
						{
						}
					}
#endif
					c33_Gdbtk_Idle(); /* re-display */

					iResult = c33_SendMessage(C33_ID_RUN_MON, 0, 0, 0, 0, 0, 0, 0);
					if ( iResult != C33_NO_ERR )
					{
						break;
					}
					/* An ESC break is carried out by the Ctrl-c. */
					old_sigint = signal (SIGINT, c33_IcdStopCtrl_C);
					iResult = c33_RecvMessage(C33_ID_RUN_MON, 0, 0, 0, 0, 0);
					signal (SIGINT, old_sigint);
					if ( iResult != C33_NO_ERR || ucRecvData[2] == C33_ID_GO )
					{
						break;
					}

				}
				while(c33TargetInfo.iKeyBreak == 0);

				if(c33TargetInfo.iKeyBreak != 0)	/* STOP ? */
				{
					/* CANCEL */
					iResult = c33_SendMessage(C33_ID_CAN, 0, 0, 0, 0, 0, 0, 0);
					if (iResult == C33_NO_ERR)
					{
						iResult = c33_RecvMessage(C33_ID_GO, 0, 0, 0, 0, 0);
					}
					if (iResult != C33_NO_ERR){
						c33_ErrorMsg();
						return;
					}
					puts_filtered (C33_MSG_KEY_BREAK);	/* add T.Tazaki 2004/04/23 */
					printf_filtered( "\n");				/* add T.Tazaki 2004/04/23 */

					/* display bus break counter */
					c33icd_BusBreakCount_display();	 /* add T.Tazaki 2004/10/29 */
				}
				/* watch,rwatch,awatch break result display   add T.Tazaki 2003/07/03 >>> */
				if (iResult == C33_NO_ERR)
				{
					if ( c33TargetInfo.iRunBreakStatus & C33_BRK_STS_BRK_MRDB )
					{
						if( g_resume_flag == 1 )
						{
							if( g_WriteWatchBreakAddrFlag == 1 )	/* add T.Tazaki 2004/10/29 */
							{
								printf_filtered( "Hardware write watchpoint %d: %s\n\n",g_WatchNum,g_What );
							}
							else if( g_ReadWatchBreakAddrFlag == 1 )
							{
								printf_filtered( "Hardware read watchpoint %d: %s\n\n",g_WatchNum,g_What );
							}
							else if( g_WriteReadWatchBreakAddrFlag == 1 )
							{
								printf_filtered( "Hardware access (read/write) watchpoint %d: %s\n\n",g_WatchNum,g_What );
							}
							/* display bus break counter */
							c33icd_BusBreakCount_display();	 /* add T.Tazaki 2004/10/29 */
						}
					}
					else if ( c33TargetInfo.iRunBreakStatus & C33_BRK_STS_BRK_MWRB )
					{
						if( g_resume_flag == 1 )
						{
							if( g_WriteWatchBreakAddrFlag == 1 )
							{
								printf_filtered( "Hardware write watchpoint %d: %s\n\n",g_WatchNum,g_What );
							}
							else if( g_ReadWatchBreakAddrFlag == 1 )	/* add T.Tazaki 2004/10/29 */
							{
								printf_filtered( "Hardware read watchpoint %d: %s\n\n",g_WatchNum,g_What );
							}
							else if( g_WriteReadWatchBreakAddrFlag == 1 )
							{
								printf_filtered( "Hardware access (read/write) watchpoint %d: %s\n\n",g_WatchNum,g_What );
							}
							/* display bus break counter */
							c33icd_BusBreakCount_display();	 /* add T.Tazaki 2004/10/29 */
						}
					}
					/* add T.Tazaki 2004/04/22 : Area Break , Bus Break display >>> */
					else if (( c33TargetInfo.iRunBreakStatus & C33_BRK_STS_BRK_AB ) && ( (c33TargetInfo.ucBrkFunc & C33_GDB_STS_AREA_BREAK) != 0 ) )
					{
						if( g_resume_flag == 1 )
						{
							printf_filtered( "\nHardware on chip Area Break!\n" );
							
							/* break detail display */
							iResult = c33_IcdAreaBreakStatus_display();
							if (iResult != C33_NO_ERR){
								c33_ErrorMsg();
								return;
							}
							/* display bus break counter */
							c33icd_BusBreakCount_display();	 /* add T.Tazaki 2004/10/29 */
						}
					}
					else if (( c33TargetInfo.iRunBreakStatus & C33_BRK_STS_BRK_BB ) && ( (c33TargetInfo.ucBrkFunc & C33_GDB_STS_BUS_BREAK) != 0 ) )
					{
						if( g_resume_flag == 1 )
						{
							printf_filtered( "\nHardware on chip Bus Break!\n" );
						}
						/* all soft break point clear 			add T.Tazaki 2005/05/11 */
						c33_ClearAllSoftBreak();
					}
					/* add T.Tazaki 2004/04/22 : Area Break , Bus Break display <<< */
					/* add T.Tazaki 2004/10/29 >>> */
					else if ( c33TargetInfo.iRunBreakStatus & C33_BRK_STS_BRK_INST )
					{
						if( g_resume_flag == 1 )
						{
							/* display bus break counter */
							c33icd_BusBreakCount_display();
						}
					}
					else if ( c33TargetInfo.iRunBreakStatus & C33_BRK_STS_BRK_HARDPC )	
					{
						if( g_resume_flag == 1 )
						{
							/* display bus break counter */
							c33icd_BusBreakCount_display();
						}
					}
					/* add T.Tazaki 2004/10/29 <<< */
				}
				/* watch,rwatch,awatch break result display   add T.Tazaki 2003/07/03 <<< */
			}
			else
			{
				/* Break response mode */
				
				/* An ESC break is carried out by the Ctrl-c. */
				old_sigint = signal (SIGINT, c33_IcdStopCtrl_C);
				iResult = c33_RecvMessage(C33_ID_GO, 0, 0, 0, 0, 0);

				/* watch,rwatch,awatch break result display   add T.Tazaki 2003/07/01 >>> */
				if (iResult == C33_NO_ERR)
				{
					if ( c33TargetInfo.iRunBreakStatus & C33_BRK_STS_BRK_MRDB )
					{
						if( g_resume_flag == 1 )
						{
							if( g_WriteWatchBreakAddrFlag == 1 )
							{
								printf_filtered( "Hardware read watchpoint %d: %s\n\n",g_WatchNum,g_What );
							}
							else if( g_WriteReadWatchBreakAddrFlag == 1 )
							{
								printf_filtered( "Hardware access (read/write) watchpoint %d: %s\n\n",g_WatchNum,g_What );
							}
							/* display bus break counter */
							c33icd_BusBreakCount_display();	 /* add T.Tazaki 2004/10/29 */
						}
					}
					else if ( c33TargetInfo.iRunBreakStatus & C33_BRK_STS_BRK_MWRB )
					{
						if( g_resume_flag == 1 )
						{
							if( g_ReadWatchBreakAddrFlag == 1 )
							{
								printf_filtered( "Hardware write watchpoint %d: %s\n\n",g_WatchNum,g_What );
							}
							else if( g_WriteReadWatchBreakAddrFlag == 1 )
							{
								printf_filtered( "Hardware access (read/write) watchpoint %d: %s\n\n",g_WatchNum,g_What );
							}

							/* display bus break counter */
							c33icd_BusBreakCount_display();	 /* add T.Tazaki 2004/10/29 */
						}
					}
					/* add T.Tazaki 2004/04/22 : Area Break , Bus Break display >>> */
					else if (( c33TargetInfo.iRunBreakStatus & C33_BRK_STS_BRK_AB ) && ( (c33TargetInfo.ucBrkFunc & C33_GDB_STS_AREA_BREAK) != 0 ) )
					{
						if( g_resume_flag == 1 )
						{
							printf_filtered( "\nHardware on chip Area Break!\n" );

							/* break detail display */
							iResult = c33_IcdAreaBreakStatus_display();
							if (iResult != C33_NO_ERR){
								c33_ErrorMsg();
								return;
							}
							/* display bus break counter */
							c33icd_BusBreakCount_display();	 /* add T.Tazaki 2004/10/29 */
						}
					}
					else if (( c33TargetInfo.iRunBreakStatus & C33_BRK_STS_BRK_BB ) && ( (c33TargetInfo.ucBrkFunc & C33_GDB_STS_BUS_BREAK) != 0 ) )
					{
						if( g_resume_flag == 1 )
						{
							printf_filtered( "\nHardware on chip Bus Break!\n" );
						}
						/* all soft break point clear 			add T.Tazaki 2005/05/11 */
						c33_ClearAllSoftBreak();
					}
					/* add T.Tazaki 2004/04/22 : Area Break , Bus Break display <<< */
				}
				/* watch,rwatch,awatch break result display   add T.Tazaki 2003/07/01 <<< */

			 	c33TargetInfo.iKeyBreak = 0;
				signal (SIGINT, old_sigint);
				if (iResult != C33_NO_ERR){
					/* add STOP button process	T.Tazaki 2004/04/23 >>> */
					/* CANCEL */
					iResult = c33_SendMessage(C33_ID_CAN, 0, 0, 0, 0, 0, 0, 0);
					if (iResult == C33_NO_ERR)
					{
						iResult = c33_RecvMessage(C33_ID_GO, 0, 0, 0, 0, 0);
					}
					if (iResult != C33_NO_ERR){
						c33_ErrorMsg();
						return;
					}
					puts_filtered (C33_MSG_KEY_BREAK);
					printf_filtered( "\n");
//					c33_ErrorMsg();			/* delete T.Tazaki 2004/04/23 */
					/* add STOP button process	T.Tazaki 2004/04/23 <<< */
					return;
				}
			}

			/*==========================================*/
			/* All Breakpoint Remove					*/
			/*==========================================*/
			/* Soft PC Break Remove */
			for( i = 0; i < g_SoftPcBreakTable_Index; ++i )
			{
				if( i < C33_SOFT_PC_BREAK_MAX )
				{
					if( ulTempBreakAddr != g_SoftPcBreakTable[ i ] )
					{
						/* change T.Tazaki 2005/05/06 >>> */
						
						/* write Original code */
						
						usCode = c33TargetInfo.usSoftPcBreakOrgCode[i];
						ulAddr = c33TargetInfo.ulSoftPcBreak[i];
						iResult = c33_Cmd_ReadWrite(C33_WRITE, ulAddr, 2, C33_HALF_WORD, (unsigned char*)&usCode);
						c33TargetInfo.ulSoftPcBreak[i] = C33_BREAK_DISABLE;
#if 0
						iResult = c33_SendMessage(C33_ID_SOFT_PC_BREAK_CLR, i + 1, 0, 0, 0, 0, 0, 0);
						if (iResult == C33_NO_ERR)
						{
							iResult = c33_RecvMessage(C33_ID_SOFT_PC_BREAK_CLR, 0, 0, 0, 0, 0);
						}
#endif
						/* change T.Tazaki 2005/05/06 <<< */

						if ( iResult != C33_NO_ERR )
						{
							c33_ErrorMsg();
							return;
						}
					}
				}
			}
			/* Hard PC Break Remove */
			for( i = 0; i < g_HardPcBreakTable_Index; ++i )
			{
				if( ulTempBreakAddr != g_HardPcBreakTable[ i ] )
				{
					if ( i == 0 )
					{
						/* Set hard PC break 1 */
						iID = C33_ID_HARD_PC_BREAK_CLR;
					}
					else
					{
						/* Set hard PC break 2 */
						iID = C33_ID_HARD_PC_BREAK2;
					}
					iResult = c33_SendMessage(iID, C33_BREAK_DISABLE, 0, 0, 0, 0, 0, 0);
					if (iResult == C33_NO_ERR)
					{
						iResult = c33_RecvMessage(iID, 0, 0, 0, 0, 0);
					}
					if ( iResult != C33_NO_ERR )
					{
						c33_ErrorMsg();
						return;
					}
				}
			}
			
			/* Write Watch Break Remove */
			if( g_WriteWatchBreakAddrFlag == 1 || g_ReadWatchBreakAddrFlag == 1 || g_WriteReadWatchBreakAddrFlag == 1 )
			{
				iResult = c33_SendMessage(C33_ID_DATA_BREAK_CLR, 0, 0, 0, 0, 0, 0, 0);
				if (iResult == C33_NO_ERR)
				{
					iResult = c33_RecvMessage(C33_ID_DATA_BREAK_CLR, 0, 0, 0, 0, 0);
				}
				if ( iResult != C33_NO_ERR )
				{
					c33_ErrorMsg();
					return;
				}
			}
		}
	}
	
	
	ulPC = read_pc();
	
	/* delete temporary break infomation */
	c33_delete_temp_breakpoint( ulPC );
	
	/*======================================*/
	/* Return Execute						*/
	/*======================================*/
	if( ulTempBreakAddr == ulPC )
	{
		if( iPcBreakFlag == 0  ) /* Break by Temprary break ? */
		{
			if(c33TargetInfo.iKeyBreak == 0)	/* No STOP ? */
			{
				/* Return */
				c33icd_fetch_registers( -1 );	/* read new register bug fix T.Tazaki 2004/10/13 */
				ulSP = read_sp();
				ulAddr = read_memory_unsigned_integer (ulSP, 4);
				if (insn_word == INSN_RETI )
				{
					ulSP += 4;
					ulAddr = read_memory_unsigned_integer (ulSP, 4);
				}

				c33_SetMMUMode(0);	/* MMU OFF add T.Tazaki 2004/08/03 */

				/*==================================================*/
				/* Advanced : Set Area break, bus break, bus trace	*/ /* add T.Tazaki 2004/04/22 */
				/*==================================================*/
				c33_SetBusMode();
				
				/* GO */
				iCondition = C33_BIT_ON_TEMP;
				iCondition |= 0x10;   /* execte condition bit 4 : bus break counter : Clear add T.Tazaki 2004/10/29 */
				c33TargetInfo.iIcdExeStatus = C33_EXE_STATUS_RUN;
				if (c33TargetInfo.iOnTheFly == C33_MD_ON_THE_FLY)
				{
				   iCondition |= C33_BIT_ON_THE_FLY;
				}
				iResult = c33_SendMessage(C33_ID_GO, iCondition, ulAddr, 0, 0, 0, 0, 0);
				if (c33TargetInfo.iOnTheFly == C33_MD_ON_THE_FLY)
				{
					
					/* On the fly mode */
					iResult = c33_RecvMessage(C33_ID_GO_ACK, 0, 0, 0, 0, 0);
					if (iResult != C33_NO_ERR){
						c33_ErrorMsg();
						return;
					}

					/* wait break */
					iWaitFirst = 1;		/* add T.Tazaki 2004/12/03 */

					do
					{
#ifdef __CYGWIN__		/* if icdv2 then Wait add T.Tazaki 2004/12/03 */
						if ((c33TargetInfo.iVerRdMonStatus & C33_BIT13_15) == C33_ICD_VER2 ) 
						{
							if( iWaitFirst == 1 )
							{
								iWaitTimer = 500;
								iWaitFirst = 0;
							}
							else
							{
								iWaitTimer = 200;
							}
							ulStartTime = GetTickCount();
							while( (ulStartTime + iWaitTimer) > GetTickCount() )
							{
							}
						}
#endif
						c33_Gdbtk_Idle(); /* re-display */

						iResult = c33_SendMessage(C33_ID_RUN_MON, 0, 0, 0, 0, 0, 0, 0);
						if ( iResult != C33_NO_ERR )
						{
							break;
						}
						/* An ESC break is carried out by the Ctrl-c. */
						old_sigint = signal (SIGINT, c33_IcdStopCtrl_C);
						iResult = c33_RecvMessage(C33_ID_RUN_MON, 0, 0, 0, 0, 0);
						signal (SIGINT, old_sigint);
						if ( iResult != C33_NO_ERR || ucRecvData[2] == C33_ID_GO )
						{
							break;
						}
					}
					while(c33TargetInfo.iKeyBreak == 0);

					if(c33TargetInfo.iKeyBreak != 0)	/* STOP ? */
					{
						c33TargetInfo.iKeyBreak = 0;
						
						/* CANSEL */
						iResult = c33_SendMessage(C33_ID_CAN, 0, 0, 0, 0, 0, 0, 0);
						if (iResult == C33_NO_ERR)
						{
							iResult = c33_RecvMessage(C33_ID_GO, 0, 0, 0, 0, 0);
						}
						if (iResult != C33_NO_ERR){
							c33_ErrorMsg();
							return;
						}

					}
				}
				else
				{
					/* Break response mode */
					iResult = c33_RecvMessage(C33_ID_GO, 0, 0, 0, 0, 0);
					if (iResult != C33_NO_ERR){
						c33_ErrorMsg();
						return;
					}
				}
			}
		}
	}
	c33TargetInfo.iKeyBreak = 0;

	/* refresh window */
	flush_cached_frames ();
	registers_changed ();
	stop_pc = read_pc ();
	set_current_frame (create_new_frame (read_fp (), stop_pc));
	select_frame (get_current_frame (), 0);

	c33_Gdbtk_Update (C33_UPDATE_REG);
}
/****************************************************************************/
/* Format		 : static int c33_IcdAreaBreakStatus_display( void );		*/
/* Arguments	 :	void													*/
/* Return value  : error status												*/
/* Explanation	 : area break status detail display.						*/
/****************************************************************************/
static int
c33_IcdAreaBreakStatus_display()
{
	int iResult = C33_NO_ERR;
	unsigned char ucAreaBusStatus;
	
	iResult = c33_Cmd_ReadWrite(C33_READ, C33_AREA_BREAK_STATUS0, 1, C33_BYTE, (unsigned char *)&ucAreaBusStatus);
	if( iResult == C33_NO_ERR )
	{
		printf_filtered("<<<Area Break Status>>>\n");
		switch( ucAreaBusStatus & 0xc0 )
		{
			case 0x00:	printf_filtered( "  Bus master : CPU\n");	break;
			case 0x40:	printf_filtered( "  Bus master : CCU\n");	break;
			case 0x80:	printf_filtered( "  Bus master : DMA\n");	break;
			case 0xc0:	printf_filtered( "  Bus master : USER\n");	break;
			default:    printf_filtered( "  Bus master : CPU\n");	break;						/* add T.Tazai 2007/07/12 */
		}
		if( ucAreaBusStatus & 0x20 )
		{
			printf_filtered( "  Access     : Read\n" );
		}
		else
		{
			printf_filtered( "  Access     : Write\n" );
		}
		printf_filtered( "  Area       : %d\n",ucAreaBusStatus & 0x1f );

		iResult = c33_Cmd_ReadWrite(C33_READ, C33_AREA_BREAK_STATUS1, 1, C33_BYTE, (unsigned char *)&ucAreaBusStatus);
		if( iResult == C33_NO_ERR )
		{
			if( ucAreaBusStatus & 0x04 )
			{
				printf_filtered( "  Data type  : Instruntion\n" );
			}
			else
			{
				printf_filtered( "  Data type  : Data\n" );
			}
			switch( ucAreaBusStatus & 0x03 )
			{
				case 0x00:	printf_filtered( "  Data size  : Byte\n");		break;
				case 0x01:	printf_filtered( "  Data size  : Half Word\n");	break;
				case 0x02:	printf_filtered( "  Data size  : Word\n");		break;
				default:  	printf_filtered( "  Data size  : Byte\n");		break;/* add T.Tazai 2007/07/12 */
			}
		}
	}
	return iResult;
}

/* add T.Tazaki 2003/07/03 */
/****************************************************************************/
/* Format		 : void c33_CheckPcBreak( unsigned long ulAddr );			*/
/* Arguments	 : unsigned long ulAddr 	  searach address				*/
/* Return value  : 1 = found, 0 = not found 								*/
/* Explanation	 : search pc break address match							*/
/****************************************************************************/
int
c33_CheckPcBreak( unsigned long ulAddr )
{
	int iPcBreakFlag = 0;
	int i;
	
	/* Soft PC Break */
	for( i = 0; i < g_SoftPcBreakTable_Index; ++i )
	{
		if( i < C33_SOFT_PC_BREAK_MAX )
		{
			if( ulAddr == g_SoftPcBreakTable[ i ] )
			{
				iPcBreakFlag = 1;
				break;
			}
		}
	}

	/* Hard PC Break */
	for( i = 0; i < g_HardPcBreakTable_Index; ++i )
	{
		if( ulAddr == g_HardPcBreakTable[ i ] )
		{
			iPcBreakFlag = 1;
			break;
		}
	}
	
	return iPcBreakFlag;
}
/* add T.Tazaki 2004/08/03 >>> */
/****************************************************************************/
/* Format		 : void c33_IcdVer3MMU(int iParamNum, unsigned long ulSP);	*/
/* Arguments																*/
/*	int iParamNum			   : Parameter number							*/
/*	unsigned long ulSP		   : Stack pointer								*/
/* Return value  : void 													*/
/* Explanation	 : Set MMU ON/OFF control and SP address set				*/
/****************************************************************************/
void
c33_IcdVer3MMU(int iParamNum, unsigned long ulSP )
{

	/* Check number of parameter */
	if ( iParamNum != 1 && iParamNum != 2 )
	{
		c33_error (C33_ERR_COMMAND_PARAM);
		return;
	}

	if ( iParamNum == 2 )
	{
		g_ulMMU_SP = ulSP &= 0xfffffffc;
		g_iMMU_Mode = 1;	/* ON */
		c33_SetMMUMode(1);	/* mmu on : add T.Tazaki 2005/02/07 */
		printf_filtered( "MMU control on.\n" );
	}
	else
	{
		c33_SetMMUMode(0);	/* mmu off : add T.Tazaki 2005/02/07 */
		g_iMMU_Mode = 0;	/* OFF */
		printf_filtered( "MMU control off.\n" );
	}
}
/****************************************************************************/
/* Format		 : static void c33_SetMMUMode( int iFlag);					*/
/* Arguments																*/
/* Return value  : int iFlag = 0 MMU Mode OFF, = 1 MMU Mode ON				*/
/* Explanation	 : Set mmu control											*/
/****************************************************************************/
static void
c33_SetMMUMode(int iFlag)
{
	int iResult = C33_NO_ERR;
	unsigned long ulProgram;
	unsigned char ucChar;
	
	if( g_iMMU_Mode == 1 )
	{
		if( iFlag == 1 )
		{
			/*******************/
			/* MMU on sequence */
			/*******************/

			/* (1) */
			/* 0x068fa0ce --> [0x7106c] write		   */
			/* ld.w %ssp,%r12 */
			/* jp	%r15	  */
			ulProgram = 0x068fa0ce;
			iResult = c33_Cmd_ReadWrite(C33_WRITE, 0x7106c, 4, C33_WORD, (unsigned char*)&ulProgram);

			/* (2) "c33 call 0x7106c g_ulMMU_PC 0 0" */
			c33_IcdCall( 0x7106c, g_ulMMU_SP, 0, 0 );

			/* (3) 3 --> [0x78071] write		: 0x7106c=MMUS(1:0) 11b=USE MMU on debug mode, 00b=NO USE */
			ucChar = 0x03;
			iResult = c33_Cmd_ReadWrite(C33_WRITE, 0x78071, 1, C33_BYTE, (unsigned char*)&ucChar);
		}
		else
		{
//			if( g_iGoStepFirst == 0 )	/* is it not first GO/STEPH */	/* del T.Tazaki 2005/02/07 */
//			{
				/********************/
				/* MMU off sequence */
				/********************/

				/* (1) */
				/* 0x068fa0ce --> [0x7106c] write		*/
				/* ld.w %ssp,%r12 */
				/* jp	%r15	  */
				ulProgram = 0x068fa0ce;
				iResult = c33_Cmd_ReadWrite(C33_WRITE, 0x7106c, 4, C33_WORD, (unsigned char*)&ulProgram);

				/* (2) "c33 call 0x7106c 0x70ffc 0 0" */
				c33_IcdCall( 0x7106c, 0x70ffc, 0, 0 );

				/* (3) 0 --> [0x78071] write   : 0x7106c=MMUS(1:0) 11b=USE MMU on debug mode, 00b=NO USE */
				ucChar = 0x0;
				iResult = c33_Cmd_ReadWrite(C33_WRITE, 0x78071, 1, C33_BYTE, (unsigned char*)&ucChar);
				
//			}
//			g_iGoStepFirst = 0;
			
		}
	}
}
/* add T.Tazaki 2004/08/03 <<< */

/****************** Windows and Linux are intermingled. ****************************************/
#ifdef __CYGWIN__

#include <windows.h>
HANDLE hDevice = INVALID_HANDLE_VALUE;

#else

#include "fcntl.h"
#define USB33_1_SEND		   1  /* 1 byte send  */
#define USB33_N_SEND		   2  /* N byte send  */
#define USB33_1_RECV		   3  /* 1 byte recv  */
#define USB33_N_RECV_LEN_HIGH  4  /* N byte recv : length high */
#define USB33_N_RECV_LEN_LOW   5  /* N byte recv : length Low  */
#define USB33_N_RECV		   6  /* N byte recv  */

int iLinux_fd = -1;

#endif

/****************************************************************************/
/* Format		 : void c33_AddCRLF(char* cTraceStr)						*/
/* Arguments																*/
/*	char* cTraceStr : string												*/
/* Return value  : void 													*/
/* Explanation	 : Add CRLF to strings. 									*/
/****************************************************************************/
void c33_AddCRLF(char* cTraceStr)
{
#ifdef __CYGWIN__
	int i;
	i = strlen(cTraceStr);
	cTraceStr[i] = 0x0d;
	cTraceStr[i + 1] = 0x0a;
	cTraceStr[i + 2] = 0x0;
#else
	strcat(cTraceStr, "\n");
#endif
}

/****************************************************************************/
/* Format		 : static void c33_IcdProcessing(unsigned long ulWait)		*/
/* Arguments																*/
/*	unsigned long ulWait : Timer											*/
/* Return value  : void 													*/
/* Explanation	 : "." will be displayed if the appointed time has come.	*/
/****************************************************************************/
static void c33_IcdProcessing(unsigned long ulWait)
{
#ifdef __CYGWIN__
	static DWORD dwStartTime;

	if(ulWait == 0)
	{
		dwStartTime = GetTickCount();
	}
	else
	{
		if( dwStartTime + ulWait < GetTickCount() )
		{
			puts_filtered (".");
			dwStartTime = GetTickCount();
		}
	}
#else
   /* Linux is unnecessary */
#endif
}

/****************************************************************************/
/* Format		 : static int c33_IcdUsbOpen(void)							*/
/* Arguments	 : void 													*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Open usb device driver.									*/
/****************************************************************************/
static int
c33_IcdUsbOpen(void)
{

#ifdef __CYGWIN__

	/* "target icd usbX"  add T.Tazaki 2005/05/16 >>> */

	char *sLinkName0 = "\\\\.\\Icd33v30Device0";
	char *sLinkName1 = "\\\\.\\Icd33v30Device1";
	char *sLinkName2 = "\\\\.\\Icd33v30Device2";

	char *sLinkName0_ICDV6 = "\\\\.\\Icd33v60Device0";	/* for ICDV6 add T.Tazaki 2007/07/02 */
	char *sLinkName1_ICDV6 = "\\\\.\\Icd33v60Device1";	/* for ICDV6 add T.Tazaki 2007/07/02 */
	char *sLinkName2_ICDV6 = "\\\\.\\Icd33v60Device2";	/* for ICDV6 add T.Tazaki 2007/07/02 */

	if( g_iICD_kind == C33_ICDV4 )
	{
		if( g_iUsbNo == 0 )
		{
			hDevice = CreateFile(sLinkName0,
							  GENERIC_READ | GENERIC_WRITE,
							  FILE_SHARE_READ,
							  NULL,
							  OPEN_EXISTING,
							  FILE_FLAG_OVERLAPPED,
							  NULL);
		}
		else if( g_iUsbNo == 1 )
		{
			hDevice = CreateFile(sLinkName1,
							  GENERIC_READ | GENERIC_WRITE,
							  FILE_SHARE_READ,
							  NULL,
							  OPEN_EXISTING,
							  FILE_FLAG_OVERLAPPED,
							  NULL);
		}
		else
		{
			hDevice = CreateFile(sLinkName2,
							  GENERIC_READ | GENERIC_WRITE,
							  FILE_SHARE_READ,
							  NULL,
							  OPEN_EXISTING,
							  FILE_FLAG_OVERLAPPED,
							  NULL);
		}
	}
	else
	{
		/* for ICDV6 add T.Tazaki 2007/07/02 */
		if( g_iUsbNo == 0 )
		{
			hDevice = CreateFile(sLinkName0_ICDV6,
							  GENERIC_READ | GENERIC_WRITE,
							  FILE_SHARE_READ,
							  NULL,
							  OPEN_EXISTING,
							  FILE_FLAG_OVERLAPPED,
							  NULL);
		}
		else if( g_iUsbNo == 1 )
		{
			hDevice = CreateFile(sLinkName1_ICDV6,
							  GENERIC_READ | GENERIC_WRITE,
							  FILE_SHARE_READ,
							  NULL,
							  OPEN_EXISTING,
							  FILE_FLAG_OVERLAPPED,
							  NULL);
		}
		else
		{
			hDevice = CreateFile(sLinkName2_ICDV6,
							  GENERIC_READ | GENERIC_WRITE,
							  FILE_SHARE_READ,
							  NULL,
							  OPEN_EXISTING,
							  FILE_FLAG_OVERLAPPED,
							  NULL);
		}
	}
		
	/* "target icd usbX"  add T.Tazaki 2005/05/16 <<< */
	if(hDevice == INVALID_HANDLE_VALUE)
	{
		return C33_ERR;
	}

#else

	iLinux_fd = open("/dev/usb33", O_RDWR);
	if (iLinux_fd == -1) {
		return C33_ERR;
	}

#endif

	return C33_NO_ERR;
}

/****************************************************************************/
/* Format		 : static void c33_IcdUsbClose(void)						*/
/* Arguments	 : void 													*/
/* Return value  : void 													*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Close usb device driver. 								*/
/****************************************************************************/
static void
c33_IcdUsbClose(void)
{

#ifdef __CYGWIN__

	if (hDevice != INVALID_HANDLE_VALUE)
	{
		CloseHandle(hDevice);
		hDevice = INVALID_HANDLE_VALUE;
	}

#else
	if( iLinux_fd != -1 )
	{
		close(iLinux_fd);
		iLinux_fd = -1;
	}

#endif

}

/****************************************************************************/
/* Format		 : static int c33_IcdUsbWrite(int iLen) 					*/
/* Arguments																*/
/*	int iLen	 : data length												*/
/* Return value  : void 													*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Write to usb device driver.								*/
/****************************************************************************/
static int
c33_IcdUsbWrite(int iLen)
{

#ifdef __CYGWIN__

/* change Syncronouse USB Write T.Tazaki 2004/05/11 >>> */

	DWORD dwWritten;
	BOOL	bResult = 0;					// RW result
	DWORD	dwResult = 0;					// wait result
	char* pBuf = ucSendData;
	DWORD dwTime;
	int	   iFlag;
	OVERLAPPED OverlappedW;
	HANDLE hEventW;
	// Event쐬(for Write)
//	char eventNameW[] = "EVENT_WRITE";
	
	char eventNameW[50];
	if( g_iUsbNo == 0 )
	{
		strcpy( eventNameW, "EVENT_WRITE0" );
	}
	else if( g_iUsbNo == 1 )
	{
		strcpy( eventNameW, "EVENT_WRITE1" );
	}
	else if( g_iUsbNo == 2 )
	{
		strcpy( eventNameW, "EVENT_WRITE2" );
	}
	
	hEventW = CreateEvent(NULL, 
								FALSE,		// AutoReset
								FALSE,		// Initially unsignaled
								eventNameW);
	
	// OVERLAPPED structer(for Write)
	OverlappedW.Internal = 0;				// Reserved for OS
	OverlappedW.InternalHigh = 0;			// Reserved for OS
	OverlappedW.Offset = 0;					// Ignored
	OverlappedW.OffsetHigh = 0;				// Ignored
	OverlappedW.hEvent = hEventW;			// Event Name


	bResult = WriteFile(hDevice, pBuf, iLen, &dwWritten, &OverlappedW);
	do
	{
		dwResult = WaitForSingleObject(hEventW, 0);
	}while( dwResult != WAIT_OBJECT_0 );

/////////////////////////////////////////////////////////////////
//	DWORD dwWritten;
//	char* pBuf = ucSendData;
//	DWORD dwTime;
//
//	dwTime = GetTickCount();/* Get time */
//	while ( iLen != 0 )
//	{
//		WriteFile(hDevice, pBuf, iLen, &dwWritten, NULL);
//		pBuf += dwWritten;
//		iLen -= dwWritten;
//		if ( (iLen != 0) && (dwTime + 3000 < GetTickCount()) )
//		{
//			return C33_ERR;
//		}
//	}
/////////////////////////////////////////////////////////////////

/* change Syncronouse USB Write T.Tazaki 2004/05/11 <<< */

#else
	int   i;
	unsigned char  ucData;
	unsigned char* pucBuf = ucSendData;

	if( iLen == 0 ) return C33_ERR;
	if( iLen == 1 )
	{
		ucData = *pucBuf;
		ioctl(iLinux_fd, USB33_1_SEND, ucData);
	}
	else
	{
		for( i = 0; i < iLen; ++i )
		{
			if (ioctl(iLinux_fd, USB33_N_SEND, *pucBuf) == -1) {
				return C33_ERR;
			}
			++pucBuf;
		}
	}

#endif

	return C33_NO_ERR;
}

/****************************************************************************/
/* Format		 : static int c33_IcdUsbRead(int iLen, int iTimeOut, unsigned char* pucRecvData) */
/* Arguments																*/
/*	int iLen				   : data length								*/
/*	int iTimeOut			   : timeout									*/
/*	unsigned char* pucRecvData : read data buffer							*/
/* Return value  : void 													*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Read from usb device driver. 							*/
/****************************************************************************/
static int
c33_IcdUsbRead(int iLen, int iTimeOut, unsigned char* pucRecvData)
{

#ifdef __CYGWIN__

/* change ASyncronouse USB Read T.Tazaki 2004/05/11 >>> */

	DWORD dwRead = 0;						/* Oǉ 2005/01/19 */
	DWORD dwTime;
	DWORD dwDelayTime;
	BOOL	bResult = 0;					// RW result
	DWORD	dwResult = 0;					// wait result
	OVERLAPPED OverlappedR;
	int	   iFlag;
	extern int (*ui_loop_hook) (int);
//	extern int g_iGdbtk_Update_cnt;		/* add T.Tazaki 2004/09/02 */ /* del T.Tazaki 2008/06/04 */
	extern Tcl_Interp *gdbtk_interp;	/* add T.Tazaki 2004/09/02 */
	HANDLE hEventR;
	
	// Create Event(for Read)
//	char eventNameR[] = "EVENT_READ";
	char eventNameR[50];
	if( g_iUsbNo == 0 )
	{
		strcpy( eventNameR, "EVENT_READ0" );
	}
	else if( g_iUsbNo == 1 )
	{
		strcpy( eventNameR, "EVENT_READ1" );
	}
	else if( g_iUsbNo == 2 )
	{
		strcpy( eventNameR, "EVENT_READ2" );
	}

	hEventR = CreateEvent(NULL,
								 FALSE,
								 FALSE,
								 eventNameR);
	
	// Overlapped(for Read)
	OverlappedR.Internal = 0;				// Reserved for OS
	OverlappedR.InternalHigh = 0;			// Reserved for OS
	OverlappedR.Offset = 0;					// Ignored
	OverlappedR.OffsetHigh = 0;				// Ignored
	OverlappedR.hEvent = hEventR;			// Event Name


  	bResult = ReadFile(hDevice, pucRecvData, iLen, &dwRead, &OverlappedR);

	g_iStep_Cancel = 0;	/* STEP cancel flag OFF */
	dwTime = GetTickCount();/* Get time */

	do
	{
		dwResult = WaitForSingleObject(hEventR, 0);
		if( dwResult == WAIT_OBJECT_0 )
		{
			break;
		}
		
	   	if( g_iID == C33_ID_STEP ) /* check Step command */
		{
			g_iNow_OnUSBRead = 1;
			/* Check cancel */
			/* when  c33_watch_update_flag=="OFF",DO NOTAupdate T.Tazaki 2004/09/02 */
			if( g_use_windows == 1 )  /* GUI Mode ? */										/* add T.Tazaki 2004/09/02 */
			{																				/* add T.Tazaki 2004/09/02 */
				Tcl_SetVar( gdbtk_interp,"c33_watch_update_flag","OFF",TCL_GLOBAL_ONLY );  	/* add T.Tazaki 2004/09/02 */
			}																				/* add T.Tazaki 2004/09/02 */
			if (c33TargetInfo.iKeyBreak == 1 || c33_CheckCancel( 0 ) == 1 ) 
			{
				g_iStep_Cancel = 1;		/* STEP cancel flag ON */
				if( g_use_windows == 1 )  /* GUI Mode ? */										/* add T.Tazaki 2004/09/02 */
				{																				/* add T.Tazaki 2004/09/02 */
					Tcl_SetVar( gdbtk_interp,"c33_watch_update_flag","ON",TCL_GLOBAL_ONLY );  	/* add T.Tazaki 2004/09/02 */
				}																				/* add T.Tazaki 2004/09/02 */
				return C33_ERR;
			}
			if( g_use_windows == 1 )  /* GUI Mode ? */										/* add T.Tazaki 2004/09/02 */
			{																				/* add T.Tazaki 2004/09/02 */
				Tcl_SetVar( gdbtk_interp,"c33_watch_update_flag","ON",TCL_GLOBAL_ONLY );  	/* add T.Tazaki 2004/09/02 */
			}																				/* add T.Tazaki 2004/09/02 */
			g_iNow_OnUSBRead = 0;

			/* wait */
			dwDelayTime = GetTickCount();/* Get time */
			do
			{
				dwResult = WaitForSingleObject(hEventR, 0);
				if( dwResult == WAIT_OBJECT_0 )
				{
					break;
				}
				if( (dwDelayTime + 500) < GetTickCount() )
				{
					break;
				}
			}while(1);
		}
		else
		{
			if( iTimeOut > 0 )
			{
				/* check time out */
				if ( (iTimeOut * 1000) < ( GetTickCount() - dwTime ))
				{
					return C33_ERR;
				}
			}
		}
	}while( dwResult != WAIT_OBJECT_0 );


	/* dummy funstion for ctrl-c (CUI USB mode) */
	if( g_use_windows == 0 )  /* CUI Mode ? */
	{
		c33_dummy();	
	}

/////////////////////////////////////////////////////////////////
//	DWORD dwRead;
//	DWORD dwTime;
//	extern int (*ui_loop_hook) (int);
//
//
//	dwTime = GetTickCount();/* Get time */
//	while ( iLen != 0 )
//	{
//		ReadFile(hDevice, pucRecvData, iLen, &dwRead, NULL);
//		pucRecvData += dwRead;
//		iLen		-= dwRead;
//
//		if ( (iLen != 0) && (dwTime + 3000 < GetTickCount()) )
//		{
//			return C33_ERR;
//		}
//		
//		/* dummy funstion for ctrl-c (CUI USB mode) */
//		if( g_use_windows == 0 )  /* CUI Mode ? */
//		{
//			c33_dummy();	
//		}
//	}
/////////////////////////////////////////////////////////////////

/* change Syncronouse USB Read T.Tazaki 2004/05/11 <<< */

#else
	int iData;
	int i;
	unsigned char* pucBuf = pucRecvData;

	if( iLen == 0 ) return C33_ERR;
	if( iLen == 1 )
	{
		ioctl(iLinux_fd, USB33_1_RECV, pucRecvData);
	}
	else
	{
		iData = iLen >> 8;	/* Length High */
		if ( ioctl(iLinux_fd, USB33_N_RECV_LEN_HIGH, iData) < 0) {
			return C33_ERR;
		}
		iData = iLen & 0xff; /* Length Low */
		if ( ioctl(iLinux_fd, USB33_N_RECV_LEN_LOW, iData) < 0) {
			return C33_ERR;
		}

		for( i = 0; i < iLen; ++i )
		{
			if(ioctl(iLinux_fd, USB33_N_RECV, pucBuf) < 0 )
			{
				return C33_ERR;
			}

			++pucBuf;
		}

	}

#endif
	return C33_NO_ERR;
}

#ifdef __CYGWIN__
/****************************************************************************/
/* Format		 : static void c33_dummy (void) 							*/
/* Arguments																*/
/* Return value 															*/
/* Explanation	 : icd usb cui mode , for ctrl-c enabled					*/
/*					gdbtk.imi file open and close( low-level C function )	*/
/****************************************************************************/
static void
c33_dummy()
{
	HANDLE hHandle = INVALID_HANDLE_VALUE;
	int iSize;
	char szPath[260];
	int  i;
	static FILE *fp = NULL;

	GetModuleFileName(hHandle, szPath, 256 );

	iSize = strlen( szPath );
	for( i = (iSize - 1); i >= 0; --i )
	{
		if( szPath[i] == '\\' )
		{
			break;
		}
	}
	szPath[i+1] = (char)NULL;
										/*				  012345678901234567  */
	strcat( szPath,"gdbtk.ini" );	/* sample szPath="c:\gnu33\gdbtk.ini" */
	
	for( i = 0; i < 10; ++i )
	{
		fp = fopen( szPath,"r" );
		if( fp != NULL )
		{
			fclose( fp );
			fp = NULL;
		}
	}
}
#endif

/****************************************************************************/
/* Format		 : static int c33_IcdRecvSpRegisterRead (int iID, unsigned char* pucRecvData) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucRecvData : Message buffer 							*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : The received register read data is stored.				*/
/****************************************************************************/
static int
c33_IcdRecvSpRegisterRead (int iID, unsigned char* pucRecvData)
{
	unsigned char ucRegTmp[REGISTER_SIZE];
	ucRegTmp[0] = 0;
	ucRegTmp[1] = 0;
	ucRegTmp[2] = 0;
	ucRegTmp[3] = 0;


	if (iID == C33_ID_SP_REG_RD_0)
	{
		/* Standard macro or ICD33Ver2.0 */
	}
	else
	{
		/* Advanced macro and ICD33Ver3.0 */

		/* Receive data check */
		if( c33_IcdRecvDataCheck(iID,pucRecvData) != C33_NO_ERR)
		{
			return C33_ERR;
		}

		pucRecvData++;
		pucRecvData++;
		pucRecvData++;

		/* Skip area break stop area */
		pucRecvData++;


		/* Preservation of register data.  */

		/* Read LCO */
		ucRegTmp[3] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[2] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[1] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[0] = *pucRecvData;
		pucRecvData++;
		supply_register(LCO_REGNUM,ucRegTmp);
		
		/* Read LSA */
		ucRegTmp[3] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[2] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[1] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[0] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[0] &= C33_LSA_MASK;
		supply_register(LSA_REGNUM,ucRegTmp);
		
		/* Read LEA */
		ucRegTmp[3] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[2] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[1] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[0] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[0] &= C33_LEA_MASK;
		supply_register(LEA_REGNUM,ucRegTmp);
		
		/* Read SOR */
		ucRegTmp[3] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[2] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[1] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[0] = *pucRecvData;
		pucRecvData++;
		supply_register(SOR_REGNUM,ucRegTmp);
		
		/* Read TTBR */
		ucRegTmp[3] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[2] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[1] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[0] = *pucRecvData;
		pucRecvData++;

		ucRegTmp[0] = 0;
		ucRegTmp[1] &= 0xfc;
		supply_register(TTBR_REGNUM,ucRegTmp);
		
		/* Read DP */
		ucRegTmp[3] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[2] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[1] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[0] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[0] &= C33_DP_MASK;
		supply_register(DP_REGNUM,ucRegTmp);
		
		/* Read IDIR */
		ucRegTmp[3] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[2] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[1] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[0] = *pucRecvData;
		pucRecvData++;
		supply_register(IDIR_REGNUM,ucRegTmp);	/* add T.Tazaki 2003/11/25 */
		
		/* Read DBBR */
		ucRegTmp[3] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[2] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[1] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[0] = *pucRecvData;
		pucRecvData++;
		supply_register(DBBR_REGNUM,ucRegTmp);	/* add T.Tazaki 2003/11/25 */
		
		/* (resreved) skip */
		pucRecvData++;
		pucRecvData++;
		pucRecvData++;
		pucRecvData++;
		
		/* Read USP */
		ucRegTmp[3] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[2] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[1] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[0] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[0] &= C33_SP_MASK;
		supply_register(USP_REGNUM,ucRegTmp);

		/* check %psr : SV == 1(User Mode) 		add T.Tazaki 2004/08/26 >>> */
		if( (registers[REGISTER_BYTE(PSR_REGNUM) + 1] & 0x10 ) == 0x10 )
		{
			/* update %usp to %sp */
			supply_register(SP_REGNUM,ucRegTmp);
		}
		/* add T.Tazaki 2004/08/26 <<< */
		
		/* (resreved) skip	add T.Tazaki 2003/12/05 */
		pucRecvData++;
		pucRecvData++;
		pucRecvData++;
		pucRecvData++;

#if 0	/*del T.Tazaki 2003/12/05 >>> */
		/* Read SSP */
		ucRegTmp[3] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[2] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[1] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[0] = *pucRecvData;
		pucRecvData++;
		ucRegTmp[0] &= C33_SP_MASK;
		supply_register(SSP_REGNUM,ucRegTmp);
#endif
	}
	return C33_NO_ERR;
}



/****************************************************************************/
/* Format		 : static int c33_IcdMakeMessageSpRegWrite (int iID, unsigned char* pucSendData) */
/* Arguments																*/
/*	int iID 				   : Message index								*/
/*	unsigned char* pucSendData : Message buffer 							*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Special Register write message data is created.			*/
/****************************************************************************/
static int
c33_IcdMakeMessageSpRegWrite (int iID, unsigned char* pucSendData)
{
	int iSendDataNum = 0; /* size(byte) of sending data */
	unsigned char* pucSendDataTop = pucSendData;

	if (iID == C33_ID_SP_REG_WT_0)
	{
		/* Standard macro or ICD33Ver2.0 */
	}
	else
	{
		/* Advanced macro and ICD33Ver3.0 */

		pucSendData++;
		pucSendData++;
		*pucSendData = (unsigned char)iID;
		pucSendData++;
		iSendDataNum++;

		/* Start register value write to buffers. (fixed) */
		*pucSendData = C33_COMMAND_START_REG_NUM;
		pucSendData++;
		iSendDataNum++;

		/* Special register value write to buffers. */
		
		/* Write LCO */
		*(pucSendData + 0) = registers[REGISTER_BYTE(LCO_REGNUM) + 3];
		*(pucSendData + 1) = registers[REGISTER_BYTE(LCO_REGNUM) + 2];
		*(pucSendData + 2) = registers[REGISTER_BYTE(LCO_REGNUM) + 1];
		*(pucSendData + 3) = registers[REGISTER_BYTE(LCO_REGNUM) + 0];
		pucSendData+=4;
		iSendDataNum+=4;

		/* Write LSA */
		*(pucSendData + 0) = registers[REGISTER_BYTE(LSA_REGNUM) + 3];
		*(pucSendData + 1) = registers[REGISTER_BYTE(LSA_REGNUM) + 2];
		*(pucSendData + 2) = registers[REGISTER_BYTE(LSA_REGNUM) + 1];
		*(pucSendData + 3) = registers[REGISTER_BYTE(LSA_REGNUM) + 0] & C33_LSA_MASK;
		registers[REGISTER_BYTE(LSA_REGNUM) + 0] &= C33_LSA_MASK;	/* add T.Tazaki 2005/06/27 */
		pucSendData+=4;
		iSendDataNum+=4;

		/* Write LEA */
		*(pucSendData + 0) = registers[REGISTER_BYTE(LEA_REGNUM) + 3];
		*(pucSendData + 1) = registers[REGISTER_BYTE(LEA_REGNUM) + 2];
		*(pucSendData + 2) = registers[REGISTER_BYTE(LEA_REGNUM) + 1];
		*(pucSendData + 3) = registers[REGISTER_BYTE(LEA_REGNUM) + 0] & C33_LEA_MASK;
		registers[REGISTER_BYTE(LEA_REGNUM) + 0] &= C33_LEA_MASK;	/* add T.Tazaki 2005/06/27 */
		pucSendData+=4;
		iSendDataNum+=4;

		/* Write SOR */
		*(pucSendData + 0) = registers[REGISTER_BYTE(SOR_REGNUM) + 3];
		*(pucSendData + 1) = registers[REGISTER_BYTE(SOR_REGNUM) + 2];
		*(pucSendData + 2) = registers[REGISTER_BYTE(SOR_REGNUM) + 1];
		*(pucSendData + 3) = registers[REGISTER_BYTE(SOR_REGNUM) + 0];
		pucSendData+=4;
		iSendDataNum+=4;

		/* Write TTBR */
		*(pucSendData + 0) = registers[REGISTER_BYTE(TTBR_REGNUM) + 3];
		*(pucSendData + 1) = registers[REGISTER_BYTE(TTBR_REGNUM) + 2];
		*(pucSendData + 2) = registers[REGISTER_BYTE(TTBR_REGNUM) + 1] & 0xfc;
		registers[REGISTER_BYTE(TTBR_REGNUM) + 1] &= 0xfc;	/* add T.Tazaki 2005/06/27 */
		*(pucSendData + 3) = registers[REGISTER_BYTE(TTBR_REGNUM) + 0] & 0x00;
		registers[REGISTER_BYTE(TTBR_REGNUM) + 0] &= 0x00;	/* add T.Tazaki 2005/06/27 */
		pucSendData+=4;
		iSendDataNum+=4;

		/* Write DP */
		*(pucSendData + 0) = registers[REGISTER_BYTE(DP_REGNUM) + 3];
		*(pucSendData + 1) = registers[REGISTER_BYTE(DP_REGNUM) + 2];
		*(pucSendData + 2) = registers[REGISTER_BYTE(DP_REGNUM) + 1];
		*(pucSendData + 3) = registers[REGISTER_BYTE(DP_REGNUM) + 0] & C33_DP_MASK;
		registers[REGISTER_BYTE(DP_REGNUM) + 0] &= C33_DP_MASK;	/* add T.Tazaki 2005/06/27 */
		pucSendData+=4;
		iSendDataNum+=4;

		/* Write IDIR  					  add T.Tazaki 2004/01/19 */
		*(pucSendData + 0) = ( g_ulIdir & 0xff000000 ) >> 24;
		*(pucSendData + 1) = ( g_ulIdir & 0x00ff0000 ) >> 16;
		*(pucSendData + 2) = ( g_ulIdir & 0x0000ff00 ) >> 8;
		*(pucSendData + 3) = g_ulIdir & 0x000000ff;
		pucSendData+=4;
		iSendDataNum+=4;

		/* Write DBBR  					  add T.Tazaki 2004/01/19 */
		*(pucSendData + 0) = ( g_ulDbbr & 0xff000000 ) >> 24;
		*(pucSendData + 1) = ( g_ulDbbr & 0x00ff0000 ) >> 16;
		*(pucSendData + 2) = ( g_ulDbbr & 0x0000ff00 ) >> 8;
		*(pucSendData + 3) = g_ulDbbr & 0x000000ff;
		pucSendData+=4;
		iSendDataNum+=4;

		/* Reserve */
		pucSendData+=4;
		iSendDataNum+=4;

		/* Write USP */
		*(pucSendData + 0) = registers[REGISTER_BYTE(USP_REGNUM) + 3];
		*(pucSendData + 1) = registers[REGISTER_BYTE(USP_REGNUM) + 2];
		*(pucSendData + 2) = registers[REGISTER_BYTE(USP_REGNUM) + 1];
		*(pucSendData + 3) = registers[REGISTER_BYTE(USP_REGNUM) + 0] & C33_SP_MASK;
		registers[REGISTER_BYTE(USP_REGNUM) + 0] &= C33_SP_MASK;	/* add T.Tazaki 2005/06/27 */
		pucSendData+=4;
		iSendDataNum+=4;

#if 0 	/* del T.Tazaki 2003/12/05 */
		/* Write SSP */
		*(pucSendData + 0) = registers[REGISTER_BYTE(SSP_REGNUM) + 3];
		*(pucSendData + 1) = registers[REGISTER_BYTE(SSP_REGNUM) + 2];
		*(pucSendData + 2) = registers[REGISTER_BYTE(SSP_REGNUM) + 1];
		*(pucSendData + 3) = registers[REGISTER_BYTE(SSP_REGNUM) + 0] & C33_SP_MASK;
#endif
		pucSendData+=4;
		iSendDataNum+=4;

		*(pucSendDataTop + 0) = (unsigned char)((iSendDataNum >>  8) & 0xff);
		*(pucSendDataTop + 1) = (unsigned char)((iSendDataNum >>  0) & 0xff);

		c33_IcdMakeBCC(pucSendDataTop);/* set BCC data */

		iSendDataNum += (2 + 1); /* add size(2)+bcc(1) */
	}
	return(iSendDataNum);

}
/****************************************************************************/
/* Format		 : void c33_ttbr_reg_get (void) 							*/
/* Arguments																*/
/* Explanation	 : Get TTBR register value. 								*/
/****************************************************************************/
unsigned long
c33_ttbr_reg_get()
{
	unsigned long ulTTBR = 0;

	ulTTBR = (unsigned long)(registers[REGISTER_BYTE(TTBR_REGNUM) + 3] & 0x00ff);	/* bug fix add "& 0x00ff" T.Tazaki 2004/08/18 */
	ulTTBR <<= 8;
	ulTTBR += (unsigned long)(registers[REGISTER_BYTE(TTBR_REGNUM) + 2]& 0x00ff);	/* bug fix add "& 0x00ff" T.Tazaki 2004/08/18 */
	ulTTBR <<= 8;
	ulTTBR += (unsigned long)(registers[REGISTER_BYTE(TTBR_REGNUM) + 1]& 0x00ff);	/* bug fix add "& 0x00ff" T.Tazaki 2004/08/18 */
	ulTTBR <<= 8;
	ulTTBR += (unsigned long)(registers[REGISTER_BYTE(TTBR_REGNUM) + 0]& 0x00ff);	/* bug fix add "& 0x00ff" T.Tazaki 2004/08/18 */
	
	return(ulTTBR);
}
/****************************************************************************/
/* Format		 : void c33_Default_reg_set (void)							*/
/* Arguments																*/
/* Explanation	 : Register default data is created.						*/
/****************************************************************************/
void
c33_Default_reg_set()
{
	int iReg;

	/* %r0 -- %r15 */
	for ( iReg = R0_REGNUM ; iReg <= (R0_REGNUM+15) ; iReg++ )
	{
		registers[REGISTER_BYTE(iReg) + 3] = 0xaa;
		registers[REGISTER_BYTE(iReg) + 2] = 0xaa;
		registers[REGISTER_BYTE(iReg) + 1] = 0xaa;
		registers[REGISTER_BYTE(iReg) + 0] = 0xaa;
	}
	
	/* SP */
	registers[REGISTER_BYTE(SP_REGNUM) + 3] = 0;
	registers[REGISTER_BYTE(SP_REGNUM) + 2] = 0;
	registers[REGISTER_BYTE(SP_REGNUM) + 1] = 0;
	registers[REGISTER_BYTE(SP_REGNUM) + 0] = 0;

	/* PSR */
	registers[REGISTER_BYTE(PSR_REGNUM) + 3] = 0;
	registers[REGISTER_BYTE(PSR_REGNUM) + 2] = 0;
	registers[REGISTER_BYTE(PSR_REGNUM) + 1] = 0;
	registers[REGISTER_BYTE(PSR_REGNUM) + 0] = 0;

	/* AHR */
	registers[REGISTER_BYTE(AHR_REGNUM) + 3] = 0xaa;
	registers[REGISTER_BYTE(AHR_REGNUM) + 2] = 0xaa;
	registers[REGISTER_BYTE(AHR_REGNUM) + 1] = 0xaa;
	registers[REGISTER_BYTE(AHR_REGNUM) + 0] = 0xaa;

	/* ALR */
	registers[REGISTER_BYTE(ALR_REGNUM) + 3] = 0xaa;
	registers[REGISTER_BYTE(ALR_REGNUM) + 2] = 0xaa;
	registers[REGISTER_BYTE(ALR_REGNUM) + 1] = 0xaa;
	registers[REGISTER_BYTE(ALR_REGNUM) + 0] = 0xaa;

}
/****************************************************************************/
/* Format		 : void c33_Default_special_reg_set (int iType) 			*/
/* Arguments																*/
/*	int iType	 : 0 - Hot reset, 1 - Cold reset							*/
/* Explanation	 : Special Register default data is created.				*/
/****************************************************************************/
void
c33_Default_special_reg_set(int iType)
{
		/* LCO */
		registers[REGISTER_BYTE(LCO_REGNUM) + 3] = 0;
		registers[REGISTER_BYTE(LCO_REGNUM) + 2] = 0;
		registers[REGISTER_BYTE(LCO_REGNUM) + 1] = 0;
		registers[REGISTER_BYTE(LCO_REGNUM) + 0] = 0;

		/* LSA */
		registers[REGISTER_BYTE(LSA_REGNUM) + 3] = 0;
		registers[REGISTER_BYTE(LSA_REGNUM) + 2] = 0;
		registers[REGISTER_BYTE(LSA_REGNUM) + 1] = 0;
		registers[REGISTER_BYTE(LSA_REGNUM) + 0] = 0;

		/* LEA */
		registers[REGISTER_BYTE(LEA_REGNUM) + 3] = 0;
		registers[REGISTER_BYTE(LEA_REGNUM) + 2] = 0;
		registers[REGISTER_BYTE(LEA_REGNUM) + 1] = 0;
		registers[REGISTER_BYTE(LEA_REGNUM) + 0] = 0;

		/* SOR */
		registers[REGISTER_BYTE(SOR_REGNUM) + 3] = 0;
		registers[REGISTER_BYTE(SOR_REGNUM) + 2] = 0;
		registers[REGISTER_BYTE(SOR_REGNUM) + 1] = 0;
		registers[REGISTER_BYTE(SOR_REGNUM) + 0] = 0;

		/* TTBR */
		if( iType == C33_ICD_COLD_RESET )
		{
			if( c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV )
			{
				/* ADV */
				registers[REGISTER_BYTE(TTBR_REGNUM) + 3] = 0x20;
				registers[REGISTER_BYTE(TTBR_REGNUM) + 2] = 0;
				registers[REGISTER_BYTE(TTBR_REGNUM) + 1] = 0;
				registers[REGISTER_BYTE(TTBR_REGNUM) + 0] = 0;
			}
			else
			{
#if 0	/* del bug fix 2005/01/25 T.Tazaki >>> Do not default setting od */
				/* PE */		/* add T.Tazaki 2004/08/09 */
				registers[REGISTER_BYTE(TTBR_REGNUM) + 3] = 0;
				registers[REGISTER_BYTE(TTBR_REGNUM) + 2] = 0xc0;
				registers[REGISTER_BYTE(TTBR_REGNUM) + 1] = 0;
				registers[REGISTER_BYTE(TTBR_REGNUM) + 0] = 0;
#endif
			}
		}

		/* DP */
		registers[REGISTER_BYTE(DP_REGNUM) + 3] = 0;
		registers[REGISTER_BYTE(DP_REGNUM) + 2] = 0;
		registers[REGISTER_BYTE(DP_REGNUM) + 1] = 0;
		registers[REGISTER_BYTE(DP_REGNUM) + 0] = 0;

		/* USP */
		registers[REGISTER_BYTE(USP_REGNUM) + 3] = 0;
		registers[REGISTER_BYTE(USP_REGNUM) + 2] = 0;
		registers[REGISTER_BYTE(USP_REGNUM) + 1] = 0;
		registers[REGISTER_BYTE(USP_REGNUM) + 0] = 0;

		/* SSP */
		registers[REGISTER_BYTE(SSP_REGNUM) + 3] = 0;
		registers[REGISTER_BYTE(SSP_REGNUM) + 2] = 0;
		registers[REGISTER_BYTE(SSP_REGNUM) + 1] = 0;
		registers[REGISTER_BYTE(SSP_REGNUM) + 0] = 0;
}
/****************************************************************************/
/* Format		 : c33_pc_reg_set(unsigned long ulPC);						*/
/* Arguments	 : unsigned long ulPC										*/
/* Explanation	 : PC Register is setted.									*/
/****************************************************************************/
void
c33_pc_reg_set(unsigned long ulPC)
{
		/* PC */
		registers[REGISTER_BYTE(PC_REGNUM) + 3] = (ulPC & 0xff000000) >> 24;
		registers[REGISTER_BYTE(PC_REGNUM) + 2] = (ulPC & 0x00ff0000) >> 16;
		registers[REGISTER_BYTE(PC_REGNUM) + 1] = (ulPC & 0x0000ff00) >> 8;
		registers[REGISTER_BYTE(PC_REGNUM) + 0] = (ulPC & 0x000000ff);
}
/****************************************************************************/
/* Format		 : void c33_pc_reg_get (void)								*/
/* Arguments																*/
/* Explanation	 : Get PC register value.									*/
/****************************************************************************/
unsigned long
c33_pc_reg_get()
{
	unsigned long ulPC = 0;
	
	ulPC = registers[REGISTER_BYTE(PC_REGNUM) + 3];
	ulPC <<= 8;
	ulPC += registers[REGISTER_BYTE(PC_REGNUM) + 2];
	ulPC <<= 8;
	ulPC += registers[REGISTER_BYTE(PC_REGNUM) + 1];
	ulPC <<= 8;
	ulPC += registers[REGISTER_BYTE(PC_REGNUM) + 0];
	
	return(ulPC);
}

/****************************************************************************/
/* Format		 : void c33_IcdDebug(int iParam1, int iParam2, int iParam3, int iParam4)	 */
/* Arguments																*/
/* Explanation	 : ICD debug command.										*/
/****************************************************************************/
void
c33_IcdDebug(int iParam1, int iParam2, int iParam3, int iParam4)
{
	static FILE *fp = NULL;
	char *pFilename = "c:/gdb_debug.txt";
	char cBuf[5000];
	char cTemp[5000];
	int i;

	switch (iParam1)
	{
		case 1:
			c33IcdTraceData.ucDebugIdleDsp = 1;
			break;
		case 2:
			/* file delete */
			iDebugCommunication = 1;/* debug enable */
			fp = fopen(pFilename, "w");
			if (fp != NULL)
			{
				/* file close */
				fclose(fp);
				fp = NULL;
			}
			break;
		case 3:
			fp = fopen(pFilename, "a+");
			if (fp != NULL)
			{
				if(iParam2 == 0)
				{
					sprintf(cBuf, "----- Messages -----\n");
					/* send */
					if(ucSendData[0] == C33_ID_CAN)
					{
						strcat(cBuf, " Send CAN\n");
					}
					else if(ucSendData[0] == C33_ID_OK)
					{
						strcat(cBuf, " Send OK?\n");
					}
					else
					{
						sprintf(cTemp, " Send ID   = %0x\n", (int)ucSendData[2]);
						strcat(cBuf, cTemp);
						sprintf(cTemp, " Send Data =");
						strcat(cBuf, cTemp);
						for( i = 0 ; i < ((int)ucSendData[0] << 8) + (int)ucSendData[1] ; i++)
						{
							sprintf(cTemp, " %02X", ucSendData[i+3]);
							strcat(cBuf,cTemp);
						}
						strcat(cBuf, "\n");
					}
					fputs(cBuf, fp);

				}
				else if(iParam2 == 1)
				{
					/* receive */
					cBuf[0] = 0;
					if(iParam3 != 0)
					{
						strcat(cBuf, " Communication error\n");
					}
					else
					{
						if(ucRecvData[0] >= C33_ICD_ACK)
						{
							if(ucRecvData[0] >= C33_ICD_ACK)
							{
								strcat(cBuf, " Recv ACK\n");
							}
							else if(ucRecvData[0] >= C33_ICD_ERR)
							{
								strcat(cBuf, " Recv ERR");
								for( i = 0 ; i < 9 ; i++)
								{
									sprintf(cTemp, " %02X", ucSendData[i+1]);
									strcat(cBuf,cTemp);
								}
								strcat(cBuf, "\n");
							}
							else if((ucRecvData[0] & C33_ICD_STS) >= C33_ICD_STS)
							{
								strcat(cBuf, " Recv STS\n");
							}
							else if((ucRecvData[0] & C33_ICD_NAK) >= C33_ICD_NAK)
							{
								strcat(cBuf, " Recv NAK\n");
							}
						}
						else
						{
							sprintf(cTemp, " Recv ID   = %0x\n", (int)ucRecvData[2]);
							strcat(cBuf, cTemp);
							sprintf(cTemp, " Recv Data =");
							strcat(cBuf, cTemp);
							for( i = 0 ; i < (int)ucRecvData[0]+(int)ucRecvData[1] ; i++)
							{
								sprintf(cTemp, " %02X", ucRecvData[i+3]);
								strcat(cBuf,cTemp);
							}
							strcat(cBuf, "\n");
						}
					}
					fputs(cBuf, fp);
				}
				/* file close */
				fclose(fp);
				fp = NULL;
			}
			break;
		case 4:
			iDebugCommunication = 0;/* debug disable */
			break;
		case 5:
			c33TargetInfo.iOnTheFly = iParam2;
			break;
		case 6:
			if (c33TargetInfo.ucAdvanced == C33_PRODUCT_STD)	/* add T.Tazaki 2003/06/16 */
			{
				printf_filtered( "\nCPU mode is Standard macro.\n");
			}
			else if (c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV)	 /* add T.Tazaki 2005/02/02 */
			{
				printf_filtered( "\nCPU mode is Advanced macro.\n");
			}
			else
			{
				printf_filtered( "\nCPU mode is PE.\n");		/* add T.Tazaki 2005/02/02 */
			}
			break;
		case 7:
			iDebugTrace = 1;/* trace data dump enable */
			break;
		case 8:
			iDebugTrace = 0;/* trace data dump disable */
			break;
		case 9:
			iDebugTrace = 2;/* PC + BUS trace data dump enable , When "c33 otd 1" command */
			break;
		case 10:
			iDebugTrace = 0;/* trace data dump disable */
			break;
		case 11:
			iDebugTrace = 3;/* bus trace dump file output enable */
			break;
		case 12:
			iDebugTrace = 0;/* bus trace dump file output enable */
			break;
		case 13:
			g_iDebugRun = 1;  /* debug exectution debug message enable */
			break;
		case 14:
			g_iDebugRun = 0;  /* debug exectution debug message enable */
			break;
		default:								/* add T.Tazai 2007/07/12 */
			break;
	}
}

/****************************************************************************/
/* Format		 : void c33_UsbLog(int iParam1, int iParam2, char *pLogFileName) */
/* Arguments																*/
/* Explanation	 : ICD debug command.										*/
/****************************************************************************/
void
c33_UsbLog(int iParam1, int iParam2, char *pLogFileName)
{
	static FILE *fp = NULL;
	static char cFileName[300];
	char cBuf[5000];
	char cTemp[5000];
	int i;


	switch (iParam1)
	{
		case 1:
			/* file delete */
			iUsbLogCommunication = 1;/* debug enable */
			strcpy( cFileName, pLogFileName );
			fp = fopen(cFileName, "w");
			if (fp != NULL)
			{
				/* file close */
				fclose(fp);
				fp = NULL;
			}
			break;
		case 2:
			fp = fopen(cFileName, "a+");
			if (fp != NULL)
			{
				if(iParam2 == 0)
				{
					sprintf(cBuf, "----- Messages -----\n");
					/* send */
					if(ucSendData[0] == C33_ID_CAN)
					{
						strcat(cBuf, " Send CAN\n");
					}
					else if(ucSendData[0] == C33_ID_OK)
					{
						strcat(cBuf, " Send OK?\n");
					}
					else
					{
						sprintf(cTemp, " Send ID   = %0x\n", (int)ucSendData[2]);
						strcat(cBuf, cTemp);
						sprintf(cTemp, " Send Data =");
						strcat(cBuf, cTemp);
						for( i = 0 ; i < ((int)ucSendData[0] << 8) + (int)ucSendData[1] ; i++)
						{
							sprintf(cTemp, " %02X", ucSendData[i+3]);
							strcat(cBuf,cTemp);
						}
						strcat(cBuf, "\n");
					}
					fputs(cBuf, fp);

				}
				else if(iParam2 == 1)
				{
					/* receive */
					cBuf[0] = 0;
					if(ucRecvData[0] >= C33_ICD_ACK)
					{
							if(ucRecvData[0] >= C33_ICD_ACK)
							{
								strcat(cBuf, " Recv ACK\n");
							}
							else if(ucRecvData[0] >= C33_ICD_ERR)
							{
								strcat(cBuf, " Recv ERR");
								for( i = 0 ; i < 9 ; i++)
								{
									sprintf(cTemp, " %02X", ucSendData[i+1]);
									strcat(cBuf,cTemp);
								}
								strcat(cBuf, "\n");
							}
							else if((ucRecvData[0] & C33_ICD_STS) >= C33_ICD_STS)
							{
								strcat(cBuf, " Recv STS\n");
							}
							else if((ucRecvData[0] & C33_ICD_NAK) >= C33_ICD_NAK)
							{
								strcat(cBuf, " Recv NAK\n");
							}
					}
					else
					{
							sprintf(cTemp, " Recv ID   = %0x\n", (int)ucRecvData[2]);
							strcat(cBuf, cTemp);
							sprintf(cTemp, " Recv Data =");
							strcat(cBuf, cTemp);
							for( i = 0 ; i < (int)ucRecvData[0]+(int)ucRecvData[1] ; i++)
							{
								sprintf(cTemp, " %02X", ucRecvData[i+3]);
								strcat(cBuf,cTemp);
							}
							strcat(cBuf, "\n");
					}
					fputs(cBuf, fp);
				}
				/* file close */
				fclose(fp);
				fp = NULL;
			}
			break;
		case 3:
			iUsbLogCommunication = 0;/* debug disable */
			break;
		default:								/* add T.Tazai 2007/07/12 */
			break;
	}
}

/****************************************************************************/
/* Format		 : int c33_IcdCheckCurrentInst(void)						*/
/* Arguments	 : void 													*/
/* Return value  : 0 : not call or C source, 1 : call 2 : C source			*/
/* Explanation	 : check if current pc code is call or call.d.				*/
/*				 : watch break point --> return 0							*/
/****************************************************************************/

int
c33_IcdCheckCurrentInst()
{
	unsigned long ulCurrentPC;
	unsigned short insn_word;
	int iStepCount = 0;
	unsigned long ulLineNo;
	char cTemp[C33_TRACE_DATA_LINE_MAX + 1];
	int iLen;

	ulCurrentPC = read_register (PC_REGNUM);
	/* check if Current PC Address is in C source */

	/* Get filename and line number */
	if (c33_FindFilename(ulCurrentPC, cTemp, &ulLineNo) == C33_NO_ERR )
	{
		iLen = strlen( cTemp );
//		if( cTemp[iLen-2] == '.' && cTemp[iLen-1] == 'c' )	/* change check extend for C++ T.Tazaki 2005/04/27 */
		if( cTemp[iLen-2] == '.' && cTemp[iLen-1] != 's' )	/* change check extend for C++ T.Tazaki 2005/04/27 */
		{
			return 2;
		}
	}


	/* Get Instruction Code on current PC */
	insn_word = read_memory_unsigned_integer (ulCurrentPC, 2);

	/* search call code */
	do
	{
		/* check if call code found */
		if ( (insn_word & 0xfff0) == INSN_CALL_RB  || (insn_word & 0xfff0) == INSN_CALL_RB_D ||
			 (insn_word & 0xff00) == INSN_CALL_IMM || (insn_word & 0xff00) == INSN_CALL_IMM_D )
		{
			return 1;
		}
		/* check if ext code found */
		if ( (insn_word & 0xe000) != INSN_EXT )
		{
			return 0;
		}
		if( ulCurrentPC <= 0xfffffffcUL )
		{
			++iStepCount;
			
			/* from start address to courrent address step count max check */
			if( iStepCount > 2 )
			{
				return 0;
			}
			ulCurrentPC += 2;
			insn_word = read_memory_unsigned_integer (ulCurrentPC, 2);
		}
		else
		{
			return 0;
		}
	}while(1);

}

/* add T.Tazaki 2004/08/24 >>> : asm("xxxxx"); + asm("label:"); next bug fix */

/****************************************************************************/
/* Format		 : int c33_IcdCheckC_asm_call(void)							*/
/* Arguments	 : void 													*/
/* Return value  : 0 : not C source and asm("callȊO");  1 : C source and asm("callȊO");  */
/* Explanation	 : check if current pc code is C soyrce and "asm("call")    */
/*				   called by infrun.c : next(step) process					*/
/****************************************************************************/
int
c33_IcdCheckC_asm_call()
{
	unsigned short insn_word;
	unsigned long ulLineNo;
	char cTemp[C33_TRACE_DATA_LINE_MAX + 1];
	int iLen;
	int i;

	if ( c33TargetInfo.iIcd_open == C33_NOT_OPEN )
	{
		return 0;
	}

	/* Get Instruction Code on current PC */
	insn_word = read_memory_unsigned_integer (g_ulStartPC, 2);

	/* check if Current PC Address is in C source */

	/* Get filename and line number */
	if (c33_FindFilename(g_ulStartPC, cTemp, &ulLineNo) == C33_NO_ERR )
	{
		iLen = strlen( cTemp );
		if( cTemp[iLen-2] == '.' && cTemp[iLen-1] == 's' )	/* `rl\[XH */
		{
			return 0;
		}
	}



	/* check if call code found */
	if ( (insn_word & 0xfff0) == INSN_CALL_RB  || (insn_word & 0xfff0) == INSN_CALL_RB_D ||
		 (insn_word & 0xff00) == INSN_CALL_IMM || (insn_word & 0xff00) == INSN_CALL_IMM_D )
	{
		return 0;
	}

	for( i = 0; i < C33_TRACE_DATA_LINE_MAX; ++i )
	{
		cTemp[i] = 0;
	}
	/* C\[X asm()H */
	c33_IcdTraceGetSourceCode( g_ulStartPC, cTemp );
	iLen = strlen(cTemp);
	for( i = 0; i < iLen; ++i )
	{
		if( strncmp( "asm",&cTemp[i],3 ) == 0 )
		{
			break;
		}
	}
	if( i == iLen )
	{
		return 0;
	}
	for( i = i + 3; i < iLen; ++i )
	{
		if( cTemp[i] == '(' )
		{
			break;
		}
		else if( cTemp[i] != ' ' )
		{
			return 0;
		}
	}
	if( i == iLen )
	{
		return 0;
	}

	return 1;	/* found "asm("xxxxx"); */
}

/* add T.Tazaki 2004/08/24 <<< : asm("xxxxx"); + asm("label:"); next bug fix */


#ifdef __CYGWIN__

HANDLE hCommParl = INVALID_HANDLE_VALUE;
COMMTIMEOUTS CommTimeOuts;			   /* commnunication function use structure,set timeout of RS */
#endif

/****************************************************************************/
/* Format		 : int c33_icdParlOpen( void )								*/
/* Arguments																*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : This function opens parallel port and sets parameters.	*/
/****************************************************************************/
int
c33icd_ParlOpen()
{
	 char  szText[64];				/* text data */
	 int iResult = C33_NO_ERR;		/* return value to com module */

#ifdef __CYGWIN__

	 sprintf(szText, "LPT%d", g_iParallel_LPTNo);			/* LPTX:1-2 */

	 hCommParl = CreateFile((LPCTSTR)szText,
						 GENERIC_WRITE,
						 0,
						 NULL,
						 OPEN_EXISTING,
						 FILE_ATTRIBUTE_NORMAL,
						 NULL);

	 /* parallel port open is success,rs parameter setup */
	 if( hCommParl != INVALID_HANDLE_VALUE ){
			CommTimeOuts.ReadIntervalTimeout		 = 0;					 /* not used */
			CommTimeOuts.ReadTotalTimeoutMultiplier  = 0;					 /* not used */
			CommTimeOuts.ReadTotalTimeoutConstant	 = TIMEOUT_RECV_CONSTANT;/* 6sec */
			CommTimeOuts.WriteTotalTimeoutMultiplier = 0;					 /* not used */
			CommTimeOuts.WriteTotalTimeoutConstant	 = TIMEOUT_SEND_CONSTANT;/* 20sec */
			/* set timeout */
			if(SetCommTimeouts(hCommParl, &CommTimeOuts) == TRUE){
				iResult = C33_NO_ERR;/* not error return */
			}
			else{
				iResult = C33_ERR;/* error occured to set timeout */
			}
	 }
	 else{
		iResult = C33_ERR;/* error occured to open rs port */
	 }
#endif

	 return iResult;
}

/****************************************************************************/
/* Format		 : int c33icd_ParlClose( void ) 							*/
/* Arguments																*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : This function closes parallel port, then is free sending */
/*				   and	receiving queue.									*/
/****************************************************************************/
int
c33icd_ParlClose()
{
	 int iResult = C33_NO_ERR;
#ifdef __CYGWIN__

	 /* parallel port close */
	 if(hCommParl != 0){
		 iResult = CloseHandle( hCommParl);
	 }
#endif
	 return iResult;
}

/****************************************************************************/
/* Format		 : int c33_ParallelSendData( unsigned char* pMyAddr, unsigned long ulLen ,int iUnit) */
/* Arguments																*/
/*	unsigned char* pMyAddr	   : Send data pointer							*/
/*	unsigned long  ulLen	   : Send data size 							*/
/*	int 		   iUnit	   : Send data type 							*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Parallel data write.(icdv2 high speed load)				*/
/****************************************************************************/
int
c33icd_ParallelSendData( unsigned char* pMyAddr, unsigned long ulLen,int iUnit )
{
	 int iResult = C33_NO_ERR;
#ifdef __CYGWIN__
	 DWORD	dwWritten;				/* written data size for WriteFile */
	 unsigned long ulTimeOutTime;	/* time out time [m sec] */
	 COMSTAT	tdComStat;			/* COMSTAT structure */
	 DWORD		dwErrors;			/* error code */
	 DWORD		dwNum;
	 unsigned char ucBCC = 0;
	 DWORD		dwCount;

	dwNum = ulLen;

	// set parallel port time out */
	CommTimeOuts.ReadIntervalTimeout = 0;/* not used */
	CommTimeOuts.ReadTotalTimeoutMultiplier = 0;/* not used */
	CommTimeOuts.ReadTotalTimeoutConstant = TIMEOUT_RECV_LOOP;/* 100mSec */
	CommTimeOuts.WriteTotalTimeoutMultiplier = 0;/* not used */
	CommTimeOuts.WriteTotalTimeoutConstant = TIMEOUT_SEND_LOOP;/* 100mSec */
	SetCommTimeouts(hCommParl, &CommTimeOuts);

   iResult = c33icd_ParlOpen();

	/* save now time */
	ulTimeOutTime = GetCurrentTime();

	/* data send */
	while(dwNum != 0 && (GetCurrentTime() - ulTimeOutTime) < TIMEOUT_SEND_CONSTANT_PARL )
	{
		
		if( WriteFile( hCommParl, ( LPCVOID ) pMyAddr, dwNum, &dwWritten, NULL ) == FALSE){ /* send data to parallelport */
		}
		else{
		}
		dwNum -= dwWritten;
		pMyAddr = pMyAddr + dwWritten;
	}
	if (dwNum > 0 ){
		c33icd_ParlClrBuff();/* clear buffer */
		c33icd_ParlClose();
		if( g_iParl_error_1st == 1 )
		{
			printf_filtered( "Error: Parallel interface time out.\n");
			g_iParl_error_1st = 0;
		}
		return C33_ERR;
	}

	c33icd_ParlClose();

#endif	// #ifdef __CYGWIN__
	return iResult;
}

/****************************************************************************/
/* Format		 : void c33icd_ParlClrBuff(void)							*/
/* Arguments																*/
/* Return value 															*/
/* Explanation	 : This function clears sending and receiving queue.		*/
/****************************************************************************/
void
c33icd_ParlClrBuff()
{
#ifdef __CYGWIN__
	 PurgeComm( hCommParl, PURGE_TXCLEAR );/* send buffer clear */
	 PurgeComm( hCommParl, PURGE_RXCLEAR );/* receive buffer clear */
#endif
}

